#!/usr/bin/perl

use strict;
use warnings;

use Data::Dumper;
$Data::Dumper::Deepcopy = 1;
$Data::Dumper::Terse    = 0;

use DBI;

use Template;

my $dbh = DBI->connect( "dbi:Pg:dbname=quotebot;host=localhost",
    "ryan", "", { RaiseError => 1 } );

sub torange {

#[16:11:46] <tybalt89> eval: $_ = '1,2,3,4,7,8,9'; s/(\d+),/$1+1<$'?$&:"$1-"/ge;s/-\d+(?=-)//g; $_
#[16:11:46] <buubot> tybalt89: 1-4,7-9
    my $episodes = shift;
    no warnings 'numeric';    #this makes fun warnings
    $episodes =~ s/(\d+),/$1+1<$'?$&:"$1-"/ge;
    $episodes =~ s/-\d+(?=-)//g;
    return $episodes;
}

sub gettemplate {
    local $/;
    my $temp = <DATA>;
    return $temp;
    close DATA;
}

sub getepisodes {
    my $sth = $dbh->prepare(
q{SELECT * FROM episodeinfo WHERE title LIKE '%Part%' AND eid != 557 ORDER BY eid ASC}
    );
    $sth->execute();

    my @epis;
    my @multipart;

    while ( my $row = $sth->fetchrow_hashref() ) {
        $row->{eidpad} = sprintf( "%03d", $row->{eid} );
        push @epis, $row;
    }

    my $pieces = 0;

    #this is possibly the least efficient way to do this, BUT it'll work
  OUTER:
    for ( my $i = 0 ;
        $i <= $#epis ; $i++ )    #fucking c style loop lets me do this right
    {
        my $le = $i;
      INNER: for my $j ( $i .. $#epis ) {
            if ( $j - $le > 1
              ) #we've got more than a 1 episode difference, assume its a new set
            {
                $pieces++;
                $i = $j - 1;
                last INNER;
            }
            elsif (
                ( $j != $le ) &&    #if we're not already starting a new set
                ( $epis[$j]{title} =~ /part\s*1/i )
              )                     #and if its part 1, assume its a new set too
            {
                $pieces++;
                $i = $j - 1;
                last INNER;
            }
            else {
                $le = $j;
                push @{ $multipart[$pieces] }, $epis[$j];
            }
        }
    }

    print "There are ", scalar(@epis), "episodes in sets\n";

    return \@multipart;
}

sub makeranges {
    my $parts = shift;

    my %ranges;

    for my $part (@$parts) {
        my $eps = join ',', map { $_->{eid} } @{$part};
        my $range = torange($eps);
        $ranges{$range} = $part;
    }

    return %ranges;
}

#------------------------MAIN--------------------
my $episodes = getepisodes();
my %ranges   = makeranges($episodes);
my $template = gettemplate();
my $config   = {
    POST_CHOMP => 0,    # cleanup whitespace
};

my $tt = Template->new($config);

for my $set ( keys %ranges ) {
    my $vars = {
        epirange => $set,
        episodes => $ranges{$set}
    };

    my $output;

    $tt->process( \$template, $vars, \$output );

    open( my $fh, ">", "$set.xml" ) or die "can't open $set: $!";

    print $fh $output;

    close($fh);
}

__DATA__
<?xml version="1.0" encoding="UTF-8"?>
<rss version="2.0" xmlns:media="http://search.yahoo.com/mrss/" xmlns:jwplayer="http://developer.longtailvideo.com/trac/wiki/FlashFormats">
	<channel>
		<title>[%epirange%]</title>[%FOREACH epi IN episodes%]
		<item>
			<title>Episode [%epi.eid%]</title>
			<media:credit role="author">[%epi.group%]</media:credit>
			<description>[%epi.title%]</description>
			<media:content url="http://conantv.net/videos/getvid.pl?file=[%epi.eidpad%]-STV.mp4"></media:content>
			<media:thumbnail url=""></media:thumbnail>
			<media:keywords></media:keywords>
		</item>[%END%]
	</channel>
</rss>
