#!/usr/bin/perl

use strict;
use warnings;

use Data::Dumper;
use DateTimeX::Easy;
use Template;
use DBI;
use File::MkTemp;

sub eidtostring {
    my $eid = shift;

    if ( $eid < 10000 ) {
        return $eid;
    }
    elsif ( $eid < 20000 ) {
        return "Movie" . ( $eid - 10000 );
    }
    elsif ( $eid < 30000 ) {
        return "OVA" . ( $eid - 20000 );
    }
    elsif ( $eid < 40000 ) {
        return "Special" . ( $eid - 30000 );
    }
    else {
        return "God damned you jd-; $eid";
    }
}

sub latexquote {
    $_[0] =~ s/([%\\&#])/\\$1/g;
}

my $vars = {
    episodes => [],
    movies   => [],
    stats    => {
        unsubbedhist => [ 0, 0, 0, 0, 0, 0 ],
        total        => {
            total => 0,
            manga => 0,
            anime => 0,
            mper  => 0,
            aper  => 0
        },
        subtotal => {
            total => 0,
            manga => 0,
            anime => 0,
            mper  => 0,
            aper  => 0,
            tper  => 0,
            tmper => 0,
            taper => 0
        },
        unsubtotal => {
            total => 0,
            manga => 0,
            anime => 0,
            mper  => 0,
            aper  => 0,
            tper  => 0,
            tmper => 0,
            taper => 0
        },
        seasons      => 0,
        epperyear    => 0,
        specials     => 0,
        spperyear    => 0,
        movieova     => 0,
        moperyear    => 0,
        totaltotal   => 0,
        totalperyear => 0,

    },
    date => scalar localtime(),
};

my $dbh = DBI->connect( "dbi:Pg:dbname=quotebot;host=andromeda128",
    "conanquotes", "conanquotes", { RaiseError => 0 } );
my $sth = $dbh->prepare(
'SELECT eid, episodeinfo.group, title, chapters, airdate FROM episodeinfo WHERE eid < 10000 ORDER BY eid ASC'
);
$sth->execute();

while ( my $row = $sth->fetchrow_hashref() ) {
    next unless defined $row->{airdate};    #skip those without an airdate.
    latexquote $_ for values %$row;
    my $special = 0;

    if ( $row->{title} =~ /\(.*special.*\)/i ) {
        $special = 1;
        $vars->{stats}{specials}++;
    }

    $vars->{stats}{total}{total}++;
    $vars->{stats}{total}{anime}++ if $row->{chapters} eq "TV Original";
    $vars->{stats}{total}{manga}++ unless $row->{chapters} eq "TV Original";

    $vars->{stats}{subtotal}{total}++ if defined $row->{group};
    $vars->{stats}{subtotal}{anime}++
      if $row->{chapters} eq "TV Original" and defined $row->{group};
    $vars->{stats}{subtotal}{manga}++
      if $row->{chapters} ne "TV Original" and defined $row->{group};

    $vars->{stats}{unsubtotal}{total}++ if !defined $row->{group};
    $vars->{stats}{unsubtotal}{anime}++
      if $row->{chapters} eq "TV Original" and !defined $row->{group};
    $vars->{stats}{unsubtotal}{manga}++
      if $row->{chapters} ne "TV Original" and !defined $row->{group};

    $vars->{stats}{unsubbedhist}[ int( $row->{eid} / 100 ) ]++
      if !defined $row->{group};

    print Dumper($row);
    push @{ $vars->{episodes} },
      { %$row, special => $special, badquality => 0 };    #[@row, 0, $special];
}

$sth = $dbh->prepare(
'SELECT * FROM episodeinfo WHERE eid >= 10000 AND title IS NOT NULL AND LENGTH(title) > 0 ORDER BY eid'
);
$sth->execute();

while ( my $row = $sth->fetchrow_hashref() ) {

    #next unless defined $row->{airdate}; #skip those without an airdate.
    latexquote $_ for values %$row;

    $vars->{stats}{movieova}++;

    print Dumper($row);
    push @{ $vars->{movies} },
      { %$row, badquality => 0,
        eidstring => eidtostring $row->{eid} };    #[@row, 0, $special];
}

$vars->{stats}{total}{aper} = int(
    1000 * $vars->{stats}{total}{anime} / $vars->{stats}{total}{total} + 0.5 )
  / 10;
$vars->{stats}{total}{mper} = int(
    1000 * $vars->{stats}{total}{manga} / $vars->{stats}{total}{total} + 0.5 )
  / 10;

$vars->{stats}{subtotal}{aper} = int(
    1000 * $vars->{stats}{subtotal}{anime} / $vars->{stats}{subtotal}{total} +
      0.5 ) / 10;
$vars->{stats}{subtotal}{mper} = int(
    1000 * $vars->{stats}{subtotal}{manga} / $vars->{stats}{subtotal}{total} +
      0.5 ) / 10;
$vars->{stats}{subtotal}{tper} =
  int( 1000 * $vars->{stats}{subtotal}{total} / $vars->{stats}{total}{total} +
      0.5 ) / 10;
$vars->{stats}{subtotal}{taper} =
  int( 1000 * $vars->{stats}{subtotal}{anime} / $vars->{stats}{total}{total} +
      0.5 ) / 10;
$vars->{stats}{subtotal}{tmper} =
  int( 1000 * $vars->{stats}{subtotal}{manga} / $vars->{stats}{total}{total} +
      0.5 ) / 10;

$vars->{stats}{unsubtotal}{aper} =
  int( 1000 * $vars->{stats}{unsubtotal}{anime} /
      $vars->{stats}{unsubtotal}{total} ) / 10;
$vars->{stats}{unsubtotal}{mper} =
  int( 1000 * $vars->{stats}{unsubtotal}{manga} /
      $vars->{stats}{unsubtotal}{total} ) / 10;
$vars->{stats}{unsubtotal}{tper} =
  int( 1000 * $vars->{stats}{unsubtotal}{total} / $vars->{stats}{total}{total} )
  / 10;
$vars->{stats}{unsubtotal}{taper} =
  int( 1000 * $vars->{stats}{unsubtotal}{anime} / $vars->{stats}{total}{total} )
  / 10;
$vars->{stats}{unsubtotal}{tmper} =
  int( 1000 * $vars->{stats}{unsubtotal}{manga} / $vars->{stats}{total}{total} )
  / 10;

my $startdate = new DateTimeX::Easy("1996/Jan/08");
my $now       = new DateTimeX::Easy("now");
my $dur       = $now - $startdate;

$vars->{stats}{seasons} = $dur->years();
$vars->{stats}{totaltotal} =
  $vars->{stats}{total}{total} + $vars->{stats}{movieova};
$vars->{stats}{epperyear} =
  int( 10 * $vars->{stats}{total}{total} / $dur->years() + 0.5 ) / 10;
$vars->{stats}{spperyear} =
  int( 10 * $vars->{stats}{specials} / $dur->years() + 0.5 ) / 10;
$vars->{stats}{moperyear} =
  int( 10 * $vars->{stats}{movieova} / $dur->years() + 0.5 ) / 10;
$vars->{stats}{totalperyear} =
  int( 10 * $vars->{stats}{totaltotal} / $dur->years() + 0.5 ) / 10;

my $tt = Template->new;

$tt->process( 'templates/table.latex.tmpl', $vars, 'table.tex' )
  || die $tt->error;

#print Dumper($vars);
system( "/usr/bin/texi2pdf", "table.tex" );
