#!/usr/bin/perl
use strict;
use warnings;

use Template;
use CGI;
use DBI;
use POE::Component::IRC::Common qw(:ALL);
use Data::Dumper;
use lib 'lib/';
use HTML::Entities;

use Epiquote;

##################################################################
#main variables
##################################################################

my $cgi = new CGI;
print $cgi->header("text/html; charset=UTF-8");

my $dbh = DBI->connect( "dbi:Pg:dbname=quotebot;host=localhost",
    "ryan", "", { RaiseError => 1 } );

my $vars = {
    action        => undef,
    eid           => undef,
    title         => undef,
    group         => undef,
    fsize         => undef,
    fcodec        => undef,
    fcrc          => undef,
    botlisting    => undef,
    filelink      => undef,
    filelink_date => undef,
    _filelink     => undef,
    airdate       => undef,
    ratings       => undef,
    epdesc        => undef,
    fcomment      => undef,
    chapters      => undef,
    animeoriginal => undef,
};

my %offsets = (
    special => 30000,
    ova     => 20000,
    movie   => 10000
);

##################################################################
#main code
##################################################################

#load up all the parameters
#print Dumper($vars);
$vars->{eid} = Epiquote::stringtoeid( $cgi->param("eid") );

{
    my $sth = $dbh->prepare("SELECT * FROM episodeinfo WHERE eid = ? LIMIT 1");
    if ( $sth->execute( $vars->{eid} ) ) {
        my $ovars = $sth->fetchrow_hashref();

        #	print Dumper($ovars);
        for my $key ( keys %$ovars ) {
            $vars->{$key} = $ovars->{$key};
        }
    }
}    #load up defaults so i can do this right

#print Dumper($vars);
$vars->{action} = $cgi->param("action");
for my $key ( keys %$vars ) {
    $vars->{cgiin}{$key} =
      $cgi->param($key);    #store this because i'm not sure what i'm getting
    $vars->{$key} = decode_entities( $cgi->param($key) )
      if ( defined( $cgi->param($key) ) );

    $vars->{animeoriginal} = $cgi->param("animeoriginal")
      if ( $key eq "animeoriginal" && $vars->{action} eq "edit" );

    $vars->{$key} = undef
      if ( defined( $vars->{$key} ) && $vars->{$key} eq "" )
      ; #i don't want empty strings!, just set them to null in the database, makes things easier later
}

$vars->{eid} = Epiquote::stringtoeid( $cgi->param("eid") )
  ;     #it gets reset after that last step
$vars->{animeselc} =
  ( $vars->{animeoriginal} eq "0" or $vars->{animeoriginal} eq "on" )
  ? "checked"
  : "";

#print Dumper($vars);

my $tt = Template->new(
    {
        INCLUDE_PATH => '/var/www/farnsworth/cgi-bin/conantemplates/',
        COMPILE_EXT  => '.ttc',
    }
) || die "$Template::ERROR\n";

my $havep = Epiquote::epindb( $vars->{eid} );

if ( defined( $vars->{action} ) && $vars->{action} eq "edit" ) {

    #save info here!
    if ($havep) {
        if ( $vars->{_filelink} eq $vars->{filelink} )    #link wasn't updated
        {
            my $sth = $dbh->prepare(
"UPDATE episodeinfo SET title = ?, \"group\" = ?, fsize = ?, fcodec = ?,"
                  . " fcrc = ?, botlisting = ?, airdate = ?, ratings = ?, epdesc = ?,"
                  . " fcomment = ?, chapters = ?, animeoriginal = ? WHERE eid = ?"
            );
            $sth->execute(
                @{$vars}{
                    qw(title group fsize fcodec fcrc botlisting airdate ratings
                      epdesc fcomment chapters animeoriginal eid)
                }
            );
        }
        else {
            my $sth = $dbh->prepare(
"UPDATE episodeinfo SET title = ?, \"group\" = ?, fsize = ?, fcodec = ?,"
                  . " fcrc = ?, filelink = ?, filelink_date = now(), botlisting = ?, airdate = ?,"
                  . " ratings = ?, epdesc = ?, fcomment = ?, chapters = ?, animeoriginal = ? WHERE eid = ?"
            );
            $sth->execute(
                @{$vars}{
                    qw(title group fsize fcodec fcrc filelink botlisting airdate ratings
                      epdesc fcomment chapters animeoriginal eid)
                }
            );
        }
    }
    else {
        my $sth = $dbh->prepare(
"INSERT INTO episodeinfo (title, \"group\", fsize, fcodec, fcrc, botlisting, filelink, airdate, ratings, epdesc, fcomment, eid, chapters, animeoriginal) VALUES 
													 (?,     ?,       ?,     ?,      ?,    ?,          ?,        ?,       ?,       ?,      ?,	?,	?,	?)"
        );
        $sth->execute(
            @{$vars}{
                qw(title group fsize fcodec fcrc botlisting filelink airdate ratings epdesc fcomment eid chapters animeoriginal)
            }
        );
    }

    Epiquote::addquote($vars);
}
elsif ( defined( $vars->{action} ) && $vars->{action} eq "pick" && $havep ) {

    #load info here!
    my $sth = $dbh->prepare("SELECT * FROM episodeinfo WHERE eid = ? LIMIT 1");
    if ( $sth->execute( $vars->{eid} ) ) {
        my $ovars = $sth->fetchrow_hashref();
        for my $key ( keys %$ovars ) {
            $vars->{$key} = $ovars->{$key};
        }
    }
}

$vars->{dump} = Dumper($vars);

if ( $vars->{action} eq "pick" ) {
    $tt->process( "edit.tmpl", $vars ) || die $tt->error(), "\n";
}
else {
    $tt->process( "pick.tmpl", $vars ) || die $tt->error(), "\n";
}
