use utf8;

package DB::Schema::Result::Quote;

=head1 NAME

DB::Schema::Result::Quote

=cut

use strict;
use warnings;

use base 'DBIx::Class::Core';

=head1 TABLE: C<quotes>

=cut

__PACKAGE__->table("quotes");

=head1 ACCESSORS

=head2 qid

  data_type: 'integer'
  is_auto_increment: 1
  is_nullable: 0
  sequence: 'qidseq'

=head2 sid

  data_type: 'integer'
  is_foreign_key: 1
  is_nullable: 0

=head2 quote

  data_type: 'text'
  is_nullable: 0

=head2 who

  data_type: 'text'
  is_nullable: 1

=head2 deleted

  data_type: 'boolean'
  default_value: false
  is_nullable: 0

=head2 counter

  data_type: 'integer'
  default_value: 0
  is_nullable: 0

=cut

__PACKAGE__->add_columns(
    "qid",
    {
        data_type         => "integer",
        is_auto_increment => 1,
        is_nullable       => 0,
        sequence          => "qidseq",
    },
    "sid",
    { data_type => "integer", is_foreign_key => 1, is_nullable => 0 },
    "quote",
    { data_type => "text", is_nullable => 0 },
    "who",
    { data_type => "text", is_nullable => 1 },
    "deleted",
    { data_type => "boolean", default_value => \"false", is_nullable => 0 },
    "counter",
    { data_type => "integer", default_value => 0, is_nullable => 0 },
);

=head1 PRIMARY KEY

=over 4

=item * L</qid>

=back

=cut

__PACKAGE__->set_primary_key("qid");

=head1 UNIQUE CONSTRAINTS

=head2 C<quotes_quote_key>

=over 4

=item * L</quote>

=item * L</sid>

=back

=cut

__PACKAGE__->add_unique_constraint("quotes_quote_key", [ "quote", "sid" ]);

=head1 RELATIONS

=head2 sid

Type: belongs_to

Related object: L<DB::Schema::Result::Source>

=cut

__PACKAGE__->belongs_to(
    "source", "DB::Schema::Result::Source",
    { sid           => "sid" },
    { is_deferrable => 0, on_delete => "NO ACTION", on_update => "NO ACTION" },
);

1;
