#!/usr/bin/perl

package Epiquote;
use strict;
use warnings;

use POE::Component::IRC::Common qw(:ALL);
use Data::Dumper;

##################################################################
#main variables
##################################################################
use BotConfig;
use DB;
my $schema = DB->new();

my $dbconf = BotConfig->instance->db;

my %offsets = (
    kaito   => 40000,
    special => 30000,
    ova     => 20000,
    movie   => 10000
);

##################################################################
#helper routines
##################################################################

sub stringtoeid {
    my $str = shift;

    if ($str =~ /^(special|ova|movie)(\d+)/i) {
        return $offsets{$1} + $2;
    }
    elsif ($str =~ /^(\d+)/) {
        return $1;
    }
    else {
        return (); #die "Fuck you jd- cut this shit out";
    }
}

sub eidtostring {
    my $eid = shift;

    if ($eid < 10000) {
        return $eid;
    }
    elsif ($eid < 20000) {
        return "MOVIE" . ($eid - 10000);
    }
    elsif ($eid < 30000) {
        return "OVA" . ($eid - 20000);
    }
    elsif ($eid < 31000) {
        return "SPECIAL" . ($eid - 30000);
    }
    elsif ($eid < 32000) {
        return "LIVE" . ($eid - 31000);
    }
    elsif ($eid < 33000) {
        return "LUPIN" . ($eid - 32000);
    }
    elsif ($eid < 50000) {
        return "KAITO" . ($eid - 40000);
    }
    else {
        use Carp qw/confess/;
        confess "God damned you jd-, $eid";
    }
}

sub epindb {
    my ($eid) = @_;
    return $schema->episodeinfo->search({eid => $eid})->count;
}

sub setuptriggers {
    my $eid         = shift;
    my $havetrigger = $schema->trigger->find({tid => -$eid});
    my $havesource  = $schema->source->find({sid => -$eid});
    my $havequote   = $schema->quote->find({qid => -$eid});
    my $havetidsource = $schema->tidsource->find({sid => -$eid, tid => -$eid});

    unless ($havetrigger) {
        $havetrigger = $schema->trigger->create({tid => -$eid, trigger => '!'.eidtostring($eid)});
    }

    unless ($havesource) {
        $havesource = $schema->source->create({sid => -$eid, name => eidtostring($eid), desc => "/notice %NICK%"});
    }

    unless ($havetidsource) {
        $havetrigger->tidsources->create({sid => $havesource->sid});
    }

    if ($havequote) {
        $havequote->delete();
    }
}

sub addquote {
    my $vars = shift;

    setuptriggers($vars->{eid});

    my $RESET = NORMAL . LIGHT_GREY . ",01";

    my $seperator   = RED . ",01 || " . $RESET;
    my $seperatorno = RED . ",01 || ";
    my $quote;

    if ($vars->{eid} < 10000) {
        $quote = TEAL . ",01Episode " . $RESET . "[" . CYAN . eidtostring($vars->{eid}) . $RESET . "]";
    }
    else {
        $quote = $RESET . "[" . CYAN . eidtostring($vars->{eid}) . $RESET . "]";
    }

    if ($vars->{title}) {
        $quote .= " " . $vars->{title};
    }

    #trim space!
    $quote =~ s/\s+$//;
    $quote =~ s/^\s+//;

    if ($vars->{group} || $vars->{fsize} || $vars->{fcodec} || $vars->{crc}) {
        $quote .= $seperator;

        if ($vars->{group}) {
            $quote .= "Subbed by [" . TEAL . $vars->{group} . $RESET . "] ";
        }

        if ($vars->{fsize}) {
            $quote .= BOLD . ORANGE . "@ " . $RESET . $vars->{fsize} . " ";
        }

        if ($vars->{fcodec}) {
            $quote .= "Codec: [" . $vars->{fcodec} . "] ";
        }

        if ($vars->{fcrc}) {
            $quote .= "CRC: [" . $vars->{fcrc} . "] ";
        }
    }

    #trim space!
    $quote =~ s/\s+$//;
    $quote =~ s/^\s+//;

    if ($vars->{filelink}) {
        $quote .= $seperatorno;

        $quote .= ORANGE . "MU Link: " . $RESET . $vars->{filelink};
    }

    #trim space!
    $quote =~ s/\s+$//;
    $quote =~ s/^\s+//;

    if ($vars->{airdate} || $vars->{ratings}) {
        $quote .= $seperatorno;

        $quote .= ORANGE . "Aired ";

        if ($vars->{airdate}) {
            $quote .= BOLD . $vars->{airdate} . NORMAL . ORANGE . ",01 ";
        }

        if ($vars->{ratings}) {
            $quote .= "with an audience share of " . BOLD . $vars->{ratings} . "%";
        }
    }

    #trim space!
    $quote =~ s/\s+$//;
    $quote =~ s/^\s+//;

    if ($vars->{chapters}) {
        $quote .= $seperatorno;

        if ($vars->{chapters} ne "TV Original") #this should check the AO flag too!
        {
            $quote .= ORANGE . ",01 Based on " . $vars->{chapters};
        }
        else {
            $quote .= $RESET . ORANGE . ",01 AO";
        }
    }

    #trim space!
    $quote =~ s/\s+$//;
    $quote =~ s/^\s+//;

    if ($vars->{fcomment}) {
        $quote .= $seperatorno;

        $quote .= ORANGE . ",01" . $vars->{fcomment};
    }

    #trim space!
    $quote =~ s/\s+$//;
    $quote =~ s/^\s+//;

    if ($vars->{botlisting}) {
        $quote .= $RESET . $seperatorno;

        $quote .= "\n" . ORANGE . ",01Bot Listing: " . $RESET . $vars->{botlisting};
    }

    #trim space!
    $quote =~ s/\s+$//;
    $quote =~ s/^\s+//;

    my $eid = $vars->{eid};
    $schema->quote->create({qid => -$eid, sid => -$eid, quote => $quote});
}

sub doupdate {
    my $eid  = shift;
    my $vars = {
        action        => undef,
        eid           => $eid,
        title         => undef,
        group         => undef,
        fsize         => undef,
        fcodec        => undef,
        fcrc          => undef,
        botlisting    => undef,
        filelink      => undef,
        airdate       => undef,
        ratings       => undef,
        epdesc        => undef,
        fcomment      => undef,
        chapters      => undef,
        animeoriginal => undef,
    };

    { #use an empty scope to let $sth die on me, no need to keep it around forever
        my $row = $schema->episodeinfo->search({eid => $eid})->first;
        
        if ($row) {
            for my $key (qw/title group fsize fcodec fcrc botlisting filelink airdate ratings epdesc fcomment chapters animeoriginal/) {
                $vars->{$key} = $row->get_column($key);
            }
        }
    } #load up defaults so i can do this right

    print Dumper($vars) if $eid > 800;

    addquote($vars);
}

1;
