#!/bin/false #to screw with people who try to run it

package IRC;

use strict;
use warnings;

use BotConfig;
use feature qw/postderef/;

my $server_conf = BotConfig->instance->server;

use POE qw(Component::IRC::State
    Queue::Array
    Component::IRC::Plugin::NickReclaim
    Component::IRC::Plugin::CTCP
    Component::IRC::Plugin::NickServID
    Component::IRC::Plugin::Connector);

use Data::Dumper;
use POE::Component::IRC::Common qw(:ALL);
use DB;

my $schema = DB->new(); # connect to DB

my $password;
$password = $ARGV[0] if @ARGV;

my @ignore     = qw(bot); #none so far yet, when i write the DB code it should change to let that be used
my @userignore = qw(child_of_eve);

my $bot = POE::Component::IRC->spawn(
    server   => $server_conf->host,
    port     => $server_conf->port,
    nick     => $server_conf->nick,
    username => $server_conf->username,
    ($server_conf->password ? (password => $server_conf->password) : ()),
    name => "Edogawa Conan",
    #           password  => "conanquotes",
    #NATAddr => 'simcop2387.info', # TODO this should autodetect
    DCCPorts => [5063],
    charset  => "utf-8", # charset the bot assumes the channel is using
    flood    => 1,
);

my $queue = POE::Queue::Array->new();

my $ses = POE::Session->create(
    package_states => [
        IRC => [
            qw(_start irc_001 irc_public irc_msg irc_dcc_request irc_dcc_done irc_dcc_error startxdcc cancelotherxdcc tock msgfromdb irc_notice)
        ],
    ],
    heap => { irc => $bot, queue => $queue, lastsend => time() },
);

sub _start {
    my ($kernel, $heap) = @_[ KERNEL, HEAP ];

    my $irc = $heap->{irc};

    $kernel->alias_set("IRCHANDLER");

    $irc->plugin_add(
        'CTCP' => POE::Component::IRC::Plugin::CTCP->new(
            version  => "Quotebot r25",
            userinfo => "See simcop2387 for more info",
        ));
    $irc->plugin_add('NickReclaim' => POE::Component::IRC::Plugin::NickReclaim->new(poll => 30));
    $irc->plugin_add(
        'NickServID',
        POE::Component::IRC::Plugin::NickServID->new(
            Password => $password,
        )) if $password;

    $irc->plugin_add('Connector' => POE::Component::IRC::Plugin::Connector->new());

    $irc->yield(register => 'all');
    $irc->yield(connect  => {});
}

sub irc_001 {
    my $sender = $_[SENDER];
    my $kernel = $_[KERNEL];

    # Since this is an irc_* event, we can get the component's object by
    # accessing the heap of the sender. Then we register and connect to the
    # specified server.
    my $irc = $sender->get_heap();

    print "Connected to ", $irc->server_name(), "\n";

    # we join our channels
    print "Joining Channels: ", join(', ', $server_conf->channels->@*), "\n";
    $irc->yield(join => $_) for ($server_conf->channels->@*);

    #            $irc->yield( join => $_ ) for ("#dctv");
    $kernel->delay_add(tock    => 0.25);
    $kernel->delay_add(comfuck => 21600); #send a ping every 50 seconds
    $irc->yield(nick => $server_conf->nick);
}

sub tock {
    my ($sender, $kernel, $heap) = @_[ SENDER, KERNEL, HEAP ];

    my $np = $heap->{queue}->get_next_priority();

    if ((defined($np)) && (time() - $heap->{lastsend} > 1.5)) {
        my ($priority, $queue_id, $payload) = $heap->{queue}->dequeue_next();
        print "TOCK: $priority, $queue_id\n";
        if ($payload->[1] =~ /^\/notice (\S+) (.*)$/i) {
            $heap->{irc}->yield(notice => $1 => $2);
        }
        else {
            my $channel = $payload->[0];
            my $msg     = $payload->[1];

            my $strip = $schema->getstrip($channel);

            #print Dumper($strip);

            if ($strip) {
                $msg = strip_formatting(strip_color($msg));
            }

            print "OUT: $channel | $msg\n";

            $heap->{irc}->yield(privmsg => $channel => $msg);
        }
        $heap->{lastsend} = time();
    }

    $kernel->delay_add(tock => 0.5);
}

sub _userignore {
    my $who = shift;
    return (grep { $who =~ /\Q$_/i } @userignore) ? 1 : 0;
}

sub _ignore {
    my $who    = shift;
    my $kernel = shift;
    return $schema->isignored($who);
}

sub _ignoremsg {
    my $who    = shift;
    my $kernel = shift;
    return $schema->getignoredmsg($who);
}

sub irc_public {
    my ($sender, $who, $where, $what, $heap, $kernel) = @_[ SENDER, ARG0 .. ARG2, HEAP, KERNEL ];
    my $nick = (split /!/, $who)[0];
    my $channel = $where->[0];

    #print Dumper($who);
    if (_ignore($nick, $kernel)) {
        if (!defined(_ignoremsg($nick, $kernel))) {
            return ();
        }
    }

    return () if ($who =~ /znc/);

    my @lines;
    my $response = "";
    if (my $rp = doadmin($nick, $where, $what, $heap, $kernel)) {
        return () if $rp eq 1;
        $response = $rp;
        @lines    = messagebreak($response);
    }
    else {
        #for when users do @find 544 so that they can get a full set of links and info
        if ($what =~ /^\@find\s+(\d+)\s*$/) {
            $what = "!$1";
        }
        elsif ($what =~ /^!epnumber\s*(\d+)\s*$/) {
            $what = "!$1";
        }

        print STDERR "GETQUOTE: $what ", $where->[0], "\n";

        my @quote = $schema->getquote($what . $where->[0], $where);

        print STDERR "\t", @quote, "\n";

        @quote = $schema->getquote($what, $where) unless $quote[0];
        print STDERR "\t", @quote, "\n";
        return "" unless $quote[0]; #don't do anything if we don't get anything back!
        $response = doformat(@quote[ 0 .. 3 ], $nick);

        @lines = messagebreak($response); #this should really never be needed, but is here so that its consistent
    }

    my $pd = getpristart($heap->{queue}, $channel);

    if (_ignore($nick, $kernel) && @lines) {
        if (my $msg = _ignoremsg($nick, $kernel)) {
            my $pd = getpristart($heap->{queue}, $channel);
            $heap->{queue}->enqueue($pd, [ $channel, "$nick: $msg" ]);
            return ();
        }
    }

    for my $p (0 .. $#lines) {
        $heap->{queue}->enqueue($p + $pd, [ $channel, "$lines[$p]" ]);
    }
}

sub doadmin {
    my ($who, $where, $what, $heap, $kernel) = @_;

    #I suppose this COULD be more modular
    if ($what =~ /^!addquote\s+(\S+)\s+(.*)$/) {
        $schema->addquote($where, $who, $1, $2);
        return 1;
    }
    elsif ($what =~ /^!deletequote\s+(?:#\s*)?(\d+)$/) {
        $schema->deletequote($where, $who, $1);
        return 1;
    }
    elsif ($what =~ /^!getquote\s+(?:#\s*)?(\d+)$/) {
        my @q = $schema->getbyqid($where, $who, $1);
        return doformat(@q[ 0 .. 3 ], $who);
    }
    elsif ($what =~ /^!getlastquote\s*$/) {
        my @q = $schema->getlastquote();
        return doformat(@q[ 0 .. 3 ], $who);
    }
    elsif ($what =~ /^!addsource\s+(\S+)\s*(\S.*)?$/) {
        $schema->addsource($where, $who, $1, $2);
        return 1;
    }
    elsif ($what =~ /^!addtrigger\s+(\S+)$/) {
        $schema->addtrigger($where, $who, $1);
        return 1;
    }
    elsif ($what =~ /^!addsourcetotrigger\s+(\S+)\s+(\S+)$/) {
        $schema->appendtotrigger($where, $who, $2, $1);
        return 1;
    }
    elsif ($what =~ /^!removesourcefromtrigger\s+(\S+)\s+(\S+)$/) {
        $schema->removefromtrigger($where, $who, $1, $2);
        return 1;
    }
    elsif ($what =~ /^!showsources$/) {
        return "/notice $who " . $schema->listsources($where, $who);
    }
    elsif ($what =~ /^!showtriggers$/) {
        return "/notice $who " . $schema->listtriggers($where, $who);
    }
    elsif ($what =~ /^!whatistrigger\s+(\S+)$/) {
        return "/notice $who " . $schema->whatistrigger($where, $who, $1);
    }
    elsif ($what =~ /^!xdccupdate/) {
        $kernel->post("BotSync", "startupdate", $who, $where);
        return "/notice $who Starting updates";
    }

    return undef;
}

sub doformat {
    my $response;
    my ($quote, $author, $qn, $desc, $nick) = @_;
    print "DOFORMAT\n";
    print Dumper(\@_);

    return "" unless $quote; #don't do anything if we don't get anything back!
    $response = $quote;
    $response .= " " . RED . ",01[" . BOLD . RED . ",01By " . LIGHT_GREY . ",01$author" . CYAN . ",01" . " # " . $qn . NORMAL . RED . ",01]"
        if ($qn > 0);
    $response = BOLD . $desc . ": " . NORMAL . $response if ($desc && $desc !~ m|^/notice|);
    $response = $desc . " " . $response if ($desc && $desc =~ m|^/notice|);
    $response =~ s/%NICK%/$nick/ig;
    $response;
}

sub msgfromdb {
    my ($sender, $who, $where, $what, $heap, $kernel) = @_[ SENDER, ARG0 .. ARG2, HEAP, KERNEL ];

    print Data::Dumper->Dump([ $who, $what, $where ], [ "who", "what", "where" ]);

    my $pd = getpristart($heap->{queue}, $who);
    $heap->{queue}->enqueue($pd + 1, [ $where, $what ]);
}

sub irc_notice {
    my ($sender, $who, $where, $what, $heap, $kernel) = @_[ SENDER, ARG0 .. ARG2, HEAP, KERNEL ];
    my $nick = (split /!/, $who)[0];
    my $channel = $where->[0];

    return () if ($nick =~ /^Global|py-bopm|\w+\.\w+.\w+$/);

    $heap->{queue}->enqueue(0, [ "#DCTV", $nick . " noticed me: " . $what ]);
}

sub irc_msg {
    my ($sender, $who, $where, $what, $heap, $kernel) = @_[ SENDER, ARG0 .. ARG2, HEAP, KERNEL ];
    my $nick = (split /!/, $who)[0];
    my $channel = $where->[0];

    print "PRIVMSG $nick: $what\n";

    if (_ignore($nick, $kernel)) {
        if (my $msg = _ignoremsg($nick, $kernel)) {
            my $pd = getpristart($heap->{queue}, $nick);
            $heap->{queue}->enqueue($pd, [ $nick, $msg ]);
            return ();
        }
    }
    return () if ($what =~ /nknown comman/);

    my $response;

    my @lines;

    if ($what eq "help") {
        $response = q{HELP FOR RANDOM QUOTE BOT

TERMS
	source - A source is a collection of quotes, sources can be used in multiple triggers.
	trigger - text that someone has to say in order to get a quote, a trigger can have multiple sources.
	quote - Gee i'm not sure....
	
COMMANDS
!addquote <source> <quote> #add a quote to the source <source>
	EXAMPLE: !addquote RCF Random Conan Fact
!deletequote <id> #deletes a quote from the database, the id is given in the byline of the quote
	EXAMPLE: !deletequote 1
!getquote <id> #retrieves a specific quote
	EXAMPLE: !getquote 1
!getlastquote  #gets the last quote that's been added
	EXAMPLE: !getquote 1
!addsource <source> [desc] #add a new source for quotes!, [desc] is an OPTIONAL description of the source, e.g. Random Conan Fact
	EXAMPLE: !addsource RCF Random Conan Fact
!addtrigger <trigger> #adds a trigger to display a quote
	EXAMPLE: !addtrigger !RCF
!addsourcetotrigger <trigger> <source> #adds a source to a trigger, without this the trigger does nothing, you can have multiple sources!
	EXAMPLE: !addsourcetotrigger !RCF RCF
!removesourcefromtrigger <trigger> <source> #removes a source from a trigger
	EXAMPLE: !removesourcefromtrigger !RCF RCF
!showtriggers #lists all triggers
!showsources #lists all sources
!whatistrigger <trigger> #lists all sources associated with a trigger
	EXAMPLE: !whatistrigger !RCF

TIPS
adding a channel to a trigger will make it specific to that channel
!addtrigger !ddl#dctp

FULL EXAMPLE OF ADDING A COMPLETELY NEW TRIGGER
  !addtrigger !4chan
  !addsource 4chan 4CHAN
  !addsourcetotrigger !4chan 4chan
  !addquote 4chan THIS IS A HORRIBLE SITE YOU SHOULDN'T GO TO
this will make a trigger "!4chan" that will say
  4CHAN: THIS IS A HORRIBLE SITE YOU SHOULDN'T GO TO

If you have further questions PM simcop2387, he'll answer them eventually, also ask in the channels just in case someone else is willing to help};
        @lines = split /\n/, $response;
    }
    elsif (my $rp = doadmin($nick, [$nick], $what, $heap, $kernel)) {
        return () if $rp eq 1;
        $response = $rp;
        @lines    = messagebreak($response);
    }
    else {
        my @quote = $schema->getquote($what, [$nick]);

        return "" unless $quote[0]; #don't do anything if we don't get anything back!
        $response = doformat(@quote[ 0 .. 3 ], $nick);

        @lines = messagebreak($response); #this should really never be needed, but is here so that its consistent
    }

    return "" unless $response; #don't do anything if we don't get anything back!

    my $pd = getpristart($heap->{queue}, $nick);

    for my $p (0 .. $#lines) {
        print "LINE: $lines[$p]\n";
        $heap->{queue}->enqueue($p + $pd, [ $nick, $p == $#lines ? "$lines[$p]" : "$lines[$p] .." ]);
    }
}

sub messagebreak {
    use Text::Wrap;

    my ($message, $size) = @_;
    $size = 430 unless $size;

    $Text::Wrap::columns = $size;

    my @pieces;

    if (length($message) > $size) {
        my $partial = "";
        my @lines = split(/\n/, $message);
        print "Message was too big, splitting\n";

        foreach my $line (@lines) {
            if (length($line) > $size) {
                #line too big, wrap her
                print "\tLine was too big wrapping\n";

                my @wrapped = split(/\n/, wrap("", "", $line));
                push(@pieces, $partial) if (length($partial) > 0);
                $partial = pop(@wrapped);

                for my $wrap (@wrapped) {
                    push(@pieces, $wrap);
                }
            }
            else {
                if (length($line) + length($partial) > $size) {
                    push(@pieces, $partial);
                    $partial = $line;
                }
                else {
                    $partial = "$partial\n$line";
                }
            }
        }

        push(@pieces, $partial) if ($partial ne "");
    }
    else {
        push(@pieces, $message);
    }

    shift @pieces if ($pieces[0] =~ /^\s*$/);

    #print Dumper(\@pieces);

    if ($pieces[0] =~ m|/notice\s+(\S+)|) {
        print "MATCHED FOR NOTICE\n";
        my $who = $1;
        for (1 .. $#pieces) {
            $pieces[$_] =~ s|^|/notice $who |;
        }
    }

    s/\n//g for (@pieces);

    return (@pieces);
}

sub getpristart {
    my $queue = shift;
    my $who   = shift;

    my @items = $queue->peek_items(sub { my $payload = shift; $payload->[0] eq $who });

    my $max = $items[0][0];
    shift @items;
    for my $item (@items) {
        if ($item->[0] > $max) { $max = $item->[0]; }
    }

    $max ||= 0; #make sure its a numba!
    return $max + 1;
}

#irc_dcc_request irc_dcc_done irc_dcc_error

sub startxdcc {
    my ($sender, $heap, $kernel) = @_[ SENDER, HEAP, KERNEL ];
    my $botname = $_[ARG0];

    $heap->{irc}->yield(privmsg => $botname, "xdcc send #1"); #simple
}

sub cancelotherxdcc {
    my ($sender, $heap, $kernel) = @_[ SENDER, HEAP, KERNEL ];
    my $botname = $_[ARG0];

    $heap->{irc}->yield(privmsg => $botname, "xdcc cancel"); #simple
}

sub irc_dcc_request {
    my ($user, $type, $port, $cookie, $file, $size, $addr) = @_[ ARG0 .. $#_ ];
    return if $type ne 'SEND';

    my $irc = $_[SENDER]->get_heap();
    my $nick = (split /!/, $user)[0];

    print "$nick wants to send me '$file' ($size bytes) from $addr:$port\n";
    $irc->yield(dcc_accept => $cookie);
}

sub irc_dcc_error {
    my ($sender, $heap, $kernel) = @_[ SENDER, HEAP, KERNEL ];
    my ($wheel, $error, $user, $type, $port, $file, $size, $addr) = @_[ ARG0 .. $#_ ];
    #     return if $type ne 'SEND';

    $kernel->post("BotSync", "dccerror", $user, $file, $error, $addr);
}

sub irc_dcc_done {
    my ($sender, $heap, $kernel) = @_[ SENDER, HEAP, KERNEL ];
    my ($wheel, $user, $type, $port, $file, $size, $addr) = @_[ ARG0 .. $#_ ];
    #     return if $type ne 'SEND';

    $kernel->post("BotSync", "dccdone", $user, $file, $addr);
}

1;
