package MangaParse;

use strict;
use warnings;

use Data::Dumper;

use POE qw(Component::Client::HTTP);
use POE::Component::IRC::Common qw( :ALL );

my $ses = POE::Session->create(
    package_states => [
        MangaParse => [qw(_start getmanga httpcallback updatelog announce)],
    ],
    heap => { nextchapter => 0 });

POE::Component::Client::HTTP->spawn(
    Agent           => 'Conanquotes (see simcop2387@simcop2387.info for more information)', # defaults to something long
    Alias           => 'mangaua', # defaults to 'weeble'
    From            => 'simcop2387@simcop2387.info', # defaults to undef (no header)
    Timeout         => 60, # defaults to 180 seconds
    FollowRedirects => 2, # defaults to 0 (off)
);

sub _start {
    my ($kernel, $heap) = @_[ KERNEL, HEAP ];

    #	$kernel->yield("getmanga");
}

sub getmanga {
    my ($kernel, $heap) = @_[ KERNEL, HEAP ];

    my $url = "http://manga.dctp.ws/Main.html";

    my $req = HTTP::Request->new(GET => $url); # a simple HTTP request

    $kernel->post(
        'ua', 'request', # http session alias & state
        'httpcallback', # my state to receive responses
        $req
    );

}

sub announce {
    my ($sender, $kernel, $heap) = @_[ SENDER, KERNEL, HEAP ];
    my $chapter = $_[ARG0];

    my $msg = UNDERLINE . WHITE . ",01New manga chapter " . BOLD . "$chapter:" . NORMAL . " read it at http://manga.dctp.ws/";

    $kernel->post(IRCHANDLER => msgfromdb => "#DCTP",          "#DCTP",          $msg);
    $kernel->post(IRCHANDLER => msgfromdb => "simcop2387-lap", "simcop2387-lap", $msg);
    $kernel->post(IRCHANDLER => msgfromdb => "simcop2387",     "simcop2387",     $msg);
}

sub updatelog {
    my ($sender, $kernel, $heap) = @_[ SENDER, KERNEL, HEAP ];
    my $msg = $_[ARG0];

    print "MangaLog: $msg\n";
    $kernel->post(IRCHANDLER => msgfromdb => "#DCTV", "#DCTV", $msg);
}

sub httpcallback {
    my ($sender, $kernel, $heap) = @_[ SENDER, KERNEL, HEAP ];
    my ($request_packet, $response_packet) = @_[ ARG0, ARG1 ];

    my $response = $response_packet->[0];
    my @buffer;

    print "Got http response\n";
    if ($response->is_success) {
        push @buffer, "manga site successful, starting to parse";

        my $q = $response->content();
        eval {
            my $tree = HTML::TreeBuilder->new_from_content($q);
            my $nextchapterstr = $tree->look_down(class => "text", width => "305")->as_text();
            push @buffer, "Parsed chapter: $nextchapterstr";

            my ($chapter) = ($nextchapterstr =~ m/Next File:\s*(\d+)/g);

            if ($heap->{nextchapter} == $chapter) {
                print "Parsed chapter, no changes\n";
            }
            else {
                $kernel->yield(updatelog => $_) for @buffer;
                $kernel->yield(updatelog => "Found new chapter, $chapter in the place of " . $heap->{nextchapter});
                $kernel->yield(updatelog => "Making announcement for " . $heap->{nextchapter});
                $kernel->yield(announce => $heap->{nextchapter}) unless !$heap->{nextchapter}; #announce unless we just started
                $heap->{nextchapter} = $chapter;
            }

        }; #ignore failures in parsing
    }
    else {
        $kernel->yield(updatelog => $_) for @buffer;
        $kernel->yield(updatelog => "manga site failed, scheduling retry in 30 minutes: @{[$response->status_line]}");
    }

    $kernel->delay_add("getmanga", 1800);
}

1;
