--
-- PostgreSQL database dump
--

SET statement_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;

SET search_path = public, pg_catalog;

SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: adminlogs; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE adminlogs (
    "when" timestamp with time zone DEFAULT clock_timestamp() NOT NULL,
    nickname text NOT NULL,
    hostmask text NOT NULL,
    command text NOT NULL
);


ALTER TABLE public.adminlogs OWNER TO conanquotes;

--
-- Name: bansource; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE bansource (
    cid integer NOT NULL,
    sid integer NOT NULL
);


ALTER TABLE public.bansource OWNER TO conanquotes;

--
-- Name: channelsetup; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE channelsetup (
    channel text NOT NULL,
    stripcolors boolean DEFAULT false,
    cid integer NOT NULL
);


ALTER TABLE public.channelsetup OWNER TO conanquotes;

--
-- Name: channelsetup_cid_seq; Type: SEQUENCE; Schema: public; Owner: conanquotes
--

CREATE SEQUENCE channelsetup_cid_seq
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.channelsetup_cid_seq OWNER TO conanquotes;

--
-- Name: channelsetup_cid_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: conanquotes
--

ALTER SEQUENCE channelsetup_cid_seq OWNED BY channelsetup.cid;


--
-- Name: channelsetup_cid_seq; Type: SEQUENCE SET; Schema: public; Owner: conanquotes
--

SELECT pg_catalog.setval('channelsetup_cid_seq', 2, true);


--
-- Name: episodeinfo; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE episodeinfo (
    eid integer NOT NULL,
    title text,
    "group" text,
    fsize text,
    fcodec text,
    fcrc text,
    botlisting text,
    filelink text,
    airdate text,
    ratings text,
    epdesc text,
    fcomment text,
    animeoriginal text,
    chapters text,
    filelink_date timestamp with time zone DEFAULT clock_timestamp() NOT NULL
);


ALTER TABLE public.episodeinfo OWNER TO conanquotes;

--
-- Name: TABLE episodeinfo; Type: COMMENT; Schema: public; Owner: conanquotes
--

COMMENT ON TABLE episodeinfo IS 'This table doesn''t get used by the bot, but does get used by the web interface so that its easier to have editing of episode information (e.g. links)';


--
-- Name: groupaliases; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE groupaliases (
    gid integer NOT NULL,
    alias text
);


ALTER TABLE public.groupaliases OWNER TO conanquotes;

--
-- Name: groupinfo; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE groupinfo (
    gid integer NOT NULL,
    fullname text NOT NULL,
    releases integer DEFAULT 0 NOT NULL,
    alive boolean DEFAULT true NOT NULL,
    mergeddctp boolean DEFAULT false NOT NULL
);


ALTER TABLE public.groupinfo OWNER TO conanquotes;

--
-- Name: groupinfo_gid_seq; Type: SEQUENCE; Schema: public; Owner: conanquotes
--

CREATE SEQUENCE groupinfo_gid_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.groupinfo_gid_seq OWNER TO conanquotes;

--
-- Name: groupinfo_gid_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: conanquotes
--

ALTER SEQUENCE groupinfo_gid_seq OWNED BY groupinfo.gid;


--
-- Name: groupinfo_gid_seq; Type: SEQUENCE SET; Schema: public; Owner: conanquotes
--

SELECT pg_catalog.setval('groupinfo_gid_seq', 1, false);


--
-- Name: ignores; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE ignores (
    nick text NOT NULL,
    message text
);


ALTER TABLE public.ignores OWNER TO conanquotes;

--
-- Name: qidseq; Type: SEQUENCE; Schema: public; Owner: conanquotes
--

CREATE SEQUENCE qidseq
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.qidseq OWNER TO conanquotes;

--
-- Name: qidseq; Type: SEQUENCE SET; Schema: public; Owner: conanquotes
--

SELECT pg_catalog.setval('qidseq', 5368, true);


--
-- Name: quotes; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE quotes (
    qid integer DEFAULT nextval('qidseq'::regclass) NOT NULL,
    sid integer NOT NULL,
    quote text NOT NULL,
    who text,
    deleted boolean DEFAULT false NOT NULL,
    counter integer DEFAULT 0 NOT NULL
);


ALTER TABLE public.quotes OWNER TO conanquotes;

--
-- Name: sidseq; Type: SEQUENCE; Schema: public; Owner: conanquotes
--

CREATE SEQUENCE sidseq
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.sidseq OWNER TO conanquotes;

--
-- Name: sidseq; Type: SEQUENCE SET; Schema: public; Owner: conanquotes
--

SELECT pg_catalog.setval('sidseq', 100, true);


--
-- Name: sources; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE sources (
    sid integer DEFAULT nextval('sidseq'::regclass) NOT NULL,
    name text NOT NULL,
    "desc" text
);


ALTER TABLE public.sources OWNER TO conanquotes;

--
-- Name: tidseq; Type: SEQUENCE; Schema: public; Owner: conanquotes
--

CREATE SEQUENCE tidseq
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.tidseq OWNER TO conanquotes;

--
-- Name: tidseq; Type: SEQUENCE SET; Schema: public; Owner: conanquotes
--

SELECT pg_catalog.setval('tidseq', 148, true);


--
-- Name: tidsources; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE tidsources (
    sid integer NOT NULL,
    tid integer NOT NULL
);


ALTER TABLE public.tidsources OWNER TO conanquotes;

--
-- Name: triggers; Type: TABLE; Schema: public; Owner: conanquotes; Tablespace: 
--

CREATE TABLE triggers (
    tid integer DEFAULT nextval('tidseq'::regclass) NOT NULL,
    trigger text NOT NULL,
    counter bigint DEFAULT 0 NOT NULL
);


ALTER TABLE public.triggers OWNER TO conanquotes;

--
-- Name: list; Type: VIEW; Schema: public; Owner: conanquotes
--

CREATE VIEW list AS
    SELECT quotes.quote, triggers.trigger, quotes.who, quotes.qid, sources.name, sources."desc", quotes.counter, sources.sid FROM triggers, quotes, tidsources, sources WHERE ((((triggers.tid = tidsources.tid) AND (quotes.sid = tidsources.sid)) AND (sources.sid = quotes.sid)) AND (NOT quotes.deleted)) ORDER BY quotes.counter;


ALTER TABLE public.list OWNER TO conanquotes;

--
-- Name: cid; Type: DEFAULT; Schema: public; Owner: conanquotes
--

ALTER TABLE channelsetup ALTER COLUMN cid SET DEFAULT nextval('channelsetup_cid_seq'::regclass);


--
-- Name: gid; Type: DEFAULT; Schema: public; Owner: conanquotes
--

ALTER TABLE groupinfo ALTER COLUMN gid SET DEFAULT nextval('groupinfo_gid_seq'::regclass);


--
-- Data for Name: adminlogs; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY adminlogs ("when", nickname, hostmask, command) FROM stdin;
\.


--
-- Data for Name: bansource; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY bansource (cid, sid) FROM stdin;
1	88
\.


--
-- Data for Name: channelsetup; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY channelsetup (channel, stripcolors, cid) FROM stdin;
#dctp	f	1
#goshoproject	f	2
\.


--
-- Data for Name: episodeinfo; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY episodeinfo (eid, title, "group", fsize, fcodec, fcrc, botlisting, filelink, airdate, ratings, epdesc, fcomment, animeoriginal, chapters, filelink_date) FROM stdin;
20002	 16 Suspects	DCTP	233mb	XviD	922F42C9	\N	\N	\N	\N	Conan, the Detective Boys, Hattori and the rest of the gang have been invited to Inspector Shiratori's new house for a party celebrating its completion. With delicious food and drink, everyone is pleased as punch to be in attendance; they are especially impressed with Shiratori's wine cellar and multi-million-dollar bottle of Chateau Latour. However, disaster soon strikes when the rare wine is discovered - broken on the floor! Everyone, including Conan, is a suspect; who committed such a crime?	\N	\N	\N	2010-05-17 17:13:51.278958-04
30029	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
567	Murderous Intent Raining on an Outdoor Spa	DCTP	172mb | 320mb	XviD | H264	9072ADFF | E3C34FD7	\N	http://www.megaupload.com/?d=1NEA2BRJ | 720: http://www.megaupload.com/?d=1Y7ZCZIP	2010/Mar/06	\N	\N	\N	\N	TV Original	2010-08-23 10:33:39.549711-04
557	A Dangerous Party of Two	DCTP	174mb | 320mb	XviD	F3FF61BB | E8B25134	\N	http://www.megaupload.com/?d=9730P0AV | 720: http://www.megaupload.com/?d=0ZRVVVKE	2009/Nov/28	\N	\N	\N	\N	Vol 65 File 6-7	2010-08-23 10:33:26.41928-04
599	A Friend of Justice	DCTP	172mb | 320mb	XviD | H264	299C5A84 | 2C2E878A	\N	http://www.megaupload.com/?d=BT72B3U3 | 720: http://www.megaupload.com/?d=JRC6N03Q	2010/Dec/11	\N	\N	\N	on	TV Original	2010-12-18 18:07:55.418588-05
601	The Dream the Kappa Saw (Part 2)	DCTP	172mb | 320mb	XviD | H264	1224DF03 | 9F375B3F	\N	http://www.megaupload.com/?d=SNG2QF5P | 720: http://www.megaupload.com/?d=9WS8N2KC	2010/Dec/25	\N	\N	Watch online at: http://dcod.ninjajosh.com/?code=Detective Conan - 601 [DCTP].m4v	\N	Vol 69 File 1-3	2011-01-08 12:23:22.000032-05
612	The Ablazed Demon Dog (Footprints Chapter)	DCTP	172 MB/350 MB	XviD/H264	4A23ACA6/1715F0CA	\N	http://www.megaupload.com/?d=3LLJTTGC | 720: http://www.megaupload.com/?d=ZB9JKJ1E	\N	\N	\N	\N	\N	\N	2011-05-04 06:15:45.666843-04
2	Company President's Daughter Kidnapping Case	AConan	224mb	XviD	3E743379	\N	http://www.megaupload.com/?d=84KW7S40	1996/Jan/15	11.9	\N	\N	\N	Vol 1 File 2-5	2010-08-23 10:24:38.365964-04
10029	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20021	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30015	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
542	Ikkaku Rock's Disappearing Fish (Part 1)	DCTP	172mb | 349mb	XviD	36AE9399 | 60A2570C	\N	http://www.megaupload.com/?d=QV4XJC02 | 720: http://www.megaupload.com/?d=KB7SKHV2	2009/July/25	6.5	\N	\N	\N	Vol 64 File 1-3	2010-08-23 10:33:09.765752-04
7	Once-A-Month Present Threat Case	AConan	218mb	\N	863DA758	\N	http://www.megaupload.com/?d=1S90X5QK	1996/Feb/19	12.7	\N	\N	\N	Vol 3 File 7-10	2010-08-23 10:24:42.134706-04
10032	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20024	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
130	The Stadium Indiscriminate Threatening Case (Part 1)	Neg	346mb	\N	E66D5124	\N	http://www.megaupload.com/?d=4I3MB34I	1999/Jan/11	21.3	\N	\N	\N	Vol 19 File 9 - Vol 20 File 1	2010-08-23 10:26:33.302193-04
387	The Dissonance of the Stradivarius (Part 3: Postlude)	DCTP	172mb	\N	ABAAD442	\N	http://www.megaupload.com/?d=Z63IIPD3	2005/Feb/06	12.9	\N	\N	\N	Vol 46 File 2-6	2010-08-23 10:30:47.495566-04
611	The Ablazed Demon Dog (Will-o'-the-Wisp Chapter)	DCTP	172 MB/350 MB	XviD/H264	32E5003C | 10CC8089	\N	http://www.megaupload.com/?d=IKFZHSJJ | 720: http://www.megaupload.com/?d=N0E5EZA1	\N	\N	\N	\N	\N	\N	2011-05-03 21:14:46.290976-04
602	The Devil in the Tennis Court	DCTP	172mb | 320mb	XviD | H264	30357DE4 | 22EBB3BB	http://www.fileserve.com/file/tu64vAR | 720: http://www.fileserve.com/file/5Gb3WWM	http://www.megaupload.com/?d=RQO5YTCJ | 720: http://www.megaupload.com/?d=MTKRYT5M	\N	\N	\N	\N	\N	\N	2011-01-17 13:38:39.633861-05
261	The Fearful Legend of the Snowy Night (Part 1)	Kienai-AZFS	172mb	h264	ED11012B	Join #AZFS and type /msg AZFS|Kudo XDCC send 22	http://megaupload.com/?d=8VIGK5Z1	2001/Dec/10	19.5	\N	\N	\N	TV Original	2010-08-23 10:28:44.541588-04
577	The Truth Lit Up by the Fireflies	DCTP	173mb | 320mb	XviD | H264	17768A3D | E5FC7B2B	\N	http://www.megaupload.com/?d=VVEAXHHX | 720: http://www.megaupload.com/?d=JPH16K0A	2010/Jun/19	\N	\N	\N	\N	TV Original	2010-08-23 10:33:47.756077-04
610	The Victim is Kudou Shinichi	DCTP	172 MB/320 MB	XviD/H264	A74EA890/D042FA37	\N	http://www.megaupload.com/?d=ZFE00INR | 720: http://www.megaupload.com/?d=1A2951DM	\N	\N	\N	\N	\N	\N	2011-05-03 21:15:19.687113-04
10001	Skyscraper on a Timer	AConan	700mb	XviD	231C23E1	\N	\N	1997/Apr/19	\N	Shinichi is invited to a party held by a famous architect and Shinichi asks Ran to go instead. There a mysterious person plants a lot of bombs and gives Shinichi clues as to the location of the bombs. Shinichi tries to uncover all the bombs and save Ran's life.	\N	\N	\N	2010-05-17 17:13:51.278958-04
30034	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
578	The Crisis Beckoned by the Red Omen	DCTP	173mb | 320mb	XviD | H264	6AE111CB | 4F29D81D	\N	http://www.megaupload.com/?d=RFS11OSR | 720: http://www.megaupload.com/?d=C1IFRHHE	2010/Jun/26	\N	\N	\N	\N	Vol 67 File 3	2010-08-23 10:33:48.562254-04
8	Art Gallery Owner Murder Case	AConan	209mb	\N	B70E0F42	\N	http://www.megaupload.com/?d=SMUDW0JK	1996/Feb/26	13.1	\N	\N	\N	Vol 4 File 1-3	2010-08-23 10:24:42.840374-04
603	The Seance's Double Locked Room Mystery Case (First Locked Room)	DCTP	172 MB/350 MB	XviD/H264	3B82DC94/281B97D6	\N	http://www.megaupload.com/?d=ZY4CTHFM | 720: http://www.megaupload.com/?d=UJL4X0EE	1/29/11	\N	\N	\N	on	\N	2011-02-04 18:55:12.435091-05
10033	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
565	The Eyewitness who Did Not See	DCTP	172mb | 320mb	XviD	3A47A234 | 3F7C5868	\N	http://www.megaupload.com/?d=HRGH5FJ6 | 720: http://www.megaupload.com/?d=XJO8XQ9J	2010/Feb/06	\N	\N	\N	\N	\N	2010-08-23 10:33:37.820197-04
492	Clash of Red and Black: Blood Relative	DCTP	172mb | 233mb	\N	E97E8995 | 5F139F72	\N	http://www.megaupload.com/?d=RJYIPIBG | 720: http://www.megaupload.com/?d=30M6WMUP	2008/Jan/21	9	\N	\N	\N	Vol 56 File 10 - Vol 57 File 4	2010-08-23 10:32:25.878638-04
47	Sports Club Murder Case	AConan	168mb	\N	313AFDAA	\N	http://www.megaupload.com/?d=KVG2R8TH	1997/Feb/10	16.6	\N	\N	\N	TV Original	2010-08-23 10:25:14.570639-04
579	The Suggestion of Black Thirteen	DCTP	172mb | 320mb	XviD | H264	BDCCE830 | FCD1AE48	\N	http://www.megaupload.com/?d=YZJJ4YK0 | 720: http://www.megaupload.com/?d=E81XCNCS	2010/Jul/03	\N	\N	\N	\N	Vol 67 Files 4-8	2010-08-23 10:33:49.37397-04
30003	Magic File 3 (Movie 13 Side-Story OVA)	DCTP	214mb	\N	642DAAF9	\N	http://www.megaupload.com/?d=J2ODW8DK	\N	\N	\N	\N	\N	\N	2010-08-23 10:37:08.117579-04
30011	Kid the Phantom Thief  (Magic Kaito)「マジク カイト」	DCTP	231mb | 484mb	\N	75950F73 | 700395F0	\N	http://www.megaupload.com/?d=N9EPK86O | 720: http://www.megaupload.com/?d=3V8ZXWFL	2010/Apr/17	\N	\N	\N	\N	\N	2010-08-23 10:37:14.261227-04
608	White Day of Betrayal (Part One)	DCTP	172 MB/350 MB	XviD/H264	26D19806/323D802D	\N	http://www.megaupload.com/?d=Y80ITI4Y | 720p: http://www.megaupload.com/?d=CEVK5CLK	\N	\N	\N	\N	\N	\N	2011-05-03 21:13:11.842882-04
296	Houseboat Fishing Shock	AZFS	174mb	XviD	FE4DB0C2	Join #AZFS and type /msg AZFS|Kudo XDCC send 26	http://megaupload.com/?d=TPWXDKW9	2002/Oct/14	15.4	\N	\N	\N	TV Original	2010-11-21 01:57:14.976198-05
604	The Seance's Double Locked Room Mystery Case (Second Locked Room)	DCTP	172 MB/350 MB	XviD/H264	D4759F0B/5B8E0EC0	\N	http://www.megaupload.com/?d=999B4QMN | 720: http://www.megaupload.com/?d=85N683CJ	2/5/2011	\N	\N	\N	on	\N	2011-02-16 20:36:26.463656-05
9	Tenkaichi Night Festival Murder Case	AConan	209mb	\N	20E49285	\N	http://www.megaupload.com/?d=AQ9PG4SM	1996/Mar/04	13	\N	\N	\N	Vol 6 File 9 - Vol 7 File 1	2010-08-23 10:24:43.661687-04
401	The Caught Red-handed Jewel Robber (Part 1)	Ani-Jiyuu	176mb	\N	606359E2	\N	http://www.megaupload.com/?d=N0776DLO	2005/Jun/06	10.6	\N	\N	\N	Vol 47 File 5-7	2010-08-23 10:31:01.623997-04
505	Lawyer Kisaki Eri's Testimony (Part 1)	DCTP	172mb | 485mb	\N	9E92A8EA | 76D08CBA	\N	http://www.megaupload.com/?d=HG8ZNL2Y	2008/Jun/16	7.9	\N	\N	\N	Vol 59 File 2-4	2010-08-23 10:32:41.803766-04
10034	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
141	The Night Before the Wedding Locked Room Murder Case (Part 1)	Neg	173mb	\N	BA8BB5AE	\N	http://www.megaupload.com/?d=2MY9ZA3T	1999/Apr/19	19.4	\N	\N	\N	Vol 21 File 11 - Vol 22 File 1-3	2010-08-23 10:26:44.841473-04
71	A Stalker's Murder Case	AConan	172mb	\N	B2E178E7	\N	http://www.megaupload.com/?d=NT5WXP6G	1997/Aug/11	13.2	\N	\N	\N	TV Original	2010-08-23 10:25:33.680681-04
30036	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
331	Suspicious Curry (Part 1)	AZFS	147mb	XviD	31A1F9E5	Join #AZFS and type /msg AZFS|Kudo XDCC send 28	http://www.megaupload.com/?d=42YBWUR7	2003/Aug/11	13.9	\N	\N	\N	Vol 40 File 4-6	2010-08-23 10:29:52.066138-04
194	The Significant Music Box (Part 1)	KnKF	175mb	\N	9B96FE12	\N	http://www.megaupload.com/?d=QDC18R9M	2000/Jun/12	19.3	\N	\N	\N	Vol 26 File 8-10	2010-08-23 10:27:33.345306-04
10014	The Lost Ship in the Sky	DCTP	1.4gb	XviD | H264	44005329 | DA6EB844	\N	Part1: http://www.fileserve.com/file/scE3n2d Part2: http://www.fileserve.com/file/4dYkaph | h264: part1: http://www.fileserve.com/file/DbduqMB part2: http://www.fileserve.com/file/yhDNXMk	2010/Apr/17	\N	In the film's story, Kid has his eyes set on the "Lady of the Sky" jewel aboard Bell 3, the largest airship in the world. However, a mysterious terrorist group called Red Shamu-neko has hijacked the airship itself, along with Conan and his allies Kogoro and Ran. What are their purposes? What will happen between Conan and Kid?	http://www.dctp.ws/Detective%20Conan%20-%20Movie%2014%20-%20The%20Lost%20Ship%20in%20the%20Sky%20[DCTP][XviD][44005329].torrent | http://www.dctp.ws/Detective%20Conan%20-%20Movie%2014%20-%20The%20Lost%20Ship%20in%20the%20Sky%20[DCTP][H264][DA6EB844].torrent	\N	\N	2010-10-27 13:16:34.806924-04
605	The Seance's Double Locked Room Mystery Case (Opened Locked Room)	DCTP	172 MB/350 MB	XviD/H264	2D33D524/8B13241E	\N	http://www.megaupload.com/?d=INYQ8XX5 | 720: http://www.megaupload.com/?d=2R74JYIY	2/12/2011	\N	\N	\N	on	\N	2011-02-22 19:05:32.06706-05
305	The Unseen Suspect (Part 1)	DCTP	348mb	XviD	221A7174	\N	http://www.megaupload.com/?d=22ADVSIO	2003/Jan/13	15.4	\N	Part 2 is concatenated.	\N	Vol 37 File 2-4	2010-08-23 10:29:30.890519-04
10	Pro Soccer Player Blackmail Case	AConan	215mb	\N	495E2C38	\N	http://www.megaupload.com/?d=AKCGW1FV	1996/Mar/11	13.7	\N	\N	\N	Vol 7 File 8 - Vol 8 File 1	2010-08-23 10:24:45.078219-04
482	The Mountain Witch's Cutlery (Part 2)	DCTP	348mb	XviD	AA275948	\N	http://www.megaupload.com/?d=KXOH1SER	2007/Aug/06	8.2	\N	Part 1 is concatenated.	\N	Vol 56 File 4-6	2010-08-23 10:32:12.968163-04
570	The Crime with Zero Possibility to Be Proven	DCTP	171mb | 321mb	XviD | H264	60676799 | 6F5879B9	\N	http://www.megaupload.com/?d=UDQKHT2Z | 720: http://www.megaupload.com/?d=4O9VC2FF	2010/Mar/27	\N	\N	There is a typo in filename's crc. It says 0676799, first digit is missing.	\N	TV Original	2010-08-23 10:33:41.871259-04
517	Fuurinkazan, Shadow and Lightning's Conclusion (Part 2)	DCTP	172mb	\N	769074C4	\N	http://www.megaupload.com/?d=LESN3QQB	2008/Nov/10	8	\N	\N	\N	Vol 59 File 5-10	2010-08-23 10:32:50.836642-04
527	The Malice Hidden Behind the Masque	DCTP	172mb	XviD	6D24B0D7	\N	http://www.megaupload.com/?d=PCSUBQPR	2009/Mar/02	8.4	\N	\N	\N	TV Original	2010-08-23 10:32:58.053555-04
30018	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30037	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
218	Megure's Sealed Secret (Part 2)	DCTP	173mb	XviD	5A6B4196	\N	http://www.megaupload.com/?d=NCWQBFTJ	2000/Dec/18	18.1	\N	\N	\N	Vol 28 File 11 - Vol 29 File 2	2011-02-05 01:01:30.16175-05
14	The Mysterious Shooting Message Case	AConan	234mb	\N	E0774045	\N	http://www.megaupload.com/?d=25H529VM	1996/Apr/29	12.5	\N	\N	\N	TV Original	2010-08-23 10:24:48.03531-04
569	Inspector Shiratori, Memories of the Cherry Blossom (Part 2)	DCTP	172mb | 320mb	XviD | H264	F2A4A477 | CC638498	\N	http://www.megaupload.com/?d=VX0YSLRG | 720: http://www.megaupload.com/?d=2Y40UCIY	2010/Mar/20	\N	\N	\N	\N	Vol 66 File 2-4	2010-08-23 10:33:41.182598-04
43	Edogawa Conan Kidnapping Case	AConan	160mb	\N	A43EE10B	\N	http://www.megaupload.com/?d=C9NSB7A3	1997/Jan/13	18.8	\N	\N	\N	Vol 5 File 10 - Vol 6 File 1	2010-08-23 10:25:11.442181-04
607	Courtroom Confrontation IV: Juror Kobayashi Sumiko (Part Two)	DCTP	172 MB/350 MB	XviD/H264	C1B60565/E25B1C83	\N	http://www.megaupload.com/?d=FB5AQIOC | 720: http://www.megaupload.com/?d=H7A5LK1N	\N	\N	\N	\N	on	\N	2011-03-06 15:02:43.398265-05
481	The Mountain Witch's Cutlery (Part 1)	DCTP	348mb	XviD	AA275948	\N	http://www.megaupload.com/?d=KXOH1SER	2007/Jul/30	8.8	\N	Part 2 is concatenated.	\N	Vol 56 File 4-6	2010-08-23 10:32:12.236796-04
10036	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20028	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20008	The Casebook of Female High-School Detective Suzuki Sonoko	DCTP	224mb	XviD	18A002E4	\N	http://www.megaupload.com/?d=U6AC55WM	\N	\N	A murder case occurred at the editorial department of "Weekly News". While the police were not getting anywhere in their investigation, Suzuki Sonoko, the high-school detective girl shows up! The Story goes towards unexpected direction...Will Conan, Detective Boys and Sonoko be able to solve the case? (Miyano-sama @ DCTP)\r\n	Original filename says CRC: [927B7E46] but that is wrong!	\N	\N	2010-08-23 10:36:54.155228-04
580	The Black Time Limit Drawing Near	DCTP	173mb | 319mb	XviD | H264	06DB6ACE | D8A23606	\N	http://www.megaupload.com/?d=493PIDK9 | 720: http://www.megaupload.com/?d=MCW61FZW	2010/Jul/10	\N	\N	\N	\N	Vol 67 Files 4-8	2010-08-23 10:33:50.84419-04
22	Luxury Liner Serial Murder Case (Part 1)	AConan	178mb	\N	700BAC97	\N	http://www.megaupload.com/?d=O2EGFKW0	1996/Jul/01	12.8	\N	\N	\N	Vol 3 File 1-6	2010-08-23 10:24:54.672089-04
606	Courtroom Confrontation IV: Juror Kobayashi Sumiko (Part One)	DCTP	172 MB/350 MB	XviD/H264	C803C1DB/A6188588	\N	http://www.megaupload.com/?d=C5B0TPYQ | 720: http://www.megaupload.com/?d=QOFZI6G5	2/19/2011	\N	\N	\N	on	\N	2011-03-06 15:02:46.078652-05
69	The Night Baron Murder Case (Part 2: The Suspicion)	AConan	172mb	\N	9206D552	\N	http://www.megaupload.com/?d=3FMBPHNH	1997/Jul/28	18.2	\N	\N	\N	Vol 8 File 2-7	2010-08-23 10:25:32.217067-04
20000	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
28	Kogoro's Class Reunion Murder Case (Part 2)	AConan	175mb	\N	A80F1708	\N	http://www.megaupload.com/?d=QLJWWO4L	1996/Aug/12	12.6	\N	\N	\N	Vol 9 File 4-6	2010-08-23 10:24:59.171982-04
67	Stage Actress Murder Case	AConan	172mb	\N	F9104ADF	\N	http://www.megaupload.com/?d=Q3LGGY0Q	1997/Jul/14	19.3	\N	\N	\N	TV Original	2010-08-23 10:25:30.527468-04
198	A Super Car's Trap (Part 2)	KnKF	172mb	\N	A8D9DA01	\N	http://www.megaupload.com/?d=V5UVI2BT	2000/Jul/10	16.8	\N	\N	\N	TV Original	2010-08-23 10:27:36.181968-04
327	The Red Horse within the Flames (Part 3: The Solution)	Kienai	172mb	\N	6368787E	Join #Kienai and type /msg Sonoko-Chan XDCC send 16	http://www.megaupload.com/?d=VAD4JX9O	2003/Jul/07	16	\N	\N	\N	Vol 39 File 1-5	2010-08-23 10:29:48.738659-04
320	The Art of Ninja Alibi Craft	DCTP	173mb	XviD	6D2ABC34	\N	http://www.megaupload.com/?d=WIHR2CEN	2003/May/19	13.4	\N	\N	\N	TV Original	2010-08-23 10:29:43.223342-04
471	Rented Car Out of Control!	DCTP	172mb	\N	260F6773	\N	http://www.megaupload.com/?d=ECH8CEZ0	2007/May/07	9.9	\N	\N	\N	TV Original	2010-08-23 10:32:04.463192-04
202	The Tenth Passenger (Part 2)	KnKF	172mb	\N	A55E8074	\N	http://www.megaupload.com/?d=UTRHB4Z2	2000/Aug/07	16.7	\N	\N	\N	TV Original	2010-08-23 10:27:39.471591-04
10003	The Last Wizard of the Century	Neg	700mb	XviD	A9501AF8	\N	\N	1999/Apr/17	\N	Kaitou Kid announces to the police that he intends to steal the Russian Imperial Easter Egg, which is currently being held in Osaka. As Conan pursues his rival, he discovers that there's more to this case then simply stopping a robbery.	\N	\N	\N	2010-05-17 17:13:51.278958-04
217	Megure's Sealed Secret (Part 1)	DCTP	173mb	XviD	51872EDC	\N	http://www.megaupload.com/?d=HDWEXHW1	2000/Dec/11	19.2	\N	\N	\N	Vol 28 File 11 - Vol 29 File 2	2011-02-05 01:01:17.57738-05
20004	Conan and Kid and the Crystal Mother	Kaizou	228mb	XviD	5DCAF827	\N	\N	\N	\N	The Kaitou Kid - aka Kaito Kuroba - is currently on the prowl for his next heist: the Crystal Mother, Europe's largest Topaz! However, acquiring the jewel won't be easy as it has been hidden by its owner, the Queen of Ingram, aboard the Royal Express luxury train. What's worse is that not only Conan, but also the crown prince of Ingram stand in his way! Will Kaitou Kid pull off the crime of the century?	\N	\N	\N	2010-05-17 17:13:51.278958-04
328	Mystery of the Birthday Wine	AZFS	172mb	XviD	690B685D	Join #AZFS and type /msg AZFS|Kudo XDCC send 27	http://www.megaupload.com/?d=A6RHA27K	2003/Jul/14	14.2	\N	\N	\N	TV Original	2010-08-23 10:29:49.459673-04
390	The Metropolitan Police Detective Love Story 6 (Part 1)	DCTP	173mb	\N	859DEC4D	\N	http://www.megaupload.com/?d=D729C6E1	2005/Feb/28	13.2	\N	\N	\N	Vol 44 File 4-6	2010-08-23 10:30:52.701055-04
32	Coffee Shop Murder Case	AConan	169mb	\N	FDA732D5	\N	http://www.megaupload.com/?d=ZX2WNC5G	1996/Sep/09	14.3	\N	\N	\N	Vol 11 File 5-7	2010-08-23 10:25:02.446709-04
84	Ski Lodge Murder Case (Part 1)	AConan	172mb	\N	53BAF26F	\N	http://www.megaupload.com/?d=TM4N0T6A	1997/Dec/08	19.7	\N	\N	\N	Vol 14 File 9 - Vol 15 File 3	2010-08-23 10:25:44.949165-04
437	Aya Ueto & Shinichi: The Promise from 4 Years Ago	DCTP	170mb	XviD	18E9D4EF	\N	http://www.megaupload.com/?d=ECXHY8A5	2006/May/08	12	\N	\N	\N	TV Original	2010-08-23 10:31:30.243447-04
295	The Smash of Determination and Love (Part 2)	Kienai	172mb	H264	647A5C64	Join #Kienai and type /msg Sonoko-Chan XDCC send 12	http://www.megaupload.com/?d=YVP5TPPY	2002/Sep/16	16.6	\N	\N	\N	TV Original	2010-08-23 10:29:22.442615-04
312	Festival Dolls Dyed in the Setting Sun (Part 1)	\N	gb	XviD	E1DD441F	\N	http://www.megaupload.com/?d=Y8F6QOPV	2003/Mar/03	16.1	\N	\N	\N	Vol 38 File 2-4	2010-08-23 10:29:36.372685-04
294	The Smash of Determination and Love (Part 1)	Kienai	173mb	H264	B6666771	Join #Kienai and type /msg Sonoko-Chan XDCC send 11	http://www.megaupload.com/?d=4YL5TP7T	2002/Sep/09	16.7	\N	\N	\N	TV Original	2010-08-23 10:29:21.391457-04
163	The Secret of the Moon, the Star, and the Sun (Part 1)	KnKF	175mb	\N	E5CF4EC7	\N	http://www.megaupload.com/?d=I516EREH	1999/Oct/11	17.3	\N	\N	\N	Vol 12 File 1-3	2010-08-23 10:27:06.608539-04
30016	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
214	The Mysterious Retro Room Case	AZFS	179mb	XviD	9AFE4475	Join #AZFS and type /msg AZFS|Kudo XDCC send 4	http://www.megaupload.com/?d=T3PYTNE8	2000/Nov/20	19.4	\N	\N	\N	TV Original	2010-08-23 10:27:49.338916-04
229	The Murderous Pottery Class (Part 2)	AZFS	173mb	XviD	74838EC0	Join #AZFS and type /msg AZFS|Kudo XDCC send 7	http://www.megaupload.com/?d=LZFUEP0N	2001/Mar/19	19.8	\N	\N	\N	Vol 30 File 10 - Vol 31 File 1	2010-08-23 10:28:06.66601-04
503	Clash of Red and Black: Ready to Die	DCTP	172mb | 233mb	\N	073FCD49 | 4BE776AF	\N	http://www.megaupload.com/?d=HHSBRKIR | 720: http://www.megaupload.com/?d=8SP77YCR	2008/May/12	9.1	\N	\N	\N	Vol 58 File 8 - Vol 59 File 1	2010-08-23 10:32:40.085558-04
68	The Night Baron Murder Case (Part 1: The Case)	AConan	172mb	\N	4588DDEC	\N	http://www.megaupload.com/?d=VFU0TD4B	1997/Jul/21	17.5	\N	\N	\N	Vol 8 File 2-7	2010-08-23 10:25:31.454761-04
188	The Desperate Revival: The Cavern of the Detective Boys	KnKF	175mb	\N	34FF1167	\N	http://www.megaupload.com/?d=0N2FRSUU	2000/May/01	19.8	\N	\N	\N	Vol 25 File 9 - Vol 26 File 7	2010-08-23 10:27:28.951176-04
298	Courtroom Confrontation 2: Kisaki VS Kujo Reiko (Part 2)	\N	\N	\N	\N	\N	http://www.megaupload.com/?d=9PK4NA72	2002/Oct/28	16.8	\N	\N	\N	TV Original	2010-05-17 17:13:51.278958-04
220	The Client Full of Lies (Part 1)	DCTP	344mb	\N	0AC8F430	\N	http://www.megaupload.com/?d=17C0PMYZ	2001/Jan/15	17.5	\N	\N	\N	Vol 28 File 3-5	2010-08-23 10:27:54.358594-04
378	Momotarou Mystery Solving Tour (Part 2)	DCTP	173mb	\N	0DE1791E	\N	http://www.megaupload.com/?d=4C386X4Z	2004/Nov/15	15	\N	\N	\N	TV Original	2010-08-23 10:30:31.627242-04
337	Hidden Circumstances of the Falling Incident	DCTP	172mb	XviD	32B174AE	\N	http://www.megaupload.com/?d=Z3Q3MPM8	2003/Oct/13	14.7	\N	\N	\N	TV Original	2010-08-23 10:29:57.795248-04
389	Kogoro Gets Drunk in Satsuma (Part 2)	Kienai	172mb	h264	86CE4F04	Join #Kienai and type /msg Sonoko-Chan XDCC send 18	http://www.megaupload.com/?d=Q9IH48IG	2005/Feb/21	12.9	\N	\N	\N	TV Original	2010-08-23 10:30:52.047366-04
413	The Half Completed Crime Mystery	DCTP	171mb	XviD	AC7BE100	\N	http://www.megaupload.com/?d=JWRJC78B	2005/Sep/05	10.1	\N	v2 fixed a styling error.	\N	TV Original	2010-08-23 10:31:13.594509-04
170	The Unseen Angle of the Darkness (Part 1)	KnKF	175mb	\N	91509B90	\N	http://www.megaupload.com/?d=ACBEDL2X	1999/Nov/29	17.5	\N	\N	\N	Vol 24 File 3-6	2010-08-23 10:27:11.906549-04
72	The Triplet's Country Home Murder Case	AConan	172mb	\N	40AF4685	\N	http://www.megaupload.com/?d=1O21EXND	1997/Aug/18	18.3	\N	\N	\N	Vol 13 File 2-4	2010-08-23 10:25:34.461954-04
20034	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30020	Live Action 01	KnKF	698mb	\N	6C34F857	\N	\N	\N	\N	Shinichi and Ran's class decides to take a trip on a boat, but Shinichi decides against going. He quickly changes his mind when he finds a letter of challenge stating that one kid will be kidnapped during the trip. He is eager to accept the challenge, but when the one that vanishes is Sonoko, Shinichi must use all of his reasoning abilities to find Sonoko and protect the other students. When at first he doesn't succeed, he recieves a call from the kidnapper stating he will now abduct another student at exactly twelve noon. Shinichi must now recover from his previous failure and hope to prevent a tragedy.&nbsp; &nbsp; &nbsp;Shinichi receives a letter from an unknown challenging him to stop a kidnapping from happening.	\N	\N	\N	2010-05-17 17:13:51.278958-04
77	A Distinguished Family's Consecutive Accidental Death Case (Part 1)	AConan	172mb	\N	22ED7172	\N	http://www.megaupload.com/?d=0DTY3XL0	1997/Oct/20	19.1	\N	\N	\N	Vol 15 File 10 - Vol 16 File 3	2010-08-23 10:25:37.960869-04
360	A Mysterious Spring Beetle	DCTP	171mb	XviD	5039E9FC	\N	http://www.megaupload.com/?d=NCE57PRK	2004/May/17	13.7	\N	\N	\N	TV Original	2010-08-23 10:30:17.990316-04
303	The Victim who Came Back	Kienai	172mb	h264	5C0AA1F1	Join #Kienai and type /msg Sonoko-Chan XDCC send 13	http://www.megaupload.com/?d=HEL3EPLS	2002/Dec/09	16.4	\N	\N	\N	TV Original	2010-08-23 10:29:28.863012-04
532	The Scar of First Love	DCTP	172mb	\N	B79B926D	\N	http://www.megaupload.com/?d=CCOM4YPE	2009/May/02	5.9	\N	\N	\N	Vol 64 File 4-6	2010-08-23 10:33:01.745971-04
16	The Antique Collector Murder Case	AConan	186mb	\N	29EC415E	\N	http://www.megaupload.com/?d=DEER0100	1996/May/20	12.3	\N	\N	\N	Vol 6 File 2-5	2010-08-23 10:24:49.624935-04
55	The Train Trick Murder Case	AConan	172mb	\N	8BE6D54F	\N	http://www.megaupload.com/?d=XZJZJP49	1997/Apr/21	18.4	\N	\N	\N	TV Original	2010-08-23 10:25:21.732077-04
233	The Evidence That Didn't Disappear (Part 1)	AZFS	174mb	XviD	5E808474	Join #AZFS and type /msg AZFS|Kudo XDCC send 9	http://www.megaupload.com/?d=C2KX9FT9	2001/May/14	18.5	\N	\N	\N	Vol 29 File 6-8	2010-08-23 10:28:09.593818-04
368	The Candy House the Witch Lives In	KnKF	175mb	\N	8AAEED40	\N	http://www.megaupload.com/?d=E4E0TFYF	2004/Jul/26	12.7	\N	\N	\N	TV Original	2010-08-23 10:30:24.441644-04
82	The Kidnapping of a Popular Artist Case (Part 2)	AConan	172mb	\N	A271CDFA	\N	http://www.megaupload.com/?d=461ZIWFW	1997/Nov/24	20.6	\N	\N	\N	Vol 15 File 4-6	2010-08-23 10:25:43.018408-04
175	The Man Who Was Killed Four Times	KnKF	175mb	\N	2661B1FC	\N	http://www.megaupload.com/?d=JTKCG6FW	2000/Jan/10	18	\N	\N	\N	TV Original	2010-08-23 10:27:15.748465-04
35	The Mountain Villa Bandaged Man Murder Case (Part 2)	AConan	152mb	\N	0BE30950	\N	http://www.megaupload.com/?d=71FIWAHD	1996/Oct/28	18.7	\N	\N	\N	Vol 5 File 1-5	2010-08-23 10:25:04.744327-04
11	Moonlight Sonata Murder Case (1 Hour Special)	AConan	402mb	\N	FE40A4FC	\N	http://www.megaupload.com/?d=U8BANZYF	1996/Apr/08	12.9	\N	\N	\N	Vol 7 File 2-7	2010-08-23 10:24:45.805778-04
30010	Lupin III vs. Detective Conan TV Special	 | Frostii-DCTP	 | 1.4gb	\N	 | 07CEE0B0	\N	http://www.megaupload.com/?d=HBCFJ01X (soft-subbed MKV - 720p)	\N	\N	The story unfolds in the Kingdom of Vespania, where the Kingdom mourns the loss of the Queen and Prince in a hunting accident. Princess Mira, the Queen�s eldest daughter is next in line to the throne, and this is how Conan and his best friend, Ran, gets involved in the events to come.\r\n\r\nA spitting image of the princess, Ran gets taken away to the Kingdom to take the place of Princess Mira. Meanwhile, Lupin has his sights set on stealing the Kingdom�s Queen�s Crown, and with the turn of events, with their individual missions, Conan and Lupin end up meeting in the Kingdom. What will happen in the clash between the century�s greatest thief and the greatest detective?	In parts http://www.megaupload.com/?d=6HM922ZV http://www.megaupload.com/?d=DY4RB5IE	\N	\N	2010-08-23 10:37:13.5727-04
409	The Simultaneous Stage Advance and Kidnapping (Part 1)	\N	\N	04F0F911	\N	\N	http://www.megaupload.com/?d=8S33WPVE	2005/Aug/08	8.7	\N	http://www.anirena.com/viewtracker.php?action=download&id=38096	\N	TV Original	2010-05-17 17:13:51.278958-04
24	The Mysterious Woman With Amnesia Case	AConan	168mb	\N	03796898	\N	http://www.megaupload.com/?d=A2CTQRAN	1996/Jul/15	14.4	\N	\N	\N	TV Original	2010-08-23 10:24:56.311594-04
412	The Shinto Shrine Torii's Surprising Code (Part 2)	HnI	171mb	\N	9EC1C8C9	\N	http://www.megaupload.com/?d=87O2EQIP	2005/Aug/29	8.4	\N	\N	\N	Vol 48 File 1-3	2010-08-23 10:31:12.906374-04
277	English Teacher VS Great Western Detective (Part 1)	Kaizou	173mb	\N	A1BE301B	\N	http://www.megaupload.com/?d=ZA2HZBLW	2002/May/13	17.7	\N	\N	\N	Vol 34 File 2-4	2010-08-23 10:28:59.025367-04
306	The Unseen Suspect (Part 2)	DCTP	348mb	XviD	221A7174	\N	http://www.megaupload.com/?d=22ADVSIO	2003/Jan/20	16.2	\N	Part 1 is concatenated.	\N	Vol 37 File 2-4	2010-08-23 10:29:31.766421-04
4	The Coded Map of the City Case	AConan	210mb	XviD	EC789C22	\N	http://www.megaupload.com/?d=2U8LQP14	1996/Jan/29	12.2	\N	\N	\N	Vol 4 File 7-10	2010-08-23 10:24:40.001372-04
10044	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20036	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
431	The Metropolitan Police Detective Love Story 7 (Part 1)	DCTP	174mb	\N	7518AB85	\N	http://www.megaupload.com/?d=4YC25BCA	2006/Feb/20	10.4	\N	\N	\N	Vol 50 File 1-4	2010-08-23 10:31:26.257034-04
382	Which One's Reasoning Show? (Part 2)	DCTP	172mb	\N	C6E445F4	\N	http://www.megaupload.com/?d=D011VQZZ	2004/Dec/13	12.8	\N	\N	\N	Vol 43 File 6-9	2010-08-23 10:30:35.061032-04
145	The North Star #3 Express Leaving Ueno (Part 2)	Neg	173mb	\N	8C7A529C	\N	http://www.megaupload.com/?d=NE9OR00M	1999/May/24	19.4	\N	\N	\N	Vol 22 File 4-7	2010-08-23 10:26:52.951218-04
285	Chinatown Deja Vu in the Rain (Part 2)	Kaizou	226mb	\N	A055891F	\N	http://www.megaupload.com/?d=764GWVLQ	2002/Jul/08	17.5	\N	\N	\N	Vol 34 File 8-10	2010-08-23 10:29:09.66293-04
23	Luxury Liner Serial Murder Case (Part 2)	AConan	173mb	\N	3CF3292F	\N	http://www.megaupload.com/?d=16AZB480	1996/Jul/08	13.4	\N	\N	\N	Vol 3 File 1-6	2010-08-23 10:24:55.556178-04
30017	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30035	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30048	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
410	Simultaneous Stages of Advancing and Kidnapping (Part 2)	Kienai	172mb	H264	67F54DB3	\N	http://www.megaupload.com/?d=KJSSONKL	2005/Aug/15	10.3	\N	http://www.anirena.com/viewtracker.php?action=download&id=38163	\N	TV Original	2010-05-17 17:13:51.278958-04
157	The Metropolitan Police Detective Love Story 2 (Part 2)	Neg	172mb	\N	6F8AD6E7	\N	http://www.megaupload.com/?d=DRWKVWHY	1999/Aug/16	13.6	\N	\N	\N	Vol 23 File 10 - Vol 24 File 2	2010-08-23 10:27:01.902046-04
383	Miracle at Koshien Ball Park! The Defiants Face the Dark Demon (2 Hour Special)	Kaizou	699mb	\N	5B322B97	\N	http://www.megaupload.com/?d=JAF94ZUV	2004/Dec/20	12.5	\N	\N	\N	Vol 43 File 10 - Vol 44 File 3	2010-08-23 10:30:35.746189-04
297	Courtroom Confrontation 2: Kisaki VS Kujo Reiko (Part 1)	\N	\N	\N	\N	\N	http://www.megaupload.com/?d=9PK4NA72	2002/Oct/21	16.9	\N	\N	\N	TV Original	2010-05-17 17:13:51.278958-04
10000	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20039	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
388	Kogoro Gets Drunk in Satsuma (Part 1)	Kienai	172mb	h264	2C7D528C	Join #Kienai and type /msg Sonoko-Chan XDCC send 17	http://www.megaupload.com/?d=V27W6SDP	2005/Feb/14	12.3	\N	\N	\N	TV Original	2010-08-23 10:30:51.209808-04
235	The Locked Wine Cellar	AZFS-BG	171mb	XviD	B1F1C7CC	Join #AZFS and type /msg AZFS|Kudo XDCC send 11	http://www.megaupload.com/?d=NQPD5QFW	2001/May/28	17.8	\N	\N	\N	TV Original	2010-08-23 10:28:11.140629-04
414	The Detective Boys' Bluebird Chase	DCTP	171mb	XviD	0E4DE4C1	\N	http://www.megaupload.com/?d=QVJFUKOI	2005/Sep/12	12.6	\N	\N	\N	TV Original	2010-08-23 10:31:14.299243-04
40	The Wealthy Daughter Murder Case (Part 2)	AConan	141mb	\N	FE7BD853	\N	http://www.megaupload.com/?d=769ISQEQ	1996/Dec/02	17.1	\N	\N	\N	Vol 9 File 7 - Vol 10 File 1	2010-08-23 10:25:09.21736-04
425	Black Impact! The Instant That the Black Organization Reaches (2.5 Hour Special)	KnKF	700mb	\N	B455E63E	\N	http://www.megaupload.com/?d=RVMFTTMC	2006/Jan/09	13.1	\N	\N	\N	Vol 48 File 9 - Vol 49 File 4	2010-08-23 10:31:22.053492-04
219	The Gathered Detectives! Shinichi VS Kaito Kid (2 Hour Special)	Kaizou	691mb	\N	50034C1F	\N	http://www.megaupload.com/?d=M7D8GP5U	2001/Jan/08	17.6	\N	\N	\N	Vol 30 File 4-7	2010-08-23 10:27:53.658741-04
10019	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20011	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20043	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
441	The Final "Ahh"	Kienai	172mb	\N	EEF44907	Join #Kienai and type /msg Sonoko-Chan XDCC send 28	http://www.megaupload.com/?d=P3F5Q0CP	2006/Jun/05	8.7	\N	\N	\N	TV Original	2010-08-23 10:31:33.220709-04
374	A Code for Star and Tabacco (Part 1)	KnKF	171mb	\N	BA352DAD	\N	http://www.megaupload.com/?d=LN70XOW6	2004/Oct/18	12.6	\N	\N	\N	Vol 45 File 9 - Vol 46 File 1	2010-08-23 10:30:28.761689-04
73	The Detective Boys' Disaster Case	AConan	172mb	\N	147B553B	\N	http://www.megaupload.com/?d=GZ9H0KCI	1997/Aug/25	18.8	\N	\N	\N	TV Original	2010-08-23 10:25:35.185496-04
489	Courtroom Confrontation III: Prosecutor as Eyewitness (1 Hour Special)	DCTP-AZFS	347mb	XviD	6C29281A	Join #AZFS and type /msg AZFS|Kudo XDCC send 30	http://www.megaupload.com/?d=KYNUDIXS	2007/Nov/26	10.1	\N	\N	\N	TV Original	2010-08-23 10:32:23.229464-04
104	The Mysterious Robbers and Mansion Case (Part 1)	AConan	172mb	\N	9FA26CAC	\N	http://www.megaupload.com/?d=T5JFICSF	1998/Jun/08	17.3	\N	\N	\N	Vol 17 File 7-9	2010-08-23 10:26:00.410439-04
228	The Murderous Pottery Class (Part 1)	AZFS	174mb	XviD	BEF9F901	Join #AZFS and type /msg AZFS|Kudo XDCC send 6	http://www.megaupload.com/?d=VCKK5D9I	2001/Mar/12	19.4	\N	\N	\N	Vol 30 File 10 - Vol 31 File 1	2010-08-23 10:28:05.970825-04
10022	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
292	Solitary Island of the Princess and the Dragon King's Palace (Part 2)	\N	gb	\N	904A9038	\N	http://www.megaupload.com/?d=1Z93N0E7	2002/Aug/26	18	\N	\N	\N	Vol 35 File 11 - Vol 36 File 4	2010-08-23 10:29:19.911114-04
502	Clash of Red and Black: Innocence	DCTP	172mb | 233mb	\N	15981E4B | 6428FF22	\N	http://www.megaupload.com/?d=2T3F110W | 720: http://www.megaupload.com/?d=H0YR7AQC	2008/Apr/28	9.6	\N	\N	\N	Vol 58 File 8 - Vol 59 File 1	2010-08-23 10:32:38.830628-04
314	Broken Fence of the Observatory	AConan	169mb	\N	2944FE1B	\N	http://www.megaupload.com/?d=W770J2DW	2003/Mar/17	14.6	\N	\N	\N	TV Original	2010-08-23 10:29:37.988252-04
239	The Osaka 3 "K" Case (Part 2)	DCTP	171mb	\N	16CCB3DC	\N	http://www.megaupload.com/?d=O7XDYHOR	2001/Jun/25	16.3	\N	\N	\N	Vol 29 File 9-11	2010-08-23 10:28:19.512173-04
10023	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20015	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
550	The Revolving Sushi Mystery (Part 2)	DCTP	170mb | 300mb	XViD	19FEBA38 | 5E698F8A	\N	http://www.megaupload.com/?d=DADCHFFU | 720: http://www.megaupload.com/?d=I1CW5NF1	2009/Oct/10	\N	\N	\N	\N	Vol 63 File 3-5	2010-08-23 10:33:15.664684-04
535	An Old Scar & The Detective's Spirit	DCTP	169mb	XviD	BC055C0C	\N	http://www.megaupload.com/?d=UYTUK7KM	2009/May/23	7.2	\N	\N	\N	Vol 64 File 7-10	2010-08-23 10:33:04.087782-04
440	The Car Stunt's Utmost Limit	DCTP	171mb	XviD	E0DC3FB7	\N	http://www.megaupload.com/?d=LUVG7CPR	2006/May/29	11.6	\N	v2 available. it fixes a name.	\N	TV Original	2010-08-23 10:31:32.54141-04
376	Time Limit is 3 o'clock	KnKF	172mb	\N	F032D0F7	\N	http://www.megaupload.com/?d=U06LB4Q8	2004/Nov/01	12.4	\N	\N	\N	TV Original	2010-08-23 10:30:30.174732-04
133	The Magic Lovers' Murder Case (Suspicion Part)	Puto	250MB	H264-AC3	781c7b1a	\N	http://www.megaupload.com/?d=MNCPBWJ5	1999/Feb/01	21.2	\N	\N	\N	Vol 20 File 2-6	2011-01-16 22:04:32.708846-05
365	Synchronicity Case (Part 2)	KnKF	175mb	\N	6C9C1C1F	\N	http://www.megaupload.com/?d=6F1FEFEK	2004/Jun/21	14	\N	\N	\N	TV Original	2010-08-23 10:30:21.623618-04
185	The Murdered Famous Detective (Part 1)	KnKF	175mb	\N	BF7F4950	\N	http://www.megaupload.com/?d=APMK65WD	2000/Apr/10	16.2	\N	\N	\N	TV Original	2010-08-23 10:27:26.674708-04
10035	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20027	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10015	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10024	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
463	Shadow of Black Organization: Strange Illumination	Kaizou-Neg	172mb | 233mb	\N	2F8BAA4D | 6143EE64	\N	http://www.megaupload.com/?d=70PV4E4V | 720: http://www.megaupload.com/?d=M68UDJCA	2007/Feb/05	10.3	\N	\N	\N	Vol 53 File 7 - Vol 54 File 2	2010-08-23 10:31:48.941122-04
62	A Ghost Ship Murder Case (Part 2)	AConan	172mb	\N	217D9718	\N	http://www.megaupload.com/?d=Z04GYJNC	1997/Jun/09	20.8	\N	\N	\N	TV Original	2010-08-23 10:25:26.784561-04
129	A University Professor and the Girl Who Came from the Black Organization (2 Hour Special)	KnKF	700mb	\N	53E92AEB	\N	http://www.megaupload.com/?d=XS6N9HCZ	1998/Dec/21	20.9	\N	\N	\N	Vol 18 File 6 - Vol 19 File 1	2010-08-23 10:26:32.519389-04
548	Two Days With the Culprit (Second Day)	DCTP	172mb | 300mb	XViD	B5D16277 | F538C882	\N	http://www.megaupload.com/?d=9K98RZBD	2009/Sept/26	9.8	\N	\N	\N	TV Original	2010-08-23 10:33:14.083305-04
135	The Search for the Disappearing Weapon Case	Neg	172mb	\N	26D6AA65	\N	http://www.megaupload.com/?d=F5L55OUZ	1999/Feb/15	22.6	\N	\N	\N	TV Original	2010-08-23 10:26:37.241835-04
10025	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30008	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
178	Meeting with the Black Organization Again (Part 3)	KnKF	175mb	\N	E09CEDF2	\N	http://www.megaupload.com/?d=27U6YNG8	2000/Jan/31	19.5	\N	\N	\N	Vol 24 File 7-11	2010-08-23 10:27:18.435516-04
486	Beckoning Cat from Right to Left	DCTP-Kienai	170mb | 320mb	XviD | H264	848045F5 | A67266F2	\N	http://www.megaupload.com/?d=91UZUA9Q | 720: http://www.megaupload.com/?d=OU2V41QO	2007/Sep/03	7	\N	\N	\N	TV Original	2010-08-23 10:32:21.033861-04
98	The Famous Potter Murder Case (Part 1)	AConan	172mb	\N	A314FCC4	\N	http://www.megaupload.com/?d=YVKRHG6Q	1998/Apr/20	19.5	\N	\N	\N	Vol 16 File 10 - Vol 17 File 2	2010-08-23 10:25:55.912783-04
485	The Location of the Black Photograph (Part 2)	DCTP	172mb | 234mb	\N	281B2896 | CE4AD850	\N	http://www.megaupload.com/?d=7Z5OL2QW | 720: http://www.megaupload.com/?d=C5H0C66O	2007/Aug/27	7.7	\N	\N	\N	Vol 56 File 7-9	2010-08-23 10:32:20.189922-04
391	The Metropolitan Police Detective Love Story 6 (Part 2)	DCTP	173mb	\N	BF83A6D1	\N	http://www.megaupload.com/?d=EOFBO0RD	2005/Mar/07	11.5	\N	\N	\N	Vol 44 File 4-6	2010-08-23 10:30:53.401025-04
21	On Location, TV Drama Murder Case	AConan	178mb	\N	C89D6A73	\N	http://www.megaupload.com/?d=4HV5VUUA	1996/Jun/24	13.4	\N	\N	\N	TV Original	2010-08-23 10:24:53.887576-04
10026	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
324	Hattori Heiji's Desperate Situation! (Part 2)	Kaizou	173mb	\N	1FEAACB0	\N	http://www.megaupload.com/?d=V0AHJVVG	2003/Jun/16	15.6	\N	\N	\N	Vol 38 File 8-10	2010-08-23 10:29:46.309093-04
180	The Nocturne of the Red Murderous Intent (Part 1)	KnKF	175mb	\N	D6757907	\N	http://www.megaupload.com/?d=7BW3HXN5	2000/Feb/14	20.1	\N	\N	\N	TV Original	2010-08-23 10:27:19.831327-04
61	A Ghost Ship Murder Case (Part 1)	AConan	172mb	\N	26D24C4C	\N	http://www.megaupload.com/?d=5I5D0NP1	1997/Jun/02	19.1	\N	\N	\N	TV Original	2010-08-23 10:25:26.070571-04
452	The Phantom of the Theatre (1 Hour Special)	HnI	350mb	\N	B3A9066D	\N	http://www.megaupload.com/?d=L9PNYQ30	2006/Sep/11	9.1	\N	\N	\N	TV Original	2010-08-23 10:31:41.288997-04
541	The Day Mouri Kogorou Ceased Being A Detective (Part 2)	DCTP	172mb | 485mb	XviD	8850C4A3 | D270660B	\N	http://www.megaupload.com/?d=2PCI9Q8Y	2009/July/18	5	\N	\N	\N	TV Original	2010-08-23 10:33:09.037374-04
537	Kaitou Kid vs. The Strongest Safe (Part 1)	DCTP	172mb | 483mb | 232mb	XviD	2A3D3E5F | 8E9B8D83 | C3B16071	\N	http://www.megaupload.com/?d=A0TWZUWQ | 720: http://www.megaupload.com/?d=UXRGI2SG	2009/Jun/13	7.1	\N	\N	\N	Vol 64 File 11 - Vol 65 File 2	2010-08-23 10:33:06.023592-04
10027	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20019	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
508	Blind Spot in the Karaoke Box (Part 2)	DCTP	172mb | 485mb	\N	1547A040 | 48087D4B	\N	http://www.megaupload.com/?d=PX6LGO63	2008/Jul/07	7.9	\N	\N	\N	Vol 59 File 11 - Vol 60 File 2	2010-08-23 10:32:44.259069-04
120	The Honey Cocktail Murder Case	Puto-F	250MB	H264	0d6811d4	\N	http://www.megaupload.com/?d=AJ2A8G9Y	1998/Oct/19	18.2	\N	\N	on	TV Original	2011-01-16 22:02:30.61679-05
369	The Lucky Man's Suspense	KnKF	176mb	\N	9788B705	\N	http://www.megaupload.com/?d=55G7218D	2004/Aug/02	13.7	\N	\N	\N	TV Original	2010-08-23 10:30:25.180175-04
538	 Kaitou Kid vs. The Strongest Safe (Part 2)	DCTP	170mb | 483mb | 231mb	XviD	968A2C89 | 745F8158 | 9C6A24E7	\N	http://www.megaupload.com/?d=4IZ36V3X | 720: http://www.megaupload.com/?d=2CKHGW86	2009/Jun/20	6.8	\N	\N	\N	Vol 64 File 11 - Vol 65 File 2	2010-08-23 10:33:06.751432-04
422	Gingko-Colored First Love (Part 2)	Kienai	172mb	\N	0A6B5B43	Join #Kienai and type /msg Sonoko-Chan XDCC send 23	http://www.megaupload.com/?d=NTYKV5K6	2005/Nov/28	9	\N	\N	\N	Vol 40 File 7-9	2010-08-23 10:31:19.873505-04
10002	The Fourteenth Target	AConan	700mb	XviD	4CF3FD87	\N	\N	1998/Apr/18	\N	Ran has a nightmare about her mother dying, but starts to remember what really happened involving herself, her mother, and her father from 10 years ago. Meanwhile, people are being injured by a certain culprit. Conan soon realizes that these people all have some kind of relation to Mouri, and that they all had a number in their name corresponding to a playing card and were being targeted in order, from the king down to the ace. The police soon decide that a murderous card dealer who just got out of jail and wanted revenge on Mouri is behind it. However, as injuries become murders and it is a race to predict who the next victims will be, Conan soon finds that the truth is something completely different, while learning about the incident from 10 years ago in the process.	\N	\N	\N	2010-05-17 17:13:51.278958-04
405	The Man Who Went to Call For an Ambulance	\N	gb	\N	546F4B5D	\N	http://www.megaupload.com/?d=Y0OD3QLD	2005/Jul/04	13	\N	\N	\N	TV Original	2010-08-23 10:31:04.673079-04
429	Two People Who Can't Return Yet (Part 1)	Kaizou	176mb	\N	1BB66556	\N	http://www.megaupload.com/?d=QMHOCN5K	2006/Feb/06	11.9	\N	\N	\N	Vol 49 File 8-10	2010-08-23 10:31:24.910173-04
241	The Bullet Train Transport Case (Part 2)	DCTP	172mb	\N	7EA18808	\N	http://www.megaupload.com/?d=2QLIG2X7	2001/Jul/09	16.7	\N	\N	\N	Vol 30 File 1-3	2010-08-23 10:28:20.998913-04
361	Teitan High School's Ghost Story (Part 1)	Shi-Fa	108mb	\N	BF4F4D29	\N	http://www.megaupload.com/?d=T02ZTT67	2004/May/24	14	\N	\N	\N	Vol 44 File 11 - Vol 45 File 2	2010-08-23 10:30:18.714986-04
555	The Stork Mystery Tour (Haruna's Pursuit Chapter)	DCTP	173mb	XviD	D6A054E7	\N	http://www.megaupload.com/?d=MBOLSCC3	2009/Nov/14	\N	\N	\N	\N	TV Original	2010-08-23 10:33:20.954214-04
201	The Tenth Passenger (Part 1)	KnKF	171mb	\N	CBD8A729	\N	http://www.megaupload.com/?d=TIWAA3M0	2000/Jul/31	19.2	\N	\N	\N	TV Original	2010-08-23 10:27:38.689634-04
46	The Snowy Mountain Villa Murder Case	AConan	168mb	\N	163C4C11	\N	http://www.megaupload.com/?d=3R3K2L1X	1997/Feb/03	18	\N	\N	\N	Vol 10 File 9 - Vol 11 File 1	2010-08-23 10:25:13.875096-04
10010	The Private Eyes' Requiem	KnKF	700mb | 700mb	XviD	AD7851FC | 7AA97A53	\N	\N	2006/Apr/15	\N	Some unknown man who knows Conan is in fact Kudo Shinichi lures Conan and the others (Mori, Ran, Tanteidan...) to Yokohama. This man takes Ran and the Tanteidan as hostages. He gives Conan a 12 hour time limit in which to solve a case. If it is not solved within that time, bombs attached to the hostages will explode.... To thwart this villain Conan has help from Heiji, Kaitou Kid, Hakuba.	\N	\N	\N	2010-05-17 17:13:51.278958-04
512	The Broken Horoscope	DCTP	172mb | 485mb	\N	23D57BA7 | 1699D57C	\N	http://www.megaupload.com/?d=MSE2AAMR	2008/Aug/11	7.7	\N	\N	\N	TV Original	2010-08-23 10:32:47.31206-04
20006	Follow the Vanished Diamond! Conan and Heiji vs Kid!	Kaizou	171mb	XviD	09673073	\N	\N	\N	\N	After receiving a strange (and unintentional) blackmail call, Conan elicits the help of Heiji Hattori to track down its intended recipient. However, once they find the victim they discover that the perpetrator has stolen his expensive and valuable jewel. a jewel that the mischievous Kaitou Kid desires for himself! Conan and Heiji must once more use their skills of deduction to unveil the identity of the villain and save the valuable prize.\r\n	\N	\N	\N	2010-05-17 17:13:51.278958-04
31	The TV Station Murder Case	AConan	170mb	\N	B8B3F6E3	\N	http://www.megaupload.com/?d=CG4OL4CL	1996/Sep/02	13.2	\N	\N	\N	Vol 11 File 2-4	2010-08-23 10:25:01.628958-04
309	Contact with the Black Organization (Part 1: The Negotiation)	Kaizou	226mb	\N	89764AC0	\N	http://www.megaupload.com/?d=C4Z8X2JO	2003/Feb/10	13.4	\N	\N	\N	Vol 37 File 8 - Vol 38 File 1	2010-08-23 10:29:33.942522-04
10008	Magician of the Silver Sky	KnKF	700mb | 700mb	XviD	4CD25276 | 7A0E0189	\N	\N	2004/Apr/17	\N	Kaito Kid strikes again in this new annual installment of the Detective Conan movies. An actress asked for Mouri Kogoro to protect a precious jewel of hers which Kid has vowed to steal. On the day of the theft, Kaito Kid dressed up as Shinichi and matched wits with Conan, and fled in the end. To thank them, the actress invited Kogoro and family and friends to Sapporo, but a bigger scheme, and a great emergency is just about to unravel high above the clouds in the plane that they're taking...	\N	\N	\N	2010-05-17 17:13:51.278958-04
10011	Jolly Roger of the Deep Azure	Kienai	699mb	h264	DDBEA582	\N	\N	2007/Apr/27	\N	The Isle of Koumi, a beautiful island in the Pacific Ocean. On the island, people pass an old legend down from generation to generation that there was the Seabed Palace, an ancient ruin at the bottom of the sea, where the treasure of 2 female pirates, Anne Bonnie & Mary Reed (who really existed 300 years ago), was left. When Conan and his friends visit Koumi Island while on vacation, they meet some treasure hunters. There was something suspicious about them.	\N	\N	\N	2010-05-17 17:13:51.278958-04
507	Blind Spot in the Karaoke Box (Part 1)	DCTP	172mb | 485mb	\N	E5B22EB5 | 3E5A0B8D	\N	http://www.megaupload.com/?d=QZLCKN3R	2008/Jun/30	6.3	\N	\N	\N	Vol 59 File 11 - Vol 60 File 2	2010-08-23 10:32:43.398706-04
144	The North Star #3 Express Leaving Ueno (Part 1)	Neg	173mb	\N	86A82794	\N	http://www.megaupload.com/?d=NUVF8BY1	1999/May/10	18.9	\N	\N	\N	Vol 22 File 4-7	2010-08-23 10:26:52.214754-04
286	Kudo Shinichi's NY Case (Part 1)	Kaizou	225mb	\N	28AA5843	\N	http://www.megaupload.com/?d=TRM8US8K	2002/Jul/15	17.3	\N	\N	\N	Vol 34 File 11 - Vol 35 File 4	2010-08-23 10:29:10.3996-04
381	Which One's Reasoning Show? (Part 1)	DCTP	172mb	\N	8D3B781E	\N	http://www.megaupload.com/?d=RKJBYG90	2004/Dec/06	13.5	\N	\N	\N	Vol 43 File 6-9	2010-08-23 10:30:34.361342-04
205	The Metropolitan Police Detective Love Story 3 (Part 1)	Neg	172mb	\N	EBBF9669	\N	http://www.megaupload.com/?d=0WP3RKB7	2000/Aug/28	19.2	\N	\N	\N	Vol 27 File 4-6	2010-08-23 10:27:42.504439-04
436	Information Gathered on the Detective Boys (Part 2)	Kaizou	163mb	\N	A4A855A6	\N	http://www.megaupload.com/?d=AVI9LQT3	2006/Apr/24	11.6	\N	\N	\N	Vol 50 File 5-7	2010-08-23 10:31:29.515528-04
386	The Dissonance of the Stradivarius (Part 2: Interlude)	DCTP	172mb	\N	D8475642	\N	http://www.megaupload.com/?d=8TFBEXT3	2005/Jan/31	12.4	\N	\N	\N	Vol 46 File 2-6	2010-08-23 10:30:46.058292-04
266	The Truth Behind Valentine's (Part 1: The Murder)	Kienai	172mb	\N	F844C1E5	Join #Kienai and type /msg Sonoko-Chan XDCC send 4	http://www.megaupload.com/?d=LKAQ7XU0	2002/Jan/28	20.4	\N	\N	\N	Vol 33 File 3-6	2010-08-23 10:28:49.402862-04
302	Parade of Malice and Saint (Part 2)	DCTP	343mb	\N	3722E962	\N	http://www.megaupload.com/?d=K8TYY6KD	2002/Dec/02	16.3	\N	\N	\N	Vol 36 File 5-7	2010-08-23 10:29:28.071463-04
338	Four Porsche's (Part 1)	DCTP	172mb	\N	3EBB47BB	\N	http://www.megaupload.com/?d=3AVB424T	2003/Oct/20	13.9	\N	\N	\N	Vol 41 File 7-9	2010-08-23 10:29:58.656771-04
464	Shadow of Black Organization: The Mysterious Big Reward	Kaizou-Neg	172mb | 172mb	\N	17DCD753 | 8BB2B378	\N	http://www.megaupload.com/?d=K0S8CEA3 | 720: http://www.megaupload.com/?d=JVQFM43H	2007/Feb/12	9.8	\N	\N	\N	Vol 53 File 7 - Vol 54 File 2	2010-08-23 10:31:49.623259-04
20005	The Target is Kogoro! The Detective Boys' Secret Investigation	DCTP	233mb	XviD	77091E38	\N	\N	\N	\N	Conan and the Detective Boys have a new homework assignment: interview a local shop keep about their line of work and plans for the future, and then give a presentation to the class. Of course, the gang chooses none other than Detective Mouri; and after a series of off topic questions and answers, they decide to shadow Mouri and see what his job is all about. Of course, things don't go exactly according to plans - and now, Conan and friends must help Mouri solve a mystery involving a strange kidnapping!\r\n	\N	\N	\N	2010-05-17 17:13:51.278958-04
10038	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20030	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30001	Magic File 1 (Movie 8)	\N	\N	\N	\N	\N	\N	\N	\N	\N	Nothing new from this, just episodes 132-134 and 196 on dvd	\N	\N	2010-05-17 17:13:51.278958-04
30024	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30041	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
136	The Old Castle Investigation Case (Part 1)	Neg	172mb	\N	4DE982A2	\N	http://www.megaupload.com/?d=LYHQL67Y	1999/Feb/22	22.8	\N	\N	\N	Vol 20 File 10 - Vol 21 File 3	2010-08-23 10:26:38.1106-04
42	A Karaoke Box Murder Case	AConan	160mb	\N	9B7C6A45	\N	http://www.megaupload.com/?d=115A76M3	1996/Dec/16	17.5	\N	\N	\N	Vol 5 File 6-9	2010-08-23 10:25:10.730419-04
171	The Unseen Angle of the Darkness (Part 2)	KnKF	162mb	\N	455414B1	\N	http://www.megaupload.com/?d=BFQCBNL6	1999/Dec/06	17.9	\N	\N	\N	Vol 24 File 3-6	2010-08-23 10:27:12.739141-04
423	The Detective Boys and the Four Cabbage Worm Brothers	Kienai	172mb	h264	87598584	Join #Kienai and type /msg Sonoko-Chan XDCC send 24	http://www.megaupload.com/?d=ZOMWA9WS	2005/Dec/05	10.8	\N	Original filename says CRC: [87598584] but that is wrong!	\N	TV Original	2010-08-23 10:31:20.589775-04
290	Mitsuhiko's Mystifying Forest (Part 2)	Kienai	172mb	h264	9AD69D2B	Join #Kienai and type /msg Sonoko-Chan XDCC send 10	http://www.megaupload.com/?d=PL2XAUYR	2002/Aug/12	14.7	\N	\N	\N	Vol 35 File 8-10	2010-08-23 10:29:18.377216-04
20032	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30042	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
586	The Kirin's Horn that Vanished into the Dark	DCTP	172mb | 320mb	XviD | H264	B925D8BC | A3813FD2	\N	http://www.megaupload.com/?d=NS23VQNP | 720: http://www.megaupload.com/?d=Z79WM5ZG	2010/Sept/04	\N	\N	\N	\N	Vol 68 File 5-8	2010-09-09 00:34:10.293402-04
1	Roller Coaster Murder Case	AConan	200mb	XviD	B58E4700	\N	http://www.megaupload.com/?d=HF35FVPU	1996/Jan/08	8.5	\N	\N	\N	Vol 1 File 1	2010-08-23 10:24:37.570575-04
151	The Truth of the Exploding Car Case (Part 2)	Neg	173mb	\N	CCDAFDE6	\N	http://www.megaupload.com/?d=6SVZC9KW	1999/Jul/05	19.5	\N	\N	\N	TV Original	2010-08-23 10:26:57.320357-04
190	The Desperate Revival: The Third Choice	KnKF	175mb	\N	901B0ABA	\N	http://www.megaupload.com/?d=607XFTXH	2000/May/15	19.8	\N	\N	\N	Vol 25 File 9 - Vol 26 File 7	2010-08-23 10:27:30.395862-04
10042	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
80	The Wandering Artist Murder Case	AConan	172mb	\N	05ED0052	\N	http://www.megaupload.com/?d=BZN32YQY	1997/Nov/10	19.2	\N	\N	\N	TV Original	2010-08-23 10:25:40.574214-04
30043	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30044	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
446	The Sealed Western-style Window (Part 1)	Kaizou	176mb	\N	1821E31B	\N	http://www.megaupload.com/?d=QMAX0Z8N	2006/Jul/24	8.4	\N	\N	\N	Vol 51 File 9-11	2010-08-23 10:31:37.077022-04
556	Intersection of Fear	DCTP	173mb	XviD	07496DE8	\N	http://www.megaupload.com/?d=U4PVA9B0	2009/Nov/21	\N	\N	\N	\N	TV Original	2010-08-23 10:33:24.683595-04
90	The Flower's Smell Murder Case	AConan	172mb	\N	05587938	\N	http://www.megaupload.com/?d=6Q7IGB0W	1998/Feb/09	22.8	\N	\N	\N	TV Original	2010-08-23 10:25:49.861745-04
102	Historical Play Performer Murder Case (Part 1)	AConan	172mb	\N	9E98E939	\N	http://www.megaupload.com/?d=ASYJT0ZG	1998/May/25	17.7	\N	\N	\N	Vol 17 File 10 - Vol 18 File 2	2010-08-23 10:25:58.777017-04
587	Kid VS The Four Divine Detective Boys	DCTP	172mb | 320mb	XviD | H264	14AF376D | 35049B33	\N	http://www.megaupload.com/?d=N6DIPF8U | 720: http://www.megaupload.com/?d=TAIXXE5G	2010/Sept/11	\N	\N	\N	\N	Vol 68 File 5-8	2010-09-15 05:42:09.759774-04
10043	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20035	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30007	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30019	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30045	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30046	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30047	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
588	The Trap of the Rooftop Farm	DCTP	172mb | 320mb	XviD | H264	8A1D3BE4 | D505AE2D	\N	http://www.megaupload.com/?d=1N30R2SP | 720: http://www.megaupload.com/?d=UZUK35U6	2010/Sept/18	\N	\N	\N	on	TV Original	2010-09-21 10:27:15.573554-04
263	The Osaka Double Mystery, The Swordsman and Toyotomi's Castle (2.5 Hour Special)	DCTP	700mb	\N	87FA434E	\N	http://www.megaupload.com/?d=NDQFYO5X	2002/Jan/07	19.5	\N	\N	\N	Vol 31 File 8 - Vol 32 File 4	2010-08-23 10:28:46.142163-04
549	The Revolving Sushi Mystery (Part 1)	DCTP	171mb | 300mb	XViD	550C3267 | 9DF41DBA	\N	http://www.megaupload.com/?d=J1QD82ER | 720: http://www.megaupload.com/?d=V4PPEY5T	2009/Oct/3	\N	\N	\N	\N	Vol 63 File 3-5	2010-08-23 10:33:14.814266-04
278	English Teacher VS Great Western Detective (Part 2)	Kaizou	172mb	\N	E7C23E5A	\N	http://www.megaupload.com/?d=Z5W1L2BQ	2002/May/20	16.9	\N	\N	\N	Vol 34 File 2-4	2010-08-23 10:28:59.780646-04
173	The Revival of the Dying Message (Part 2)	KnKF	175mb	\N	F419783C	\N	http://www.megaupload.com/?d=MD4CWP7D	1999/Dec/20	15.9	\N	\N	\N	Vol 25 File 1-3	2010-08-23 10:27:14.24748-04
54	A Game Company's Murder Case	AConan	172mb	\N	1DCAD67E	\N	http://www.megaupload.com/?d=RKK2IL0Q	1997/Apr/14	19.1	\N	\N	\N	Vol 12 File 4-6	2010-08-23 10:25:21.008578-04
10045	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20037	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10046	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20038	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10047	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30049	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30050	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
589	The Worst Birthday (Part 1)	DCTP	172mb | 320mb	XviD | H264	866BC59A | 11BAF210	\N	http://www.megaupload.com/?d=S4GTCZ1S | 720: http://www.megaupload.com/?d=88GCCNWB	2010/Sept/25	\N	\N	\N	\N	Vol 68 File 2-4	2010-09-26 09:24:15.96492-04
164	The Secret of the Moon, the Star, and the Sun (Part 2)	KnKF	172mb	\N	648025B9	\N	http://www.megaupload.com/?d=WMBNMUN8	1999/Oct/18	18.3	\N	\N	\N	Vol 12 File 1-3	2010-08-23 10:27:07.322456-04
10016	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10048	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20040	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10017	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10049	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20041	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10018	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10050	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20042	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
138	The Final Screening Murder Case (Part 1)	Neg	172mb	\N	33292D88	\N	http://www.megaupload.com/?d=ZJU85234	1999/Mar/22	21.1	\N	\N	\N	Vol 23 File 1-3	2010-08-23 10:26:39.548063-04
30022	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
590	The Worst Birthday (Part 2)	DCTP	172mb | 320mb	XviD | H264	B857DD38 | DEDC48BA	\N	http://www.megaupload.com/?d=8IS1SVZA | 720: http://www.megaupload.com/?d=V4O677AQ	2010/Oct/02	\N	\N	\N	\N	Vol 68 File 2-4	2010-10-03 10:13:09.785238-04
513	Coffee Aroma's Murderous Intent (Part 1)	DCTP	172mb | 485mb	\N	3E0355DB | 59B5C9E2	\N	http://www.megaupload.com/?d=68G42943	2008/Sep/01	9.3	\N	\N	\N	Vol 60 File 9-11	2010-08-23 10:32:48.053749-04
421	Gingko-Colored First Love (Part 1)	Kienai	172mb	\N	2EA3BA75	Join #Kienai and type /msg Sonoko-Chan XDCC send 22	http://www.megaupload.com/?d=UD0WSN0F	2005/Nov/21	9.6	\N	\N	\N	Vol 40 File 7-9	2010-08-23 10:31:19.150521-04
10020	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20012	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20044	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10021	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20013	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20045	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
279	The Hooligan's Labyrinth (Part 1)	MCT	gb	h264	677B2F46	\N	http://www.megaupload.com/?d=WGIAEJSY	2002/May/27	16.7	\N	\N	\N	Vol 34 File 5-7	2010-08-23 10:29:01.047679-04
434	The Great Dog Coeur's Triumph	DCTP	172mb	XviD	307F88CC	\N	http://www.megaupload.com/?d=OB5GTJAX	2006/Apr/10	7.7	\N	\N	on	TV Original	2010-11-26 23:25:50.495034-05
465	Shadow of Black Organization: A Pearl Shooting Star	Kaizou-Neg	172mb | 233mb	\N	43FDD0EC | EB2F7986	\N	http://www.megaupload.com/?d=NEWJ052N | 720: http://www.megaupload.com/?d=7NDWUQQQ	2007/Feb/19	9.5	\N	Alternate CRC for some copies 7A11DBF6 still plays fine	\N	Vol 53 File 7 - Vol 54 File 2	2010-08-23 10:31:50.386875-04
281	The Small Eye Witnesses	DCTP-AZFS	172MB	\N	\N	\N	http://www.megaupload.com/?d=HHOIRKH5	2002/Jun/10	13.2	\N	\N	on	TV Original	2011-02-23 20:04:56.65649-05
30025	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
236	The Nanki-Shirahama Mystery Tour (Part 1)	Puto	252MB	H264	563cf59e	\N	http://megaupload.com/?d=O6WPEIT0	2001/Jun/04	14.7	\N	\N	on	TV Original	2010-12-02 20:59:57.100154-05
20014	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20046	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30000	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
591	The House with the Aquarium	DCTP	172mb | 320mb	XviD | H264	B8FA4A05 | 18332712	\N	http://www.megaupload.com/?d=KOJUCWTX | 720: http://www.megaupload.com/?d=TXWIT1YM	2010/Oct/16	\N	\N	\N	on	TV Original	2010-10-25 21:01:25.112333-04
397	Spicy, Bitter & Sweet Soup	Kienai	172mb	H264	459CFEF5	Join #Kienai and type /msg Sonoko-Chan XDCC send 21	http://www.megaupload.com/?d=S7KUR12M	2005/May/09	12.9	\N	\N	on	TV Original	2010-09-02 23:42:39.642272-04
123	The Weather Lady Abduction Case	Puto-F	250MB	H264	f44f8771	\N	http://www.megaupload.com/?d=83IBPHUQ	1998/Nov/09	21.7	\N	\N	on	TV Original	2011-01-16 22:02:52.69668-05
248	The Alibi of the Soothing Forest	AZFS	173mb	\N	526F3797	Join #AZFS and type /msg AZFS|Kudo XDCC send 13	http://www.megaupload.com/?d=E8IZIEJ8	2001/Aug/27	19.3	\N	\N	\N	TV Original	2010-05-17 17:13:51.278958-04
199	Kogoro Mouri, Suspect (Part 1)	KnKF	172mb	\N	FB11F027	\N	http://www.megaupload.com/?d=4UAUCXN6	2000/Jul/17	18.7	\N	\N	\N	Vol 27 File 1-3	2010-08-23 10:27:37.068363-04
317	The Sullied Masked Hero (Part 2)	DCTP	348mb	XviD	2411959B	\N	http://www.megaupload.com/?d=ZW5IJM23	2003/Apr/28	15	\N	Part 1 is concatenated.	\N	Vol 38 File 5-7	2010-08-23 10:29:40.466268-04
384	The Target is Mouri Kogoro	\N	gb	\N	E343A077	\N	http://www.megaupload.com/?d=20Z852GR	2005/Jan/17	11.7	\N	\N	\N	TV Original	2010-08-23 10:30:39.53466-04
592	The Detective Memoir of Monkey and Rake (Part 1)	DCTP	172mb | 320mb	XviD | H264	DF204E87 | 1B3A3FF3	\N	http://www.megaupload.com/?d=Y5J0DSWM | 720: http://www.megaupload.com/?d=H0YVTGUK	2010/Oct/23	\N	\N	\N	\N	Vol 68 File 9-11	2010-10-30 19:21:27.614729-04
20017	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
193	The Desperate Revival: The Promised Place	KnKF	175mb	\N	EC557BF7	\N	http://www.megaupload.com/?d=K017X4DX	2000/Jun/05	20.1	\N	\N	\N	Vol 25 File 9 - Vol 26 File 7	2010-08-23 10:27:32.589343-04
593	The Detective Memoir of Monkey and Rake (Part 2)	DCTP	172mb | 319mb	XviD | H264	EC3CFEE0 | 37480AB1	\N	http://www.megaupload.com/?d=SDK6G7LX | 720: http://www.megaupload.com/?d=D2JEVJEP	2010/Oct/30	\N	\N	\N	\N	Vol 68 File 9-11	2010-11-07 12:57:17.827142-05
529	Might Over Mystery (Part 2)	DCTP	172mb	\N	0D7F6740	\N	http://www.megaupload.com/?d=S90Z0G6Z	2009/Mar/16	8.2	\N	\N	\N	Vol 62 File 2-4	2010-08-23 10:32:59.703998-04
435	Information Gathered on the Detective Boys (Part 1)	Kaizou	169mb	\N	87DACF98	\N	http://www.megaupload.com/?d=2E29Y4L0	2006/Apr/17	9.7	\N	\N	\N	Vol 50 File 5-7	2010-08-23 10:31:28.868431-04
268	The Truth Behind Valentine's (Part 3: The Solution)	Kienai	172mb	\N	87E57591	Join #Kienai and type /msg Sonoko-Chan XDCC send 6	http://www.megaupload.com/?d=DM8BFEBE	2002/Feb/11	21.2	\N	\N	\N	Vol 33 File 3-6	2010-08-23 10:28:51.312091-04
20018	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
172	The Revival of the Dying Message (Part 1)	KnKF	175mb	\N	45562C92	\N	http://www.megaupload.com/?d=5FMJCDDM	1999/Dec/13	17.3	\N	\N	\N	Vol 25 File 1-3	2010-08-23 10:27:13.494427-04
406	Conan and Heiji's Reasoning Magic (Part 1: The Trick)	\N	gb	\N	5D43038E	\N	http://www.megaupload.com/?d=3YWZLJC0	2005/Jul/11	11.4	\N	\N	\N	Vol 47 File 8-11	2010-08-23 10:31:08.468128-04
19	An Elevator Murder Case	AConan	172mb	\N	AFE1C198	\N	http://www.megaupload.com/?d=5781ADMM	1996/Jun/10	15.6	\N	\N	\N	TV Original	2010-08-23 10:24:52.149235-04
470	Kaitou Kid and the 4 Masterpieces (Part 2)	DCTP	172mb	\N	D1AB9D56	\N	http://www.megaupload.com/?d=YKXUQ9MY	2007/Apr/23	10.7	\N	\N	\N	Vol 53 File 1-4	2010-08-23 10:32:03.635488-04
283	The Mystery of the Water-Flowing Stone Garden (Part 2)	DCTP	350 MB	XviD	9BED9C34	\N	http://www.megaupload.com/?d=SG6A7VR7	2002/Jun/24	16.6	\N	\N	\N	TV Original	2011-01-26 22:23:37.253531-05
350	The Forgetten Cellphone (Part 1)	DCTP	350 MB	XviD	DA59B622	\N	http://www.megaupload.com/?d=HHSW0S73	2004/Feb/09	14.1	\N	\N	\N	Vol 43 File 3-5	2011-01-26 14:16:34.732163-05
122	The Sealed Bathroom Murder Case (Part 2)	Puto-F	250MB	H264	bef658fe	\N	http://www.megaupload.com/?d=BZV6U432	1998/Nov/02	20	\N	\N	\N	Vol 20 File 7-9	2011-01-16 22:02:42.574996-05
125	A Mysterious Sniper Murder Case (Part 2)	DCTP	350 MB	XVID-MP3	B6DE84CB	\N	http://www.megaupload.com/?d=W4QXGOPQ	1998/Nov/23	16.4	\N	\N	on	TV Original	2011-02-23 20:06:32.24932-05
321	The Vanished Kidnapper's Getaway Car (Part 1)	Kienai	??? mb	\N	5F8FBC7D	\N	http://www.megaupload.com/?d=V5V7UA0V	2003/May/26	14.2	\N	\N	\N	TV Original	2010-11-20 23:53:34.605722-05
168	The Monster of the Tottori Spider Mansion (Part 3: The Solution)	KnKF	175mb	\N	BADEED18	\N	http://www.megaupload.com/?d=DL4HBAJ0	1999/Nov/15	17.6	\N	\N	\N	Vol 25 File 4-8	2010-08-23 10:27:10.327343-04
600	The Dream the Kappa Saw (Part 1)	DCTP	172mb | 320mb	XviD | H264	27610801 | 46A0C679	\N	http://www.megaupload.com/?d=EVF3NGIV | 720: http://www.megaupload.com/?d=IPZ4CGJO	2010/Dec/18	\N	\N	\N	\N	Vol 69 File 1-3	2010-12-30 00:43:47.65251-05
30009	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30028	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
547	Two Days With the Culprit (First Day)	DCTP	172mb	XviD	FA8B445E	\N	http://www.megaupload.com/?d=TZT8SDOQ	2009/Sept/20	9	\N	\N	\N	TV Original	2010-08-23 10:33:13.387115-04
10028	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20020	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
358	The Metropolitan Police Detective Love Story 5 (Part 1)	Kaizou-Neg	172mb	\N	3B4E158E	\N	http://www.megaupload.com/?d=L3XU4EXP	2004/May/03	8.8	\N	\N	\N	Vol 40 File 1-3	2010-08-23 10:30:16.387708-04
571	Battle of the Haunted Warehouse's Treasure (Part 1)	DCTP	172mb | 323mb	XviD | H264	1DAAAC73 | 41B509C2	\N	http://www.megaupload.com/?d=VPFN1UDU | 720: http://www.megaupload.com/?d=ZNIE74IS	2010/May/01	\N	\N	\N	\N	Vol 66 File 5-7	2010-08-23 10:33:42.743658-04
10005	Countdown to Heaven	Neg	696mb | 675mb	XviD	24196F62 | 4F6780C7	\N	\N	2001/Apr/21	\N	As a pair of towers in Tokyo are being prepared for their grand opening, there is a series of murders of people connected to the towers. Conan suspects that the mysterious Syndicate may also be involved.	\N	\N	\N	2010-05-17 17:13:51.278958-04
392	The Mysterious Height Difference of 20cm	Kienai	172mb	h264	893B9CDB	Join #Kienai and type /msg Sonoko-Chan XDCC send 19	http://www.megaupload.com/?d=8VL65Z0P	2005/Mar/14	12.5	\N	\N	\N	TV Original	2010-08-23 10:30:54.082582-04
544	The Hand That Plays in Dissonance	DCTP	172mb	XviD	643472A1	\N	http://www.megaupload.com/?d=SX0CHM18	2009/August/8	6.5	\N	\N	\N	TV Original	2010-08-23 10:33:11.345701-04
393	A Kidnapping... So It Would Seem	Kienai	172mb	h264	5C2C0BB0	Join #Kienai and type /msg Sonoko-Chan XDCC send 20	http://www.megaupload.com/?d=24VDFUOY	2005/Mar/21	10.5	\N	\N	\N	TV Original	2010-08-23 10:30:54.733642-04
10006	The Phantom of Baker Street	DCTP	659mb | 660mb	XviD	706FEDE6 | 36124CA0	\N	\N	2002/Apr/20	\N	Conan was invited to a party with many other guests. They were there to test a new virtual game system, when a murder mystery occured. Now Conan must go into the game system to figure out who the murderer is with the help of the famous book character Sherlock Holmes. The lives of 49 kids testing the game are in his hands. 	\N	\N	\N	2010-05-17 17:13:51.278958-04
585	Timeless Sakura's Love	DCTP	172mb | 320mb	XviD | H264	665D7CB3 | B2886C56	\N	http://www.megaupload.com/?d=JT2NSD09 | 720: http://www.megaupload.com/?d=8QZC2011	2010/Aug/28	\N	\N	\N	\N	Vol 67 File 9 - Vol 68 File 1	2010-08-29 01:42:01.156379-04
379	Case of the Long Sleeved Kimono at the Hidden Hot Spring on a Snowy Night (Part 1)	\N	gb	\N	B468E254	\N	http://www.megaupload.com/?d=OSVKVVT9	2004/Nov/22	13.5	\N	\N	\N	TV Original	2010-08-23 10:30:32.937649-04
10030	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20022	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10031	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20023	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10037	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20029	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20025	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10040	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20026	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10041	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20033	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
506	Lawyer Kisaki Eri's Testimony (Part 2)	DCTP	172mb | 485mb	\N	D60A3ACF | 1CB1C2F7	\N	http://www.megaupload.com/?d=RPOO0CNJ	2008/Jun/23	8	\N	\N	\N	Vol 59 File 2-4	2010-08-23 10:32:42.657762-04
566	The Partner is Santa-san	DCTP	173mb | 320mb	XviD	C0D8FE4B | 08642737	\N	http://www.megaupload.com/?d=CPIKMKKH | 720: http://www.megaupload.com/?d=RVR4L1DC	2010/Feb/27	\N	\N	\N	\N	\N	2010-08-23 10:33:38.541751-04
520	Indictment by Red Wine	DCTP	172mb	\N	AFBFD601	\N	http://www.megaupload.com/?d=R3PB2FWA	2008/Dec/15	9.6	\N	\N	\N	TV Original	2010-08-23 10:32:52.972099-04
208	The Entrance to the Maze, The Anger of the Colossus (1 Hour Special)	Neg	333mb	\N	68DECD23	\N	http://www.megaupload.com/?d=VRB4FIYZ	2000/Oct/09	18.6	\N	\N	\N	TV Original	2010-08-23 10:27:44.781306-04
568	Inspector Shiratori, Memories of the Cherry Blossom (Part 1)	DCTP	171mb | 320mb	XviD | H264	350C0F45 | CE9865AA	\N	http://www.megaupload.com/?d=6MJD7FZ4 | 720: http://www.megaupload.com/?d=ZCYNHJ0W	2010/Mar/13	\N	\N	\N	\N	Vol 66 File 2-4	2010-08-23 10:33:40.405786-04
259	The Man from Chicago (Part 2)	Kaizou	172mb	\N	C1C1D598	\N	http://www.megaupload.com/?d=MWWS4N9L	2001/Nov/26	18.2	\N	\N	\N	Vol 32 File 8-10	2010-08-23 10:28:39.976508-04
444	Gathering Shellfish With a Sigh (Part 2)	\N	gb	\N	42F90667	\N	http://www.megaupload.com/?d=ZRNPUFHF	2006/Jul/03	8.3	\N	\N	\N	Vol 51 File 4-6	2010-08-23 10:31:35.535481-04
96	Caught Up with the Great Detective! Two Murder Cases (2 Hour Special)	AConan	688mb	\N	9E6BA9A8	\N	http://www.megaupload.com/?d=HI4ROCC2	1998/Mar/23	22.4	\N	\N	\N	Vol 14 File 1-8	2010-08-23 10:25:54.503037-04
211	The Water Palace of Five Colors (Part 2)	\N	gb	\N	EE5A4E84	\N	http://www.megaupload.com/?d=4ACS5KZX	2000/Oct/30	20.4	\N	\N	\N	TV Original	2010-08-23 10:27:47.136974-04
461	The Missing Page	DCTP	171mb	XviD	0C6ABCFB	\N	http://www.megaupload.com/?d=PG5BO49F	2007/Jan/22	9.8	\N	\N	\N	TV Original	2010-08-23 10:31:47.55153-04
533	The Scar That Invokes the Past	DCTP	172mb	\N	C4ED94A4	\N	http://www.megaupload.com/?d=L9HRE7AH	2009/May/09	6.5	\N	\N	\N	Vol 64 File 4-6	2010-08-23 10:33:02.487928-04
226	The Battle Game Trap (Part 1)	Kaizou	173mb	\N	D75485F6	\N	http://www.megaupload.com/?d=CHOWE04N	2001/Feb/26	20.9	\N	\N	\N	Vol 27 File 7-9	2010-08-23 10:28:04.428263-04
139	The Final Screening Murder Case (Part 2)	Neg	172mb	\N	38D8DC20	\N	http://www.megaupload.com/?d=V168JTM1	1999/Apr/05	21.3	\N	\N	\N	Vol 23 File 1-3	2010-08-23 10:26:40.259921-04
347	Find the Buttock's Mark (Part 2)	DCTP	172mb	XviD	A72565FA	\N	http://www.megaupload.com/?d=5V8M3BE1	2004/Jan/19	14.7	\N	\N	\N	Vol 42 File 11 - Vol 43 File 2	2010-08-23 10:30:06.773564-04
15	Missing Corpse Murder Case	AConan	209mb	\N	0CCDA374	\N	http://www.megaupload.com/?d=PZ931C09	1996/May/13	13.9	\N	\N	\N	Vol 6 File 6-8	2010-08-23 10:24:48.861185-04
534	A New Scar & The Whistling Man	DCTP	172mb	\N	0950AC6A	\N	http://www.megaupload.com/?d=ZBT7Q7BY	2009/May/16	8.7	\N	\N	\N	Vol 64 File 7-10	2010-08-23 10:33:03.233651-04
442	The Man Obstructed by the Steel Frame	DCTP	172mb	\N	565245AF	\N	http://www.megaupload.com/?d=BF5ODP41	2006/Jun/12	10.5	\N	Details will be filled out on sunday :P	\N	TV Original	2010-08-23 10:31:33.92819-04
560	The Mansion of Death and the Red Wall: The Late Koumei	DCTP	172mb | 320mb	XviD	E7770DA4 | 102B9F54	\N	http://www.megaupload.com/?d=0AU49AB9 | 720: http://www.megaupload.com/?d=R60UT0H1	2009/Dec/19	\N	\N	\N	\N	Vol 65 File 8 - Vol 66 File 1	2010-08-23 10:33:29.684854-04
203	The Black Wings of Icarus (Part 1)	DCTP	350mb	\N	57A97A97	\N	http://www.megaupload.com/?d=0JDG3QQU	2000/Aug/14	14.2	\N	\N	\N	TV Original	2010-08-23 10:27:40.302619-04
443	Gathering Shellfish With a Sigh (Part 1)	\N	gb	\N	66689482	\N	http://www.megaupload.com/?d=37HIVZBF	2006/Jun/26	9.5	\N	\N	\N	Vol 51 File 4-6	2010-08-23 10:31:34.616586-04
127	The Travelling Drama Troupe Murder Case (Part 2)	Puto	225mb	H264-AC3	3F6914E2	\N	http://www.megaupload.com/?d=SLP4GODE	1998/Dec/07	21.3	\N	\N	on	TV Original	2011-01-16 22:04:07.0029-05
559	The Mansion of Death and the Red Wall: Item in Hand	DCTP	172mb | 318mb	XviD	F52DCAA8 | E739F95E	\N	http://www.megaupload.com/?d=E3IP1GEE | 720: http://www.megaupload.com/?d=Z47OGM4X	2009/Dec/12	\N	\N	\N	\N	Vol 65 File 8 - Vol 66 File 1	2010-08-23 10:33:28.786154-04
179	The Case of the Coffee Shop Truck's Wild Entrance	KnKF	175mb	\N	D86DF1D1	\N	http://www.megaupload.com/?d=Z9YAGKZ5	2000/Feb/07	20.1	\N	\N	\N	TV Original	2010-08-23 10:27:19.125807-04
20007	A Challenge from Agasa	DCTP	233mb	XviD	82978EC7	\N	\N	\N	\N	Conan and the Detective Boys have been asked to meet with Professor Agasa at his house, but they soon receive a chilling note from the Professor himself: he has been kidnapped, and would like for Conan and company to find him! After following the clues laid out before them, the gang realizes that the Professor is simply playing a game with them that is, until the "game" turns dangerous and becomes real. Now, with the Professor actually kidnapped, Conan and the Detective Boys must race against the clock to find him before he gets hurt!	\N	\N	\N	2010-05-17 17:13:51.278958-04
30006	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20001	Conan vs Kid vs YAIBA	DCTP	233mb	XviD	FE02E18F	\N	\N	\N	\N	As a much-needed break from his sleuthing duties, Conan is looking forward to reading some newly-borrowed manga in the evening; however, once more, duty calls! Kogorou Mouri has been asked by the wife and daughter of the Raizou family to protect a Treasure Sword that has been "claimed" by the Kaitou Kid; however, Mine Raizou and his mother are certain they can handle the problem themselves. Conan, the Detective Boys and the rest of the gang must defend against Kaitou Kid's raid with the help of the young swordsman Yaiba!	\N	\N	\N	2010-05-17 17:13:51.278958-04
30012	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30013	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30014	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30023	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30026	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30030	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30031	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30032	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30033	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30038	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
10039	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20031	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
355	A Small Client (Part 2)	DCTP	172mb	\N	ABD88C9C	\N	http://www.megaupload.com/?d=R9XTOOPI	2004/Mar/15	13	\N	\N	\N	Vol 39 File 9-11	2010-08-23 10:30:13.67225-04
449	The Metropolitan Police Detective Love Story, The Fake Wedding (1 Hour Special)	Kaizou	382mb	\N	DE318ABE	\N	http://www.megaupload.com/?d=7WD8XVZ5	2006/Aug/21	11.8	\N	\N	\N	Vol 52 File 3-5	2010-08-23 10:31:39.238323-04
10004	Captured In Her Eyes	Neg	660mb | 654mb	XviD	40561A83 | EE49972B	\N	\N	2000/Apr/22	\N	A series of murders have occured with police officers as victims. One of the officers is shot right in front of Ran's eyes, and the shock of the incident causes Ran to lose her memory of everyone and everything around her. Now Conan must help Ran regain her lost memories, while also protecting her from the muderer, who is targeting Ran for witnessing the crime.	\N	\N	\N	2010-05-17 17:13:51.278958-04
558	The Mansion of Death and the Red Wall: Three Visits	DCTP	172mb | 320mb	XviD	FF51923A | 2D18F3C4	\N	http://www.megaupload.com/?d=6TMOYQC1 | 720: http://www.megaupload.com/?d=QLV5EP7B	2009/Dec/5	\N	\N	\N	\N	Vol 65 File 8 - Vol 66 File 1	2010-08-23 10:33:27.883096-04
329	A Friendship That Can't Be Bought (Part 1)	DCTP	172mb	\N	FDFCDF93	\N	http://www.megaupload.com/?d=B7ZZKGW9	2003/Jul/28	16	\N	\N	\N	Vol 39 File 6-8	2010-08-23 10:29:50.343788-04
310	Contact with the Black Organization (Part 2: The Pursuit)	Kaizou	215mb	\N	E28D0AC6	\N	http://www.megaupload.com/?d=2LYY4H8N	2003/Feb/17	15.8	\N	\N	\N	Vol 37 File 8 - Vol 38 File 1	2010-08-23 10:29:34.762381-04
5	The Shinkansen's Bomb Case	AConan	266mb	\N	4CD51933	\N	http://www.megaupload.com/?d=8F9LA0OV	1996/Feb/05	13.4	\N	\N	\N	Vol 4 File 4-6	2010-08-23 10:24:40.726029-04
484	The Location of the Black Photograph (Part 1)	DCTP	172mb | 233mb	\N	65F4D5BF | 7D8F5A3F	\N	http://www.megaupload.com/?d=07ZRZBTZ | 720: http://www.megaupload.com/?d=4IKFNWEW	2007/Aug/20	8.2	\N	\N	\N	Vol 56 File 7-9	2010-08-23 10:32:19.438239-04
554	The Stork Mystery Tour (Ran's Search Chapter)	DCTP	172mb	XviD	3399CA00	\N	http://www.megaupload.com/?d=V2HF5DLR	2009/Nov/07	\N	\N	\N	\N	TV Original	2010-08-23 10:33:19.244352-04
56	The Ojamanbou Murder Case	AConan	172mb	\N	2A8770F5	\N	http://www.megaupload.com/?d=6TGSX65B	1997/Apr/28	17.6	\N	\N	\N	TV Original	2010-08-23 10:25:22.461502-04
363	The City's Crows	KnKF	175mb	\N	40161E8E	\N	http://www.megaupload.com/?d=2HCRHPJH	2004/Jun/07	11.5	\N	\N	\N	TV Original	2010-08-23 10:30:20.124032-04
523	What She Truly Wants to Ask	DCTP	172mb	XviD	BE5261BC	\N	http://www.megaupload.com/?d=BGSQID1N	2009/Feb/02	7.9	\N	\N	\N	Vol 63 File 1-2	2010-08-23 10:32:55.15949-04
272	Secret Rushed Omission (Part 2)	Kaizou	173mb	\N	1F93FA50	\N	http://www.megaupload.com/?d=UFOCUL7J	2002/Mar/18	20.9	\N	\N	\N	Vol 33 File 10 - Vol 34 File 1	2010-08-23 10:28:55.027316-04
36	Monday Night 7:30 PM Murder Case	AConan	165mb	\N	A22525D9	\N	http://www.megaupload.com/?d=3ANT5DBF	1996/Nov/04	17	\N	\N	\N	TV Original	2010-08-23 10:25:05.562622-04
339	Four Porsche's (Part 2)	DCTP	173mb	\N	C8667C21	\N	http://www.megaupload.com/?d=J4L5N6UL	2003/Oct/27	11.7	\N	\N	\N	Vol 41 File 7-9	2010-08-23 10:29:59.420417-04
472	Adventure of Young Kudo Shinichi (Part 1)	DCTP	172mb	\N	A2CC7396	\N	http://www.megaupload.com/?d=5NXIUP31	2007/May/14	10.6	\N	\N	\N	Vol 55 File 6-9	2010-08-23 10:32:05.261501-04
177	Meeting with the Black Organization Again (Part 2)	KnKF	175mb	\N	CAC93F36	\N	http://www.megaupload.com/?d=2SGURN31	2000/Jan/24	18.8	\N	\N	\N	Vol 24 File 7-11	2010-08-23 10:27:17.446554-04
109	Detective Club Pursuit Case	Atavus	172mb	XviD	C64716F5	\N	http://www.megaupload.com/?d=7M2FEO1T	1998/Jul/13	16.7	\N	\N	\N	TV Original	2010-08-23 10:26:04.109786-04
460	The Big Battle of 1-B	HnI	145mb	\N	BFDC3111	\N	http://www.megaupload.com/?d=6F40D308	2007/Jan/15	9	\N	\N	\N	Vol 53 File 5-6	2010-08-23 10:31:46.815972-04
192	The Desperate Revival: Shinichi's Return	KnKF	175mb	\N	9E091060	\N	http://www.megaupload.com/?d=OG2G00TT	2000/May/29	20.3	\N	\N	\N	Vol 25 File 9 - Vol 26 File 7	2010-08-23 10:27:31.854321-04
255	The 14th Round of the Matsue Tamatsukuri Linked Verse Contest (Part 1)	AZFS	169mb	XviD	3A542034	Join #AZFS and type /msg AZFS|Kudo XDCC send 18	http://megaupload.com/?d=YNCUVED2	2001/Oct/29	19.3	\N	\N	on	TV Original	2010-11-21 01:56:40.010737-05
411	The Shinto Shrine Torii's Surprising Code (Part 1)	HnI	188mb	\N	61F7B59A	\N	http://www.megaupload.com/?d=B159AZP0	2005/Aug/22	11.2	\N	\N	\N	Vol 48 File 1-3	2010-08-23 10:31:12.07923-04
335	Secret of the Touto Film Developer Place (Part 1)	DCTP	171mb	\N	D55F9653	\N	http://www.megaupload.com/?d=6VQ7AFZV	2003/Sep/08	12.9	\N	\N	\N	Vol 41 File 4-6	2010-08-23 10:29:56.197409-04
501	Clash of Red and Black: Suspicion	DCTP	172mb | 233mb	\N	AB9C91B7 | B4208A26	\N	http://www.megaupload.com/?d=67G9L6MU | 720: http://www.megaupload.com/?d=MALMYO4B	2008/Apr/14	8.1	\N	\N	\N	Vol 58 File 8 - Vol 59 File 1	2010-08-23 10:32:32.875392-04
94	Snow Woman Legend Murder Case	AConan	173mb	\N	FA49D107	\N	http://www.megaupload.com/?d=5UM977CB	1998/Mar/09	21.2	\N	\N	\N	TV Original	2010-08-23 10:25:52.975729-04
249	The Idol Stars' Secret (Part 1)	AZFS	173mb	XviD	FAE201AF	Join #AZFS and type /msg AZFS|Kudo XDCC send 14	http://www.megaupload.com/?d=UBE2QQ49	2001/Sep/03	18.8	\N	\N	\N	Vol 32 File 5-7	2010-08-23 10:28:31.999107-04
494	Clash of Red and Black: Hades	DCTP	172mb | 233mb	\N	1E53B779 | 277D1F90	\N	http://www.megaupload.com/?d=NHCG9K3I | 720: http://www.megaupload.com/?d=XBGP00BU	2008/Feb/04	9.4	\N	\N	\N	Vol 56 File 10 - Vol 57 File 4	2010-08-23 10:32:27.40385-04
200	Kogoro Mouri, Suspect (Part 2)	KnKF	172mb	\N	3115FE41	\N	http://www.megaupload.com/?d=O8ZKXTYL	2000/Jul/24	19.2	\N	\N	\N	Vol 27 File 1-3	2010-08-23 10:27:37.821524-04
500	Clash of Red and Black: Testament	DCTP	171mb | 233mb	\N	2E12755B | 2EAF56C8	\N	http://www.megaupload.com/?d=UTDDIYKT | 720: http://www.megaupload.com/?d=GMDQXZZX	2008/Mar/17	9.5	\N	\N	\N	Vol 58 File 3 - Vol 58 File 7	2010-08-23 10:32:32.151445-04
10007	Crossroad in the Ancient Capital	KnKF	702mb | 699mb	XviD	A9EAE906 | E4AD16FE	\N	\N	2003/Apr/19	\N	Mouri Kogoro is called to a special case in the ancient capital of Kyoto. There, Conan meets Heiji and they team up once again to solve the case, recover the stolen Healing Buddha statue, and even discover the identity of Heiji's first love.	\N	\N	\N	2010-05-17 17:13:51.278958-04
30005	Magic File 5	DCTP	233MB/250MB	XviD/H264	4D464E9E/0AF4137E	\N	http://www.fileserve.com/file/rJmFFTD | h264: http://www.fileserve.com/file/J2sDe6A	\N	\N	\N	\N	\N	\N	2011-05-03 21:09:07.212098-04
20009	The Stranger in 10 Years...	DCTP	233mb | 233mb	XviD | h264	BED9C41A | FA9E88C7	\N	\N	\N	\N	Conan suffers fever one day and learns that Haibara just made an Antidote to APTX 4869. He then takes the antidote and becomes unconscious. Upon waking up, Conan found himself has returned to Shinichi! But everyone and the city look different?! What will be the destination of love with Ran?	\N	\N	\N	2010-05-17 17:13:51.278958-04
30039	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
30040	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
515	Kaitou Kid's Teleportation Magic (1 Hour Special)	DCTP	344mb	\N	672A3AC6	\N	http://www.megaupload.com/?d=XLS27GJI	2008/Oct/20	9.6	\N	\N	\N	Vol 61 File 1-4	2010-08-23 10:32:49.455744-04
137	The Old Castle Investigation Case (Part 2)	Neg	172mb	\N	B202E1CF	\N	http://www.megaupload.com/?d=6UPX0HZC	1999/Mar/08	23.4	\N	\N	\N	Vol 20 File 10 - Vol 21 File 3	2010-08-23 10:26:38.832458-04
63	Big Monster Gomera Murder Case	AConan	172mb	\N	DC7F5E02	\N	http://www.megaupload.com/?d=N0VQRGZX	1997/Jun/16	17.7	\N	\N	\N	Vol13 File 8-10	2010-08-23 10:25:27.523369-04
251	The Tragedy at the OK Corral	AZFS	172mb	XviD	39A9BBAD	Join #AZFS and type /msg AZFS|Kudo XDCC send 16	http://www.megaupload.com/?d=LL0NB8ZW	2001/Sep/17	19.4	\N	\N	\N	TV Original	2010-08-23 10:28:33.568419-04
105	The Mysterious Robbers and Mansion Case (Part 2)	AConan	172mb	\N	B3A96D15	\N	http://www.megaupload.com/?d=9VLYKCJT	1998/Jun/15	19.1	\N	\N	\N	Vol 17 File 7-10	2010-08-23 10:26:01.133272-04
3	An Idol's Locked Room Murder Case	AConan	177mb	XviD	2E9C1FF3	\N	http://www.megaupload.com/?d=AKZFY76V	1996/Jan/22	11.4	\N	\N	\N	Vol 1 File 6-9	2010-08-23 10:24:39.308956-04
142	The Night Before the Wedding Locked Room Murder Case (Part 2)	Neg	173mb	\N	F5EA9297	\N	http://www.megaupload.com/?d=U3PZP30I	1999/Apr/26	18.7	\N	\N	\N	Vol 21 File 11 - Vol 22 File 1-3	2010-08-23 10:26:45.607749-04
183	A Dangerous Receipe	KnKF	175mb	\N	776F6CDC	\N	http://www.megaupload.com/?d=6WD0BY0X	2000/Mar/06	19.5	\N	\N	\N	TV Original	2010-08-23 10:27:22.048291-04
519	The Meiji Restoration Mystery Tour (Part 2: Deciphering Chapter)	DCTP	172mb	\N	67CA3A32	\N	http://www.megaupload.com/?d=0X8A5WRE	2008/Dec/08	7.7	\N	\N	\N	TV Original	2010-08-23 10:32:52.244769-04
516	Fuurinkazan, The Mysterious Armored Warrior (Part 1) (1 Hour Special)	DCTP	344mb	\N	FF03FAE3	\N	http://www.megaupload.com/?d=WP85M326	2008/Nov/03	10.8	\N	\N	\N	Vol 59 File 5-10	2010-08-23 10:32:50.16274-04
209	The Falling from Mt. Ryuujin Case	AZFS	171mb	XviD	E259CFCF	Join #AZFS and type /msg AZFS|Kudo XDCC send 3	http://www.megaupload.com/?d=0HA1RB6M	2000/Oct/16	18.9	\N	\N	\N	TV Original	2010-08-23 10:27:45.613592-04
232	The Falling Off the Condo Case	AZFS	174mb	XviD	EEB51E7F	Join #AZFS and type /msg AZFS|Kudo XDCC send 8	http://www.megaupload.com/?d=O6X15C9O	2001/May/07	15.8	\N	\N	\N	TV Original	2010-08-23 10:28:08.865505-04
224	And Then There Were No Mermaids (Part 3: The Solution)	DCTP	344mb	\N	1388584C	\N	http://www.megaupload.com/?d=VVHLEWU5	2001/Feb/12	19.9	\N	\N	\N	Vol 28 File 6-10	2010-08-23 10:28:02.796568-04
181	The Nocturne of the Red Murderous Intent (Part 2)	KnKF	175mb	\N	BEAADFE6	\N	http://www.megaupload.com/?d=1AKWPDXL	2000/Feb/21	20.1	\N	\N	\N	TV Original	2010-08-23 10:27:20.593469-04
174	The 20 Year old Murder Case, The Symphony Serial Murders (2 Hour Special)	KnKF	700mb	\N	5EBFABF8	\N	http://www.megaupload.com/?d=FSSETVIT	2000/Jan/03	14.6	\N	\N	\N	Vol 23 File 4-9	2010-08-23 10:27:14.97158-04
53	The Mystery Weapon Murder Case	AConan	173mb	\N	8678BAEF	\N	http://www.megaupload.com/?d=D078YNUE	1997/Apr/07	17.3	\N	\N	\N	TV Original	2010-08-23 10:25:20.310206-04
562	Rainbow Color Kidnapping	DCTP	173mb | 320mb	XviD	8E336675 | C74F5412	\N	http://www.megaupload.com/?d=ZUJJF346 | 720: http://www.megaupload.com/?d=RVUEYBQ2	2010/Jan/16	\N	\N	\N	\N	\N	2010-08-23 10:33:35.556688-04
561	The Mansion of Death and the Red Wall: Empty Fort Strategy	DCTP	172mb | 320mb	XviD	268F7374 | 02ED1FA9	\N	http://www.megaupload.com/?d=6BB91NZE | 720: http://www.megaupload.com/?d=OC8HMI12	2009/Dec/26	\N	\N	\N	\N	Vol 65 File 8 - Vol 66 File 1	2010-08-23 10:33:34.590366-04
44	The Three Hotta Brothers Murder Case	AConan	167mb	\N	10CB5E24	\N	http://www.megaupload.com/?d=JIP7DOJN	1997/Jan/20	17.6	\N	\N	\N	TV Original	2010-08-23 10:25:12.320146-04
457	Sonoko's Red Handkerchief (Part 1)	Kaizou	175mb	\N	94459766	\N	http://www.megaupload.com/?d=DVWZN9V5	2006/Nov/20	10.2	\N	\N	\N	Vol 52 File 9-11	2010-08-23 10:31:44.788571-04
394	Big Adventure at the Unconventional Mansion (Sealing Part)	Kienai	172mb	H264	C0256C47	/MSG [xdcc]Ecchilicious XDCC SEND 375	http://www.megaupload.com/?d=PWKCZ7S9	2005/Apr/18	11.4	\N	http://www.nyaa.eu/?page=download&tid=195813	\N	Vol 46 File 7-10	2011-03-26 20:52:07.985207-04
333	Alike Princesses (Part 1)	Neg	172mb	\N	B66C1706	\N	http://www.megaupload.com/?d=CDPKFPLA	2003/Aug/25	12.9	\N	\N	\N	Vol 40 File 10 - Vol 41 File 3	2010-08-23 10:29:53.791484-04
433	Conan: A Strange Child	Kienai	173mb	\N	8B30ECF0	Join #Kienai and type /msg Sonoko-Chan XDCC send 27	http://www.megaupload.com/?d=AUOSFF7Y	2006/Mar/06	10	\N	\N	\N	TV Original	2010-08-23 10:31:27.613156-04
97	The Farewell Wine Murder Case	AConan	172mb	\N	13C8C197	\N	http://www.megaupload.com/?d=47MRKP0K	1998/Apr/13	20.1	\N	\N	\N	TV Original	2010-08-23 10:25:55.238342-04
525	Blue Spark of Hate (Part 2)	DCTP	172mb	\N	33E5E754	\N	http://www.megaupload.com/?d=2O5T2019	2009/Feb/16	9.4	\N	\N	\N	Vol 61 File 5-8	2010-08-23 10:32:56.639639-04
238	The Osaka 3 "K" Case (Part 1)	DCTP	171mb	\N	BB1D048A	\N	http://www.megaupload.com/?d=GEV36P9H	2001/Jun/18	17.3	\N	\N	\N	Vol 29 File 9-11	2010-08-23 10:28:18.804089-04
74	The Death God Jinnai Murder Case	AConan	172mb	\N	449F2616	\N	http://www.megaupload.com/?d=2QQX4EFX	1997/Sep/01	19.3	\N	\N	\N	TV Original	2010-08-23 10:25:35.868052-04
18	A June Bride Murder Case	AConan	176mb	\N	448DF412	\N	http://www.megaupload.com/?d=79X6A6WU	1996/Jun/03	14.6	\N	\N	\N	Vol 8 File 8-10	2010-08-23 10:24:51.261345-04
521	Murderer, Kudou Shinichi (1 Hour Special)	DCTP	344mb	XviD	0D16476F	\N	http://www.megaupload.com/?d=VG15DQJB	2009/Jan/19	9.6	\N	\N	\N	Vol 62 File 5-11	2010-08-23 10:32:53.761231-04
101	The Memories of First Love Case (Part 2)	AConan	172mb	\N	341DDDE3	\N	http://www.megaupload.com/?d=M8AQGN0U	1998/May/18	19.5	\N	\N	\N	Vol 18 File 3-5	2010-08-23 10:25:58.105104-04
459	A Mysterious Man, Overly Strict with Regulations	HnI	160mb	\N	0C0519C4	\N	http://www.megaupload.com/?d=ATT7UH4X	2006/Dec/04	10.9	\N	\N	\N	TV Original	2010-08-23 10:31:46.144406-04
240	The Bullet Train Transport Case (Part 1)	DCTP	171mb	\N	2B3050BE	\N	http://www.megaupload.com/?d=EMNJ2BXX	2001/Jul/02	17	\N	\N	\N	Vol 30 File 1-3	2010-08-23 10:28:20.262574-04
147	The Metropolitan Police Detective Love Story (Part 2)	Neg	175mb	\N	93EC64F9	\N	http://www.megaupload.com/?d=GPB9KXZI	1999/Jun/07	20.2	\N	\N	\N	Vol 21 File 8-10	2010-08-23 10:26:54.462868-04
428	Super Secret of the Road to School (Part 2)	Kienai	172mb	\N	D797179F	Join #Kienai and type /msg Sonoko-Chan XDCC send 26	http://www.megaupload.com/?d=U3899RNR	2006/Jan/30	10.9	\N	\N	\N	Vol 49 File 5-7	2010-08-23 10:31:24.211113-04
49	The Diplomat Murder Case (Part 2)	AConan	172mb	\N	BBC1768D	\N	http://www.megaupload.com/?d=T3D2WD80	1997/Feb/24	18.2	\N	\N	\N	Vol 10 File 2-6	2010-08-23 10:25:16.653152-04
189	The Desperate Revival: A Wounded Great Detective	KnKF	176mb	\N	BCDE6234	\N	http://www.megaupload.com/?d=4C9W6ANW	2000/May/08	20.6	\N	\N	\N	Vol 25 File 9 - Vol 26 File 7	2010-08-23 10:27:29.677504-04
524	Blue Spark of Hate (Part 1)	DCTP	172mb	XviD	80D7888F	\N	http://www.megaupload.com/?d=QHW919ZQ	2009/Feb/09	8.2	\N	\N	\N	Vol 61 File 5-8	2010-08-23 10:32:55.94422-04
371	A Course Without Protest (Part 1)	KnKF	168mb	\N	69C27BA7	\N	http://www.megaupload.com/?d=DA1MEP96	2004/Aug/23	11.8	\N	\N	\N	Vol 45 File 6-8	2010-08-23 10:30:26.590806-04
167	The Monster of the Tottori Spider Mansion (Part 2: The Suspicion)	KnKF	175mb	\N	2C304964	\N	http://www.megaupload.com/?d=MBN4VLF0	1999/Nov/08	17.7	\N	\N	\N	Vol 25 File 4-8	2010-08-23 10:27:09.558986-04
10009	Strategy Above the Depths	KnKF	700mb | 698mb	XviD	359C82E9 | 422D609A	\N	\N	2005/Apr/19	\N	On a luxurious cruise trip paid for by Sonoko, Conan and his gang enjoy the high life on the open seas. However, a shrewd plot for revenge was brewing right under Conan's nose. And this time Mori takes the spotlight, proving his own intuitions right, and beats Conan at his own deduction.	\N	\N	\N	2010-05-17 17:13:51.278958-04
20010	Kid in Trap Island	DCTP	231mb | 232mb	XviD | H264	7A4B799E | 33564CE5	\N	http://www.megaupload.com/?d=VM06KOBH | H264: http://www.megaupload.com/?d=37ZOMFSA	\N	\N	\N	\N	\N	\N	2010-08-23 10:36:56.45657-04
30002	Magic File 2 (Movie 12 Side-Story OVA)	DCTP	252mb	\N	BB07F5EC	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
511	Deduction Showdown! Shinichi VS Okiya Subaru	DCTP	172mb | 485mb	XviD	686C8273 | E527E9D6	\N	http://www.megaupload.com/?d=TC3VY7ID	2008/Aug/04	7.8	\N	\N	\N	Vol 61 File 8-10	2010-08-23 10:32:46.648048-04
495	Clash of Red and Black: Coma	DCTP	172mb | 233mb	\N	3BFE1F11 | F9314E79	\N	http://www.megaupload.com/?d=S67HB5V6 | 720: http://www.megaupload.com/?d=LZVD8JYA	2008/Feb/11	9.2	\N	\N	\N	Vol 57 File 9 - Vol 58 File 2	2010-08-23 10:32:28.123693-04
110	Cooking Classroom Murder Case (Part 1)	Atavus	171mb	XviD	1EB1F20A	\N	http://www.megaupload.com/?d=DB2CJK62	1998/Jul/27	19.1	\N	\N	\N	TV Original	2010-08-23 10:26:09.924159-04
466	Unsmashable Snowman (Part 1)	Kaizou	172mb	\N	98ED9167	\N	http://www.megaupload.com/?d=Z5DQE64E	2007/Feb/26	9.9	\N	\N	\N	Vol 54 File 3-5	2010-08-23 10:31:51.048567-04
539	A Fool's Inheritance	DCTP	171mb	XviD-MP3	A25593AD	\N	http://www.megaupload.com/?d=HNJOEULJ	2009/July/4	6.7	\N	\N	\N	TV Original	2010-08-23 10:33:07.454633-04
271	Secret Rushed Omission (Part 1)	Kaizou	172mb	\N	7557DBA3	\N	http://www.megaupload.com/?d=SVIFZO2F	2002/Mar/11	19.6	\N	\N	\N	Vol 33 File 10 - Vol 34 File 1	2010-08-23 10:28:54.185924-04
191	The Desperate Revival: The Black Knight	KnKF	175mb	\N	C17911F7	\N	http://www.megaupload.com/?d=7Z33C4KB	2000/May/22	20.2	\N	\N	\N	Vol 25 File 9 - Vol 26 File 7	2010-08-23 10:27:31.091293-04
462	Shadow of Black Organization: A Very Young Witness	Kaizou-Neg	178mb | 233mb	\N	912A3CBD | 27B0E5C1	\N	http://www.megaupload.com/?d=N1KY8F8P | 720: http://www.megaupload.com/?d=QGLVSP63	2007/Jan/29	9.7	\N	\N	\N	Vol 53 File 7 - Vol 54 File 2	2010-08-23 10:31:48.233723-04
111	Cooking Classroom Murder Case (Part 2)	Atavus	172mb	XviD	12DBC0DF	\N	http://www.megaupload.com/?d=467O0NZA	1998/Aug/03	18	\N	\N	\N	TV Original	2010-08-23 10:26:10.708243-04
493	Clash of Red and Black: Exclamation	DCTP	172mb | 233mb	\N	FD72DF16 | 5F855570	\N	http://www.megaupload.com/?d=A4T5PQ37 | 720: http://www.megaupload.com/?d=WTQDTTIG	2008/Jan/28	9.5	\N	\N	\N	Vol 56 File 10 - Vol 57 File 4	2010-08-23 10:32:26.648292-04
231	The Mysterious Passenger (Part 2)	Kaizou	173mb	\N	7CFBC791	\N	http://www.megaupload.com/?d=UFU939AS	2001/Apr/23	19.3	\N	\N	\N	Vol 29 File 3-5	2010-08-23 10:28:08.140167-04
149	The Amusement Park Bungee Jumping Case	Neg	176mb	\N	3C7D7980	\N	http://www.megaupload.com/?d=6F440EZV	1999/Jun/21	19.3	\N	\N	\N	TV Original	2010-08-23 10:26:55.856466-04
128	The Black Corps One Billion Yen Robbery Case	DCTP	173mb	XviD	7D1B5A8B	\N	http://www.megaupload.com/?d=E1FV31L2	1998/Dec/14	20.9	\N	\N	\N	TV Original	2011-02-25 23:18:35.679562-05
499	Clash of Red and Black: Disguise	DCTP	172mb | 233mb	\N	D12FD54B | 5BC2C4DF	\N	http://www.megaupload.com/?d=EADHT74Q | 720: http://www.megaupload.com/?d=FTV8YI4Y	2008/Mar/10	8.5	\N	\N	\N	Vol 58 File 3 - Vol 58 File 7	2010-08-23 10:32:31.410618-04
490	Hattori Heiji VS Kudo Shinichi, Deduction Showdown at Ski Slope (1 Hour Special)	DCTP	347mb | 466mb	\N	6F27DBA4 | FC50E0A2	\N	http://www.megaupload.com/?d=R1AM2OAW | 720: http://www.megaupload.com/?d=73P8D50W	2007/Dec/03	8.8	\N	\N	\N	Vol 50 File 8 - Vol 51 File 1	2010-08-23 10:32:23.96605-04
30	Alibi Testimony Murder Case	AConan	170mb	\N	92A002D6	\N	http://www.megaupload.com/?d=H1T5H036	1996/Aug/26	13.7	\N	\N	\N	TV Original	2010-08-23 10:25:00.763796-04
267	The Truth Behind Valentine's (Part 2: The Deduction)	Kienai	172mb	\N	467EE1AE	Join #Kienai and type /msg Sonoko-Chan XDCC send 5	http://www.megaupload.com/?d=G7AODAE0	2002/Feb/04	20	\N	\N	\N	Vol 33 File 3-6	2010-08-23 10:28:50.37587-04
242	Genta's Disaster	AZFS	172mb	XviD	D33CD745	Join #AZFS and type /msg AZFS|Kudo XDCC send 12	http://www.megaupload.com/?d=66NGF060	2001/Jul/16	19.4	\N	\N	\N	Vol 30 File 8-9	2010-08-23 10:28:26.770825-04
530	The Truth Behind the Urban Legend (Part 1)	DCTP	172mb	DivX	3302AE4A	\N	http://www.megaupload.com/?d=LGZCUYLY	2009/Apr/18	5.8	\N	\N	\N	Vol 60 File 6-8	2010-08-23 10:33:00.386456-04
416	The Evil Spirit Appeared on an Unlucky Day (Part 2: The Suspicion)	HnI	159mb	\N	DEE9936A	\N	http://www.megaupload.com/?d=ASXBJ485	2005/Oct/17	10.6	\N	\N	\N	Vol 48 File 4-8	2010-08-23 10:31:15.651704-04
509	Red, White, Yellow and the Detective Boys	DCTP	172mb | 485mb	\N	AC1B7554 | 882B5470	\N	http://www.megaupload.com/?d=YFHKDX8H	2008/Jul/14	8.5	\N	\N	\N	Vol 60 File 3-5	2010-08-23 10:32:44.994776-04
427	Super Secret of the Road to School (Part 1)	Kienai	172mb	\N	54364B63	Join #Kienai and type /msg Sonoko-Chan XDCC send 25	http://www.megaupload.com/?d=KRVIXATU	2006/Jan/23	10.6	\N	\N	\N	Vol 49 File 5-7	2010-08-23 10:31:23.469843-04
156	The Metropolitan Police Detective Love Story 2 (Part 1)	Neg	172mb	\N	81584EFD	\N	http://www.megaupload.com/?d=VU0GNROI	1999/Aug/09	16.9	\N	\N	\N	Vol 23 File 10 - Vol 24 File 2	2010-08-23 10:27:01.108418-04
356	Kaitou Kid's Miracle Midair Walk (1 Hour Special)	Kaizou	336mb	\N	0FE85F33	\N	http://www.megaupload.com/?d=YOXBL51M	2004/Apr/12	13.7	\N	\N	\N	Vol 44 File 7-10	2010-08-23 10:30:14.603195-04
92	The Fearful Mountain Trek Murder Case (Part 1)	AConan	172mb	\N	F38B0AFC	\N	http://www.megaupload.com/?d=5MOFKUAD	1998/Feb/23	19.7	\N	\N	\N	TV Original	2010-08-23 10:25:51.518693-04
491	Clash of Red and Black: The Beginning	DCTP	173mb | 233mb	\N	81DC9A9C | BD36FFA0	\N	http://www.megaupload.com/?d=PKQ84P4Z	2008/Jan/14	9.6	\N	\N	\N	Vol 56 File 10 - Vol 57 File 4	2010-08-23 10:32:24.663057-04
477	Genta's Killer Shoot (Part 2)	DCTP	172mb	\N	5D3584FF	\N	http://www.megaupload.com/?d=KD2NL5KE	2007/Jul/02	8.7	\N	\N	\N	Vol 55 File 3-5	2010-08-23 10:32:09.229956-04
396	Big Adventure at the Unconventional Mansion (Solution Part))	Kienai	172mb	H264	89E40A09	/MSG [xdcc]Ecchilicious XDCC SEND 377	http://www.megaupload.com/?d=RYZOLBD3	2005/May/02	10	\N	http://www.nyaa.eu/?page=download&tid=202100	\N	Vol 46 File 7-10	2011-03-26 20:50:48.293177-04
30021	Live Action 02	TimeLes	782mb	\N	\N	\N	\N	\N	\N	Conan attends a party for the "Miss Japanesque" awards, where he happens to eat a cake that temporarily transforms him back into Kudo Shinichi. While there, a murder takes place and he has to solve the case. The special will also feature the first live-action appearance of the Black Organization.	\N	\N	\N	2010-05-17 17:13:51.278958-04
227	The Battle Game Trap (Part 2)	Kaizou	173mb	\N	51177822	\N	http://www.megaupload.com/?d=XMU045AD	2001/Mar/05	20.4	\N	\N	\N	Vol 27 File 7-9	2010-08-23 10:28:05.187977-04
134	The Magic Lovers' Murder Case (Case Closed)	Puto	255MB	H264-AC3	38c90d6e	\N	http://www.megaupload.com/?d=HDULTMR8	1999/Feb/08	23	\N	\N	\N	Vol 20 File 2-6	2011-01-16 22:04:51.540894-05
20047	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
99	The Famous Potter Murder Case (Part 2)	AConan	172mb	\N	F6904CAF	\N	http://www.megaupload.com/?d=MCD7H1P1	1998/Apr/27	18.6	\N	\N	\N	Vol 16 File 10 - Vol 17 File 2	2010-08-23 10:25:56.63681-04
280	The Hooligan's Labyrinth (Part 2)	MCT	gb	h264	E8BB2836	\N	http://www.megaupload.com/?d=6D5LUI8F	2002/Jun/03	17.2	\N	\N	\N	Vol 34 File 5-7	2010-08-23 10:29:01.895931-04
66	Night Road Murder Case	AConan	172mb	\N	00A17344	\N	http://www.megaupload.com/?d=Z5N633IP	1997/Jul/07	18	\N	\N	\N	TV Original	2010-08-23 10:25:29.737933-04
225	The Secret of the High Sales	AZFS	172mb	XviD	8D3D6EEA	Join #AZFS and type /msg AZFS|Kudo XDCC send 5	http://www.megaupload.com/?d=AGIPFG6V	2001/Feb/19	21.6	\N	\N	\N	TV Original	2010-08-23 10:28:03.619284-04
563	Detective Boys VS Robber Group (Part 1: Turmoil)	DCTP	173mb | 320mb	XviD	C16CB080 | 2359EB05	\N	http://www.megaupload.com/?d=K1899GWQ | 720: http://www.megaupload.com/?d=DBX69M3T	2010/Jan/23	\N	\N	\N	\N	\N	2010-08-23 10:33:36.342693-04
237	The Nanki-Shirahama Mystery Tour (Part 2)	Puto	243MB	H264	dac2884e	\N	http://megaupload.com/?d=3UITLVZ7	2001/Jun/11	17.7	\N	\N	on	TV Original	2010-12-02 21:00:50.005118-05
197	A Super Car's Trap (Part 1)	KnKF	171mb	\N	98765DA0	\N	http://www.megaupload.com/?d=0LFF6WRK	2000/Jul/03	16.3	\N	\N	\N	TV Original	2010-08-23 10:27:35.465067-04
424	The Photograph Mail from a Clown	HnI	152mb	\N	89BCC6D2	\N	http://www.megaupload.com/?d=8F7W36C9	2005/Dec/19	9.4	\N	\N	\N	TV Original	2010-08-23 10:31:21.281958-04
146	The Metropolitan Police Detective Love Story (Part 1)	Neg	175mb	\N	0AF8635A	\N	http://www.megaupload.com/?d=PKESM38F	1999/May/31	21	\N	\N	\N	Vol 21 File 8-10	2010-08-23 10:26:53.758255-04
48	The Diplomat Murder Case (Part 1)	AConan	167mb	\N	C4DEA49E	\N	http://www.megaupload.com/?d=LZS1W55J	1997/Feb/17	19	\N	\N	\N	Vol 10 File 2-6	2010-08-23 10:25:15.914521-04
37	Cactus Flower Murder Case	AConan	160mb	\N	AA1A75EE	\N	http://www.megaupload.com/?d=4PQUUW9S	1996/Nov/11	18.5	\N	\N	\N	TV Original	2010-08-23 10:25:06.5461-04
78	A Distinguished Family's Consecutive Accidental Death Case (Part 2)	AConan	172mb	\N	6D09EB18	\N	http://www.megaupload.com/?d=EI0NSTDE	1997/Oct/27	19.8	\N	\N	\N	Vol 15 File 10 - Vol 16 File 3	2010-08-23 10:25:39.144487-04
17	Department Store Highjacking Case	AConan	181mb	\N	C3D57183	\N	http://www.megaupload.com/?d=BWCI86J8	1996/May/27	15.7	\N	\N	\N	TV Original	2010-08-23 10:24:50.454506-04
87	The Crane's Return of a Favor Murder Case	AConan	172mb	\N	D5C714FA	\N	http://www.megaupload.com/?d=GP64P1KI	1998/Jan/19	20.8	\N	\N	\N	TV Original	2010-08-23 10:25:47.630869-04
75	The Loan Company President's Murder Case	AConan	172mb	\N	B6B2645E	\N	http://www.megaupload.com/?d=64OT6IP2	1997/Sep/08	18.5	\N	\N	\N	Vol 15 File 7-9	2010-08-23 10:25:36.5673-04
395	Big Adventure at the Unconventional Mansion (Karakuri Part)	Kienai	172mb	H264	673C955E	/MSG [xdcc]Ecchilicious XDCC SEND 376	http://www.megaupload.com/?d=RMFZ071J	2005/Apr/25	11.4	\N	http://www.nyaa.eu/?page=download&tid=202099	\N	Vol 46 File 7-10	2011-03-26 20:51:45.18468-04
165	The Case of Disappearing Detective Boys	KnKF	175mb	\N	62E3F9FC	\N	http://www.megaupload.com/?d=AJ8KCHGP	1999/Oct/25	18	\N	\N	\N	TV Original	2010-08-23 10:27:08.065405-04
195	The Significant Music Box (Part 2)	KnKF	175mb	\N	8515BF23	\N	http://www.megaupload.com/?d=4J3V9964	2000/Jun/19	18.9	\N	\N	\N	Vol 26 File 8-10	2010-08-23 10:27:34.069378-04
326	The Red Horse within the Flames (Part 2: The Investigation)	Kienai	172mb	\N	0C548183	Join #Kienai and type /msg Sonoko-Chan XDCC send 15	http://www.megaupload.com/?d=1KMGD5JE	2003/Jun/29	14.8	\N	\N	\N	Vol 39 File 1-5	2010-08-23 10:29:47.988508-04
400	Ran's Suspicions	Shinsen-Subs	174mb	\N	0363196E	\N	http://www.megaupload.com/?d=DI82P1CX	2005/May/30	12.3	\N	\N	\N	Vol 47 File 4	2010-08-23 10:31:00.266114-04
496	Clash of Red and Black: Invasion	DCTP	172mb | 233mb	\N	832C75DF | DED5E6AE	\N	http://www.megaupload.com/?d=ZMKUQWPK | 720: http://www.megaupload.com/?d=6P6QH25Q	2008/Feb/18	8.2	\N	\N	\N	Vol 57 File 9 - Vol 58 File 2	2010-08-23 10:32:28.843826-04
487	The Metropolitan Police Detective Love Story 8, The Ring on the Left Hand (1 Hour Special)	DCTP-Neg	347mb | 466mb	\N	31221C8B | 0E97872A	\N	http://www.megaupload.com/?d=4CM6UAJ4	2007/Sep/15	10.9	\N	\N	\N	Vol 56 File 1-3	2010-08-23 10:32:21.783497-04
514	Coffee Aroma's Murderous Intent (Part 2)	DCTP	172mb | 485mb	\N	45B629CF | C7F40E2D	\N	http://www.megaupload.com/?d=ID5JF11K	2008/Sep/08	7.9	\N	\N	\N	Vol 60 File 9-11	2010-08-23 10:32:48.748049-04
419	Sword of the Eight-headed Serpent (Part 1)	Kienai	172mb	H264	DD6451DA	\N	http://www.megaupload.com/?d=CY0BTI10	2005/Nov/07	11.2	\N	\N	\N	TV Original	2010-08-23 10:31:17.749731-04
469	Kaitou Kid and the 4 Masterpieces (Part 1)	DCTP	172mb	\N	99866371	\N	http://www.megaupload.com/?d=M6HCZ1QT	2007/Apr/16	11.1	\N	\N	\N	Vol 53 File 1-4	2010-08-23 10:32:02.92155-04
215	The Bay of Revenge (Part 1)	\N	gb	XviD	49F6C29F	\N	http://www.megaupload.com/?d=0EEJ6DUR	2000/Nov/27	17.9	\N	\N	\N	TV Original	2010-08-23 10:27:50.05335-04
420	Sword of the Eight-headed Serpent (Part 2)	Kienai	172mb	H264	EED6A84C	\N	http://www.megaupload.com/?d=FJTUVUNW	2005/Nov/14	10.7	\N	\N	\N	TV Original	2010-08-23 10:31:18.476708-04
245	The Gunshot in the Sunflower Estate	AZFS	171 MB	XVID-MP3	FF4EEEEF	\N	http://www.megaupload.com/?d=9450J6FZ	2001/Aug/06	18.2	\N	\N	on	TV Original	2010-12-22 18:29:03.375869-05
373	Poisonous Spider Trap	KnKF	171mb	\N	FA170B2B	\N	http://www.megaupload.com/?d=10MR33YL	2004/Sep/06	12.6	\N	\N	\N	TV Original	2010-08-23 10:30:28.033146-04
38	Akaoni Village Fire Festival Murder Case	AConan	160mb	\N	811948F7	\N	http://www.megaupload.com/?d=4M59JZS2	1996/Nov/18	17.8	\N	\N	\N	Vol 2 File 1-3	2010-08-23 10:25:07.437353-04
531	The Truth Behind the Urban Legend (Part 2)	DCTP	172mb	\N	884F3F2A	\N	http://www.megaupload.com/?d=S3BF3IZ1	2009/Apr/25	9.1	\N	\N	\N	Vol 60 File 6-8	2010-08-23 10:33:01.095232-04
253	The Metropolitan Police Detective Love Story 4 (Part 1)	Kaizou	172mb	\N	5B9C6D5D	\N	http://www.megaupload.com/?d=7QGN63K6	2001/Oct/15	17.3	\N	\N	\N	Vol 32 File 11 - Vol 33 File 2	2010-08-23 10:28:35.025607-04
81	The Kidnapping of a Popular Artist Case (Part 1)	AConan	172mb	\N	7089777F	\N	http://www.megaupload.com/?d=RAF7CJ8E	1997/Nov/17	20.6	\N	\N	\N	Vol 15 File 4-6	2010-08-23 10:25:41.325067-04
308	The Dark Footprint (Part 2)	Kaizou	226mb	\N	D9C056A4	\N	http://www.megaupload.com/?d=LQO8YCI6	2003/Feb/03	16.2	\N	\N	\N	Vol 37 File 5-7	2010-08-23 10:29:33.246124-04
504	Clash of Red and Black: Killed in the Line of Duty	DCTP	172mb | 485mb	\N	4BCB1005 | B9A1B707	\N	http://www.megaupload.com/?d=SDCPOACC	2008/May/19	8.1	\N	\N	\N	Vol 58 File 8 - Vol 59 File 1	2010-08-23 10:32:40.97357-04
553	The Interrogation Room	DCTP	171mb	XviD	A37149EB	\N	http://www.megaupload.com/?d=YKCBLII8	2009/Oct/31	\N	\N	\N	\N	TV Original	2010-08-23 10:33:18.246521-04
364	Synchronicity Case (Part 1)	KnKF	176mb	\N	4988B7A3	\N	http://www.megaupload.com/?d=146OQKKO	2004/Jun/14	13.1	\N	\N	\N	TV Original	2010-08-23 10:30:20.874257-04
116	The Mystery Writer Disappearance Case (Part 1)	Puto-F	404MB	H264-AAC	de80222a	\N	http://www.megaupload.com/?d=YPZQBEBS	1998/Sep/14	19.7	\N	\N	\N	Vol 19 File 2-4	2011-02-25 17:34:35.929172-05
230	The Mysterious Passenger (Part 1)	Kaizou	172mb	\N	5F262FF3	\N	http://www.megaupload.com/?d=I34X2C0R	2001/Apr/16	18.4	\N	\N	\N	Vol 29 File 3-5	2010-08-23 10:28:07.362664-04
39	The Wealthy Daughter Murder Case (Part 1)	AConan	163mb	\N	A20A5263	\N	http://www.megaupload.com/?d=4HLHV0TF	1996/Nov/25	17.4	\N	\N	\N	Vol 9 File 7 - Vol 10 File 1	2010-08-23 10:25:08.343586-04
20016	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20048	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
366	The Tragedy of the Pier in Plain Sight (Part 1)	KnKF	176mb	\N	F8AD935E	\N	http://www.megaupload.com/?d=OTHM9CPE	2004/Jul/05	13.2	\N	\N	\N	Vol 45 File 3-5	2010-08-23 10:30:22.353433-04
33	Detective Boys' Survival Case	AConan	184mb	\N	A1859733	\N	http://www.megaupload.com/?d=750QCK20	1996/Oct/14	14.7	\N	\N	\N	TV Original	2010-08-23 10:25:03.211843-04
20	A Haunted Mansion Murder Case	AConan	168mb	\N	2D43F421	\N	http://www.megaupload.com/?d=ANOEG72A	1996/Jun/17	14.3	\N	\N	\N	Vol 2 File 7-10	2010-08-23 10:24:53.025287-04
162	The Case of the Flying Locked Room, Shinichi Kudo's First Case (1 Hour Special)	KnKF	350mb	\N	6BC13D6D	\N	http://www.megaupload.com/?d=2F6TRCT4	1999/Oct/04	19.8	\N	\N	\N	Vol 21 File 4-7	2010-08-23 10:27:05.715421-04
304	Trembling Metropolitan Police Headquarters, 12 Million Hostages (2 Hour Special)	DCTP	667mb	\N	56216D8A	\N	http://www.megaupload.com/?d=CVMH6EUM	2003/Jan/06	17	\N	\N	\N	Vol 36 File 8 - Vol 37 File 1	2010-08-23 10:29:30.132982-04
522	Shinichi's True Face & Ran's Tears (1 Hour Special)	DCTP	344mb	XviD	37764E7C	\N	http://www.megaupload.com/?d=7PLOMJRF	2009/Jan/26	11.3	\N	\N	\N	Vol 62 File 5-11	2010-08-23 10:32:54.468337-04
169	Venus' Kiss	KnKF	175mb	\N	5DC05E70	\N	http://www.megaupload.com/?d=7YQV7CQL	1999/Nov/22	18.1	\N	\N	\N	TV Original	2010-08-23 10:27:11.055571-04
483	The Vanished Policeman	DCTP	172mb	XviD	A66A1F55	\N	http://www.megaupload.com/?d=H4Y5IU9J	2007/Aug/13	6.7	\N	\N	\N	TV Original	2010-08-23 10:32:13.711241-04
216	The Bay of Revenge (Part 2)	\N	gb	XviD	F638835B	\N	http://www.megaupload.com/?d=L062THZZ	2000/Dec/04	18.1	\N	\N	\N	TV Original	2010-08-23 10:27:50.756678-04
26	Pet Dog Jon Murder Case	AConan	175mb	\N	4B2D3093	\N	http://www.megaupload.com/?d=K37JGJSX	1996/Jul/29	13.6	\N	\N	\N	TV Original	2010-08-23 10:24:57.767219-04
244	Mori Kogoro's Imposter (Part 2)	DCTP	350	XviD	EAA4972B	\N	http://www.megaupload.com/?d=NW97NYLK	2001/Jul/30	18.4	\N	\N	\N	Vol 31 File 2-4	2011-01-26 15:33:57.129103-05
112	The Seven Mysteries of Teitan Elementary	Atavus	171mb	XviD	B237EAD7	\N	http://www.megaupload.com/?d=WYKB2C4U	1998/Aug/10	18.9	\N	\N	\N	Vol 16 File 4-5	2010-08-23 10:26:11.438374-04
25	The False Kidnapping and Hostage Case	AConan	175mb	\N	2A9B7781	\N	http://www.megaupload.com/?d=M2CP6PN7	1996/Jul/22	16.2	\N	\N	\N	TV Original	2010-08-23 10:24:57.026083-04
257	The Mysterious Punishment from Heaven	AZFS	170mb	\N	E746202E	Join #AZFS and type /msg AZFS|Kudo XDCC send 20	http://www.megaupload.com/?d=WU2H9IDS	2001/Nov/12	19.6	\N	\N	\N	TV Original	2010-05-17 17:13:51.278958-04
468	Mysterious Case Near the Pond	DCTP	171mb	XviD	5D4E105E	\N	http://www.megaupload.com/?d=E6B45891	2007/Mar/12	10	\N	\N	on	TV Original	2010-05-17 17:13:51.278958-04
166	The Monster of the Tottori Spider Mansion (Part 1: The Murder)	KnKF	175mb	\N	CFBF0F53	\N	http://www.megaupload.com/?d=HY3QK59X	1999/Nov/01	18.5	\N	\N	\N	Vol 25 File 4-8	2010-08-23 10:27:08.829413-04
107	Mogura Star Man Murder Case (Part 1)	Atavus	172mb	XviD	D1FA39FC	\N	http://www.megaupload.com/?d=7S7PJ3BP	1998/Jun/29	18.5	\N	\N	\N	TV Original	2010-08-23 10:26:02.611879-04
148	The Sudden Streetcar Stopping Case	Neg	174mb	\N	F6A8EBAB	\N	http://www.megaupload.com/?d=HMAOYTT1	1999/Jun/14	19.7	\N	\N	\N	TV Original	2010-08-23 10:26:55.185697-04
475	Bad Luck Grand Prix	DCTP	172mb	XviD	E1FACDD4	\N	http://www.megaupload.com/?d=FDHEBADC	2007/Jun/18	10	\N	\N	\N	TV Original	2010-08-23 10:32:07.755003-04
119	The Kamen Yaiba Murder Case	Conan-A	172mb	H264	82B6F710	\N	http://www.megaupload.com/?d=EDN0F52B	1998/Oct/12	18	\N	\N	on	TV Original	2011-02-25 16:14:34.567233-05
260	The Shaking Restaurant	AZFS	174mb	XviD	DA23B0A0	Join #AZFS and type /msg AZFS|Kudo XDCC send 21	http://www.megaupload.com/?d=HQ0DHEPL	2001/Dec/03	17.3	\N	\N	\N	TV Original	2010-08-23 10:28:43.704065-04
13	The Strange Person Hunt Murder Case	AConan	220mb	\N	D4CF9F2C	\N	http://www.megaupload.com/?d=WCPQ78V0	1996/Apr/22	12.7	\N	\N	\N	Vol 2 File 4-6	2010-08-23 10:24:47.253689-04
243	Mori Kogoro's Imposter (Part 1)	DCTP	350	XviD	EAA4972B	\N	http://www.megaupload.com/?d=NW97NYLK	2001/Jul/23	19.7	\N	\N	\N	Vol 31 File 2-4	2011-01-26 15:33:55.571709-05
342	The Bride of Huis Ten Bosch (1 Hour Special)	DCTP	351mb	XviD	AEA537AB	\N	http://www.megaupload.com/?d=Y1C0XSE5	2003/Nov/17	12.9	\N	\N	\N	TV Original	2010-08-23 10:30:02.31386-04
45	Facial Pack Murder Case	AConan	170mb	\N	76BC4D37	\N	http://www.megaupload.com/?d=IMZ61GS0	1997/Jan/27	17	\N	\N	\N	TV Original	2010-08-23 10:25:13.077347-04
289	Mitsuhiko's Mystifying Forest (Part 1)	Kienai	172mb	h264	7FA43E1F	Join #Kienai and type /msg Sonoko-Chan XDCC send 9	http://www.megaupload.com/?d=0WZ1B5HZ	2002/Aug/05	15	\N	\N	\N	Vol 35 File 8-10	2010-08-23 10:29:17.645507-04
160	The Legend of the Mysterious Pagoda (Part 2)	\N	gb	\N	B79A77D2	\N	http://www.megaupload.com/?d=AX31KV10	1999/Sep/20	17.7	\N	\N	\N	TV Original	2010-08-23 10:27:04.09814-04
432	The Metropolitan Police Detective Love Story 7 (Part 2)	DCTP	174mb	\N	CD58B73F	\N	http://www.megaupload.com/?d=X6D1REYM	2006/Feb/27	9.1	\N	\N	\N	Vol 50 File 1-4	2010-08-23 10:31:26.933126-04
207	The Deduction That Was Too Good	AZFS	172mb	XviD	AC7A59A3	Join #AZFS and type /msg AZFS|Kudo XDCC send 2	http://www.megaupload.com/?d=TN5B3HUC	2000/Sep/11	21.4	\N	\N	\N	TV Original	2010-08-23 10:27:43.999851-04
377	Momotarou Mystery Solving Tour (Part 1)	KnKF	172mb	\N	5B3CABFD	\N	http://www.megaupload.com/?d=OD93TF2U	2004/Nov/08	14.9	\N	\N	\N	TV Original	2010-08-23 10:30:30.91721-04
439	And It'd Be Nice If Everybody Disappeared	DCTP	171mb	XviD	779D8E3D	\N	http://www.megaupload.com/?d=6713L5A5	2006/May/22	10.5	\N	\N	\N	TV Original	2010-08-23 10:31:31.845114-04
159	The Legend of the Mysterious Pagoda (Part 1)	\N	gb	\N	584A2EBF	\N	http://www.megaupload.com/?d=PN2M7WCJ	1999/Sep/13	19.3	\N	\N	\N	TV Original	2010-08-23 10:27:03.406273-04
106	Scoop Picture Murder Case	AConan	172mb	\N	766C8E09	\N	http://www.megaupload.com/?d=2RE1RNZH	1998/Jun/22	19.8	\N	\N	\N	TV Original	2010-08-23 10:26:01.861955-04
223	And Then There Were No Mermaids (Part 2: The Deduction)	DCTP	344mb	\N	1388584C	\N	http://www.megaupload.com/?d=VVHLEWU5	2001/Feb/05	19.6	\N	\N	\N	Vol 28 File 6-10	2010-08-23 10:28:01.904656-04
41	The Victory Flag Vandalism Case	AConan	173mb	\N	35B2122D	\N	http://www.megaupload.com/?d=LUBMWY2X	1996/Dec/09	17.6	\N	\N	\N	TV Original	2010-08-23 10:25:09.966059-04
143	The Suspicious Astronomical Observation	Neg	173mb	\N	07A2A410	\N	http://www.megaupload.com/?d=A1GCZN7Y	1999/May/03	15.6	\N	\N	\N	TV Original	2010-08-23 10:26:51.444565-04
12	Ayumi-chan's Kidnapping Case	AConan	231mb	\N	1CAD6E80	\N	http://www.megaupload.com/?d=YZJW2EM0	1996/Apr/15	13.6	\N	\N	\N	Vol 9 File 1-3	2010-08-23 10:24:46.512042-04
252	The Kidnapper in the Picture	AZFS	171mb	XviD	67D69097	Join #AZFS and type /msg AZFS|Kudo XDCC send 17	http://www.megaupload.com/?d=0RQE9UME	2001/Oct/08	17.5	\N	\N	\N	TV Original	2010-11-30 01:08:41.656093-05
30027	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
380	Case of the Long Sleeved Kimono at the Hidden Hot Spring on a Snowy Night (Part 2)	\N	gb	\N	43773BC2	\N	http://www.megaupload.com/?d=QZUADL45	2004/Nov/29	11.8	\N	\N	\N	TV Original	2010-08-23 10:30:33.64643-04
20049	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
20050	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N	2010-05-17 17:13:51.278958-04
100	The Memories of First Love Case (Part 1)	AConan	172mb	\N	D0B6ADD7	\N	http://www.megaupload.com/?d=MO68NX67	1998/May/11	20.3	\N	\N	\N	Vol 18 File 3-5	2010-08-23 10:25:57.37034-04
57	Holmes Freak Murder Case (Part 1)	AConan	172mb	\N	EAAEB05D	\N	http://www.megaupload.com/?d=SM1CKVFD	1997/May/05	16	\N	\N	\N	Vol 12 File 7 - Vol 13 File 1	2010-08-23 10:25:23.160784-04
301	Parade of Malice and Saint (Part 1)	DCTP	343mb	\N	3722E962	\N	http://www.megaupload.com/?d=K8TYY6KD	2002/Nov/25	15.4	\N	\N	\N	Vol 36 File 5-7	2010-08-23 10:29:27.385328-04
269	The Forgotten Memento from the Crime (Part 1)	Kienai	172mb	h264	FECC0A05	Join #Kienai and type /msg Sonoko-Chan XDCC send 7	http://www.megaupload.com/?d=Q3TZ69PA	2002/Feb/18	18.7	\N	\N	\N	Vol 33 File 7-9	2010-08-23 10:28:52.254305-04
274	Truth About the Haunted House (Part 1)	DCTP-AZFS	350mb	XviD	2E75803F	Join #AZFS and type /msg AZFS|Kudo XDCC send 25	http://www.megaupload.com/?d=2GHYKWF5	2002/Apr/15	17.8	\N	Two parter, contains 275 also	\N	Vol 35 File 5-7	2010-12-01 14:46:02.144445-05
204	The Black Wings of Icarus (Part 2)	DCTP	350mb	\N	57A97A97	\N	http://www.megaupload.com/?d=0JDG3QQU	2000/Aug/21	18.7	\N	\N	\N	TV Original	2010-08-23 10:27:41.049796-04
89	Dracula Villa Murder Case (Part 2)	AConan	172mb	\N	C27DBA8F	\N	http://www.megaupload.com/?d=31PACSK0	1998/Feb/02	20	\N	\N	\N	TV Original	2010-08-23 10:25:49.120534-04
176	Meeting with the Black Organization Again (Part 1)	KnKF	175mb	\N	BABEBFDA	\N	http://www.megaupload.com/?d=GWO4NW49	2000/Jan/17	22	\N	\N	\N	Vol 24 File 7-11	2010-08-23 10:27:16.705705-04
140	SOS! A Messages from Ayumi	Neg	177mb	\N	BCD9D3C8	\N	http://www.megaupload.com/?d=Z9JZ6TO5	1999/Apr/12	18.4	\N	\N	\N	TV Original	2010-08-23 10:26:41.058122-04
29	Computer Murder Case	AConan	169mb	\N	33EBDD5A	\N	http://www.megaupload.com/?d=5AC765RM	1996/Aug/19	12.9	\N	\N	\N	TV Original	2010-08-23 10:24:59.982811-04
210	The Water Palace of Five Colors (Part 1)	\N	gb	\N	9B826A8A	\N	http://www.megaupload.com/?d=Y7G3C9DQ	2000/Oct/23	19.6	\N	\N	\N	TV Original	2010-08-23 10:27:46.299542-04
264	The Courtroom Battle: Kisaki VS Kogoro (Part 1)	\N	gb	\N	7BC84F90	\N	http://www.megaupload.com/?d=0XU8I92W	2002/Jan/14	17.8	\N	\N	\N	TV Original	2010-08-23 10:28:47.574506-04
438	The Pursuit of the Fish Mail	\N	gb	\N	3CF87E6C	\N	http://www.megaupload.com/?d=786SL1DE	2006/May/15	11	\N	\N	\N	Vol 51 File 2-3	2010-08-23 10:31:30.920392-04
456	The Mystery I Loved	HnI	162mb	\N	777634B0	\N	http://www.megaupload.com/?d=O99HJ8YW	2006/Nov/13	9.7	\N	\N	\N	TV Original	2010-08-23 10:31:44.081735-04
408	Conan and Heiji's Reasoning Magic (Part 3: The Solution)	\N	gb	\N	9EB951CF	\N	http://www.megaupload.com/?d=MU74XD0U	2005/Aug/01	8.6	\N	\N	\N	Vol 47 File 8-11	2010-08-23 10:31:09.925408-04
407	Conan and Heiji's Reasoning Magic (Part 2: The Mansion)	\N	gb	\N	CD2083E2	\N	http://www.megaupload.com/?d=114T8EPO	2005/Jul/18	10.8	\N	\N	\N	Vol 47 File 8-11	2010-08-23 10:31:09.171439-04
287	Kudo Shinichi's NY Case (Part 2)	Kaizou	225mb	\N	3813C54A	\N	http://www.megaupload.com/?d=E1510SHN	2002/Jul/22	16.5	\N	\N	\N	Vol 34 File 11 - Vol 35 File 4	2010-08-23 10:29:11.134975-04
51	The Golf Driving Range Murder Case	AConan	172mb	\N	445A16D5	\N	http://www.megaupload.com/?d=JZCJYD7X	1997/Mar/10	15.3	\N	\N	\N	TV Original	2010-08-23 10:25:18.134092-04
152	The Mysterious Old Man Disappearance Case	Neg	173mb	\N	1D95568B	\N	http://www.megaupload.com/?d=J4H708M4	1999/Jul/12	19.6	\N	\N	\N	TV Original	2010-08-23 10:26:58.072797-04
83	General Hospital Murder Case	AConan	172mb	\N	3D9F6B92	\N	http://www.megaupload.com/?d=45Y5SZ9G	1997/Dec/01	23.1	\N	\N	\N	TV Original	2010-08-23 10:25:44.192159-04
341	Hidden Bathroom Secret (Part 2)	DCTP	172mb	\N	D5A62685	\N	http://www.megaupload.com/?d=Z842F6BG	2003/Nov/10	13.1	\N	\N	\N	Vol 41 File 10 - Vol 42 File 1	2010-08-23 10:30:01.38023-04
221	The Client Full of Lies (Part 2)	DCTP	344mb	\N	0AC8F430	\N	http://www.megaupload.com/?d=17C0PMYZ	2001/Jan/22	20.3	\N	\N	\N	Vol 28 File 3-5	2010-08-23 10:27:55.309668-04
426	Love Letter to Ran	\N	gb	\N	EF80A53A	\N	http://www.megaupload.com/?d=H4RPWNTR	2006/Jan/16	9.8	\N	\N	\N	TV Original	2010-08-23 10:31:22.756028-04
27	Kogoro's Class Reunion Murder Case (Part 1)	AConan	168mb	\N	29D2940D	\N	http://www.megaupload.com/?d=H5F29J73	1996/Aug/05	13.4	\N	\N	\N	Vol 9 File 4-6	2010-08-23 10:24:58.484554-04
158	The Silent Kanjo Line	Neg	172mb	\N	3576093A	\N	http://www.megaupload.com/?d=GT32JV20	1999/Sep/06	17.8	\N	\N	\N	TV Original	2010-08-23 10:27:02.676787-04
124	A Mysterious Sniper Murder Case (Part 1)	DCTP	350 MB	XVID-MP3	B6DE84CB	\N	http://www.megaupload.com/?d=W4QXGOPQ	1998/Nov/16	19.9	\N	\N	on	TV Original	2011-02-23 20:06:01.264901-05
398	Weird Family's Request (Part 1)	\N	gb	\N	1DAE8482	\N	http://www.megaupload.com/?d=HZT1EMH2	2005/May/16	12.7	\N	\N	\N	Vol 46 File 11 - Vol 47 File 3	2010-08-23 10:30:58.296398-04
316	The Sullied Masked Hero (Part 1)	DCTP	348mb	XviD	2411959B	\N	http://www.megaupload.com/?d=ZW5IJM23	2003/Apr/21	15.4	\N	Part 2 is concatenated.	\N	Vol 38 File 5-7	2010-08-23 10:29:39.535645-04
60	An Illustrator Murder Case	AConan	172mb	\N	9A37D2D8	\N	http://www.megaupload.com/?d=Y2IR7H0R	1997/May/26	19.5	\N	\N	\N	Vol 13 File 5-7	2010-08-23 10:25:25.308203-04
153	Sonoko's Dangerous Summer Story (Part 1)	Neg	174mb	\N	20F59BCF	\N	http://www.megaupload.com/?d=0MQIQ6JM	1999/Jul/19	18.3	\N	\N	\N	Vol 22 File 8-10	2010-08-23 10:26:58.79447-04
50	Library Murder Case	AConan	168mb	\N	44495EDE	\N	http://www.megaupload.com/?d=91YHWFJ8	1997/Mar/03	16.8	\N	\N	\N	Vol 10 File 7-8	2010-08-23 10:25:17.377107-04
117	The Mystery Writer Disappearance Case (Part 2)	Puto-F	404MB	H264-AAC	de80222a	\N	http://www.megaupload.com/?d=YPZQBEBS	1998/Sep/21	17.4	\N	\N	\N	Vol 19 File 2-5	2011-02-25 17:34:41.988589-05
79	The Bank Heist Murder Case	AConan	172mb	\N	B266E4D0	\N	http://www.megaupload.com/?d=MB0XX7HS	1997/Nov/03	18.9	\N	\N	\N	TV Original	2010-08-23 10:25:39.9046-04
291	Solitary Island of the Princess and the Dragon King's Palace (Part 1)	\N	gb	\N	BB65FE79	\N	http://www.megaupload.com/?d=FEG0XW1N	2002/Aug/19	17	\N	\N	\N	Vol 35 File 11 - Vol 36 File 4	2010-08-23 10:29:19.103957-04
372	A Course Without Protest (Part 2)	KnKF	171mb	\N	708F0077	\N	http://www.megaupload.com/?d=JWJZ3VW1	2004/Aug/30	13.1	\N	\N	\N	Vol 45 File 6-8	2010-08-23 10:30:27.339975-04
246	The Mystery in the Net (Part 1)	DCTP	347mb	XviD	5F7ECB81	\N	http://www.megaupload.com/?d=BZJEMGS9	2001/Aug/13	13.6	\N	Part 2 is concatenated.	\N	Vol 31 File 5-7	2010-08-23 10:28:29.693481-04
64	The Third Fingerprint Murder Case	AConan	172mb	\N	49C7C8E2	\N	http://www.megaupload.com/?d=SXCPB85X	1997/Jun/23	18	\N	\N	\N	TV Original	2010-08-23 10:25:28.331229-04
403	The Mysterious Angel's Mansion (Part 1)	\N	gb	\N	50313521	\N	http://www.megaupload.com/?d=RWTULF9K	2005/Jun/20	11.5	\N	\N	\N	TV Original	2010-08-23 10:31:03.19498-04
150	The Truth of the Exploding Car Case (Part 1)	Neg	173mb	\N	119A37EC	\N	http://www.megaupload.com/?d=9WCCM6A3	1999/Jun/28	17.2	\N	\N	\N	TV Original	2010-08-23 10:26:56.538627-04
91	Hospitalized Burgulary Suspect Case	AConan	172mb	\N	A6F17D48	\N	http://www.megaupload.com/?d=183YECZY	1998/Feb/16	21.5	\N	\N	\N	Vol 17 File 6	2010-08-23 10:25:50.629726-04
155	The Underwater Key in the Locked Room Case	Neg	172mb	\N	A75E88A1	\N	http://www.megaupload.com/?d=3QHMOHKI	1999/Aug/02	16.4	\N	\N	\N	TV Original	2010-08-23 10:27:00.326451-04
86	The Kidnapping Location Case	AConan	172mb	\N	81F600B8	\N	http://www.megaupload.com/?d=C9XT8LYA	1998/Jan/12	22.7	\N	\N	\N	Special Vol 1 File 2	2010-08-23 10:25:46.815256-04
118	The Naniwa Serial Murder Case (1 Hour Special)	\N	gb	\N	6FC6004C	\N	http://www.megaupload.com/?d=GBVQYDYO	1998/Sep/28	21.2	\N	\N	\N	Vol 19 File 5-8	2010-08-23 10:26:21.385876-04
52	The Mist Goblin Legend Murder Case (1 Hour Special)	AConan	344mb	\N	D473694A	\N	http://www.megaupload.com/?d=0KVFK0PH	1997/Mar/17	17.9	\N	\N	\N	Vol 11 File 8 - 10	2010-08-23 10:25:19.551882-04
282	The Mystery of the Water-Flowing Stone Garden (Part 1)	DCTP	350 MB	XviD	9BED9C34	\N	http://www.megaupload.com/?d=SG6A7VR7	2002/Jun/17	16.5	\N	\N	\N	TV Original	2011-01-26 22:23:39.443474-05
85	Ski Lodge Murder Case (Part 2)	AConan	172mb	\N	E15CFC51	\N	http://www.megaupload.com/?d=HOP7GQ7W	1997/Dec/15	21.2	\N	\N	\N	Vol 14 File 9 - Vol 15 File 3	2010-08-23 10:25:45.929233-04
467	Unsmashable Snowman (Part 2)	Kaizou	172mb	\N	0ED1C3E2	\N	http://www.megaupload.com/?d=FWC23FMU	2007/Mar/05	8.3	\N	\N	\N	Vol 54 File 3-5	2010-08-23 10:31:51.716003-04
528	Might Over Mystery (Part 1)	DCTP	172mb	\N	2A348B59	\N	http://www.megaupload.com/?d=X3PTEMC1	2009/Mar/09	7.4	\N	\N	\N	Vol 62 File 2-4	2010-08-23 10:32:58.864933-04
359	The Metropolitan Police Detective Love Story 5 (Part 2)	Kaizou-Neg	172mb	\N	6DC042DC	\N	http://www.megaupload.com/?d=6ATJ3R5F	2004/May/10	12.8	\N	\N	\N	Vol 40 File 1-3	2010-08-23 10:30:17.293786-04
59	The First Errand Murder Case	AConan	172mb	\N	5DBA8AA7	\N	http://www.megaupload.com/?d=MP146DUO	1997/May/19	19.2	\N	\N	\N	TV Original	2010-08-23 10:25:24.59746-04
526	A Present from the True Culprit	\N	gb	\N	97258452	\N	http://www.megaupload.com/?d=RHN2DWI0	2009/Feb/23	8.9	\N	\N	\N	Vol 61 File 11 - Vol 62 File 1	2010-08-23 10:32:57.365857-04
362	Teitan High School's Ghost Story (Part 2)	Shi-Fa&KNKF	178mb	\N	5ABF0899	\N	http://www.megaupload.com/?d=K109XSF8	2004/May/31	13.1	\N	\N	\N	Vol 44 File 11 - Vol 45 File 2	2010-08-23 10:30:19.432848-04
319	The Cigar Case of Good Fortune (Part 2)	Kienai	172mb	H264	8800AA1E	\N	http://www.megaupload.com/?d=XEBSD6IE	2003/May/12	15.1	\N	\N	on	TV Original	2010-09-18 20:41:13.670175-04
34	The Mountain Villa Bandaged Man Murder Case (Part 1)	AConan	170mb	\N	92B7A16B	\N	http://www.megaupload.com/?d=FUXU34R6	1996/Oct/21	17.8	\N	\N	\N	Vol 5 File 1-5	2010-08-23 10:25:03.957853-04
258	The Man from Chicago (Part 1)	Kaizou	172mb	\N	DBE80602	\N	http://www.megaupload.com/?d=I1TNU2TJ	2001/Nov/19	19.3	\N	\N	\N	Vol 32 File 8-10	2010-08-23 10:28:39.234914-04
186	The Murdered Famous Detective (Part 2)	KnKF	175mb	\N	B5FAC159	\N	http://www.megaupload.com/?d=L5GI9ARX	2000/Apr/17	18.9	\N	\N	\N	TV Original	2010-08-23 10:27:27.428864-04
154	Sonoko's Dangerous Summer Story (Part 2)	Neg	174mb	\N	F85274CD	\N	http://www.megaupload.com/?d=VG38BOGG	1999/Jul/26	17.3	\N	\N	\N	Vol 22 File 8-10	2010-08-23 10:26:59.513235-04
307	The Dark Footprint (Part 1)	Kaizou	226mb	\N	E2D65F07	\N	http://www.megaupload.com/?d=KMY5AK56	2003/Jan/27	16.5	\N	\N	\N	Vol 37 File 5-7	2010-08-23 10:29:32.505157-04
478	Real 30 Minutes	DCTP	172mb	\N	EE23923B	\N	http://www.megaupload.com/?d=7VB70K9Y	2007/Jul/09	9.1	\N	\N	\N	TV Original	2010-08-23 10:32:09.968706-04
182	The Big Investigation of the Nine Doors	KnKF	175mb	\N	C51366F9	\N	http://www.megaupload.com/?d=FT8W7O51	2000/Feb/28	21	\N	\N	\N	TV Original	2010-08-23 10:27:21.342375-04
132	The Magic Lovers' Murder Case (Case Part)	Puto	250MB	H264-AC3	14715c2e	\N	http://www.megaupload.com/?d=SBP7ER9W	1999/Jan/25	21.9	\N	\N	\N	Vol 20 File 2-6	2011-01-16 22:04:21.459657-05
273	Quiz-Lady's Dart Clue Case	AZFS	172mb	XviD	DD2E01DF	Join #AZFS and type /msg AZFS|Kudo XDCC send 24	http://www.megaupload.com/?d=GF2J4YDO	2002/Apr/08	17.9	\N	\N	on	TV Original	2010-12-05 13:41:38.732519-05
402	The Caught Red-handed Jewel Robber (Part 2)	Kaizou	167mb	\N	9169FA30	\N	http://www.megaupload.com/?d=K7UD67R9	2005/Jun/13	11.4	\N	\N	\N	Vol 47 File 5-7	2010-08-23 10:31:02.37424-04
480	Yellow Alibi	DCTP	171mb	XviD	3050B6D1	\N	http://www.megaupload.com/?d=WMPCE3C0	2007/Jul/23	9.4	\N	\N	\N	TV Original	2010-08-23 10:32:11.517718-04
343	The Convenience Store Trap (Part 1)	DCTP	172mb	\N	76A7914B	\N	http://www.megaupload.com/?d=R143MN5L	2003/Dec/01	13	\N	\N	\N	Vol 42 File 2-4	2010-08-23 10:30:03.187881-04
276	Case of the Missing Policeman's Notebook	\N	gb	\N	AF9F5452	\N	http://www.megaupload.com/?d=E23MI6ML	2002/May/06	19.4	\N	\N	\N	TV Original	2010-08-23 10:28:58.269628-04
115	The Scuba Diving Murder Case (Part 2)	Atavus-MCT	171mb	XviD	18F93A3E	\N	http://www.megaupload.com/?d=0MSRDOT8	1998/Aug/31	19.8	\N	\N	\N	Vol 17 File 3-5	2010-08-23 10:26:19.032221-04
450	Trick VS Magic (Part 1)	\N	gb	\N	600984B9	\N	http://www.megaupload.com/?d=C5MSQMR1	2006/Aug/28	9.4	\N	\N	\N	TV Original	2010-08-23 10:31:39.919638-04
370	Running Away in a Game	KnKF	176mb	\N	9E9786BD	\N	http://www.megaupload.com/?d=NL3US52R	2004/Aug/09	11.5	\N	\N	\N	TV Original	2010-08-23 10:30:25.864393-04
385	The Dissonance of the Stradivarius (Part 1: Prelude)	DCTP	172mb	\N	AAE5D213	\N	http://www.megaupload.com/?d=GWCN70QA	2005/Jan/24	12.3	\N	\N	\N	Vol 46 File 2-6	2010-08-23 10:30:45.317879-04
453	Preview Meeting of Friendship and Fate	\N	gb	\N	77E1DA00	\N	http://www.megaupload.com/?d=XNGZWCIA	2006/Oct/23	9	\N	\N	\N	Vol 52 File 1-2   (First HD)	2010-08-23 10:31:41.948317-04
114	The Scuba Diving Murder Case (Part 1)	Atavus-MCT	172mb	XviD	55C159DC	\N	http://www.megaupload.com/?d=QCCU74BT	1998/Aug/24	20.1	\N	\N	\N	Vol 17 File 3-5	2010-08-23 10:26:13.222594-04
455	Upsetting Outcome (Part 2)	anfs	184mb	XviD	5E7FA899	\N	http://www.megaupload.com/?d=KTHGVUF4	2006/Nov/06	8.7	\N	\N	\N	Vol 52 File 6-8	2010-08-23 10:31:43.373546-04
344	The Convenience Store Trap (Part 2)	DCTP	172mb	\N	7D7E0B51	\N	http://www.megaupload.com/?d=SXGIGHCO	2003/Dec/08	16	\N	\N	\N	Vol 42 File 2-4	2010-08-23 10:30:04.115953-04
95	Kogoro's Date Murder Case	AConan	172mb	\N	F7838CEF	\N	http://www.megaupload.com/?d=APUAYQ91	1998/Mar/16	16.8	\N	\N	\N	TV Original	2010-08-23 10:25:53.750367-04
184	A Cursed Mask Coldly Laughs (1 Hour Special)	KnKF	350mb	\N	5FD4A836	\N	http://www.megaupload.com/?d=RAOFVDGT	2000/Mar/13	20.1	\N	\N	\N	TV Original	2010-08-23 10:27:25.833228-04
330	A Friendship That Can't Be Bought (Part 2)	DCTP	172mb	\N	2042CB43	\N	http://www.megaupload.com/?d=NCLT26QN	2003/Aug/04	14.5	\N	\N	\N	Vol 39 File 6-8	2010-08-23 10:29:51.208946-04
476	Genta's Killer Shoot (Part 1)	DCTP	172mb	\N	92F68EFA	\N	http://www.megaupload.com/?d=58HD0OLH	2007/Jun/25	9.3	\N	\N	\N	Vol 55 File 3-5	2010-08-23 10:32:08.473386-04
445	Secret of the Russian Blue	\N	gb	\N	1A1F18BE	\N	http://www.megaupload.com/?d=DND0OV1V	2006/Jul/10	9.3	\N	\N	\N	Vol 51 File 7-8	2010-08-23 10:31:36.371489-04
404	The Mysterious Angel's Mansion (Part 2)	\N	gb	\N	F2905876	\N	http://www.megaupload.com/?d=94ORX99H	2005/Jun/27	11	\N	\N	\N	TV Original	2010-08-23 10:31:03.962283-04
103	Historical Play Performer Murder Case (Part 2)	AConan	172mb	\N	96F07295	\N	http://www.megaupload.com/?d=9DYBHZUI	1998/Jun/01	17.6	\N	\N	\N	Vol 17 File 10 - Vol 18 File 2	2010-08-23 10:25:59.572525-04
88	Dracula Villa Murder Case (Part 1)	AConan	172mb	\N	FF5B52A4	\N	http://www.megaupload.com/?d=KE644CDY	1998/Jan/26	20.8	\N	\N	\N	TV Original	2010-08-23 10:25:48.434115-04
161	The Murder Floating in the Stream Restaurant	Neg-Kaizou	175mb	\N	0517C3A8	\N	http://www.megaupload.com/?d=SXNWE5OI	1999/Sep/27	20.1	\N	\N	\N	TV Original	2010-08-23 10:27:04.951354-04
451	Trick VS Magic (Part 2)	\N	gb	\N	0A2DC789	\N	http://www.megaupload.com/?d=SZ6XPBDO	2006/Sep/04	8.8	\N	\N	\N	TV Original	2010-08-23 10:31:40.609916-04
540	The Day Mouri Kogorou Ceased Being A Detective (Part 1)	DCTP	172mb | 527mb	XviD	97902320 | DD063EDF	\N	http://www.megaupload.com/?d=T8RXZMTF	2009/July/11	5.9	\N	\N	\N	TV Original	2010-08-23 10:33:08.247762-04
351	The Forgetten Cellphone (Part 2)	DCTP	350 MB	XviD	DA59B622	\N	http://www.megaupload.com/?d=HHSW0S73	2004/Feb/16	12.2	\N	\N	\N	Vol 43 File 3-5	2011-01-26 14:17:28.976481-05
334	Alike Princesses (Part 2)	Neg	173mb	\N	681D5915	\N	http://www.megaupload.com/?d=U16ZUQLK	2003/Sep/01	15	\N	\N	\N	Vol 40 File 10 - Vol 41 File 3	2010-08-23 10:29:55.424454-04
121	The Sealed Bathroom Murder Case (Part 1)	Puto-F	250MB	H264	ce857e31	\N	http://www.megaupload.com/?d=HNOGWOBO	1998/Oct/26	20.9	\N	\N	\N	Vol 20 File 7-9	2011-01-16 22:02:13.653722-05
213	Mushrooms, Bears, and the Detective Boys (Part 2)	\N	gb	\N	CA243F6D	\N	http://www.megaupload.com/?d=L9CS3D3U	2000/Nov/13	19.7	\N	\N	\N	Vol 27 File 10 - Vol 28 File 2	2010-08-23 10:27:48.613229-04
212	Mushrooms, Bears, and the Detective Boys (Part 1)	\N	gb	\N	CA243F6D	\N	http://www.megaupload.com/?d=L9CS3D3U	2000/Nov/06	18.2	\N	\N	\N	Vol 27 File 10 - Vol 28 File 2	2010-08-23 10:27:47.892533-04
315	Place Exposed to the Sun	DCTP	171mb	XviD	0073C61A	\N	http://www.megaupload.com/?d=7M0P60VG	2003/Apr/14	14.9	\N	\N	\N	TV Original	2010-08-23 10:29:38.758054-04
399	Weird Family's Request (Part 2)	\N	gb	\N	E8DDD693	\N	http://www.megaupload.com/?d=ITOEAXZX	2005/May/23	12.2	\N	\N	\N	Vol 46 File 11 - Vol 47 File 3	2010-08-23 10:30:59.568439-04
448	The Black-eyed Saury Case	\N	gb	\N	3F19027A	\N	http://www.megaupload.com/?d=SCJ6V6V0	2006/Aug/14	8	\N	\N	\N	TV Original	2010-08-23 10:31:38.522785-04
375	A Code for Star and Tabacco (Part 2)	KnKF	173mb	\N	3DA3D7BF	\N	http://www.megaupload.com/?d=L5L4EO0T	2004/Oct/25	14.4	\N	\N	\N	Vol 45 File 9 - Vol 46 File 1	2010-08-23 10:30:29.439055-04
552	The Culprit is Genta's Dad (Part 2)	DCTP	170mb	XViD	0ABA0722	\N	http://www.megaupload.com/?d=9BQZAP53	2009/Oct/24	\N	\N	\N	\N	Vol 63 File 6-8	2010-08-23 10:33:17.356692-04
265	The Courtroom Battle: Kisaki VS Kogoro (Part 2)	\N	gb	\N	D5A317D1	\N	http://www.megaupload.com/?d=XM6IVT6D	2002/Jan/21	21.3	\N	\N	\N	TV Original	2010-08-23 10:28:48.467188-04
300	Kanmon Strait of Friendship & Murderous Intent (Part 2)	Kienai	172 MB	H264	0283716B	\N	http://www.fileserve.com/file/v6u6zqw	2002/Nov/18	15.3	\N	\N	\N	TV Original	2011-05-22 20:23:43.680985-04
113	The White Sand Beach Murder Case	Atavus	171mb	XviD	83E384F7	\N	http://www.megaupload.com/?d=055QDKT7	1998/Aug/17	18.4	\N	\N	\N	Special Vol 1 File 6	2010-08-23 10:26:12.387586-04
322	The Vanished Kidnapper's Getaway Vehicle (Part 2)	Kienai	172 MB	H264	327361F5	\N	http://www.megaupload.com/?d=YY7CIEF2	2003/Jun/02	14.1	\N	\N	\N	TV Original	2011-02-20 20:30:18.550529-05
415	The Evil Spirit Appeared on an Unlucky Day (Part 1: The Case)	HnI	180mb	\N	D5C6D422	\N	http://www.megaupload.com/?d=V9K6Y60W	2005/Oct/10	11.3	\N	\N	\N	Vol 48 File 4-8	2010-08-23 10:31:14.967867-04
6	Valentine Murder Case	AConan	210mb	\N	5A16D9BA	\N	http://www.megaupload.com/?d=JGROZCZ2	1996/Feb/12	12	\N	\N	\N	TV Original	2010-08-23 10:24:41.444248-04
196	The Invisible Weapon, Ran's First Investigations	KnKF	171mb	\N	40A35E42	\N	http://www.megaupload.com/?d=TVN26ZOH	2000/Jun/26	18.7	\N	\N	\N	TV Original	2010-08-23 10:27:34.766232-04
108	Mogura Star Man Murder Case (Part 2)	Atavus	172mb	XviD	8977AF04	\N	http://www.megaupload.com/?d=YNF7KOR5	1998/Jul/06	19.5	\N	\N	\N	TV Original	2010-08-23 10:26:03.398971-04
417	The Evil Spirit Appeared on an Unlucky Day (Part 3: The Solution)	HnI	157mb	\N	0F57C657	\N	http://www.megaupload.com/?d=VSSSWNJ8	2005/Oct/24	11.1	\N	\N	\N	Vol 48 File 4-8	2010-08-23 10:31:16.342872-04
353	The Fishing Tournament Tragedy (Part 2)	DCTP	347mb	XviD	34715510	\N	http://www.megaupload.com/?d=7K75JP9Y	2004/Mar/01	13.8	\N	Part 1 is concatenated.	\N	TV Original	2010-08-23 10:30:12.006089-04
346	Find the Buttocks' Mark (Part 1)	DCTP	172mb	XviD	415766DC	\N	http://www.megaupload.com/?d=XNN09RHW	2004/Jan/12	12.5	\N	\N	\N	Vol 42 File 11 - Vol 43 File 2	2010-08-23 10:30:05.961935-04
313	Festival Dolls Dyed in the Setting Sun (Part 2)	\N	gb	XviD	0CC54970	\N	http://www.megaupload.com/?d=2VVQLRI0	2003/Mar/10	14.3	\N	\N	\N	Vol 38 File 2-4	2010-08-23 10:29:37.169019-04
318	The Cigar Case of Good Fortune (Part 1)	Kienai	171mb	H264	F0DB8BA4	\N	http://www.megaupload.com/?d=L8I7VEF0	2003/May/05	14.6	\N	\N	on	TV Original	2010-09-18 20:40:43.185118-04
70	The Night Baron Murder Case (Part 3: The Solution)	AConan	172mb	\N	8A40F61C	\N	http://www.megaupload.com/?d=WFV8GZDW	1997/Aug/04	17.2	\N	\N	\N	Vol 8 File 2-7	2010-08-23 10:25:32.932805-04
93	The Fearful Mountain Trek Murder Case (Part 2)	AConan	172mb	\N	93754999	\N	http://www.megaupload.com/?d=O5LCECH7	1998/Mar/02	19.9	\N	\N	\N	TV Original	2010-08-23 10:25:52.245539-04
348	Love, Ghosts, and World Heritage (Part 1)	DCTP	350mb	XviD	E6388A1A	\N	http://www.megaupload.com/?d=ZXWEZ40H	2004/Jan/26	13.5	\N	v2 available. Part 2 is concatenated.	\N	TV Original	2010-08-23 10:30:07.58215-04
545	The Witch Enshrouded in Fog (Part 1)	DCTP	172mb	XViD	3B02B0B4	\N	http://www.megaupload.com/?d=433FLRLH	2009/Sept/5	8.4	\N	\N	\N	Vol 63 File 9-11	2010-08-23 10:33:12.011143-04
474	The Love of Lawyer Kisaki Eri	DCTP	172mb	\N	967EDEF0	\N	http://www.megaupload.com/?d=KM1FLOI7	2007/Jun/04	8.6	\N	\N	\N	Vol 55 File 10-11	2010-08-23 10:32:06.817591-04
284	Chinatown Deja Vu in the Rain (Part 1)	Kaizou	247mb	\N	A9A544F9	\N	http://www.megaupload.com/?d=38IC7KC9	2002/Jul/01	17.3	\N	\N	\N	Vol 34 File 8-10	2010-08-23 10:29:08.731163-04
336	Secret of the Touto Film Developer Place (Part 2)	DCTP	172mb	\N	DA3BF7B0	\N	http://www.megaupload.com/?d=DMT7LE1H	2003/Sep/15	12.5	\N	\N	\N	Vol 41 File 4-6	2010-08-23 10:29:57.032727-04
293	Solitary Island of the Princess and the Dragon King's Palace (Part 3)	\N	gb	\N	3B4D9CF1	\N	http://www.megaupload.com/?d=G7D8DI0F	2002/Sep/02	18.4	\N	\N	\N	Vol 35 File 11 - Vol 36 File 4	2010-08-23 10:29:20.683691-04
187	The Mysterious Gun Shot in the Dark	KnKF	175mb	\N	17C11F91	\N	http://www.megaupload.com/?d=U9ASOUE1	2000/Apr/24	21.6	\N	\N	\N	TV Original	2010-08-23 10:27:28.161748-04
352	The Fishing Tournament Tragedy (Part 1)	DCTP	347mb	XviD	34715510	\N	http://www.megaupload.com/?d=7K75JP9Y	2004/Feb/23	13.6	\N	Part 2 is concatenated.	\N	TV Original	2010-08-23 10:30:11.022809-04
323	Hattori Heiji's Desperate Situation! (Part 1)	Kaizou	173mb	\N	C13D5F40	\N	http://www.megaupload.com/?d=R8YDTAZQ	2003/Jun/09	16.8	\N	\N	\N	Vol 38 File 8-10	2010-08-23 10:29:45.590958-04
131	The Stadium Indiscriminate Threatening Case (Part 2)	Neg	346mb	\N	E66D5124	\N	http://www.megaupload.com/?d=4I3MB34I	1999/Jan/18	23.2	\N	\N	\N	Vol 19 File 9 - Vol 20 File 1	2010-08-23 10:26:34.173583-04
299	Kanmon Strait of Friendship & Murderous Intent (Part 1)	Kienai	172 MB	H264	1E965555	\N	http://www.fileserve.com/file/bsazt2g	2002/Nov/04	16.3	\N	\N	\N	TV Original	2011-05-22 20:23:19.185198-04
126	The Travelling Drama Troupe Murder Case (Part 1)	Puto	225mb	H264-AC3	FBDCE79B	\N	http://www.megaupload.com/?d=II5YWNXI	1998/Nov/30	20.1	\N	\N	on	TV Original	2011-01-16 22:03:55.068245-05
250	The Idol Stars' Secret (Part 2)	AZFS	172mb	XviD	8B0B852B	Join #AZFS and type /msg AZFS|Kudo XDCC send 15	http://www.megaupload.com/?d=ICZYP72K	2001/Sep/10	20.9	\N	\N	\N	Vol 32 File 5-7	2010-08-23 10:28:32.785749-04
479	Three Days with Hattori Heiji (2 Hour Special)	DCTP	700mb	\N	E4400147	\N	http://www.megaupload.com/?d=6M9PQWGD	2007/Jul/16	10.5	\N	\N	\N	Vol 54 File 4 - Vol 55 File 2	2010-08-23 10:32:10.79393-04
497	Clash of Red and Black: Awakening	DCTP	172mb | 233mb	\N	F8C62E12 | 8D7E2EBB	\N	http://www.megaupload.com/?d=D4X7GBKH | 720: http://www.megaupload.com/?d=QFV1GTNZ	2008/Feb/25	9.8	\N	\N	\N	Vol 57 File 9 - Vol 58 File 2	2010-08-23 10:32:29.69593-04
551	The Culprit is Genta's Dad (Part 1)	DCTP	171mb	XviD	98C947A7	\N	http://www.megaupload.com/?d=024YEW54	2009/Oct/17	\N	\N	\N	\N	Vol 63 File 6-8	2010-08-23 10:33:16.489793-04
247	The Mystery in the Net (Part 2)	DCTP	347mb	XviD	5F7ECB81	\N	http://www.megaupload.com/?d=BZJEMGS9	2001/Aug/20	19.6	\N	Part 1 is concatenated.	\N	Vol 31 File 5-7	2010-08-23 10:28:30.50127-04
430	Two People Who Can't Return Yet (Part 2)	Kaizou	176mb	\N	C84B3664	\N	http://www.megaupload.com/?d=4GUWAEIM	2006/Feb/13	9.7	\N	\N	\N	Vol 49 File 8-10	2010-08-23 10:31:25.598542-04
340	Hidden Bathroom Secret (Part 1)	DCTP	174mb	\N	C65D4487	\N	http://www.megaupload.com/?d=4DNQRACJ	2003/Nov/03	12.1	\N	\N	\N	Vol 41 File 10 - Vol 42 File 1	2010-08-23 10:30:00.313792-04
473	Adventure of Young Kudo Shinichi (Part 2)	DCTP	172mb	\N	3BF6B43B	\N	http://www.megaupload.com/?d=UJYJ1GCQ	2007/May/21	11	\N	\N	\N	Vol 55 File 6-9	2010-08-23 10:32:05.961505-04
454	Upsetting Outcome (Part 1)	anfs	\N	XviD	D1F3746D	\N	http://www.megaupload.com/?d=7ZFFTK3Q	2006/Oct/30	8.2	\N	\N	\N	Vol 52 File 6-8	2010-08-23 10:31:42.689103-04
349	Love, Ghosts, and World Heritage (Part 2)	DCTP	350mb	XviD	E6388A1A	\N	http://www.megaupload.com/?d=ZXWEZ40H	2004/Feb/02	13.4	\N	v2 available. Part 1 is concatenated.	\N	TV Original	2010-08-23 10:30:08.610786-04
254	The Metropolitan Police Detective Love Story 4 (Part 2)	Kaizou	173mb	\N	6FC400C1	\N	http://www.megaupload.com/?d=HEH3W1GB	2001/Oct/22	20.5	\N	\N	\N	Vol 32 File 11 - Vol 33 File 2	2010-08-23 10:28:35.90777-04
572	Battle of the Haunted Warehouse's Treasure (Part 2)	DCTP	172mb | 321mb	XviD | H264	916F59D1 | 6E5E1655	\N	http://www.megaupload.com/?d=WWYS9QQ0 | 720: http://www.megaupload.com/?d=E1B29BNU	2010/May/08	\N	\N	\N	\N	Vol 66 File 5-7	2010-08-23 10:33:43.494472-04
575	The Alibi of the Black Dress (Part 1)	DCTP	171mb | 320mb	XviD | H264	5E3C28F1 | 95D00067	\N	http://www.megaupload.com/?d=LK6B7XP2 | 720: http://www.megaupload.com/?d=ZU71OL2N	2010/May/29	\N	\N	\N	\N	Vol 66 File 11 - Vol 67 File 2	2010-08-23 10:33:45.886596-04
206	The Metropolitan Police Detective Love Story 3 (Part 2)	Neg	172mb	\N	A80FD81B	\N	http://www.megaupload.com/?d=EXGSMYFO	2000/Sep/04	18.6	\N	\N	\N	Vol 27 File 4-6	2010-08-23 10:27:43.279629-04
325	The Red Horse within the Flames (Part 1: The Case)	Kienai	172mb	\N	C179102B	Join #Kienai and type /msg Sonoko-Chan XDCC send 14	http://www.megaupload.com/?d=UYHL1FR3	2003/Jun/23	16.1	\N	\N	\N	Vol 39 File 1-5	2010-08-23 10:29:47.063643-04
458	Sonoko's Red Handkerchief (Part 2)	Kaizou	173mb	\N	9B4BC73A	\N	http://www.megaupload.com/?d=UZT5PLW7	2006/Nov/27	8.2	\N	\N	\N	Vol 52 File 9-11	2010-08-23 10:31:45.481543-04
10013	The Raven Chaser	DCTP	700mb | 700mb | 1.4gb	XviD	65983713 | 8D95E707 | EFC07D98	\N	XviD Part 1: http://www.megaupload.com/?d=Y1DH1LPI :: XviD Part 2: http://www.megaupload.com/?d=TZRWH16L	2009/Apr/18	\N	A new member from the Black Organization that shrunk Shinichi's body manages to find out about Shinichi's transformation into Conan. This discovery starts to put those around him in danger as Gin and the other BO members starts to take action by eliminating Shinichi and his friends. Has time finally run out for Shinichi? And can he prevail against the enemies who have come to finish the job they started?	H264-AC3: [multipart rar] http://www.megaupload.com/?d=KU19K9MJ http://www.megaupload.com/?d=NUBUQY12 http://www.megaupload.com/?d=WNXB4VTZ	\N	\N	2010-11-21 15:12:42.405327-05
576	The Alibi of the Black Dress (Part 2)	DCTP	173mb | 320mb	XviD | H264	CFFEBCAE | 16A5EA2A	\N	http://www.megaupload.com/?d=V1UYPM27 | 720: http://www.megaupload.com/?d=BDOX810C	2010/Jun/5	\N	\N	\N	\N	Vol 66 File 11 - Vol 67 File 2	2010-08-23 10:33:46.776713-04
65	A Crab and Whale Kidnapping Case	AConan	172mb	\N	74840D91	\N	http://www.megaupload.com/?d=9R1RWHFA	1997/Jun/30	17.3	\N	\N	\N	TV Original	2010-08-23 10:25:29.065869-04
357	Sweetheart is an Illusion of Spring	DCTP	171mb	XviD	BBCA9E2E	\N	http://www.megaupload.com/?d=RQPGK4QX	2004/Apr/26	12.4	\N	\N	\N	TV Original	2010-08-23 10:30:15.492021-04
354	A Small Client (Part 1)	DCTP	172mb	\N	66FF93DB	\N	http://www.megaupload.com/?d=ZOCGC4G0	2004/Mar/08	14.1	\N	\N	\N	Vol 39 File 9-11	2010-08-23 10:30:12.858018-04
564	Detective Boys VS Robber Group (Part 2: Silence)	DCTP	172mb | 320mb	XviD	36493561 | 2003310D	\N	http://www.megaupload.com/?d=1TLAT5A6 | 720: http://www.megaupload.com/?d=PC1TDCA9	2010/Jan/30	\N	\N	\N	\N	\N	2010-08-23 10:33:37.072787-04
546	The Witch Enshrouded in Fog (Part 2)	DCTP	172mb	XViD	2E4E7047	\N	http://www.megaupload.com/?d=R54L2O22	2009/Sept/12	8.7	\N	\N	\N	Vol 63 File 9-11	2010-08-23 10:33:12.717669-04
10012	Full Score of Fear	DCTP	1.4gb | 700mb | 1.4gb	XviD	AF83B2E0 | 52DC8F36 | BA79F197	\N	\N	2008/Apr/19	\N	Serial murders involving all kinds have happened. All the victims are from a Music School led by a famous pianist. Conan and company have been invited to a opening concert of the Music Hall built by the pianist. The biggest attraction of this concert is the world famous violin called "Stradivarius" and a special appearance of a singer who has perfect pitch.	\N	\N	\N	2010-05-17 17:13:51.278958-04
418	Home of Beika's Grenier	DCTP	173mb	\N	8DE508E2	\N	http://www.megaupload.com/?d=M79LYD6D	2005/Oct/31	11	\N	\N	\N	TV Original	2010-08-23 10:31:17.036321-04
573	The Whereabouts of the Embarrasing Charm (Part 1)	DCTP	171mb | 321mb	XviD | H264	1C686E18 | CD729068	\N	http://www.megaupload.com/?d=YMCYJ9G8 | 720: http://www.megaupload.com/?d=9Y5F693A	2010/May/15	\N	\N	\N	\N	Vol 66 File 8-10	2010-08-23 10:33:44.435177-04
20003	Conan and Heiji and the Vanished Boy	DCTP	234mb	XviD	10BCA14F	\N	\N	\N	\N	Ran and Conan have been invited to visit Osaka by Heiji and Kazuha. Once there, they run across three children - Mayumi, Mikihiko, and Kenta - who have an uncanny resemblance to Ayumi, Mitsuhiko, and Genta. The trio is searching for their friend Satoshi who has gone missing, and they want Heiji's help to find him. Soon, our detectives discover that Satoshi's disappearance may have something to do with a bank robbery that happened earlier in the day. It's a race against the clock to save Satoshi before it's too late!	\N	\N	\N	2010-05-17 17:13:51.278958-04
256	The 14th Round of the Matsue Tamatsukuri Linked Verse Contest (Part 2)	AZFS	172mb	XviD	5221C707	Join #AZFS and type /msg AZFS|Kudo XDCC send 19	http://www.megaupload.com/?d=6F5YCKRN	2001/Nov/05	18.6	\N	\N	on	TV Original	2010-11-23 23:42:37.26325-05
510	Conan VS W-Code Mystery	DCTP	172mb | 485mb	\N	0F65CA68 | 358CC177	\N	http://www.megaupload.com/?d=S60I5L7I	2008/Jul/28	8.5	\N	\N	\N	Vol 61 File 8-10	2010-08-23 10:32:45.726927-04
488	The Devil of the TV Station (1 Hour Special)	DCTP	351mb	XviD	E1AF0CC1	\N	http://www.megaupload.com/?d=ZXUV0EV1	2007/Oct/22	9.4	\N	\N	\N	Vol 57 File 5-7	2010-08-23 10:32:22.521353-04
447	The Sealed Western-style Window (Part 2)	Kaizou	176mb	\N	FFEEF6B8	\N	http://www.megaupload.com/?d=CFTF18XG	2006/Jul/31	8.4	\N	\N	\N	Vol 51 File 9-11	2010-08-23 10:31:37.757921-04
594	The Seven Wonders of the Hiroshima Miyajima Tour (Part 1: Miyajima)	DCTP	172mb | 320mb	XviD | H264	E83101ED | 1E82699E	\N	http://www.megaupload.com/?d=K4L6BO13 | 720: http://www.megaupload.com/?d=B997VF5C	2010/Nov/06	\N	\N	\N	on	TV Original	2010-11-10 15:28:27.073814-05
543	Ikkaku Rock's Disappearing Fish (Part 2)	DCTP	172mb | 341mb	XviD	5E49558B | 569A7A36	\N	http://www.megaupload.com/?d=7JVO4TJF | 720: http://www.megaupload.com/?d=0J2B7PGM	2009/August/1	5.9	\N	\N	\N	Vol 64 File 1-3	2010-08-23 10:33:10.66129-04
518	The Meiji Restoration Mystery Tour (Part 1: Investigation Chapter)	DCTP	172mb	\N	113F4C02	\N	http://www.megaupload.com/?d=58ZGXWIB	2008/Dec/01	8.8	\N	\N	\N	TV Original	2010-08-23 10:32:51.512744-04
536	The Secret of the Vanished Masterpiece	DCTP	172mb	XviD	35BEA4EF	\N	http://www.megaupload.com/?d=ESMJ4MKJ	2009/May/30	6.7	\N	\N	\N	TV Original	2010-08-23 10:33:05.192954-04
609	White Day of Betrayal (Part Two)	DCTP	172 MB/350 MB	XviD/H264	7FA69BC0/5AB609EF	\N	http://www.megaupload.com/?d=BNHLKC7W | 720: http://www.megaupload.com/?d=A8A3OU2C	\N	\N	\N	\N	\N	\N	2011-03-27 15:25:41.896697-04
581	The Red Shaking Target	DCTP	172mb | 320mb	XviD | H264	01139EB3 | 4414B661	\N	http://www.megaupload.com/?d=3V6W0JPB | 720: http://www.megaupload.com/?d=Q8NZCMRQ	2010/Jul/17	\N	\N	\N	\N	Vol 67 Files 4-8	2010-08-23 10:33:51.844136-04
234	The Evidence That Didn't Disappear (Part 2)	AZFS	174mb	XviD	0BFB42BC	Join #AZFS and type /msg AZFS|Kudo XDCC send 10	http://www.megaupload.com/?d=K5GCRA2C	2001/May/21	19.5	\N	\N	\N	Vol 29 File 6-8	2010-08-23 10:28:10.365778-04
58	Holmes Freak Murder Case (Part 2)	AConan	172mb	\N	818D5D25	\N	http://www.megaupload.com/?d=QIUOTS9K	1997/May/12	16.5	\N	\N	\N	Vol 12 File 7 - Vol 13 File 1	2010-08-23 10:25:23.915985-04
582	The Night the Zombie Died	DCTP	173mb | 320mb	XviD | H264	8F0914FD | BE21C825	\N	http://www.megaupload.com/?d=MELJGJVH | 720: http://www.megaupload.com/?d=P4FFR9UC	2010/Jul/24	\N	\N	\N	on	TV Original	2010-08-23 10:33:52.664468-04
583	Kobayashi-sensei's Love	DCTP	172mb | 320mb	XviD | H264	C4D3DE4A | 711C3A8F	\N	http://www.megaupload.com/?d=SC2LZ2T3 | 720: http://www.megaupload.com/?d=OMNVFO02	2010/Aug/14	\N	\N	\N	\N	Vol 67 File 9 - Vol 68 File 1	2010-08-23 10:33:53.528165-04
584	Inspector Shiratori's Lost Love	DCTP	173mb | 320mb	XviD | H264	B9BF3C0B | 51C5C7A8	\N	http://www.megaupload.com/?d=FDQL2AQ1 | 720: http://www.megaupload.com/?d=J2XQ80GJ	2010/Aug/21	\N	\N	\N	\N	Vol 67 File 9 - Vol 68 File 1	2010-08-23 10:33:55.156019-04
30004	Magic File 4 (Movie 14 Side-Story OVA)	DCTP	230mb | 253mb	XviD | H264	E61E3B41 | 6C982512	\N	http://www.megaupload.com/?d=VMJVLHYO | Alt: http://www.megaupload.com/?d=0KKCXUM6	\N	\N	\N	please yell at simcop2387 if any info is wrong	\N	TV Original	2010-08-23 10:37:08.902321-04
275	Truth About the Haunted House (Part 2)	DCTP-AZFS	350mb	XviD	2E75803F	Join #AZFS and type /msg AZFS|Kudo XDCC send 25	http://www.megaupload.com/?d=2GHYKWF5	2002/Apr/22	18.3	\N	Two parter, contains 274 also	\N	Vol 35 File 5-7	2010-12-01 14:46:39.286338-05
262	The Fearful Legend of the Snowy Night (Part 2)	Kienai-AZFS	172mb	h264	47F6A339	Join #AZFS and type /msg AZFS|Kudo XDCC send 23	http://www.megaupload.com/?d=5EGFXYOB	2001/Dec/17	18.8	\N	\N	\N	TV Original	2010-08-23 10:28:45.301613-04
498	Clash of Red and Black: Disturbance	DCTP	172mb | 233mb	\N	D01E9532 | 6AD78584	\N	http://www.megaupload.com/?d=7P786YUI | 720: http://www.megaupload.com/?d=YQ9OGARV	2008/Mar/03	10	\N	\N	\N	Vol 58 File 3 - Vol 58 File 7	2010-08-23 10:32:30.497966-04
311	Contact with the Black Organization (Part 3: The Desperation)	Kaizou	212mb	\N	04177903	\N	http://www.megaupload.com/?d=88XPXWAC	2003/Feb/24	16.4	\N	\N	\N	Vol 37 File 8 - Vol 38 File 1	2010-08-23 10:29:35.620714-04
288	Kudo Shinichi's NY Case (Part 3)	Kaizou	226mb	\N	C2A293F7	\N	http://www.megaupload.com/?d=380DRGD2	2002/Jul/29	15.8	\N	\N	\N	Vol 34 File 11 - Vol 35 File 4	2010-08-23 10:29:16.885998-04
574	The Whereabouts of the Embarrasing Charm (Part 2)	DCTP	171mb | 320mb	XviD | H264	E8E0AB2D | 34B845E5	\N	http://www.megaupload.com/?d=FKR29HAI | 720: http://www.megaupload.com/?d=QWT4VKO5	2010/May/22	\N	\N	\N	\N	Vol 66 File 8-10	2010-08-23 10:33:45.130511-04
332	Suspicious Curry (Part 2)	AZFS	139mb	XviD	82027A13	Join #AZFS and type /msg AZFS|Kudo XDCC send 29	http://www.megaupload.com/?d=XWST5AIF	2003/Aug/18	15	\N	\N	\N	Vol 40 File 4-6	2010-08-23 10:29:52.846418-04
598	The Scenario of the Steaming Locked Room (Part 2)	DCTP	172mb | 320mb	XviD | H264	EE6E0333 | BB0BEFBE	\N	http://www.megaupload.com/?d=0CMN50M8 | 720: http://www.megaupload.com/?d=YPHSH72V	2010/Dec/04	\N	\N	\N	\N	Vol 69 File 4-6	2010-12-10 17:40:51.616988-05
270	The Forgotten Memento from the Crime (Part 2)	Kienai	172mb	h264	11AE90F3	Join #Kienai and type /msg Sonoko-Chan XDCC send 8	http://www.megaupload.com/?d=WPO5MIAD	2002/Mar/04	19.7	\N	\N	\N	Vol 33 File 7-9	2010-08-23 10:28:53.228766-04
367	The Tradegy of the Pier in Plain Sight (Part 2)	KnKF	176mb	\N	7A27E55A	\N	http://www.megaupload.com/?d=ZK7RQE7L	2004/Jul/12	13	\N	\N	\N	Vol 45 File 3-5	2010-08-23 10:30:23.068367-04
222	And Then There Were No Mermaids (Part 1: The Murder)	DCTP	344mb	\N	1388584C	\N	http://www.megaupload.com/?d=VVHLEWU5	2001/Jan/29	19.2	\N	\N	\N	Vol 28 File 6-10	2010-08-23 10:27:56.020954-04
345	Head-to-Head Match With Black Organization, Two Mysteries on the Night of a Full Moon (2.5 Hour Special)	Kaizou	698mb | 698mb	\N	48D4549B | 1BF6B09E	\N	http://www.megaupload.com/?d=DNE6LPTU | http://www.megaupload.com/?d=LAOOK4BH	2004/Jan/05	16.7	\N	\N	\N	Vol 42 File 5-10	2010-08-23 10:30:05.182644-04
597	The Scenario of the Steaming Locked Room (Part 1)	DCTP	172mb | 320mb	XviD | H264	1C171D92 | 14502DDA	\N	http://www.megaupload.com/?d=4W657AYB | 720: http://www.megaupload.com/?d=4VHHCIU1	2010/Nov/27	\N	\N	\N	\N	Vol 69 File 4-6	2010-12-04 13:19:23.918037-05
595	The Seven Wonders of the Hiroshima Miyajima Tour (Part 2: Hiroshima)	DCTP	172mb | 320mb	XviD | H264	2E47E0E2 | 68E4988D	\N	http://www.megaupload.com/?d=GVT6DKER | 720: http://www.megaupload.com/?d=UH7MJ7E0	2010/Nov/13	\N	\N	\N	on	TV Original	2010-11-20 12:31:02.511687-05
76	Conan VS Kaitou Kid (1 Hour Special)	AConan	344mb	\N	1D551AE9	\N	http://www.megaupload.com/?d=JSBD0J0M	1997/Sep/22	21.7	\N	\N	\N	Vol 16 File 6-9	2010-08-23 10:25:37.25759-04
596	The Alibi of the Fall	DCTP	172mb | 320mb	XviD | H264	7B5353ED | 6EA6D9A5	http://www.fileserve.com/file/EezFsby | 720: http://www.fileserve.com/file/X8sF33N	http://www.megaupload.com/?d=US4YSTLV | 720: http://www.megaupload.com/?d=M9CQARCW	2010/Nov/20	\N	\N	\N	on	TV Original	2010-11-14 13:13:43.637628-05
616	Holmes's Revelation (Holmes's Apprentice)	DCTP	172 MB/320 MB	XviD/H264	3729831C/C1902CFC	\N	[SD] http://www.megaupload.com/?d=TL5C2LJ0 || [HD] http://www.megaupload.com/?d=RNRQEA22	\N	\N	\N	\N	\N	\N	2011-05-24 17:19:34.625313-04
617	Holmes's Revelation (Love Is 0)	DCTP	172 MB/350 MB	XviD/H264	C7E7101A/2AEFE8E8	\N	[SD] http://www.megaupload.com/?d=ZUGCHZHU || [HD] http://www.megaupload.com/?d=7XDQ8XMG	05/29/11	\N	\N	\N	\N	\N	2011-05-29 23:59:12.942907-04
618	Holmes's Revelation (Satan)	DCTP	171mb | 320mb	XviD | H264	AC8F5A74 | F9F50F17	\N	http://www.fileserve.com/file/eHskkAV | 720: http://www.fileserve.com/file/66GMpWZ	6/4/11	\N	Revelation 	\N	\N	\N	2011-06-11 13:52:07.681776-04
\.


--
-- Data for Name: groupaliases; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY groupaliases (gid, alias) FROM stdin;
\.


--
-- Data for Name: groupinfo; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY groupinfo (gid, fullname, releases, alive, mergeddctp) FROM stdin;
\.


--
-- Data for Name: ignores; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY ignores (nick, message) FROM stdin;
child_of_eve	/notice child_of_eve You is the ignored
\.


--
-- Data for Name: quotes; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY quotes (qid, sid, quote, who, deleted, counter) FROM stdin;
45	6	Check out all my quotes at http://farnsworth.sexypenguins.com/test/quotes.html	simcop2387-vnc	t	22
21	2	He is aiding a mass-murderer in remaining undercover	Shin-Chan	f	18
-20018	-20018	15,01[11OVA1815,01]	\N	f	0
104	11	<Sato ep 431> Yumi, where are you going, i can't handle all these guys by myself.	simcop2387	f	34
39	14	The 4869 in Apotoxin 4869 comes from the number of 69's that Shiho/Ai had had up to the point of developing the drug	simcop2387-lab	f	66
35	1	The complete name of the poison given to Conan and taken by Haibara is Apotoxin 4869	simcop2387-lab	f	66
11	2	Underage drinking	simcop2387	f	17
1643	25	Bars deter cuckoo bird	simcop2387	f	1
42	1	In Detective Conan, there is an actual character named Conan	conankudo	t	0
49	9	ECIEVES SOMETHING LIKE YOU ON THE BIRTHDAY	simcop2387	t	0
64	2	......	simcop2387	t	0
46	3	Jd thinks Zdm321 is second best!	conankudo|secret	t	0
71	12	Aoko is actually pregnant but doesn't want to tell kaito because she wants to get more money out of him in court	simcop2387	f	36
47	3	Jd thinks Zdm321 is not the worst!	conankudo|secret	t	0
28	2	Attempted suicide [Cyanide filled bun]	Rellik	f	17
107	666	189dc.DLL	simcop2387	f	1
108	666	9xmig.dll	simcop2387	f	1
109	666	ACMCMPRS.DLL	simcop2387	f	1
110	666	ACMWrapperV2.dll	simcop2387	f	0
111	666	ACTXPRXY.DLL	simcop2387	f	1
112	666	ADVAPI32.DLL	simcop2387	f	1
113	666	ADVPACK.DLL	simcop2387	f	1
114	666	AIImport.dll	simcop2387	f	1
115	666	AI_EPS_PDF_Import.dll	simcop2387	f	1
116	666	ASWHook.dll	simcop2387	f	0
117	666	ASYCFILT.DLL	simcop2387	f	1
20	2	He has driven at least one person to suicide.	Shin-Chan	f	18
14	2	At least one count of physically abusing a dog	simcop2387	f	18
27	2	Nearly causing the death of Mouri Kogoro	Rellik	f	18
9	2	Probably causing severe long term medical issues from forcing Mouri Kogoro to abuse sedatives.	simcop2387	f	18
58	3	normally Tech > Raj > Jd, but in terms of editing, Jd > Tech > Raj	conankudo	f	34
72	12	Shinichi/Conan seems to have been put under an amnesia spell by Kaito Kid, as he can't seem to remember the first time they encountered	kyuuketsuki	f	29
19	2	Obstructing justice	Juansmarts	f	18
13	2	Operating a motorized vehicle while underage	simcop2387	f	18
63	14	Heiji and Kaito would be BBF if it weren't for Kudo tearing them apart	haruka-chan	f	66
65	2	Conan has commited several acts of invasion of privacy	kyuuketsuki	f	18
43	7	There is almost no one in the DCTP staff with an SN relating to DC	conankudo	f	42
92	3	Jd has a boyfriend	alfonsina	f	33
31	2	Underaged and unlicensed use and possession of a firearm	Xytan & sdf1macross	f	18
26	2	Impersonating police officers	Rellik	f	16
29	2	Outright theft (ran's valentine chocolate, admittedly made for him but he still stole it)	Rellik	f	18
62	3	Jd is some kind of transformer from outerspace	haruka-chan	f	33
55	1	Cases from 163/164 and 54 have the greatest episode gap between two cases that occur in the same volume in the manga (99 episodes apart)	conankudo	f	69
17	2	Sexual harassment	simcop2387	f	18
53	7	One day, DCTP will gain enough support to rule the world WITH AN IRON FIST!	conankudo	f	30
68	12	despite kaito kids fatal weakness, no one has figured out that all you need to stop him is some fish or fish pictures	kyuuketsuki	f	28
7	2	Constantly lying to his girlfriend about his whereabouts.	simcop2387	f	17
8	2	Defrauding the police by pretending to be Mouri Kogoro.	simcop2387	f	18
15	2	Doing drugs!	simcop2387	f	18
48	2	Try again, but have a look at http://forums.dctp.ws/index.php?topic=798.0 for the discussion on the forums	simcop2387	f	18
69	11	<Jd> I swear I read that as "OMG my inbred fails"	simcop2387	f	34
67	14	Conan's head is so hard that Kaito Kid once used it to smash bullet proof glass to steal a diamond	simcop2387	f	66
6	3	JD has infact quit subbing conan at least 67 distinct times but for some reason keeps coming back	Zdm321	f	33
118	666	ATL.DLL	simcop2387	f	1
119	666	ATL80.dll	simcop2387	f	1
120	666	AVICAP.DLL	simcop2387	f	1
25	5	See my source at https://vdiddy.sexypenguins.com/subversion/quotebot/ it's currently not documented though.	simcop2387	t	9
-20019	-20019	15,01[11OVA1915,01]	\N	f	0
94	11	<Akai> Shop smart. Shop S-Mart	simcop2387	f	34
86	3	Jd rove conan rong timu	simcop2387	f	33
16	2	Probably hundreds of counts of assault 	simcop2387	f	19
122	666	Accessibility.dll	simcop2387	f	1
123	666	AccessibleMarshal.dll	simcop2387	f	0
124	666	Adaptoid_v1_0.dll	simcop2387	f	0
125	666	AxInterop.ShockwaveFlashObjects.dll	simcop2387	f	1
126	666	AziAudio.dll	simcop2387	f	0
127	666	BINDFILE.DLL	simcop2387	f	0
128	666	BROWSELC.DLL	simcop2387	f	0
129	666	BROWSEUI.DLL	simcop2387	f	1
130	666	BROWSEWM.DLL	simcop2387	f	1
131	666	BgImage.dll	simcop2387	f	0
132	666	Bonobo_module.dll	simcop2387	f	0
133	666	Boo.Lang.CodeDom.dll	simcop2387	f	0
134	666	Boo.Lang.Compiler.dll	simcop2387	f	1
135	666	Boo.Lang.Extensions.dll	simcop2387	f	0
136	666	Boo.Lang.Interpreter.dll	simcop2387	f	0
137	666	Boo.Lang.Parser.dll	simcop2387	f	0
138	666	Boo.Lang.Useful.dll	simcop2387	f	0
139	666	Boo.Lang.dll	simcop2387	f	1
140	666	Boo.NAnt.Tasks.dll	simcop2387	f	1
141	666	ByteFX.Data.dll	simcop2387	f	1
142	666	CARDS.DLL	simcop2387	f	1
143	666	CDEngine.dll	simcop2387	f	0
144	666	CEWMDM.dll	simcop2387	f	1
145	666	CNV_ENUS.DLL	simcop2387	f	0
146	666	COMCAT.DLL	simcop2387	f	1
147	666	COMCTL32.DLL	simcop2387	f	0
148	666	COMDLG32.DLL	simcop2387	f	0
149	666	COMMDLG.DLL	simcop2387	f	0
150	666	COMPOBJ.DLL	simcop2387	f	0
151	666	COOL.DLL	simcop2387	f	0
152	666	CORPOL.DLL	simcop2387	f	0
153	666	CRTDLL.DLL	simcop2387	f	0
33	6	Add a random fact about Jd. Do, !addquote RJDF Fact Goes here	simcop2387	f	29
66	11	<Detective> it's really hard	simcop2387	f	33
93	11	<Akai> THIS IS MY BOOM STICK!	simcop2387	f	34
84	2	Destruction of public property	kirite	f	18
57	1	Kogoro's voice is #59 on the Conan bowtie voice modulator	conankudo	f	69
97	11	<Theirah> how do I get to zdm from here? <Zdm321> hi	simcop2387	f	34
85	6	Add some random fiction about Conan. do, !addquote RCFICT Fiction goes here	simcop2387	f	30
82	2	Paragliding from a skyscraper (ALSO underaged for paragliding i believe)	Detective	f	18
83	2	Surfing/skateboarding on the top of a train and roller coaster. (also underaged for that!)	Detective & Kirite	f	17
88	2	conan or shinichi also truanting school on several occasions	ganymedie	f	18
96	2	Stunning random people who get in his way!	detective	f	18
90	3	Jd likes caps so much that he'd kill Bill Maher to get them	Ryunichi	f	34
77	7	if there were to be an episode released with heiji kissing a boy, any boy at all, for any reason at all, it would forever be haruka-chan's favorite and the picture of it would end up burned into her monitor from it being on there 24/7	simcop2387	f	37
80	2	Numerous counts of stalking	kyuuketsuki	f	17
37	14	Megure voted for Nader in 2000	Jd	f	66
74	13	The DDL went down because iZ` tried to download it all	simcop2387	f	38
105	9	<conan> DESERVE A RED CARD FOR A LOSER	simcop2387	f	37
18	2	Numerous counts of trespassing and breaking and entering	Juansmarts	f	19
89	3	His name is Jed, Jed Dinks.	Resident	f	34
22	3	Jd wants to have Bill Maher's babies	simcop2387	f	34
1694	11	[1:01pm] emkuf-drunk is now known as emkuf-hungOver.	Jing	f	34
76	3	Jd is a big meanie, he doesn't care about the AO's	simcop2387	f	33
10	2	Encouraging elementary school students to become vigilanties. 	simcop2387	f	18
81	2	Numerous counts of invasion of privacy	kyuuketsuki	f	18
102	4	For those of you who want to know how Conan can figure out Haibara's recorder message, MOONLIGHT SONATA!	simcop2387	f	48
91	11	<Joel Hodgeson> There's nothing more tragic than a war in an Eskimo village. 	simcop2387	f	34
87	14	There is no such way to say "Dying Message" in Japanese, that's why everyone says it in English.	simcop2387	f	66
23	1	Episodes 107 and 108 were subtitled into english by 5 different groups: XCHKS, HnI, Mystery-Otaku, Afreaky, and Atavus	Zdm321	f	70
1620	9	Any line from Detective Conan - 443 [anfs]	Zdm321	f	37
41	3	Jd thinks Zdm321 is the best!	Zdm321	f	34
75	3	Jd in fact looks like Boris Karloff and he doesn't even care	simcop2387	f	33
59	1	chapter 594 is the shortest separate case in DC, consisting of only 1 chapter and has not as of yet been turned into an anime episode	conankudo	f	67
79	2	Endangering and Contributing to the deliquency of minors (again with the detective boys!)	x64_02	f	18
95	4	<Haibara Ai> Now he's sending shivers down my spine for a different reason.	simcop2387	f	39
99	14	Ran likes little boys.	Zdm321	f	66
5	1	Conan is in fact NOT named Jimmy Kudo	Zdm321 (according to Jd)	f	69
30	3	Jd would also wouldn't mind having Maddox's babies, but would instead prefer to just use Maddox for his own pleasure.	simcop2387	f	34
32	6	Add a random fact about Conan. do, !addquote RCF Fact Goes here	simcop2387	f	41
36	1	Gin drives a black Porsche 356 A 	simcop2387-lab	f	68
154	666	CRYPT32.DLL	simcop2387	f	0
155	666	CRYPTDLG.DLL	simcop2387	f	0
156	666	CRYPTEXT.DLL	simcop2387	f	1
157	666	CRYPTNET.DLL	simcop2387	f	0
159	666	CSPMAN.DLL	simcop2387	f	0
160	666	CSSEQCHK.DLL	simcop2387	f	0
161	666	CTCCW.DLL	simcop2387	f	0
162	666	CTL3D.DLL	simcop2387	f	0
163	666	CTL3DV2.DLL	simcop2387	f	0
165	666	CTREMOTE.DLL	simcop2387	f	0
166	666	CTRES.DLL	simcop2387	f	0
167	666	CTWFLT.DLL	simcop2387	f	0
168	666	Cdbootable.dll	simcop2387	f	0
1644	25	SkedAddled	simcop2387	f	0
170	666	Commons.Xml.Relaxng.dll	simcop2387	f	0
171	666	CoreTypes.dll	simcop2387	f	0
172	666	CoreUtils.dll	simcop2387	f	0
173	666	Ctl3d32.dll	simcop2387	f	0
174	666	CustomMarshalers.dll	simcop2387	f	1
175	666	DDEML.DLL	simcop2387	f	1
178	666	DISPEX.DLL	simcop2387	f	0
179	666	DSETUP.dll	simcop2387	f	0
181	666	DWINTL.DLL	simcop2387	f	0
184	666	DeviceEmulator.dll	simcop2387	f	0
185	666	EMLaunch.dll	simcop2387	f	0
186	666	EREGLIB.DLL	simcop2387	f	0
187	666	ERHttp01.dll	simcop2387	f	0
188	666	EReg3201.dll	simcop2387	f	1
190	666	EXEInstallPlugin.dll	simcop2387	f	0
191	666	EuCondPlugin.dll	simcop2387	f	1
192	666	ExportControllerPS.dll	simcop2387	f	1
260	666	IR32.DLL	simcop2387	f	1
194	666	FCLSPBW.DLL	simcop2387	f	0
195	666	FFTW2dll.dll	simcop2387	f	0
121	666	AVIFILE.DLL	simcop2387	f	1
197	666	FLiteClient.dll	simcop2387	f	0
198	666	FLiteClientRes.dll	simcop2387	f	0
199	666	FTSRCH.DLL	simcop2387	f	0
201	666	FirebirdSql.Data.Firebird.dll	simcop2387	f	0
202	666	FlashLiteOneXRes.dll	simcop2387	f	0
1646	25	Animal sullied street	simcop2387	f	0
204	666	FlashResources.dll	simcop2387	f	1
205	666	Flash_8_Video_Encoder.dll	simcop2387	f	1
207	666	Flash_8_Video_Extension_Resources.dll	simcop2387	f	0
208	666	FreeImage_Win32Release.dll	simcop2387	f	0
209	666	GAMEDG.DLL	simcop2387	f	0
210	666	GAMEFM.DLL	simcop2387	f	0
211	666	GdiPlus.dll	simcop2387	f	0
212	666	GeckoHtmlRender.dll	simcop2387	f	0
213	666	Gendarme.Framework.dll	simcop2387	f	1
214	666	Gendarme.Rules.BadPractice.dll	simcop2387	f	0
215	666	Gendarme.Rules.Concurrency.dll	simcop2387	f	0
216	666	Gendarme.Rules.Correctness.dll	simcop2387	f	0
217	666	Gendarme.Rules.Design.dll	simcop2387	f	0
218	666	Gendarme.Rules.Exceptions.dll	simcop2387	f	0
219	666	Gendarme.Rules.Interoperability.dll	simcop2387	f	0
220	666	Gendarme.Rules.Naming.dll	simcop2387	f	0
222	666	Gendarme.Rules.Portability.dll	simcop2387	f	0
223	666	Gendarme.Rules.Security.dll	simcop2387	f	1
224	666	Gendarme.Rules.Smells.dll	simcop2387	f	0
225	666	Gendarme.Rules.Ui.dll	simcop2387	f	0
226	666	GhostScript.dll	simcop2387	f	0
227	666	GtkHtmlHtmlRender.dll	simcop2387	f	0
228	666	HLINK.DLL	simcop2387	f	0
229	666	HMMAPI.DLL	simcop2387	f	0
230	666	I18N.CJK.dll	simcop2387	f	0
233	666	I18N.Rare.dll	simcop2387	f	0
234	666	I18N.West.dll	simcop2387	f	0
235	666	I18N.dll	simcop2387	f	0
236	666	IBM.Data.DB2.dll	simcop2387	f	0
237	666	ICSharpCode.SharpCvsLib.Console.dll	simcop2387	f	1
239	666	ICSharpCode.SharpZipLib.dll	simcop2387	f	1
240	666	IE4.DLL	simcop2387	f	1
176	666	DIGEST.DLL	simcop2387	f	1
241	666	IEDKCS32.DLL	simcop2387	f	0
242	666	IEMIGRAT.DLL	simcop2387	f	0
243	666	IERNONCE.DLL	simcop2387	f	0
245	666	IFFPCX.DLL	simcop2387	f	0
246	666	IFFTIFF.DLL	simcop2387	f	0
247	666	IKVM.AWT.WinForms.dll	simcop2387	f	0
249	666	IKVM.Runtime.dll	simcop2387	f	0
250	666	ILog.dll	simcop2387	f	0
251	666	IMAGEHLP.DLL	simcop2387	f	0
252	666	IMGUTIL.DLL	simcop2387	f	0
253	666	INETCPLC.DLL	simcop2387	f	0
177	666	DISPDIB.DLL	simcop2387	f	1
248	666	IKVM.OpenJDK.ClassLibrary.dll	simcop2387	f	1
200	666	FhDbRdr.dll	simcop2387	f	1
164	666	CTRCC.DLL	simcop2387	f	1
183	666	DXTRANS.DLL	simcop2387	f	1
182	666	DXTMSFT.DLL	simcop2387	f	1
189	666	ERegUI32.dll	simcop2387	f	1
180	666	DSSBASE.DLL	simcop2387	f	1
254	666	INITPKI.DLL	simcop2387	f	0
255	666	INIUPD.DLL	simcop2387	f	1
256	666	INSENG.DLL	simcop2387	f	0
258	666	INSTSCH.DLL	simcop2387	f	1
259	666	IR21_R.DLL	simcop2387	f	1
261	666	IR41.DLL	simcop2387	f	0
262	666	ISO9660.dll	simcop2387	f	0
263	666	ISRT.dll	simcop2387	f	0
264	666	IScrCnv.dll	simcop2387	f	0
265	666	IUserCnv.dll	simcop2387	f	0
266	666	IYVU9.DLL	simcop2387	f	1
267	666	Instaide.dll	simcop2387	f	0
268	666	InstallOptions.dll	simcop2387	f	0
269	666	Interop.ShockwaveFlashObjects.dll	simcop2387	f	0
158	666	CRYPTUI.DLL	simcop2387	f	1
231	666	I18N.MidEast.dll	simcop2387	f	1
206	666	Flash_8_Video_Extension.dll	simcop2387	f	1
238	666	ICSharpCode.SharpCvsLib.dll	simcop2387	f	1
270	666	Ir41_qc.dll	simcop2387	f	0
271	666	Ir41_qcx.dll	simcop2387	f	0
272	666	IronMath.dll	simcop2387	f	1
273	666	IronPython.Modules.dll	simcop2387	f	0
274	666	IronPython.dll	simcop2387	f	1
275	666	IronPythonTest.dll	simcop2387	f	0
276	666	JCalg1.dll	simcop2387	f	0
244	666	IESETUP.DLL	simcop2387	f	1
278	666	JSCRIPT.DLL	simcop2387	f	0
279	666	JSysTray.dll	simcop2387	f	0
280	666	JVM.DLL	simcop2387	f	0
281	666	Jabo_DInput.dll	simcop2387	f	0
282	666	Jabo_Direct3D6.dll	simcop2387	f	0
283	666	Jabo_Direct3D8.dll	simcop2387	f	0
284	666	Jabo_Dsound.dll	simcop2387	f	0
285	666	Joliet.dll	simcop2387	f	0
286	666	JpegReader.dll	simcop2387	f	0
287	666	LZ32.DLL	simcop2387	f	0
288	666	LZEXPAND.DLL	simcop2387	f	1
289	666	Libidn.dll	simcop2387	f	1
290	666	Library.dll	simcop2387	f	0
291	666	M5_EmuHw.dll	simcop2387	f	1
292	666	M5_EmuSmw5.dll	simcop2387	f	0
294	666	MAPIX.DLL	simcop2387	f	0
295	666	MCIOLE.DLL	simcop2387	f	0
296	666	MEDV12.DLL	simcop2387	f	0
297	666	METER.DLL	simcop2387	f	0
298	666	MFC71.dll	simcop2387	f	0
299	666	MFC71u.dll	simcop2387	f	0
300	666	MIGRATE.DLL	simcop2387	f	0
301	666	MLANG.DLL	simcop2387	f	0
302	666	MMAWARE.DLL	simcop2387	f	1
303	666	MMNotes.dll	simcop2387	f	0
304	666	MMSYSTEM.DLL	simcop2387	f	0
305	666	MMUTILSE.DLL	simcop2387	f	0
306	666	MMxptResources.dll	simcop2387	f	0
307	666	MORICONS.DLL	simcop2387	f	0
308	666	MSABC110.DLL	simcop2387	f	0
309	666	MSACM.DLL	simcop2387	f	1
311	666	MSAFINX.DLL	simcop2387	f	0
312	666	MSAJT110.DLL	simcop2387	f	0
313	666	MSAJU110.DLL	simcop2387	f	0
314	666	MSAPI.DLL	simcop2387	f	0
315	666	MSCAT32.DLL	simcop2387	f	1
316	666	MSCOMSTF.DLL	simcop2387	f	0
317	666	MSCONV97.DLL	simcop2387	f	0
318	666	MSCUISTF.DLL	simcop2387	f	0
319	666	MSDETSTF.DLL	simcop2387	f	0
320	666	MSENCODE.DLL	simcop2387	f	0
321	666	MSFTREG.DLL	simcop2387	f	0
322	666	MSHTML.DLL	simcop2387	f	0
323	666	MSHTMLED.DLL	simcop2387	f	0
324	666	MSHTMLER.DLL	simcop2387	f	0
325	666	MSIInstallPlugin.dll	simcop2387	f	0
326	666	MSINSSTF.DLL	simcop2387	f	0
327	666	MSLS31.DLL	simcop2387	f	1
328	666	MSMIXMGR.DLL	simcop2387	f	0
329	666	MSOSS.DLL	simcop2387	f	0
330	666	MSPST.DLL	simcop2387	f	0
331	666	MSR2C.DLL	simcop2387	f	0
333	666	MSRATELC.DLL	simcop2387	f	1
334	666	MSRATING.DLL	simcop2387	f	0
335	666	MSSETUP.DLL	simcop2387	f	0
336	666	MSSHLSTF.DLL	simcop2387	f	0
337	666	MSSIGN32.DLL	simcop2387	f	0
338	666	MSSIP32.DLL	simcop2387	f	0
339	666	MSSPELL.DLL	simcop2387	f	0
340	666	MSTIME.DLL	simcop2387	f	0
343	666	MSVCRT.DLL	simcop2387	f	1
293	666	MAPIU.DLL	simcop2387	f	1
345	666	MSXML.DLL	simcop2387	f	0
346	666	MSXML3.DLL	simcop2387	f	0
347	666	MSXML3A.DLL	simcop2387	f	0
348	666	MSXML3R.DLL	simcop2387	f	0
349	666	MSXMLA.DLL	simcop2387	f	0
1645	25	Libation for an affectionate puppy called Sounder	simcop2387	f	1
351	666	MVFS12.DLL	simcop2387	f	1
352	666	MVSRCH12.DLL	simcop2387	f	0
353	666	MVTTL12.DLL	simcop2387	f	0
354	666	MediaPlayerV2.dll	simcop2387	f	0
257	666	INSTRSA.DLL	simcop2387	f	1
355	666	Microsoft.Build.Engine.dll	simcop2387	f	0
356	666	Microsoft.Build.Framework.dll	simcop2387	f	0
357	666	Microsoft.Build.Tasks.dll	simcop2387	f	0
358	666	Microsoft.Build.Utilities.dll	simcop2387	f	0
359	666	Microsoft.JScript.dll	simcop2387	f	1
360	666	Microsoft.Scripting.dll	simcop2387	f	0
361	666	Microsoft.VisualBasic.dll	simcop2387	f	0
362	666	Microsoft.VisualC.dll	simcop2387	f	0
363	666	Microsoft.Vsa.dll	simcop2387	f	0
364	666	MobileErrors.dll	simcop2387	f	0
365	666	Mono.C5.dll	simcop2387	f	0
366	666	Mono.Cairo.dll	simcop2387	f	0
367	666	Mono.Cecil.Mdb.dll	simcop2387	f	0
368	666	Mono.Cecil.dll	simcop2387	f	0
342	666	MSVBVM60.DLL	simcop2387	f	1
341	666	MSUILSTF.DLL	simcop2387	f	1
332	666	MSR2CENU.DLL	simcop2387	f	1
370	666	Mono.Data.SqlSharp.DatabaseBrowser.dll	simcop2387	f	0
371	666	Mono.Data.Sqlite.dll	simcop2387	f	1
372	666	Mono.Data.SqliteClient.dll	simcop2387	f	1
1647	25	Encrypted Message: Jfcr vx qctf, Zay rum umy kcytd. E...	simcop2387	f	0
374	666	Mono.Data.Tds.dll	simcop2387	f	0
310	666	MSAES110.DLL	simcop2387	f	1
376	666	Mono.Data.dll	simcop2387	f	1
377	666	Mono.GetOptions.dll	simcop2387	f	0
378	666	Mono.Http.dll	simcop2387	f	1
380	666	Mono.Posix.dll	simcop2387	f	0
381	666	Mono.Security.Win32.dll	simcop2387	f	0
382	666	Mono.Security.dll	simcop2387	f	0
383	666	Mono.Web.dll	simcop2387	f	0
384	666	Mono.WebServer.dll	simcop2387	f	0
385	666	Mono.WebServer2.dll	simcop2387	f	0
386	666	MonoPosixHelper.dll	simcop2387	f	0
387	666	Msvcirt.dll	simcop2387	f	0
388	666	Msvcp60.dll	simcop2387	f	1
389	666	Mts2Reader.dll	simcop2387	f	1
390	666	Mts3Reader.dll	simcop2387	f	1
391	666	MtsWriter.dll	simcop2387	f	0
392	666	NAnt.CompressionTasks.dll	simcop2387	f	0
393	666	NAnt.Core.dll	simcop2387	f	1
394	666	NAnt.DotNetTasks.dll	simcop2387	f	1
395	666	NAnt.MSBuild.dll	simcop2387	f	0
396	666	NAnt.NUnit.dll	simcop2387	f	0
397	666	NAnt.NUnit1Tasks.dll	simcop2387	f	0
398	666	NAnt.NUnit2Tasks.dll	simcop2387	f	0
440	666	PictureViewerLocalized.dll	simcop2387	f	1
400	666	NAnt.VSNetTasks.dll	simcop2387	f	0
401	666	NAnt.VisualCppTasks.dll	simcop2387	f	0
402	666	NAnt.Win32Tasks.dll	simcop2387	f	0
403	666	NDoc.Core.dll	simcop2387	f	0
404	666	NDoc.Documenter.Msdn.dll	simcop2387	f	0
405	666	NDoc.ExtendedUI.dll	simcop2387	f	1
406	666	NETAPI32.DLL	simcop2387	f	0
408	666	NPSWF32.dll	simcop2387	f	0
409	666	NRage_DInput8_V2.dll	simcop2387	f	0
411	666	NUnitCore.dll	simcop2387	f	0
412	666	Novell.Directory.Ldap.dll	simcop2387	f	0
413	666	Npgsql.dll	simcop2387	f	0
414	666	OCCACHE.DLL	simcop2387	f	0
415	666	OLE2.DLL	simcop2387	f	0
416	666	OLE2CONV.DLL	simcop2387	f	0
417	666	OLE2DISP.DLL	simcop2387	f	0
418	666	OLE2NLS.DLL	simcop2387	f	0
419	666	OLE2PROX.DLL	simcop2387	f	0
420	666	OLE2THK.DLL	simcop2387	f	1
421	666	OLE32.DLL	simcop2387	f	0
422	666	OLEAUT32.DLL	simcop2387	f	0
423	666	OLECLI.DLL	simcop2387	f	0
424	666	OLECLI32.DLL	simcop2387	f	1
425	666	OLEDLG.DLL	simcop2387	f	0
426	666	OLEPRO32.DLL	simcop2387	f	0
428	666	OLESVR32.DLL	simcop2387	f	0
429	666	OS21ICON.DLL	simcop2387	f	0
430	666	Olepro32.dll	simcop2387	f	1
431	666	OpenSystem.C.dll	simcop2387	f	0
433	666	PEAPI.dll	simcop2387	f	1
434	666	PROGLIB.DLL	simcop2387	f	0
435	666	PSBASE.DLL	simcop2387	f	1
436	666	PSTOREC.DLL	simcop2387	f	1
437	666	PSTORERC.DLL	simcop2387	f	1
438	666	PTW_ENUS.DLL	simcop2387	f	0
439	666	PictureViewer.dll	simcop2387	f	0
441	666	PocketEngine.dll	simcop2387	f	0
442	666	QMIXER.DLL	simcop2387	f	0
443	666	QTJava.dll	simcop2387	f	1
444	666	QTJavaNative.dll	simcop2387	f	0
445	666	QTMLClient.dll	simcop2387	f	0
446	666	QTOControl.dll	simcop2387	f	1
447	666	QTOLibrary.dll	simcop2387	f	0
448	666	QTUIPanelControl.dll	simcop2387	f	0
449	666	QuickTime.dll	simcop2387	f	0
450	666	QuickTimeAudioSupportLocalized.dll	simcop2387	f	0
451	666	QuickTimewhoingLocalized.dll	simcop2387	f	1
452	666	QuickTimeLocalized.dll	simcop2387	f	0
454	666	QuickTimeWebHelper.dll	simcop2387	f	0
455	666	QuickTimeWebHelperLocalized.dll	simcop2387	f	0
456	666	RAVE.DLL	simcop2387	f	0
457	666	RECORDER.DLL	simcop2387	f	0
458	666	RICHED32.DLL	simcop2387	f	1
459	666	RNAPH.DLL	simcop2387	f	0
460	666	RSABASE.DLL	simcop2387	f	0
461	666	RSASIG.DLL	simcop2387	f	1
462	666	RSP.dll	simcop2387	f	0
463	666	Resources.dll	simcop2387	f	0
464	666	RiceVideo.dll	simcop2387	f	0
465	666	Riched20.dll	simcop2387	f	0
466	666	Riched32.dll	simcop2387	f	0
467	666	SCIDLL.DLL	simcop2387	f	0
468	666	SCK16THK.DLL	simcop2387	f	0
469	666	SCROBJ.DLL	simcop2387	f	0
470	666	SDL.dll	simcop2387	f	0
471	666	SDL_image.dll	simcop2387	f	0
472	666	SDL_mixer.dll	simcop2387	f	0
473	666	SDL_net.dll	simcop2387	f	1
474	666	SENDMAIL.DLL	simcop2387	f	0
475	666	SERPDRAW.DLL	simcop2387	f	1
476	666	SETUPL.DLL	simcop2387	f	1
477	666	SETUPWBV.DLL	simcop2387	f	0
478	666	SFMAN32.DLL	simcop2387	f	1
479	666	SHD401LC.DLL	simcop2387	f	0
480	666	SHDOC401.DLL	simcop2387	f	0
481	666	SHDOCLC.DLL	simcop2387	f	0
482	666	SHDOCVW.DLL	simcop2387	f	1
483	666	SHELL.DLL	simcop2387	f	0
484	666	SHELL32.DLL	simcop2387	f	1
485	666	SHLWAPI.DLL	simcop2387	f	0
486	666	SIntf16.dll	simcop2387	f	0
487	666	SIntf32.dll	simcop2387	f	1
488	666	SIntfNT.dll	simcop2387	f	1
489	666	SOFTPUB.DLL	simcop2387	f	1
490	666	STORAGE.DLL	simcop2387	f	0
491	666	ScriptingObjectModel.dll	simcop2387	f	0
492	666	SiteMapReader_1.1.dll	simcop2387	f	0
493	666	SoftwareUpdateFiles.dll	simcop2387	f	1
494	666	SoftwareUpdateFilesLocalized.dll	simcop2387	f	0
495	666	SoftwareUpdateLocalized.dll	simcop2387	f	0
496	666	Sound.dll	simcop2387	f	1
497	666	StartMenu.dll	simcop2387	f	0
536	666	Tigldt.dll	simcop2387	f	1
499	666	System.Configuration.dll	simcop2387	f	1
500	666	System.Core.dll	simcop2387	f	1
502	666	System.Data.dll	simcop2387	f	0
503	666	System.Design.dll	simcop2387	f	1
427	666	OLESVR.DLL	simcop2387	f	1
432	666	PBRUSH.DLL	simcop2387	f	1
504	666	System.DirectoryServices.dll	simcop2387	f	0
507	666	System.EnterpriseServices.dll	simcop2387	f	1
509	666	System.Messaging.dll	simcop2387	f	0
510	666	System.Runtime.Remoting.dll	simcop2387	f	0
506	666	System.Drawing.dll	simcop2387	f	1
512	666	System.Security.dll	simcop2387	f	0
513	666	System.ServiceProcess.dll	simcop2387	f	0
514	666	System.Transactions.dll	simcop2387	f	0
515	666	System.Web.Extensions.Design.dll	simcop2387	f	0
517	666	System.Web.Services.dll	simcop2387	f	1
518	666	System.Web.dll	simcop2387	f	0
519	666	System.Windows.Forms.dll	simcop2387	f	0
520	666	System.Xml.Core.dll	simcop2387	f	1
521	666	System.Xml.Linq.dll	simcop2387	f	0
522	666	System.Xml.dll	simcop2387	f	0
523	666	System.dll	simcop2387	f	1
524	666	THUMBVW.DLL	simcop2387	f	0
525	666	TOOLHELP.DLL	simcop2387	f	1
526	666	TOONRSRC.DLL	simcop2387	f	0
527	666	TRIEDIT.DLL	simcop2387	f	0
528	666	TSearchDll.dll	simcop2387	f	0
530	666	Tbl6A.dll	simcop2387	f	1
531	666	Tbl6B.dll	simcop2387	f	0
532	666	Tbl6Main.dll	simcop2387	f	0
533	666	Tests.XSP.Security.dll	simcop2387	f	1
534	666	TextureMngr.dll	simcop2387	f	0
535	666	Ti89pick.dll	simcop2387	f	1
537	666	Translator.dll	simcop2387	f	0
516	666	System.Web.Extensions.dll	simcop2387	f	1
539	666	URL.DLL	simcop2387	f	1
540	666	URLMON.DLL	simcop2387	f	0
541	666	UWAOLE.DLL	simcop2387	f	0
542	666	Udf.dll	simcop2387	f	1
543	666	Uninst.dll	simcop2387	f	0
544	666	VB6STKIT.DLL	simcop2387	f	1
545	666	VBDB300.DLL	simcop2387	f	0
546	666	VBOA300.DLL	simcop2387	f	0
547	666	VBRUN300.DLL	simcop2387	f	0
548	666	VER.DLL	simcop2387	f	0
549	666	VERSION.DLL	simcop2387	f	0
551	666	W32SCOMB.DLL	simcop2387	f	0
552	666	W32SKRNL.DLL	simcop2387	f	0
553	666	W32SYS.DLL	simcop2387	f	0
554	666	W95INF16.DLL	simcop2387	f	0
556	666	WARPICON.DLL	simcop2387	f	0
557	666	WAVMIX16.DLL	simcop2387	f	0
558	666	WEANETR.DLL	simcop2387	f	0
559	666	WFLT0001.DLL	simcop2387	f	0
560	666	WFLT0002.DLL	simcop2387	f	1
561	666	WFLT0006.DLL	simcop2387	f	0
562	666	WFLT0007.DLL	simcop2387	f	0
563	666	WFLT0011.DLL	simcop2387	f	0
564	666	WFLT0200.DLL	simcop2387	f	1
565	666	WIN32S16.DLL	simcop2387	f	0
566	666	WIN87EM.DLL	simcop2387	f	0
567	666	WING.DLL	simcop2387	f	0
568	666	WING32.DLL	simcop2387	f	1
570	666	WININET.DLL	simcop2387	f	1
571	666	WINMM.DLL	simcop2387	f	0
572	666	WINMM16.DLL	simcop2387	f	1
573	666	WINTRUST.DLL	simcop2387	f	0
550	666	W32N50.DLL	simcop2387	f	1
575	666	WMAMgr.dll	simcop2387	f	0
576	666	WPS_ENUS.DLL	simcop2387	f	0
577	666	WSOCK32.DLL	simcop2387	f	1
578	666	WaveletReader.dll	simcop2387	f	0
579	666	WonDLL.dll	simcop2387	f	1
581	666	XX_MDR.DLL	simcop2387	f	0
583	666	XX_MDRAG.DLL	simcop2387	f	1
584	666	XX_MDRIS.DLL	simcop2387	f	0
585	666	XX_MDRMT.DLL	simcop2387	f	1
586	666	XX_MDRPA.DLL	simcop2387	f	1
587	666	XX_MDRS1.DLL	simcop2387	f	0
588	666	XX_MDRS2.DLL	simcop2387	f	0
589	666	XX_MDRSB.DLL	simcop2387	f	0
590	666	XX_MDRT3.DLL	simcop2387	f	0
591	666	XX_MDRYM.DLL	simcop2387	f	1
592	666	ZDataI51.dll	simcop2387	f	0
593	666	Zilmar_Audio.dll	simcop2387	f	0
594	666	_ISREG32.DLL	simcop2387	f	0
595	666	_ISRES1033.dll	simcop2387	f	0
596	666	_IsRes.dll	simcop2387	f	0
597	666	_WUTL951.DLL	simcop2387	f	0
555	666	W95INF32.DLL	simcop2387	f	1
508	666	System.Management.dll	simcop2387	f	1
1648	25	Encrypted Message: Zu gotdy od mpy nrmy stpimf	simcop2387	f	1
600	666	a32gfx.dll	simcop2387	f	1
601	666	acctres.dll	simcop2387	f	0
602	666	actxprxy.dll	simcop2387	f	1
603	666	advapi32.dll	simcop2387	f	1
604	666	advpack.dll	simcop2387	f	0
605	666	amstream.dll	simcop2387	f	0
606	666	antlr.astframe.dll	simcop2387	f	0
607	666	antlr.runtime.dll	simcop2387	f	1
608	666	appcomps.dll	simcop2387	f	0
610	666	art-sharp.dll	simcop2387	f	0
611	666	asferror.dll	simcop2387	f	0
612	666	asfsipc.dll	simcop2387	f	0
613	666	asi32_13.dll	simcop2387	f	1
615	666	atk-sharp.dll	simcop2387	f	0
616	666	atl.dll	simcop2387	f	0
617	666	atl90.dll	simcop2387	f	0
618	666	auth.dll	simcop2387	f	0
619	666	authplay.dll	simcop2387	f	0
620	666	autoconfig.dll	simcop2387	f	0
621	666	avahi-sharp.dll	simcop2387	f	0
622	666	avahi-ui-sharp.dll	simcop2387	f	0
623	666	bindfile.dll	simcop2387	f	0
624	666	binkw32.dll	simcop2387	f	0
625	666	blackbox.dll	simcop2387	f	1
626	666	browselc.dll	simcop2387	f	0
627	666	browserdirprovider.dll	simcop2387	f	0
582	666	XX_MDRAD.DLL	simcop2387	f	1
529	666	TYPELIB.DLL	simcop2387	f	1
569	666	WINGDE.DLL	simcop2387	f	1
628	666	browseui.dll	simcop2387	f	0
629	666	browsewm.dll	simcop2387	f	0
631	666	cabinet.dll	simcop2387	f	1
691	666	dbghelp.dll	simcop2387	f	1
633	666	caps.dll	simcop2387	f	0
634	666	cdDataPS.dll	simcop2387	f	0
635	666	cdExtra.dll	simcop2387	f	0
636	666	cdfview.dll	simcop2387	f	0
637	666	cdmp3.dll	simcop2387	f	0
638	666	cdral.dll	simcop2387	f	0
639	666	cdrtc.dll	simcop2387	f	0
640	666	cfgmgr32.dll	simcop2387	f	0
641	666	cg.dll	simcop2387	f	0
642	666	cgGL.dll	simcop2387	f	0
643	666	charset.dll	simcop2387	f	0
644	666	chrome.dll	simcop2387	f	0
645	666	cli_basetypes.dll	simcop2387	f	0
646	666	cli_cppuhelper.dll	simcop2387	f	0
599	666	_setup.dll	simcop2387	f	1
648	666	cli_uno_bridge.dll	simcop2387	f	0
649	666	cli_ure.dll	simcop2387	f	0
650	666	cli_uretypes.dll	simcop2387	f	1
651	666	clothlib.dll	simcop2387	f	0
652	666	cms16.dll	simcop2387	f	0
653	666	cms32_95.dll	simcop2387	f	0
654	666	cms32_nt.dll	simcop2387	f	0
655	666	codebehind1.dll	simcop2387	f	0
657	666	comctl32.dll	simcop2387	f	1
658	666	comdlg32.dll	simcop2387	f	0
659	666	compobj.dll	simcop2387	f	0
660	666	composer.dll	simcop2387	f	1
661	666	cookie.dll	simcop2387	f	1
662	666	cool.dll	simcop2387	f	0
663	666	corpol.dll	simcop2387	f	0
664	666	creator.dll	simcop2387	f	0
665	666	crypt32.dll	simcop2387	f	0
666	666	cryptdlg.dll	simcop2387	f	1
667	666	cryptext.dll	simcop2387	f	0
668	666	cryptnet.dll	simcop2387	f	0
669	666	cryptui.dll	simcop2387	f	0
670	666	csapi3t1.dll	simcop2387	f	0
671	666	cscompmgd.dll	simcop2387	f	0
676	666	custsat.dll	simcop2387	f	1
673	666	ctl3dv2.dll	simcop2387	f	0
674	666	ctlgldi.dll	simcop2387	f	0
675	666	ctor.dll	simcop2387	f	0
677	666	d2htools.dll	simcop2387	f	0
678	666	d3d8.dll	simcop2387	f	1
679	666	d3d8thk.dll	simcop2387	f	1
680	666	d3d9.dll	simcop2387	f	0
681	666	d3dim.dll	simcop2387	f	0
682	666	d3dim700.dll	simcop2387	f	0
683	666	d3dpmesh.dll	simcop2387	f	0
684	666	d3dramp.dll	simcop2387	f	0
685	666	d3drm.dll	simcop2387	f	1
686	666	d3dxof.dll	simcop2387	f	0
687	666	database.dll	simcop2387	f	0
688	666	daz-qsa.dll	simcop2387	f	0
689	666	daz-qt-mt.dll	simcop2387	f	1
692	666	ddraw.dll	simcop2387	f	0
693	666	ddrawex.dll	simcop2387	f	1
694	666	devenum.dll	simcop2387	f	0
695	666	diactfrm.dll	simcop2387	f	0
696	666	digest.dll	simcop2387	f	1
697	666	dimap.dll	simcop2387	f	1
698	666	dinput.dll	simcop2387	f	0
699	666	dinput8.dll	simcop2387	f	0
700	666	dirapi.dll	simcop2387	f	0
701	666	direct-schro.dll	simcop2387	f	0
702	666	directdb.dll	simcop2387	f	0
703	666	diskio.dll	simcop2387	f	0
704	666	dispex.dll	simcop2387	f	0
705	666	dll16.dll	simcop2387	f	0
706	666	dmband.dll	simcop2387	f	0
707	666	dmcompos.dll	simcop2387	f	0
708	666	dmime.dll	simcop2387	f	1
709	666	dmloader.dll	simcop2387	f	0
710	666	dmscript.dll	simcop2387	f	0
656	666	comcat.dll	simcop2387	f	1
711	666	dmstyle.dll	simcop2387	f	0
712	666	dmsynth.dll	simcop2387	f	0
714	666	docshell.dll	simcop2387	f	0
715	666	dpadpro.dll	simcop2387	f	1
716	666	dplayx.dll	simcop2387	f	0
609	666	appshell.dll	simcop2387	f	1
630	666	brwsrcmp.dll	simcop2387	f	1
717	666	dpmodemx.dll	simcop2387	f	0
718	666	dpnaddr.dll	simcop2387	f	0
719	666	dpnet.dll	simcop2387	f	0
720	666	dpnhpast.dll	simcop2387	f	1
826	666	gtksharpglue-2.dll	simcop2387	f	1
723	666	dpvacm.dll	simcop2387	f	0
724	666	dpvoice.dll	simcop2387	f	0
725	666	dpvvox.dll	simcop2387	f	0
726	666	dpwsockx.dll	simcop2387	f	0
728	666	drmclien.dll	simcop2387	f	1
729	666	drmstor.dll	simcop2387	f	0
730	666	drmv2clt.dll	simcop2387	f	0
731	666	dsdmo.dll	simcop2387	f	1
732	666	dsdmoprp.dll	simcop2387	f	0
733	666	dsetup32.dll	simcop2387	f	0
734	666	dsound.dll	simcop2387	f	0
735	666	dsound3d.dll	simcop2387	f	1
736	666	dssbase.dll	simcop2387	f	0
738	666	dwintl.dll	simcop2387	f	0
739	666	dx7vb.dll	simcop2387	f	0
740	666	dx8vb.dll	simcop2387	f	0
741	666	dxdiagn.dll	simcop2387	f	1
742	666	dxmasf.dll	simcop2387	f	1
743	666	dxtmsft.dll	simcop2387	f	0
744	666	dxtrans.dll	simcop2387	f	0
745	666	dz3dconnexion.dll	simcop2387	f	0
746	666	dz3delight.dll	simcop2387	f	0
747	666	dz3dmouse.dll	simcop2387	f	0
748	666	dzalignace.dll	simcop2387	f	0
749	666	dzbryce6.dll	simcop2387	f	0
727	666	driversV2.dll	simcop2387	f	1
713	666	dmusic.dll	simcop2387	f	1
750	666	dzbvh.dll	simcop2387	f	1
751	666	dzcarrara.dll	simcop2387	f	0
752	666	dzcollada.dll	simcop2387	f	0
753	666	dzcomponents.dll	simcop2387	f	1
754	666	dzcore.dll	simcop2387	f	0
755	666	dzdelightrenderer.dll	simcop2387	f	0
756	666	dzdformeditor.dll	simcop2387	f	0
757	666	dzdynlod.dll	simcop2387	f	0
758	666	dzeasyloader.dll	simcop2387	f	0
759	666	dzhexagonbridge.dll	simcop2387	f	0
760	666	dzimageio.dll	simcop2387	f	0
761	666	dzmimicbasic.dll	simcop2387	f	1
762	666	dzobjexporter.dll	simcop2387	f	0
763	666	dzpowerpose.dll	simcop2387	f	0
764	666	dzprimitives.dll	simcop2387	f	0
765	666	dzpuppeteer.dll	simcop2387	f	0
766	666	dzpz3importer.dll	simcop2387	f	1
767	666	dzquickstart.dll	simcop2387	f	1
768	666	dzsavefilters.dll	simcop2387	f	0
769	666	dzsceneinfo.dll	simcop2387	f	0
770	666	dzspotrendertool.dll	simcop2387	f	0
771	666	dzstartupwizard.dll	simcop2387	f	1
772	666	dzu3dexporter.dll	simcop2387	f	0
773	666	dzwinvideo.dll	simcop2387	f	1
774	666	dzwrapper.dll	simcop2387	f	1
776	666	embedcomponents.dll	simcop2387	f	0
777	666	encapi.dll	simcop2387	f	0
778	666	enhsig.dll	simcop2387	f	0
780	666	esenu.dll	simcop2387	f	0
781	666	esshared.dll	simcop2387	f	0
782	666	estier2.dll	simcop2387	f	0
783	666	evalcom2.dll	simcop2387	f	0
784	666	extensions.dll	simcop2387	f	0
785	666	flash.dll	simcop2387	f	0
786	666	flashlite1x.dll	simcop2387	f	0
787	666	flashsnddec.dll	simcop2387	f	0
788	666	freebl3.dll	simcop2387	f	1
789	666	freetype6.dll	simcop2387	f	1
790	666	gamedg.dll	simcop2387	f	1
791	666	gamefm.dll	simcop2387	f	0
792	666	gcdef.dll	simcop2387	f	0
793	666	gconf-sharp-peditors.dll	simcop2387	f	0
794	666	gconf-sharp.dll	simcop2387	f	1
795	666	gdi32.dll	simcop2387	f	0
796	666	gdk-sharp.dll	simcop2387	f	0
797	666	gdksharpglue-2.dll	simcop2387	f	1
798	666	gdksharpglue.dll	simcop2387	f	0
799	666	gecko-sharp.dll	simcop2387	f	0
800	666	gettextlib.dll	simcop2387	f	1
801	666	gettextpo.dll	simcop2387	f	1
802	666	gettextsrc.dll	simcop2387	f	1
804	666	gkgfxwin.dll	simcop2387	f	0
805	666	gklayout.dll	simcop2387	f	1
806	666	gkparser.dll	simcop2387	f	0
807	666	gkplugin.dll	simcop2387	f	0
808	666	gkwidget.dll	simcop2387	f	0
803	666	gkgfx.dll	simcop2387	f	1
810	666	glade-sharp.dll	simcop2387	f	0
811	666	gladesharpglue-2.dll	simcop2387	f	0
812	666	gladesharpglue.dll	simcop2387	f	0
813	666	glew32.dll	simcop2387	f	0
814	666	glib-sharp.dll	simcop2387	f	0
815	666	glibsharpglue-2.dll	simcop2387	f	1
816	666	glibsharpglue.dll	simcop2387	f	1
817	666	glu32.dll	simcop2387	f	0
818	666	gluezilla.dll	simcop2387	f	0
819	666	glut32.dll	simcop2387	f	0
821	666	gnomesharpglue.dll	simcop2387	f	1
822	666	granny2.dll	simcop2387	f	0
823	666	gtk-dotnet.dll	simcop2387	f	1
825	666	gtkhtml-sharp.dll	simcop2387	f	0
737	666	dswave.dll	simcop2387	f	1
820	666	gnome-sharp.dll	simcop2387	f	1
827	666	gtksharpglue.dll	simcop2387	f	1
828	666	hcwutl32.dll	simcop2387	f	1
829	666	hcwxds.dll	simcop2387	f	0
830	666	hhsetup.dll	simcop2387	f	1
831	666	hlink.dll	simcop2387	f	0
832	666	hmmapi.dll	simcop2387	f	0
833	666	i18n.dll	simcop2387	f	0
834	666	iGdiCnv.dll	simcop2387	f	1
836	666	iconv.dll	simcop2387	f	1
837	666	icwconn.dll	simcop2387	f	0
839	666	icwdl.dll	simcop2387	f	0
840	666	icwhelp.dll	simcop2387	f	0
842	666	icwres.dll	simcop2387	f	0
843	666	icwutil.dll	simcop2387	f	0
844	666	idctref.dll	simcop2387	f	0
845	666	idctsse.dll	simcop2387	f	1
847	666	iedetect.dll	simcop2387	f	0
848	666	iedkcs32.dll	simcop2387	f	0
849	666	iemigrat.dll	simcop2387	f	0
850	666	iepeers.dll	simcop2387	f	1
851	666	iernonce.dll	simcop2387	f	1
852	666	iesetup.dll	simcop2387	f	1
853	666	ijl10.dll	simcop2387	f	0
854	666	ikvm-native.dll	simcop2387	f	0
855	666	im-am-et.dll	simcop2387	f	0
857	666	im-cyrillic-translit.dll	simcop2387	f	0
858	666	im-ime.dll	simcop2387	f	1
859	666	im-inuktitut.dll	simcop2387	f	0
860	666	im-ipa.dll	simcop2387	f	0
861	666	im-thai-broken.dll	simcop2387	f	1
862	666	im-ti-er.dll	simcop2387	f	0
863	666	im-ti-et.dll	simcop2387	f	0
864	666	im-viqr.dll	simcop2387	f	1
866	666	imgicon.dll	simcop2387	f	1
867	666	imglib2.dll	simcop2387	f	0
868	666	imgutil.dll	simcop2387	f	0
869	666	iml32.dll	simcop2387	f	1
1649	25	Fruit Loop on stove	simcop2387	f	0
871	666	inetcfg.dll	simcop2387	f	1
775	666	editor.dll	simcop2387	f	1
838	666	icwdial.dll	simcop2387	f	1
779	666	es.dll	simcop2387	f	1
872	666	inetcomm.dll	simcop2387	f	0
873	666	inetcplc.dll	simcop2387	f	0
874	666	inetres.dll	simcop2387	f	0
875	666	initpki.dll	simcop2387	f	0
876	666	inseng.dll	simcop2387	f	1
877	666	install.res.1028.dll	simcop2387	f	0
878	666	install.res.1031.dll	simcop2387	f	1
880	666	install.res.1036.dll	simcop2387	f	0
881	666	install.res.1040.dll	simcop2387	f	0
882	666	install.res.1041.dll	simcop2387	f	0
883	666	install.res.1042.dll	simcop2387	f	0
884	666	install.res.2052.dll	simcop2387	f	1
885	666	install.res.3082.dll	simcop2387	f	0
887	666	instsch.dll	simcop2387	f	1
888	666	intl.dll	simcop2387	f	0
889	666	intlcmpt.dll	simcop2387	f	0
890	666	iprop.dll	simcop2387	f	1
891	666	iscript.dll	simcop2387	f	1
892	666	isign32.dll	simcop2387	f	0
893	666	isrt.dll	simcop2387	f	0
894	666	itircl.dll	simcop2387	f	1
895	666	itss.dll	simcop2387	f	0
922	666	libIDL-2-0.dll	simcop2387	f	1
897	666	iuengine.dll	simcop2387	f	0
898	666	iuser.dll	simcop2387	f	1
899	666	jar50.dll	simcop2387	f	0
900	666	javaxpcomglue.dll	simcop2387	f	0
901	666	jgaw400.dll	simcop2387	f	0
902	666	jgdw400.dll	simcop2387	f	0
903	666	jgmd400.dll	simcop2387	f	1
904	666	jgpl400.dll	simcop2387	f	0
905	666	jgsd400.dll	simcop2387	f	0
906	666	jgsh400.dll	simcop2387	f	1
907	666	jobexec.dll	simcop2387	f	0
908	666	jpeg.dll	simcop2387	f	0
909	666	jpeg62.dll	simcop2387	f	0
910	666	js3250.dll	simcop2387	f	1
911	666	jscript.dll	simcop2387	f	0
912	666	jsd3250.dll	simcop2387	f	0
914	666	jsproxy.dll	simcop2387	f	1
915	666	kailleraclient.dll	simcop2387	f	0
916	666	kernel32.dll	simcop2387	f	0
917	666	kpp.dll	simcop2387	f	0
918	666	kppaddon.dll	simcop2387	f	0
919	666	ksuser.dll	simcop2387	f	0
920	666	language.dll	simcop2387	f	0
921	666	laprxy.dll	simcop2387	f	1
923	666	libORBit-2-0.dll	simcop2387	f	0
924	666	libORBit-imodule-2-0.dll	simcop2387	f	0
925	666	libORBitCosNaming-2-0.dll	simcop2387	f	0
926	666	libart_lgpl_2-2.dll	simcop2387	f	1
927	666	libatk-1.0-0.dll	simcop2387	f	0
928	666	libbonobo-2-0.dll	simcop2387	f	0
835	666	icfg95.dll	simcop2387	f	1
930	666	libbonobo.dll	simcop2387	f	0
931	666	libbonoboui-2-0.dll	simcop2387	f	0
932	666	libbzip2.dll	simcop2387	f	0
933	666	libcairo-2.dll	simcop2387	f	0
934	666	libcanvas.dll	simcop2387	f	0
935	666	libcomputer.dll	simcop2387	f	1
936	666	libeay32.dll	simcop2387	f	0
937	666	libexpat.dll	simcop2387	f	0
938	666	libfile.dll	simcop2387	f	1
939	666	libfontconfig-1.dll	simcop2387	f	0
940	666	libgail.dll	simcop2387	f	0
941	666	libgailutil-18.dll	simcop2387	f	0
943	666	libgconfbackend-oldxml.dll	simcop2387	f	0
944	666	libgconfbackend-xml.dll	simcop2387	f	0
945	666	libgdk-win32-2.0-0.dll	simcop2387	f	0
946	666	libgdk_pixbuf-2.0-0.dll	simcop2387	f	0
947	666	libglade-2.0-0.dll	simcop2387	f	0
948	666	libgladegtk.dll	simcop2387	f	1
886	666	instrsa.dll	simcop2387	f	1
913	666	jsj3250.dll	simcop2387	f	1
865	666	imagehlp.dll	simcop2387	f	1
846	666	ie4.dll	simcop2387	f	1
856	666	im-cedilla.dll	simcop2387	f	1
949	666	libgladeui-1-3.dll	simcop2387	f	1
950	666	libglib-2.0-0.dll	simcop2387	f	0
951	666	libgmodule-2.0-0.dll	simcop2387	f	0
952	666	libgnome-2-0.dll	simcop2387	f	1
953	666	libgnome-gtkhtml-editor-3.8.dll	simcop2387	f	0
954	666	libgnome.dll	simcop2387	f	1
955	666	libgnomecanvas-2-0.dll	simcop2387	f	0
956	666	libgnomeprint-2-2-0.dll	simcop2387	f	0
957	666	libgnomeprint-clip.dll	simcop2387	f	1
958	666	libgnomeprint-custom.dll	simcop2387	f	0
959	666	libgnomeprint-draft.dll	simcop2387	f	1
960	666	libgnomeprint-file.dll	simcop2387	f	0
961	666	libgnomeprint-frgba.dll	simcop2387	f	0
962	666	libgnomeprint-lpr.dll	simcop2387	f	1
963	666	libgnomeprint-multipage.dll	simcop2387	f	1
964	666	libgnomeprint-position.dll	simcop2387	f	0
965	666	libgnomeprint-rotate.dll	simcop2387	f	0
967	666	libgnomeprint-zoom.dll	simcop2387	f	0
968	666	libgnomeprintlpd.dll	simcop2387	f	0
969	666	libgnomeprintui-2-2-0.dll	simcop2387	f	1
970	666	libgnomeprintwin32.dll	simcop2387	f	0
992	666	libpixbufloader-gif.dll	simcop2387	f	1
972	666	libgnomevfs-2-0.dll	simcop2387	f	1
973	666	libgobject-2.0-0.dll	simcop2387	f	1
975	666	libgsf-gnome-1-114.dll	simcop2387	f	0
976	666	libgsf-win32-1-114.dll	simcop2387	f	0
977	666	libgthread-2.0-0.dll	simcop2387	f	0
978	666	libgtk-win32-2.0-0.dll	simcop2387	f	0
979	666	libgtkhtml-3.8-15.dll	simcop2387	f	0
980	666	libimage.dll	simcop2387	f	0
981	666	libjpeg.dll	simcop2387	f	0
982	666	libmoniker_extra_2.dll	simcop2387	f	0
983	666	libmoniker_gnome_vfs_std.dll	simcop2387	f	0
984	666	libmoniker_std_2.dll	simcop2387	f	0
985	666	libogg-0.dll	simcop2387	f	0
986	666	libpango-1.0-0.dll	simcop2387	f	0
987	666	libpangocairo-1.0-0.dll	simcop2387	f	0
988	666	libpangoft2-1.0-0.dll	simcop2387	f	1
989	666	libpangowin32-1.0-0.dll	simcop2387	f	1
990	666	libpixbufloader-ani.dll	simcop2387	f	0
991	666	libpixbufloader-bmp.dll	simcop2387	f	0
993	666	libpixbufloader-ico.dll	simcop2387	f	0
994	666	libpixbufloader-jpeg.dll	simcop2387	f	0
996	666	libpixbufloader-png.dll	simcop2387	f	0
997	666	libpixbufloader-pnm.dll	simcop2387	f	1
998	666	libpixbufloader-ras.dll	simcop2387	f	1
999	666	libpixbufloader-tga.dll	simcop2387	f	0
1000	666	libpixbufloader-tiff.dll	simcop2387	f	0
1001	666	libpixbufloader-wbmp.dll	simcop2387	f	1
1002	666	libpixbufloader-xbm.dll	simcop2387	f	0
1003	666	libpixbufloader-xpm.dll	simcop2387	f	1
1004	666	libpixmap.dll	simcop2387	f	0
1005	666	libpng-3.dll	simcop2387	f	0
1006	666	libpng12-0.dll	simcop2387	f	0
1007	666	libpng13.dll	simcop2387	f	0
1008	666	libpopt-0.dll	simcop2387	f	0
1009	666	librle3.dll	simcop2387	f	0
1010	666	librsvg-2-2.dll	simcop2387	f	0
1011	666	librsvg-2.dll	simcop2387	f	0
1012	666	libssl32.dll	simcop2387	f	1
1013	666	libsvg.dll	simcop2387	f	0
1014	666	libtar.dll	simcop2387	f	0
1015	666	libtiff3.dll	simcop2387	f	0
1016	666	libungif4.dll	simcop2387	f	1
1017	666	libvfs-test.dll	simcop2387	f	0
1018	666	libvorbis-0.dll	simcop2387	f	0
1020	666	libvorbisfile-3.dll	simcop2387	f	0
1021	666	libwimp.dll	simcop2387	f	0
1022	666	libxml2.dll	simcop2387	f	0
1023	666	licmgr10.dll	simcop2387	f	0
1024	666	log4net.dll	simcop2387	f	1
1025	666	mameiplib.dll	simcop2387	f	1
1026	666	mapistub.dll	simcop2387	f	0
1027	666	mciqtenu.dll	simcop2387	f	0
1028	666	mciqtz32.dll	simcop2387	f	0
1029	666	mergemod.dll	simcop2387	f	1
1030	666	mfc40.dll	simcop2387	f	1
1031	666	mfc42.dll	simcop2387	f	0
1032	666	mfc42u.dll	simcop2387	f	0
1034	666	mfc80.dll	simcop2387	f	1
1035	666	mfc80CHS.dll	simcop2387	f	1
1036	666	mfc80CHT.dll	simcop2387	f	0
1037	666	mfc80DEU.dll	simcop2387	f	1
1038	666	mfc80ENU.dll	simcop2387	f	1
1039	666	mfc80ESP.dll	simcop2387	f	1
1040	666	mfc80FRA.dll	simcop2387	f	0
1041	666	mfc80ITA.dll	simcop2387	f	0
1042	666	mfc80JPN.dll	simcop2387	f	1
1043	666	mfc80KOR.dll	simcop2387	f	0
1044	666	mfc80u.dll	simcop2387	f	0
1045	666	mfc90.dll	simcop2387	f	0
1046	666	mfc90chs.dll	simcop2387	f	0
1048	666	mfc90deu.dll	simcop2387	f	0
1049	666	mfc90enu.dll	simcop2387	f	0
1050	666	mfc90esn.dll	simcop2387	f	0
1051	666	mfc90esp.dll	simcop2387	f	0
1052	666	mfc90fra.dll	simcop2387	f	0
1053	666	mfc90ita.dll	simcop2387	f	0
1054	666	mfc90jpn.dll	simcop2387	f	0
1055	666	mfc90kor.dll	simcop2387	f	0
1056	666	mfc90u.dll	simcop2387	f	0
1057	666	mfcm80.dll	simcop2387	f	0
1058	666	mfcm80u.dll	simcop2387	f	0
1059	666	mfcm90.dll	simcop2387	f	1
1060	666	mfcm90u.dll	simcop2387	f	0
1062	666	mlang.dll	simcop2387	f	0
1063	666	mlcad.dll	simcop2387	f	0
1064	666	mmutilse.dll	simcop2387	f	0
1065	666	mmxptresources.dll	simcop2387	f	1
1066	666	mobsync.dll	simcop2387	f	0
1067	666	mono.dll	simcop2387	f	0
1069	666	mork.dll	simcop2387	f	1
1070	666	mozcrt19.dll	simcop2387	f	0
1071	666	mozctl.dll	simcop2387	f	1
1072	666	mozctlx.dll	simcop2387	f	1
1073	666	mozfind.dll	simcop2387	f	1
1074	666	mozz.dll	simcop2387	f	0
1075	666	mp43dmod.dll	simcop2387	f	1
1076	666	mp4sdmod.dll	simcop2387	f	0
1077	666	mpg4dmod.dll	simcop2387	f	0
1078	666	mpvis.dll	simcop2387	f	0
1079	666	mscat32.dll	simcop2387	f	0
1657	25	Cheesy gadget that sounds larger	simcop2387	f	0
1081	666	msconv97.dll	simcop2387	f	0
1082	666	mscoree.dll	simcop2387	f	1
1083	666	mscorlib.dll	simcop2387	f	0
1084	666	mscrlrev.dll	simcop2387	f	1
1085	666	msdetstf.dll	simcop2387	f	0
1047	666	mfc90cht.dll	simcop2387	f	1
1087	666	msdia90.dll	simcop2387	f	1
1088	666	msdmo.dll	simcop2387	f	1
1068	666	monodoc.dll	simcop2387	f	1
1090	666	msencode.dll	simcop2387	f	0
1091	666	mshtml.dll	simcop2387	f	1
1092	666	mshtmled.dll	simcop2387	f	1
1093	666	mshtmler.dll	simcop2387	f	0
1094	666	msi.dll	simcop2387	f	0
1095	666	msident.dll	simcop2387	f	0
1096	666	msidle.dll	simcop2387	f	1
1097	666	msidntld.dll	simcop2387	f	0
1098	666	msihnd.dll	simcop2387	f	0
1099	666	msimsg.dll	simcop2387	f	0
1100	666	msinsstf.dll	simcop2387	f	0
1101	666	msisip.dll	simcop2387	f	1
1102	666	msls31.dll	simcop2387	f	0
1103	666	msnetobj.dll	simcop2387	f	0
1104	666	msoe.dll	simcop2387	f	1
1105	666	msoeacct.dll	simcop2387	f	0
1061	666	migrate.dll	simcop2387	f	1
995	666	libpixbufloader-pcx.dll	simcop2387	f	1
1138	666	msxml3.dll	simcop2387	f	1
1107	666	msoert2.dll	simcop2387	f	1
1108	666	msoss.dll	simcop2387	f	1
1109	666	mspatcha.dll	simcop2387	f	1
1110	666	mspmsnsv.dll	simcop2387	f	0
1111	666	mspmsp.dll	simcop2387	f	1
1112	666	msratelc.dll	simcop2387	f	1
1113	666	msrating.dll	simcop2387	f	1
1114	666	msscp.dll	simcop2387	f	1
1157	666	npwmsdrm.dll	simcop2387	f	1
1116	666	msshlstf.dll	simcop2387	f	0
1117	666	mssign32.dll	simcop2387	f	0
1118	666	mssip32.dll	simcop2387	f	0
1119	666	mstask.dll	simcop2387	f	0
1120	666	mstime.dll	simcop2387	f	0
1121	666	msuilstf.dll	simcop2387	f	0
1144	666	myspell.dll	simcop2387	f	1
1123	666	msvcm80.dll	simcop2387	f	0
1125	666	msvcp60.dll	simcop2387	f	0
1126	666	msvcp71.dll	simcop2387	f	0
1127	666	msvcp80.dll	simcop2387	f	0
1128	666	msvcp90.dll	simcop2387	f	1
1129	666	msvcr71.dll	simcop2387	f	1
1130	666	msvcr80.dll	simcop2387	f	0
1131	666	msvcr80d.dll	simcop2387	f	0
1132	666	msvcr90.dll	simcop2387	f	1
1133	666	msvcrt.dll	simcop2387	f	0
1134	666	msvidctl.dll	simcop2387	f	0
1135	666	mswebdvd.dll	simcop2387	f	0
1136	666	mswmdm.dll	simcop2387	f	1
1137	666	msxml.dll	simcop2387	f	1
1139	666	msxml3a.dll	simcop2387	f	0
1140	666	msxml3r.dll	simcop2387	f	1
1141	666	msxmla.dll	simcop2387	f	1
1142	666	msxmlr.dll	simcop2387	f	0
1143	666	msyuv.dll	simcop2387	f	1
1145	666	necko.dll	simcop2387	f	0
1146	666	necko2.dll	simcop2387	f	0
1147	666	npdrmv2.dll	simcop2387	f	0
1148	666	npmozax.dll	simcop2387	f	0
1149	666	npnul32.dll	simcop2387	f	0
1151	666	npqtplugin2.dll	simcop2387	f	1
1152	666	npqtplugin3.dll	simcop2387	f	0
1153	666	npqtplugin4.dll	simcop2387	f	0
1154	666	npqtplugin5.dll	simcop2387	f	0
1155	666	npqtplugin6.dll	simcop2387	f	0
1156	666	npqtplugin7.dll	simcop2387	f	0
1158	666	nspr4.dll	simcop2387	f	0
1159	666	nsprefm.dll	simcop2387	f	1
1160	666	nss3.dll	simcop2387	f	0
1161	666	nssckbi.dll	simcop2387	f	0
1162	666	nssdbm3.dll	simcop2387	f	0
1164	666	ntdll.dll	simcop2387	f	0
1165	666	nunit.core.dll	simcop2387	f	1
1167	666	nunit.mocks.dll	simcop2387	f	0
1168	666	nunit.util.dll	simcop2387	f	0
1169	666	objectps.dll	simcop2387	f	1
1170	666	objpscnv.dll	simcop2387	f	1
1171	666	occache.dll	simcop2387	f	1
1172	666	oeimport.dll	simcop2387	f	0
1173	666	oemiglib.dll	simcop2387	f	0
1174	666	oji.dll	simcop2387	f	0
1175	666	ole2.dll	simcop2387	f	1
1176	666	ole32.dll	simcop2387	f	0
1177	666	oleaut32.dll	simcop2387	f	0
1178	666	olecnv32.dll	simcop2387	f	0
1179	666	oledlg.dll	simcop2387	f	0
1180	666	olepro32.dll	simcop2387	f	0
1181	666	olethk32.dll	simcop2387	f	1
1182	666	opaque.dll	simcop2387	f	0
1183	666	opengl32.dll	simcop2387	f	0
1184	666	pango-sharp.dll	simcop2387	f	1
1185	666	pangosharpglue-2.dll	simcop2387	f	0
1124	666	msvcm90.dll	simcop2387	f	1
1186	666	pangosharpglue.dll	simcop2387	f	0
1187	666	pdQCustDlg.dll	simcop2387	f	0
1188	666	pid.dll	simcop2387	f	0
1189	666	pidgen.dll	simcop2387	f	0
1191	666	pipnss.dll	simcop2387	f	0
1192	666	pippki.dll	simcop2387	f	0
1193	666	playenu.dll	simcop2387	f	0
1194	666	plc4.dll	simcop2387	f	0
1195	666	plds4.dll	simcop2387	f	1
1196	666	pngfilt.dll	simcop2387	f	0
1197	666	policy.1.0.cli_basetypes.dll	simcop2387	f	0
1198	666	policy.1.0.cli_cppuhelper.dll	simcop2387	f	0
1199	666	policy.1.0.cli_oootypes.dll	simcop2387	f	0
1200	666	policy.1.0.cli_ure.dll	simcop2387	f	0
1201	666	policy.1.0.cli_uretypes.dll	simcop2387	f	0
1202	666	policy.2.4.art-sharp.dll	simcop2387	f	0
1203	666	policy.2.4.atk-sharp.dll	simcop2387	f	1
1204	666	policy.2.4.gconf-sharp.dll	simcop2387	f	0
1205	666	policy.2.4.gdk-sharp.dll	simcop2387	f	0
1206	666	policy.2.4.glade-sharp.dll	simcop2387	f	0
1207	666	policy.2.4.glib-sharp.dll	simcop2387	f	0
1208	666	policy.2.4.gtk-dotnet.dll	simcop2387	f	1
1209	666	policy.2.4.gtk-sharp.dll	simcop2387	f	1
1210	666	policy.2.4.pango-sharp.dll	simcop2387	f	0
1211	666	policy.2.4.rsvg-sharp.dll	simcop2387	f	1
1212	666	policy.2.6.art-sharp.dll	simcop2387	f	0
1213	666	policy.2.6.atk-sharp.dll	simcop2387	f	0
1214	666	policy.2.6.gconf-sharp.dll	simcop2387	f	0
1215	666	policy.2.6.gdk-sharp.dll	simcop2387	f	1
1216	666	policy.2.6.glade-sharp.dll	simcop2387	f	1
1217	666	policy.2.6.glib-sharp.dll	simcop2387	f	1
1218	666	policy.2.6.gtk-dotnet.dll	simcop2387	f	0
1219	666	policy.2.6.gtk-sharp.dll	simcop2387	f	0
1220	666	policy.2.6.pango-sharp.dll	simcop2387	f	1
1221	666	policy.2.6.rsvg-sharp.dll	simcop2387	f	0
1222	666	policy.2.8.art-sharp.dll	simcop2387	f	0
1224	666	policy.2.8.gconf-sharp.dll	simcop2387	f	1
1166	666	nunit.framework.dll	simcop2387	f	1
1273	666	runtime.dll	simcop2387	f	1
1226	666	policy.2.8.glade-sharp.dll	simcop2387	f	1
1227	666	policy.2.8.glib-sharp.dll	simcop2387	f	0
1228	666	policy.2.8.gtk-dotnet.dll	simcop2387	f	1
1229	666	policy.2.8.gtk-sharp.dll	simcop2387	f	0
1230	666	policy.2.8.pango-sharp.dll	simcop2387	f	0
1231	666	policy.2.8.rsvg-sharp.dll	simcop2387	f	1
1232	666	profile.dll	simcop2387	f	0
1233	666	proj.dll	simcop2387	f	0
1235	666	psbase.dll	simcop2387	f	0
1236	666	psisdecd.dll	simcop2387	f	1
1237	666	pstorec.dll	simcop2387	f	1
1238	666	pstorerc.dll	simcop2387	f	0
1239	666	python24.dll	simcop2387	f	0
1240	666	qasf.dll	simcop2387	f	0
1241	666	qcap.dll	simcop2387	f	0
1242	666	qdv.dll	simcop2387	f	0
1243	666	qdvd.dll	simcop2387	f	0
1244	666	qedit.dll	simcop2387	f	0
1245	666	qedwipes.dll	simcop2387	f	0
1246	666	qthndlr.dll	simcop2387	f	0
1247	666	qtim.dll	simcop2387	f	0
1248	666	qtimcmgr.dll	simcop2387	f	1
1249	666	qtole.dll	simcop2387	f	0
1250	666	quartz.dll	simcop2387	f	0
1251	666	rdf.dll	simcop2387	f	0
1252	666	regex.dll	simcop2387	f	0
1253	666	resource_eng.dll	simcop2387	f	0
1254	666	riched20.dll	simcop2387	f	0
1255	666	riched32.dll	simcop2387	f	0
1256	666	rmconfig.dll	simcop2387	f	0
1258	666	rpcltc1.dll	simcop2387	f	1
1259	666	rpcltc5.dll	simcop2387	f	1
1276	666	schannel.dll	simcop2387	f	1
1261	666	rpclts5.dll	simcop2387	f	0
1262	666	rpcltscm.dll	simcop2387	f	1
1263	666	rpcltspx.dll	simcop2387	f	1
1264	666	rpcmqcl.dll	simcop2387	f	0
1265	666	rpcmqsvr.dll	simcop2387	f	0
1266	666	rpcns4.dll	simcop2387	f	0
1267	666	rpcrt4.dll	simcop2387	f	0
1268	666	rsabase.dll	simcop2387	f	0
1269	666	rsaenh.dll	simcop2387	f	0
1270	666	rsasig.dll	simcop2387	f	0
1271	666	rsl.dll	simcop2387	f	0
1272	666	rsvg-sharp.dll	simcop2387	f	0
1274	666	rxtxSerial.dll	simcop2387	f	0
1275	666	sage.dll	simcop2387	f	1
1277	666	scidll.dll	simcop2387	f	0
1278	666	scrobj.dll	simcop2387	f	0
1279	666	scrrun.dll	simcop2387	f	0
1280	666	sdbapi.dll	simcop2387	f	0
1281	666	secur32.dll	simcop2387	f	0
1282	666	sendmail.dll	simcop2387	f	1
1283	666	sens.dll	simcop2387	f	0
1284	666	sensapi.dll	simcop2387	f	0
1285	666	senscfg.dll	simcop2387	f	0
1286	666	setupapi.dll	simcop2387	f	0
1287	666	setupenu.dll	simcop2387	f	0
1288	666	shd401lc.dll	simcop2387	f	1
1289	666	shdoc401.dll	simcop2387	f	1
1290	666	shdoclc.dll	simcop2387	f	0
1291	666	shdocvw.dll	simcop2387	f	0
1292	666	shell32.dll	simcop2387	f	0
1293	666	shfolder.dll	simcop2387	f	0
1294	666	shlwapi.dll	simcop2387	f	0
1295	666	smime3.dll	simcop2387	f	0
1296	666	softokn3.dll	simcop2387	f	1
1297	666	softpub.dll	simcop2387	f	0
1298	666	spellchk.dll	simcop2387	f	0
1299	666	sqlite3.dll	simcop2387	f	1
1300	666	srchsvc.dll	simcop2387	f	1
1190	666	pipboot.dll	simcop2387	f	1
1301	666	sroaming.dll	simcop2387	f	0
1302	666	ssPlugin.dll	simcop2387	f	0
1303	666	ssl3.dll	simcop2387	f	0
1304	666	storage.dll	simcop2387	f	0
1306	666	strmdll.dll	simcop2387	f	0
1307	666	svg_loader.dll	simcop2387	f	0
1308	666	t2embed.dll	simcop2387	f	0
1309	666	tabcontrol.dll	simcop2387	f	0
1310	666	tabcontrol2.dll	simcop2387	f	0
1311	666	tcl83.dll	simcop2387	f	0
1312	666	tcldde83.dll	simcop2387	f	0
1313	666	tclpip83.dll	simcop2387	f	0
1314	666	tclreg83.dll	simcop2387	f	0
1315	666	thumbvw.dll	simcop2387	f	0
1316	666	ticalc.dll	simcop2387	f	0
1317	666	tk83.dll	simcop2387	f	1
1318	666	tkimg13.dll	simcop2387	f	0
1319	666	tkimgbmp13.dll	simcop2387	f	0
1320	666	transformiix.dll	simcop2387	f	0
1321	666	treeview.dll	simcop2387	f	0
1322	666	trialoc.dll	simcop2387	f	0
1323	666	triedit.dll	simcop2387	f	0
1324	666	twain_32.dll	simcop2387	f	0
1325	666	twapi-2.0a7.dll	simcop2387	f	0
1326	666	txmgr.dll	simcop2387	f	0
1327	666	typeaheadfind.dll	simcop2387	f	0
1328	666	typedesc.dll	simcop2387	f	1
1329	666	uconv.dll	simcop2387	f	0
1330	666	ucvmath.dll	simcop2387	f	0
1332	666	universalchardet.dll	simcop2387	f	0
1333	666	unowinreg.dll	simcop2387	f	0
1334	666	url.dll	simcop2387	f	1
1335	666	urlmon.dll	simcop2387	f	0
1336	666	user32.dll	simcop2387	f	0
1337	666	usp10.dll	simcop2387	f	0
1338	666	valobj.dll	simcop2387	f	1
1257	666	rnaph.dll	simcop2387	f	1
1341	666	vcomp90.dll	simcop2387	f	0
1342	666	version.dll	simcop2387	f	0
1343	666	vgx.dll	simcop2387	f	1
1344	666	vidx16.dll	simcop2387	f	0
1345	666	viewenu.dll	simcop2387	f	1
1346	666	vise32ex.dll	simcop2387	f	0
1331	666	unicows.dll	simcop2387	f	1
1347	666	w3lh.dll	simcop2387	f	0
1348	666	w95inf16.dll	simcop2387	f	0
1349	666	w95inf32.dll	simcop2387	f	0
1350	666	wab32.dll	simcop2387	f	1
1351	666	wab32res.dll	simcop2387	f	0
1352	666	wabfind.dll	simcop2387	f	0
1353	666	wabimp.dll	simcop2387	f	1
1354	666	wallet.dll	simcop2387	f	0
1355	666	webbrwsr.dll	simcop2387	f	0
1356	666	webcheck.dll	simcop2387	f	1
1357	666	websrvcs.dll	simcop2387	f	0
1359	666	winmm.dll	simcop2387	f	1
1360	666	wintab32.dll	simcop2387	f	0
1361	666	wintrust.dll	simcop2387	f	0
1362	666	wldap32.dll	simcop2387	f	0
1363	666	wlltvwrs.dll	simcop2387	f	0
1364	666	wmadmod.dll	simcop2387	f	1
1365	666	wmadmoe.dll	simcop2387	f	1
1366	666	wmasf.dll	simcop2387	f	1
1367	666	wmdmlog.dll	simcop2387	f	0
1368	666	wmdmps.dll	simcop2387	f	0
1369	666	wmerror.dll	simcop2387	f	0
1370	666	wmidx.dll	simcop2387	f	1
1371	666	wmnetmgr.dll	simcop2387	f	0
1372	666	wmp.dll	simcop2387	f	0
1373	666	wmpasf.dll	simcop2387	f	0
1374	666	wmpcd.dll	simcop2387	f	1
1376	666	wmpdxm.dll	simcop2387	f	0
1377	666	wmploc.dll	simcop2387	f	0
1378	666	wmpns.dll	simcop2387	f	0
1379	666	wmpshell.dll	simcop2387	f	0
1380	666	wmpui.dll	simcop2387	f	1
1381	666	wmsdmod.dll	simcop2387	f	0
1382	666	wmsdmoe2.dll	simcop2387	f	0
1383	666	wmspdmod.dll	simcop2387	f	1
1384	666	wmspdmoe.dll	simcop2387	f	1
1385	666	wmvcore.dll	simcop2387	f	0
1386	666	wmvdmod.dll	simcop2387	f	0
1387	666	wmvdmoe2.dll	simcop2387	f	0
1358	666	wininet.dll	simcop2387	f	1
1390	666	wshext.dll	simcop2387	f	0
1391	666	wsock32.dll	simcop2387	f	0
1392	666	wstdecod.dll	simcop2387	f	1
1393	666	xenroll.dll	simcop2387	f	1
1394	666	xerces-c_2_3_0.dll	simcop2387	f	0
1395	666	xerces-c_2_6.dll	simcop2387	f	0
1396	666	xmlextras.dll	simcop2387	f	0
1397	666	xpc3250.dll	simcop2387	f	0
1398	666	xpcom.dll	simcop2387	f	0
1400	666	xpcom_compat_c.dll	simcop2387	f	0
1401	666	xpcom_core.dll	simcop2387	f	0
1402	666	xpinstal.dll	simcop2387	f	0
1403	666	xpistub.dll	simcop2387	f	0
1404	666	xppref32.dll	simcop2387	f	0
1405	666	xul.dll	simcop2387	f	0
1406	666	xulutil.dll	simcop2387	f	1
1407	666	z.dll	simcop2387	f	1
1234	666	psapi.dll	simcop2387	f	1
896	666	iuctl.dll	simcop2387	f	1
1089	666	msdxmlc.dll	simcop2387	f	1
1375	666	wmpcore.dll	simcop2387	f	1
538	666	TrkWrite.dll	simcop2387	f	1
598	666	_isres_modified.dll	simcop2387	f	1
974	666	libgsf-1-114.dll	simcop2387	f	1
722	666	dpnlobby.dll	simcop2387	f	1
1660	25	Slyness holding shipment in choppe?	simcop2387	f	1
1408	666	zlib1.dll	simcop2387	f	1
1411	667	Shrimp Spread	Bubba	f	3
1522	667	Shrimp Curry	Bubba	f	3
1500	667	Citrus Shrimp and Scallops	Bubba	f	3
1496	667	Walnut Chicken/Shrimp Stir-Fry	Bubba	f	3
1426	667	Ginger-Garlic Shrimp, China Royal	Bubba	f	3
1429	667	Shrimp-Cheese Roll	Bubba	f	3
1509	667	Corn Chowder with Shrimp	Bubba	f	3
1441	667	SPICY TOMATILLO SHRIMP	Bubba	f	3
1414	667	Baby Shrimp And Tarragon Mustard	Bubba	f	3
1461	667	Sadies Shrimp Salad Dressing	Bubba	f	3
1456	667	Pasta With Asparagus And Shrimp	Bubba	f	3
1501	667	Ham & Shrimp Creole	Bubba	f	3
1503	667	Pizza with shrimp	Bubba	f	3
1526	667	Stuffed Shrimp with Bacon	Bubba	f	2
1427	667	Grilled Shrimp	Bubba	f	3
1432	667	Assam Spicy Shrimp	Bubba	f	3
1423	667	Shrimp Newburg	Bubba	f	3
1417	667	Layered Shrimp Crepe	Bubba	f	4
1450	667	Spring Shrimp Salad	Bubba	f	3
1487	667	Shrimp Delgado	Bubba	f	3
1444	667	TIGER SHRIMP	Bubba	f	3
1488	667	Marinated Gingered Shrimp	Bubba	f	3
1659	25	Blend a TEAPOT SHOT and the pearlies won't rot	simcop2387	f	1
1435	667	Acadian Peppered Shrimp	Bubba	f	3
1454	667	Charleston Shrimp Pie	Bubba	f	3
1462	667	Shrimp Creole Salad	Bubba	f	3
1464	667	Shrimp Paste Dipping Sauce - Nam Prik Goong Siap *	Bubba	f	3
1472	667	Grecian Style Shrimp Scampi	Bubba	f	3
1412	667	MTKs Sauced Shrimp	Bubba	f	3
1483	667	New Orleans Shrimp Creole	Bubba	f	3
1518	667	Thai Shrimp-Chicken Soup	Bubba	f	3
1434	667	Shrimp Gumbo	Bubba	f	3
1453	667	Rogers Scampi	Bubba	f	3
1415	667	Sherried Shrimp Filling	Bubba	f	3
1521	667	Shrimp and Veggie Curry	Bubba	f	3
1409	667	Crawfish or Shrimp Cocktail	Bubba	f	3
1499	667	Cabbage Stuffed with Orzo and Shrimp	Bubba	f	3
1513	667	Shrimp Chowder	Bubba	f	4
1495	667	Walnut Shrimp	Bubba	f	4
1437	667	Fried Shrimp Balls	Bubba	f	4
1399	666	xpcom_compat.dll	simcop2387	f	1
1523	667	Shrimp Scampi over Linguini	Bubba	f	4
1498	667	Steamed Fish With Shrimp Gravy	Bubba	f	4
1413	667	Shrimp Mold	Bubba	f	4
1457	667	Pasta Ala Oglio with Shrimp	Bubba	f	4
1431	667	Angel Hair Pasta with Shrimp	Bubba	f	3
1448	667	Pickled shrimp	Bubba	f	3
1502	667	Shrimp Baleares	Bubba	f	3
1425	667	Venezuelan Peppers With Shrimp	Bubba	f	3
1512	667	Shrimp Soup	Bubba	f	3
1460	667	Sadies Shrimp Salad	Bubba	f	3
1418	667	Baked Eggplant with Shrimp	Bubba	f	3
1474	667	Spicy Marinated Shrimp and Garlic	Bubba	f	3
1473	667	Sweet Cajun Shrimp	Bubba	f	3
1455	667	Clay Pot Shrimp With Bean Thread Noodles - Goong Ope Maw	Bubba	f	3
1439	667	Crispy Coconut Shrimp	Bubba	f	3
1494	667	Shrimp Fra Diavolo	Bubba	f	3
1430	667	Curried Apples and Shrimp	Bubba	f	3
1452	667	Garlic Shrimp Au Gratin	Bubba	f	3
1442	667	Prawn Vindaloo (Hot Prawns)	Bubba	f	3
1527	667	Curried Shrimp and Macaroni Salad	Bubba	f	3
1505	667	Shrimp in Adobo Sauce	Bubba	f	3
1482	667	Almond Fried Shrimp	Bubba	f	3
1485	667	Shrimp & Scallops in Wine Sauce	Bubba	f	3
1480	667	Coconut Beer Batter Shrimp	Bubba	f	3
1466	667	Pasta with Shrimp and Vegetables	Bubba	f	3
1481	667	Grilled Tomatillo Shrimp	Bubba	f	3
1515	667	Steak-and-Shrimp Kabobs	Bubba	f	3
1528	667	Shrimp Delight	Bubba	f	3
1497	667	Dilled Shrimp	Bubba	f	3
1438	667	Hot and Spicy Shrimp	Bubba	f	3
1440	667	Brigittes Shrimp or Tuna Mousse	Bubba	f	3
1507	667	Shrimps in Chipotle Sauce	Bubba	f	3
1433	667	Shrimp & Scallop Kabobs	Bubba	f	3
1465	667	Crab-Topped Shrimp	Bubba	f	3
1451	667	Garlic Shrimp	Bubba	f	3
1486	667	Lemon Herb Barbecued Shrimp	Bubba	f	3
1510	667	Hot & Sour Shrimp Soup	Bubba	f	3
1477	667	Faux Shrimp	Bubba	f	3
1467	667	Fiery Shrimp Curry	Bubba	f	3
929	666	libbonobo-activation-4.dll	simcop2387	f	1
203	666	FlashMobilePlayer.dll	simcop2387	f	1
410	666	NTMSG.DLL	simcop2387	f	1
1633	23	Rock-Man has a filthy mouth because he ran out of gum	simcop2387	t	400
1685	17	Direct Download Link: DDL IS CURRENTLY DOWN! The DDL will not be back until Matlock returns for another season.	Zdm321	t	222
1638	25	Winter Coat worn for a Mixer	simcop2387	f	0
1388	666	ws2_32.dll	simcop2387	f	1
350	666	MSXMLR.DLL	simcop2387	f	1
1150	666	npqtplugin.dll	simcop2387	f	1
399	666	NAnt.SourceControlTasks.dll	simcop2387	f	1
1225	666	policy.2.8.gdk-sharp.dll	simcop2387	f	1
1641	25	A Heart attack could put you into the ground	simcop2387	f	0
1642	25	Battleground	simcop2387	f	0
647	666	cli_oootypes.dll	simcop2387	f	1
505	666	System.Drawing.Design.dll	simcop2387	f	1
1557	17	http://forums.dctp.ws/index.php?topic=710.0	simcop2387	t	7
824	666	gtk-sharp.dll	simcop2387	f	1
1445	667	Chicken Acapulco with Creamy Shrimp Sauce	Bubba	f	3
1525	667	Coconut Beer Shrimp	Bubba	t	2
1558	17	Temporary: ddl.dctp.ws : ddl : ?h59uqVpnG	Zdm321	t	16
971	666	libgnomeui-2-0.dll	simcop2387	f	1
1559	17	Temporary: ddl.dctp.ws : user ddl : password at http://ddl.dctp.ws/pass.txt	simcop2387	t	181
1115	666	mssetup.dll	simcop2387	f	1
511	666	System.Runtime.Serialization.Formatters.Soap.dll	simcop2387	f	1
1651	25	A distant, ancient Castle Keep This famous Prince a place to sleep To sleep, perchance to dream Of an upset teagarden indochine	simcop2387	f	0
870	666	inet16.dll	simcop2387	f	1
1652	25	A man-horse on the fly sounds like a wounded bull's eye	simcop2387	f	0
1575	17	DDL bot is down, when its back do !delquote (QuoteNumberHere)	Zdm321	t	72
1628	1	Shinhei loves ShinHei	Shinhei	t	34
1653	25	Put an olive in a stein, mix it up, and get the equivalent of a fool's London subway	simcop2387	f	1
1654	25	A Vital, instrumental part	simcop2387	f	1
1655	25	22233642-736846873	simcop2387	f	1
1589	20	Soy Sauce <episode 412>	MrWiz88	f	4
1587	20	Lemon Pie <episode 99>	MrWiz88	f	5
1590	20	Location on a Watch <episode 191>	MrWiz88	f	4
1421	667	Shrimp Casserole Charleston	Bubba	f	3
1489	667	Spicy Hot Shrimp Cocktail	Bubba	f	3
1469	667	Wolfgang Pucks Barbecued Shrimp BLT	Bubba	f	4
1478	667	Barbecued Shrimp and Chicken	Bubba	f	4
1424	667	Shrimp Creole	Bubba	f	3
1475	667	Shrimp curry	Bubba	f	3
1508	667	Asian Shrimp & Noodle Soup	Bubba	f	3
1524	667	Grilled Shrimp with Rosemary - Martha Stewart Living	Bubba	f	3
1416	667	Curried Shrimp Toasts	Bubba	f	3
1530	9	back in moddle schoole [Gantz]	simcop2387	f	37
1490	667	Poor Mans Shrimp	Bubba	f	3
1479	667	Shrimp Delgado #2	Bubba	f	4
1458	667	Sauteed Shrimp with Curried Banana Sauce	Bubba	f	4
1506	667	Shrimp Quesadillas	Bubba	f	4
1420	667	Shrimp and Rice Casserole	Bubba	f	4
1517	667	Cheese-Shrimp Chowder	Bubba	f	3
1484	667	Charleston Shrimp Supreme	Bubba	f	3
1428	667	Shrimp in Indian Sauce	Bubba	f	4
1514	667	Hot and Sour Shrimp Soup - Tom Yam Goong *	Bubba	f	4
1471	667	Broccoli with Shrimp Sauce	Bubba	f	4
1436	667	Elegant Southern Seafood Cake	Bubba	f	3
1449	667	Shrimp Rolls	Bubba	f	4
1492	667	Shrimp & Scallop Kabobs from Fred Goslin	Bubba	f	4
1520	667	Spaghetti Squash With Mushrooms and Shrimp	Bubba	f	4
1422	667	Shrimp Louis Dip	Bubba	f	4
1446	667	Chili Shrimp with Basil	Bubba	f	4
1459	667	Shrimp Fried Rice, Shanghai	Bubba	f	4
369	666	Mono.CompilerServices.SymbolWriter.dll	simcop2387	f	1
1339	666	vbscript.dll	simcop2387	f	1
1586	20	Night Baron <episode 42>	MrWiz88	f	4
373	666	Mono.Data.SybaseClient.dll	simcop2387	f	1
1640	25	Artsy, excited lecher	simcop2387	f	1
1639	25	Rolling rock, bottle cap	simcop2387	f	1
1504	667	Avocado, Shrimp and Cucumber Makimono	Bubba	f	3
1519	667	Shrimp-Stuffed Mirliton	Bubba	f	3
1544	11	Statistics say Teen pregnancy drops of significantly after age 25	simcop2387	f	34
1585	20	Lipstick <episode 98>	MrWiz88	f	5
1573	667	Deep Fried Shrimp in Cow Eye Sauce	IceGodNakakaul	f	3
1588	20	Heads and Tails <episode 411>	MrWiz88	f	5
1529	667	Seafood Enchiladas	Bubba	f	3
1656	25	Light piece from great orchestra	simcop2387	f	1
1591	20	Finger Print <episode 415>	MrWiz88	f	4
1592	20	Siblings <episode 416>	MrWiz88	f	4
1593	20	Nameplate <episode 417>	MrWiz88	f	4
1572	14	Testing 1 2 3	simcop2387	t	17
1594	20	Exspensive Vase <episode 492>	MrWiz88	t	4
1595	20	Footstool <episode 493>	MrWiz88	f	5
1580	17	Still down	Atticus	t	87
1574	17	DDL bot is down, READ THAT ITS DOWN! OPS ONLY WHEN IT IS BACK DO !deletequote quotenumberhere	simcop2387	t	359
1583	17	Its valentines day, what are you doing here trying to get conan for? also the ddl is down.	simcop2387	t	123
1596	20	Seven Children <episode 494>	MrWiz88	f	3
1598	20	Miyano Akemi <episode 496>	MrWiz88	f	4
1599	20	Ballpoint pen <episode 07>	MrWiz88	f	5
1605	20	Bank Book <episode 201>	MrWiz88	f	4
1602	20	Luminescant Substance <episode 178>	MrWiz88	t	3
1604	20	Luminescant Substance <episode 177>	MrWiz88	f	4
1608	20	Calendar <episode 381>	MrWiz88	f	3
1609	20	Retired Unifrom Number <episode 382>	MrWiz88	f	3
1612	20	Bluebird <episode 511>	MrWiz88	f	5
1613	20	Gum <episode 432>	MrWiz88	f	4
1617	20	Expensive Vase <episode 492>	MrWiz88	f	4
966	666	libgnomeprint-select.dll	simcop2387	f	1
1223	666	policy.2.8.atk-sharp.dll	simcop2387	f	1
1080	666	mscomstf.dll	simcop2387	f	1
1621	11	<kitten> im not a pedo :(	MrWiz88	t	22
1629	9	His name is: Detective Conan!	dc30wn4g3	t	15
221	666	Gendarme.Rules.Performance.dll	simcop2387	f	1
721	666	dpnhupnp.dll	simcop2387	f	1
344	666	MSVIDEO.DLL	simcop2387	f	1
501	666	System.Data.OracleClient.dll	simcop2387	f	1
942	666	libgconf-2-4.dll	simcop2387	f	1
1681	11	<Jd-> Canvas is actually a nice name for you, Zdm  <Jd-> Think about it  <Jd-> It's just a way of saying, "Come paint over us <3"	Jing	t	26
1389	666	wshcon.dll	simcop2387	f	1
379	666	Mono.Mozilla.dll	simcop2387	f	1
614	666	asycfilt.dll	simcop2387	f	1
1019	666	libvorbisenc-2.dll	simcop2387	f	1
1305	666	strgcmps.dll	simcop2387	f	1
1552	16	Random Conan Facts are a detriment to normal society, they prevent people from passing exams and prevent Conan from getting laid	simcop2387	f	20
1163	666	nssutil3.dll	simcop2387	f	1
1570	14	Shinichi won't go 69 with a girl because he thinks that girls are icky	simcop2387	f	66
1603	20	Bus Schedule <episode 178>	MrWiz88	f	4
1611	20	Mini-Car <episode 509>	MrWiz88	f	5
1606	20	Lunch Box <episode 202>	MrWiz88	f	5
1531	11	<Kaitoukidsama> i left wendy's in the middle of a shit..... they prolly won't let me back no matter the cricumstances <Kaitoukidsama> *shift <Kaitoukidsama> >.> <Jd> lmao	Zdm321	f	34
1560	3	Jd is left-handed	Ohnoesmilk	f	33
1561	2	Porn on his phone (ep 411-412)	Ohnoesmilk	f	18
1540	2	Stealing a bike and getting it blown up (movie 1)	simcop2387	f	18
1568	3	Jd was afraid of a giant T-Rex showing up when he was little	cancerkani	f	33
1534	3	Jd supports 118v2 by NEG	Zdm321	f	33
1551	11	<droffi> seen alot , but got none on this comp so wanted some number so easier to dl tit	CooKieLord	f	33
1569	3	[21:24:44pm] <Jd> Jd doesn't love Naifu, he fucking LOVES Naifu	Zdm321	f	33
1543	2	Tampering with evidence ("Suicide" will in Live Action 1)	CooKieLord	f	18
1419	667	Arroz Amarillo con Camarones -Yellow Rice & Shrimp Casserole	Bubba	f	3
1554	11	http://www.youtube.com/watch?v=Yu_moia-oVI&feature=related	CooKieLord	t	34
1549	3	Jd doesn't like baked cookies.	CooKieLord	f	33
1547	1	Conan has perfect pitch.	CooKieLord	f	69
1584	9	[01:32:50] <Jd-> Yes, I still love you all	simcop2387	f	38
1537	1	Conan has two cell phones.	CooKieLord	f	69
1607	20	Trace of Tire Slip <episode 208>	MrWiz88	f	4
1556	11	http://www.youtube.com/watch?v=A8SksS4dM9U	Ohnoesmilk	f	34
1541	2	Breaking and Entering (all over the place!)	simcop2387	f	18
1632	22	First rule of DCTV is do not talk about DCTV	simcop2387	f	10
1581	9	however he had already hiden the goin coins on us. <conan epi 4>	MrWiz88	f	38
1538	3	Jd's hovercraft is NEVER full of eels :(	simcop2387	f	34
1511	667	Kidney Bean Soup with Chopped Shrimp & Chicken	Bubba	f	3
1539	11	[23:44] <Haruka-chan> Like Bonan! Mayumi, Mikihiko, and KEnta!	detective	f	35
1533	3	Jd is a BIG fan of XCHKS	simcop2387	f	34
1601	20	Romanji <episode 167>	MrWiz88	f	5
1536	12	KaitoKid is really the father of Sato's child, not Takagi as Gosho would like you to believe.	simcop2387-lab	f	35
1545	15	SECURE CONNECTION ESTABLISHED	Zdm321	f	34
1548	1	Conan doesn't understand women	devedo	f	70
1571	3	Don't let him talk about music, or he'll never finish anything.	cancerkani	f	34
1706	27	Hey woman!	simcop2387	f	2
1708	27	Do you know me? Of course you do. 'Cause I'm famous!	simcop2387	f	2
1470	667	Honey-Thyme Shrimp	Bubba	f	3
1619	9	"Seeing through the only one fact, kid-looking but have grown wit and intelligence, yes his or my name is Meitantei Conan The Detecitve Conan." [ANFS # 443]	simcop2387	f	36
1631	21	There is no downloading in DCTV!	simcop2387	f	10
1614	20	Fox(kitsune) <episode 516>	MrWiz88	f	4
1600	20	Misa, Sae, Emi <episode 166>	MrWiz88	f	5
1597	20	Scar <episode 495>	MrWiz88	f	5
1661	25	Poor drainage could still produce a flower	simcop2387	f	0
1662	25	Sounds like it got higher from wine	simcop2387	f	1
1663	25	What kind of jewelry is angrier?	simcop2387	f	0
1664	25	Instrument is sharp, but missing its head	simcop2387	f	0
1665	25	A defective truck with a crane makes for a ball-busting ballot	simcop2387	f	1
1666	25	Key missing first misprinted label	simcop2387	f	0
1667	25	Disabled cutting edge	simcop2387	f	1
1668	25	Unreasonable reason	simcop2387	f	0
1669	25	Paper used in unusual theses	simcop2387	f	1
1670	25	If you're adroit, you're handy, smart, clever..	simcop2387	f	0
1671	25	A desserted Arthropod	simcop2387	f	0
1698	27	No, I don't hate Balboa. I pity the fool.	simcop2387	f	2
1672	25	663 264625 46 2 6455466	simcop2387	f	1
1673	25	Drink left at sea	simcop2387	f	0
1674	25	Snake, baby, trap	simcop2387	f	1
1675	25	A letter from Greece is quite a number in Rome	simcop2387	f	0
3191	37	The Monster Gomera Murder Case (Episode 63)	shiranpuri	f	0
3192	37	The Third Set of Fingerprints Murder Case (Episode 64)	shiranpuri	f	0
3193	37	The Crabs and Whales Kidnapping Case (Episode 65)	shiranpuri	f	0
169	666	CmdLineExt03.dll	simcop2387	f	1
3194	37	The Dark Street Murder Case (Episode 66)	shiranpuri	f	0
672	666	csseqchk.dll	simcop2387	f	1
1677	14	don't say Candle Jack	Candlejack	t	33
1616	17	DDL IS CURRENTLY DOWN! The DDL will not be back until Matlock returns for another season.	Zdm321	t	924
1683	11	[20:30] <@Jd-> Canvas is actually a nice name for you, Zdm <@Jd-> Think about it <@Jd-> It's just a way of saying, "Come paint over us <3"	Kaitoukidsama	t	22
1692	20	Pregnancy Test!	simcop2387	f	4
1693	11	[00:57:43] <Zdm321> yeah, I was ganged >_<	simcop2387	t	23
1748	27	Teachin' fools some basic rules! (I pity the fool)	simcop2387	f	2
1736	27	Be there!	simcop2387	f	2
1770	28	The Four Horsemen of the Apocalypse actually live in Chuck Norris's nutsack.	simcop2387	f	2
1735	27	What!?! Idiot shot the tires on my van!!	simcop2387	f	2
1728	27	I ain't getting on no plane, Hannibal!	simcop2387	f	2
1745	27	I don't hate fools, I pity them! (I pity the fool)	simcop2387	f	1
1759	28	Chuck Norris is 1/8th Cherokee. This has nothing to do with ancestry, the man ate a fucking Indian.	simcop2387	f	3
1722	27	It takes a smart guy to play dumb.	simcop2387	f	2
1684	11	<&jmaeshawn> well, there's no normal girls in any of the places i go to at school  <&jmaeshawn> i know there has to be some somewhere  @Jd- is betting you're talking about the guy's locker room now	Kaitoukidsama	f	34
1624	9	the bread douche is too thick	simcop2387	f	34
1676	26	DDL is down! down down down!	simcop2387	f	32
1678	14	kogoro in fact wants there to be more farside cartoons to be made.	simcop2387	f	64
1565	14	Haibara is really Shinichi's Sister, neither of them knows it.	simcop2387	f	66
1610	20	Soaked Jacket <episode 512>	MrWiz88	f	5
1579	6	i also work in pm's! (i also have a help menu in pm only!)	simcop2387	f	26
1623	9	the bread douche is too think	simcop2387	f	37
1703	27	I'm gonna bust you up.	simcop2387	f	3
1690	3	Conan was wrong all the time. Actually, in most cases Jd was the murderer.	FujiwaranoSai	f	34
1576	13	the ddl got deleted because some brazillian got his dick stuck in a meat grinder and decided to toss packets at it.	simcop2387	f	43
1577	19	Sonoko nee-chan your panties are showing!	simcop2387	f	39
1705	27	You're washed up, finished! I'm the baddest in the world!	simcop2387	f	3
1622	9	[Zero In, chp 19] Japane is ruled by lights and darks	simcop2387	f	38
1679	1	"detective conan" is an anagram for: "cat not evidence"	simcop2387	f	70
1711	27	Calvin Klein and Gloria Vanderbilt don't wear clothes with your name on it, so why should you wear their name? --Be Somebody or Be Somebody's Fool!	simcop2387	f	3
1627	1	physics doesnt affect Conan	dc30wn4g3	f	69
1737	27	It's a crack baby....FOOL	simcop2387	f	3
1733	27	I pity the fool, thug, or soul who tries to take over the world.	simcop2387	f	3
56	1	Before Shinichi's dad, Yusaku, was properly introduced in episode 43 (based on manga), He was mentioned and referenced in episode 33 (anime original), which goes to show why AOs are messed up	conankudo	f	68
1721	27	For 5 years Mr. T disappeared. Fools went unpitied and Jibba-Jabba went unchallenged!	simcop2387	f	3
1732	27	I'm on a real short leash here, and I'm tired of your crazy rap!	simcop2387	f	2
1726	27	Hey, sucka!	simcop2387	f	2
1717	27	As a kid, I got three meals a day. Oatmeal, miss-a-meal and no meal.	simcop2387	f	3
1704	27	What'd you say to me paper champion?	simcop2387	f	3
1566	14	Conan wants a cracker	simcop2387	f	66
1578	6	You can also see all my triggers by doing; !showtriggers	simcop2387	f	31
1567	14	Haibara only has epiphanies when she's in a 69, thats why she hasn't been able to cure Shinichi, he won't do that with her.	simcop2387	f	66
1752	28	If you can see Chuck Norris, he can see you. If you can't see Chuck Norris you may be only seconds away from death.	simcop2387	f	3
1767	28	Chuck Norris once ate an entire ream of rice paper and shat out origami swans and Mister Miyagi from Karate Kid.	simcop2387	f	3
1749	28	Chuck Norris' tears cure cancer. Too bad he has never cried.	simcop2387	f	3
1763	28	Chuck Norris is currently suing NBC, claiming Law and Order are trademarked names for his left and right legs.	simcop2387	f	3
1751	28	Chuck Norris does not hunt because the word hunting infers the probability of failure. Chuck Norris goes killing.	simcop2387	f	3
1762	28	The chief export of Chuck Norris is pain.	simcop2387	f	3
1716	27	I'm Mr. T and I'm a "Night Elf Mohawk"!	simcop2387	f	2
1720	27	When I was growing up, my family was so poor we couldn't afford to pay attention.	simcop2387	f	2
1731	27	Don't make me mad, Arrr!	simcop2387	f	2
1744	27	Hey fool, this ain't no football game! (A-Team)	simcop2387	f	2
1701	27	I can't be beat and I won't be beat!	simcop2387	f	2
1746	27	Life's tough, but I'm tougher! (I pity the fool)	simcop2387	f	3
1730	27	I got no time for the jibba-jabba.	simcop2387	f	2
1713	27	Quit yo Jibber-jabber! You ain't hurt, yo pathetic!, Argh! If I ever catch you out like a crazy fool again, you're gonna meet my friend pain!, Snickers, get some nuts!	simcop2387	f	2
1702	27	Pain...	simcop2387	f	3
1699	27	I'm gonna torture him. I'm gonna crucify him. Real bad.	simcop2387	f	2
1719	27	I believe in the Golden Rule - The Man with the Gold... rules.	simcop2387	f	3
1740	27	Take it from me...Mr.T	simcop2387	f	3
1729	27	Where's the meat? This sandwich is full of weeds! I ain't eatin' nothing I don't understand!	simcop2387	f	3
1712	27	Anger - use it, but don't lose it! --Be Somebody or Be Somebody's Fool!	simcop2387	f	3
1765	28	Crop circles are Chuck Norris' way of telling the world that sometimes corn needs to lie the fuck down.	simcop2387	f	2
1739	27	I pity that chump Conan O'Brien.	simcop2387	f	2
1741	27	8! 8! That's only a third, and third rhymes with Turd!	simcop2387	f	2
1754	28	When the Boogeyman goes to sleep every night he checks his closet for Chuck Norris.	simcop2387	f	2
1766	28	Chuck Norris once commented, "There are few problems in this world that cannot be solved by a swift roundhouse kick to the face. In fact, there are none."	simcop2387	f	2
1724	27	I pity the fool who did your mother!	simcop2387	f	2
1771	28	When observing a Chuck Norris roundhouse kick in slow motion, one finds that Chuck Norris actually rapes his victim in the ass, smokes a cigarette with Dennis Leary, and then roundhouse kicks them in the face.	simcop2387	f	2
1802	28	Chuck Norris doesn't worry about changing his clock twice a year for daylight savings time. The sun rises and sets when Chuck tells it to.	simcop2387	f	2
1785	28	Chuck Norris once kicked a baby elephant into puberty	simcop2387	f	2
1789	28	Once a grizzly bear threatened to eat Chuck Norris. Chuck showed the bear his fist and the bear proceeded to eat himself, because it would be the less painful way to die.	simcop2387	f	2
1783	28	When Chuck Norris had surgery, the anesthesia was applied to the doctors.	simcop2387	f	2
1803	28	Chuck Norris only masturbates to pictures of Chuck Norris.	simcop2387	f	2
1714	27	Shut up, fool!	simcop2387	f	3
1786	28	Multiple people have died from Chuck Norris giving them the finger.	simcop2387	f	3
1804	28	Filming on location for Walker: Texas Ranger, Chuck Norris brought a stillborn baby lamb back to life by giving it a prolonged beard rub. Shortly after the farm animal sprang back to life and a crowd had gathered, Chuck Norris roundhouse kicked the animal, breaking its neck, to remind the crew once more that Chuck giveth, and the good Chuck, he taketh away.	simcop2387	f	2
1769	28	Chuck Norris is a man of few words. Chuck Norris is not a man of few roundhouse kicks to the face.	simcop2387	f	3
1793	28	Chuck Norris ordered a Big Mac at Burger King, and got one.	simcop2387	f	2
1761	28	In fine print on the last page of the Guinness Book of World Records it notes that all world records are held by Chuck Norris, and those listed in the book are simply the closest anyone else has ever gotten.	simcop2387	f	3
1800	28	When asked what type of vehicle he drives, Chuck Norris responded slyly with "Don't you mean what kind of vehicle drives me?"	simcop2387	f	3
1747	27	The jibba jabba stops here! (I pity the fool)	simcop2387	f	3
1757	28	They once made a Chuck Norris toilet paper, but it wouldn't take shit from anybody.	simcop2387	f	3
1727	27	Shut up Murdock, crazy fool!	simcop2387	f	3
1809	28	Although it is not common knowledge, there are actually three sides to the Force: the light side, the dark side, and Chuck Norris.	simcop2387	f	3
1707	27	First name Mr, middle name 'period', last name T!	simcop2387	f	3
1743	27	You gonna lose a deal over $35? Thats chump change! My lunch cost $35!	simcop2387	f	3
1738	27	I pity Screech, because everybody pitied Screech.	simcop2387	f	3
1780	28	Chuck Norris is what Willis was talking about	simcop2387	f	3
1715	27	Well, maybe Mr. T hacked the game and created a Mohawk class! Maybe, Mr. T's pretty handy with computers! Had that occurred to you, Mr. "Condescending" Director?!	simcop2387	f	3
1718	27	I pity the fool who drinks soy milk.	simcop2387	f	3
1564	14	Conan Edogawa is the Alpha and the Omega. Also the Beta and Gamma too, but thats only in the AOs	simcop2387	f	65
-606	-606	10,01Episode 15,01[1160615,01] Courtroom Confrontation IV: Juror Kobayashi Sumiko (Part One)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [C803C1DB/A6188588]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=C5B0TPYQ | 720: http://www.megaupload.com/?d=QOFZI6G504,01 || 07Aired 2/19/201107,01	\N	f	3
1775	28	Chuck Norris kicked Neo out of Zion , now Neo is "The Two"	simcop2387	f	3
1725	27	You don't rehearse Mr T, you just turn him loose.	simcop2387	f	3
1794	28	Chuck Norris always has sex on the first date. Always.	simcop2387	f	3
1764	28	The Great Wall of China was originally created to keep Chuck Norris out. It failed misserably.	simcop2387	f	3
1792	28	Chuck Norris can touch MC Hammer.	simcop2387	f	3
1796	28	Chuck Norris frequently donates blood to the Red Cross. Just never his own.	simcop2387	f	3
1795	28	A Handicap parking sign does not signify that this spot is for handicapped people. It is actually in fact a warning, that the spot belongs to Chuck Norris and that you will be handicapped if you park there.	simcop2387	f	3
1756	28	Chuck Norris has already been to Mars; that's why there are no signs of life there.	simcop2387	f	3
1790	28	If Chuck Norris is late, time better slow the fuck down	simcop2387	f	3
1750	28	Chuck Norris counted to infinity - twice.	simcop2387	f	3
1791	28	Chuck Norris sleeps with a night light. Not because Chuck Norris is afraid of the dark, but the dark is afraid of Chuck Norris	simcop2387	f	3
1787	28	Chuck Norris once tried to wear glasses. The result was him seeing around the world to the point where he was looking at the back of his own head.	simcop2387	f	2
1755	28	Chuck Norris built a time machine and went back in time to stop the JFK assassination. As Oswald shot, Chuck Norris met all three bullets with his beard, deflecting them. JFK's head exploded out of sheer amazement.	simcop2387	f	2
1799	28	Chuck Norris does not procreate, he breeds	simcop2387	f	3
1779	28	China was once bordering the United States, until Chuck Norris roundhouse kicked it all the way through the Earth.	simcop2387	f	3
1776	28	Chuck Norris' iPod came with a real charger instead of just a USB cord	simcop2387	f	3
1777	28	Chuck Norris knows where Carmen Sandiego is.	simcop2387	f	3
1788	28	Pee Wee Herman got arrested for masturbating in public. The same day, Chuck Norris got an award for masturbating in public.	simcop2387	f	3
1797	28	There is no such thing as tornados. Chuck Norris just hates trailer parks.	simcop2387	f	3
1781	28	If you have five dollars and Chuck Norris has five dollars, Chuck Norris has more money than you.	simcop2387	f	3
-20020	-20020	15,01[11OVA2015,01]	\N	f	0
1807	28	If paper beats rock, rock beats scissors, and scissors beats paper, what beats all 3 at the same time? Answer: Chuck Norris.	simcop2387	f	3
1798	28	Chuck Norris never cries, because of this when he's sad he roundhouse kicks himself and it makes him feel better since he knows he is the only one who can survive the roundhouse.	simcop2387	f	2
1723	27	I said "Hell no, because you didn't find a weapon; or if that's the charge you would have to lock me up because my hands and feet are deadly weapons."	simcop2387	f	3
1784	28	Chuck Norris once broke the land speed record on a bicycle that was missing its chain and the back tire.	simcop2387	f	2
1841	28	Chuck Norris once walked down the street with a massive erection. There we no survivors.	simcop2387	f	2
1833	28	In the original pilot for Star Trek Next Generation, Chuck Norris can be seen powering the USS Enterprise warp drive with his roundhouse kicks.	simcop2387	f	2
1774	28	Chuck Norris made Ellen Degeneres straight.	simcop2387	f	3
1848	28	There are no disabled people. Only people who have met Chuck Norris.	simcop2387	f	3
1843	28	Chuck Norris doesnt have AIDS but he gives it to people anyway.	simcop2387	f	2
1832	28	Before science was invented it was once believed that autumn occurred when Chuck Norris roundhouse kicked every tree in existence.	simcop2387	f	2
1822	28	Chuck Norris eats transformer toys in vehicle mode and poos them out transformed into a robot.	simcop2387	f	2
1821	28	If Superman and The Flash were to race to the edge of space you know who would win? Chuck Norris.	simcop2387	f	2
1868	28	Chuck Norris does not "style" his hair. It lays perfectly in place out of sheer terror.	simcop2387	f	2
1871	29	List of Available Detective Conan Fansubs & Scanlations :: http://forums.dctp.ws/index.php?topic=3.0	simcop2387	f	1
1886	29	Other Favorite Anime :: http://forums.dctp.ws/index.php?topic=20.0	conanquotes	f	1
1887	29	- EE's Graphics - :: http://forums.dctp.ws/index.php?topic=22.0	conanquotes	f	1
1888	29	- Code Geass: Lelouch of the Rebellion  - :: http://forums.dctp.ws/index.php?topic=23.0	conanquotes	f	1
1872	29	The Rules & FAQ :: http://forums.dctp.ws/index.php?topic=5.0	conanquotes	f	1
1873	29	[DCTP] Detective Conan Episode 498 :: http://forums.dctp.ws/index.php?topic=6.0	conanquotes	f	1
1874	29	Next Episode Appearance? :: http://forums.dctp.ws/index.php?topic=7.0	conanquotes	f	1
1875	29	- Current Arc - :: http://forums.dctp.ws/index.php?topic=8.0	conanquotes	f	1
1876	29	- Favorite Movie - :: http://forums.dctp.ws/index.php?topic=10.0	conanquotes	f	1
1877	29	Thanks for this forum :: http://forums.dctp.ws/index.php?topic=11.0	conanquotes	f	1
1878	29	- Current Arc - (ANIME VIEWERS)(UPDATED) :: http://forums.dctp.ws/index.php?topic=12.0	conanquotes	f	1
1879	29	Movie 11 :: http://forums.dctp.ws/index.php?topic=13.0	conanquotes	f	1
1880	29	Trouble remembering.... :: http://forums.dctp.ws/index.php?topic=14.0	conanquotes	f	1
1881	29	Due props to my Tantei Conan heroes - the subbers :: http://forums.dctp.ws/index.php?topic=15.0	conanquotes	f	1
1882	29	Interview with Jd- :: http://forums.dctp.ws/index.php?topic=16.0	conanquotes	f	1
1883	29	Subtittle edit :: http://forums.dctp.ws/index.php?topic=17.0	conanquotes	f	1
1884	29	Naruto Anyone? (Manga Spoilers) :: http://forums.dctp.ws/index.php?topic=18.0	conanquotes	f	1
1885	29	Suggestion for topics :: http://forums.dctp.ws/index.php?topic=19.0	conanquotes	f	1
1836	28	Scientists in Washington have recently conceded that, if there were a nuclear war, all that would remain are cockroaches and Chuck Norris.	simcop2387	f	3
1851	28	a meteor, and still owes him a beer.	simcop2387	f	2
1865	28	Nobody doesn't like Sara Lee. Except Chuck Norris.	simcop2387	f	3
1813	28	Chuck Norris drinks napalm to quell his heartburn.	simcop2387	f	3
1778	28	Rudolph has a red nose because he got lippy and Chuck Norris roundhouse kicked him across the face several times	simcop2387	f	3
1819	28	Chuck Norris has never blinked in his entire life. Never.	simcop2387	f	3
1801	28	Chuck Norris' evil twin brother, Richard Simmons, once approached Chuck with the hope of reconciliation, but at the sight of Richard's curly, well kept hair, Chuck Norris became so enraged that he turned green with hate and ripped Richard Simmons arms and legs off. This action was the origin of the Marvel Comic badass, The Incredible Hulk.	simcop2387	f	3
1828	28	One time while sparring with Wolverine, Chuck Norris accidentally lost his left testicle. You might be familiar with it to this very day by its technical term: Jupiter.	simcop2387	f	3
1849	28	Chuck Norris won 'Jumanji' without ever saying the word. He simply beat the living shit out of everything that was thrown at him, and the game forfeited.	simcop2387	f	3
1830	28	Contrary to popular belief, Chuck Norris, not the box jellyfish of northern Australia, is the most venomous creature on earth. Within 3 minutes of being bitten, a human being experiences the following symptoms: fever, blurred vision, beard rash, tightness of the jeans, and the feeling of being repeatedly kicked through a car windshield.	simcop2387	f	3
1815	28	Chuck Norris once tried to defeat Garry Kasparov in a game of chess. When Norris lost, he won in life by roundhouse kicking Kasparov in the side of the face.	simcop2387	f	3
1825	28	Chuck Norris invented water.	simcop2387	f	3
1844	28	When you open a can of whoop-ass, Chuck Norris jumps out.	simcop2387	f	3
1753	28	Chuck Norris sold his soul to the devil for his rugged good looks and unparalleled martial arts ability. Shortly after the transaction was finalized, Chuck roundhouse kicked the devil in the face and took his soul back. The devil, who appreciates irony, couldn't stay mad and admitted he should have seen it coming. They now play poker every second Wednesday of the month.	simcop2387	f	3
1850	28	Chuck Norris once bet NASA he could survive re-entry without a spacesuit. On July 19th, 1999, a naked Chuck Norris re-entered the earth's atmosphere, streaking over 14 states and reaching a temperature of 3000 degrees. An embarrassed NASA publicly claimed it was	simcop2387	f	3
-20021	-20021	15,01[11OVA2115,01]	\N	f	0
1889	29	Interview with Rajhaziel :: http://forums.dctp.ws/index.php?topic=24.0	conanquotes	f	1
1890	29	- Video Game System - :: http://forums.dctp.ws/index.php?topic=25.0	conanquotes	f	1
1892	29	Detective Conan OVA 8 :: http://forums.dctp.ws/index.php?topic=27.0	conanquotes	f	1
1893	29	Episodes i think that should be considered being subbed :: http://forums.dctp.ws/index.php?topic=28.0	conanquotes	f	1
1894	29	- Detective Conan Movie: Full Score of Fear(movie 12) - :: http://forums.dctp.ws/index.php?topic=29.0	conanquotes	f	1
1895	29	- Favorite Conan Character - :: http://forums.dctp.ws/index.php?topic=30.0	conanquotes	f	1
1897	29	End of Detective Conan... :: http://forums.dctp.ws/index.php?topic=32.0	conanquotes	f	1
1898	29	Does DCTP Have all the RAW's of the Unsubbed Episodes? :: http://forums.dctp.ws/index.php?topic=33.0	conanquotes	f	1
1899	29	- Detective Conan Live Action Drama - :: http://forums.dctp.ws/index.php?topic=34.0	conanquotes	f	1
1896	29	Opinions on the OVAs? :: http://forums.dctp.ws/index.php?topic=31.0	conanquotes	f	2
1858	28	Chuck Norris found out about Conan O'Brien's lever that shows clips from "Walker: Texas Ranger" and is working on a way to make it show clips of Norris having sex with Conan's wife.	simcop2387	f	2
-20022	-20022	15,01[11OVA2215,01]	\N	f	0
1863	28	Chuck Norris invented his own type of karate. It's called Chuck-Will-Kill.	simcop2387	f	3
1867	28	Chuck Norris is the reason why Waldo is hiding.	simcop2387	f	2
1900	29	- Pokemon Diamond and Pearl - :: http://forums.dctp.ws/index.php?topic=35.0	conanquotes	f	1
1901	29	- Mobile Suit Gundam 00 - :: http://forums.dctp.ws/index.php?topic=36.0	conanquotes	f	1
1902	29	Clash of Red and Black Manga and Anime :: http://forums.dctp.ws/index.php?topic=37.0	conanquotes	f	1
1903	29	Poirot Reference (Manga Chapter 641) :: http://forums.dctp.ws/index.php?topic=38.0	conanquotes	f	1
1904	29	The Movie Actor Game :: http://forums.dctp.ws/index.php?topic=39.0	conanquotes	f	1
1905	29	Your favorite Mystery novels :: http://forums.dctp.ws/index.php?topic=41.0	conanquotes	f	1
1906	29	Direct Download :: http://forums.dctp.ws/index.php?topic=42.0	conanquotes	f	1
1907	29	who is you favorite DC girl :: http://forums.dctp.ws/index.php?topic=43.0	conanquotes	f	1
1908	29	Play guitar, anyone? :: http://forums.dctp.ws/index.php?topic=44.0	conanquotes	f	1
1909	29	Favorite Detective Conan Arc :: http://forums.dctp.ws/index.php?topic=45.0	conanquotes	f	1
1891	29	Seven Days Rhapsody?? :: http://forums.dctp.ws/index.php?topic=26.0	conanquotes	f	2
1910	29	- Bleach - :: http://forums.dctp.ws/index.php?topic=46.0	conanquotes	f	1
1911	29	- Favorite Kaitou Kid Episode/OVA/Movie - :: http://forums.dctp.ws/index.php?topic=47.0	conanquotes	f	1
1912	29	- That 70's Show - :: http://forums.dctp.ws/index.php?topic=48.0	conanquotes	f	1
1913	29	Ai + Conan :: http://forums.dctp.ws/index.php?topic=49.0	conanquotes	f	1
1914	29	ANIME BACKGROUND MUSIC :: http://forums.dctp.ws/index.php?topic=50.0	conanquotes	f	1
1915	29	Favorite Conan Theme (4-parter: Op,Ed,Movie,Insert/Soundtrack) :: http://forums.dctp.ws/index.php?topic=51.0	conanquotes	f	1
1916	29	What is an American Counterpart to Eel Rice? :: http://forums.dctp.ws/index.php?topic=52.0	conanquotes	f	1
1917	29	DETECTIVE CONAN'S STATUS IN JAPAN :: http://forums.dctp.ws/index.php?topic=53.0	conanquotes	f	1
1852	28	Chuck Norris got a perfect score on his SAT's, simply by writing Chuck Norris for every answer.	simcop2387	f	3
1918	29	Mop Girl :: http://forums.dctp.ws/index.php?topic=54.0	conanquotes	f	1
1919	29	Watch Out! Exploding Bleach Episodes Recalled! :: http://forums.dctp.ws/index.php?topic=56.0	conanquotes	f	1
1920	29	Professor Layton DS :: http://forums.dctp.ws/index.php?topic=57.0	conanquotes	f	1
1921	29	Kaitou Kid Anime Series or Not? :: http://forums.dctp.ws/index.php?topic=59.0	conanquotes	f	1
1922	29	Live Action #2 - Icky? :: http://forums.dctp.ws/index.php?topic=61.0	conanquotes	f	1
1923	29	- Worst 2 Hour Special - :: http://forums.dctp.ws/index.php?topic=62.0	conanquotes	f	1
1924	29	Anime Original :: http://forums.dctp.ws/index.php?topic=63.0	conanquotes	f	1
1925	29	Best Case Deduction? (Spoiler alert) :: http://forums.dctp.ws/index.php?topic=64.0	conanquotes	f	1
1926	29	AMVs :: http://forums.dctp.ws/index.php?topic=65.0	conanquotes	f	1
1927	29	Hey.. DCTP members.. how's it goin.. :: http://forums.dctp.ws/index.php?topic=66.0	conanquotes	f	1
1855	28	After much debate, President Truman decided to drop the atomic bomb on Hiroshima rather than the alternative of sending Chuck Norris. It was more "humane".	simcop2387	f	2
1864	28	Chuck Norris invented black. In fact, he invented the entire spectrum of visible light. Except pink. Tom Cruise invented pink.	simcop2387	f	2
1856	28	Chuck Norris doesn't see dead people. He makes people dead.	simcop2387	f	3
1866	28	The show Survivor had the original premise of putting people on an island with Chuck Norris. there were no survivors and the pilot episode tape has been burned.	simcop2387	f	2
1853	28	Aliens do exist. They're just waiting for Chuck Norris to die before they attack.	simcop2387	f	3
1861	28	There are no races, only countries of people Chuck Norris has beaten to different shades of black and blue.	simcop2387	f	3
1862	28	The grass is always greener on the other side, unless Chuck Norris has been there. In that case the grass is most likely soaked in blood and tears.	simcop2387	f	2
1854	28	Chuck Norris once ate three 72 oz. steaks in one hour. He spent the first 45 minutes having sex with his waitress.	simcop2387	f	3
1869	28	Along with his black belt, Chuck Norris often chooses to wear brown shoes. No one has DARED call him on it. Ever.	simcop2387	f	3
1928	29	The True ending to Detective Conan :: http://forums.dctp.ws/index.php?topic=67.0	conanquotes	f	1
1929	29	KAITOU KID AMV from me  :: http://forums.dctp.ws/index.php?topic=68.0	conanquotes	f	1
1930	29	Code Lyoko :: http://forums.dctp.ws/index.php?topic=69.0	conanquotes	f	1
1931	29	Heiji Episodes :: http://forums.dctp.ws/index.php?topic=70.0	conanquotes	f	1
1932	29	favorite music :: http://forums.dctp.ws/index.php?topic=71.0	conanquotes	f	1
1933	29	best dc 2 hour special/ best episode with 2 or more parts :: http://forums.dctp.ws/index.php?topic=72.0	conanquotes	f	1
1934	29	[DCTP] Detective Conan Episode 499 :: http://forums.dctp.ws/index.php?topic=73.0	conanquotes	f	1
1935	29	Who do you think is "THAT MAN" :: http://forums.dctp.ws/index.php?topic=74.0	conanquotes	f	1
1936	29	Want to guess what our next project is? :: http://forums.dctp.ws/index.php?topic=75.0	conanquotes	f	1
1937	29	- Favorite Japanese Band/Singer - :: http://forums.dctp.ws/index.php?topic=76.0	conanquotes	f	1
1938	29	My Artworks :: http://forums.dctp.ws/index.php?topic=77.0	conanquotes	f	1
1939	29	Conan Archive Bot On IRC :: http://forums.dctp.ws/index.php?topic=78.0	conanquotes	f	1
1940	29	- Conan Trivia Game - (Some Spoilers Maybe) :: http://forums.dctp.ws/index.php?topic=79.0	conanquotes	f	1
1941	29	Halo 3 :: http://forums.dctp.ws/index.php?topic=80.0	conanquotes	f	1
1942	29	favorite ''Lupin the third'' character :: http://forums.dctp.ws/index.php?topic=83.0	conanquotes	f	1
1943	29	Detective Conan 22th Opening+ new Movie theme by Zard on CD :: http://forums.dctp.ws/index.php?topic=84.0	conanquotes	f	1
1944	29	lupin the third live-action :: http://forums.dctp.ws/index.php?topic=85.0	conanquotes	f	1
1945	29	Please re-seed Character Development. :: http://forums.dctp.ws/index.php?topic=86.0	conanquotes	f	1
1946	29	about detective conan ending?? :: http://forums.dctp.ws/index.php?topic=87.0	conanquotes	f	1
1947	29	Interesting Miyano Akemi Query :: http://forums.dctp.ws/index.php?topic=88.0	conanquotes	f	1
1948	29	Banners... :: http://forums.dctp.ws/index.php?topic=89.0	conanquotes	f	1
1949	29	Lupin :: http://forums.dctp.ws/index.php?topic=90.0	conanquotes	f	1
1950	29	hey :: http://forums.dctp.ws/index.php?topic=91.0	conanquotes	f	1
1951	29	Rurouni Kenshin :: http://forums.dctp.ws/index.php?topic=92.0	conanquotes	f	1
1952	29	Anime That Brought Tears :: http://forums.dctp.ws/index.php?topic=93.0	conanquotes	f	1
1953	29	Gamer's Topic :: http://forums.dctp.ws/index.php?topic=94.0	conanquotes	f	1
1954	29	How Do You Rip a DVD + 1 Subtitle into Avi? :: http://forums.dctp.ws/index.php?topic=95.0	conanquotes	f	1
1955	29	Jd.. 500 :: http://forums.dctp.ws/index.php?topic=96.0	conanquotes	f	1
1956	29	[DCTP] Detective Conan Episode 500 :: http://forums.dctp.ws/index.php?topic=97.0	conanquotes	f	1
1957	29	Please Re-seed Episode 263 & 345 :: http://forums.dctp.ws/index.php?topic=99.0	conanquotes	f	1
1958	29	Side story for movie 12 :: http://forums.dctp.ws/index.php?topic=100.0	conanquotes	f	1
1959	29	What was the worst Dub Name Change in your opinion? :: http://forums.dctp.ws/index.php?topic=101.0	conanquotes	f	1
1960	29	My worker ^^ :: http://forums.dctp.ws/index.php?topic=102.0	conanquotes	f	1
1961	29	Crack The Code :: http://forums.dctp.ws/index.php?topic=103.0	conanquotes	f	1
1962	29	question regardind mIRC :: http://forums.dctp.ws/index.php?topic=105.0	conanquotes	f	1
1963	29	Detective Conan Movie 11: Jolly Roger in the Deep Azure :: http://forums.dctp.ws/index.php?topic=106.0	conanquotes	f	1
1964	29	Abt mIRC :: http://forums.dctp.ws/index.php?topic=107.0	conanquotes	f	1
1966	29	vol. 1 slip translation :: http://forums.dctp.ws/index.php?topic=109.0	conanquotes	f	1
1967	29	Best/Favorite Movie :: http://forums.dctp.ws/index.php?topic=110.0	conanquotes	f	1
1968	29	How many episodes of Conan  do you have (on all media) :: http://forums.dctp.ws/index.php?topic=111.0	conanquotes	f	1
1969	29	Nazonokami's Episode 418 :: http://forums.dctp.ws/index.php?topic=113.0	conanquotes	f	1
1970	29	Shinichi's Musical Ability? :: http://forums.dctp.ws/index.php?topic=114.0	conanquotes	f	1
1971	29	What brought you here.. :: http://forums.dctp.ws/index.php?topic=115.0	conanquotes	f	1
1972	29	new releases during break of broadcast conan in TV? :: http://forums.dctp.ws/index.php?topic=117.0	conanquotes	f	1
1973	29	[DCTP] Detective Conan Episodes 301-302 :: http://forums.dctp.ws/index.php?topic=118.0	conanquotes	f	1
1974	29	File 645 Spoiler :: http://forums.dctp.ws/index.php?topic=119.0	conanquotes	f	1
1975	29	Is conan even 8 now ? :: http://forums.dctp.ws/index.php?topic=121.0	conanquotes	f	1
1976	29	Officer Shiratori,Black Org. member/Boss? [Refer the 1 post here pls ] :: http://forums.dctp.ws/index.php?topic=122.0	conanquotes	f	1
1977	29	what if this hapen in the DC manga or  story line how will thing be :: http://forums.dctp.ws/index.php?topic=123.0	conanquotes	f	1
1978	29	All  the Wallpers you ever need  over 500. i have  so far here  for your u injoy :: http://forums.dctp.ws/index.php?topic=124.0	conanquotes	f	1
1979	29	The worst of the worst ending according to you? :: http://forums.dctp.ws/index.php?topic=125.0	conanquotes	f	1
1980	29	Who else knows about Conan=Shinchi? :: http://forums.dctp.ws/index.php?topic=126.0	conanquotes	f	1
1981	29	Videophiles  :: http://forums.dctp.ws/index.php?topic=128.0	conanquotes	f	1
1982	29	118 :: http://forums.dctp.ws/index.php?topic=129.0	conanquotes	f	1
1983	29	Let see who knows Detective Conan the Best[Pls post ur question after answering] :: http://forums.dctp.ws/index.php?topic=131.0	conanquotes	f	1
1984	29	Best DC Couple? :: http://forums.dctp.ws/index.php?topic=132.0	conanquotes	f	1
1985	29	Anokata?  :: http://forums.dctp.ws/index.php?topic=134.0	conanquotes	f	1
1986	29	Detective Boys? :: http://forums.dctp.ws/index.php?topic=136.0	conanquotes	f	1
1987	29	Dub?  :: http://forums.dctp.ws/index.php?topic=137.0	conanquotes	f	1
1988	29	Meaning of your name in your Language and in Japanese Language! :: http://forums.dctp.ws/index.php?topic=138.0	conanquotes	f	1
1989	29	RSS feed For Notifying New Episodes Release! :: http://forums.dctp.ws/index.php?topic=139.0	conanquotes	f	1
1990	29	Which DC character Describes you the best-honestly and why? :: http://forums.dctp.ws/index.php?topic=140.0	conanquotes	f	1
1991	29	Most responsible parent? :: http://forums.dctp.ws/index.php?topic=141.0	conanquotes	f	1
1992	29	Where is everyone from? :: http://forums.dctp.ws/index.php?topic=142.0	conanquotes	f	1
1993	29	Hello there! :: http://forums.dctp.ws/index.php?topic=143.0	conanquotes	f	1
1994	29	Guess the next project, again :: http://forums.dctp.ws/index.php?topic=144.0	conanquotes	f	1
1995	29	DCTP Direct Download Online :: http://forums.dctp.ws/index.php?topic=145.0	conanquotes	f	1
1997	29	Vodka.. (***SPOILERS***) :: http://forums.dctp.ws/index.php?topic=147.0	conanquotes	f	1
1998	29	Bugging me for some time now.... :: http://forums.dctp.ws/index.php?topic=148.0	conanquotes	f	1
1999	29	v10-1 translation question :: http://forums.dctp.ws/index.php?topic=149.0	conanquotes	f	1
2000	29	A link back to the homepage :: http://forums.dctp.ws/index.php?topic=150.0	conanquotes	f	1
2001	29	A Chat box Please!!! :: http://forums.dctp.ws/index.php?topic=151.0	conanquotes	f	1
2002	29	if kogorou was in the BO... :: http://forums.dctp.ws/index.php?topic=152.0	conanquotes	f	1
2003	29	Wich Detective conan   character  look like you and     Describes   it  :: http://forums.dctp.ws/index.php?topic=153.0	conanquotes	f	1
2004	29	[DCTP] Detective Conan Episodes 203-204 :: http://forums.dctp.ws/index.php?topic=154.0	conanquotes	f	1
2005	29	DC Kids :: http://forums.dctp.ws/index.php?topic=155.0	conanquotes	f	1
2006	29	4869 :: http://forums.dctp.ws/index.php?topic=156.0	conanquotes	f	1
2007	29	About the subbed episodes.... :: http://forums.dctp.ws/index.php?topic=158.0	conanquotes	f	1
2008	29	Akai (SPOILERS) :: http://forums.dctp.ws/index.php?topic=161.0	conanquotes	f	1
2009	29	[DCTP] Detective Conan Episode 478 :: http://forums.dctp.ws/index.php?topic=162.0	conanquotes	f	1
2010	29	[DCTP] Detective Conan Episode 418 :: http://forums.dctp.ws/index.php?topic=163.0	conanquotes	f	1
2011	29	Pointless Fangirling. Shoot me I know.  :: http://forums.dctp.ws/index.php?topic=164.0	conanquotes	f	1
2012	29	Not sure how to request subbed episodes? :: http://forums.dctp.ws/index.php?topic=165.0	conanquotes	f	1
2013	29	When did the OAVs come out? :: http://forums.dctp.ws/index.php?topic=166.0	conanquotes	f	1
2014	29	MapleStory :: http://forums.dctp.ws/index.php?topic=167.0	conanquotes	f	1
2015	29	weird covers :: http://forums.dctp.ws/index.php?topic=168.0	conanquotes	f	1
2016	29	Trailer.... :: http://forums.dctp.ws/index.php?topic=170.0	conanquotes	f	1
2017	29	Jodie's name :: http://forums.dctp.ws/index.php?topic=172.0	conanquotes	f	1
2018	29	[DCTP]'s Next Release is... :: http://forums.dctp.ws/index.php?topic=173.0	conanquotes	f	1
2019	29	Seven Baby Crows song :: http://forums.dctp.ws/index.php?topic=174.0	conanquotes	f	1
2020	29	Favorite OP/ED :: http://forums.dctp.ws/index.php?topic=175.0	conanquotes	f	1
2021	29	M.U.G.E.N. anyone :: http://forums.dctp.ws/index.php?topic=176.0	conanquotes	f	1
2022	29	[DCTP] Detective Conan Episodes 390-391 :: http://forums.dctp.ws/index.php?topic=177.0	conanquotes	f	1
2023	29	Vote on our our next case! :: http://forums.dctp.ws/index.php?topic=179.0	conanquotes	f	1
2024	29	File 646 Manga English (Shinichi Is Back!!!) :: http://forums.dctp.ws/index.php?topic=180.0	conanquotes	f	1
2025	29	Beck : MCS ending :: http://forums.dctp.ws/index.php?topic=181.0	conanquotes	f	1
2026	29	Bot not working?? :: http://forums.dctp.ws/index.php?topic=182.0	conanquotes	f	1
2027	29	FuN ArT - Detective Conan Movies DVD covers by Psykotaku78 :: http://forums.dctp.ws/index.php?topic=183.0	conanquotes	f	1
2028	29	How many Cases have you solved? :: http://forums.dctp.ws/index.php?topic=184.0	conanquotes	f	1
2029	29	Haibara Ai episodes :: http://forums.dctp.ws/index.php?topic=185.0	conanquotes	f	1
2030	29	?I ASK YOU WHO IS THIS ?  :: http://forums.dctp.ws/index.php?topic=186.0	conanquotes	f	1
2031	29	?CAn any 1 gess were this cene is from   ? comment pleace :: http://forums.dctp.ws/index.php?topic=187.0	conanquotes	f	1
2032	29	File 646: Haibara is Evil !! :: http://forums.dctp.ws/index.php?topic=188.0	conanquotes	f	1
2033	29	Updated Direct Download System :: http://forums.dctp.ws/index.php?topic=189.0	conanquotes	f	1
2034	29	Question about episodes 13 and 128 (129 too technically) :: http://forums.dctp.ws/index.php?topic=190.0	conanquotes	f	1
2035	29	What other mystery shows have you watched/recommend? :: http://forums.dctp.ws/index.php?topic=191.0	conanquotes	f	1
2036	29	Conan's cellphone :: http://forums.dctp.ws/index.php?topic=192.0	conanquotes	f	1
2037	29	Meitantei Mouri Kogoro :: http://forums.dctp.ws/index.php?topic=193.0	conanquotes	f	1
2038	29	GinRei :: http://forums.dctp.ws/index.php?topic=194.0	conanquotes	f	1
2039	29	The Conan Wiki Project: Call for logo and Names :: http://forums.dctp.ws/index.php?topic=195.0	conanquotes	f	1
2040	29	What are the Takagi x Sato development episodes? :: http://forums.dctp.ws/index.php?topic=196.0	conanquotes	f	1
2041	29	shinichi sketch :: http://forums.dctp.ws/index.php?topic=197.0	conanquotes	f	1
2042	29	How to Be a Teenage Detective :: http://forums.dctp.ws/index.php?topic=198.0	conanquotes	f	1
2043	29	Conan-Haibara dialogues :: http://forums.dctp.ws/index.php?topic=199.0	conanquotes	f	1
2044	29	Code Breaking!!  :: http://forums.dctp.ws/index.php?topic=200.0	conanquotes	f	1
2045	29	ANFS? :: http://forums.dctp.ws/index.php?topic=201.0	conanquotes	f	1
2046	29	Minute Mysteries :: http://forums.dctp.ws/index.php?topic=202.0	conanquotes	f	1
2047	29	Dragon GT  ending theme   weth my voice  :: http://forums.dctp.ws/index.php?topic=203.0	conanquotes	f	1
2049	29	anime to manga crossover :: http://forums.dctp.ws/index.php?topic=205.0	conanquotes	f	1
2050	29	FILE.647 English Manga (Amnesia, Real Or Fake?) :: http://forums.dctp.ws/index.php?topic=206.0	conanquotes	f	1
2051	29	224 :: http://forums.dctp.ws/index.php?topic=207.0	conanquotes	f	1
2052	29	XP_On's Art collections :: http://forums.dctp.ws/index.php?topic=208.0	conanquotes	f	1
2053	29	New B'z Single: BURN -Fumetsu no Face- :: http://forums.dctp.ws/index.php?topic=209.0	conanquotes	f	1
2054	29	Does Conan ever make you laugh? :: http://forums.dctp.ws/index.php?topic=210.0	conanquotes	f	1
2055	29	#1412 :: http://forums.dctp.ws/index.php?topic=211.0	conanquotes	f	1
2056	29	murderer motive :: http://forums.dctp.ws/index.php?topic=212.0	conanquotes	f	1
2057	29	Detective Kogorou :: http://forums.dctp.ws/index.php?topic=213.0	conanquotes	f	1
2058	29	Correct Order :: http://forums.dctp.ws/index.php?topic=214.0	conanquotes	f	1
2059	29	Will DCTP finally get 100%? :: http://forums.dctp.ws/index.php?topic=215.0	conanquotes	f	1
2060	29	Other mystery anime/manga? :: http://forums.dctp.ws/index.php?topic=216.0	conanquotes	f	1
2061	29	Movie 11 :: http://forums.dctp.ws/index.php?topic=218.0	conanquotes	f	1
2062	29	[DCTP] Detective Conan Episode 501 :: http://forums.dctp.ws/index.php?topic=219.0	conanquotes	f	1
2063	29	Ran + Kazuha??? :: http://forums.dctp.ws/index.php?topic=220.0	conanquotes	f	1
2064	29	File 648 English Manga (Shinichi The Suspect) :: http://forums.dctp.ws/index.php?topic=221.0	conanquotes	f	1
2065	29	Movie 13 confirmed as usual... :: http://forums.dctp.ws/index.php?topic=222.0	conanquotes	f	1
2066	29	the NEW Movie  wallapers  what you thinks like them or what? sum  uthere one 2   :: http://forums.dctp.ws/index.php?topic=223.0	conanquotes	f	1
2067	29	Detective conan and  kaito kid and ran Winamp skins :: http://forums.dctp.ws/index.php?topic=224.0	conanquotes	f	1
2068	29	who is this supposed to be? :: http://forums.dctp.ws/index.php?topic=226.0	conanquotes	f	1
2069	29	THANKS FOR EVERYTHING :: http://forums.dctp.ws/index.php?topic=229.0	conanquotes	f	1
2070	29	Your Favorite OST? :: http://forums.dctp.ws/index.php?topic=230.0	conanquotes	f	1
2071	29	Truth Serum??? (ep500) :: http://forums.dctp.ws/index.php?topic=231.0	conanquotes	f	1
2072	29	What does the BO really want????? :: http://forums.dctp.ws/index.php?topic=232.0	conanquotes	f	1
2073	29	Toichi Kuroba    y his wus killd and  y his son want 2 get back at theblack org  :: http://forums.dctp.ws/index.php?topic=235.0	conanquotes	f	1
2074	29	V24-11 -- the pisco case :: http://forums.dctp.ws/index.php?topic=236.0	conanquotes	f	1
2075	29	Akai  Discussion   :: http://forums.dctp.ws/index.php?topic=239.0	conanquotes	f	1
2076	29	Matsuda jinpei  the short lived Detective  Discussion :: http://forums.dctp.ws/index.php?topic=240.0	conanquotes	f	1
2077	29	Gin intell and more  Discussion Spoilers :: http://forums.dctp.ws/index.php?topic=241.0	conanquotes	f	1
2078	29	Files 646+ Who is The Forest Guy? :: http://forums.dctp.ws/index.php?topic=242.0	conanquotes	f	1
2079	29	Funny/Interesting Detective Conan Sceen Caps/Manga Pages :: http://forums.dctp.ws/index.php?topic=243.0	conanquotes	f	1
2080	29	Name and Logo for the DC Wiki, Part 2 :: http://forums.dctp.ws/index.php?topic=244.0	conanquotes	f	1
2081	29	do you always wondered what it would be like if those two exchanged places  :: http://forums.dctp.ws/index.php?topic=245.0	conanquotes	f	1
2082	29	NFL draft :: http://forums.dctp.ws/index.php?topic=246.0	conanquotes	f	1
2083	29	Who is the real one? The Amnesiac Shinichi or the Forrest Guy? :: http://forums.dctp.ws/index.php?topic=249.0	conanquotes	f	1
2084	29	I have a problem with episode 96 :: http://forums.dctp.ws/index.php?topic=250.0	conanquotes	f	1
2085	29	One for the boys.. :: http://forums.dctp.ws/index.php?topic=253.0	conanquotes	f	1
2086	29	[DCTP] Detective Conan Episodes 431-432 :: http://forums.dctp.ws/index.php?topic=254.0	conanquotes	f	1
2087	29	File 649 English Manga (Ran's In Trouble) :: http://forums.dctp.ws/index.php?topic=256.0	conanquotes	f	1
2088	29	Kindaichi special :: http://forums.dctp.ws/index.php?topic=257.0	conanquotes	f	1
2089	29	Hello and a Question :: http://forums.dctp.ws/index.php?topic=258.0	conanquotes	f	1
2090	29	IRC Help... :: http://forums.dctp.ws/index.php?topic=259.0	conanquotes	f	1
2091	29	Glasses... :: http://forums.dctp.ws/index.php?topic=260.0	conanquotes	f	1
2092	29	Some quick questions about your future releases :: http://forums.dctp.ws/index.php?topic=261.0	conanquotes	f	1
2093	29	Your Choice for Worst Spoken English Scene :: http://forums.dctp.ws/index.php?topic=262.0	conanquotes	f	1
2094	29	Request for subtitles files :: http://forums.dctp.ws/index.php?topic=263.0	conanquotes	f	1
2095	29	"Almost, but Not Quite" Moments :: http://forums.dctp.ws/index.php?topic=264.0	conanquotes	f	1
2096	29	Fillers :: http://forums.dctp.ws/index.php?topic=265.0	conanquotes	f	1
2097	29	what do you think heiji will do ? :: http://forums.dctp.ws/index.php?topic=266.0	conanquotes	f	1
2098	29	How will it work? :: http://forums.dctp.ws/index.php?topic=267.0	conanquotes	f	1
2099	29	DC Gameboy games :: http://forums.dctp.ws/index.php?topic=269.0	conanquotes	f	1
2100	29	----Sherlock Holmes---- :: http://forums.dctp.ws/index.php?topic=270.0	conanquotes	f	1
2101	29	Gin and Vodka on the Jetcoaster? :: http://forums.dctp.ws/index.php?topic=271.0	conanquotes	f	1
2102	29	Colored Manga :: http://forums.dctp.ws/index.php?topic=272.0	conanquotes	f	1
2103	29	1 for the girls  this how in shape i am comment pls :: http://forums.dctp.ws/index.php?topic=274.0	conanquotes	f	1
2104	29	Website down? :: http://forums.dctp.ws/index.php?topic=275.0	conanquotes	f	1
2105	29	Does Conan Fit inside anybody's Outfits?? :: http://forums.dctp.ws/index.php?topic=276.0	conanquotes	f	1
2106	29	[DCTP] Detective Conan Episode 502 :: http://forums.dctp.ws/index.php?topic=277.0	conanquotes	f	1
2107	29	File 646+: Who is the culprit? :: http://forums.dctp.ws/index.php?topic=279.0	conanquotes	f	1
2108	29	FILE 650 Spoilers (Shinichi The Crybaby) :: http://forums.dctp.ws/index.php?topic=280.0	conanquotes	f	1
2109	29	'Introduce yourself' forum :: http://forums.dctp.ws/index.php?topic=281.0	conanquotes	f	1
2110	29	'' yes yes... Guess who's on third..... Lupe steal like 'Lupin the III'.." :: http://forums.dctp.ws/index.php?topic=282.0	conanquotes	f	1
2112	29	Would you take APTX4869? :: http://forums.dctp.ws/index.php?topic=284.0	conanquotes	f	1
2113	29	Conan Doyle... Everybody's Fool! :: http://forums.dctp.ws/index.php?topic=285.0	conanquotes	f	1
2115	29	Site's Return & Other News :: http://forums.dctp.ws/index.php?topic=287.0	conanquotes	f	1
2116	29	Gosho Aoyama is Coward :: http://forums.dctp.ws/index.php?topic=288.0	conanquotes	f	1
2117	29	Where to get these? And missing CRC? :: http://forums.dctp.ws/index.php?topic=289.0	conanquotes	f	1
2118	29	Wallpapers anyone? :: http://forums.dctp.ws/index.php?topic=290.0	conanquotes	f	1
2119	29	Kaito Kid FanFic: Crisis :: http://forums.dctp.ws/index.php?topic=292.0	conanquotes	f	1
2120	29	Huge Spoiler; A look into the future!? :: http://forums.dctp.ws/index.php?topic=293.0	conanquotes	f	1
2121	29	Most unexpected culprit ? :: http://forums.dctp.ws/index.php?topic=294.0	conanquotes	f	1
2122	29	Playback problemss :: http://forums.dctp.ws/index.php?topic=295.0	conanquotes	f	1
2123	29	Kudo Shinichi Fanfic: In The End :: http://forums.dctp.ws/index.php?topic=296.0	conanquotes	f	1
2124	29	5 Centimeters Per Second (Byousoku 5cm) :: http://forums.dctp.ws/index.php?topic=297.0	conanquotes	f	1
2125	29	Detective Conan: Special Volume 26? :: http://forums.dctp.ws/index.php?topic=298.0	conanquotes	f	1
2126	29	movie 12 reviews??? :: http://forums.dctp.ws/index.php?topic=300.0	conanquotes	f	1
2127	29	File 651 Spoilers (True Identify???) :: http://forums.dctp.ws/index.php?topic=303.0	conanquotes	f	1
2128	29	[DCTP] Detective Conan Episode 503 :: http://forums.dctp.ws/index.php?topic=304.0	conanquotes	f	1
2129	29	Got a question? :: http://forums.dctp.ws/index.php?topic=305.0	conanquotes	f	1
2130	29	Shiranpuri??? :: http://forums.dctp.ws/index.php?topic=306.0	conanquotes	f	1
2131	29	"OUR" First AMV...... :: http://forums.dctp.ws/index.php?topic=307.0	conanquotes	f	1
2132	29	Stopping My FanFic :: http://forums.dctp.ws/index.php?topic=309.0	conanquotes	f	1
2133	29	What do you think about this kind of stuff? :: http://forums.dctp.ws/index.php?topic=310.0	conanquotes	f	1
2134	29	Fanfic: Project Fenris :: http://forums.dctp.ws/index.php?topic=312.0	conanquotes	f	1
2135	29	What does Ran want from Shinichi? Suggestions about file 652~!! :: http://forums.dctp.ws/index.php?topic=313.0	conanquotes	f	1
2136	29	Background music, Conan? :: http://forums.dctp.ws/index.php?topic=314.0	conanquotes	f	1
2137	29	Reseed 487? :: http://forums.dctp.ws/index.php?topic=315.0	conanquotes	f	1
2138	29	[DCTP] Detective Conan Episode 504 :: http://forums.dctp.ws/index.php?topic=316.0	conanquotes	f	1
2139	29	Colouring :: http://forums.dctp.ws/index.php?topic=317.0	conanquotes	f	1
2140	29	People who aren't american (and your thoughts about their countrys) :: http://forums.dctp.ws/index.php?topic=318.0	conanquotes	f	1
2141	29	Help- how to have an avatar!? :: http://forums.dctp.ws/index.php?topic=320.0	conanquotes	f	1
2142	29	My first AMV  :: http://forums.dctp.ws/index.php?topic=322.0	conanquotes	f	1
2143	29	Will there be a third Live Action Drama?? :: http://forums.dctp.ws/index.php?topic=323.0	conanquotes	f	1
2144	29	Ai's guinea pig - APTX's influence on Shinichi/Conan's health  :: http://forums.dctp.ws/index.php?topic=325.0	conanquotes	f	1
2145	29	Thankful ! :: http://forums.dctp.ws/index.php?topic=326.0	conanquotes	f	1
2146	29	DetectiveShinichi.com :: http://forums.dctp.ws/index.php?topic=327.0	conanquotes	f	1
2147	29	My channel/My AMVs :: http://forums.dctp.ws/index.php?topic=328.0	conanquotes	f	1
2148	29	Videogames?? :: http://forums.dctp.ws/index.php?topic=329.0	conanquotes	f	1
2149	29	Re-Seed 2hour Special 383 :: http://forums.dctp.ws/index.php?topic=330.0	conanquotes	f	1
2150	29	File 652 Spoiler (Close Call) :: http://forums.dctp.ws/index.php?topic=332.0	conanquotes	f	1
2151	29	Additional seeds for 492,493 please :: http://forums.dctp.ws/index.php?topic=333.0	conanquotes	f	1
2152	29	[DCTP] Detective Conan Episodes 220-221 & 222-224 :: http://forums.dctp.ws/index.php?topic=334.0	conanquotes	f	1
2153	29	fanfic - Trouble? :: http://forums.dctp.ws/index.php?topic=335.0	conanquotes	f	1
2154	29	Movie 12 :: http://forums.dctp.ws/index.php?topic=337.0	conanquotes	f	1
2155	29	Banned IP :: http://forums.dctp.ws/index.php?topic=338.0	conanquotes	f	1
2156	29	Yusaku Kudo Is The Head Of Black Organization (I guess) :: http://forums.dctp.ws/index.php?topic=341.0	conanquotes	f	1
2157	29	File 653 Spoiler :: http://forums.dctp.ws/index.php?topic=342.0	conanquotes	f	1
2158	29	Question about movies & OVAs :: http://forums.dctp.ws/index.php?topic=343.0	conanquotes	f	1
2159	29	Do the FBI know about Conan's true identity? :: http://forums.dctp.ws/index.php?topic=344.0	conanquotes	f	1
2160	29	Past KnKF Episodes? :: http://forums.dctp.ws/index.php?topic=345.0	conanquotes	f	1
2161	29	Question about raw DC manga? :: http://forums.dctp.ws/index.php?topic=346.0	conanquotes	f	1
2162	29	File 653 out! :: http://forums.dctp.ws/index.php?topic=348.0	conanquotes	f	1
2163	29	Playback problem :: http://forums.dctp.ws/index.php?topic=349.0	conanquotes	f	1
2164	29	Please kindly sub these episodes :: http://forums.dctp.ws/index.php?topic=351.0	conanquotes	f	1
2165	29	Change of anime staff :: http://forums.dctp.ws/index.php?topic=353.0	conanquotes	f	1
2166	29	File 654 Spoiler (Honey, I Shrunk The Detective) :: http://forums.dctp.ws/index.php?topic=354.0	conanquotes	f	1
2167	29	Case Closed Opening English Slide Show :: http://forums.dctp.ws/index.php?topic=356.0	conanquotes	f	1
2168	29	Emoticons :: http://forums.dctp.ws/index.php?topic=357.0	conanquotes	f	1
2169	29	Preview New Opening Theme Song #23 ''Ichibyou goto ni Love for you'' -Kuraki Mai :: http://forums.dctp.ws/index.php?topic=358.0	conanquotes	f	1
2170	29	Metal Gear Solid :: http://forums.dctp.ws/index.php?topic=359.0	conanquotes	f	1
2171	29	Movie 10 :: http://forums.dctp.ws/index.php?topic=360.0	conanquotes	f	1
2172	29	NEW AMV: Akai Shuichi and Akemi Miyano :: http://forums.dctp.ws/index.php?topic=361.0	conanquotes	f	1
2173	29	Summer Memories -ED :: http://forums.dctp.ws/index.php?topic=362.0	conanquotes	f	1
2174	29	Conan sender is translated anglice :: http://forums.dctp.ws/index.php?topic=363.0	conanquotes	f	1
2175	29	Anokata :: http://forums.dctp.ws/index.php?topic=364.0	conanquotes	f	1
2176	29	[DCTP] Detective Conan Episodes 354-355 :: http://forums.dctp.ws/index.php?topic=365.0	conanquotes	f	1
2177	29	[DCTP] Detective Conan File 653 :: http://forums.dctp.ws/index.php?topic=366.0	conanquotes	f	1
2178	29	Do u think Shinichi will tell Ran the truth of Conan? :: http://forums.dctp.ws/index.php?topic=367.0	conanquotes	f	1
2179	29	which charcther is you're favorite and why?? :: http://forums.dctp.ws/index.php?topic=368.0	conanquotes	f	1
2180	29	In your opinion, which is the most cruel way a victim has been killed ? :: http://forums.dctp.ws/index.php?topic=369.0	conanquotes	f	1
2181	29	Floating Heads :: http://forums.dctp.ws/index.php?topic=371.0	conanquotes	f	1
2182	29	The Melancholy of Haruhi Suzumiya :: http://forums.dctp.ws/index.php?topic=372.0	conanquotes	f	1
2183	29	File 654 out :: http://forums.dctp.ws/index.php?topic=374.0	conanquotes	f	1
2184	29	Random: Languages? :: http://forums.dctp.ws/index.php?topic=375.0	conanquotes	f	1
2185	29	Which movie of Conan is best?? :: http://forums.dctp.ws/index.php?topic=376.0	conanquotes	f	1
2186	29	Help Needed for Kudo's Bookshelf :: http://forums.dctp.ws/index.php?topic=377.0	conanquotes	f	1
2187	29	FBI Or CIA   what will you join if you got a invite into one of this 2? :: http://forums.dctp.ws/index.php?topic=378.0	conanquotes	f	1
2188	29	The Famous Stars of Conan :: http://forums.dctp.ws/index.php?topic=379.0	conanquotes	f	1
2189	29	What made you start watching DC? :: http://forums.dctp.ws/index.php?topic=380.0	conanquotes	f	1
2190	29	FILE 655 Spoilers (Sushi Eating Time) :: http://forums.dctp.ws/index.php?topic=381.0	conanquotes	f	1
2191	29	The "Introduce Yourself" Topic :: http://forums.dctp.ws/index.php?topic=383.0	conanquotes	f	1
2192	29	DC Licensing? :: http://forums.dctp.ws/index.php?topic=384.0	conanquotes	f	1
2193	29	Fanart: Embarrased Kaito KID :: http://forums.dctp.ws/index.php?topic=385.0	conanquotes	f	1
2194	29	Your favorite television series :: http://forums.dctp.ws/index.php?topic=386.0	conanquotes	f	1
2195	29	Convince VIZ to stop changing the characters' names! :: http://forums.dctp.ws/index.php?topic=387.0	conanquotes	f	1
2196	29	Fanart: A Shocked Conan! :: http://forums.dctp.ws/index.php?topic=388.0	conanquotes	f	1
2197	29	Question About Recently Discussed Theory (Spoiler Here) :: http://forums.dctp.ws/index.php?topic=389.0	conanquotes	f	1
2198	29	Detective Conan - Novel Edition :: http://forums.dctp.ws/index.php?topic=390.0	conanquotes	f	1
2199	29	I need help ^^"" :: http://forums.dctp.ws/index.php?topic=392.0	conanquotes	f	1
2200	29	[DCTP] Detective Conan File 654 :: http://forums.dctp.ws/index.php?topic=393.0	conanquotes	f	1
2201	29	please re-seed 503 HD :: http://forums.dctp.ws/index.php?topic=394.0	conanquotes	f	1
2202	29	[DCTP] Detective Conan Episode 505 :: http://forums.dctp.ws/index.php?topic=395.0	conanquotes	f	1
2203	29	need help playing torrents :: http://forums.dctp.ws/index.php?topic=396.0	conanquotes	f	1
2204	29	Opening ports for the bittorrent download.  :: http://forums.dctp.ws/index.php?topic=397.0	conanquotes	f	1
2205	29	dont you wunder how meny Dead so far?   :: http://forums.dctp.ws/index.php?topic=398.0	conanquotes	f	1
2206	29	sum   the best eletrick guitar music and morre  :: http://forums.dctp.ws/index.php?topic=399.0	conanquotes	f	1
2207	29	FILE 656 Spoilers :: http://forums.dctp.ws/index.php?topic=401.0	conanquotes	f	1
2208	29	Okiya Subaru's True Identify :: http://forums.dctp.ws/index.php?topic=402.0	conanquotes	f	1
2209	29	Who has been to Japan? :: http://forums.dctp.ws/index.php?topic=403.0	conanquotes	f	1
2210	29	Is Ai love conan :: http://forums.dctp.ws/index.php?topic=404.0	conanquotes	f	1
2211	29	yo   guys  a friend of mine  open his  Detective conan  Merchandise store    :: http://forums.dctp.ws/index.php?topic=405.0	conanquotes	f	1
2212	29	What is/are your favorite volume(s) of the DC manga? :: http://forums.dctp.ws/index.php?topic=406.0	conanquotes	f	1
2213	29	[DCTP] Detective Conan File 655 :: http://forums.dctp.ws/index.php?topic=408.0	conanquotes	f	1
2214	29	Need help on IRC :: http://forums.dctp.ws/index.php?topic=409.0	conanquotes	f	1
2215	29	Gin car number ??? :: http://forums.dctp.ws/index.php?topic=410.0	conanquotes	f	1
2217	29	Kid Versus Ran :: http://forums.dctp.ws/index.php?topic=412.0	conanquotes	f	1
2218	29	Fantasy Dub Voice Casting :: http://forums.dctp.ws/index.php?topic=413.0	conanquotes	f	1
2219	29	Are you ready for Conan to end? :: http://forums.dctp.ws/index.php?topic=414.0	conanquotes	f	1
2220	29	DCTP Domain is active (www.dctp.ws) :: http://forums.dctp.ws/index.php?topic=416.0	conanquotes	f	1
2221	29	File 656 - Quick translation :: http://forums.dctp.ws/index.php?topic=417.0	conanquotes	f	1
2222	29	who do you think  i look like =?= :: http://forums.dctp.ws/index.php?topic=419.0	conanquotes	f	1
2223	29	Haibara and Conan ending up together?.... maybe :: http://forums.dctp.ws/index.php?topic=420.0	conanquotes	f	1
2224	29	When did conan knew about the black organization and their code name e.g. Gin :: http://forums.dctp.ws/index.php?topic=423.0	conanquotes	f	1
2226	29	Does Conan have a photographic memory? :: http://forums.dctp.ws/index.php?topic=425.0	conanquotes	f	1
2227	29	HD versions :: http://forums.dctp.ws/index.php?topic=426.0	conanquotes	f	1
2228	29	Any DJ? :: http://forums.dctp.ws/index.php?topic=427.0	conanquotes	f	1
2229	29	700 Detective conan wallapersgot more just post them agin cus i forgot sum :: http://forums.dctp.ws/index.php?topic=428.0	conanquotes	f	1
2230	29	[DCTP] Detective Conan File 656 :: http://forums.dctp.ws/index.php?topic=429.0	conanquotes	f	1
2231	29	FILE 657 (Sushi Junkie) :: http://forums.dctp.ws/index.php?topic=430.0	conanquotes	f	1
2232	29	Old Conan vs. New Conan :: http://forums.dctp.ws/index.php?topic=433.0	conanquotes	f	1
2233	29	487 and 499 h264 needed :: http://forums.dctp.ws/index.php?topic=434.0	conanquotes	f	1
2234	29	[DCTP] Detective Conan Episode 506 :: http://forums.dctp.ws/index.php?topic=435.0	conanquotes	f	1
2235	29	[DCTP] Detective Conan File 657 :: http://forums.dctp.ws/index.php?topic=436.0	conanquotes	f	1
2236	29	Hiroki Sawada pictures :: http://forums.dctp.ws/index.php?topic=437.0	conanquotes	f	1
2237	29	Conan and the Detective Boys :: http://forums.dctp.ws/index.php?topic=438.0	conanquotes	f	1
2238	29	"English" Parts in the Dub :: http://forums.dctp.ws/index.php?topic=439.0	conanquotes	f	1
2239	29	Song Name Help? :: http://forums.dctp.ws/index.php?topic=440.0	conanquotes	f	1
2240	29	FILE 658's spiolers(Genta's father) :: http://forums.dctp.ws/index.php?topic=442.0	conanquotes	f	1
2241	29	Takagi's Look :: http://forums.dctp.ws/index.php?topic=443.0	conanquotes	f	1
2242	29	Fanfic character :: http://forums.dctp.ws/index.php?topic=444.0	conanquotes	f	1
2243	29	Vermouth?... :: http://forums.dctp.ws/index.php?topic=445.0	conanquotes	f	1
2244	29	Okiya's seiyuu :: http://forums.dctp.ws/index.php?topic=446.0	conanquotes	f	1
2245	29	How long would DC last? :: http://forums.dctp.ws/index.php?topic=447.0	conanquotes	f	1
2246	29	Wow! "Deduction Showdown! Shinichi vs. Subaru Okiya" = Episode 511?  :: http://forums.dctp.ws/index.php?topic=448.0	conanquotes	f	1
2247	29	Post images :: http://forums.dctp.ws/index.php?topic=449.0	conanquotes	f	1
2248	29	New cases,new places,new characters-what Gosho might think out?? Let's guess :-) :: http://forums.dctp.ws/index.php?topic=450.0	conanquotes	f	1
2249	29	Shouldn't Kogoro have figured out Conan's identity by now? :: http://forums.dctp.ws/index.php?topic=451.0	conanquotes	f	1
2250	29	BO :: http://forums.dctp.ws/index.php?topic=452.0	conanquotes	f	1
2251	29	Final Fantasy VII :: http://forums.dctp.ws/index.php?topic=454.0	conanquotes	f	1
2252	29	Who is the Spy on the BO...? :: http://forums.dctp.ws/index.php?topic=456.0	conanquotes	f	1
2253	29	Conan 1080p Videos :: http://forums.dctp.ws/index.php?topic=457.0	conanquotes	f	1
2254	29	There's something about....Satou?? :: http://forums.dctp.ws/index.php?topic=459.0	conanquotes	f	1
2255	29	My fan arts :: http://forums.dctp.ws/index.php?topic=461.0	conanquotes	f	1
2256	29	when will be the movie 12 be available? :: http://forums.dctp.ws/index.php?topic=462.0	conanquotes	f	1
2257	29	Where did BO head guessing come from? :: http://forums.dctp.ws/index.php?topic=463.0	conanquotes	f	1
2258	29	[req] detective conan episode 345 :: http://forums.dctp.ws/index.php?topic=464.0	conanquotes	f	1
2259	29	Avatar: The Last Airbender :: http://forums.dctp.ws/index.php?topic=465.0	conanquotes	f	1
2260	29	Yes, we're still workin' :: http://forums.dctp.ws/index.php?topic=467.0	conanquotes	f	1
2261	29	[Req] Episodes :: http://forums.dctp.ws/index.php?topic=468.0	conanquotes	f	1
2262	29	Kogoro would went psycho / mad afermath :: http://forums.dctp.ws/index.php?topic=469.0	conanquotes	f	1
2263	29	VIZ Launches Hollywood Film Company :: http://forums.dctp.ws/index.php?topic=470.0	conanquotes	f	1
2264	29	Conan 1080p raw :: http://forums.dctp.ws/index.php?topic=472.0	conanquotes	f	1
2265	29	DC Fans :: http://forums.dctp.ws/index.php?topic=473.0	conanquotes	f	1
2266	29	DC Places :: http://forums.dctp.ws/index.php?topic=474.0	conanquotes	f	1
2267	29	Kaitou-Shinichi long lost twins??? :: http://forums.dctp.ws/index.php?topic=475.0	conanquotes	f	1
2268	29	File.660 spoilers ("108") :: http://forums.dctp.ws/index.php?topic=476.0	conanquotes	f	1
2269	29	Conan in 1080P :: http://forums.dctp.ws/index.php?topic=477.0	conanquotes	f	1
2270	29	Status Update: Future Releases :: http://forums.dctp.ws/index.php?topic=478.0	conanquotes	f	1
2271	29	Does Jodie-Sensei Knows Conan's true identity? :: http://forums.dctp.ws/index.php?topic=479.0	conanquotes	f	1
2273	29	conan freak out  :: http://forums.dctp.ws/index.php?topic=491.0	conanquotes	f	1
2274	29	[DCTP] Detective Conan Episode 507 :: http://forums.dctp.ws/index.php?topic=492.0	conanquotes	f	1
2275	29	Some off topic pics drawn by me 1 :: http://forums.dctp.ws/index.php?topic=493.0	conanquotes	f	1
2276	29	Bot? :: http://forums.dctp.ws/index.php?topic=494.0	conanquotes	f	1
2277	29	EATH QAKE 5.8-chino hills 8.9 :: http://forums.dctp.ws/index.php?topic=495.0	conanquotes	f	1
2278	29	Age restrictions for Conan in japan? :: http://forums.dctp.ws/index.php?topic=496.0	conanquotes	f	1
2279	29	Detective Conan AMV - Mythical (song Endless Love) :: http://forums.dctp.ws/index.php?topic=497.0	conanquotes	f	1
2280	29	I'm Back! :: http://forums.dctp.ws/index.php?topic=498.0	conanquotes	f	1
2281	29	BO big boss would be revealed soon !!! :: http://forums.dctp.ws/index.php?topic=499.0	conanquotes	f	1
2282	29	Help Pleas! Searching for a Conan song!!? :: http://forums.dctp.ws/index.php?topic=503.0	conanquotes	f	1
2283	29	Cutepeipei's Drawings... :: http://forums.dctp.ws/index.php?topic=504.0	conanquotes	f	1
2284	29	Movie 12: Full Score of Fear Announcement :: http://forums.dctp.ws/index.php?topic=505.0	conanquotes	f	1
2285	29	Which of these three detectives do you like the most? :: http://forums.dctp.ws/index.php?topic=506.0	conanquotes	f	1
2286	29	Reseed request :] Detective Conan 435-436v2 :: http://forums.dctp.ws/index.php?topic=507.0	conanquotes	f	1
2287	29	What do you think is under AKAI SHUICHI'S hat? :: http://forums.dctp.ws/index.php?topic=508.0	conanquotes	f	1
2288	29	[DCTP] Detective Conan Episode 508 :: http://forums.dctp.ws/index.php?topic=509.0	conanquotes	f	1
2289	29	* Hondou Eisuke, Akai Shuichi and Okiya Subaru.  -SPOiLERS MAYBE- :: http://forums.dctp.ws/index.php?topic=510.0	conanquotes	f	1
2290	29	More speculation (ugh), The true name of the Black Organization :: http://forums.dctp.ws/index.php?topic=511.0	conanquotes	f	1
2291	29	Why does Vermouth cares so much?... :: http://forums.dctp.ws/index.php?topic=512.0	conanquotes	f	1
2292	29	Why Conan Shrank :: http://forums.dctp.ws/index.php?topic=513.0	conanquotes	f	1
2293	29	Black Org boss('es) do we know him/her yet ? :: http://forums.dctp.ws/index.php?topic=515.0	conanquotes	f	1
2294	29	lyf :: http://forums.dctp.ws/index.php?topic=517.0	conanquotes	f	1
2295	29	File 661: Boss hint? :: http://forums.dctp.ws/index.php?topic=518.0	conanquotes	f	1
2296	29	Wedding - which episode?? :: http://forums.dctp.ws/index.php?topic=519.0	conanquotes	f	1
2297	29	What does Heiji Call  Haibara? :: http://forums.dctp.ws/index.php?topic=520.0	conanquotes	f	1
2298	29	Peticion :: http://forums.dctp.ws/index.php?topic=521.0	conanquotes	f	1
2300	29	Does anyone know where is this picture from???! :: http://forums.dctp.ws/index.php?topic=525.0	conanquotes	f	1
2301	29	Updates on DC outside Japan :: http://forums.dctp.ws/index.php?topic=527.0	conanquotes	f	1
2302	29	Kirite's "drawings" :: http://forums.dctp.ws/index.php?topic=529.0	conanquotes	f	1
2303	29	- = : Detective : Conan : Fanfic : = - :: http://forums.dctp.ws/index.php?topic=530.0	conanquotes	f	1
2304	29	DC works for the Gov't (in real life) :: http://forums.dctp.ws/index.php?topic=531.0	conanquotes	f	1
2305	29	Kappei Yamaguchi  :: http://forums.dctp.ws/index.php?topic=532.0	conanquotes	f	1
2306	29	Xbox Live Gametags :: http://forums.dctp.ws/index.php?topic=533.0	conanquotes	f	1
2307	29	Sub Status Page on Kaizoufansubs Website? :: http://forums.dctp.ws/index.php?topic=534.0	conanquotes	f	1
2308	29	[Discussion] .:Decent Games for the Wii:.  :: http://forums.dctp.ws/index.php?topic=535.0	conanquotes	f	1
2309	29	what episode is this? :: http://forums.dctp.ws/index.php?topic=536.0	conanquotes	f	1
2310	29	Sherlock Homes and the Red Demon :: http://forums.dctp.ws/index.php?topic=537.0	conanquotes	f	1
2311	29	9Dragons :: http://forums.dctp.ws/index.php?topic=539.0	conanquotes	f	1
2312	29	Fansite list :: http://forums.dctp.ws/index.php?topic=540.0	conanquotes	f	1
2313	29	my new pics what up all  :: http://forums.dctp.ws/index.php?topic=541.0	conanquotes	f	1
2314	29	My drawings (warning, some images are big) :: http://forums.dctp.ws/index.php?topic=542.0	conanquotes	f	1
2315	29	Can anybody help? O.o :: http://forums.dctp.ws/index.php?topic=543.0	conanquotes	f	1
2316	29	Which is the episode when... :: http://forums.dctp.ws/index.php?topic=544.0	conanquotes	f	1
2317	29	Mini fanfics, haha... :: http://forums.dctp.ws/index.php?topic=545.0	conanquotes	f	1
2318	29	Questions about recording Detective Conan :: http://forums.dctp.ws/index.php?topic=546.0	conanquotes	f	1
2320	29	Sad Murder Cases :: http://forums.dctp.ws/index.php?topic=548.0	conanquotes	f	1
2322	29	[DCTP] Detective Conan Files 658-660 :: http://forums.dctp.ws/index.php?topic=550.0	conanquotes	f	1
2323	29	just wondering :: http://forums.dctp.ws/index.php?topic=551.0	conanquotes	f	1
2319	29	I want to learn Japanese :: http://forums.dctp.ws/index.php?topic=547.0	conanquotes	f	2
2299	29	HAND UP FREEZE :: http://forums.dctp.ws/index.php?topic=522.0	conanquotes	f	2
2324	29	Kaito Kid & Kudo Shinichi on Ipod Commercial! xD :: http://forums.dctp.ws/index.php?topic=553.0	conanquotes	f	1
2325	29	[DCTP] Detective Conan File 661 :: http://forums.dctp.ws/index.php?topic=554.0	conanquotes	f	1
2326	29	Conan singing Boku Ga Iru   Conan looks likes he's really into it  :: http://forums.dctp.ws/index.php?topic=556.0	conanquotes	f	1
2327	29	Chapter issues :: http://forums.dctp.ws/index.php?topic=558.0	conanquotes	f	1
2328	29	509 coming...? xD :: http://forums.dctp.ws/index.php?topic=560.0	conanquotes	f	1
2329	29	Request 401-402 :: http://forums.dctp.ws/index.php?topic=561.0	conanquotes	f	1
2330	29	[DCTP] Detective Conan Episode 509 :: http://forums.dctp.ws/index.php?topic=562.0	conanquotes	f	1
2331	29	Detective/Police Ranks for members :D :: http://forums.dctp.ws/index.php?topic=563.0	conanquotes	f	1
2332	29	Spoiler...? :: http://forums.dctp.ws/index.php?topic=564.0	conanquotes	f	1
2333	29	Conan Movie 8 broken? :: http://forums.dctp.ws/index.php?topic=565.0	conanquotes	f	1
2334	29	Disk Space? :: http://forums.dctp.ws/index.php?topic=567.0	conanquotes	f	1
2335	29	Staff Introduction :: http://forums.dctp.ws/index.php?topic=569.0	conanquotes	f	1
2336	29	Hi, new here... :: http://forums.dctp.ws/index.php?topic=570.0	conanquotes	f	1
2338	29	Mitsuhiko Tsuburaya  + Ai Haibara  will he smush in the next drawing  :: http://forums.dctp.ws/index.php?topic=573.0	conanquotes	f	1
2339	29	Genta's Look :: http://forums.dctp.ws/index.php?topic=574.0	conanquotes	f	1
2340	29	Google.com down? :: http://forums.dctp.ws/index.php?topic=575.0	conanquotes	f	1
2341	29	Shuichi Akai and  Subaru Okiya   what you think ? of this  :: http://forums.dctp.ws/index.php?topic=576.0	conanquotes	f	1
2342	29	!DDL eps.411 :: http://forums.dctp.ws/index.php?topic=577.0	conanquotes	f	1
2343	29	[DCTP] Detective Conan File 662 :: http://forums.dctp.ws/index.php?topic=578.0	conanquotes	f	1
2344	29	Chapter 662 :: http://forums.dctp.ws/index.php?topic=579.0	conanquotes	f	1
2345	29	REQ: reseed rotten apple story arc :: http://forums.dctp.ws/index.php?topic=580.0	conanquotes	f	1
2346	29	kirite here my new  prettyful signature  what you  think oh and of my newdrawing :: http://forums.dctp.ws/index.php?topic=581.0	conanquotes	f	1
2347	29	AVatar up grade for Maruku cus old one just not that good  here you go man  :: http://forums.dctp.ws/index.php?topic=585.0	conanquotes	f	1
2348	29	188-193 :: http://forums.dctp.ws/index.php?topic=586.0	conanquotes	f	1
2349	29	Itachi. here your avatar you ask me two make you   injoy  :: http://forums.dctp.ws/index.php?topic=587.0	conanquotes	f	1
2350	29	conan and haibara sitting together... which episode? :: http://forums.dctp.ws/index.php?topic=588.0	conanquotes	f	1
2351	29	can you name her _? :: http://forums.dctp.ws/index.php?topic=590.0	conanquotes	f	1
2352	29	Crime Discussion: Robert Hansen :: http://forums.dctp.ws/index.php?topic=592.0	conanquotes	f	1
2353	29	What would be the right age to watch Detective Conan? :: http://forums.dctp.ws/index.php?topic=593.0	conanquotes	f	1
2354	29	episodes 127 jump? SPOILERS :: http://forums.dctp.ws/index.php?topic=594.0	conanquotes	f	1
2355	29	Happy birthday  Thief1412  and erly present from me  :: http://forums.dctp.ws/index.php?topic=595.0	conanquotes	f	1
2356	29	here kakashi true face. Itachi what do you think of it ? :: http://forums.dctp.ws/index.php?topic=596.0	conanquotes	f	1
2357	29	FILE 663 (The Witch's True Identify) :: http://forums.dctp.ws/index.php?topic=597.0	conanquotes	f	1
2358	29	Direct Download: Down for the count :: http://forums.dctp.ws/index.php?topic=598.0	conanquotes	f	1
2359	29	Let's Play Shiritori! :: http://forums.dctp.ws/index.php?topic=601.0	conanquotes	f	1
2360	29	Future Possible Story About Bourbon :: http://forums.dctp.ws/index.php?topic=602.0	conanquotes	f	1
2361	29	[DCTP] Detective Conan File 663 :: http://forums.dctp.ws/index.php?topic=603.0	conanquotes	f	1
2362	29	the old :: http://forums.dctp.ws/index.php?topic=604.0	conanquotes	f	1
2363	29	DCTP Needs an Image Person! :: http://forums.dctp.ws/index.php?topic=605.0	conanquotes	f	1
2364	29	Request for 323-324, 375, 376 and 449 :: http://forums.dctp.ws/index.php?topic=606.0	conanquotes	f	1
2365	29	[DCTP] Detective Conan Episode 510 :: http://forums.dctp.ws/index.php?topic=607.0	conanquotes	f	1
2366	29	Vodka  gin little brothere   :: http://forums.dctp.ws/index.php?topic=608.0	conanquotes	f	1
2367	29	Direct Download: Back in the ring :: http://forums.dctp.ws/index.php?topic=611.0	conanquotes	f	1
2368	29	Disappearing Posts :: http://forums.dctp.ws/index.php?topic=612.0	conanquotes	f	1
2369	29	here you go  itachi  you  ask for  it and i made it for you now injoy :: http://forums.dctp.ws/index.php?topic=613.0	conanquotes	f	1
2370	29	Spoiler Tags Added :: http://forums.dctp.ws/index.php?topic=615.0	conanquotes	f	1
2371	29	Why not XCHKS, GS, B-Org, etc. files? :: http://forums.dctp.ws/index.php?topic=616.0	conanquotes	f	1
2372	29	REQ: reseed dark contact story arc :: http://forums.dctp.ws/index.php?topic=618.0	conanquotes	f	1
2373	29	Requesting 356 :: http://forums.dctp.ws/index.php?topic=619.0	conanquotes	f	1
2374	29	IRC downloading help :: http://forums.dctp.ws/index.php?topic=620.0	conanquotes	f	1
2375	29	manga or anime? :: http://forums.dctp.ws/index.php?topic=624.0	conanquotes	f	1
2376	29	[DCTP] Detective Conan File 664 :: http://forums.dctp.ws/index.php?topic=625.0	conanquotes	f	1
2377	29	Magic Kaito :: http://forums.dctp.ws/index.php?topic=626.0	conanquotes	f	1
2378	29	Miwako Sato's True Car? :: http://forums.dctp.ws/index.php?topic=627.0	conanquotes	f	1
2379	29	AI QQ all over  :: http://forums.dctp.ws/index.php?topic=629.0	conanquotes	f	1
2380	29	[Kaizou] Lupin the Third Y Chapter 001 :: http://forums.dctp.ws/index.php?topic=630.0	conanquotes	f	1
2381	29	So why does it take 40 people 2 weeks to sub 1 episode???? :: http://forums.dctp.ws/index.php?topic=631.0	conanquotes	f	1
2382	29	Official Episode Requests Thread :: http://forums.dctp.ws/index.php?topic=632.0	conanquotes	f	1
2383	29	Yusaku Kudo teen face = weth no glases  young form  :: http://forums.dctp.ws/index.php?topic=633.0	conanquotes	f	1
2384	29	Episode numbering error? :: http://forums.dctp.ws/index.php?topic=634.0	conanquotes	f	1
2385	29	Music in Conan's Episodes :: http://forums.dctp.ws/index.php?topic=635.0	conanquotes	f	1
2386	29	FLAC :: http://forums.dctp.ws/index.php?topic=636.0	conanquotes	f	1
2387	29	Complete randomness about Case Closed website :: http://forums.dctp.ws/index.php?topic=637.0	conanquotes	f	1
2388	29	Detective conan subs :: http://forums.dctp.ws/index.php?topic=638.0	conanquotes	f	1
2389	29	conan about two tell the trurh and freak out AI  :: http://forums.dctp.ws/index.php?topic=639.0	conanquotes	f	1
2390	29	I'm wondering on the status of DCTP. :: http://forums.dctp.ws/index.php?topic=640.0	conanquotes	f	1
2391	29	Case Closed :: http://forums.dctp.ws/index.php?topic=641.0	conanquotes	f	1
2392	29	[Kaizou] Lupin the Third Y Chapter 002 :: http://forums.dctp.ws/index.php?topic=644.0	conanquotes	f	1
2393	29	Favorite characters :: http://forums.dctp.ws/index.php?topic=645.0	conanquotes	f	1
2394	29	Anybody Watch the Japanese Series "33pun Tantei"? :: http://forums.dctp.ws/index.php?topic=646.0	conanquotes	f	1
2395	29	Greatest Rivalry :: http://forums.dctp.ws/index.php?topic=650.0	conanquotes	f	1
2396	29	FanFic. "In and out. "Stoned" case." :: http://forums.dctp.ws/index.php?topic=651.0	conanquotes	f	1
2397	29	conan chilling :: http://forums.dctp.ws/index.php?topic=652.0	conanquotes	f	1
2398	29	FREEZEE AI :: http://forums.dctp.ws/index.php?topic=653.0	conanquotes	f	1
2399	29	Hair colours :: http://forums.dctp.ws/index.php?topic=654.0	conanquotes	f	1
2400	29	a qicky old drawn i made long ago who   is he? can you name him? :: http://forums.dctp.ws/index.php?topic=656.0	conanquotes	f	1
2401	29	kaito kid glareing at you  :: http://forums.dctp.ws/index.php?topic=657.0	conanquotes	f	1
2402	29	you know her  write  :: http://forums.dctp.ws/index.php?topic=658.0	conanquotes	f	1
2403	29	File 665 :: http://forums.dctp.ws/index.php?topic=659.0	conanquotes	f	1
2404	29	[DCTP] Detective Conan File 665 :: http://forums.dctp.ws/index.php?topic=662.0	conanquotes	f	1
2405	29	Anymore DC episodes :: http://forums.dctp.ws/index.php?topic=664.0	conanquotes	f	1
2406	29	Re-Seed Request Removed?? :: http://forums.dctp.ws/index.php?topic=665.0	conanquotes	f	1
2407	29	Favorite Opening songs :: http://forums.dctp.ws/index.php?topic=667.0	conanquotes	f	1
2408	29	just wanna share (DC OP1-23 songs) :: http://forums.dctp.ws/index.php?topic=670.0	conanquotes	f	1
2409	29	Status Update II: Upcoming Projects :: http://forums.dctp.ws/index.php?topic=671.0	conanquotes	f	1
2410	29	Detective Conan getting cancelled in Japan?... :: http://forums.dctp.ws/index.php?topic=672.0	conanquotes	f	1
2411	29	[DCTP] Detective Conan Movie 12: Full Score of Fear (International Edition) :: http://forums.dctp.ws/index.php?topic=673.0	conanquotes	f	1
2412	29	Sonoko and  Makoto back 2 back  :: http://forums.dctp.ws/index.php?topic=674.0	conanquotes	f	1
2413	29	from what episode is this picture? :: http://forums.dctp.ws/index.php?topic=676.0	conanquotes	f	1
2414	29	episodes 492 to 499 avi files - clash of red and black :: http://forums.dctp.ws/index.php?topic=677.0	conanquotes	f	1
2415	29	[DCTP] Detective Conan Episode 511 :: http://forums.dctp.ws/index.php?topic=679.0	conanquotes	f	1
2416	29	Chaos;Head :: http://forums.dctp.ws/index.php?topic=680.0	conanquotes	f	1
2417	29	Lupin 2006 Released! :: http://forums.dctp.ws/index.php?topic=681.0	conanquotes	f	1
2418	29	Lupin the Third board :: http://forums.dctp.ws/index.php?topic=682.0	conanquotes	f	1
2419	29	akai's... :: http://forums.dctp.ws/index.php?topic=683.0	conanquotes	f	1
2420	29	bourbon's identity :: http://forums.dctp.ws/index.php?topic=684.0	conanquotes	f	1
2421	29	we can be both of god and the devil, since we're trying to raise the dead again :: http://forums.dctp.ws/index.php?topic=685.0	conanquotes	f	1
2422	29	KID, conan and BO :: http://forums.dctp.ws/index.php?topic=686.0	conanquotes	f	1
2423	29	Quoting problem :: http://forums.dctp.ws/index.php?topic=687.0	conanquotes	f	1
2424	29	Official DC Art :: http://forums.dctp.ws/index.php?topic=688.0	conanquotes	f	1
2425	29	Rotten Apple Story Arc :: http://forums.dctp.ws/index.php?topic=690.0	conanquotes	f	1
2426	29	Ideas for Jd-'s new online name :: http://forums.dctp.ws/index.php?topic=691.0	conanquotes	f	1
2427	29	Thanks :: http://forums.dctp.ws/index.php?topic=692.0	conanquotes	f	1
2428	29	[DCTP] Detective Conan File 666 :: http://forums.dctp.ws/index.php?topic=693.0	conanquotes	f	1
2429	29	ok i be  organized from now on :: http://forums.dctp.ws/index.php?topic=694.0	conanquotes	f	1
2430	29	The irc password wont work :: http://forums.dctp.ws/index.php?topic=695.0	conanquotes	f	1
2431	29	[DCTP] Detective Conan Episode 513 :: http://forums.dctp.ws/index.php?topic=696.0	conanquotes	f	1
2432	29	[DCTP] Detective Conan Movie 12 (Trailer) :: http://forums.dctp.ws/index.php?topic=697.0	conanquotes	f	1
2433	29	Reworking episodes/files :: http://forums.dctp.ws/index.php?topic=698.0	conanquotes	f	1
2434	29	I am not sure where to post this :: http://forums.dctp.ws/index.php?topic=699.0	conanquotes	f	1
2435	29	Skip Beat Anime! :: http://forums.dctp.ws/index.php?topic=700.0	conanquotes	f	1
2436	29	Lupin Green vs. Red released! :: http://forums.dctp.ws/index.php?topic=701.0	conanquotes	f	1
2437	29	Please reseed OVA 6 and ep 464 :: http://forums.dctp.ws/index.php?topic=702.0	conanquotes	f	1
2438	29	Something I saw after our retreat :: http://forums.dctp.ws/index.php?topic=703.0	conanquotes	f	1
2439	29	[DCTP] Detective Conan Episodes 335-336 :: http://forums.dctp.ws/index.php?topic=704.0	conanquotes	f	1
2440	29	Naruto's opening theme (aoi sora)... anyone has it? :: http://forums.dctp.ws/index.php?topic=705.0	conanquotes	f	1
2441	29	vermouth, kid, may be a connection :: http://forums.dctp.ws/index.php?topic=706.0	conanquotes	f	1
2442	29	Conan & Ai  :: http://forums.dctp.ws/index.php?topic=707.0	conanquotes	f	1
2443	29	[DCTP] Detective Conan Magic File 2 (Movie 12 Side-Story OVA) :: http://forums.dctp.ws/index.php?topic=708.0	conanquotes	f	1
2444	29	Shinichi and Ran cameos :: http://forums.dctp.ws/index.php?topic=709.0	conanquotes	f	1
2445	29	Detective Conan + OVAs + Movies + Live Actions [AVI]-[Current Episode: 531] :: http://forums.dctp.ws/index.php?topic=710.0	conanquotes	f	1
2446	29	New OP & ED Samples :: http://forums.dctp.ws/index.php?topic=712.0	conanquotes	f	1
2447	29	APTX 4869  :: http://forums.dctp.ws/index.php?topic=713.0	conanquotes	f	1
2448	29	Anime DVD Download :: http://forums.dctp.ws/index.php?topic=715.0	conanquotes	f	1
2449	29	[DCTP] Detective Conan Episode 514 :: http://forums.dctp.ws/index.php?topic=717.0	conanquotes	f	1
2450	29	[DCTP] Detective Conan File 667 :: http://forums.dctp.ws/index.php?topic=719.0	conanquotes	f	1
2451	29	works of mine :: http://forums.dctp.ws/index.php?topic=720.0	conanquotes	f	1
2453	29	Unsolved Mysteries Returns To TV Tomorrow! :: http://forums.dctp.ws/index.php?topic=723.0	conanquotes	f	1
2454	29	NYC Comic Con 2009 :: http://forums.dctp.ws/index.php?topic=724.0	conanquotes	f	1
2455	29	Tab icons for DCTP website :: http://forums.dctp.ws/index.php?topic=725.0	conanquotes	f	1
2456	29	D.Gray-Man :: http://forums.dctp.ws/index.php?topic=726.0	conanquotes	f	1
2457	29	Katekyo hitman reborn fans anyoneee??? :: http://forums.dctp.ws/index.php?topic=727.0	conanquotes	f	1
2458	29	Name Meanings :: http://forums.dctp.ws/index.php?topic=728.0	conanquotes	f	1
2459	29	Short preview about movie 13 :: http://forums.dctp.ws/index.php?topic=729.0	conanquotes	f	1
2460	29	Tazmo - Profiting from free stuff :: http://forums.dctp.ws/index.php?topic=730.0	conanquotes	f	1
2462	29	Real locations in detective conan :: http://forums.dctp.ws/index.php?topic=732.0	conanquotes	f	1
2463	29	One Piece :: http://forums.dctp.ws/index.php?topic=733.0	conanquotes	f	1
2464	29	Fake OVA? :: http://forums.dctp.ws/index.php?topic=734.0	conanquotes	f	1
2465	29	Interview with Techman (Coming Soon!) :: http://forums.dctp.ws/index.php?topic=735.0	conanquotes	f	1
2466	29	Vermouth's true agenda (unmarked spoilers for volumes 24-63) :: http://forums.dctp.ws/index.php?topic=736.0	conanquotes	f	1
2467	29	Words of gin in Movie 13 preview :: http://forums.dctp.ws/index.php?topic=737.0	conanquotes	f	1
2468	29	Random drabbles... Scars ??  :: http://forums.dctp.ws/index.php?topic=738.0	conanquotes	f	1
2469	29	[DCTP] Detective Conan File 668 :: http://forums.dctp.ws/index.php?topic=739.0	conanquotes	f	1
2472	29	DCTP Family Tree and Timeline :: http://forums.dctp.ws/index.php?topic=743.0	conanquotes	f	1
2473	29	The information of M13 will come out in the next 3 weeks :: http://forums.dctp.ws/index.php?topic=744.0	conanquotes	f	1
2474	29	Movie 10,11 & 12 OSTs :: http://forums.dctp.ws/index.php?topic=745.0	conanquotes	f	1
2475	29	Wait, is that really...? :: http://forums.dctp.ws/index.php?topic=746.0	conanquotes	f	1
2476	29	Movie 10 :: http://forums.dctp.ws/index.php?topic=747.0	conanquotes	f	1
2477	29	Detective Conan Video Games Translation :: http://forums.dctp.ws/index.php?topic=748.0	conanquotes	f	1
2478	29	668 :: http://forums.dctp.ws/index.php?topic=749.0	conanquotes	f	1
2479	29	I need to know where this scene is from!!!! :: http://forums.dctp.ws/index.php?topic=750.0	conanquotes	f	1
2480	29	DC Character's Birthdays :: http://forums.dctp.ws/index.php?topic=751.0	conanquotes	f	1
2482	29	hi!!!  :: http://forums.dctp.ws/index.php?topic=753.0	conanquotes	f	1
2483	29	Favorite Conan gadget :: http://forums.dctp.ws/index.php?topic=755.0	conanquotes	f	1
2484	29	Could you help me understand how Eisuke's blood type changed? :: http://forums.dctp.ws/index.php?topic=756.0	conanquotes	f	1
2485	29	New Information of M13(OCT. 27) :: http://forums.dctp.ws/index.php?topic=757.0	conanquotes	f	1
2486	29	[DCTP] Detective Conan File 669 :: http://forums.dctp.ws/index.php?topic=759.0	conanquotes	f	1
2487	29	Lol, Conan Pown? :: http://forums.dctp.ws/index.php?topic=760.0	conanquotes	f	1
2488	29	Question about the Movies and ovas :: http://forums.dctp.ws/index.php?topic=761.0	conanquotes	f	1
2489	29	MUST SEE Anime original episodes :: http://forums.dctp.ws/index.php?topic=762.0	conanquotes	f	1
2490	29	older episode search :: http://forums.dctp.ws/index.php?topic=763.0	conanquotes	f	1
2491	29	[DCTP] Detective Conan Movie 12 (Coming Today!) :: http://forums.dctp.ws/index.php?topic=764.0	conanquotes	f	1
2492	29	Movie 13 (Black Org): "Jet Black Chaser" Discussion :: http://forums.dctp.ws/index.php?topic=765.0	conanquotes	f	1
2493	29	Detective Conan File 670 RAW :: http://forums.dctp.ws/index.php?topic=766.0	conanquotes	f	1
2494	29	Ran Suspicions of Conan Eps :: http://forums.dctp.ws/index.php?topic=768.0	conanquotes	f	1
2495	29	My Detective Conan/Case Closed Cosplay :: http://forums.dctp.ws/index.php?topic=769.0	conanquotes	f	1
2496	29	1080p releases of 504-507 :: http://forums.dctp.ws/index.php?topic=770.0	conanquotes	f	1
2497	29	Where we got our usernames :: http://forums.dctp.ws/index.php?topic=771.0	conanquotes	f	1
2498	29	when will movie 11 and movie 12 comeout? :: http://forums.dctp.ws/index.php?topic=772.0	conanquotes	f	1
2499	29	must episodes! :: http://forums.dctp.ws/index.php?topic=773.0	conanquotes	f	1
2500	29	Detective Conan Soundtracks Collection :: http://forums.dctp.ws/index.php?topic=774.0	conanquotes	f	1
2501	29	[DCTP] Detective Conan File 670 :: http://forums.dctp.ws/index.php?topic=775.0	conanquotes	f	1
2502	29	need help in translate a signature on my  swords  tang  :: http://forums.dctp.ws/index.php?topic=777.0	conanquotes	f	1
2461	29	Continuation of Previous Akai Shuichi thread :: http://forums.dctp.ws/index.php?topic=731.0	conanquotes	f	2
2503	29	Can't Play HD Files? Look here! :: http://forums.dctp.ws/index.php?topic=778.0	conanquotes	f	1
2504	29	better couple :: http://forums.dctp.ws/index.php?topic=779.0	conanquotes	f	1
2505	29	Harry Potter :: http://forums.dctp.ws/index.php?topic=781.0	conanquotes	f	1
2506	29	Is Detective Conan becoming lame? :: http://forums.dctp.ws/index.php?topic=782.0	conanquotes	f	1
2507	29	Facts and Speculations :: http://forums.dctp.ws/index.php?topic=783.0	conanquotes	f	1
2508	29	[DCTP] Detective Conan Movie 12: Full Score of Fear :: http://forums.dctp.ws/index.php?topic=784.0	conanquotes	f	1
2509	29	REBORN! colorings :: http://forums.dctp.ws/index.php?topic=785.0	conanquotes	f	1
2510	29	Work in progress HSD Kenichi quadruple spread :: http://forums.dctp.ws/index.php?topic=786.0	conanquotes	f	1
2511	29	twilight SAGA :: http://forums.dctp.ws/index.php?topic=793.0	conanquotes	f	1
2512	29	what you want! :: http://forums.dctp.ws/index.php?topic=794.0	conanquotes	f	1
2513	29	Detective conan movie 12 :: http://forums.dctp.ws/index.php?topic=796.0	conanquotes	f	1
2514	29	No Torrent for Ep 425 :: http://forums.dctp.ws/index.php?topic=797.0	conanquotes	f	1
2515	29	Evil deeds and crimes done by Conan :: http://forums.dctp.ws/index.php?topic=798.0	conanquotes	f	1
2516	29	May I ask when will M12 HD and multi-language release? :: http://forums.dctp.ws/index.php?topic=799.0	conanquotes	f	1
2517	29	A MAN WANTS TO MARRY A MANGA CHARACTER????????? :: http://forums.dctp.ws/index.php?topic=800.0	conanquotes	f	1
2518	29	Quick (stupid?) question :: http://forums.dctp.ws/index.php?topic=803.0	conanquotes	f	1
2519	29	Interview with Techman :: http://forums.dctp.ws/index.php?topic=804.0	conanquotes	f	1
2520	29	The End... Of DC? :: http://forums.dctp.ws/index.php?topic=805.0	conanquotes	f	1
2521	29	DC movie question :: http://forums.dctp.ws/index.php?topic=806.0	conanquotes	f	1
2522	29	How smart are Japanese people? :: http://forums.dctp.ws/index.php?topic=808.0	conanquotes	f	1
2523	29	[DCTP] Detective Conan Movie 12: Full Score of Fear OST :: http://forums.dctp.ws/index.php?topic=810.0	conanquotes	f	1
2524	29	Detective Conan File 671 RAW :: http://forums.dctp.ws/index.php?topic=811.0	conanquotes	f	1
2525	29	Announcement from Kudo's Bookshelf :: http://forums.dctp.ws/index.php?topic=812.0	conanquotes	f	1
2527	29	OVA 4, please? :: http://forums.dctp.ws/index.php?topic=816.0	conanquotes	f	1
2528	29	What exactly is Shiho's specialty? :: http://forums.dctp.ws/index.php?topic=817.0	conanquotes	f	1
2529	29	My DC Artwork :: http://forums.dctp.ws/index.php?topic=818.0	conanquotes	f	1
2530	29	High School Girl Detective Sonoko Suzuki's Case Files? :: http://forums.dctp.ws/index.php?topic=819.0	conanquotes	f	1
2531	29	Wanna know how old are the people that watch DC? :: http://forums.dctp.ws/index.php?topic=820.0	conanquotes	f	1
2532	29	just a suggestion.... :: http://forums.dctp.ws/index.php?topic=823.0	conanquotes	f	1
2272	29	well this sum of my fan art i made this past last  2 week  hope you all like it  :: http://forums.dctp.ws/index.php?topic=485.0	conanquotes	f	2
2533	29	Conan's pitch :: http://forums.dctp.ws/index.php?topic=824.0	conanquotes	f	1
2534	29	[DCTP] Detective Conan File 671 :: http://forums.dctp.ws/index.php?topic=825.0	conanquotes	f	1
2535	29	[DCTP] Detective Conan Episode 515 :: http://forums.dctp.ws/index.php?topic=827.0	conanquotes	f	1
2536	29	671 Murderer :: http://forums.dctp.ws/index.php?topic=829.0	conanquotes	f	1
2537	29	Detective Conan File 672 RAW :: http://forums.dctp.ws/index.php?topic=831.0	conanquotes	f	1
2538	29	[HSD Ken'ichi] Miu Furinji :: http://forums.dctp.ws/index.php?topic=832.0	conanquotes	f	1
2539	29	The manga book..... :: http://forums.dctp.ws/index.php?topic=833.0	conanquotes	f	1
2540	29	[DCTP] Detective Conan TV: Re-launched! (UPDATE) :: http://forums.dctp.ws/index.php?topic=834.0	conanquotes	f	1
2541	29	Full Metal Alchemist 2 :: http://forums.dctp.ws/index.php?topic=835.0	conanquotes	f	1
2542	29	[DCTP] Detective Conan Episode 512 (HD Released!) :: http://forums.dctp.ws/index.php?topic=836.0	conanquotes	f	1
2543	29	Mysteries :: http://forums.dctp.ws/index.php?topic=837.0	conanquotes	f	1
2544	29	Nico Nico Medley, Conan style. :: http://forums.dctp.ws/index.php?topic=838.0	conanquotes	f	1
2545	29	Conan's watch :: http://forums.dctp.ws/index.php?topic=839.0	conanquotes	f	1
2546	29	Movie 12: Conan's absolute pitch :: http://forums.dctp.ws/index.php?topic=841.0	conanquotes	f	1
2547	29	Vote for the icon for Kudo's Bookshelf! :: http://forums.dctp.ws/index.php?topic=842.0	conanquotes	f	1
2548	29	Creative word game. :: http://forums.dctp.ws/index.php?topic=843.0	conanquotes	f	1
2549	29	Soundtrack 12 Questions :: http://forums.dctp.ws/index.php?topic=844.0	conanquotes	f	1
2550	29	[DCTP] Detective Conan Episode 516 :: http://forums.dctp.ws/index.php?topic=845.0	conanquotes	f	1
2551	29	[DCTP] Detective Conan File 672 :: http://forums.dctp.ws/index.php?topic=846.0	conanquotes	f	1
2552	29	[fixed]Issue W/ DCTP iRC  :: http://forums.dctp.ws/index.php?topic=847.0	conanquotes	f	1
2553	29	Detective Conan File 673 RAW :: http://forums.dctp.ws/index.php?topic=848.0	conanquotes	f	1
2554	29	DCTP is looking for Japanese & Chinese translators! :: http://forums.dctp.ws/index.php?topic=849.0	conanquotes	f	1
2555	29	Detective Sherry ^^ :: http://forums.dctp.ws/index.php?topic=850.0	conanquotes	f	1
2556	29	[DCTP] New Premiere Coming Tomorrow! :: http://forums.dctp.ws/index.php?topic=851.0	conanquotes	f	1
2557	29	Ran's role changed? Other's too? :: http://forums.dctp.ws/index.php?topic=852.0	conanquotes	f	1
2558	29	MOVED: Crack The Code :: http://forums.dctp.ws/index.php?topic=853.0	conanquotes	f	1
2559	29	[DCTP] Detective Conan Episodes 338-339 :: http://forums.dctp.ws/index.php?topic=854.0	conanquotes	f	1
2560	29	why? :: http://forums.dctp.ws/index.php?topic=856.0	conanquotes	f	1
2561	29	Lost+Brain finished :: http://forums.dctp.ws/index.php?topic=857.0	conanquotes	f	1
2562	29	Anybody else having trouble with the DCTP TV chat? :: http://forums.dctp.ws/index.php?topic=858.0	conanquotes	f	1
2563	29	Okay, I cannot believe I am doing this. DCTV Marathon Ideas! :: http://forums.dctp.ws/index.php?topic=859.0	conanquotes	f	1
2564	29	Volumes 40-43 :: http://forums.dctp.ws/index.php?topic=861.0	conanquotes	f	1
2565	29	Detective Conan Episodes 338-339  ( Questions ) :: http://forums.dctp.ws/index.php?topic=862.0	conanquotes	f	1
2566	29	[DCTP] Detective Conan File 673 :: http://forums.dctp.ws/index.php?topic=863.0	conanquotes	f	1
2567	29	detective conan 222-224 :: http://forums.dctp.ws/index.php?topic=864.0	conanquotes	f	1
2568	29	Ova 4 I NEED TORRENT :: http://forums.dctp.ws/index.php?topic=865.0	conanquotes	f	1
2569	29	Chapter 673 :: http://forums.dctp.ws/index.php?topic=868.0	conanquotes	f	1
2570	29	hmm..the live action special #2 wasn't half bad :: http://forums.dctp.ws/index.php?topic=869.0	conanquotes	f	1
2571	29	Gosho Aoyama's Collection of Short Stories :: http://forums.dctp.ws/index.php?topic=870.0	conanquotes	f	1
2572	29	Cheap Online Storage? :: http://forums.dctp.ws/index.php?topic=871.0	conanquotes	f	1
2573	29	WHAT IS THIS!!!!! :: http://forums.dctp.ws/index.php?topic=872.0	conanquotes	f	1
2574	29	Track List :: http://forums.dctp.ws/index.php?topic=873.0	conanquotes	f	1
2575	29	DCTP manga :: http://forums.dctp.ws/index.php?topic=876.0	conanquotes	f	1
2576	29	takagi satou scene 673 :: http://forums.dctp.ws/index.php?topic=877.0	conanquotes	f	1
2577	29	Conan movies are AO? :: http://forums.dctp.ws/index.php?topic=878.0	conanquotes	f	1
2578	29	Plot for M13 Revealed from latest Sunday! + Poster :: http://forums.dctp.ws/index.php?topic=879.0	conanquotes	f	1
2579	29	DCTP System Used for HD Encoding? :: http://forums.dctp.ws/index.php?topic=880.0	conanquotes	f	1
2580	29	Anime recommendation :: http://forums.dctp.ws/index.php?topic=881.0	conanquotes	f	1
2581	29	Aoyama's Collection of Short Stories :: http://forums.dctp.ws/index.php?topic=882.0	conanquotes	f	1
2582	29	About Sharon Vineyard :: http://forums.dctp.ws/index.php?topic=883.0	conanquotes	f	1
2583	29	Episode 209,210 and others that are missing :: http://forums.dctp.ws/index.php?topic=884.0	conanquotes	f	1
2584	29	manga recommendations :: http://forums.dctp.ws/index.php?topic=885.0	conanquotes	f	1
2585	29	Detectiveconan.us has gone to the dark side  :: http://forums.dctp.ws/index.php?topic=886.0	conanquotes	f	1
2586	29	674? :: http://forums.dctp.ws/index.php?topic=887.0	conanquotes	f	1
2587	29	Detective Conan: The Raven Chaser :: http://forums.dctp.ws/index.php?topic=888.0	conanquotes	f	1
2588	29	The banner and logo of The Raven Chaser  :: http://forums.dctp.ws/index.php?topic=889.0	conanquotes	f	1
2589	29	detectiveconan.us is back...????? its working again  :: http://forums.dctp.ws/index.php?topic=890.0	conanquotes	f	1
2590	29	Detective Conan File 674 RAW :: http://forums.dctp.ws/index.php?topic=891.0	conanquotes	f	1
2591	29	?.......... :: http://forums.dctp.ws/index.php?topic=892.0	conanquotes	f	1
2592	29	Conan's Height :: http://forums.dctp.ws/index.php?topic=893.0	conanquotes	f	1
2593	29	Is it just me or... :: http://forums.dctp.ws/index.php?topic=894.0	conanquotes	f	1
2594	29	Movie 13.....  marks the end of conan? :: http://forums.dctp.ws/index.php?topic=895.0	conanquotes	f	1
2595	29	Should Conan Die? :: http://forums.dctp.ws/index.php?topic=897.0	conanquotes	f	1
2596	29	HQ Scan of M13 Poster and Latest News! :: http://forums.dctp.ws/index.php?topic=898.0	conanquotes	f	1
2597	29	What if Takagi would be gay? :: http://forums.dctp.ws/index.php?topic=899.0	conanquotes	f	1
2598	29	[DCTP] Detective Conan File 674 :: http://forums.dctp.ws/index.php?topic=900.0	conanquotes	f	1
2599	29	On Our DDL & Donations :: http://forums.dctp.ws/index.php?topic=902.0	conanquotes	f	1
2600	29	DCTP....Help me please... :: http://forums.dctp.ws/index.php?topic=903.0	conanquotes	f	1
2601	29	Iron Tanuki: 674 :: http://forums.dctp.ws/index.php?topic=904.0	conanquotes	f	1
2602	29	[DCTP] Detective Conan Episode 471 :: http://forums.dctp.ws/index.php?topic=905.0	conanquotes	f	1
2603	29	Something strange about shuichi's death...... :: http://forums.dctp.ws/index.php?topic=906.0	conanquotes	f	1
2604	29	Best ending  :: http://forums.dctp.ws/index.php?topic=907.0	conanquotes	f	1
2605	29	DetectiveShinichi.com Now For Your iPhone! :: http://forums.dctp.ws/index.php?topic=908.0	conanquotes	f	1
2606	29	[DCTP] Detective Conan Episode 517 :: http://forums.dctp.ws/index.php?topic=909.0	conanquotes	f	1
2607	29	Wildest theory in DC :: http://forums.dctp.ws/index.php?topic=910.0	conanquotes	f	1
2608	29	New Timeslot for DC? :: http://forums.dctp.ws/index.php?topic=911.0	conanquotes	f	1
2609	29	Another (stupid?) question :: http://forums.dctp.ws/index.php?topic=912.0	conanquotes	f	1
2610	29	About that Movie 11 Announcement... :: http://forums.dctp.ws/index.php?topic=913.0	conanquotes	f	1
2611	29	Stupid but want to know :P :: http://forums.dctp.ws/index.php?topic=914.0	conanquotes	f	1
2612	29	Detective Conan File 675 RAW :: http://forums.dctp.ws/index.php?topic=915.0	conanquotes	f	1
2613	29	two novembers :: http://forums.dctp.ws/index.php?topic=916.0	conanquotes	f	1
2614	29	Please: Very Important :: http://forums.dctp.ws/index.php?topic=918.0	conanquotes	f	1
2114	29	My detective conan story :: http://forums.dctp.ws/index.php?topic=286.0	conanquotes	f	2
2615	29	Pay attention to Conan's hands... :: http://forums.dctp.ws/index.php?topic=919.0	conanquotes	f	1
2616	29	Chinese Conan's fan come here to say hello! :: http://forums.dctp.ws/index.php?topic=920.0	conanquotes	f	1
2617	29	Is it me or??? :: http://forums.dctp.ws/index.php?topic=921.0	conanquotes	f	1
2618	29	The new trailer for movie 13  :: http://forums.dctp.ws/index.php?topic=922.0	conanquotes	f	1
2619	29	When was the last time Conan used his skateboard? :: http://forums.dctp.ws/index.php?topic=923.0	conanquotes	f	1
2620	29	Some clues from the Mermaid Case? :: http://forums.dctp.ws/index.php?topic=924.0	conanquotes	f	1
2621	29	What do you think....movie 13?? :: http://forums.dctp.ws/index.php?topic=925.0	conanquotes	f	1
2622	29	[DCTP] Detective Conan Episode 518 :: http://forums.dctp.ws/index.php?topic=926.0	conanquotes	f	1
2623	29	[DCTP] Detective Conan File 675 :: http://forums.dctp.ws/index.php?topic=927.0	conanquotes	f	1
2624	29	Question: Soundtracks in the DDL :: http://forums.dctp.ws/index.php?topic=928.0	conanquotes	f	1
2625	29	APTX 4869 effects :: http://forums.dctp.ws/index.php?topic=929.0	conanquotes	f	1
2626	29	MY cat kyo here sum picture   of my cat  :: http://forums.dctp.ws/index.php?topic=930.0	conanquotes	f	1
2627	29	(Early) Story Arcs List? :: http://forums.dctp.ws/index.php?topic=931.0	conanquotes	f	1
2628	29	Ending of DC :: http://forums.dctp.ws/index.php?topic=932.0	conanquotes	f	1
2629	29	[DCTP] Detective Conan Episode 329-330, 340-341 & Special OVA :: http://forums.dctp.ws/index.php?topic=933.0	conanquotes	f	1
2630	29	Hockey :: http://forums.dctp.ws/index.php?topic=934.0	conanquotes	f	1
2631	29	Character death poll :: http://forums.dctp.ws/index.php?topic=935.0	conanquotes	f	1
2632	29	Don't shoot me but... :: http://forums.dctp.ws/index.php?topic=936.0	conanquotes	f	1
2633	29	Help w/xmass mirical :: http://forums.dctp.ws/index.php?topic=937.0	conanquotes	f	1
2634	29	tequila? :: http://forums.dctp.ws/index.php?topic=938.0	conanquotes	f	1
2635	29	about Detective Conan movie 13: the Raven Chaser trailer :: http://forums.dctp.ws/index.php?topic=939.0	conanquotes	f	1
2636	29	Detective Conan File 676 RAW :: http://forums.dctp.ws/index.php?topic=940.0	conanquotes	f	1
2637	29	A Request for those who wish to help sub Conan :: http://forums.dctp.ws/index.php?topic=941.0	conanquotes	f	1
2638	29	Someone help me with KID? :: http://forums.dctp.ws/index.php?topic=942.0	conanquotes	f	1
2639	29	Is the new DDL slower? :: http://forums.dctp.ws/index.php?topic=943.0	conanquotes	f	1
2640	29	I was bored so... :: http://forums.dctp.ws/index.php?topic=944.0	conanquotes	f	1
2641	29	Can SomeonE help me?? are there any detective conan games in english language?? :: http://forums.dctp.ws/index.php?topic=945.0	conanquotes	f	1
2642	29	This topic is about the greatest character ever :: http://forums.dctp.ws/index.php?topic=947.0	conanquotes	f	1
2643	29	sumting if found  and i had 2 show you all  :: http://forums.dctp.ws/index.php?topic=948.0	conanquotes	f	1
2644	29	[DCTP] Status Update: 2009 & Beyond :: http://forums.dctp.ws/index.php?topic=949.0	conanquotes	f	1
2645	29	Please explain me something :: http://forums.dctp.ws/index.php?topic=950.0	conanquotes	f	1
2646	29	[DCTP] Detective Conan File 676 :: http://forums.dctp.ws/index.php?topic=951.0	conanquotes	f	1
2647	29	Gambling Apocalypse Kaiji :: http://forums.dctp.ws/index.php?topic=952.0	conanquotes	f	1
2648	29	How long have you been following the DC series? :: http://forums.dctp.ws/index.php?topic=953.0	conanquotes	f	1
2649	29	conan vs. kid - shark & jewel :: http://forums.dctp.ws/index.php?topic=954.0	conanquotes	f	1
2650	29	AN EMERGENCY QUESTION :: http://forums.dctp.ws/index.php?topic=955.0	conanquotes	f	1
2651	29	What is the "Company"? :: http://forums.dctp.ws/index.php?topic=956.0	conanquotes	f	1
2652	29	Sherlock Holmes comes to the Silver Screen! :: http://forums.dctp.ws/index.php?topic=957.0	conanquotes	f	1
2653	29	A Notice to the Manga Team :: http://forums.dctp.ws/index.php?topic=958.0	conanquotes	f	1
2654	29	Code Name RUM   :: http://forums.dctp.ws/index.php?topic=959.0	conanquotes	f	1
2655	29	A question  :: http://forums.dctp.ws/index.php?topic=960.0	conanquotes	f	1
2656	29	what happened to http://kaizoufansubs.net? :: http://forums.dctp.ws/index.php?topic=961.0	conanquotes	f	1
2657	29	Detective Conan piano sheet music :: http://forums.dctp.ws/index.php?topic=962.0	conanquotes	f	1
2658	29	Kindaichi vs Conan  :: http://forums.dctp.ws/index.php?topic=963.0	conanquotes	f	1
2659	29	Detective Conan Theme Sheet Music :: http://forums.dctp.ws/index.php?topic=964.0	conanquotes	f	1
2660	29	Mina's colorings  :: http://forums.dctp.ws/index.php?topic=965.0	conanquotes	f	1
2662	29	Favourite Detective Conan music/opening/ending :: http://forums.dctp.ws/index.php?topic=968.0	conanquotes	f	1
2663	29	How can Kir detect lies? :: http://forums.dctp.ws/index.php?topic=969.0	conanquotes	f	1
2664	29	just curious about the process of doing it :: http://forums.dctp.ws/index.php?topic=970.0	conanquotes	f	1
2665	29	Just questions i'm confused with :: http://forums.dctp.ws/index.php?topic=971.0	conanquotes	f	1
2666	29	Flashgame?!  :: http://forums.dctp.ws/index.php?topic=972.0	conanquotes	f	1
2667	29	What did Vermouth mean to say? :: http://forums.dctp.ws/index.php?topic=973.0	conanquotes	f	1
2668	29	Jd- is an imbecile that I care enough for to make a topic about! <3 :: http://forums.dctp.ws/index.php?topic=974.0	conanquotes	f	1
2669	29	A Riddle for eveyone :: http://forums.dctp.ws/index.php?topic=976.0	conanquotes	f	1
2670	29	FanFic: Forgiveness :: http://forums.dctp.ws/index.php?topic=977.0	conanquotes	f	1
2671	29	Questions about Sherry, the FBI and Shinichi :: http://forums.dctp.ws/index.php?topic=978.0	conanquotes	f	1
2672	29	[DCTP] Detective Conan Episode 519 :: http://forums.dctp.ws/index.php?topic=979.0	conanquotes	f	1
2673	29	Spoiler picture of 677 :: http://forums.dctp.ws/index.php?topic=980.0	conanquotes	f	1
2674	29	my hope for DC in this year of 2009 + poll :: http://forums.dctp.ws/index.php?topic=981.0	conanquotes	f	1
2675	29	479 audio problems :: http://forums.dctp.ws/index.php?topic=982.0	conanquotes	f	1
2676	29	( sword tang ) SUper Quilaty name still unkown (   need help translateing )  :: http://forums.dctp.ws/index.php?topic=983.0	conanquotes	f	1
2677	29	Episodes/movies/OVA's with Ai haibara in them :: http://forums.dctp.ws/index.php?topic=984.0	conanquotes	f	1
2678	29	[DCTP] Detective Conan Episode 342 (One Hour Special - Coming Soon!) :: http://forums.dctp.ws/index.php?topic=985.0	conanquotes	f	1
2679	29	DCTV player :: http://forums.dctp.ws/index.php?topic=986.0	conanquotes	f	1
2680	29	Detective Conan File 677 RAW :: http://forums.dctp.ws/index.php?topic=987.0	conanquotes	f	1
2681	29	[DCTP] Detective Conan File 677 :: http://forums.dctp.ws/index.php?topic=988.0	conanquotes	f	1
2661	29	Kat's Drawings ;) :: http://forums.dctp.ws/index.php?topic=967.0	conanquotes	f	2
2682	29	Can anyone post up the AC3 track of the 12th movie? :: http://forums.dctp.ws/index.php?topic=989.0	conanquotes	f	1
2683	29	The 14th Target dub (not sub) - revisited :: http://forums.dctp.ws/index.php?topic=990.0	conanquotes	f	1
2684	29	Series ending, probably fake, but fun to read. :: http://forums.dctp.ws/index.php?topic=991.0	conanquotes	f	1
2685	29	A case (Number I) :: http://forums.dctp.ws/index.php?topic=992.0	conanquotes	f	1
2686	29	"Akai's return?" Arc Discussion and Poll :: http://forums.dctp.ws/index.php?topic=993.0	conanquotes	f	1
2687	29	Lateral Thinking I :: http://forums.dctp.ws/index.php?topic=995.0	conanquotes	f	1
2688	29	Question... :: http://forums.dctp.ws/index.php?topic=996.0	conanquotes	f	1
2689	29	Unofficial Torrent release of volumes 28-current (Jan 8th 2009) :: http://forums.dctp.ws/index.php?topic=997.0	conanquotes	f	1
2690	29	678?? :: http://forums.dctp.ws/index.php?topic=998.0	conanquotes	f	1
2691	29	what is the difference of being a smut and being a slut? :: http://forums.dctp.ws/index.php?topic=999.0	conanquotes	f	1
2692	29	Deceased BO members coming back into the story :: http://forums.dctp.ws/index.php?topic=1000.0	conanquotes	f	1
2693	29	Detective Conan: Conan vs Kid - SHARK & JEWEL - (movie) :: http://forums.dctp.ws/index.php?topic=1001.0	conanquotes	f	1
2694	29	Who is the other woman?  :: http://forums.dctp.ws/index.php?topic=1002.0	conanquotes	f	1
2695	29	Isn't the Antidote a huge risk? + Possibilities of defeating the BO [poll] :: http://forums.dctp.ws/index.php?topic=1003.0	conanquotes	f	1
2696	29	Characters that should return :: http://forums.dctp.ws/index.php?topic=1004.0	conanquotes	f	1
2697	29	Wich BO-Member should die next? :: http://forums.dctp.ws/index.php?topic=1005.0	conanquotes	f	1
2698	29	[DCTP] Detective Conan Episode 520 (Version 2 Released!) :: http://forums.dctp.ws/index.php?topic=1006.0	conanquotes	f	1
2699	29	Winter :: http://forums.dctp.ws/index.php?topic=1007.0	conanquotes	f	1
2700	29	Lateral Thinking II :: http://forums.dctp.ws/index.php?topic=1008.0	conanquotes	f	1
2701	29	Haibara's Fate? (poll) :: http://forums.dctp.ws/index.php?topic=1009.0	conanquotes	f	1
2702	29	can someone please explain.... file 677 :: http://forums.dctp.ws/index.php?topic=1010.0	conanquotes	f	1
2703	29	Beneath the Surface :: http://forums.dctp.ws/index.php?topic=1011.0	conanquotes	f	1
2704	29	File 678 - So, did Akai... :: http://forums.dctp.ws/index.php?topic=1012.0	conanquotes	f	1
2705	29	DC Game for Wii Released in European Languages! :: http://forums.dctp.ws/index.php?topic=1013.0	conanquotes	f	1
2706	29	Detective Conan File 678 - RAW :: http://forums.dctp.ws/index.php?topic=1014.0	conanquotes	f	1
2707	29	Detective Conan File 678 RAW :: http://forums.dctp.ws/index.php?topic=1015.0	conanquotes	f	1
2708	29	Will Conan or Haibara meet Akai in File 679 :: http://forums.dctp.ws/index.php?topic=1016.0	conanquotes	f	1
2709	29	[DCTP] Detective Conan File 678 :: http://forums.dctp.ws/index.php?topic=1017.0	conanquotes	f	1
2710	29	The first Ova of (aoyama's short stories) very HQ (DVD RIP) :: http://forums.dctp.ws/index.php?topic=1019.0	conanquotes	f	1
2711	29	Episode 521 Preview (Video) :: http://forums.dctp.ws/index.php?topic=1020.0	conanquotes	f	1
2712	29	Jing > Funimation :: http://forums.dctp.ws/index.php?topic=1021.0	conanquotes	f	1
2713	29	New Opening and Ending :: http://forums.dctp.ws/index.php?topic=1022.0	conanquotes	f	1
2714	29	Movie 3 - Magician of the Century  :: http://forums.dctp.ws/index.php?topic=1023.0	conanquotes	f	1
2715	29	please DCTP... :: http://forums.dctp.ws/index.php?topic=1025.0	conanquotes	f	1
2716	29	Detective Conan File 679 RAW :: http://forums.dctp.ws/index.php?topic=1026.0	conanquotes	f	1
2717	29	File 679 SPOILER :: http://forums.dctp.ws/index.php?topic=1027.0	conanquotes	f	1
2718	29	Translating DC Games? (Conan Vs. Kindaichi on p2!) :: http://forums.dctp.ws/index.php?topic=1028.0	conanquotes	f	1
2719	29	Gosho talking to us? :: http://forums.dctp.ws/index.php?topic=1030.0	conanquotes	f	1
2720	29	Haibara's English voice? :: http://forums.dctp.ws/index.php?topic=1031.0	conanquotes	f	1
2721	29	[DCTP] Detective Conan File 679 :: http://forums.dctp.ws/index.php?topic=1032.0	conanquotes	f	1
2722	29	Jd- says DCTP is "kinda" like a circus so.... :: http://forums.dctp.ws/index.php?topic=1033.0	conanquotes	f	1
2723	29	Madi HotPot! :: http://forums.dctp.ws/index.php?topic=1034.0	conanquotes	f	1
2724	29	Where are you from? :D :: http://forums.dctp.ws/index.php?topic=1035.0	conanquotes	f	1
2725	29	Case Closed Volume 27 (English Release) :: http://forums.dctp.ws/index.php?topic=1036.0	conanquotes	f	1
2726	29	Lupin vs. Conan TV Special Announced! :: http://forums.dctp.ws/index.php?topic=1037.0	conanquotes	f	1
2727	29	Detective Conan the anime's sound :: http://forums.dctp.ws/index.php?topic=1038.0	conanquotes	f	1
2728	29	searching for other mystery series :: http://forums.dctp.ws/index.php?topic=1039.0	conanquotes	f	1
2729	29	Information on The Raven Chaser Preview #2? :: http://forums.dctp.ws/index.php?topic=1040.0	conanquotes	f	1
2730	29	I have a radical idea about detective conan :: http://forums.dctp.ws/index.php?topic=1041.0	conanquotes	f	1
2731	29	Fanfic3Fanart-A Trip in Osaka :: http://forums.dctp.ws/index.php?topic=1042.0	conanquotes	f	1
2732	29	is anyone happy about funi relasing season 4 box set (80-105) of detective conan :: http://forums.dctp.ws/index.php?topic=1043.0	conanquotes	f	1
2733	29	The greatest mystery anime of all time :: http://forums.dctp.ws/index.php?topic=1044.0	conanquotes	f	1
2734	29	DCTV new video sub catagory  :: http://forums.dctp.ws/index.php?topic=1045.0	conanquotes	f	1
2735	29	DCTP raws :: http://forums.dctp.ws/index.php?topic=1046.0	conanquotes	f	1
2736	29	What does 9/8c mean? :: http://forums.dctp.ws/index.php?topic=1047.0	conanquotes	f	1
2737	29	DCTP Direct Download Question  :: http://forums.dctp.ws/index.php?topic=1048.0	conanquotes	f	1
2738	29	Tez's plushies :: http://forums.dctp.ws/index.php?topic=1049.0	conanquotes	f	1
2739	29	mIRC connect and receive problem :: http://forums.dctp.ws/index.php?topic=1050.0	conanquotes	f	1
2740	29	File 680 (spoiler) :: http://forums.dctp.ws/index.php?topic=1051.0	conanquotes	f	1
2741	29	Episode 522 - Screencaptures  :: http://forums.dctp.ws/index.php?topic=1054.0	conanquotes	f	1
2742	29	We should.... :: http://forums.dctp.ws/index.php?topic=1055.0	conanquotes	f	1
2743	29	Detective Conan File 680 RAW :: http://forums.dctp.ws/index.php?topic=1056.0	conanquotes	f	1
2744	29	Mystery Novels :: http://forums.dctp.ws/index.php?topic=1057.0	conanquotes	f	1
2745	29	Original Japanese scans? :: http://forums.dctp.ws/index.php?topic=1058.0	conanquotes	f	1
2746	29	[DCTP] Detective Conan Episode 521 (Finally Released!) :: http://forums.dctp.ws/index.php?topic=1059.0	conanquotes	f	1
2747	29	New theory!! Bourbon identity  :: http://forums.dctp.ws/index.php?topic=1060.0	conanquotes	f	1
2748	29	Fullscreen/Widescreen Episodes :: http://forums.dctp.ws/index.php?topic=1062.0	conanquotes	f	1
2749	29	Hello. New person here. :: http://forums.dctp.ws/index.php?topic=1063.0	conanquotes	f	1
2750	29	[DCTP] New Forum Design! :: http://forums.dctp.ws/index.php?topic=1064.0	conanquotes	f	1
2751	29	Episode 405 :: http://forums.dctp.ws/index.php?topic=1065.0	conanquotes	f	1
2752	29	Persona 4 :: http://forums.dctp.ws/index.php?topic=1066.0	conanquotes	f	1
2753	29	LUPIN VS CONAN WAGER TIME: WHO WILL BE VICTORIOUS :: http://forums.dctp.ws/index.php?topic=1068.0	conanquotes	f	1
2754	29	Another Big Release Day: This Saturday! :: http://forums.dctp.ws/index.php?topic=1069.0	conanquotes	f	1
2755	29	Volume 24...plothole? :: http://forums.dctp.ws/index.php?topic=1070.0	conanquotes	f	1
2756	29	Ghost  in the Shell :: http://forums.dctp.ws/index.php?topic=1071.0	conanquotes	f	1
2757	29	Full Score of Fear Ranked number 12 in the 2008 Box Office in Japan :: http://forums.dctp.ws/index.php?topic=1072.0	conanquotes	f	1
2758	29	batch torrent requests for specials and Arcs :: http://forums.dctp.ws/index.php?topic=1073.0	conanquotes	f	1
2759	29	Need help finding an OST :: http://forums.dctp.ws/index.php?topic=1074.0	conanquotes	f	1
2760	29	Live actions :: http://forums.dctp.ws/index.php?topic=1075.0	conanquotes	f	1
2761	29	Fanfic :: http://forums.dctp.ws/index.php?topic=1076.0	conanquotes	f	1
2762	29	[DCTP] Detective Conan File 680 :: http://forums.dctp.ws/index.php?topic=1077.0	conanquotes	f	1
2452	29	The Scar :: http://forums.dctp.ws/index.php?topic=722.0	conanquotes	f	2
2763	29	ai's fear :: http://forums.dctp.ws/index.php?topic=1078.0	conanquotes	f	1
2764	29	Superbowl :: http://forums.dctp.ws/index.php?topic=1079.0	conanquotes	f	1
2765	29	If I were Shinichi, I would tell Ran the truth now. :: http://forums.dctp.ws/index.php?topic=1080.0	conanquotes	f	1
2766	29	BZ? :: http://forums.dctp.ws/index.php?topic=1081.0	conanquotes	f	1
2767	29	Black Lagoon :: http://forums.dctp.ws/index.php?topic=1082.0	conanquotes	f	1
2768	29	My fanfictions... :: http://forums.dctp.ws/index.php?topic=1083.0	conanquotes	f	1
2769	29	A case (Number II) :: http://forums.dctp.ws/index.php?topic=1084.0	conanquotes	f	1
2770	29	Shinjitsu .:Detective Conan Fic:. :: http://forums.dctp.ws/index.php?topic=1085.0	conanquotes	f	1
2771	29	Norowarete Iru Negai: A Wish Gone Wrong .:Detective Conan Fic:. :: http://forums.dctp.ws/index.php?topic=1086.0	conanquotes	f	1
2772	29	The Kaitou and the Tantei .:Detective Conan Fic:. :: http://forums.dctp.ws/index.php?topic=1087.0	conanquotes	f	1
2773	29	Crushed (for kirite) .:Detective Conan Songfic:. :: http://forums.dctp.ws/index.php?topic=1088.0	conanquotes	f	1
2774	29	Over It .:Detective Conan songfic:. :: http://forums.dctp.ws/index.php?topic=1089.0	conanquotes	f	1
2775	29	Detective Conan - File 681 RAW :: http://forums.dctp.ws/index.php?topic=1090.0	conanquotes	f	1
2776	29	Literature Detectives Poll :: http://forums.dctp.ws/index.php?topic=1092.0	conanquotes	f	1
2778	29	Detective Conan File 681 RAW :: http://forums.dctp.ws/index.php?topic=1094.0	conanquotes	f	1
2779	29	The Silhouette Contest (unofficial thread :) ) :: http://forums.dctp.ws/index.php?topic=1095.0	conanquotes	f	1
2780	29	New theory about their goal(BO related) :: http://forums.dctp.ws/index.php?topic=1097.0	conanquotes	f	1
2781	29	I saw something funny today... :: http://forums.dctp.ws/index.php?topic=1098.0	conanquotes	f	1
2782	29	Role Playing :: http://forums.dctp.ws/index.php?topic=1099.0	conanquotes	f	1
2783	29	I know this is the dctp forum but, :: http://forums.dctp.ws/index.php?topic=1100.0	conanquotes	f	1
2784	29	DCTP "Create-A-Shirt" Project - Design Step :: http://forums.dctp.ws/index.php?topic=1101.0	conanquotes	f	1
2785	29	Conan Best Collection DVD Set Question :: http://forums.dctp.ws/index.php?topic=1103.0	conanquotes	f	1
2786	29	Detective Conan Sheet Music :: http://forums.dctp.ws/index.php?topic=1104.0	conanquotes	f	1
2787	29	503 errors :: http://forums.dctp.ws/index.php?topic=1105.0	conanquotes	f	1
2788	29	my bad if this sounds stupid :: http://forums.dctp.ws/index.php?topic=1106.0	conanquotes	f	1
2789	29	Conan's Parode :: http://forums.dctp.ws/index.php?topic=1107.0	conanquotes	f	1
2790	29	Can anyone reconmend some recent conan eps? :: http://forums.dctp.ws/index.php?topic=1108.0	conanquotes	f	1
2791	29	Case Closed Season 4 :: http://forums.dctp.ws/index.php?topic=1109.0	conanquotes	f	1
2792	29	In Defense Of Evil:  Why Good Is Bad And Bad Is Good :: http://forums.dctp.ws/index.php?topic=1110.0	conanquotes	f	1
2793	29	Can you identify with... :: http://forums.dctp.ws/index.php?topic=1111.0	conanquotes	f	1
2794	29	Randomness :: http://forums.dctp.ws/index.php?topic=1112.0	conanquotes	f	1
2795	29	[DCTP] Detective Conan File 681 :: http://forums.dctp.ws/index.php?topic=1113.0	conanquotes	f	1
2796	29	DC New Live Action Movie in US? :: http://forums.dctp.ws/index.php?topic=1114.0	conanquotes	f	1
2797	29	"Evil is your sour flavor."  ~Marilyn Manson :: http://forums.dctp.ws/index.php?topic=1115.0	conanquotes	f	1
2798	29	Help with the game of Conan and Kindachi :: http://forums.dctp.ws/index.php?topic=1117.0	conanquotes	f	1
2799	29	File 681 (Spoiler) :: http://forums.dctp.ws/index.php?topic=1118.0	conanquotes	f	1
2777	29	How would Gin die? :: http://forums.dctp.ws/index.php?topic=1093.0	conanquotes	f	2
2800	29	Look at the top of the main page where it says latest releases... :: http://forums.dctp.ws/index.php?topic=1119.0	conanquotes	f	1
2801	29	Sci-Fi now SF? :: http://forums.dctp.ws/index.php?topic=1120.0	conanquotes	f	1
2802	29	MY Fan Art :: http://forums.dctp.ws/index.php?topic=1121.0	conanquotes	f	1
2803	29	Random Conan parody opening/endings :: http://forums.dctp.ws/index.php?topic=1122.0	conanquotes	f	1
2804	29	Archive Page Problem :: http://forums.dctp.ws/index.php?topic=1123.0	conanquotes	f	1
2805	29	Little riddles :: http://forums.dctp.ws/index.php?topic=1125.0	conanquotes	f	1
2806	29	Who is angrier? :: http://forums.dctp.ws/index.php?topic=1126.0	conanquotes	f	1
2807	29	Riddles riddles XD :: http://forums.dctp.ws/index.php?topic=1127.0	conanquotes	f	1
2808	29	MO 13 Drawing :: http://forums.dctp.ws/index.php?topic=1128.0	conanquotes	f	1
2809	29	what do people think about kaitkou kids english VA :: http://forums.dctp.ws/index.php?topic=1130.0	conanquotes	f	1
2810	29	Questions :: http://forums.dctp.ws/index.php?topic=1131.0	conanquotes	f	1
2811	29	Revive by Kuraki Mai? anyone?... :: http://forums.dctp.ws/index.php?topic=1132.0	conanquotes	f	1
2812	29	Funny Thing Happened Today. :: http://forums.dctp.ws/index.php?topic=1134.0	conanquotes	f	1
2813	29	[GinRei] Detective Conan Episodes 291-293 (Version 2) :: http://forums.dctp.ws/index.php?topic=1135.0	conanquotes	f	1
2814	29	Bonjour from the Pill world!  :: http://forums.dctp.ws/index.php?topic=1136.0	conanquotes	f	1
2815	29	Best story ARC.... :: http://forums.dctp.ws/index.php?topic=1137.0	conanquotes	f	1
2816	29	Archery video :: http://forums.dctp.ws/index.php?topic=1138.0	conanquotes	f	1
2817	29	Fakesagan with Oshunavani vs. CapnOAwesome  :: http://forums.dctp.ws/index.php?topic=1139.0	conanquotes	f	1
2818	29	APTX theory :: http://forums.dctp.ws/index.php?topic=1140.0	conanquotes	f	1
2819	29	Favourite Murder Trick :: http://forums.dctp.ws/index.php?topic=1141.0	conanquotes	f	1
2820	29	Problems with "Official Episode Requests Thread" :: http://forums.dctp.ws/index.php?topic=1142.0	conanquotes	f	1
2821	29	Vermouth :: http://forums.dctp.ws/index.php?topic=1143.0	conanquotes	f	1
2822	29	KID drawing :: http://forums.dctp.ws/index.php?topic=1144.0	conanquotes	f	1
2823	29	Favourtite sleeping detective :: http://forums.dctp.ws/index.php?topic=1145.0	conanquotes	f	1
2824	29	Shinran's Manga Spoofs :: http://forums.dctp.ws/index.php?topic=1146.0	conanquotes	f	1
2825	29	Movie 13 question :: http://forums.dctp.ws/index.php?topic=1147.0	conanquotes	f	1
2826	29	The Valentine's Big Release Day (2/14/09) @ 5:00 PM EST! :: http://forums.dctp.ws/index.php?topic=1148.0	conanquotes	f	1
2827	29	My Fan Art website :: http://forums.dctp.ws/index.php?topic=1149.0	conanquotes	f	1
2828	29	DDL error :: http://forums.dctp.ws/index.php?topic=1150.0	conanquotes	f	1
2829	29	[DCTP] Detective Conan Episodes 343-344 :: http://forums.dctp.ws/index.php?topic=1151.0	conanquotes	f	1
2830	29	Clannad TV ~ After Story~ Episode 18 :: http://forums.dctp.ws/index.php?topic=1152.0	conanquotes	f	1
2831	29	The voice change :: http://forums.dctp.ws/index.php?topic=1153.0	conanquotes	f	1
2832	29	help me find a song? :: http://forums.dctp.ws/index.php?topic=1154.0	conanquotes	f	1
2833	29	Lateral Thinking III :: http://forums.dctp.ws/index.php?topic=1155.0	conanquotes	f	1
2834	29	Lateral thinking IV: Two strings :: http://forums.dctp.ws/index.php?topic=1156.0	conanquotes	f	1
2835	29	Detective Conan 1-202 [DDL] :: http://forums.dctp.ws/index.php?topic=1157.0	conanquotes	f	1
2836	29	Black--Conan songfic :: http://forums.dctp.ws/index.php?topic=1158.0	conanquotes	f	1
2837	29	Blue Spark of Hatred? :: http://forums.dctp.ws/index.php?topic=1159.0	conanquotes	f	1
2838	29	Sakura-con :: http://forums.dctp.ws/index.php?topic=1160.0	conanquotes	f	1
2839	29	Tez's fanart :: http://forums.dctp.ws/index.php?topic=1161.0	conanquotes	f	1
2840	29	Maiyeng B'z fancomics :: http://forums.dctp.ws/index.php?topic=1162.0	conanquotes	f	1
2841	29	You're My Greatest Mystery [ShinxRan Song] :: http://forums.dctp.ws/index.php?topic=1163.0	conanquotes	f	1
2842	29	The One Truth [ShinxRan Poem] :: http://forums.dctp.ws/index.php?topic=1164.0	conanquotes	f	1
2843	29	goroawase game :: http://forums.dctp.ws/index.php?topic=1165.0	conanquotes	f	1
2844	29	Kudo Shinichi eps? :: http://forums.dctp.ws/index.php?topic=1166.0	conanquotes	f	1
2845	29	LAST request before I can open Kudo's Bookshelf :: http://forums.dctp.ws/index.php?topic=1167.0	conanquotes	f	1
2846	29	Won't You Believe in Me? [Poem in Conan's POV] :: http://forums.dctp.ws/index.php?topic=1168.0	conanquotes	f	1
2847	29	Key Holes/End of Volume Files :: http://forums.dctp.ws/index.php?topic=1169.0	conanquotes	f	1
2848	29	Lateral Thinking V :: http://forums.dctp.ws/index.php?topic=1170.0	conanquotes	f	1
2849	29	detective conan on mythbusters :: http://forums.dctp.ws/index.php?topic=1171.0	conanquotes	f	1
2850	29	DC Manga/Anime Case Help? :: http://forums.dctp.ws/index.php?topic=1172.0	conanquotes	f	1
2851	29	movie 13 the raven chaser :: http://forums.dctp.ws/index.php?topic=1173.0	conanquotes	f	1
2852	29	Beck : MCS :: http://forums.dctp.ws/index.php?topic=1174.0	conanquotes	f	1
2853	29	who bought funis relase of case closed season 4 :: http://forums.dctp.ws/index.php?topic=1175.0	conanquotes	f	1
2854	29	Someone's Signature :: http://forums.dctp.ws/index.php?topic=1176.0	conanquotes	f	1
2855	29	Favorite Case :: http://forums.dctp.ws/index.php?topic=1177.0	conanquotes	f	1
2856	29	episode help :: http://forums.dctp.ws/index.php?topic=1178.0	conanquotes	f	1
2857	29	i don't understand... :: http://forums.dctp.ws/index.php?topic=1179.0	conanquotes	f	1
2858	29	Case Number III :: http://forums.dctp.ws/index.php?topic=1180.0	conanquotes	f	1
2859	29	Lateral Thinking VI and VII (very difficult) :: http://forums.dctp.ws/index.php?topic=1181.0	conanquotes	f	1
2860	29	DCTV awesome chat moments :: http://forums.dctp.ws/index.php?topic=1182.0	conanquotes	f	1
2861	29	KNOW EVERYTHING ABOUT B'Z RIGHT HERE :: http://forums.dctp.ws/index.php?topic=1183.0	conanquotes	f	1
2862	29	Little Riddles II :: http://forums.dctp.ws/index.php?topic=1184.0	conanquotes	f	1
2863	29	Unbelieveable Murder in Argentina :: http://forums.dctp.ws/index.php?topic=1185.0	conanquotes	f	1
2864	29	movie 8 :: http://forums.dctp.ws/index.php?topic=1186.0	conanquotes	f	1
2865	29	Thanks, you guys! :: http://forums.dctp.ws/index.php?topic=1187.0	conanquotes	f	1
2866	29	Conan and the Case of the Stolen Adult DVDs :: http://forums.dctp.ws/index.php?topic=1188.0	conanquotes	f	1
2867	29	Stupiesd Detective-Boy :: http://forums.dctp.ws/index.php?topic=1189.0	conanquotes	f	1
2868	29	Got a question and trying not to sound impatient at the same time... :: http://forums.dctp.ws/index.php?topic=1191.0	conanquotes	f	1
2869	29	Kimi ga Ireba in English :: http://forums.dctp.ws/index.php?topic=1192.0	conanquotes	f	1
2870	29	Detective Conan File 682 RAW :: http://forums.dctp.ws/index.php?topic=1193.0	conanquotes	f	1
2871	29	Tricky Lateral Thinking I :: http://forums.dctp.ws/index.php?topic=1194.0	conanquotes	f	1
2872	29	Lateral Thinking VIII and IX and more ;-) :: http://forums.dctp.ws/index.php?topic=1195.0	conanquotes	f	1
2873	29	Case Number IV :: http://forums.dctp.ws/index.php?topic=1196.0	conanquotes	f	1
2874	29	Help for Arsene Lupin quotes! :: http://forums.dctp.ws/index.php?topic=1197.0	conanquotes	f	1
2875	29	Harley Hartwell :: http://forums.dctp.ws/index.php?topic=1198.0	conanquotes	f	1
2876	29	Chat Names for DC Characters! :D :: http://forums.dctp.ws/index.php?topic=1199.0	conanquotes	f	1
2877	29	JetLev :: http://forums.dctp.ws/index.php?topic=1200.0	conanquotes	f	1
2878	29	MISSING CHILD ALERT :: http://forums.dctp.ws/index.php?topic=1201.0	conanquotes	f	1
2879	29	Muahahah! I made more art! :: http://forums.dctp.ws/index.php?topic=1203.0	conanquotes	f	1
2880	29	File 682 (spoiler) :: http://forums.dctp.ws/index.php?topic=1204.0	conanquotes	f	1
2881	29	[DCTP] Detective Conan File 682 :: http://forums.dctp.ws/index.php?topic=1205.0	conanquotes	f	1
2882	29	Case V: Conan and the nine explosives :: http://forums.dctp.ws/index.php?topic=1206.0	conanquotes	f	1
2883	29	story arc page help :: http://forums.dctp.ws/index.php?topic=1208.0	conanquotes	f	1
2884	29	I was wondering... :: http://forums.dctp.ws/index.php?topic=1209.0	conanquotes	f	1
2885	29	Lateral Thinking X :: http://forums.dctp.ws/index.php?topic=1210.0	conanquotes	f	1
2886	29	Detective Conan File 683 RAW :: http://forums.dctp.ws/index.php?topic=1212.0	conanquotes	f	1
2887	29	File 683 (spoiler) :: http://forums.dctp.ws/index.php?topic=1213.0	conanquotes	f	1
2888	29	Cryptogram Puzzles :: http://forums.dctp.ws/index.php?topic=1214.0	conanquotes	f	1
2889	29	Hetalia :: http://forums.dctp.ws/index.php?topic=1217.0	conanquotes	f	1
2890	29	Kudo's Bookshelf is Now Open! :: http://forums.dctp.ws/index.php?topic=1218.0	conanquotes	f	1
2891	29	[DCTP] Detective Conan Episodes 522 & 523 (Released!) :: http://forums.dctp.ws/index.php?topic=1219.0	conanquotes	f	1
2892	29	New BO Member: "Irish" :: http://forums.dctp.ws/index.php?topic=1220.0	conanquotes	f	1
2893	29	Movie 13 O.S.T :: http://forums.dctp.ws/index.php?topic=1221.0	conanquotes	f	1
2894	29	BO Boss Theory :: http://forums.dctp.ws/index.php?topic=1223.0	conanquotes	f	1
2895	29	Best/Favourite Movie, OVA and Episode? :: http://forums.dctp.ws/index.php?topic=1224.0	conanquotes	f	1
2896	29	i just noticed something... :: http://forums.dctp.ws/index.php?topic=1225.0	conanquotes	f	1
2897	29	The Question and Answer - Help - Topic! :: http://forums.dctp.ws/index.php?topic=1226.0	conanquotes	f	1
2898	29	Lateral Thinking XI :: http://forums.dctp.ws/index.php?topic=1227.0	conanquotes	f	1
2899	29	Should I buy it? :: http://forums.dctp.ws/index.php?topic=1228.0	conanquotes	f	1
2900	29	The Hitman. :: http://forums.dctp.ws/index.php?topic=1229.0	conanquotes	f	1
2901	29	[DCTP] Detective Conan File 683 :: http://forums.dctp.ws/index.php?topic=1230.0	conanquotes	f	1
2902	29	Detective Conan File 684 RAW :: http://forums.dctp.ws/index.php?topic=1231.0	conanquotes	f	1
2903	29	LINK to the NEW DC OPENING 25 [[[ REVIVE]]] Mai Kuraki :: http://forums.dctp.ws/index.php?topic=1232.0	conanquotes	f	1
2904	29	::::::Detective of the South Koshimizu Natsuki :::::::::: :: http://forums.dctp.ws/index.php?topic=1233.0	conanquotes	f	1
2905	29	Detective Conan Movie 13 The Raven Chaser Official Final Trailer  :: http://forums.dctp.ws/index.php?topic=1234.0	conanquotes	f	1
2906	29	Tez's Plushies 2 :: http://forums.dctp.ws/index.php?topic=1236.0	conanquotes	f	1
2907	29	Which opening is that?.... :: http://forums.dctp.ws/index.php?topic=1237.0	conanquotes	f	1
2908	29	Does anyone have the file for this song? :: http://forums.dctp.ws/index.php?topic=1238.0	conanquotes	f	1
2909	29	Fav couple for shinichi or Conan? :: http://forums.dctp.ws/index.php?topic=1239.0	conanquotes	f	1
2910	29	Your Case closed manga collection! :: http://forums.dctp.ws/index.php?topic=1240.0	conanquotes	f	1
2911	29	The story of movie 13. :: http://forums.dctp.ws/index.php?topic=1241.0	conanquotes	f	1
2912	29	Ran + Eisuke :: http://forums.dctp.ws/index.php?topic=1243.0	conanquotes	f	1
2913	29	[DCTP] Detective Conan File 684 :: http://forums.dctp.ws/index.php?topic=1245.0	conanquotes	f	1
2914	29	Files 683 and 684 (Spoilers) :: http://forums.dctp.ws/index.php?topic=1246.0	conanquotes	f	1
2915	29	HD episodes :: http://forums.dctp.ws/index.php?topic=1247.0	conanquotes	f	1
2917	29	Fav DC song ^o^ :: http://forums.dctp.ws/index.php?topic=1249.0	conanquotes	f	1
2918	29	Lateral Thinking XII :: http://forums.dctp.ws/index.php?topic=1250.0	conanquotes	f	1
2919	29	Mai Kuraki "Revive"--25OP PV Preview :: http://forums.dctp.ws/index.php?topic=1251.0	conanquotes	f	1
2920	29	Mai Kuraki "Revive"--25OP Full Version :: http://forums.dctp.ws/index.php?topic=1252.0	conanquotes	f	1
2921	29	A malicious issue! :: http://forums.dctp.ws/index.php?topic=1253.0	conanquotes	f	1
2922	29	DC Magic File 3 :: http://forums.dctp.ws/index.php?topic=1254.0	conanquotes	f	1
2923	29	Detective Conan File 685 RAW :: http://forums.dctp.ws/index.php?topic=1255.0	conanquotes	f	1
2924	29	Lsteral Thinking XIII :: http://forums.dctp.ws/index.php?topic=1256.0	conanquotes	f	1
2925	29	Good music related anime :: http://forums.dctp.ws/index.php?topic=1257.0	conanquotes	f	1
2926	29	Will DCTP relase a fan sub of the digitally remastered episode one :: http://forums.dctp.ws/index.php?topic=1258.0	conanquotes	f	1
2927	29	OP26 BREAKERZ - Everlasting Luv :: http://forums.dctp.ws/index.php?topic=1259.0	conanquotes	f	1
2928	29	A Problem :: http://forums.dctp.ws/index.php?topic=1261.0	conanquotes	f	1
2929	29	MISSING CLASSMATE!!!!!! :o :: http://forums.dctp.ws/index.php?topic=1263.0	conanquotes	f	1
2930	29	Test of enbeded Flash :: http://forums.dctp.ws/index.php?topic=1265.0	conanquotes	f	1
2931	29	Heiji Hattori Pixel Art :: http://forums.dctp.ws/index.php?topic=1266.0	conanquotes	f	1
2932	29	[Kaizou] Lupin the Third Y Chapters 003 & 004 :: http://forums.dctp.ws/index.php?topic=1267.0	conanquotes	f	1
2933	29	should i buy the case closed DVD? :: http://forums.dctp.ws/index.php?topic=1268.0	conanquotes	f	1
2934	29	NAIFU's album is out (it's good!) :: http://forums.dctp.ws/index.php?topic=1270.0	conanquotes	f	1
2935	29	Lupin III Starter Kit (DDL links) :: http://forums.dctp.ws/index.php?topic=1271.0	conanquotes	f	1
2936	29	DCTP TV LOCKED :: http://forums.dctp.ws/index.php?topic=1272.0	conanquotes	f	1
2937	29	the Conan dub minor character names are AWESOME :: http://forums.dctp.ws/index.php?topic=1273.0	conanquotes	f	1
2938	29	[DCTP] Detective Conan File 685 :: http://forums.dctp.ws/index.php?topic=1274.0	conanquotes	f	1
2939	29	Revive Promotion Video Released!!! :: http://forums.dctp.ws/index.php?topic=1275.0	conanquotes	f	1
2940	29	Which Episode? :: http://forums.dctp.ws/index.php?topic=1276.0	conanquotes	f	1
2941	29	A Shinichi/Ran AMV :: http://forums.dctp.ws/index.php?topic=1277.0	conanquotes	f	1
2942	29	Need help colouring this Ai fan-art :: http://forums.dctp.ws/index.php?topic=1278.0	conanquotes	f	1
2943	29	The Grammar Topic :: http://forums.dctp.ws/index.php?topic=1279.0	conanquotes	f	1
2944	29	Puzzle Full Live Version(Link Changed - Mediafire) :: http://forums.dctp.ws/index.php?topic=1280.0	conanquotes	f	1
2945	29	RI makes  of my old drawings and few new ones  :: http://forums.dctp.ws/index.php?topic=1281.0	conanquotes	f	1
2946	29	If you know Dutch... :: http://forums.dctp.ws/index.php?topic=1282.0	conanquotes	f	1
2947	29	Who do you like better? :: http://forums.dctp.ws/index.php?topic=1283.0	conanquotes	f	1
2948	29	Detective Conan 686 RAW :: http://forums.dctp.ws/index.php?topic=1284.0	conanquotes	f	1
2949	29	do you think file 687 is BO related? :: http://forums.dctp.ws/index.php?topic=1285.0	conanquotes	f	1
2950	29	The Raven Chaser.... Are Shinichi and Ran going to die?!?!?! COMFORT TOPIC!! :: http://forums.dctp.ws/index.php?topic=1286.0	conanquotes	f	1
2951	29	Kuraki Mai - Puzzle (PV) :: http://forums.dctp.ws/index.php?topic=1288.0	conanquotes	f	1
2952	29	Detective Conan Episodes 530&531 is going to be released in Japan Soon :D :: http://forums.dctp.ws/index.php?topic=1289.0	conanquotes	f	1
2953	29	Episode 1 ?! AGAIN? :o :: http://forums.dctp.ws/index.php?topic=1290.0	conanquotes	f	1
2954	29	Too much kaitou if... - Sentences xD :: http://forums.dctp.ws/index.php?topic=1291.0	conanquotes	f	1
2955	29	Miwako's last name :: http://forums.dctp.ws/index.php?topic=1292.0	conanquotes	f	1
2956	29	Best/Worst dub voices? :: http://forums.dctp.ws/index.php?topic=1293.0	conanquotes	f	1
2957	29	Lateral Thinking XIV :: http://forums.dctp.ws/index.php?topic=1294.0	conanquotes	f	1
2958	29	Conan/Shinichi's Relationship Poll: Your favorite vs. the final pairing :: http://forums.dctp.ws/index.php?topic=1295.0	conanquotes	f	1
2959	29	Shinichi is dancing here... :: http://forums.dctp.ws/index.php?topic=1296.0	conanquotes	f	1
2960	29	Organizing Love Topics :: http://forums.dctp.ws/index.php?topic=1297.0	conanquotes	f	1
2961	29	hacker warning :: http://forums.dctp.ws/index.php?topic=1298.0	conanquotes	f	1
2962	29	Video Of Conan Booth At the Tokyo Anime Expo :: http://forums.dctp.ws/index.php?topic=1299.0	conanquotes	f	1
2963	29	Do you think they are overusing "Mysterious" characters? :: http://forums.dctp.ws/index.php?topic=1300.0	conanquotes	f	1
2964	29	Detective Conan File 687 RAW :: http://forums.dctp.ws/index.php?topic=1302.0	conanquotes	f	1
2965	29	[DCTP] Detective Conan File 686 :: http://forums.dctp.ws/index.php?topic=1303.0	conanquotes	f	1
2966	29	April Fools :: http://forums.dctp.ws/index.php?topic=1304.0	conanquotes	f	1
2967	29	[Frostii & DCTP] Lupin the 3rd vs. Detective Conan (Coming *EXTREMELY* Soon!) :: http://forums.dctp.ws/index.php?topic=1306.0	conanquotes	f	1
2968	29	Lateral Thinking XV :: http://forums.dctp.ws/index.php?topic=1307.0	conanquotes	f	1
2969	29	Need help on a certain background music :: http://forums.dctp.ws/index.php?topic=1309.0	conanquotes	f	1
2970	29	Favorite anime original episode? :: http://forums.dctp.ws/index.php?topic=1310.0	conanquotes	f	1
2971	29	Opening 26 & Ending 33 :: http://forums.dctp.ws/index.php?topic=1312.0	conanquotes	f	1
2972	29	Kaitou Kid can help Conan :: http://forums.dctp.ws/index.php?topic=1315.0	conanquotes	f	1
2973	29	Who's the smartest? :: http://forums.dctp.ws/index.php?topic=1316.0	conanquotes	f	1
2974	29	Higurashi no Naku Koro ni + Kai + Rei + Umineko {Manga/Anime} :: http://forums.dctp.ws/index.php?topic=1317.0	conanquotes	f	1
2975	29	Req: Re-seed 484/485 SD :: http://forums.dctp.ws/index.php?topic=1318.0	conanquotes	f	1
2976	29	Favorite opening/ending animation? :: http://forums.dctp.ws/index.php?topic=1319.0	conanquotes	f	1
2977	29	Lateral Thinking XVI :: http://forums.dctp.ws/index.php?topic=1320.0	conanquotes	f	1
2978	29	[DCTP] Detective Conan File 687 :: http://forums.dctp.ws/index.php?topic=1321.0	conanquotes	f	1
2979	29	Kaitou Kid revealed :: http://forums.dctp.ws/index.php?topic=1322.0	conanquotes	f	1
2980	29	Detective Conan File 688 RAW :: http://forums.dctp.ws/index.php?topic=1323.0	conanquotes	f	1
2981	29	The Vermouth Proof :: http://forums.dctp.ws/index.php?topic=1324.0	conanquotes	f	1
2982	29	Lupin vs DC? :: http://forums.dctp.ws/index.php?topic=1325.0	conanquotes	f	1
2983	29	Ch. 687: Shiratori's Memories :: http://forums.dctp.ws/index.php?topic=1326.0	conanquotes	f	1
2984	29	Movie 10 :3 - if you didn't saw it, dont klick because its something special xD :: http://forums.dctp.ws/index.php?topic=1327.0	conanquotes	f	1
2985	29	Bleach: Ichigo's New Form :: http://forums.dctp.ws/index.php?topic=1328.0	conanquotes	f	1
2986	29	[GinRei] Detective Conan Episode 276 (Version 2) :: http://forums.dctp.ws/index.php?topic=1330.0	conanquotes	f	1
2987	29	Reseed Eps 510-520 please!! :: http://forums.dctp.ws/index.php?topic=1331.0	conanquotes	f	1
2988	29	[DCTP] Detective Conan Episode 526 (IT'S RELEASED!!!!!!!!!!!) :: http://forums.dctp.ws/index.php?topic=1332.0	conanquotes	f	1
2989	29	Riddle I'm stuck with for some reason :: http://forums.dctp.ws/index.php?topic=1333.0	conanquotes	f	1
2990	29	Want some carification :: http://forums.dctp.ws/index.php?topic=1334.0	conanquotes	f	1
2991	29	Episodes 524-525 (The Blue Spark of Hate) Discussion topic :: http://forums.dctp.ws/index.php?topic=1335.0	conanquotes	f	1
2992	29	Locations you'd like to see a case occur at. :: http://forums.dctp.ws/index.php?topic=1336.0	conanquotes	f	1
2993	29	Timeline Question :: http://forums.dctp.ws/index.php?topic=1337.0	conanquotes	f	1
2994	29	Biggest Revelation in Detective Conan :: http://forums.dctp.ws/index.php?topic=1339.0	conanquotes	f	1
2995	29	491-499 episodes HD :: http://forums.dctp.ws/index.php?topic=1340.0	conanquotes	f	1
2996	29	Bleach Photoshop Thread :: http://forums.dctp.ws/index.php?topic=1342.0	conanquotes	f	1
2997	29	Question about Streaming Sites :: http://forums.dctp.ws/index.php?topic=1343.0	conanquotes	f	1
2998	29	Detective Fiction :: http://forums.dctp.ws/index.php?topic=1344.0	conanquotes	f	1
2999	29	[DCTP] Detective Conan File 688 :: http://forums.dctp.ws/index.php?topic=1345.0	conanquotes	f	1
3000	29	File 688!! :: http://forums.dctp.ws/index.php?topic=1346.0	conanquotes	f	1
3001	29	Mathematical Thinking I, II, .... (previously Puzzle: Married?) :: http://forums.dctp.ws/index.php?topic=1351.0	conanquotes	f	1
3002	29	Share your unique and interesting opinion all about detective conan.. :: http://forums.dctp.ws/index.php?topic=1352.0	conanquotes	f	1
3003	29	For detective conan... :: http://forums.dctp.ws/index.php?topic=1353.0	conanquotes	f	1
3004	29	Which will be the next DCTP realease or surprise? :: http://forums.dctp.ws/index.php?topic=1356.0	conanquotes	f	1
3005	29	Dragon Ball Evolution :: http://forums.dctp.ws/index.php?topic=1357.0	conanquotes	f	1
3006	29	OVA 9 :: http://forums.dctp.ws/index.php?topic=1358.0	conanquotes	f	1
3007	29	Detective Conan File 689 RAW :: http://forums.dctp.ws/index.php?topic=1359.0	conanquotes	f	1
3008	29	DCTV Frequency issue? :: http://forums.dctp.ws/index.php?topic=1360.0	conanquotes	f	1
3009	29	Yaiba anime subbed? :: http://forums.dctp.ws/index.php?topic=1361.0	conanquotes	f	1
3010	29	Tis mah B-Day :: http://forums.dctp.ws/index.php?topic=1363.0	conanquotes	f	1
3011	29	Detective Conan fanart :: http://forums.dctp.ws/index.php?topic=1364.0	conanquotes	f	1
3012	29	689 (spoilers) :: http://forums.dctp.ws/index.php?topic=1366.0	conanquotes	f	1
3013	29	Matantei Loki Ragnarok :) :: http://forums.dctp.ws/index.php?topic=1367.0	conanquotes	f	1
3014	29	just a tiny little thought.... :: http://forums.dctp.ws/index.php?topic=1368.0	conanquotes	f	1
3015	29	My DC AMV :3 :: http://forums.dctp.ws/index.php?topic=1369.0	conanquotes	f	1
3016	29	DDL Down :: http://forums.dctp.ws/index.php?topic=1370.0	conanquotes	f	1
3017	29	Mai Kuraki - National Anthem & Opening Pitch :: http://forums.dctp.ws/index.php?topic=1371.0	conanquotes	f	1
3018	29	DC [Downloads] (Ipods/PSP: .MP4) -Movies, Episodes- {Recent:Ep 65-75} :: http://forums.dctp.ws/index.php?topic=1372.0	conanquotes	f	1
3019	29	check this out!!! :: http://forums.dctp.ws/index.php?topic=1373.0	conanquotes	f	1
3020	29	What is the only truth?? :: http://forums.dctp.ws/index.php?topic=1374.0	conanquotes	f	1
3021	29	Lupin III - Sweet Lost Night -- Released :: http://forums.dctp.ws/index.php?topic=1375.0	conanquotes	f	1
3022	29	Ayumi and Ai sleeps on Conan's shoulder? :: http://forums.dctp.ws/index.php?topic=1376.0	conanquotes	f	1
3023	29	Lupin VS DC :: http://forums.dctp.ws/index.php?topic=1377.0	conanquotes	f	1
3024	29	For those who were able to watch Raven Chaser... :: http://forums.dctp.ws/index.php?topic=1378.0	conanquotes	f	1
3025	29	M13 OST 320K/MP3 (Add Emule Links) :: http://forums.dctp.ws/index.php?topic=1379.0	conanquotes	f	1
3026	29	Not seen? Dont klick -Part from episode 425 - Wth is wrong with...? XD :: http://forums.dctp.ws/index.php?topic=1380.0	conanquotes	f	1
3027	29	Who do you prefer? :D :: http://forums.dctp.ws/index.php?topic=1381.0	conanquotes	f	1
3028	29	Movie 14 Announced?! :: http://forums.dctp.ws/index.php?topic=1382.0	conanquotes	f	1
3029	29	Movie 13 Review! [WARNING! THERE ARE SPOILERS IN THIS THREAD! ><] :: http://forums.dctp.ws/index.php?topic=1383.0	conanquotes	f	1
3030	29	Magic File 3 OVA - M13 SideStory :: http://forums.dctp.ws/index.php?topic=1384.0	conanquotes	f	1
3031	29	[DCTP] Detective Conan Episodes 346-347 (Released!) :: http://forums.dctp.ws/index.php?topic=1385.0	conanquotes	f	1
3032	29	[The World] Matlock Season 10 - Coming Soon. With Your Help. :: http://forums.dctp.ws/index.php?topic=1386.0	conanquotes	f	1
3033	29	Drawing :3 -- inspiration from Kirite xD :: http://forums.dctp.ws/index.php?topic=1387.0	conanquotes	f	1
3034	29	Ever think of a case scenario in your head? :: http://forums.dctp.ws/index.php?topic=1388.0	conanquotes	f	1
3035	29	HAPPY BIRTHDAY SHINICHI! :: http://forums.dctp.ws/index.php?topic=1389.0	conanquotes	f	1
3036	29	DETECTIVE CONAN FANMADE TRAILER :: http://forums.dctp.ws/index.php?topic=1390.0	conanquotes	f	1
3037	29	List of chapters/episodes relations? :: http://forums.dctp.ws/index.php?topic=1391.0	conanquotes	f	1
3038	29	Detective Conan Toys at McDonald's :: http://forums.dctp.ws/index.php?topic=1392.0	conanquotes	f	1
3039	29	OVA 9 Stranger in 10 Years... :: http://forums.dctp.ws/index.php?topic=1393.0	conanquotes	f	1
3040	29	Real or fake? :: http://forums.dctp.ws/index.php?topic=1394.0	conanquotes	f	1
3041	29	Yaiba scanlations :: http://forums.dctp.ws/index.php?topic=1395.0	conanquotes	f	1
3042	29	New to Detective Conan. :: http://forums.dctp.ws/index.php?topic=1396.0	conanquotes	f	1
3043	29	how is that possible??? :: http://forums.dctp.ws/index.php?topic=1397.0	conanquotes	f	1
3044	29	Detective Conan Movie 13 Raw sources? :: http://forums.dctp.ws/index.php?topic=1398.0	conanquotes	f	1
3045	29	What would it take for DC to become more popular in the U.S.? :: http://forums.dctp.ws/index.php?topic=1400.0	conanquotes	f	1
3046	29	isn't the furinkazan case part of the story arc? :: http://forums.dctp.ws/index.php?topic=1402.0	conanquotes	f	1
3047	29	movies,OVAs,episodes :: http://forums.dctp.ws/index.php?topic=1405.0	conanquotes	f	1
3048	29	Les Aventures de Meitantei Conan :: http://forums.dctp.ws/index.php?topic=1406.0	conanquotes	f	1
3049	29	your thought please!! :: http://forums.dctp.ws/index.php?topic=1407.0	conanquotes	f	1
3050	29	Lateral Thinking XVII :: http://forums.dctp.ws/index.php?topic=1410.0	conanquotes	f	1
3051	29	[DCTP] Detective Conan File 689 :: http://forums.dctp.ws/index.php?topic=1411.0	conanquotes	f	1
3052	29	Detective Conan [PSP MP4] 531 added :: http://forums.dctp.ws/index.php?topic=1412.0	conanquotes	f	1
3053	29	how does Kaitou Kid deduce the police plan on capturing him? :: http://forums.dctp.ws/index.php?topic=1413.0	conanquotes	f	1
3054	29	A Conan RP  channel on IRC. [Hope this belongs here. lol] :: http://forums.dctp.ws/index.php?topic=1414.0	conanquotes	f	1
3055	29	Avatar Problems... :: http://forums.dctp.ws/index.php?topic=1415.0	conanquotes	f	1
3056	29	I need to post this :: http://forums.dctp.ws/index.php?topic=1416.0	conanquotes	f	1
3057	29	Magic File 1 and Conan Special Volumes :: http://forums.dctp.ws/index.php?topic=1417.0	conanquotes	f	1
3058	29	What video games are you currently playing? :: http://forums.dctp.ws/index.php?topic=1418.0	conanquotes	f	1
3059	29	Detective Conan File 690 RAW :: http://forums.dctp.ws/index.php?topic=1419.0	conanquotes	f	1
3060	29	Business ethics? :: http://forums.dctp.ws/index.php?topic=1420.0	conanquotes	f	1
3061	29	Detective Conan doodles of mine (Warning: They are BAD) :: http://forums.dctp.ws/index.php?topic=1421.0	conanquotes	f	1
3062	29	Conan and haibara fanart :: http://forums.dctp.ws/index.php?topic=1424.0	conanquotes	f	1
3063	29	Lateral Thinking XVIII :: http://forums.dctp.ws/index.php?topic=1425.0	conanquotes	f	1
3064	29	Can you give me a list of episodes where Shinichi Kudou (not Conan) is present? :: http://forums.dctp.ws/index.php?topic=1426.0	conanquotes	f	1
3065	29	File 690: Who will be the one hide under the table? :: http://forums.dctp.ws/index.php?topic=1427.0	conanquotes	f	1
3066	29	PLANET X/NIBIRU?!?!?!?!?! :: http://forums.dctp.ws/index.php?topic=1428.0	conanquotes	f	1
3067	29	Hattori and Ellery Queen :: http://forums.dctp.ws/index.php?topic=1429.0	conanquotes	f	1
3068	29	Different animes, Gosho art style. :: http://forums.dctp.ws/index.php?topic=1430.0	conanquotes	f	1
3069	29	Tsubasa or XXXholic? :: http://forums.dctp.ws/index.php?topic=1431.0	conanquotes	f	1
3070	29	Growing of My Heart :: http://forums.dctp.ws/index.php?topic=1433.0	conanquotes	f	1
3071	29	[DCTP] HD Encodes Back on Track :: http://forums.dctp.ws/index.php?topic=1434.0	conanquotes	f	1
3072	29	OP's animation sequences :: http://forums.dctp.ws/index.php?topic=1435.0	conanquotes	f	1
3073	29	Manga that brought tears :: http://forums.dctp.ws/index.php?topic=1436.0	conanquotes	f	1
3074	29	Nanako's Singing <3 :: http://forums.dctp.ws/index.php?topic=1437.0	conanquotes	f	1
3075	29	KID and the idea of "eternal life" :: http://forums.dctp.ws/index.php?topic=1438.0	conanquotes	f	1
3076	29	Happy Birthday Shinichi<3 :: http://forums.dctp.ws/index.php?topic=1439.0	conanquotes	f	1
3077	29	I have no gift 4 Shin'ichi :: http://forums.dctp.ws/index.php?topic=1440.0	conanquotes	f	1
3078	29	Shiratori is no Longer a Suspect Candidate... :: http://forums.dctp.ws/index.php?topic=1441.0	conanquotes	f	1
3079	29	Midori no Shinichi :: http://forums.dctp.ws/index.php?topic=1443.0	conanquotes	f	1
3080	29	When is Ai's Birthday? :: http://forums.dctp.ws/index.php?topic=1444.0	conanquotes	f	1
3081	29	Detective Conan: The Manga's Fate :: http://forums.dctp.ws/index.php?topic=1445.0	conanquotes	f	1
3082	29	An AMV :: http://forums.dctp.ws/index.php?topic=1446.0	conanquotes	f	1
3083	29	Detective Conan Manga: Chapter Covers :: http://forums.dctp.ws/index.php?topic=1447.0	conanquotes	f	1
3084	29	He's eating Kirite's Cake :: http://forums.dctp.ws/index.php?topic=1448.0	conanquotes	f	1
3085	31	<MrWiz88> hes faster then goemon's sword draw, more powerful then a tak solo, and can ban whole channels in a single bound <MrWiz88> hes....Jd-	shiranpuri	t	3
1650	25	Dreams abound of arming the rebels. What of nocturnal horses schedules?	simcop2387	f	1
3087	32	0768 User Datagram Protocol J. Postel [ August 1980 ] ( TXT = 5896 bytes)(Also STD0006) (Status: STANDARD)	shiranpuri	f	2
3086	32	User Datagram Protocol J. Postel [ August 1980 ] ( TXT = 5896 bytes)(Also STD0006) (Status: STANDARD)	shiranpuri	t	1
3090	32	0793 Transmission Control Protocol J. Postel [ September 1981 ] ( TXT = 172710 bytes)(Updated by RFC1122, RFC3168) (Also STD0007) (Status: STANDARD)	shiranpuri	f	1
3116	33	Dihydrolipoamide dehydrogenase is a flavoprotein enzyme which degrades lipoamide, and produces dihydrolipoamide.	shiranpuri	f	8
3100	32	0862 Echo Protocol J. Postel [ May 1983 ] ( TXT = 1237 bytes)(Also STD0020) (Status: STANDARD)	shiranpuri	f	1
3109	32	2813 Internet Relay Chat: Server Protocol C. Kalt [ April 2000 ] ( TXT = 56681 bytes)(Updates RFC1459) (Status: INFORMATIONAL)	shiranpuri	f	1
1996	29	Video Streaming :: http://forums.dctp.ws/index.php?topic=146.0	conanquotes	f	2
3107	32	0855 Telnet Option Specifications J. Postel, J.K. Reynolds [ May 1983 ] ( TXT = 6218 bytes)(Obsoletes NIC18640) (Also STD0008) (Status: STANDARD)	shiranpuri	f	2
3108	32	0854 Telnet Protocol Specification J. Postel, J.K. Reynolds [ May 1983 ] ( TXT = 39371 bytes)(Obsoletes RFC0764) (Updated by RFC5198) (Also STD0008) (Status: STANDARD)	shiranpuri	f	2
2526	29	quick question :: http://forums.dctp.ws/index.php?topic=814.0	conanquotes	f	2
3105	32	0857 Telnet Echo Option J. Postel, J. Reynolds [ May 1983 ] ( TXT = 10859 bytes)(Obsoletes NIC15390) (Also STD0028) (Status: STANDARD)	shiranpuri	f	2
3098	32	0864 Character Generator Protocol J. Postel [ May 1983 ] ( TXT = 6842 bytes)(Also STD0022) (Status: STANDARD)	shiranpuri	f	2
3093	32	0826 Ethernet Address Resolution Protocol: Or Converting Network Protocol Addresses to 48.bit Ethernet Address for Transmission on Ethernet Hardware D. Plummer [ November 1982 ] ( TXT = 21556 bytes)(Updated by RFC5227, RFC5494) (Also STD0037) (Status: STANDARD)	shiranpuri	f	2
3104	32	0858 Telnet Suppress Go Ahead Option J. Postel, J. Reynolds [ May 1983 ] ( TXT = 3712 bytes)(Obsoletes NIC15392) (Also STD0029) (Status: STANDARD)	shiranpuri	f	2
3096	32	0866 Active users J. Postel [ May 1983 ] ( TXT = 2029 bytes)(Also STD0024) (Status: STANDARD)	shiranpuri	f	2
3113	32	1459 Internet Relay Chat Protocol J. Oikarinen, D. Reed [ May 1993 ] ( TXT = 138964 bytes)(Updated by RFC2810, RFC2811, RFC2812, RFC2813) (Status: EXPERIMENTAL)	shiranpuri	f	2
3106	32	0856 Telnet Binary Transmission J. Postel, J. Reynolds [ May 1983 ] ( TXT = 8965 bytes)(Obsoletes NIC15389) (Also STD0027) (Status: STANDARD)	shiranpuri	f	2
3110	32	2812 Internet Relay Chat: Client Protocol C. Kalt [ April 2000 ] ( TXT = 122637 bytes)(Updates RFC1459) (Status: INFORMATIONAL)	shiranpuri	f	2
3092	32	0822 STANDARD FOR THE FORMAT OF ARPA INTERNET TEXT MESSAGES D. Crocker [ August 1982 ] ( TXT = 106299 bytes)(Obsoletes RFC0733) (Obsoleted by RFC2822) (Updated by RFC1123, RFC2156, RFC1327, RFC1138, RFC1148) (Also STD0011) (Status: STANDARD)	shiranpuri	f	2
3112	32	2810 Internet Relay Chat: Architecture C. Kalt [ April 2000 ] ( TXT = 19087 bytes)(Updates RFC1459) (Status: INFORMATIONAL)	shiranpuri	f	2
3102	32	0860 Telnet Timing Mark Option J. Postel, J. Reynolds [ May 1983 ] ( TXT = 7881 bytes)(Obsoletes NIC16238) (Also STD0031) (Status: STANDARD)	shiranpuri	f	2
1687	11	<kyuuketsuki> oh wow...  <kyuuketsuki> i'm curious  <kyuuketsuki> Butt Attack Punisher Girl Gautaman - OVA	Jing	f	33
3111	32	2811 Internet Relay Chat: Channel Management C. Kalt [ April 2000 ] ( TXT = 40788 bytes)(Updates RFC1459) (Status: INFORMATIONAL)	shiranpuri	f	2
3118	27	Bored enough to listen to me? I pity you, the fool.	shiranpuri	f	3
3099	32	0863 Discard Protocol J. Postel [ May 1983 ] ( TXT = 1239 bytes)(Also STD0021) (Status: STANDARD)	shiranpuri	f	3
1637	24	Bust, waist, hip sizes?	simcop2387	f	18
3088	32	0791 Internet Protocol J. Postel [ September 1981 ] ( TXT = 97779 bytes)(Obsoletes RFC0760) (Updated by RFC1349) (Also STD0005) (Status: STANDARD)	shiranpuri	f	2
3095	32	0867 Daytime Protocol J. Postel [ May 1983 ] ( TXT = 2289 bytes)(Also STD0025) (Status: STANDARD)	shiranpuri	f	2
3094	32	0868 Time Protocol J. Postel, K. Harrenstien [ May 1983 ] ( TXT = 3024 bytes)(Also STD0026) (Status: STANDARD)	shiranpuri	f	3
3097	32	0865 Quote of the Day Protocol J. Postel [ May 1983 ] ( TXT = 1676 bytes)(Also STD0023) (Status: STANDARD)	shiranpuri	f	3
3101	32	0861 Telnet Extended Options: List Option J. Postel, J. Reynolds [ May 1983 ] ( TXT = 3068 bytes)(Obsoletes NIC16239) (Also STD0032) (Status: STANDARD)	shiranpuri	f	3
3103	32	0859 Telnet Status Option J. Postel, J. Reynolds [ May 1983 ] ( TXT = 4273 bytes)(Obsoletes RFC0651) (Also STD0030) (Status: STANDARD)	shiranpuri	f	2
3089	32	0792 Internet Control Message Protocol J. Postel [ September 1981 ] ( TXT = 30404 bytes)(Obsoletes RFC0777) (Updated by RFC0950, RFC4884) (Also STD0005) (Status: STANDARD)	shiranpuri	f	2
3117	33	DLD is a library package of C functions that performs dynamic link editing. Programs that use DLD can add or remove compiled object code from a process anytime during its execution.	shiranpuri	f	8
3119	34	See http://forums.dctp.ws/index.php?topic=710.0 for a list of megaupload links for detective conan. TheQuyen updates it frequently, typically adding new episodes within the same day or next.	shiranpuri	t	1
3121	17	DDL is down! Use the XDCC bots as an alternative to download, or see !MU for megaupload links. DDL will not be back until Matlock returns for another season, or until there's a new season of Sliders.	shiranpuri	t	273
1686	17	DDL is down! Use the XDCC bots as an alternative to download, The DDL will not be back until Matlock returns for another season, or until there's a new season of Sliders.	Kaitoukidsama	t	544
3120	34	See http://forums.dctp.ws/index.php?topic=710.0 for a list of megaupload links for detective conan. TheQuyen updates it frequently, typically adding new episodes within a day or two.	shiranpuri	t	8
3138	37	The Idol`s Locked Room Murder Case (Episode 3)	shiranpuri	f	0
3139	37	The City Coded Map Case (Episode 4)	shiranpuri	f	0
3140	37	The Bullet Train Bombing Case (Episode 5)	shiranpuri	f	0
3141	37	The Valentine`s Murder Case (Episode 6)	shiranpuri	f	0
3142	37	The Incident of the Monthly Presents and Threats (Episode 7)	shiranpuri	f	0
3143	37	The Murder of the Art Gallery Owner (Episode 8)	shiranpuri	f	0
3144	37	Tenkaichi Night Festival Murder Case (Episode 9)	shiranpuri	f	0
3145	37	The Case of the Blackmailing of a Pro Soccer Player (Episode 10)	shiranpuri	f	0
3146	37	The Case of the Moonlight Sonata Murders (Episode 11)	shiranpuri	f	0
3147	37	The Case of Ayumi`s Kidnapping (Episode 12)	shiranpuri	f	0
3148	37	The Strange Case of the Person-Finding Murder (Episode 13)	shiranpuri	f	0
3149	37	The Mysterious Message Sniper Case (Episode 14)	shiranpuri	f	0
3150	37	The Disappearing Corpse Murder Case (Episode 15)	shiranpuri	f	0
3151	37	The Antique Collector Murder Case (Episode 16)	shiranpuri	f	0
3152	37	The Department Store High-Jacking Case (Episode 17)	shiranpuri	f	0
3153	37	The June Bride Murder Case (Episode 18)	shiranpuri	f	0
3176	37	The Case of the Facial Pack Murder (Episode 45)	shiranpuri	f	1
3129	36	Character Development	shiranpuri	f	0
3130	36	Rotten Apple	shiranpuri	f	0
3131	36	Dark Contact	shiranpuri	f	0
3132	36	Deep Suspicion	shiranpuri	f	0
3133	36	Vital Prospect	shiranpuri	f	0
3134	36	Clash of Red and Black	shiranpuri	f	0
3135	36	Artificial Veneer	shiranpuri	f	0
3136	37	Roller Coaster Murder Case (Episode 1)	shiranpuri	f	0
3137	37	The Company President`s Daughter Kidnapping Case (Episode 2)	shiranpuri	f	0
3154	37	The Case of the Elevator Murder (Episode 19)	shiranpuri	f	0
3155	37	The Haunted Mansion Murder Case (Episode 20)	shiranpuri	f	0
3156	37	The Case of the TV Drama Murder (Episode 21)	shiranpuri	f	0
3157	37	The Cruise Ship Serial Murder Case (Episodes 22-23)	shiranpuri	f	0
3158	37	The Mysterious Beauty Amnesia Case (Episode 24)	shiranpuri	f	0
3159	37	A Fake Kidnapping Fraud Incident (Episode 25)	shiranpuri	f	0
3160	37	Pet Dog John Murder Case (Episode 26)	shiranpuri	f	0
3161	37	Kogoro`s Reunion Murder Case (Episodes 27-28)	shiranpuri	f	0
3162	37	Computer Murder Case (Episode 29)	shiranpuri	f	0
3163	37	Alibi Testimony Murder Case (Episode 30)	shiranpuri	f	0
3164	37	Television Station Murder Case (Episode 31)	shiranpuri	f	0
3165	37	The Coffee Shop Murder Case (Episode 32)	shiranpuri	f	0
3166	37	Detective Boys Survival Case (Episode 33)	shiranpuri	f	0
3167	37	Mountain Villa Bandaged Man Murder Case (Episodes 34-35)	shiranpuri	f	0
3168	37	The Monday Night 7:30 PM Murder Case (Episode 36)	shiranpuri	f	0
3169	37	Cactus Flower Murder Case (Episode 37)	shiranpuri	f	0
3170	37	Akaoni Town Festival Fire Murder Case (Episode 38)	shiranpuri	f	0
3171	37	The Wealthy Daughter Murder Case (Episodes 39-40)	shiranpuri	f	0
3172	37	The Championship Flag Slicing Case (Episode 41)	shiranpuri	f	0
3173	37	Karaoke Box Murder Case (Episode 42)	shiranpuri	f	0
3174	37	The Edogawa Conan Kidnapping Case (Episode 43)	shiranpuri	f	0
3175	37	The Three Hotta Brothers Murder Case (Episode 44)	shiranpuri	f	0
3177	37	The Case of the Snowy Mountain Murder (Episode 46)	shiranpuri	f	0
3178	37	The Case of the Sports Club Murder (Episode 47)	shiranpuri	f	0
3179	37	The Case of the Murdered Diplomat (Episodes 48-49)	shiranpuri	f	0
3181	37	A Golf Club Murder Case (Episode 51)	shiranpuri	f	0
3182	37	The Kiri Tengu Legend Murder Case (Episode 52)	shiranpuri	f	0
3183	37	The Mysterious Weapon Murder Case (Episode 53)	shiranpuri	f	0
3185	37	The Train Trick Murder Case (Episode 55)	shiranpuri	f	0
3186	37	The Ojamambo Murder Case (Episode 56)	shiranpuri	f	0
3187	37	The Holmes Freak Murder Case (Episodes 57-58)	shiranpuri	f	0
3188	37	The First Errand Murder Case (Episode 59)	shiranpuri	f	0
3189	37	The Illustrator Murder Case (Episode 60)	shiranpuri	f	0
3184	37	The Game Company Murder Case (Episode 54)	shiranpuri	f	1
1860	28	Chuck Norris's girlfriend once asked him how much wood a woodchuck could chuck if a woodchuck could chuck wood. He then shouted, "HOW DARE YOU RHYME IN THE PRESENCE OF CHUCK NORRIS!" and ripped out her throat. Holding his girlfriend's bloody throat in his hand he bellowed, "Don't fuck with Chuck!" Two years and five months later he realized the irony of this statement and laughed so	simcop2387	f	3
3114	28	The real reason for the downfall of Sparta? Chuck Norris.	Zdm321	f	3
3190	37	The Ghost Ship Murder Case (Episodes 61-62)	shiranpuri	f	0
3195	37	The Stage Actress Murder Case (Episode 67)	shiranpuri	f	0
3196	37	The Night Baron Murder Case (Episodes 68-70)	shiranpuri	f	0
3197	37	Stalker Murder Case (Episode 71)	shiranpuri	f	0
3198	37	Triplet Resort Murder Case (Episode 72)	shiranpuri	f	0
3199	37	The Detective Boys Shipwrecked Case (Episode 73)	shiranpuri	f	0
3200	37	The Shinigami Jinnai Murder Case (Episode 74)	shiranpuri	f	0
3201	37	The Financial Company President Murder Case (Episode 75)	shiranpuri	f	0
3202	37	Conan vs. Kaito Kid (Episode 76)	shiranpuri	f	0
3203	37	The Famous Family Mysterious Death Case (Episodes 77-78)	shiranpuri	f	0
3204	37	The Bank Robbery Murder Case (Episode 79)	shiranpuri	f	0
3205	37	The Wandering Artist Murder Case (Episode 80)	shiranpuri	f	0
3206	37	The Kidnapping of an Idol (Episodes 81-82)	shiranpuri	f	0
3207	37	The General Hospital Murder Case (Episode 83)	shiranpuri	f	0
3208	37	The Ski Lodge Murder Case (Episodes 84-85)	shiranpuri	f	0
3209	37	Finding the Kidnapping Location Case (Episode 86)	shiranpuri	f	0
3210	37	The Grateful Crane Case (Episode 87)	shiranpuri	f	0
3211	37	Dracula Mansion Case (Episodes 88-89)	shiranpuri	f	0
3212	37	The Flower`s Fragrance Murder Case (Episode 90)	shiranpuri	f	0
3213	37	The Armed Patient Case (Episode 91)	shiranpuri	f	0
3214	37	The Fearful Traverse Murder Case (Episodes 92-93)	shiranpuri	f	0
3215	37	The Legendary Snow Woman Case (Episode 94)	shiranpuri	f	0
3216	37	Kogorou`s Date Murder Case (Episode 95)	shiranpuri	f	0
3217	37	The Pursued Detective! 2 Serial Killings (Episode 96)	shiranpuri	f	0
3218	37	The Farewell Wine Murder Case (Episode 97)	shiranpuri	f	0
3219	37	The Famous Potter Murder Case (Episodes 98-99)	shiranpuri	f	0
3220	37	The Case of The First Love (Episodes 100-101)	shiranpuri	f	0
3221	37	The Case of The Period Artiste (Episodes 102-103)	shiranpuri	f	0
3222	37	The Case of The Bandit Mansion (Episodes 104-105)	shiranpuri	f	0
3223	37	The Case of The Newspaper Photograph (Episode 106)	shiranpuri	f	0
3224	37	The Case of The Alien Mole (Episodes 107-108)	shiranpuri	f	0
3225	37	The Case of The Great Detective Pursuit (Episode 109)	shiranpuri	f	0
3226	37	The Case of The Home Economics Room (Episodes 110-111)	shiranpuri	f	0
3227	37	The Seven Mysteries of Teitan Primary (Episode 112)	shiranpuri	f	0
3228	37	The Case of White Sands Beach (Episode 113)	shiranpuri	f	0
3229	37	The Case of the Oxygene Diving Tank (Episodes 114-115)	shiranpuri	f	0
3230	37	The Missing Mystery Writer Case (Episodes 116-117)	shiranpuri	f	0
3231	37	The Naniwa Serial Murder Case (Episode 118)	shiranpuri	f	0
3232	37	The Masked Yaiber Murder Case (Episode 119)	shiranpuri	f	0
3233	37	The Honey Cocktail Murder Case (Episode 120)	shiranpuri	f	0
3234	37	The Case of the Sealed Bathroom (Episodes 121-122)	shiranpuri	f	0
3235	37	The Weather Girl Kidnapping Case (Episode 123)	shiranpuri	f	0
3236	37	The Mysterious Sniper Murder Case (Episodes 124-125)	shiranpuri	f	0
3237	37	The Traveling Drama Murder Case (Episodes 126-127)	shiranpuri	f	0
3238	37	The Case of the Black Syndicate: One Billion Yen Robbery! (Episode 128)	shiranpuri	f	0
3239	37	The Woman from the Black Organization: The University Professor Murder Case (Episode 129)	shiranpuri	f	0
3240	37	The Stadium Indiscriminate Threatening Case (Episodes 130-131)	shiranpuri	f	0
3241	37	Magic Lover`s Murder Case (Episodes 132-134)	shiranpuri	f	0
3242	37	The Search for the Disappearing Weapon Case (Episode 135)	shiranpuri	f	0
3243	37	The Old Castle Investigation Case (Episodes 136-137)	shiranpuri	f	0
3244	37	The Final Screening Murder Case (Episodes 138-139)	shiranpuri	f	0
3245	37	SOS! The Message from Ayumi (Episode 140)	shiranpuri	f	0
3246	37	The Night Before the Wedding Locked Room Murder Case (Episodes 141-142)	shiranpuri	f	0
3249	37	The Metropolitan Police Detective Love Story (Episodes 146-147)	shiranpuri	f	0
3250	37	The Sudden Streetcar Stopping Case (Episode 148)	shiranpuri	f	0
3252	37	The Truth of the Exploding Car Case (Episodes 150-151)	shiranpuri	f	0
3253	37	The Mysterious Old Man Disappearance Case (Episode 152)	shiranpuri	f	0
3254	37	Sonoko`s Dangerous Summer Story (Episodes 153-154)	shiranpuri	f	0
3255	37	The Underwater Locked Room Case (Episode 155)	shiranpuri	f	0
3256	37	The Metropolitan Police Detective Love Story II (Episodes 156-157)	shiranpuri	f	0
3257	37	The Silent Kanjo Line (Episode 158)	shiranpuri	f	0
3258	37	The Legend of the Mysterious Pagoda (Episodes 159-160)	shiranpuri	f	0
3259	37	The Murder Floating in the Stream Restaurant (Episode 161)	shiranpuri	f	0
3260	37	The Flying Locked Room: Kudo Shinichi`s First Case (Episode 162)	shiranpuri	f	0
3261	37	The secret of the sun, moon and stars! (Episodes 163-164)	shiranpuri	f	0
3262	37	The Vanishing Detective Club Case (Episode 165)	shiranpuri	f	0
3263	37	The Monster of the Tottori Spider Mansion (Episodes 166-168)	shiranpuri	f	0
3264	37	The Kiss of Venus (Episode 169)	shiranpuri	f	0
3265	37	The Blind Spot in the Darkness (Episodes 170-171)	shiranpuri	f	0
3266	37	The Resurrected Dying Message (Episodes 172-173)	shiranpuri	f	0
3267	37	The 20 Year old Murder Case: The Symphony Serial Murders (Episode 174)	shiranpuri	f	0
3268	37	The Man Who Was Killed Four Times (Episode 175)	shiranpuri	f	0
3248	37	The North Star #3 Express Leaving Ueno (Episodes 144-145)	shiranpuri	f	1
3269	37	The Reunion with the Black Corps (Episodes 176-178)	shiranpuri	f	0
3270	37	The Coffee Shop Truck Crash Case (Episode 179)	shiranpuri	f	0
3271	37	The Red Murderous Nocturne (Episodes 180-181)	shiranpuri	f	0
3272	37	The Nine Doors Investigation (Episode 182)	shiranpuri	f	0
3273	37	The Dangerous Recipe (Episode 183)	shiranpuri	f	0
3274	37	The Cursed Masks Smile Coldly (Episode 184)	shiranpuri	f	0
3275	37	The Murdered Detective (Episodes 185-186)	shiranpuri	f	0
3276	37	The Mysterious Gunshot Echoing in the Darkness (Episode 187)	shiranpuri	f	0
3277	37	The Life or Death Recovery (Episodes 188-193)	shiranpuri	f	0
3279	37	The Unseen Weapon: Ran`s First Deduction (Episode 196)	shiranpuri	f	0
3280	37	The Trap of the Super Car (Episodes 197-198)	shiranpuri	f	0
3281	37	Suspect: Mori Kogoro (Episodes 199-200)	shiranpuri	f	0
3283	37	The Black Wings of Icarus (Episodes 203-204)	shiranpuri	f	0
3284	37	The Metropolitan Police Detective Love Story 3 (Episodes 205-206)	shiranpuri	f	0
3285	37	The Deduction That Was Too Good (Episode 207)	shiranpuri	f	0
3286	37	The Entrance to the Maze: The Anger of the Colossus (Episode 208)	shiranpuri	f	0
3287	37	The Falling from Mt. Ryushin Case (Episode 209)	shiranpuri	f	0
3289	37	Mushrooms, Bears, and the Detective Boys (Episodes 212-213)	shiranpuri	f	0
3290	37	The Mysterious Retro Room Case (Episode 214)	shiranpuri	f	0
3291	37	The Bay of Revenge (Episodes 215-216)	shiranpuri	f	0
3292	37	Megure`s Locked Away Secret (Episodes 217-218)	shiranpuri	f	0
3293	37	The Gathered Detectives! Shinichi vs. Kaito Kid (Episode 219)	shiranpuri	f	0
3294	37	The Client Full of Lies (Episodes 220-221)	shiranpuri	f	0
3295	37	And Then There Were No Mermaids (Episodes 222-224)	shiranpuri	f	0
3296	37	The Secret of the High Sales (Episode 225)	shiranpuri	f	0
3297	37	The Battle Game Trap (Episodes 226-227)	shiranpuri	f	0
3298	37	The Murderous Pottery Class (Episodes 228-229)	shiranpuri	f	0
3300	37	The Falling Off The Condo Case (Episode 232)	shiranpuri	f	0
3301	37	The Evidence That Didn`t Disappear (Episodes 233-234)	shiranpuri	f	0
3302	37	The Locked Wine Cellar (Episode 235)	shiranpuri	f	0
3303	37	The Nanki Beach Mystery Tour (Episodes 236-237)	shiranpuri	f	0
3304	37	The Osaka 3 "K" Case (Episodes 238-239)	shiranpuri	f	0
3305	37	The Bullet Train Transport Case (Episodes 240-241)	shiranpuri	f	0
3306	37	Boy Genta`s Misfortune (Episode 242)	shiranpuri	f	0
3307	37	Mouri Kogoro`s Imposter (Episodes 243-244)	shiranpuri	f	0
3308	37	The Gunshot in the Sunflower Building (Episode 245)	shiranpuri	f	0
3309	37	The Mystery in the Net (Episodes 246-247)	shiranpuri	f	0
3310	37	The Alibi of the Soothing Forest (Episode 248)	shiranpuri	f	0
3311	37	The Idol Stars` Secret (Episodes 249-250)	shiranpuri	f	0
3312	37	The Tragedy at the OK Corral (Episode 251)	shiranpuri	f	0
3313	37	The Kidnapper in the Picture (Episode 252)	shiranpuri	f	0
3314	37	The Metro Police HQ Love Story IV (Episodes 253-254)	shiranpuri	f	0
3315	37	The 14th Round of the Matsue Tamazo Linked Verse Contest (Episodes 255-256)	shiranpuri	f	0
3316	37	The Mysterious Punishment from Heaven (Episode 257)	shiranpuri	f	0
3317	37	The Man from Chicago (Episodes 258-259)	shiranpuri	f	0
3318	37	The Shaking Restaurant (Episode 260)	shiranpuri	f	0
3319	37	The Fearful Legend of the Snowy Night (Episodes 261-262)	shiranpuri	f	0
3320	37	The Osaka Double Mystery - The Swordsman and Toyotomi`s Castle (Episode 263)	shiranpuri	f	0
3321	37	The Courtroom Battle: Kisaki vs. Kogoro (Episodes 264-265)	shiranpuri	f	0
3322	37	The Truth Behind Valentine`s (Episodes 266-268)	shiranpuri	f	0
3323	37	The Forgotten Memento from the Crime (Episodes 269-270)	shiranpuri	f	0
3324	37	Secret Rushed Omission (Episodes 271-272)	shiranpuri	f	0
3325	37	The Quiz Oba-san`s Dart Clue Case (Episode 273)	shiranpuri	f	0
3326	37	Truth About the Haunted House (Episodes 274-275)	shiranpuri	f	0
3328	37	English Teacher VS Great Western Detective (Episodes 277-278)	shiranpuri	f	0
3329	37	Labyrinth`s Hooligan (Episodes 279-280)	shiranpuri	f	0
3330	37	The Small Eye Witnesses (Episode 281)	shiranpuri	f	0
3331	37	Mystery of the Water Flowing Stone Garden (Episodes 282-283)	shiranpuri	f	0
3332	37	Chinatown Deja Vu in the Rain (Episodes 284-285)	shiranpuri	f	0
3333	37	The Kudou Shinichi NY Case (Episodes 286-288)	shiranpuri	f	0
3334	37	Mitsuhiko`s Mystifying Forest (Episodes 289-290)	shiranpuri	f	0
3335	37	Solitary Island of the Princess and the Dragon King`s Palace (Episodes 291-293)	shiranpuri	f	0
3336	37	Smash of Love and Determination (Episodes 294-295)	shiranpuri	f	0
3337	37	Houseboat Fishing Shock (Episode 296)	shiranpuri	f	0
3338	37	Courtroom Confrontation II: Kisaki VS Kujo Reiko (Episodes 297-298)	shiranpuri	f	0
3339	37	Kanmon Strait of Friendship and Murderous Intent (Episodes 299-300)	shiranpuri	f	0
3340	37	Parade of Malice and Saints (Episodes 301-302)	shiranpuri	f	0
3341	37	The Victim Who Came Back (Episode 303)	shiranpuri	f	0
3342	37	12 Million Hostages - New Years Special (Episode 304)	shiranpuri	f	0
3343	37	The Unseen Suspect (Episodes 305-306)	shiranpuri	f	0
3344	37	The Dark Footprint (Episodes 307-308)	shiranpuri	f	0
3345	37	Contact with the Black Organization (Episodes 309-311)	shiranpuri	f	0
3346	37	Festival Dolls Dyed in the Setting Sun (Episodes 312-313)	shiranpuri	f	0
3299	37	The Mysterious Passenger (Episodes 230-231)	shiranpuri	f	1
3327	37	Case of the Missing Policeman`s Notebook (Episode 276)	shiranpuri	f	1
3347	37	The Broken Fence of the Viewing Platform (Episode 314)	shiranpuri	f	0
3348	37	Place Exposed to the Sun (Episode 315)	shiranpuri	f	0
3349	37	The Tarnished Masked Hero (Episodes 316-317)	shiranpuri	f	0
3350	37	Cigar Case of Good Fortune (Episodes 318-319)	shiranpuri	f	0
3351	37	Art of Ninja Alibi Craft (Episode 320)	shiranpuri	f	0
3352	37	The Vanished Kidnapper`s Getaway Car (Episodes 321-322)	shiranpuri	f	0
3353	37	Hattori Heiji`s Desperate Situation! (Episodes 323-324)	shiranpuri	f	0
3354	37	Red Horse Inside the Flame (Episodes 325-327)	shiranpuri	f	0
3355	37	Mystery of the Birthday Wine (Episode 328)	shiranpuri	f	0
3356	37	Friendship Torn Apart (Episodes 329-330)	shiranpuri	f	0
3358	37	Alike Princesses (Episodes 333-334)	shiranpuri	f	0
3359	37	Secret of the Touto Film Developer Place (Episodes 335-336)	shiranpuri	f	0
3360	37	Hidden Circumstances of the Falling Incident (Episode 337)	shiranpuri	f	0
3361	37	Four Porsches (Episodes 338-339)	shiranpuri	f	0
3362	37	Hidden Bathroom Secret (Episodes 340-341)	shiranpuri	f	0
3363	37	Bride of Huis Ten Bosch (Episode 342)	shiranpuri	f	0
3364	37	The Convenience Store Trap (Episodes 343-344)	shiranpuri	f	0
3365	37	Confrontation with the Black Connection. Coincidence of the Two Events on a Night with a Full Moon (Episode 345)	shiranpuri	f	0
3366	37	Find the Buttocks` Mark! (Episodes 346-347)	shiranpuri	f	0
3367	37	Love, a Ghost, and a Wordly Inheritance (Episodes 348-349)	shiranpuri	f	0
3369	37	Fishing Tournament Tragedy (Episodes 352-353)	shiranpuri	f	0
3370	37	A Small Client (Episodes 354-355)	shiranpuri	f	0
3371	37	Kaitou Kid`s Miracle Midair Walk (Episode 356)	shiranpuri	f	0
3372	37	Sweetheart is an Illusion of Spring (Episode 357)	shiranpuri	f	0
3373	37	Metropolitan Police Detective Love Story 5 (Episodes 358-359)	shiranpuri	f	0
3374	37	A Mysterious Spring Beetle (Episode 360)	shiranpuri	f	0
3375	37	The Teitan High Hauntings (Episodes 361-362)	shiranpuri	f	0
3376	37	The City Crows (Episode 363)	shiranpuri	f	0
3377	37	The Synchronicity Case (Episodes 364-365)	shiranpuri	f	0
3378	37	The Tragedy of the Pier in Plain Sight (Episodes 366-367)	shiranpuri	f	0
3379	37	The Candy House the Witch Lives In (Episode 368)	shiranpuri	f	0
3380	37	A Lucky Man`s Suspense (Episode 369)	shiranpuri	f	0
3381	37	Running Away in a Game (Episode 370)	shiranpuri	f	0
3382	37	A Course Without Protest (Episodes 371-372)	shiranpuri	f	0
3383	37	Poisonous Spider Trap (Episode 373)	shiranpuri	f	0
3385	37	Time Limit: 1500 Hours (Episode 376)	shiranpuri	f	0
3386	37	Momotarou Mystery Solving Tour (Episodes 377-378)	shiranpuri	f	0
3387	37	Case of the Long Sleeved Kimono at the Hidden Hot Spring on a Snowy Night (Episodes 379-380)	shiranpuri	f	0
3388	37	Which One`s Reasoning Show (Episodes 381-382)	shiranpuri	f	0
3389	37	Miracle at Koshien Ball Park! The Defiants Face the Dark Demon (Episode 383)	shiranpuri	f	0
3390	37	The Target is Mouri Kogoro! (Episode 384)	shiranpuri	f	0
3391	37	Dissonance of the Stradivarius Violin (Episodes 385-387)	shiranpuri	f	0
3392	37	Kogoro Gets Drunk in Satsuma (Episodes 388-389)	shiranpuri	f	0
3393	37	Metropolitan Police Detective Love Story 6 (Episodes 390-391)	shiranpuri	f	0
3394	37	The Mysterious Height Difference of 20cm (Episode 392)	shiranpuri	f	0
3395	37	The Apparent Kidnapping Case (Episode 393)	shiranpuri	f	0
3396	37	Grand Adventure in the Eccentric Mansion (Episodes 394-396)	shiranpuri	f	0
3397	37	The Harsh Bittersweet Juice (Episode 397)	shiranpuri	f	0
3398	37	The Strange Family`s Request (Episodes 398-399)	shiranpuri	f	0
3399	37	Ran`s Suspicions (Episode 400)	shiranpuri	f	0
3400	37	The Caught Red-Handed Jewel Robber (Episodes 401-402)	shiranpuri	f	0
3402	37	Man Who Went to Call an Ambulance (Episode 405)	shiranpuri	f	0
3403	37	Conan Heiji`s Mysterious Magic (Episodes 406-408)	shiranpuri	f	0
3404	37	Advanced Staging and Kidnapping At the Same Time (Episodes 409-410)	shiranpuri	f	0
3405	37	The Shrine Gate`s Suprising Password (Episodes 411-412)	shiranpuri	f	0
3406	37	The Perfect Half Crime`s Riddle (Episode 413)	shiranpuri	f	0
3407	37	Young Boys` Dectective Bluebird (Episode 414)	shiranpuri	f	0
3409	37	Beika Town`s Grenier House (Episode 418)	shiranpuri	f	0
3410	37	Eight Headed Serpent Sword (Episodes 419-420)	shiranpuri	f	0
3411	37	First Love of Ginkgo Color (Episodes 421-422)	shiranpuri	f	0
3412	37	Detective Team and 4 Caterpillar Siblings (Episode 423)	shiranpuri	f	0
3413	37	Photo Mail from the Clown (Episode 424)	shiranpuri	f	0
3414	37	Black Impact! The Instant Where the Hand of Organization Reaches (Episode 425)	shiranpuri	f	0
3415	37	Love Letter to Ran (Episode 426)	shiranpuri	f	0
3416	37	Super Secret of the Road to School (Episodes 427-428)	shiranpuri	f	0
3417	37	Two People Who Can`t Return Yet (Episodes 429-430)	shiranpuri	f	0
3418	37	Metropolitan Police Detective Love Story 7 (Episodes 431-432)	shiranpuri	f	0
3419	37	Conan, A Strange Child (Episode 433)	shiranpuri	f	0
3420	37	Famous Cool Dog`s Achievement (Episode 434)	shiranpuri	f	0
3421	37	Information Gathered On the Detective Boys (Episodes 435-436)	shiranpuri	f	0
3422	37	Aya Ueto and Shin`ichi -- The Promise from 4 Years Ago (Episode 437)	shiranpuri	f	0
3423	37	The Pursuit of the Fish Mail (Episode 438)	shiranpuri	f	0
3401	37	The Mysterious Angel`s Mansion (Episodes 403-404)	shiranpuri	f	1
3368	37	The Forgotten Cell Phone (Episodes 350-351)	shiranpuri	f	1
3425	37	Car Stunt at the Utmost Limit (Episode 440)	shiranpuri	f	0
3426	37	The "Ahh" at the End of Life (Episode 441)	shiranpuri	f	0
3427	37	The Man Obstructing the Steel Frame (Episode 442)	shiranpuri	f	0
3428	37	Gathering Shellfish With a Sigh (Episodes 443-444)	shiranpuri	f	0
3429	37	The Secret of the Russian Blue (Episode 445)	shiranpuri	f	0
3430	37	The Sealed Western-style Window (Episodes 446-447)	shiranpuri	f	0
3431	37	The Meguro Fish Case (Episode 448)	shiranpuri	f	0
3432	37	Metropolitan Police Detective Love Story -- The Wedding of Lies (Episode 449)	shiranpuri	f	0
3433	37	Trick vs. Magic (Episodes 450-451)	shiranpuri	f	0
3434	37	The Mysterious Person from Konpira (Episode 452)	shiranpuri	f	0
3435	37	Preview Meeting of Friendship and Fate (Episode 453)	shiranpuri	f	0
3436	37	Upsetting Outcome (Episodes 454-455)	shiranpuri	f	0
3437	37	The Mystery We Loved (Episode 456)	shiranpuri	f	0
3438	37	Sonoko`s Red Handkerchief (Episodes 457-458)	shiranpuri	f	0
3439	37	Mysterious Person, The Strict Regulations Man (Episode 459)	shiranpuri	f	0
3440	37	Year 1 Class B`s Big Strategy! (Episode 460)	shiranpuri	f	0
3441	37	The Missing Page (Episode 461)	shiranpuri	f	0
3442	37	Shadow of the Black Organization (Episodes 462-465)	shiranpuri	f	0
3443	37	The Unsmashable Snowman (Episodes 466-467)	shiranpuri	f	0
3444	37	The Mysterious Case Near the Pond (Episode 468)	shiranpuri	f	0
3445	37	Kaito Kid and the 4 Masterpieces (Episodes 469-470)	shiranpuri	f	0
3446	37	The Uncontrollable Rent-A-Car! (Episode 471)	shiranpuri	f	0
3447	37	Young Kudo Shinichi`s Adventure (Episodes 472-473)	shiranpuri	f	0
3448	37	Attorney Kisaki Eri`s Love (Episode 474)	shiranpuri	f	0
3449	37	Bad Luck Grand Prix (Episode 475)	shiranpuri	f	0
3450	37	Genta`s Certain Kill Shot (Episodes 476-477)	shiranpuri	f	0
3451	37	Real 30 Minutes (Episode 478)	shiranpuri	f	0
3452	37	Three Days with Hattori Heiji (Episode 479)	shiranpuri	f	0
3453	37	A Yellow Alibi (Episode 480)	shiranpuri	f	0
3454	37	A Mountain Witch`s Knife (Episodes 481-482)	shiranpuri	f	0
3455	37	Disappearing Policeman (Episode 483)	shiranpuri	f	0
3456	37	The Whereabouts Of The Black Photograph (Episodes 484-485)	shiranpuri	f	0
3457	37	Beckoning Cat From the Right to the Left (Episode 486)	shiranpuri	f	0
3458	37	Metropolitan Police Detective Love Story 8 -- The Wedding Finger (Episode 487)	shiranpuri	f	0
3459	37	Demon of the TV Station (Episode 488)	shiranpuri	f	0
3460	37	Courtroom Confrontation III A Lawyer as Eyewitness (Episode 489)	shiranpuri	f	0
3461	37	Hattori Heiji vs. Kudo Shinichi! The Grand Deduction Battle (Episode 490)	shiranpuri	f	0
3462	37	Clash of Red and Black! (Episodes 491-504)	shiranpuri	f	0
3463	37	Lawyer Eri Kisaki`s Testimony (Episodes 505-506)	shiranpuri	f	0
3464	37	The Blind Spot of the Karaoke Box (Episodes 507-508)	shiranpuri	f	0
3465	37	Red, White, Yellow, and the Detective Boys (Episode 509)	shiranpuri	f	0
3466	37	Double Code Mystery (Episode 510)	shiranpuri	f	0
3467	37	Deduction Showdown! Shinichi vs. Subaru Okiya (Episode 511)	shiranpuri	f	0
3468	37	The Broken Horoscope (Episode 512)	shiranpuri	f	0
3469	37	Coffee Aroma with Murderous Intention (Episodes 513-514)	shiranpuri	f	0
3470	37	Kaitou Kid`s Teleportation Magic (Episode 515)	shiranpuri	f	0
3471	37	Furinkazan (Episodes 516-517)	shiranpuri	f	0
3472	37	Meiji Restoration Mystery Tour (Episodes 518-519)	shiranpuri	f	0
3474	37	Murderer, Kudou Shin`ichi (Episode 521)	shiranpuri	f	0
3475	37	Shin`ichi`s True Colours and Ran`s Tears (Episode 522)	shiranpuri	f	0
3476	37	What She Truly wants to Ask (Episode 523)	shiranpuri	f	0
3477	37	The Blue Spark of Hate (Episodes 524-525)	shiranpuri	f	0
3478	37	A Present from the True Culprit (Episode 526)	shiranpuri	f	0
3479	37	Evil Intent Behind a Mask (Episode 527)	shiranpuri	f	0
3480	37	Might over mystery (Episodes 528-529)	shiranpuri	f	0
3481	37	The Truth of The Urban Legend (Episodes 530-531)	shiranpuri	f	0
3482	37	The Scar of First Love (Episode 532)	shiranpuri	f	0
3483	37	The Scar that Invokes the Past (Episode 533)	shiranpuri	f	0
3247	37	The Suspicious Astronomical Observation (Episode 143)	shiranpuri	f	1
3408	37	Evil Spirit Appearing on an Unlucky Day (Episodes 415-417)	shiranpuri	f	1
3484	37	Conan vs Kid vs Yaiba (OVA 1)	shiranpuri	f	0
3485	37	16 Suspects (OVA 2)	shiranpuri	f	0
3486	37	Conan and Heiji and the Vanished Boy (OVA 3)	shiranpuri	f	0
3487	37	Conan and Kid and Crystal Mother (OVA 4)	shiranpuri	f	0
3488	37	The Target is Kogoro! The Detective Boys` Secret Investigation (OVA 5)	shiranpuri	f	0
3489	37	Follow the Vanished Diamond! Conan & Heiji vs. Kid! (OVA 6)	shiranpuri	f	0
3490	37	Dr. Agasa`s Challenge - Agasa Vs Conan & Detective Boys (OVA 7)	shiranpuri	f	0
3491	37	High School Girl Detective Sonoko Suzuki`s Case Files (OVA 8)	shiranpuri	f	0
3492	37	The Time Bombed Skyscraper (Movie 1)	shiranpuri	f	0
3493	37	The Fourteenth Target (Movie 2)	shiranpuri	f	0
3495	37	Captured In her Eyes (Movie 4)	shiranpuri	f	0
3496	37	Countdown To Heaven (Movie 5)	shiranpuri	f	0
3497	37	The Phantom of Baker Street (Movie 6)	shiranpuri	f	0
3424	37	And Everyone Should Have Stopped Being (Episode 439)	shiranpuri	f	1
3498	37	Crossroad in the Ancient Capital (Movie 7)	shiranpuri	f	0
3499	37	Magician of the Silver Sky (Movie 8)	shiranpuri	f	0
3500	37	Strategy Above the Depths (Movie 9)	shiranpuri	f	0
3501	37	Requiem of the Detectives (Movie 10)	shiranpuri	f	0
3502	37	Jolly Roger in the Deep Azure (Movie 11)	shiranpuri	f	0
3503	37	Full Score of Fear (Movie 12)	shiranpuri	f	0
3504	37	The Raven Chaser (Movie 13)	shiranpuri	f	0
3282	37	The Tenth Passenger (Episodes 201-202)	shiranpuri	f	1
1845	28	On June 7th 1994, Chuck Norris entered the same restaurant supermodel Cindy Crawford was eating at. Instinctively, Cindy swept everything off the table, threw herself on it in a fit of lust, and begged Chuck to ravish her. After Chuck finished his beer, he obliged her. When Chuck's magnificent lead sperm cannoned into Cindy's womb it went straight to one of her ovaries and roared, "	simcop2387	f	2
3508	38	http://img230.imageshack.us/img230/1800/shot04685vk4.jpg http://img230.imageshack.us/img230/6382/shot09561vs2.jpg http://img230.imageshack.us/img230/3752/shot20402yr1.jpg http://img230.imageshack.us/img230/6946/shot09922yk1.jpg	shiranpuri	t	1
3506	38	http://img155.imageshack.us/img155/6813/shot33473fw7.jpg	shiranpuri	t	1
3505	38	http://img340.imageshack.us/img340/6070/shot54321vi1.jpg	shiranpuri	t	1
3507	38	http://img155.imageshack.us/img155/1871/shot04537vv3.jpg	shiranpuri	t	0
3510	38	http://img230.imageshack.us/img230/3752/shot20402yr1.jpg	shiranpuri	t	2
3509	38	http://img230.imageshack.us/img230/1800/shot04685vk4.jpg http://img230.imageshack.us/img230/6382/shot09561vs2.jpg http://img230.imageshack.us/img230/6946/shot09922yk1.jpg	shiranpuri	t	2
3511	38	http://img529.imageshack.us/img529/8240/shot14766ny3.jpg http://img87.imageshack.us/img87/1682/shot14917cy8.jpg http://img529.imageshack.us/img529/1640/shot14950rd2.jpg	shiranpuri	t	2
3524	38	http://tinyurl.com/pxxr5v	shiranpuri	f	3
3473	37	Indictment by Red Wine (Episode 520)	shiranpuri	f	1
3512	38	http://img153.imageshack.us/img153/8391/shot13611js1.jpg http://img153.imageshack.us/img153/8171/shot13706js1.jpg http://img153.imageshack.us/img153/5795/shot13932cf5.jpg http://img153.imageshack.us/img153/5592/shot13986nj4.jpg	shiranpuri	t	2
3523	38	http://tinyurl.com/ousn5a	shiranpuri	f	7
3525	38	http://tinyurl.com/olrtxj http://tinyurl.com/olrtxj http://tinyurl.com/ogkzxa	shiranpuri	t	2
3529	28	Chuck Norris has the greatest Poker-Face of all time. He won the 1983 World Series of Poker, despite holding only a Joker, a Get out of Jail Free Monopoloy card, a 2 of clubs, 7 of spades and a green #4 card from the game UNO.	shiranpuri	t	1
2916	29	[DCTP] Detective Conan Episodes 524-525 (Released!) :: http://forums.dctp.ws/index.php?topic=1248.0	conanquotes	f	2
3532	39	http://www.sendspace.com/file/omxbx5	simcop2387	t	1
3534	40	testing	simcop2387	t	1
3537	1	Eri loves cats because she is one!!!	zero0zero	t	42
1688	11	<+Jing> Just like if someone uses something I said, I will beat anyone who heard it to death until they understand that I came up with it.  <+Kaitoukidsama> ...   <+Jing> It's a metaphor  <+Jing> Get with it.  <+Jing> Or I'll beat you to death   <+Jing> <3	Kaitoukidsama	f	34
3518	38	http://tinyurl.com/q5bwyd http://tinyurl.com/q5t9or http://tinyurl.com/pmhgfp	shiranpuri	f	11
-20023	-20023	15,01[11OVA2315,01]	\N	f	0
3530	28	Chuck Norris has the greatest poker-face of all time. He won the 1983 World Poker Series, despite holding only a joker, a get out of jail free monopoly card, a 2 of clubs, 7 of spades and a green #4 UNO card.	shiranpuri	f	3
3128	17	http://dctp.highscoretv.com --- New and fast DDL host that carries only the very latest releases. For older episodes use the XDCC bots, or !MU for megaupload links, or ! plus the episode number to get a link directly (e.g !1, !2, !530, etc.).	Jd	t	603
3531	28	In the movie "The Matrix", Chuck Norris is the Matrix. If you pay close attention in the green "falling code" scenes, you can make out the faint texture of his beard.	shiranpuri	f	3
3515	38	http://tinyurl.com/pvf827	shiranpuri	f	8
1709	27	Hey, you with the teeth...	simcop2387	f	3
1680	11	[Kaitokidsama on #kienai website] And, boy, do we love 6 year old Detectives.	simcop2387	f	33
3517	38	http://tinyurl.com/qcxy5c http://tinyurl.com/p5ob9e http://tinyurl.com/osbger http://tinyurl.com/olra6w	shiranpuri	f	5
3522	38	http://tinyurl.com/okc6vr	shiranpuri	f	5
3536	1	Eri coulden't cook if her life depended on it!	zero0zero	f	68
3539	1	Up to January 2009, only 4 out of 13 seasons have been completely subbed: 1996 (1st), 1997 (2nd), 1999 (4th) and 2008 (13th).	Xoph	f	70
3516	38	http://tinyurl.com/qabap6	shiranpuri	f	13
3528	38	http://img41.imageshack.us/img41/9744/panda2n.jpg	zero0zero	f	12
3526	14	Conan in fact thinks that %NICK% isn't very bright	simcop2387-lab	f	66
3527	38	http://tinyurl.com/qtr8p5 http://tinyurl.com/olrtxj http://tinyurl.com/ogkzxa	shiranpuri	f	12
3519	38	http://tinyurl.com/ppvzlx http://tinyurl.com/rxbl6n http://tinyurl.com/po4cnx	shiranpuri	f	10
3513	38	http://tinyurl.com/r7v5e4	shiranpuri	f	11
3520	38	http://tinyurl.com/pe9plq	shiranpuri	f	12
3763	56	shiranpuri has a pet rock named %NICK%	shiranpuri	f	14
73	11	<Jd> EVERYBODY STAY CALM <Jd> STAY IN YOUR CHAIRS  <Jd> IT'S ONLY A TREMOR <Naruto_Kan> oh , jd is mad !  <Jd> That's actually the complete opposite of mad  <Jd> That's like me saying, "You're smart!" right now	simcop2387	f	34
3777	64	Ran and Sonoko elope.	Vylen	f	39
-20024	-20024	15,01[11OVA2415,01]	\N	f	0
4936	9	"Would cause the katakana to fall to the floor, leaving the trick would unexposed"	Vylen	f	35
3521	38	http://tinyurl.com/q8hcbe	shiranpuri	f	4
3538	3	The reason there is always a culprit in Detective Conan is because Jd payed them	zero0zero	f	33
-20025	-20025	15,01[11OVA2515,01]	\N	f	0
3514	38	http://tinyurl.com/qkhauh http://tinyurl.com/oftntw	shiranpuri	f	7
4939	38	http://tinyurl.com/ybjtyfv	Vylen	f	10
-20026	-20026	15,01[11OVA2615,01]	\N	f	0
4940	38	http://img297.imageshack.us/img297/5299/mitsuhaibara.png	Vylen	f	8
3494	37	The Last Wizard of the Century (Movie 3)	shiranpuri	f	1
3251	37	The Amusement Park Bungee Jumping Case (Episode 149)	shiranpuri	f	1
4580	67	Sacramento blackfish	simcop2387-lap	f	1
3678	9	Zettai Karen Children ch176p7: No i'll be glateful if you leave it.	simcop2387	f	37
3676	11	<@MrWiz88> "vageta, whats the scanner say his power level is?" <@kyuuketsuki> ITS OVER 9000!!!!!!!!! * @kyuuketsuki crushes scouter <@MrWiz88> "what!, 9000! that's not possible"	MrWiz88	f	34
-20027	-20027	15,01[11OVA2715,01]	\N	f	0
3675	11	<~Kaitoukidsama> I actually made Salem newspaper <~Kaitoukidsama> lemme see if i can find it online <@MrWiz88> "Girl missing, age 6, whereabouts still unknown. Trail ends at public library"	MrWiz88	f	34
4941	9	"Case of the Bee-Chicken-Tail (Episode 120)"	Abs-	f	33
3674	11	<~Kaitoukidsama> I actually made Salem newspaper	MrWiz88	t	26
-20028	-20028	15,01[11OVA2815,01]	\N	f	0
1859	28	Chuck Norris doesn't have normal white blood cells like you and I. His have a small black ring around them. This signifies that they are black belts in every form of martial arts and they roundhouse kick the shit out of viruses. That's why Chuck Norris never gets ill.	simcop2387	f	3
1772	28	Chuck Norris put humpty dumpty back together again, only to roundhouse kick him in the face. Later Chuck dined on scrambled eggs with all the king's horses and all the king's men. The king himself could not attend for unspecified reasons. Coincidentally, the autopsoy revealed the cause of death to be a roundhouse kick to the face. There is only one King.	simcop2387	f	3
-254	-254	10,01Episode 15,01[1125415,01] The Metropolitan Police Detective Love Story 4 (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [6FC400C1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HEH3W1GB04,01 || 07Aired 2001/Oct/2207,01 with an audience share of 20.5%04,01 || 07,01 Based on Vol 32 File 11 - Vol 33 File 2	\N	f	1
-255	-255	10,01Episode 15,01[1125515,01] The 14th Round of the Matsue Tamatsukuri Linked Verse Contest (Part 1)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01169mb Codec: [XviD] CRC: [3A542034]04,01 || 07MU Link: 15,01http://megaupload.com/?d=YNCUVED204,01 || 07Aired 2001/Oct/2907,01 with an audience share of 19.3%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 18	\N	f	4
1443	667	Freds Hottest Shrimp ****	Bubba	f	3
-261	-261	10,01Episode 15,01[1126115,01] The Fearful Legend of the Snowy Night (Part 1)04,01 || 15,01Subbed by [10Kienai-AZFS15,01] 07@ 15,01172mb Codec: [h264] CRC: [ED11012B]04,01 || 07MU Link: 15,01http://megaupload.com/?d=8VIGK5Z104,01 || 07Aired 2001/Dec/1007,01 with an audience share of 19.5%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 22	\N	f	2
-245	-245	10,01Episode 15,01[1124515,01] The Gunshot in the Sunflower Estate04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01171 MB Codec: [XVID-MP3] CRC: [FF4EEEEF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9450J6FZ04,01 || 07Aired 2001/Aug/0607,01 with an audience share of 18.2%04,01 || 15,0107,01 AO	\N	f	5
-253	-253	10,01Episode 15,01[1125315,01] The Metropolitan Police Detective Love Story 4 (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [5B9C6D5D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7QGN63K604,01 || 07Aired 2001/Oct/1507,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 32 File 11 - Vol 33 File 2	\N	f	5
-251	-251	10,01Episode 15,01[1125115,01] The Tragedy at the OK Corral04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [39A9BBAD]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LL0NB8ZW04,01 || 07Aired 2001/Sep/1707,01 with an audience share of 19.4%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 16	\N	f	4
-607	-607	10,01Episode 15,01[1160715,01] Courtroom Confrontation IV: Juror Kobayashi Sumiko (Part Two)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [C1B60565/E25B1C83]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FB5AQIOC | 720: http://www.megaupload.com/?d=H7A5LK1N	\N	f	4
-258	-258	10,01Episode 15,01[1125815,01] The Man from Chicago (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [DBE80602]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=I1TNU2TJ04,01 || 07Aired 2001/Nov/1907,01 with an audience share of 19.3%04,01 || 07,01 Based on Vol 32 File 8-10	\N	f	3
-260	-260	10,01Episode 15,01[1126015,01] The Shaking Restaurant04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01174mb Codec: [XviD] CRC: [DA23B0A0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HQ0DHEPL04,01 || 07Aired 2001/Dec/0307,01 with an audience share of 17.3%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 21	\N	f	2
-249	-249	10,01Episode 15,01[1124915,01] The Idol Stars' Secret (Part 1)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01173mb Codec: [XviD] CRC: [FAE201AF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UBE2QQ4904,01 || 07Aired 2001/Sep/0307,01 with an audience share of 18.8%04,01 || 07,01 Based on Vol 32 File 5-715,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 14	\N	f	5
-263	-263	10,01Episode 15,01[1126315,01] The Osaka Double Mystery, The Swordsman and Toyotomi's Castle (2.5 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01700mb CRC: [87FA434E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NDQFYO5X04,01 || 07Aired 2002/Jan/0707,01 with an audience share of 19.5%04,01 || 07,01 Based on Vol 31 File 8 - Vol 32 File 4	\N	f	2
-250	-250	10,01Episode 15,01[1125015,01] The Idol Stars' Secret (Part 2)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [8B0B852B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ICZYP72K04,01 || 07Aired 2001/Sep/1007,01 with an audience share of 20.9%04,01 || 07,01 Based on Vol 32 File 5-715,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 15	\N	f	5
-252	-252	10,01Episode 15,01[1125215,01] The Kidnapper in the Picture04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01171mb Codec: [XviD] CRC: [67D69097]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0RQE9UME04,01 || 07Aired 2001/Oct/0807,01 with an audience share of 17.5%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 17	\N	f	5
-20029	-20029	15,01[11OVA2915,01]	\N	f	0
-20030	-20030	15,01[11OVA3015,01]	\N	f	0
-20031	-20031	15,01[11OVA3115,01]	\N	f	0
-20032	-20032	15,01[11OVA3215,01]	\N	f	0
-20033	-20033	15,01[11OVA3315,01]	\N	f	0
-20034	-20034	15,01[11OVA3415,01]	\N	f	0
-20035	-20035	15,01[11OVA3515,01]	\N	f	0
-20036	-20036	15,01[11OVA3615,01]	\N	f	0
3679	54	Chuck Norris once tried to roundhouse kick Billy Mays. Before his foot even left the ground, Chuck was already on his first of three easy payments of $29.95	MrWiz88	f	4
3682	54	Mr. T once tried to throw Billy Mays. Instead of grabbing Billy, however, Mr. T found only a bucket of OxiClean in his hands. He immediately sought atonement through cleaning his gold chains.	MrWiz88	f	8
3680	54	Not only did Billy Mays sell ice to the Eskimos, he also sold them the extended warranty.	MrWiz88	f	14
1516	667	Pineapple Curry - Gaeng Kua Sapparod *	Bubba	f	4
3684	54	Billy Mays sold a lifetime of pity and a warehouse full of fools to Mr. T.	MrWiz88	f	9
-20037	-20037	15,01[11OVA3715,01]	\N	f	0
3681	54	A Nigerian scammer once emailed Billy Mays. He is now on his 400th package of Mighty Putty.	MrWiz88	f	14
3685	54	After hearing a Billy Mays infomercial, R. Lee Ermey reconsidered his own personal commitment to perpetually yelling.	MrWiz88	f	15
3688	1	conan had sexual relations with sherlock holmes, that's why he's bias towards him.	intx	t	47
2225	29	I'm still confused about lots of things :: http://forums.dctp.ws/index.php?topic=424.0	conanquotes	f	2
1410	667	Mexican Shrimp Cocktail	Bubba	f	3
3697	56	shiranpuri has a pet cat named %NICK%	shiranpuri	f	19
3701	57	SPIDER puts cup on %NICK%.	Huty	t	11
3696	56	shiranpuri has a pet dog named %NICK%	shiranpuri	f	16
100	14	The "psychic" in episode 512 is really a man, you can see his junk protrude through the dress just before the body is collected by the police.	simcop2387-lab	f	66
3683	54	Billy Mays attributes the power of his on-screen persona to an incident during his teenage years when he accidentally ingested a Caps Lock key from an IBM PS/2 keyboard.	MrWiz88	f	16
3707	14	in the Mexican Dubs, Shinichi is known as Jesus De Pablo Kudo	simcop2387	f	66
3687	11	[16:31] <Blob55> Coke makes swallowing pleasurable :)	MrWiz88	f	34
3686	54	If you act now... hahaha, just kidding! Only Billy Mays has the power to compel you to act now.	MrWiz88	f	10
-20038	-20038	15,01[11OVA3815,01]	\N	f	0
3703	11	Zdm321 is a winner.	Jing	f	34
3698	57	%NICK% puts cup on SPIDER	shiranpuri	t	18
1493	667	Emerald Dip and Shamrock Shrimp	Bubba	f	3
3704	11	conanquotes doesn't like to be called conanquotes	child_of_eve	f	34
-20039	-20039	15,01[11OVA3915,01]	\N	f	0
3702	11	<MrWiz88> wheres jds when ya need him?  <Jing> Sucking Jd	Zdm321	f	34
3699	56	shiranpuri likes to put cups on %NICK%.	Huty	f	16
-20040	-20040	15,01[11OVA4015,01]	\N	f	0
-20041	-20041	15,01[11OVA4115,01]	\N	f	0
3693	55	shiranpuri	shiranpuri	f	5
3695	56	shiranpuri lives in a mental institution.	Huty	f	11
3692	55	sit down please	shiranpuri	f	7
3705	57	%NICK% puts a cup on SPIDER	shiranpuri	f	470
4931	69	No you can't play uno in japanese	simcop2387-lap	f	41
3694	55	feign ignorance	shiranpuri	f	16
3689	42	http://farnsworth.simcop2387.info/test/quotes.html	simcop2387	f	53
3691	5	See my source at https://simcop2387.info/subversion/quotebot/ it's currently not documented though.	simcop2387	f	23
3700	56	shiranpuri has boston cream pie for everyone.	Huty	f	13
3706	11	Your mom is so fat she sat on a binary tree and turned it into a linked list in constant time!	notme	f	34
38	1	The Fourteenth Target was inspired by The A.B.C. Murders by Agatha Christie	trtr3434	f	68
1553	14	Conan is not in fact Kudo Shinichi, there never was a Kudo Shinichi, he's just the result of a cloning experiment, and an experiment in artificially aging clones so that the B-Org can create a hyper intelligent army to take on the world.  Ran, Eri, Kogoro, Shinichi's Parents, Agasa, Haibara (also a clone), Heiji, Kazuha, Sonoko, the "FBI" agents in japan, Vermouth, Gin, Pisco, Tequila, Sato, Megure, Takagi, Shiratori, etc. are all in on it.  Kaito Kid is the only one who is trying to stop it.	simcop2387	f	66
1812	28	When Chuck Norris was denied a Bacon McMuffin at McDonalds because it was 10:35, he roundhouse kicked the store so hard it became a KFC.	simcop2387	f	3
-20042	-20042	15,01[11OVA4215,01]	\N	f	0
-20043	-20043	15,01[11OVA4315,01]	\N	f	0
-609	-609	10,01Episode 15,01[1160915,01] White Day of Betrayal (Part Two)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [7FA69BC0/5AB609EF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BNHLKC7W | 720: http://www.megaupload.com/?d=A8A3OU2C	\N	f	2
-20044	-20044	15,01[11OVA4415,01]	\N	f	0
-20045	-20045	15,01[11OVA4515,01]	\N	f	0
-20046	-20046	15,01[11OVA4615,01]	\N	f	0
-30005	-30005	15,01[11SPECIAL515,01] Magic File 504,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01233MB/250MB Codec: [XviD/H264] CRC: [4D464E9E/0AF4137E]04,01 || 07MU Link: 15,01http://www.fileserve.com/file/rJmFFTD | h264: http://www.fileserve.com/file/J2sDe6A	\N	f	1
-20047	-20047	15,01[11OVA4715,01]	\N	f	0
-20048	-20048	15,01[11OVA4815,01]	\N	f	0
2321	29	..::Random Japanese Word Thread::.. :: http://forums.dctp.ws/index.php?topic=549.0	conanquotes	f	2
-20049	-20049	15,01[11OVA4915,01]	\N	f	0
-20050	-20050	15,01[11OVA5015,01]	\N	f	0
3127	35	See http://forums.dctp.ws/index.php?topic=710.0 for a list of megaupload links for detective conan. TheQuyen updates it frequently, typically adding new episodes within a day or two.	sitdownplease	f	1042
3357	37	Suspicious Curry (Episodes 331-332)	shiranpuri	f	1
-30000	-30000	15,01[11SPECIAL015,01]	\N	f	0
-608	-608	10,01Episode 15,01[1160815,01] White Day of Betrayal (Part One)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [26D19806/323D802D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Y80ITI4Y | 720p: http://www.megaupload.com/?d=CEVK5CLK	\N	f	2
1491	667	Shrimp Creole Ala Fred	Bubba	f	4
-611	-611	10,01Episode 15,01[1161115,01] The Ablazed Demon Dog (Will-o'-the-Wisp Chapter)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [32E5003C | 10CC8089]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=IKFZHSJJ | 720: http://www.megaupload.com/?d=N0E5EZA1	\N	f	2
5325	91	!MU for megaupload links, or !MF for mediafire links, or ! plus the episode number to get a link directly (e.g !1, !2, !560, etc.)	simcop2387-lap	t	1
-610	-610	10,01Episode 15,01[1161015,01] The Victim is Kudou Shinichi04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/320 MB Codec: [XviD/H264] CRC: [A74EA890/D042FA37]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZFE00INR | 720: http://www.megaupload.com/?d=1A2951DM	\N	f	4
5326	91	!MU for megaupload links, or !MF for mediafire links, !youtube episode (e.g. !youtube 580), or ! plus the episode number to get a link directly (e.g !1, !2, !560, etc.)	simcop2387-lap	f	17
-612	-612	10,01Episode 15,01[1161215,01] The Ablazed Demon Dog (Footprints Chapter)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [4A23ACA6/1715F0CA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3LLJTTGC | 720: http://www.megaupload.com/?d=ZB9JKJ1E	\N	f	10
-275	-275	10,01Episode 15,01[1127515,01] Truth About the Haunted House (Part 2)04,01 || 15,01Subbed by [10DCTP-AZFS15,01] 07@ 15,01350mb Codec: [XviD] CRC: [2E75803F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2GHYKWF504,01 || 07Aired 2002/Apr/2207,01 with an audience share of 18.3%04,01 || 07,01 Based on Vol 35 File 5-704,01 || 07,01Two parter, contains 274 also15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 25	\N	f	1
-285	-285	10,01Episode 15,01[1128515,01] Chinatown Deja Vu in the Rain (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01226mb CRC: [A055891F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=764GWVLQ04,01 || 07Aired 2002/Jul/0807,01 with an audience share of 17.5%04,01 || 07,01 Based on Vol 34 File 8-10	\N	f	1
-208	-208	10,01Episode 15,01[1120815,01] The Entrance to the Maze, The Anger of the Colossus (1 Hour Special)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01333mb CRC: [68DECD23]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VRB4FIYZ04,01 || 07Aired 2000/Oct/0907,01 with an audience share of 18.6%04,01 || 15,0107,01 AO	\N	f	5
-571	-571	10,01Episode 15,01[1157115,01] Battle of the Haunted Warehouse's Treasure (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 323mb Codec: [XviD | H264] CRC: [1DAAAC73 | 41B509C2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VPFN1UDU | 720: http://www.megaupload.com/?d=ZNIE74IS04,01 || 07Aired 2010/May/0107,0104,01 || 07,01 Based on Vol 66 File 5-7	\N	f	4
-266	-266	10,01Episode 15,01[1126615,01] The Truth Behind Valentine's (Part 1: The Murder)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [F844C1E5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LKAQ7XU004,01 || 07Aired 2002/Jan/2807,01 with an audience share of 20.4%04,01 || 07,01 Based on Vol 33 File 3-615,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 4	\N	f	3
-270	-270	10,01Episode 15,01[1127015,01] The Forgotten Memento from the Crime (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [11AE90F3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WPO5MIAD04,01 || 07Aired 2002/Mar/0407,01 with an audience share of 19.7%04,01 || 07,01 Based on Vol 33 File 7-915,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 8	\N	f	2
-296	-296	10,01Episode 15,01[1129615,01] Houseboat Fishing Shock04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01174mb Codec: [XviD] CRC: [FE4DB0C2]04,01 || 07MU Link: 15,01http://megaupload.com/?d=TPWXDKW904,01 || 07Aired 2002/Oct/1407,01 with an audience share of 15.4%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 26	\N	f	2
-269	-269	10,01Episode 15,01[1126915,01] The Forgotten Memento from the Crime (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [FECC0A05]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Q3TZ69PA04,01 || 07Aired 2002/Feb/1807,01 with an audience share of 18.7%04,01 || 07,01 Based on Vol 33 File 7-915,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 7	\N	f	6
-295	-295	10,01Episode 15,01[1129515,01] The Smash of Determination and Love (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [647A5C64]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YVP5TPPY04,01 || 07Aired 2002/Sep/1607,01 with an audience share of 16.6%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 12	\N	f	2
34	1	Haibara Ai's real name is Miyano Shiho	simcop2387-lab	f	69
3542	14	APTX-4869 is really just processed Melange, the BO is the start of the Spacing Guild	simcop2387	f	66
3556	47	See http://forums.dctp.ws/index.php?topic=1523.0 for a basic guide on downloading on irc.	shiranpuri	f	101
3544	11	Internet Explorer 8 is the latest version of the familiar Web browser that you are most comfortable using.  Internet Explorer 8 helps you get everything that you want from the Web faster, easier, and more privately and securely than ever!	MrWiz88	f	34
3552	46	Bring my tux.	Zdm321	f	13
-10000	-10000	15,01[11MOVIE015,01]	\N	f	0
3548	46	sit down please	shiranpuri	f	18
1760	28	There is no chin behind Chuck Norris' beard. There is only another fist.	simcop2387	f	2
2481	29	Conan & Heiji :: http://forums.dctp.ws/index.php?topic=752.0	conanquotes	f	2
3547	46	shiranpuri	shiranpuri	f	8
1260	666	rpcltccm.dll	simcop2387	f	1
1870	28	When you're Chuck Norris, anything + anything is equal to 1.... 1 roundhouse kick to the face..	simcop2387	f	3
407	666	NISysInf.dll	simcop2387	f	1
3541	42	http://farnsworth.sexypenguins.com/test/quotes.html	simcop2387	t	17
277	666	JOBEXEC.DLL	simcop2387	f	1
3554	11	Jan 1st, 1 AD was a thursday	simcop2387	t	25
3545	43	I'm not your prank monkey	simcop2387	f	33
3555	47	See http://forums.dctp.ws/index.php?topic=1523.0 for a basic guide on downloading from bots on irc.	shiranpuri	t	1
3551	46	Bring my tux	kyuuketsuki	t	6
1658	25	500 =100 =10	simcop2387	f	1
1847	28	Chuck Norris can piss into gale force winds.	simcop2387	f	3
3557	64	in Movie 14, conan fans have been disappearing one by one.  Mouri Kogoro, as always, points at whomever he feels like: %NICK%, the culprit is YOU!  It is now up to conan to find out who is really behind it, and what is really going on~	shiranpuri	f	42
1582	1	Jodie's gun is a classic revolver made by (Smith & Wesson) a famous American firearms company	Wing-ZERO	f	70
3180	37	The Case of the Library Murder (Episode 50)	shiranpuri	f	1
3553	11	00:01 kyuu : hurray its 00:00	MrWiz88	f	33
3559	10	01,01Shiratori confesses to Megure	Ohnoesmilk	f	12
1542	2	Endangering the lives of police officers for personal gain	simcop2387	f	18
1811	28	God offered Chuck Norris the gift to fly, which he swiftly declined for super strength roundhouse ability.	simcop2387	f	3
1831	28	Chuck Norris does not use spell check. If he happens to misspell a word, Oxford will simply change the actual spelling of it.	simcop2387	f	3
3564	10	01,01Conan Gets Aids	Kaitoukidsama	f	13
3563	10	01,01Genta goes begging for his mommy AGAIN	simcop2387	f	9
3558	10	01,01Haibara gives Conan a raunchy blowjob in the second X rated scene of a conan movie	simcop2387	f	8
3560	10	01,01Shinhei died upon seeing Heiji in the preview.	Shinhei	f	8
3562	10	01,01Megure Gets Raped By A BO member	dc30wnag3	f	13
3565	10	01,01Conan Finds Jesus.... and accuses him of being the criminal	Kaitoukidsama	f	9
3561	10	01,01New Bo Member Irish	dc30wnag3	f	10
1846	28	open by the Chuck!?" All of the eggs cowered in the corner. The same thing happened at the other ovary. "I didn't fucking think so!" shouted the lead sperm which then lead the rest of the troops back into Chuck's balls. Chuck pulled out; roundhouse kicked Cindy in the face and told her, "Don't ever waste my time again."	simcop2387	f	2
1773	28	When Chuck Norris played golf for money, chuck marked down a hole in 0 every time, a pro at the golf club, said to Chuck: "excuse me sir, but you cant score zero on a hole". Chuck Norris turned towards the man and said, im Chuck Norris, the man then proceeded to pour gas over his body and set himself on fire because that would be less painful than getting roundhouse kicked by Chuck 	simcop2387	f	2
1768	28	Chuck Norris is ten feet tall, weighs two-tons, breathes fire, and could eat a hammer and take a shotgun blast standing.	simcop2387	f	3
3566	3	Jd smokes the ganja.	Jing	f	33
-1000	-1000	10,01Episode 15,01[11100015,01] Luckily in 2020, if not, in 2021! ROFL	\N	f	34
40	3	Jd does not ever have an ETA on THAT EPISODE	simcop2387	f	16
1758	28	A blind man once stepped on Chuck Norris' shoe. Chuck replied, "Don't you know who I am? I'm Chuck Norris!" The mere mention of his name cured this man blindness. Sadly the first, last, and only thing this man ever saw, was a fatal roundhouse delivered by Chuck Norris.	simcop2387	f	3
3568	49	!11TH	shiranpuri	t	1
3567	49	!M5	shiranpuri	t	0
3593	49	!BADTRANS	shiranpuri	t	0
3592	49	!SHRIMP	shiranpuri	t	0
3591	49	!DLL	shiranpuri	t	0
3590	49	!M13	shiranpuri	t	0
3589	49	!FUNNYSHIT	shiranpuri	t	0
3588	49	!MRT	shiranpuri	t	0
3587	49	!RJDF	shiranpuri	t	0
3586	49	!RRCFF	shiranpuri	t	0
3585	49	!JDF	shiranpuri	t	0
3584	49	!RCFICT	shiranpuri	t	0
3583	49	!RFC	shiranpuri	t	0
3582	49	!NCH	shiranpuri	t	0
3581	49	!RCC	shiranpuri	t	0
3580	49	!DLD	shiranpuri	t	0
3579	49	!FUNNY	shiranpuri	t	0
3578	49	!QUOTES	shiranpuri	t	0
3577	49	!M8	shiranpuri	t	0
3576	49	!RANDOMCONANCRIME	shiranpuri	t	0
3575	49	!NORRIS	shiranpuri	t	0
3574	49	!RCFACT	shiranpuri	t	0
3573	49	!RCT	shiranpuri	t	0
3572	49	!RDFT	shiranpuri	t	0
3571	49	!RCFIC	shiranpuri	t	0
3570	49	!RANDOMCRIME	shiranpuri	t	0
3569	49	!RCI	shiranpuri	t	0
3602	32	2795 The Infinite Monkey Protocol Suite (IMPS). S. Christey. April 1 2000. (Format: TXT=42902 bytes) (Status: INFORMATIONAL)	shiranpuri	f	2
3594	27	I pity %NICK%, the fool.	shiranpuri	f	2
3608	11	<@MrWiz88> they were all removed due to some douchebaggery	MrWiz88	f	34
3607	11	http://smouch.net/lol	MrWiz88	f	34
1857	28	Chuck Norris once went to a frat party, and proceeded to roundhouse every popped collar in sight. He then trank three kegs and shat on their floor, just because he's Chuck Norris.	simcop2387	f	3
3648	52	[Track 6.] Kodoku ni Dance in Vain [Album: B'z]	MrWiz88	f	1
3606	11	[22:58] <andy92> !episode [22:58] * andy92 (~wonton4u9@Rizon-5F36DFD0.hsd1.ca.comcast.net) Quit [22:58] <&simcop2387> !addtrigger !episode [22:58] <&simcop2387> !addsource notepisode [22:58] <&simcop2387> !addsourcetotrigger !episode notepisode [22:59] <&simcop2387> !addquote notepisode You have misunderstood, please try !213 for an example	Jing	f	34
3600	32	2795 The Infinite Monkey Protocol Suite (IMPS). S. Christey. April 1	shiranpuri	t	1
3645	52	[Track 3.] Heart Mo Nureru Number ~Stay Tonight~ [Album: B'z]	MrWiz88	f	2
3664	52	[Track 3.] BREAK THROUGH [Album: Break Through]	MrWiz88	f	2
3644	52	[Track 2.] Half Tone Lady [Album: B'z]	MrWiz88	f	2
3649	52	[Track 7.] It's not a dream [Album: B'z]	MrWiz88	f	2
70	14	Eri used to punish Kogoro with a riding crop whenever he did something wrong in a case, the problem was he liked it, so he made more mistakes until he lost his job as a police officer	simcop2387	f	66
3550	3	[21:16] * @Jd dances	Jing	f	34
3603	50	%NICK% is a ninny	simcop2387	f	33
3599	9	l hope the time pass slowly	simcop2387	f	38
3657	52	[Track 6.] Yoru ni Furaretemo [Album: Off The Lock]	MrWiz88	f	2
3647	52	[Track 5.] Nothing To Change [Album: B'z]	MrWiz88	f	3
3656	52	[Track 5.] GUITAR KIDS RHAPSODY [Album: Off The Lock]	MrWiz88	f	2
3658	52	[Track 7.] LOVING ALL NIGHT [Album: Off The Lock]	MrWiz88	f	2
3662	52	[Track 1.] LADY-GO-ROUND [Album: Break Through]	MrWiz88	f	3
3655	52	[Track 4.] SAFETY LOVE [Album: Off The Lock]	MrWiz88	f	2
3643	52	[Track 1.] Dakara Sono Te wo Hanashite [Album: B'z]	MrWiz88	f	2
3654	52	[Track 3.] NEVER LET YOU GO [Album: Off The Lock]	MrWiz88	f	2
3651	52	[Track 9.] Fake Lips [Album: B'z]	MrWiz88	f	1
3646	52	[Track 4.] Yuube no Crying (This Is My Truth) [Album: B'z]	MrWiz88	f	1
3663	52	[Track 2.] B.U.M [Album: Break Through]	MrWiz88	f	2
3653	52	[Track 2.] HURRY UP! [Album: Off The Lock]	MrWiz88	f	2
3660	52	[Track 9.] ROSY [Album: Off The Lock]	MrWiz88	f	3
3661	52	[Track 10.] OFF THE LOCK [Album: Off The Lock]	MrWiz88	f	2
3627	52	Logic	MrWiz88	t	0
3628	52	Home	MrWiz88	t	0
3629	52	Love Phantom	MrWiz88	t	0
3631	52	Juice	MrWiz88	t	0
3641	52	Ima de wa...Ima nara...Ima mo...	MrWiz88	t	0
3609	52	Lady-Go-Round	MrWiz88	t	0
3611	52	Shodou	MrWiz88	t	0
3612	52	Shinjiru kurai ii darou	MrWiz88	t	0
3613	52	Konya tsuki no mieru oka ni	MrWiz88	t	0
3614	52	Tokyo Devil	MrWiz88	t	0
3615	52	May	MrWiz88	t	0
3616	52	Ai no Bakudan	MrWiz88	t	0
3617	52	Splash	MrWiz88	t	0
3618	52	Koigokoro	MrWiz88	t	0
3620	52	Ai no Prisoner	MrWiz88	t	0
3621	52	Ring	MrWiz88	t	0
3622	52	Raging River	MrWiz88	t	0
3623	52	Tobira	MrWiz88	t	0
3632	52	Yuruginai mono hitotsu	MrWiz88	t	0
3634	52	MVP	MrWiz88	t	0
3635	52	Be There	MrWiz88	t	0
3636	52	Lady Navigation	MrWiz88	t	0
3637	52	Ai no mama ni wagamama ni Boku wa kimi dake o kizutsukenai	MrWiz88	t	0
3638	52	Mienai Chikara ~INVISIBLE ONE~	MrWiz88	t	0
3639	52	Negai	MrWiz88	t	0
3640	52	Samayoeru aoi dangan	MrWiz88	t	0
3624	52	Kobushi wo nigire	MrWiz88	t	1
3630	52	All Out Attack	MrWiz88	t	1
3626	52	KIENAI NIJI	MrWiz88	t	1
3633	52	Giri Giri Chop	MrWiz88	t	1
3619	52	Thinking of You	MrWiz88	t	1
3642	52	Gimme Your Love - Fukutsu no Love Driver	MrWiz88	t	0
3625	52	Kirameku hito	MrWiz88	t	1
3650	52	[Track 8.] Kimi o Dakitai [Album: B'z]	MrWiz88	f	2
3659	52	[Track 8.] OH! GIRL [Album: Off The Lock]	MrWiz88	f	1
3605	51	You have misunderstood, please try !213 for an example	simcop2387	t	42
3596	9	l'll show you next time	simcop2387	f	35
3597	9	l will wait for you	simcop2387	f	32
3595	3	Jd is in fact a gigantic recluse and insists on having better than clean room sanitation in his hotel penthouse residence, he's also a BIG fan of Howard Hughes	simcop2387	f	33
3115	3	<MrWiz88> hes faster then goemon's sword draw, more powerful then a tak solo, and can ban whole channels in a single bound <MrWiz88> hes....Jd	shiranpuri	f	34
1555	14	The Japanese police are just a front for the Yokuza who are just a front for the Illuminati.  Conan will bust them in Volume 76.	simcop2387	f	67
3598	9	then l can watch you more	simcop2387	f	33
3673	11	[23:14] <imandrool> who wants to reccomend me a site for porn.	MrWiz88	f	34
3669	52	[Track 8.] HEY BROTHER [Album: Break Through]	MrWiz88	f	1
3672	52	[Track 11.] STARDUST TRAIN [Album: Break Through]	MrWiz88	f	1
3665	52	[Track 4.] BOYS IN TOWN [Album: Break Through]	MrWiz88	f	2
3652	52	[Track 1.] Kimi no Naka de Odoritai [Album: Off The Lock]	MrWiz88	f	2
3666	52	[Track 5.] Guitar wa naiteiru [Album: Break Through]	MrWiz88	f	2
3667	52	[Track 6.] LOVE & CHAIN [Album: Break Through]	MrWiz88	f	2
3668	52	[Track 7.] Tonari de Nemurasete [Album: Break Through]	MrWiz88	f	1
3670	52	[Track 9.] Ima de wa...Ima nara...Ima mo... [Album: Break Through]	MrWiz88	f	2
3671	52	[Track 10.] SAVE ME!_ [Album: Break Through]	MrWiz88	f	2
-10015	-10015	15,01[11MOVIE1515,01]	\N	f	4
-10016	-10016	15,01[11MOVIE1615,01]	\N	f	0
-10017	-10017	15,01[11MOVIE1715,01]	\N	f	0
-10018	-10018	15,01[11MOVIE1815,01]	\N	f	0
-10019	-10019	15,01[11MOVIE1915,01]	\N	f	0
-10020	-10020	15,01[11MOVIE2015,01]	\N	f	0
-10021	-10021	15,01[11MOVIE2115,01]	\N	f	0
-10022	-10022	15,01[11MOVIE2215,01]	\N	f	0
-10023	-10023	15,01[11MOVIE2315,01]	\N	f	0
-10024	-10024	15,01[11MOVIE2415,01]	\N	f	0
-10025	-10025	15,01[11MOVIE2515,01]	\N	f	0
-10026	-10026	15,01[11MOVIE2615,01]	\N	f	0
-10027	-10027	15,01[11MOVIE2715,01]	\N	f	0
3711	58	04,03NONE OF US IN HERE ARE ANY KIND OF BOTS, JOIN #KIENAI, #AZFS, AND #DCTP YOU CAN GET YOUR CONAN THERE	simcop2387	f	3
-10028	-10028	15,01[11MOVIE2815,01]	\N	f	0
-10029	-10029	15,01[11MOVIE2915,01]	\N	f	0
-10030	-10030	15,01[11MOVIE3015,01]	\N	f	0
-10031	-10031	15,01[11MOVIE3115,01]	\N	f	0
3708	58	04,03NONE OF US IN HERE ARE ANY KIND OF BOTS, JOIN #DCTP, #AZFS, AND #KIENAI YOU CAN GET YOUR CONAN THERE	simcop2387	f	3
3710	58	04,03NONE OF US IN HERE ARE ANY KIND OF BOTS, JOIN #KIENAI, #DCTP, AND #AZFS YOU CAN GET YOUR CONAN THERE	simcop2387	f	3
3709	58	04,03NONE OF US IN HERE ARE ANY KIND OF BOTS, JOIN #DCTP, #KIENAI, AND #AZFS YOU CAN GET YOUR CONAN THERE	simcop2387	f	4
3720	60	"You want to know how two chemicals interact, do you ask them? No, they're going to lie through their lying little chemical teeth. Throw them in a beaker and apply heat."	ut0pia	f	10
3715	59	http://tinyurl.com/l9mtx4	shiranpuri	f	20
3713	58	04,03NONE OF US IN HERE ARE ANY KIND OF BOTS, JOIN #AZFS, #DCTP, AND #KIENAI YOU CAN GET YOUR CONAN THERE	simcop2387	f	4
3712	58	04,03NONE OF US IN HERE ARE ANY KIND OF BOTS, JOIN #AZFS, #KIENAI, AND #DCTP YOU CAN GET YOUR CONAN THERE	simcop2387	f	3
-10032	-10032	15,01[11MOVIE3215,01]	\N	f	0
3718	60	Stacy: "House we have to talk about our relationship and--" | House: "I'm gay!"	ut0pia	f	10
3384	37	A Code for Star and Tobacco (Episodes 374-375)	shiranpuri	f	1
3721	60	"I don't ask why patients lie, I just assume they all do."	ut0pia	f	8
3716	59	http://tinyurl.com/kn2y25	shiranpuri	f	8
3719	60	"I'm gay ~"	ut0pia	f	8
-10033	-10033	15,01[11MOVIE3315,01]	\N	f	0
3717	59	http://tinyurl.com/mtz8go	shiranpuri	f	8
-10034	-10034	15,01[11MOVIE3415,01]	\N	f	0
-10035	-10035	15,01[11MOVIE3515,01]	\N	f	0
-10036	-10036	15,01[11MOVIE3615,01]	\N	f	0
-10037	-10037	15,01[11MOVIE3715,01]	\N	f	0
-10038	-10038	15,01[11MOVIE3815,01]	\N	f	0
-10039	-10039	15,01[11MOVIE3915,01]	\N	f	0
3722	11	<~Kaitoukidsama> library science?  <~Kaitoukidsama> wtf  <~Kaitoukidsama> it's a BOOK  <&Jing|afk> rofl  <~Kaitoukidsama> it goes HERE  <~Kaitoukidsama> end of class	Jing	f	33
3729	14	in the latest chapter of the manga, Conan turns back into Shinichi.  He and Ran have wonderful date, they go back to his place, and then it cuts to the next morning.  Rumor is that in a few weeks Ran will be panicking about missing something.	simcop2387	f	66
-10040	-10040	15,01[11MOVIE4015,01]	\N	f	0
-10041	-10041	15,01[11MOVIE4115,01]	\N	f	0
-10042	-10042	15,01[11MOVIE4215,01]	\N	f	0
-10043	-10043	15,01[11MOVIE4315,01]	\N	f	0
3749	9	[from google translate] Stitching a microphone, xmixer Please set the following in a blender and so on. 	simcop2387	f	36
580	666	XENROLL.DLL	simcop2387	f	1
690	666	daznews.dll	simcop2387	f	1
1106	666	msoeres.dll	simcop2387	f	1
196	666	FLfile.dll	simcop2387	f	1
2111	29	Hentai :: http://forums.dctp.ws/index.php?topic=283.0	conanquotes	f	2
1468	667	Garlic Shrimp - Goong Kratiem Prik Thai *	Bubba	f	3
3753	-14	"Conan will put a cup on SPIDER."	child_of_eve	f	17
3730	11	<nilum> thanks for not unbanning me <nilum> thankfully i have over 5000 mac addresses to use *nilum was kicked by Aeon (nilum) <%Aeon> make that 4999	MrWiz88	f	34
-10044	-10044	15,01[11MOVIE4415,01]	\N	f	0
3726	11	http://animewriter.files.wordpress.com/2009/04/useless-fanservice-motivator.jpg	MrWiz88	f	35
232	666	I18N.Other.dll	simcop2387	f	1
3732	62	http://ninxmz.org/images/demotivators/spetsnaz.jpg	MrWiz88	t	3
3731	62	http://ninxmz.org/images/demotivators/pronunciation.jpg	MrWiz88	t	2
3734	11	http://ninxmz.org/images/epic_threads/1201466548338.jpeg	MrWiz88	t	30
3737	64	this spoiler TBA	simcop2387	t	2
3748	10	Haibara is Conan's Yoko	simcop2387	f	10
3742	64	Movie 14 is going to be a live-action movie.	Zdm321	t	5
3733	11	http://ninxmz.org/images/epic_threads/1201465869776.jpeg	MrWiz88	t	29
3757	14	Random Conan Fiction: Haibara never took the drug to shrink herself, she just ate a loli loli fruit	JD_SUCKS	t	55
3750	64	Rumour has it that Kogoro will be one of the animated characters while Ran will be played by an AV Idol who takes her top off in every movie she's in	simcop2387	f	44
3738	64	Conan finally finds Waldo.	Zdm321	f	33
3744	64	Jd- will show up again.	SPIDER	f	48
3735	667	Simcop2387 On A Platter.	Jing	f	4
3739	64	Conan finally finds Carmen Sandiego.	Zdm321	f	33
3743	64	Movie 14 is going to be a live-action movie with animated characters added in just like Who Framed Roger Rabbit.	simcop2387	f	35
3754	64	Conan will put a cup on SPIDER.	JD_SUCKS	f	47
3758	14	Haibara never took the drug to shrink herself, she just ate a loli loli fruit	JD_SUCKS	f	66
3741	64	Conan finally catches Kaitou Kid.	Zdm321	f	42
3724	11	[17:09] <&Jing> PERIOD NOT INTENDED   [17:10] <~simcop2387> THATS WHAT YOUR GF SAID!   [17:10] <~simcop2387> sorry that was horrible	Jing	f	34
3740	64	Vermouth turns out to be Carmen Sandiego	simcop2387	f	50
3751	11	<@MrWiz88> im buying manga online from a place thats local to kyuu.  <%simcop2387> so does he pick it up, and then read it to you over the phone?	MrWiz88	f	34
3752	11	vidi, vici, veni	simcop2387	f	34
3723	11	<simcop2387> so thats, #blackorghq, #goshoproject, and #nazonokami, and #yami i've stolen, any other old conan groups channels to steal? || simcop2387 is still waiting on #kaizoufansubs || Jd- (~jd@kaizou.is.jd) has joined #KaizouFansubs || <Jd-> AND YOU'LL NEVER GET IT || Jd- (~jd@kaizou.is.jd) has left #KaizouFansubs	simcop2387	f	34
3728	61	PANG	simcop2387	f	150
3727	11	http://animewriter.files.wordpress.com/2009/04/youre-doing-it-wrong-motivator-001.jpg	MrWiz88	f	34
-10045	-10045	15,01[11MOVIE4515,01]	\N	f	0
3756	1	Random Conan Fiction Haibara never took the drug to shrink herself, she just ate a loli loli fruit	JD_SUCKS	t	54
841	666	icwphbk.dll	simcop2387	f	1
3759	11	<Kaitoukidsama> that's why she has to come to me... not the other way around lol  <Kaitoukidsama> i know how to catch a predator works	Jing	f	33
3760	11	<jmaeshawn> i found out it was the little bluetooth dongle that was right next to the wireless card antenna  <MrWiz88> lol you typed dongle  <Jing> lol, "dongle"	Jing	f	34
-10046	-10046	15,01[11MOVIE4615,01]	\N	f	0
3755	11	http://img121.imageshack.us/img121/9725/fag3.jpg	Jing	f	34
5233	14	Genta is actually an experiment of the B.O gone wrong - it's why he's so annoying and won't go away.	Vylen	f	66
-10047	-10047	15,01[11MOVIE4715,01]	\N	f	0
5232	14	Genta's father knows all the B.O. members on a first-name basis.	Abs-	f	66
498	666	System.Configuration.Install.dll	simcop2387	f	1
-10048	-10048	15,01[11MOVIE4815,01]	\N	f	0
-10050	-10050	15,01[11MOVIE5015,01]	\N	f	3
1033	666	mfc71.dll	simcop2387	f	1
-10049	-10049	15,01[11MOVIE4915,01]	\N	f	0
3762	56	shiranpuri has a pet wyvern named %NICK%	shiranpuri	f	14
-20000	-20000	15,01[11OVA015,01]	\N	f	0
3761	56	shiranpuri has a pet dragon named %NICK%	shiranpuri	f	16
5359	98	simcop2387 was raised by a snack vending machine and a microwave.	Huty	f	10
5327	57	%NICK% uses the CUP-BAZOOKA on SPIDER	shiranpuri	f	202
5358	98	testing	simcop2387	t	1
3770	65	Jd- gives %NICK% the boot.	shiranpuri	f	8
3769	65	Kamikazeeee gives %NICK% the boot.	shiranpuri	f	7
3764	65	/me gives %NICK the boot.	shiranpuri	t	8
3765	65	/me gives %NICK% the boot.	shiranpuri	t	4
3766	66	gives %NICK% the boot.	shiranpuri	t	1
-20012	-20012	15,01[11OVA1215,01]	\N	f	2
3774	65	%NICK% gives Kamikazeeee the boot.	shiranpuri	f	8
3773	65	%NICK% gives SPIDER the boot.	shiranpuri	f	9
-20011	-20011	15,01[11OVA1115,01]	\N	f	17
3775	65	shiranpuri gives %NICK% the boot.	shiranpuri	f	8
3771	65	Zdm321 gives %NICK% the boot.	shiranpuri	f	9
3767	65	child_of_eve gives %NICK% the boot.	shiranpuri	f	10
3776	65	%NICK% gives shiranpuri the boot.	shiranpuri	f	7
3768	65	SPIDER gives %NICK% the boot.	shiranpuri	f	8
3772	65	%NICK% gives child_of_eve the boot.	shiranpuri	f	9
-20013	-20013	15,01[11OVA1315,01]	\N	f	1
-20014	-20014	15,01[11OVA1415,01]	\N	f	1
4712	67	spiny basslet	simcop2387-lap	f	1
-20015	-20015	15,01[11OVA1515,01]	\N	f	0
-20016	-20016	15,01[11OVA1615,01]	\N	f	0
-20017	-20017	15,01[11OVA1715,01]	\N	f	0
3781	11	<~Kaitoukidsama> can you upload all the ones you don't have	Kaitoukidsama	f	34
3779	11	<Kaitoukidsama> "<jmaeshawn> i did lots of public stuff as a kid"  <---- wow i took that wrong	Jing	f	34
4203	67	gudgeon	simcop2387-lap	f	1
5360	98	Simcop2387 can't ever stop asking Dave_Lister when the new series starts, and if that smeghead ever grows a brain	simcop2387	f	7
3780	11	<kyuuketsuki> hi  <Parkur> yo kyuu-kun  <Kaitoukidsama> lol  <Jd-> Hey  <Jd-> It's illegal to call him anything but kks  <Jd-> LEARN THE RULES, SON  <Jd-> Wait  <Jd-> Nvm <_<	Zdm321	f	33
3778	11	<Jing> You just made my dad <Jing> day*	jmaeshawn	f	34
3782	11	<&MrWiz88> farnsworth: "JD_Sucks stole all of my lolis and wd4's" -> mangle   <farnsworth> MrWiz88:  "Perhaps Lolita JD_Sucks QD4 can be separated."	MrWiz88	f	34
-236	-236	10,01Episode 15,01[1123615,01] The Nanki-Shirahama Mystery Tour (Part 1)04,01 || 15,01Subbed by [10Puto15,01] 07@ 15,01252MB Codec: [H264] CRC: [563cf59e]04,01 || 07MU Link: 15,01http://megaupload.com/?d=O6WPEIT004,01 || 07Aired 2001/Jun/0407,01 with an audience share of 14.7%04,01 || 15,0107,01 AO	\N	f	66
3833	67	Australasian salmon	simcop2387-lap	f	1
61	14	Eri only left because Kogoro couldn't take the constant S&M anymore	simcop2387	f	66
1550	3	Jd prefers Techman to all other Western European immigrants.	Jd	f	33
-237	-237	10,01Episode 15,01[1123715,01] The Nanki-Shirahama Mystery Tour (Part 2)04,01 || 15,01Subbed by [10Puto15,01] 07@ 15,01243MB Codec: [H264] CRC: [dac2884e]04,01 || 07MU Link: 15,01http://megaupload.com/?d=3UITLVZ704,01 || 07Aired 2001/Jun/1107,01 with an audience share of 17.7%04,01 || 15,0107,01 AO	\N	f	52
3834	67	Australian grayling	simcop2387-lap	f	0
3851	67	barbeled houndshark	simcop2387-lap	f	1
1122	666	msvcirt.dll	simcop2387	f	1
1340	666	vcomp.dll	simcop2387	f	1
1463	667	Shrimp Sauce Piquant	Bubba	f	3
809	666	glN64_v0.4.1.dll	simcop2387	f	1
1086	666	msdia80.dll	simcop2387	f	1
574	666	WLDAP32.DLL	simcop2387	f	1
632	666	caca-sharp.dll	simcop2387	f	1
3784	67	African lungfish	simcop2387-lap	f	0
3785	67	aholehole	simcop2387-lap	f	0
3786	67	airbreathing catfish	simcop2387-lap	f	0
3787	67	airsac catfish	simcop2387-lap	f	0
3788	67	Alaska blackfish	simcop2387-lap	f	0
3789	67	Albacore	simcop2387-lap	f	0
3790	67	Alewife	simcop2387-lap	f	0
3791	67	Alfonsino	simcop2387-lap	f	0
3792	67	Algae eater	simcop2387-lap	f	0
3793	67	alligatorfish	simcop2387-lap	f	0
3794	67	Amago	simcop2387-lap	f	0
3795	67	American sole	simcop2387-lap	f	0
3796	67	Amur pike	simcop2387-lap	f	0
3798	67	Anemonefish	simcop2387-lap	f	0
3799	67	Angelfish	simcop2387-lap	f	0
3800	67	Freshwater angelfish	simcop2387-lap	f	0
3801	67	Marine angelfish	simcop2387-lap	f	0
3802	67	Angel shark	simcop2387-lap	f	0
3803	67	Anglemouth	simcop2387-lap	f	0
3804	67	Atlantic spadefish	simcop2387-lap	f	0
3805	67	Angler	simcop2387-lap	f	0
3806	67	Angler catfish	simcop2387-lap	f	0
3808	67	Antarctic cod	simcop2387-lap	f	0
3809	67	Antarctic dragonfish	simcop2387-lap	f	0
3810	67	Antarctic icefish	simcop2387-lap	f	0
3811	67	Antenna codlet	simcop2387-lap	f	0
3812	67	Arapaima	simcop2387-lap	f	0
3813	67	Archerfish	simcop2387-lap	f	0
3814	67	Arctic char	simcop2387-lap	f	0
3815	67	Armored catfish	simcop2387-lap	f	0
3783	67	African glass catfish	simcop2387-lap	f	1
3816	67	Armored gurnard	simcop2387-lap	f	0
3817	67	Armored searobin	simcop2387-lap	f	0
3818	67	Armorhead	simcop2387-lap	f	0
3819	67	Armorhead catfish	simcop2387-lap	f	0
3820	67	Arowana	simcop2387-lap	f	0
3821	67	Arrowtooth eel	simcop2387-lap	f	0
3822	67	Aruana	simcop2387-lap	f	0
3823	67	Asian carps	simcop2387-lap	f	0
3824	67	Asiatic glassfish	simcop2387-lap	f	0
3825	67	Atka mackerel	simcop2387-lap	f	0
3826	67	Atlantic cod	simcop2387-lap	f	0
3827	67	Atlantic eel	simcop2387-lap	f	0
3828	67	Atlantic herring	simcop2387-lap	f	0
3829	67	Atlantic salmon	simcop2387-lap	f	0
3830	67	Atlantic saury	simcop2387-lap	f	0
3831	67	Atlantic silverside	simcop2387-lap	f	0
3832	67	Atlantic trout	simcop2387-lap	f	0
3835	67	Australian herring	simcop2387-lap	f	0
3836	67	Australian lungfish	simcop2387-lap	f	0
3837	67	Australian prowfish	simcop2387-lap	f	0
3838	67	Ayu	simcop2387-lap	f	0
3839	67	Baikal oilfish	simcop2387-lap	f	0
3840	67	Bala shark	simcop2387-lap	f	0
3841	67	ballan wrasse	simcop2387-lap	f	0
3842	67	bamboo shark	simcop2387-lap	f	0
3843	67	banded killifish	simcop2387-lap	f	0
3844	67	bandfish	simcop2387-lap	f	0
3845	67	bango	simcop2387-lap	f	0
3847	67	banjo catfish	simcop2387-lap	f	0
3848	67	barb	simcop2387-lap	f	0
3849	67	barbel	simcop2387-lap	f	0
3850	67	barbeled dragonfish	simcop2387-lap	f	0
3852	67	barbelless catfish	simcop2387-lap	f	0
3797	67	Anchovy	simcop2387-lap	f	1
3846	67	bangu	simcop2387-lap	f	1
44	3	Contrary to what most people think, Jd is in fact NOT a ravenous ban-monster. Unless you piss him off.	Wycom	f	32
101	14	Conan and Ayumi did it 15 TIMES...before Haibara showed up.	Zdm321	f	66
3853	67	barfish	simcop2387-lap	f	0
3854	67	barracuda	simcop2387-lap	f	0
3855	67	barracudina	simcop2387-lap	f	0
3856	67	barramundi	simcop2387-lap	f	0
3857	67	barred danio	simcop2387-lap	f	0
3858	67	barreleye	simcop2387-lap	f	0
3860	67	bass	simcop2387-lap	f	0
3861	67	basslet	simcop2387-lap	f	0
3862	67	batfish	simcop2387-lap	f	0
3863	67	bat ray	simcop2387-lap	f	0
3865	67	beaked salmon	simcop2387-lap	f	0
3867	67	beardfish	simcop2387-lap	f	0
3868	67	beluga sturgeon	simcop2387-lap	f	0
3870	67	bent-tooth	simcop2387-lap	f	0
3871	67	betta	simcop2387-lap	f	0
3872	67	bichir	simcop2387-lap	f	0
3873	67	bigeye	simcop2387-lap	f	0
3874	67	bigeye squaretail	simcop2387-lap	f	0
3875	67	bighead carp	simcop2387-lap	f	0
3876	67	bigmouth buffalo	simcop2387-lap	f	0
3877	67	bigscale	simcop2387-lap	f	0
3878	67	bigscale fish	simcop2387-lap	f	0
3879	67	bigscale pomfret	simcop2387-lap	f	0
3880	67	billfish	simcop2387-lap	f	0
3881	67	bitterling	simcop2387-lap	f	0
3882	67	black angelfish	simcop2387-lap	f	0
3883	67	black bass	simcop2387-lap	f	0
3884	67	black dragonfish	simcop2387-lap	f	0
3886	67	blackfish	simcop2387-lap	f	0
3887	67	blacktip reef shark	simcop2387-lap	f	0
3888	67	black mackerel	simcop2387-lap	f	0
3889	67	black pickerel	simcop2387-lap	f	0
3890	67	black prickleback	simcop2387-lap	f	0
3891	67	black scalyfin	simcop2387-lap	f	0
3892	67	black sea bass	simcop2387-lap	f	0
3893	67	black scabbardfish	simcop2387-lap	f	0
3894	67	blacksmelt	simcop2387-lap	f	0
3895	67	black swallower	simcop2387-lap	f	0
3896	67	black tetra	simcop2387-lap	f	0
3897	67	black triggerfish	simcop2387-lap	f	0
3898	67	bleak	simcop2387-lap	f	0
3899	67	blenny	simcop2387-lap	f	0
3900	67	blind goby	simcop2387-lap	f	0
3901	67	blind shark	simcop2387-lap	f	0
3902	67	blobfish	simcop2387-lap	f	0
3903	67	blue catfish	simcop2387-lap	f	0
3904	67	blue danio	simcop2387-lap	f	0
3905	67	blue-redstripe danio	simcop2387-lap	f	0
3906	67	blue eye	simcop2387-lap	f	0
3907	67	bluefin tuna	simcop2387-lap	f	0
3908	67	bluefish	simcop2387-lap	f	0
3909	67	bluegill	simcop2387-lap	f	0
3910	67	blue gourami	simcop2387-lap	f	0
3911	67	blue shark	simcop2387-lap	f	0
3912	67	blue triggerfish	simcop2387-lap	f	0
3913	67	blue whiting	simcop2387-lap	f	0
3915	67	bluntnose minnow	simcop2387-lap	f	0
3916	67	boafish	simcop2387-lap	f	0
3917	67	boarfish	simcop2387-lap	f	0
3918	67	bobtail snipe eel	simcop2387-lap	f	0
3919	67	bocaccio	simcop2387-lap	f	0
3920	67	boga	simcop2387-lap	f	0
3921	67	Bombay duck	simcop2387-lap	f	0
3922	67	bonefish	simcop2387-lap	f	0
3923	67	bonito	simcop2387-lap	f	0
3924	67	bonnetmouth	simcop2387-lap	f	0
3925	67	bonytail chub	simcop2387-lap	f	0
3926	67	bonytongue	simcop2387-lap	f	0
3928	67	bowfin	simcop2387-lap	f	0
3859	67	basking shark	simcop2387-lap	f	1
3930	67	bramble shark	simcop2387-lap	f	0
3931	67	bream	simcop2387-lap	f	0
3927	67	bottlenose	simcop2387-lap	f	1
3933	67	bristlenose catfish	simcop2387-lap	f	0
3934	67	broadband dogfish	simcop2387-lap	f	0
3935	67	brook lamprey	simcop2387-lap	f	0
3936	67	brook trout	simcop2387-lap	f	0
3937	67	brotula	simcop2387-lap	f	0
3938	67	brown trout	simcop2387-lap	f	0
3939	67	buffalofish	simcop2387-lap	f	0
3940	67	bullhead	simcop2387-lap	f	0
3941	67	bullhead shark	simcop2387-lap	f	0
3942	67	bull shark	simcop2387-lap	f	0
3943	67	bull trout	simcop2387-lap	f	0
3944	67	burbot	simcop2387-lap	f	0
3945	67	buri	simcop2387-lap	f	0
3946	67	burma danio	simcop2387-lap	f	0
3947	67	burrowing goby	simcop2387-lap	f	0
3948	67	butt monkey	simcop2387-lap	f	0
3807	67	Anglerfish	simcop2387-lap	f	1
3949	67	butterfly ray	simcop2387-lap	f	0
3950	67	butterflyfish	simcop2387-lap	f	0
3951	67	California flyingfish	simcop2387-lap	f	0
3952	67	California halibut	simcop2387-lap	f	0
3953	67	California smoothtongue	simcop2387-lap	f	0
3954	67	canary rockfish	simcop2387-lap	f	0
3955	67	candiru	simcop2387-lap	f	0
3956	67	candlefish	simcop2387-lap	f	0
3957	67	capelin	simcop2387-lap	f	0
3958	67	cardinalfish	simcop2387-lap	f	0
3959	67	carp	simcop2387-lap	f	0
3960	67	carpetshark	simcop2387-lap	f	0
3961	67	carpsucker	simcop2387-lap	f	0
3962	67	catalufa	simcop2387-lap	f	0
3963	67	catfish	simcop2387-lap	f	0
3964	67	catla	simcop2387-lap	f	0
3965	67	cat shark	simcop2387-lap	f	0
3864	67	beachsalmon	simcop2387-lap	f	1
3869	67	bengal danio	simcop2387-lap	f	1
3914	67	bluntnose knifefish	simcop2387-lap	f	1
3885	67	blackchin	simcop2387-lap	f	1
3866	67	beaked sandfish	simcop2387-lap	f	1
3966	67	cavefish	simcop2387-lap	f	0
3967	67	Celebes rainbowfish	simcop2387-lap	f	0
3968	67	central mudminnow	simcop2387-lap	f	0
3969	67	cepalin	simcop2387-lap	f	0
3970	67	chain pickerel	simcop2387-lap	f	0
3971	67	channel bass	simcop2387-lap	f	0
3972	67	channel catfish	simcop2387-lap	f	0
3973	67	char	simcop2387-lap	f	0
3974	67	cherry salmon	simcop2387-lap	f	0
3975	67	chimaera	simcop2387-lap	f	0
3976	67	Chinook salmon	simcop2387-lap	f	0
3977	67	Cherubfish	simcop2387-lap	f	0
3978	67	chub	simcop2387-lap	f	0
3979	67	chubsucker	simcop2387-lap	f	0
3981	67	cichlid	simcop2387-lap	f	0
3982	67	cisco	simcop2387-lap	f	0
3983	67	climbing catfish	simcop2387-lap	f	0
3984	67	climbing gourami	simcop2387-lap	f	0
3985	67	climbing perch	simcop2387-lap	f	0
3986	67	clingfish	simcop2387-lap	f	0
3988	67	clown loach	simcop2387-lap	f	0
3989	67	clown triggerfish	simcop2387-lap	f	0
3990	67	cobbler	simcop2387-lap	f	0
3992	67	cod	simcop2387-lap	f	0
3993	67	cod icefish	simcop2387-lap	f	0
3994	67	codlet	simcop2387-lap	f	0
3995	67	codling	simcop2387-lap	f	0
3996	67	coelacanth	simcop2387-lap	f	0
3997	67	coffinfish	simcop2387-lap	f	0
3999	67	coley	simcop2387-lap	f	0
4000	67	collared carpetshark	simcop2387-lap	f	0
4001	67	collared dogfish	simcop2387-lap	f	0
4002	67	Colorado squawfish	simcop2387-lap	f	0
4003	67	combfish	simcop2387-lap	f	0
4004	67	combtail gourami	simcop2387-lap	f	0
4005	67	combtooth blenny	simcop2387-lap	f	0
4006	67	common carp	simcop2387-lap	f	0
4007	67	common tunny	simcop2387-lap	f	0
4008	67	conger eel	simcop2387-lap	f	0
4009	67	convict blenny	simcop2387-lap	f	0
4010	67	convict cichlid	simcop2387-lap	f	0
4011	67	cookie-cutter shark	simcop2387-lap	f	0
4012	67	coolie loach	simcop2387-lap	f	0
4013	67	cornetfish	simcop2387-lap	f	0
4014	67	cowfish	simcop2387-lap	f	0
4015	67	cownose ray	simcop2387-lap	f	0
4016	67	cow shark	simcop2387-lap	f	0
4017	67	crappie	simcop2387-lap	f	0
4018	67	creek chub	simcop2387-lap	f	0
4019	67	crestfish	simcop2387-lap	f	0
4020	67	crevice kelpfish	simcop2387-lap	f	0
4021	67	croaker	simcop2387-lap	f	0
4022	67	crocodile icefish	simcop2387-lap	f	0
4023	67	crocodile shark	simcop2387-lap	f	0
4024	67	crucian carp	simcop2387-lap	f	0
4025	67	cuchia	simcop2387-lap	f	0
4026	67	cuckoo wrasse	simcop2387-lap	f	0
4027	67	cusk-eel	simcop2387-lap	f	0
4028	67	cuskfish	simcop2387-lap	f	0
4029	67	cutlassfish	simcop2387-lap	f	0
4030	67	cutthroat eel	simcop2387-lap	f	0
4031	67	cutthroat trout	simcop2387-lap	f	0
4032	67	dab	simcop2387-lap	f	0
4033	67	dace	simcop2387-lap	f	0
4034	67	daggertooth pike conger	simcop2387-lap	f	0
4035	67	damselfish	simcop2387-lap	f	0
4036	67	danio	simcop2387-lap	f	0
4037	67	darter	simcop2387-lap	f	0
4038	67	dartfish	simcop2387-lap	f	0
4039	67	dealfish	simcop2387-lap	f	0
4040	67	Death Valley pupfish	simcop2387-lap	f	0
4041	67	deep sea anglerfish	simcop2387-lap	f	0
4042	67	deep sea bonefish	simcop2387-lap	f	0
4044	67	deep sea smelt	simcop2387-lap	f	0
3998	67	coho salmon	simcop2387-lap	f	1
4048	67	delta smelt	simcop2387-lap	f	0
4049	67	demoiselle	simcop2387-lap	f	0
4050	67	denticle herring	simcop2387-lap	f	0
4051	67	desert pupfish	simcop2387-lap	f	0
4053	67	devil ray	simcop2387-lap	f	0
4054	67	discus	simcop2387-lap	f	0
4043	67	deep sea eel	simcop2387-lap	f	1
4056	67	dogfish	simcop2387-lap	f	0
4057	67	dogfish shark	simcop2387-lap	f	0
4058	67	dogteeth tetra	simcop2387-lap	f	0
4059	67	dojo loach	simcop2387-lap	f	0
4060	67	Dolly Varden trout	simcop2387-lap	f	0
4061	67	dorab	simcop2387-lap	f	0
4062	67	dorado	simcop2387-lap	f	0
4063	67	dory	simcop2387-lap	f	0
4064	67	dottyback	simcop2387-lap	f	0
4065	67	dragonet	simcop2387-lap	f	0
4066	67	dragonfish	simcop2387-lap	f	0
4067	67	dragon goby	simcop2387-lap	f	0
4068	67	driftfish	simcop2387-lap	f	0
4070	67	drum 	simcop2387-lap	f	0
4071	67	duckbill	simcop2387-lap	f	0
4072	67	duckbilled barracudina	simcop2387-lap	f	0
4073	67	duckbill eel	simcop2387-lap	f	0
4074	67	dusky grouper	simcop2387-lap	f	0
4075	67	dwarf gourami	simcop2387-lap	f	0
4076	67	dwarf loach	simcop2387-lap	f	0
4077	67	eagle ray	simcop2387-lap	f	0
4078	67	earthworm eel	simcop2387-lap	f	0
4069	67	driftwood catfish	simcop2387-lap	f	1
3991	67	cobia	simcop2387-lap	f	1
3987	67	clownfish	simcop2387-lap	f	1
4052	67	Devario	simcop2387-lap	f	1
4046	67	deepwater flathead	simcop2387-lap	f	1
4045	67	deepwater cardinalfish	simcop2387-lap	f	1
3980	67	chum salmon	simcop2387-lap	f	1
4047	67	deepwater stingray	simcop2387-lap	f	1
4079	67	eel	simcop2387-lap	f	0
4080	67	eelblenny	simcop2387-lap	f	0
4081	67	eel cod	simcop2387-lap	f	0
4082	67	eel-goby	simcop2387-lap	f	0
4083	67	eelpout	simcop2387-lap	f	0
4084	67	eeltail catfish	simcop2387-lap	f	0
4085	67	elasmobranch	simcop2387-lap	f	0
4086	67	electric catfish	simcop2387-lap	f	0
4087	67	electric eel	simcop2387-lap	f	0
4088	67	electric knifefish	simcop2387-lap	f	0
4089	67	electric ray	simcop2387-lap	f	0
4090	67	electric stargazer	simcop2387-lap	f	0
4091	67	elephantfish	simcop2387-lap	f	0
4092	67	elephantnose fish	simcop2387-lap	f	0
4093	67	elver	simcop2387-lap	f	0
4094	67	emperor	simcop2387-lap	f	0
4095	67	emperor angelfish	simcop2387-lap	f	0
4096	67	emperor bream	simcop2387-lap	f	0
4097	67	escolar	simcop2387-lap	f	0
4098	67	eucla cod	simcop2387-lap	f	0
4099	67	eulachon	simcop2387-lap	f	0
4100	67	European chub	simcop2387-lap	f	0
4101	67	European eel	simcop2387-lap	f	0
4102	67	European flounder	simcop2387-lap	f	0
4103	67	European minnow	simcop2387-lap	f	0
4104	67	European perch	simcop2387-lap	f	0
4106	67	false cat shark	simcop2387-lap	f	0
4107	67	false moray	simcop2387-lap	f	0
4108	67	false trevally	simcop2387-lap	f	0
4109	67	fangtooth	simcop2387-lap	f	0
4110	67	fathead sculpin	simcop2387-lap	f	0
4111	67	featherback	simcop2387-lap	f	0
4112	67	featherfin knifefish	simcop2387-lap	f	0
4113	67	fierasfer	simcop2387-lap	f	0
4114	67	filefish	simcop2387-lap	f	0
4115	67	finback cat shark	simcop2387-lap	f	0
4116	67	fingerfish	simcop2387-lap	f	0
4117	67	fire bar danio	simcop2387-lap	f	0
4118	67	firefish	simcop2387-lap	f	0
4119	67	flabby whalefish	simcop2387-lap	f	0
4120	67	flagblenny	simcop2387-lap	f	0
4121	67	flagfin	simcop2387-lap	f	0
4122	67	flagfish	simcop2387-lap	f	0
4123	67	flagtail	simcop2387-lap	f	0
4124	67	flashlight fish	simcop2387-lap	f	0
4125	67	flatfish	simcop2387-lap	f	0
4126	67	flathead	simcop2387-lap	f	0
4127	67	flathead catfish	simcop2387-lap	f	0
4128	67	flat loach	simcop2387-lap	f	0
4129	67	flier	simcop2387-lap	f	0
4130	67	flounder	simcop2387-lap	f	0
4131	67	flying characin	simcop2387-lap	f	0
4132	67	flying gurnard	simcop2387-lap	f	0
4133	67	flyingfish	simcop2387-lap	f	0
4134	67	footballfish	simcop2387-lap	f	0
4135	67	forehead brooder	simcop2387-lap	f	0
4136	67	four-eyed fish	simcop2387-lap	f	0
4137	67	french angelfish	simcop2387-lap	f	0
4138	67	freshwater eel	simcop2387-lap	f	0
4139	67	freshwater flyingfish	simcop2387-lap	f	0
4140	67	freshwater hatchetfish	simcop2387-lap	f	0
4142	67	freshwater shark	simcop2387-lap	f	0
4143	67	frigate mackerel	simcop2387-lap	f	0
4144	67	frilled shark	simcop2387-lap	f	0
4145	67	frogfish	simcop2387-lap	f	0
4146	67	frogmouth catfish	simcop2387-lap	f	0
4147	67	fusilier fish	simcop2387-lap	f	0
4148	67	galjoen fish	simcop2387-lap	f	0
4149	67	Ganges shark	simcop2387-lap	f	0
4150	67	gar	simcop2387-lap	f	0
4153	67	garpike	simcop2387-lap	f	0
4154	67	ghost flathead	simcop2387-lap	f	0
4155	67	ghost carp	simcop2387-lap	f	0
4156	67	ghost knifefish	simcop2387-lap	f	0
4157	67	ghost pipefish	simcop2387-lap	f	0
4158	67	ghoul	simcop2387-lap	f	0
4159	67	giant danio	simcop2387-lap	f	0
4160	67	giant gourami	simcop2387-lap	f	0
4162	67	giant wels	simcop2387-lap	f	0
4163	67	gianttail	simcop2387-lap	f	0
4164	67	gibberfish	simcop2387-lap	f	0
4165	67	Gila trout	simcop2387-lap	f	0
4166	67	gizzard shad	simcop2387-lap	f	0
4167	67	glass catfish	simcop2387-lap	f	0
4168	67	glassfish	simcop2387-lap	f	0
4169	67	glass knifefish	simcop2387-lap	f	0
4170	67	glowlight danio	simcop2387-lap	f	0
4172	67	goblin shark	simcop2387-lap	f	0
4173	67	goby	simcop2387-lap	f	0
4174	67	golden dojo	simcop2387-lap	f	0
4175	67	golden loach	simcop2387-lap	f	0
4176	67	golden shiner	simcop2387-lap	f	0
4177	67	golden trout	simcop2387-lap	f	0
4178	67	goldeye	simcop2387-lap	f	0
4179	67	goldfish	simcop2387-lap	f	0
4180	67	goldspotted killifish	simcop2387-lap	f	0
4181	67	gombessa	simcop2387-lap	f	0
4182	67	goosefish	simcop2387-lap	f	0
4183	67	gopher rockfish	simcop2387-lap	f	0
4184	67	gouramie	simcop2387-lap	f	0
4185	67	grass carp	simcop2387-lap	f	0
4171	67	goatfish	simcop2387-lap	f	1
4187	67	gray eel-catfish	simcop2387-lap	f	0
4188	67	grayling	simcop2387-lap	f	0
4189	67	gray mullet	simcop2387-lap	f	0
4190	67	gray reef shark	simcop2387-lap	f	0
4152	67	garibaldi	simcop2387-lap	f	1
4141	67	freshwater herring	simcop2387-lap	f	1
4105	67	false brotula	simcop2387-lap	f	1
4161	67	giant sea bass	simcop2387-lap	f	1
4151	67	garden eel	simcop2387-lap	f	1
4191	67	great white shark	simcop2387-lap	f	0
4192	67	green swordtail	simcop2387-lap	f	0
4193	67	greeneye	simcop2387-lap	f	0
4194	67	greenling	simcop2387-lap	f	0
4195	67	grenadier	simcop2387-lap	f	0
4196	67	grideye	simcop2387-lap	f	0
4197	67	ground shark	simcop2387-lap	f	0
4198	67	grouper	simcop2387-lap	f	0
4199	67	grunion	simcop2387-lap	f	0
4201	67	grunter	simcop2387-lap	f	0
4204	67	guitarfish	simcop2387-lap	f	0
4205	67	gulf menhaden	simcop2387-lap	f	0
4206	67	gulper eel	simcop2387-lap	f	0
4208	67	gunnel	simcop2387-lap	f	0
4209	67	guppy	simcop2387-lap	f	0
4210	67	gurnard	simcop2387-lap	f	0
4211	67	haddock	simcop2387-lap	f	0
4212	67	hagfish	simcop2387-lap	f	0
4215	67	half-gill	simcop2387-lap	f	0
4216	67	halfbeak	simcop2387-lap	f	0
4217	67	halfmoon	simcop2387-lap	f	0
4218	67	halibut	simcop2387-lap	f	0
4219	67	halosaur	simcop2387-lap	f	0
4220	67	hamlet	simcop2387-lap	f	0
4221	67	hammerhead shark	simcop2387-lap	f	0
4222	67	Hammerjaw	simcop2387-lap	f	0
4223	67	handfish	simcop2387-lap	f	0
4224	67	hardhead catfish	simcop2387-lap	f	0
4225	67	harelip sucker	simcop2387-lap	f	0
4227	67	hawkfish	simcop2387-lap	f	0
4229	67	herring smelt	simcop2387-lap	f	0
4230	67	hillstream loach	simcop2387-lap	f	0
4231	67	hog sucker	simcop2387-lap	f	0
4232	67	hoki	simcop2387-lap	f	0
4233	67	horn shark	simcop2387-lap	f	0
4234	67	horsefish	simcop2387-lap	f	0
4235	67	houndshark	simcop2387-lap	f	0
4213	67	hairtail	simcop2387-lap	f	1
4237	67	humuhumu-nukunuku-apua‘a	simcop2387-lap	f	0
4238	67	hussar	simcop2387-lap	f	0
4239	67	icefish	simcop2387-lap	f	0
4240	67	ide	simcop2387-lap	f	0
4241	67	ilisha	simcop2387-lap	f	0
4242	67	inanga	simcop2387-lap	f	0
4243	67	inconnu	simcop2387-lap	f	0
4244	67	Indian mul	simcop2387-lap	f	0
4245	67	jack	simcop2387-lap	f	0
4246	67	jackfish	simcop2387-lap	f	0
4247	67	Jack Dempsey	simcop2387-lap	f	0
4248	67	Japanese eel	simcop2387-lap	f	0
4250	67	jellynose fish	simcop2387-lap	f	0
4251	67	jewelfish	simcop2387-lap	f	0
4252	67	jewel tetra	simcop2387-lap	f	0
4253	67	jewfish	simcop2387-lap	f	0
4254	67	john dory	simcop2387-lap	f	0
4255	67	Kafue pike	simcop2387-lap	f	0
4256	67	kahawai	simcop2387-lap	f	0
4257	67	kaluga	simcop2387-lap	f	0
4258	67	kanyu	simcop2387-lap	f	0
4259	67	kelp perch	simcop2387-lap	f	0
4260	67	kelpfish	simcop2387-lap	f	0
4261	67	killifish	simcop2387-lap	f	0
4262	67	king of herring	simcop2387-lap	f	0
4265	67	kissing gourami	simcop2387-lap	f	0
4266	67	knifefish	simcop2387-lap	f	0
4267	67	knifejaw	simcop2387-lap	f	0
4268	67	koi	simcop2387-lap	f	0
4269	67	kokanee	simcop2387-lap	f	0
4270	67	kokopu	simcop2387-lap	f	0
4271	67	kuhli loach	simcop2387-lap	f	0
4272	67	kappy	simcop2387-lap	f	0
4273	67	labyrinth fish	simcop2387-lap	f	0
4274	67	ladyfish	simcop2387-lap	f	0
4275	67	lagena	simcop2387-lap	f	0
4276	67	lake chub	simcop2387-lap	f	0
4278	67	lake whitefish	simcop2387-lap	f	0
4279	67	lampfish	simcop2387-lap	f	0
4280	67	lamprey	simcop2387-lap	f	0
4281	67	lancetfish	simcop2387-lap	f	0
4282	67	lanternfish	simcop2387-lap	f	0
4283	67	large-eye bream	simcop2387-lap	f	0
4284	67	largemouth bass	simcop2387-lap	f	0
4285	67	largenose fish	simcop2387-lap	f	0
4286	67	leaffish	simcop2387-lap	f	0
4287	67	leatherjacket	simcop2387-lap	f	0
4289	67	lemon shark	simcop2387-lap	f	0
4291	67	leopard danio	simcop2387-lap	f	0
4292	67	lightfish	simcop2387-lap	f	0
4293	67	lighthousefish	simcop2387-lap	f	0
4294	67	limia	simcop2387-lap	f	0
4296	67	ling	simcop2387-lap	f	0
4297	67	ling cod	simcop2387-lap	f	0
4298	67	lionfish	simcop2387-lap	f	0
4299	67	livebearer	simcop2387-lap	f	0
4300	67	lizardfish	simcop2387-lap	f	0
4301	67	loach	simcop2387-lap	f	0
4302	67	loach catfish	simcop2387-lap	f	0
4303	67	loach goby	simcop2387-lap	f	0
4304	67	loach minnow	simcop2387-lap	f	0
4305	67	longfin	simcop2387-lap	f	0
4306	67	longfin dragonfish	simcop2387-lap	f	1
4288	67	lefteye flounder	simcop2387-lap	f	1
4214	67	hake	simcop2387-lap	f	1
4200	67	grunt	simcop2387-lap	f	1
4290	67	lenok	simcop2387-lap	f	1
4263	67	Kingfish	simcop2387-lap	f	1
4249	67	jawfish	simcop2387-lap	f	1
4264	67	king-of-the-salmon	simcop2387-lap	f	1
4202	67	grunt sculpin	simcop2387-lap	f	1
4277	67	lake trout	simcop2387-lap	f	1
4228	67	herring	simcop2387-lap	f	1
4226	67	hatchetfish	simcop2387-lap	f	1
4207	67	gulper	simcop2387-lap	f	1
4295	67	lined sole	simcop2387-lap	f	1
4307	67	longfin escolar	simcop2387-lap	f	0
4308	67	longfin smelt	simcop2387-lap	f	0
4309	67	long-finned char	simcop2387-lap	f	0
4310	67	long-finned pike	simcop2387-lap	f	0
4311	67	longjaw mudsucker	simcop2387-lap	f	0
4312	67	longneck eel	simcop2387-lap	f	0
4313	67	longnose chimaera	simcop2387-lap	f	0
4314	67	longnose dace	simcop2387-lap	f	0
4315	67	longnose lancetfish	simcop2387-lap	f	0
4316	67	longnose sucker	simcop2387-lap	f	0
4317	67	longnose whiptail catfish	simcop2387-lap	f	0
4318	67	long-whiskered catfish	simcop2387-lap	f	0
4319	67	lookdown catfish	simcop2387-lap	f	0
4320	67	loosejaw	simcop2387-lap	f	0
4321	67	Lost River sucker	simcop2387-lap	f	0
4322	67	louvar	simcop2387-lap	f	0
4323	67	loweye catfish	simcop2387-lap	f	0
4324	67	luminous hake	simcop2387-lap	f	0
4325	67	luderick	simcop2387-lap	f	0
4416	67	olive flounder	simcop2387-lap	f	1
4327	67	lumpsucker	simcop2387-lap	f	0
4328	67	lungfish	simcop2387-lap	f	0
4329	67	lyretail	simcop2387-lap	f	0
4330	67	mackerel	simcop2387-lap	f	0
4331	67	mackerel shark	simcop2387-lap	f	0
4332	67	madtom	simcop2387-lap	f	0
4334	67	mahseer	simcop2387-lap	f	0
4335	67	mail-cheeked fish	simcop2387-lap	f	0
4336	67	mako shark	simcop2387-lap	f	0
4337	67	manefish	simcop2387-lap	f	0
4338	67	man-of-war fish	simcop2387-lap	f	0
4339	67	Manta Ray	simcop2387-lap	f	0
4340	67	marblefish	simcop2387-lap	f	0
4341	67	marine hatchetfish	simcop2387-lap	f	0
4342	67	marlin	simcop2387-lap	f	0
4343	67	masu salmon	simcop2387-lap	f	0
4344	67	medaka	simcop2387-lap	f	0
4345	67	medusafish	simcop2387-lap	f	0
4346	67	megamouth shark	simcop2387-lap	f	0
4347	67	menhaden	simcop2387-lap	f	0
4348	67	merluccid hake	simcop2387-lap	f	0
4349	67	Mexican blind cavefish	simcop2387-lap	f	0
4353	67	minnow	simcop2387-lap	f	0
4354	67	Modoc sucker	simcop2387-lap	f	0
4355	67	mojarra	simcop2387-lap	f	0
4356	67	mola	simcop2387-lap	f	0
4357	67	molly	simcop2387-lap	f	0
4358	67	monkeyface prickleback	simcop2387-lap	f	0
4359	67	monkfish	simcop2387-lap	f	0
4360	67	mooneye	simcop2387-lap	f	0
4361	67	moonfish	simcop2387-lap	f	0
4362	67	Moorish idol	simcop2387-lap	f	0
4363	67	mora	simcop2387-lap	f	0
4365	67	morid cod	simcop2387-lap	f	0
4366	67	morwong	simcop2387-lap	f	0
4367	67	Moses sole	simcop2387-lap	f	0
4368	67	mosquitofish	simcop2387-lap	f	0
4370	67	mouthbrooder	simcop2387-lap	f	0
4371	67	Mozambique tilapia	simcop2387-lap	f	0
4372	67	mrigal	simcop2387-lap	f	0
4373	67	mud catfish	simcop2387-lap	f	0
4374	67	mudfish	simcop2387-lap	f	0
4375	67	mudminnow	simcop2387-lap	f	0
4376	67	mud minnow	simcop2387-lap	f	0
4377	67	mudskipper	simcop2387-lap	f	0
4378	67	mudsucker	simcop2387-lap	f	0
4379	67	mullet	simcop2387-lap	f	0
4380	67	mummichog	simcop2387-lap	f	0
4381	67	murray cod	simcop2387-lap	f	0
4382	67	muskellunge	simcop2387-lap	f	0
4383	67	mustache triggerfish	simcop2387-lap	f	0
4384	67	mustard eel	simcop2387-lap	f	0
4385	67	naked-back knifefish	simcop2387-lap	f	0
4386	67	nase	simcop2387-lap	f	0
4387	67	needlefish	simcop2387-lap	f	0
4388	67	neon tetra	simcop2387-lap	f	0
4389	67	New World rivuline	simcop2387-lap	f	0
4390	67	New Zealand smelt	simcop2387-lap	f	0
4391	67	nibbler	simcop2387-lap	f	0
4392	67	noodlefish	simcop2387-lap	f	0
4393	67	North American darter	simcop2387-lap	f	0
4394	67	North American freshwater catfish	simcop2387-lap	f	0
4395	67	North Pacific daggertooth	simcop2387-lap	f	0
4396	67	northern anchovy	simcop2387-lap	f	0
4397	67	northern clingfish	simcop2387-lap	f	0
4398	67	northern lampfish	simcop2387-lap	f	0
4399	67	northern pearleye	simcop2387-lap	f	0
4400	67	northern pike	simcop2387-lap	f	0
4401	67	northern sea robin	simcop2387-lap	f	0
4402	67	northern squawfish	simcop2387-lap	f	0
4403	67	northern Stargazer	simcop2387-lap	f	0
4404	67	Norwegian Atlantic salmon	simcop2387-lap	f	0
4406	67	nurse shark	simcop2387-lap	f	0
4407	67	oarfish	simcop2387-lap	f	0
4408	67	ocean perch	simcop2387-lap	f	0
4409	67	ocean sunfish	simcop2387-lap	f	0
4410	67	oceanic flyingfish	simcop2387-lap	f	0
4411	67	oceanic whitetip shark	simcop2387-lap	f	0
4412	67	oilfish	simcop2387-lap	f	0
4413	67	oldwife	simcop2387-lap	f	0
4414	67	Old World knifefish	simcop2387-lap	f	0
4415	67	Old World rivuline	simcop2387-lap	f	0
4417	67	opaleye	simcop2387-lap	f	0
4405	67	nurseryfish	simcop2387-lap	f	1
4364	67	moray eel	simcop2387-lap	f	1
4333	67	mahi-mahi	simcop2387-lap	f	1
4369	67	mosshead warbonnet	simcop2387-lap	f	1
4352	67	milkfish	simcop2387-lap	f	1
4351	67	midshipman	simcop2387-lap	f	1
4350	67	Mexican golden trout	simcop2387-lap	f	1
4469	67	pineconefish	simcop2387-lap	f	1
4419	67	orangespine unicorn fish	simcop2387-lap	f	0
4420	67	orangestriped triggerfish	simcop2387-lap	f	0
4421	67	orbicular batfish	simcop2387-lap	f	0
4422	67	orbicular velvetfish	simcop2387-lap	f	0
4423	67	Oregon chub	simcop2387-lap	f	0
4424	67	Oriental loach	simcop2387-lap	f	0
4425	67	Owens pupfish	simcop2387-lap	f	0
4426	67	Pacific albacore	simcop2387-lap	f	0
4427	67	Pacific argentine	simcop2387-lap	f	0
4428	67	Pacific cod	simcop2387-lap	f	0
4429	67	Pacific hake	simcop2387-lap	f	0
4430	67	Pacific herring	simcop2387-lap	f	0
4431	67	Pacific lamprey	simcop2387-lap	f	0
4432	67	Pacific salmon	simcop2387-lap	f	0
4433	67	Pacific saury	simcop2387-lap	f	0
4434	67	Pacific trout	simcop2387-lap	f	0
4435	67	Pacific viperfish	simcop2387-lap	f	0
4436	67	paddlefish	simcop2387-lap	f	0
4437	67	panga	simcop2387-lap	f	0
4438	67	paperbone	simcop2387-lap	f	0
4439	67	paradise fish	simcop2387-lap	f	0
4440	67	parasitic catfish	simcop2387-lap	f	0
4441	67	parrotfish	simcop2387-lap	f	0
4442	67	peacock flounder	simcop2387-lap	f	0
4443	67	peamouth	simcop2387-lap	f	0
4444	67	pearleye	simcop2387-lap	f	0
4445	67	pearlfish	simcop2387-lap	f	0
4446	67	pearl danio	simcop2387-lap	f	0
4447	67	pearl perch	simcop2387-lap	f	0
4448	67	pejerrey	simcop2387-lap	f	0
4449	67	peladillo	simcop2387-lap	f	0
4450	67	pelagic cod	simcop2387-lap	f	0
4451	67	pelican eel	simcop2387-lap	f	0
4452	67	pelican gulper	simcop2387-lap	f	0
4454	67	pencilfish	simcop2387-lap	f	0
4455	67	pencilsmelt	simcop2387-lap	f	0
4456	67	perch	simcop2387-lap	f	0
4457	67	Peter's elephantnose fish	simcop2387-lap	f	0
4458	67	pickerel	simcop2387-lap	f	0
4459	67	pigfish	simcop2387-lap	f	0
4460	67	pike characid	simcop2387-lap	f	0
4476	67	plaice	simcop2387-lap	f	1
4462	67	pike eel	simcop2387-lap	f	0
4463	67	pike	simcop2387-lap	f	0
4464	67	pikeblenny	simcop2387-lap	f	0
4465	67	pikehead	simcop2387-lap	f	0
4466	67	pikeperch	simcop2387-lap	f	0
4467	67	pilchard	simcop2387-lap	f	0
4484	67	pollock	simcop2387-lap	f	1
4470	67	pink salmon	simcop2387-lap	f	0
4471	67	píntano	simcop2387-lap	f	0
4472	67	pipefish	simcop2387-lap	f	0
4474	67	pirarucu	simcop2387-lap	f	0
4475	67	pirate perch	simcop2387-lap	f	0
4477	67	platy	simcop2387-lap	f	0
4478	67	platyfish	simcop2387-lap	f	0
4480	67	plownose chimaera	simcop2387-lap	f	0
4481	67	plunderfish	simcop2387-lap	f	0
4482	67	poacher	simcop2387-lap	f	0
4483	67	pollyfish	simcop2387-lap	f	0
4485	67	pomfret	simcop2387-lap	f	0
4486	67	pompano	simcop2387-lap	f	0
4487	67	pompano dolphinfish	simcop2387-lap	f	0
4490	67	popeye catafula	simcop2387-lap	f	0
4491	67	porbeagle shark	simcop2387-lap	f	0
4492	67	porcupinefish	simcop2387-lap	f	0
4493	67	porgy	simcop2387-lap	f	0
4494	67	Port Jackson shark	simcop2387-lap	f	0
4495	67	powen	simcop2387-lap	f	0
4496	67	priapumfish	simcop2387-lap	f	0
4497	67	prickleback	simcop2387-lap	f	0
4498	67	pricklefish	simcop2387-lap	f	0
4499	67	prickly shark	simcop2387-lap	f	0
4500	67	prowfish	simcop2387-lap	f	0
4501	67	pufferfish	simcop2387-lap	f	0
4502	67	pumpkinseed	simcop2387-lap	f	0
4503	67	pupfish	simcop2387-lap	f	0
4504	67	pygmy sunfish	simcop2387-lap	f	0
4506	67	queen parrotfish	simcop2387-lap	f	0
4507	67	queen triggerfish	simcop2387-lap	f	0
4508	67	quillback	simcop2387-lap	f	0
4509	67	quillfish	simcop2387-lap	f	0
4510	67	rabbitfish	simcop2387-lap	f	0
4511	67	raccoon butterfly fish	simcop2387-lap	f	0
4512	67	ragfish	simcop2387-lap	f	0
4513	67	rainbow trout	simcop2387-lap	f	0
4514	67	rainbowfish	simcop2387-lap	f	0
4515	67	rasbora	simcop2387-lap	f	0
4516	67	ratfish	simcop2387-lap	f	0
4517	67	rattail	simcop2387-lap	f	0
4518	67	ray	simcop2387-lap	f	0
4519	67	razorback sucker	simcop2387-lap	f	0
4520	67	razorfish	simcop2387-lap	f	0
4522	67	redfin perch	simcop2387-lap	f	0
4523	67	redfish	simcop2387-lap	f	0
4524	67	redhorse sucker	simcop2387-lap	f	0
4479	67	pleco	simcop2387-lap	f	1
4505	67	queen danio	simcop2387-lap	f	1
4521	67	red snapper	simcop2387-lap	f	1
4488	67	ponyfish	simcop2387-lap	f	1
4489	67	poolfish	simcop2387-lap	f	1
4473	67	piranha	simcop2387-lap	f	1
4453	67	pencil catfish	simcop2387-lap	f	1
4418	67	orange roughy	simcop2387-lap	f	1
4525	67	redlip blenny	simcop2387-lap	f	0
4526	67	redmouth whalefish	simcop2387-lap	f	0
4527	67	redside	simcop2387-lap	f	0
4528	67	redtooth triggerfish	simcop2387-lap	f	0
4529	67	red velvetfish	simcop2387-lap	f	0
4530	67	red whalefish	simcop2387-lap	f	0
4531	67	reedfish	simcop2387-lap	f	0
4532	67	reef triggerfish	simcop2387-lap	f	0
4533	67	regal whiptail catfish	simcop2387-lap	f	0
4534	67	remora	simcop2387-lap	f	0
4535	67	requiem shark	simcop2387-lap	f	0
4536	67	ribbon eel	simcop2387-lap	f	0
4537	67	ribbon sawtail fish	simcop2387-lap	f	0
4539	67	ribbonfish	simcop2387-lap	f	0
4540	67	rice eel	simcop2387-lap	f	0
4541	67	ricefish	simcop2387-lap	f	0
4542	67	ridgehead	simcop2387-lap	f	0
4543	67	riffle dace	simcop2387-lap	f	0
4544	67	righteye flounder	simcop2387-lap	f	0
4546	67	river loach	simcop2387-lap	f	0
4547	67	river shark	simcop2387-lap	f	0
4548	67	river stingray	simcop2387-lap	f	0
4549	67	rivuline	simcop2387-lap	f	0
4550	67	roach	simcop2387-lap	f	0
4551	67	roanoke bass	simcop2387-lap	f	0
4552	67	rock bass	simcop2387-lap	f	0
4553	67	rock beauty	simcop2387-lap	f	0
4554	67	rock cod	simcop2387-lap	f	0
4555	67	rocket danio	simcop2387-lap	f	0
4556	67	rockfish	simcop2387-lap	f	0
4557	67	rockling	simcop2387-lap	f	0
4558	67	rockweed gunnel	simcop2387-lap	f	0
4619	67	scythe butterfish	simcop2387-lap	f	1
4560	67	ronquil	simcop2387-lap	f	0
4561	67	roosterfish	simcop2387-lap	f	0
4562	67	ropefish	simcop2387-lap	f	0
4563	67	rough pomfret	simcop2387-lap	f	0
4564	67	rough scad	simcop2387-lap	f	0
4566	67	roughy	simcop2387-lap	f	0
4567	67	roundhead	simcop2387-lap	f	0
4568	67	round herring	simcop2387-lap	f	0
4569	67	round stingray	simcop2387-lap	f	0
4570	67	round whitefish	simcop2387-lap	f	0
4571	67	rudd	simcop2387-lap	f	0
4572	67	rudderfish	simcop2387-lap	f	0
4573	67	ruffe	simcop2387-lap	f	0
4574	67	Russian sturgeon	simcop2387-lap	f	0
4575	67	sabalo	simcop2387-lap	f	0
4576	67	sabertooth	simcop2387-lap	f	0
4577	67	saber-toothed blenny	simcop2387-lap	f	0
4578	67	sabertooth fish	simcop2387-lap	f	0
4579	67	sablefish	simcop2387-lap	f	0
4581	67	Sacramento splittail	simcop2387-lap	f	0
4582	67	sailback scorpionfish	simcop2387-lap	f	0
4583	67	sailbearer	simcop2387-lap	f	0
4584	67	sailfin silverside	simcop2387-lap	f	0
4585	67	sailfish	simcop2387-lap	f	0
4586	67	salamanderfish	simcop2387-lap	f	0
4587	67	salmon	simcop2387-lap	f	0
4589	67	sandbar shark	simcop2387-lap	f	0
4590	67	sandburrower	simcop2387-lap	f	0
4591	67	sand dab	simcop2387-lap	f	0
4592	67	sanddiver	simcop2387-lap	f	0
4594	67	sandfish	simcop2387-lap	f	0
4595	67	sand goby	simcop2387-lap	f	0
4597	67	sand lance	simcop2387-lap	f	0
4598	67	sandperch	simcop2387-lap	f	0
4599	67	sandroller	simcop2387-lap	f	0
4600	67	sand stargazer	simcop2387-lap	f	0
4601	67	sand tiger	simcop2387-lap	f	0
4603	67	sarcastic fringehead	simcop2387-lap	f	0
4604	67	sardine	simcop2387-lap	f	0
4605	67	sargassum fish	simcop2387-lap	f	0
4606	67	sauger	simcop2387-lap	f	0
4607	67	saury	simcop2387-lap	f	0
4608	67	sawfish	simcop2387-lap	f	0
4609	67	saw shark	simcop2387-lap	f	0
4610	67	sawtooth eel	simcop2387-lap	f	0
4611	67	scabbard fish	simcop2387-lap	f	0
4612	67	scaleless black dragonfish	simcop2387-lap	f	0
4613	67	scaly dragonfish	simcop2387-lap	f	0
4615	67	scissor-tail rasbora	simcop2387-lap	f	0
4616	67	scorpionfish	simcop2387-lap	f	0
4617	67	sculpin	simcop2387-lap	f	0
4618	67	scup	simcop2387-lap	f	0
4620	67	sea bass	simcop2387-lap	f	0
4622	67	sea chub	simcop2387-lap	f	0
4623	67	seadevil	simcop2387-lap	f	0
4624	67	seadragon	simcop2387-lap	f	0
4625	67	seahorse	simcop2387-lap	f	0
4626	67	sea lamprey	simcop2387-lap	f	0
4627	67	seamoth	simcop2387-lap	f	0
4628	67	sea raven	simcop2387-lap	f	0
4629	67	searobin	simcop2387-lap	f	0
4630	67	sea snail	simcop2387-lap	f	0
4631	67	sea toad	simcop2387-lap	f	0
4632	67	Sevan trout	simcop2387-lap	f	0
4633	67	seatrout	simcop2387-lap	f	0
4634	67	sergeant major	simcop2387-lap	f	0
4635	67	shad	simcop2387-lap	f	0
4637	67	sharksucker	simcop2387-lap	f	0
4538	67	ribbonbearer	simcop2387-lap	f	1
4593	67	sand eel	simcop2387-lap	f	1
4614	67	scat	simcop2387-lap	f	1
4588	67	salmon shark	simcop2387-lap	f	1
4636	67	shark	simcop2387-lap	f	1
4602	67	sand tilefish	simcop2387-lap	f	1
4596	67	sand knifefish	simcop2387-lap	f	1
4545	67	Rio Grande perch	simcop2387-lap	f	1
4621	67	sea catfish	simcop2387-lap	f	1
4565	67	rough sculpin	simcop2387-lap	f	1
4638	67	sharpnose pufferfish	simcop2387-lap	f	0
4639	67	sheatfish	simcop2387-lap	f	0
4640	67	sheepshead	simcop2387-lap	f	0
4641	67	sheepshead minnow	simcop2387-lap	f	0
4642	67	shell-ear	simcop2387-lap	f	0
4643	67	shiner	simcop2387-lap	f	0
4644	67	shortnose chimaera	simcop2387-lap	f	0
4645	67	shortnose greeneye	simcop2387-lap	f	0
4646	67	shortnose sucker	simcop2387-lap	f	0
4647	67	shovelnose sturgeon	simcop2387-lap	f	0
4648	67	shrimpfish	simcop2387-lap	f	0
4649	67	Siamese fighting fish	simcop2387-lap	f	0
4650	67	sillago	simcop2387-lap	f	0
4651	67	silver carp	simcop2387-lap	f	0
4652	67	silver dollar	simcop2387-lap	f	0
4654	67	silver hake	simcop2387-lap	f	0
4655	67	silverside	simcop2387-lap	f	0
4657	67	sixgill ray	simcop2387-lap	f	0
4658	67	sixgill shark	simcop2387-lap	f	0
4660	67	skilfish	simcop2387-lap	f	0
4661	67	skipjack tuna	simcop2387-lap	f	0
4662	67	skipping goby	simcop2387-lap	f	0
4663	67	slender barracudina	simcop2387-lap	f	0
4665	67	slender snipe eel	simcop2387-lap	f	0
4666	67	sleeper	simcop2387-lap	f	0
4667	67	sleeper shark	simcop2387-lap	f	0
4668	67	slickhead	simcop2387-lap	f	0
4669	67	slimehead	simcop2387-lap	f	0
4670	67	slimy mackerel	simcop2387-lap	f	0
4671	67	slimy sculpin	simcop2387-lap	f	0
4672	67	slipmouth	simcop2387-lap	f	0
4673	67	smalleye squaretail	simcop2387-lap	f	0
4674	67	smalltooth sawfish	simcop2387-lap	f	0
4675	67	smelt	simcop2387-lap	f	0
4676	67	smelt-whiting	simcop2387-lap	f	0
4677	67	smooth dogfish	simcop2387-lap	f	0
4679	67	snailfish	simcop2387-lap	f	0
4680	67	snake eel	simcop2387-lap	f	0
4681	67	snakehead	simcop2387-lap	f	0
4682	67	snake mackerel	simcop2387-lap	f	0
4684	67	snapper	simcop2387-lap	f	0
4685	67	snipe eel	simcop2387-lap	f	0
4686	67	snipefish	simcop2387-lap	f	0
4687	67	snoek	simcop2387-lap	f	0
4688	67	snook	simcop2387-lap	f	0
4689	67	snubnose eel	simcop2387-lap	f	0
4690	67	snubnose parasitic eel	simcop2387-lap	f	0
4691	67	soapfish	simcop2387-lap	f	0
4692	67	sockeye salmon	simcop2387-lap	f	0
4693	67	soldierfish	simcop2387-lap	f	0
4694	67	sole	simcop2387-lap	f	0
4695	67	South American darter	simcop2387-lap	f	0
4696	67	South American Lungfish	simcop2387-lap	f	0
4697	67	southern Dolly Varden	simcop2387-lap	f	0
4698	67	southern flounder	simcop2387-lap	f	0
4699	67	southern grayling	simcop2387-lap	f	0
4700	67	southern hake	simcop2387-lap	f	0
4702	67	southern smelt	simcop2387-lap	f	0
4703	67	spadefish	simcop2387-lap	f	0
4704	67	spaghetti eel	simcop2387-lap	f	0
4705	67	Spanish mackerel	simcop2387-lap	f	0
4706	67	spearfish	simcop2387-lap	f	0
4709	67	spikefish	simcop2387-lap	f	0
4710	67	spinefoot	simcop2387-lap	f	0
4711	67	spiny-back	simcop2387-lap	f	0
4653	67	silver driftfish	simcop2387-lap	f	1
4713	67	spiny dogfish	simcop2387-lap	f	0
4714	67	spiny dwarf catfish	simcop2387-lap	f	0
4715	67	spiny eel	simcop2387-lap	f	0
4716	67	spinyfin	simcop2387-lap	f	0
4718	67	spookfish	simcop2387-lap	f	0
4719	67	spotted danio	simcop2387-lap	f	0
4721	67	sprat	simcop2387-lap	f	0
4723	67	squarehead catfish	simcop2387-lap	f	0
4724	67	squaretail	simcop2387-lap	f	0
4725	67	squawfish	simcop2387-lap	f	0
4726	67	squeaker	simcop2387-lap	f	0
4727	67	squirrelfish	simcop2387-lap	f	0
4728	67	staghorn sculpin	simcop2387-lap	f	0
4729	67	stargazer	simcop2387-lap	f	0
4730	67	starry flounder	simcop2387-lap	f	0
4731	67	steelhead	simcop2387-lap	f	0
4732	67	stickleback	simcop2387-lap	f	0
4733	67	stingfish	simcop2387-lap	f	0
4734	67	stingray	simcop2387-lap	f	0
4735	67	stonecat	simcop2387-lap	f	0
4736	67	stonefish	simcop2387-lap	f	0
4737	67	stoneroller minnow	simcop2387-lap	f	0
4738	67	straptail	simcop2387-lap	f	0
4739	67	stream catfish	simcop2387-lap	f	0
4707	67	speckled trout	simcop2387-lap	f	1
4741	67	striped bass	simcop2387-lap	f	0
4742	67	striped burrfish	simcop2387-lap	f	0
4743	67	sturgeon	simcop2387-lap	f	0
4744	67	sucker	simcop2387-lap	f	0
4745	67	suckermouth armored catfish	simcop2387-lap	f	0
4746	67	summer flounder	simcop2387-lap	f	0
4747	67	Sundaland noodlefish	simcop2387-lap	f	0
4748	67	sunfish 	simcop2387-lap	f	0
4749	67	sunfish (mola mola)	simcop2387-lap	f	0
4656	67	sind danio	simcop2387-lap	f	1
4720	67	spotted dogfish	simcop2387-lap	f	1
4664	67	slender mola	simcop2387-lap	f	1
4717	67	splitfin	simcop2387-lap	f	1
4701	67	southern sandfish	simcop2387-lap	f	1
4708	67	spiderfish	simcop2387-lap	f	1
4678	67	smoothtongue	simcop2387-lap	f	1
4659	67	skate	simcop2387-lap	f	1
4722	67	springfish	simcop2387-lap	f	1
4683	67	snake mudhead	simcop2387-lap	f	1
4750	67	surf sardine	simcop2387-lap	f	0
4751	67	surfperch	simcop2387-lap	f	0
4752	67	surgeonfish	simcop2387-lap	f	0
4753	67	swallower	simcop2387-lap	f	0
4754	67	swamp-eel	simcop2387-lap	f	0
4755	67	swampfish	simcop2387-lap	f	0
4756	67	sweeper	simcop2387-lap	f	0
4757	67	swordfish	simcop2387-lap	f	0
4758	67	swordtail	simcop2387-lap	f	0
4759	67	tadpole cod	simcop2387-lap	f	0
4760	67	tadpole fish	simcop2387-lap	f	0
4761	67	tailor	simcop2387-lap	f	0
4762	67	taimen	simcop2387-lap	f	0
4763	67	tang	simcop2387-lap	f	0
4764	67	tapetail	simcop2387-lap	f	0
4801	67	torrent catfish	simcop2387-lap	f	1
4766	67	tarwhine	simcop2387-lap	f	0
4767	67	telescopefish	simcop2387-lap	f	0
4768	67	temperate bass	simcop2387-lap	f	0
4769	67	temperate ocean-bass	simcop2387-lap	f	0
4770	67	temperate perch	simcop2387-lap	f	0
4771	67	tench	simcop2387-lap	f	0
4772	67	tenpounder	simcop2387-lap	f	0
4773	67	tenuis	simcop2387-lap	f	0
4774	67	tetra	simcop2387-lap	f	0
4775	67	thorny catfish	simcop2387-lap	f	0
4776	67	thornfish	simcop2387-lap	f	0
4778	67	threadfin	simcop2387-lap	f	0
4779	67	threadfin bream	simcop2387-lap	f	0
4780	67	threadsail	simcop2387-lap	f	0
4781	67	threadtail	simcop2387-lap	f	0
4782	67	three spot gourami	simcop2387-lap	f	0
4783	67	threespine stickleback	simcop2387-lap	f	0
4784	67	three-toothed puffer	simcop2387-lap	f	0
4785	67	thresher shark	simcop2387-lap	f	0
4786	67	tidewater goby	simcop2387-lap	f	0
4787	67	tiger barb	simcop2387-lap	f	0
4788	67	tigerperch	simcop2387-lap	f	0
4789	67	tiger shark	simcop2387-lap	f	0
4790	67	tiger shovelnose catfish	simcop2387-lap	f	0
4791	67	tilapia	simcop2387-lap	f	0
4793	67	titan triggerfish	simcop2387-lap	f	0
4794	67	toadfish	simcop2387-lap	f	0
4795	67	tommy ruff	simcop2387-lap	f	0
4796	67	tompot blenny	simcop2387-lap	f	0
4797	67	tonguefish	simcop2387-lap	f	0
4798	67	tope	simcop2387-lap	f	0
4799	67	topminnow	simcop2387-lap	f	0
4800	67	torpedo	simcop2387-lap	f	0
4802	67	torrent fish	simcop2387-lap	f	0
4803	67	trahira	simcop2387-lap	f	0
4804	67	treefish	simcop2387-lap	f	0
4805	67	trevally	simcop2387-lap	f	0
4806	67	triggerfish	simcop2387-lap	f	0
4807	67	triplefin blenny	simcop2387-lap	f	0
4808	67	triplespine	simcop2387-lap	f	0
4809	67	tripletail	simcop2387-lap	f	0
4810	67	tripod fish	simcop2387-lap	f	0
4811	67	trout	simcop2387-lap	f	0
4812	67	trout cod	simcop2387-lap	f	0
4813	67	trout-perch	simcop2387-lap	f	0
4814	67	trumpeter	simcop2387-lap	f	0
4815	67	trumpetfish	simcop2387-lap	f	0
4816	67	trunkfish	simcop2387-lap	f	0
4817	67	tubeblenny	simcop2387-lap	f	0
4818	67	tube-eye	simcop2387-lap	f	0
4819	67	tube-snout	simcop2387-lap	f	0
4820	67	tubeshoulder	simcop2387-lap	f	0
4821	67	tui chub	simcop2387-lap	f	0
4822	67	tuna	simcop2387-lap	f	0
4823	67	turbot	simcop2387-lap	f	0
4824	67	turkeyfish	simcop2387-lap	f	0
4825	67	unicornfish	simcop2387-lap	f	0
4826	67	upside-down catfish	simcop2387-lap	f	0
4827	67	velvet-belly shark	simcop2387-lap	f	0
4828	67	velvet catfish	simcop2387-lap	f	0
4829	67	velvetfish	simcop2387-lap	f	0
4830	67	vendace	simcop2387-lap	f	0
4831	67	vimba	simcop2387-lap	f	0
4832	67	viperfish	simcop2387-lap	f	0
4833	67	wahoo	simcop2387-lap	f	0
4834	67	walking catfish	simcop2387-lap	f	0
4835	67	wallago	simcop2387-lap	f	0
4836	67	walleye	simcop2387-lap	f	0
4837	67	walleye pollock	simcop2387-lap	f	0
4838	67	walu	simcop2387-lap	f	0
4839	67	warbonnet	simcop2387-lap	f	0
4840	67	warmouth	simcop2387-lap	f	0
4841	67	warty angler	simcop2387-lap	f	0
4842	67	waryfish	simcop2387-lap	f	0
4843	67	wasp fish	simcop2387-lap	f	0
4844	67	weasel shark	simcop2387-lap	f	0
4845	67	weatherfish	simcop2387-lap	f	0
4846	67	weever	simcop2387-lap	f	0
4847	67	weeverfish	simcop2387-lap	f	0
4848	67	wels catfish	simcop2387-lap	f	0
4849	67	whale catfish	simcop2387-lap	f	0
4850	67	whalefish	simcop2387-lap	f	0
4851	67	whale shark	simcop2387-lap	f	0
4852	67	whiff	simcop2387-lap	f	0
4853	67	whiptail gulper	simcop2387-lap	f	0
4854	67	whitebait	simcop2387-lap	f	0
4855	67	white croaker	simcop2387-lap	f	0
4856	67	whitefish	simcop2387-lap	f	0
4857	67	white marlin	simcop2387-lap	f	0
4858	67	white shark	simcop2387-lap	f	0
4859	67	whitetip reef shark	simcop2387-lap	f	0
4860	67	whiting	simcop2387-lap	f	0
4861	67	wobbegong	simcop2387-lap	f	0
4862	67	wolf-eel	simcop2387-lap	f	0
4863	67	wolffish	simcop2387-lap	f	0
4864	67	wolf-herring	simcop2387-lap	f	0
4777	67	thornyhead	simcop2387-lap	f	1
4792	67	tilefish	simcop2387-lap	f	1
4865	67	woody sculpin	simcop2387-lap	f	0
4866	67	worm eel	simcop2387-lap	f	0
4867	67	wormfish	simcop2387-lap	f	0
4868	67	wrasse	simcop2387-lap	f	0
4870	67	x-ray tetra	simcop2387-lap	f	0
4871	67	yellow-and-black triplefin	simcop2387-lap	f	0
4872	67	yellowbanded perch	simcop2387-lap	f	0
4873	67	yellow bass	simcop2387-lap	f	0
4875	67	yellow-eye mullet	simcop2387-lap	f	0
4879	67	yellowfin grouper	simcop2387-lap	f	0
4880	67	yellowfin pike	simcop2387-lap	f	0
4881	67	yellowfin surgeonfish	simcop2387-lap	f	0
4882	67	yellowfin tuna	simcop2387-lap	f	0
4883	67	yellow jack	simcop2387-lap	f	0
4884	67	yellowmargin triggerfish	simcop2387-lap	f	0
4885	67	yellow moray	simcop2387-lap	f	0
4886	67	yellow perch	simcop2387-lap	f	0
4887	67	yellowtail	simcop2387-lap	f	0
4888	67	yellowtail amberjack	simcop2387-lap	f	0
4889	67	yellowtail barracuda	simcop2387-lap	f	0
4890	67	yellowtail clownfish	simcop2387-lap	f	0
4891	67	yellowtail horse mackerel	simcop2387-lap	f	0
4892	67	yellowtail kingfish	simcop2387-lap	f	0
4893	67	yellowtail snapper	simcop2387-lap	f	0
4894	67	yellow tang	simcop2387-lap	f	0
4895	67	yellow weaver	simcop2387-lap	f	0
4896	67	zander	simcop2387-lap	f	0
4897	67	zebra bullhead shark	simcop2387-lap	f	0
4898	67	zebra danio	simcop2387-lap	f	0
4899	67	zebrafish	simcop2387-lap	f	0
4900	67	zebra lionfish	simcop2387-lap	f	0
4902	67	zebra oto	simcop2387-lap	f	0
4903	67	zebra pleco	simcop2387-lap	f	0
4904	67	zebra shark	simcop2387-lap	f	0
4905	67	zebra tilapia	simcop2387-lap	f	0
4906	67	ziege	simcop2387-lap	f	0
4907	67	zingel	simcop2387-lap	f	0
3725	17	For episode downloads use the XDCC bots, or !MU for megaupload links, or ! plus the episode number to get a link directly (e.g !1, !2, !530, etc.).	Huty	t	441
4461	67	pike conger	simcop2387-lap	f	1
4559	67	rohu	simcop2387-lap	f	1
4869	67	wrymouth	simcop2387-lap	f	1
4877	67	yellowfin croaker	simcop2387-lap	f	1
3543	11	[18:54] <~jmaeshawn> it is bad and merged the red background into her [18:54] <~Kaitoukidsama> I'm sure that's not all you merged into her 	Jing	f	34
4740	67	streamer fish	simcop2387-lap	f	1
4876	67	yellowhead jawfish	simcop2387-lap	f	1
4901	67	zebra loach	simcop2387-lap	f	1
1618	9	"Even a huge wall becomes a large door if u change a way to go out." [ANFS # 443]	simcop2387	f	37
4874	67	yellow-edged moray	simcop2387-lap	f	1
1615	9	Jd: And download Religulous--it's a comedy documentary starring Bill Maher that covers why religion is so dangerous for humanity: http://www.mininova.org/tor/2225295	simcop2387	f	38
4878	67	yellowfin cutthroat trout	simcop2387-lap	f	1
4236	67	huchen	simcop2387-lap	f	1
4186	67	graveldiver	simcop2387-lap	f	1
4911	68	"Typhoon Rips Through Cemetery; Hundreds Dead"	jmaeshawn	f	5
4923	68	"DOE to do NEPA's EIS on BNFL's AMWTP at INEEL after SRA protest"	jmaeshawn	f	7
3929	67	boxfish	simcop2387-lap	f	1
4915	68	"Alton Attorney Accidently Sues Himself"	jmaeshawn	f	8
4913	68	"Congress Enacts Ban On Soliciting Dead - Voters Protest"	jmaeshawn	f	7
4055	67	diver: New Zealand sand diver or Long-finned sand diver	simcop2387-lap	f	1
3091	32	0821 Simple Mail Transfer Protocol J. Postel [ August 1982 ] ( TXT = 124482 bytes)(Obsoletes RFC0788) (Obsoleted by RFC2821) (Also STD0010) (Status: STANDARD)	shiranpuri	f	2
1742	27	Love is a verb... and Verbs show action	simcop2387	f	3
4909	68	"Internet Nominated For 2010 Nobel Peace Prize"	Zdm321	f	4
4908	68	"Turns Out You Actually Can Be Bored To Death"	Zdm321	f	5
3533	39	http://www.dctp.ws/DC_Subbed_List.pdf	Xoph	f	441
4916	68	"Caskets Found as Workers Demolish Mausoleum -- 'We had no idea anyone was buried there,' workers replied."	jmaeshawn	f	6
4912	68	"Enraged Cow Injures Farmer with Axe"	jmaeshawn	f	7
-6013	-6013	4Detective Conan Movie 13 3:: 4XviD Part 1: 2http://usershare.net/uqlv0pb39vwe http://www.megaupload.com/?d=Y1DH1LPI 3:: 4XviD Part 2: 2http://usershare.net/0hi2emt0byiy http://www.megaupload.com/?d=TZRWH16L H264-AC3: [need winrar] http://www.megaupload.com/?d=KU19K9MJ http://www.megaupload.com/?d=NUBUQY12 http://www.megaupload.com/?d=WNXB4VTZ	\N	f	366
4919	68	"Tiger Woods Plays with Own Balls, Nike says"	jmaeshawn	f	7
4922	68	"Federal Agents Raid Gun Shop, Find Weapons"	jmaeshawn	f	6
4920	68	"Volunteers search for old Civil War planes"	jmaeshawn	f	4
4914	68	"If Strike isn't Settled Quickly, It May Last a While"	jmaeshawn	f	5
4921	68	"Meeting on open meetings is closed"	jmaeshawn	f	6
4917	68	"County to Pay $250,000 to Advertise its Lack of Funds"	jmaeshawn	f	3
4910	68	"Microsoft announces it has perfected its newest version of Windows so it crashes BEFORE installation is completed."	Zdm321	f	7
4918	68	"Local 19-Month-Old Child Wins Rifle from Fundraiser"	jmaeshawn	f	7
5260	50	%NICK% breaks things like Techman_16	Vylen	f	34
4924	9	"People's hand does not reach, so merely has the worship." Actual song lyrics from Clannad After Story	simcop2387-lap	f	38
3604	50	%NICK% should crawl in a corner and die.	Atticus|AFK	f	32
5261	50	%NICK% has never played FFVII	Vylen	f	34
5254	50	%NICK% likes BLEACH.	MasterO	f	33
5264	50	%NICK% has no potential in life	Vylen	f	34
1696	11	[23:43:06] <simcop2387> Zdm321: playing tennis with a spherical duck (mallard in this case) in vacuum gives the duck 2000 joules of kinetic energy [23:44:17] * Jing has no clue what you're talking about, got a B- in Chemistry [23:44:51] <MrWiz88> thats cuz kinetic energy is physics	simcop2387	f	34
3677	11	<@kyuuketsuki> not everything can be like DBZ   <@MrWiz88> KAAAAAAAAAA MAAAAAAAY HAAAAAAA MAAAAAAAYYYY  *commercial break*   HAAAAAAAAAAAAAAAAAAAAA	MrWiz88	f	34
5263	50	%NICK% has manboobs	Vylen	f	34
5262	50	%NICK% is overrated	Vylen	f	34
5258	50	%NICK% was touched by JD_SUCKS	Vylen	f	34
5253	50	%NICK% is a fan of Naruto	Vylen	f	33
5255	50	%NICK% looks like JD_SUCKS	Vylen	f	34
5259	50	%NICK% fails more than Zdm321	JD_SUCKS	f	34
5256	50	%NICK% is just like Huty	Vylen	f	35
5252	82	%NICK% is bonked on the head with a vuvuzela	Vylen	f	16
5249	82	4Bzzzzzzzzzzzzzzzzzzzzzzzzz	Vylen	f	13
5250	82	Bzz12zzz14zzz11zzz9zzz13zzz7zzz6zzz4zzz8zzz	Vylen	f	16
5257	50	Zdm321 will crush %NICK% like a tiny flea inside a paper cup	Vylen	f	33
5251	82	Bzz2zzz3zzz4zzz5zzz6zzz7zzz8zzz9zzz10zzz11zzz12zzz13zzz14zzz15zzz	Vylen	f	14
5354	94	http://www.hidden-3d.com/games_stereogram_tetris.php	____	f	7
5353	94	Stereogram tetris: http://www.hidden-3d.com/games_stereogram_tetris.php	____	t	1
5265	50	%NICK% couldn't deduce their way out of a wet paper bag	Vylen	f	32
5268	50	When %NICK% boards a train, it can't go anywhere since they're so fat	Vylen	f	34
5267	50	%NICK% is a narutard	JD_SUCKS	f	34
5266	50	%NICK% is worse than Kariyu	Vylen	f	34
5282	14	Ran is actually the love-child of Eri and the boss of the Black Organisation.	Vylen	f	66
1689	3	<Jd> I just would never post anything remotely heartfelt on YouTube, because you'd get a reply that's like "have debt problems? see WWW.GAMBLERSPARADISE.COM for the answer to all your problems!!!"	Jing	f	34
5223	9	Its the first time i see a athtube like this	simcop2387	f	33
4927	14	Ran does not have any feet, this is why they never show below her shins	simcop2387-lap	f	66
5330	59	http://oi56.tinypic.com/207nuhh.jpg	shiranpuri	f	12
5332	78	http://tinyurl.com/3xpt5za	shiranpuri	f	4
5331	78	http://frankandjan.com/WIT/wp-content/uploads/2010/08/cut-paste-undo.jpg	shiranpuri	t	4
5333	78	http://tinyurl.com/32ap4ar	shiranpuri	f	5
3714	59	http://tinyurl.com/mvkztt	shiranpuri	f	20
4933	68	Psychic convention cancelled due to unforseen circumstances.	Null^	f	5
4934	14	Not only does Megure's magical hat stay on his head no matter what, it also is impact-proof (maybe even bullet-proof, but that has yet to be confirmed).	Null^	f	66
103	14	Kudo Shinichi is really a huge coke fiend, that was the real reason he went and followed the men in black, he was looking to score some and then do lines off Ran's back	simcop2387	f	66
4932	68	Psychic convention canceled due to unforseen circumstances.	Null^	t	3
4935	1	Kogoro-chan used to be the school's naughtiest boy.	Null^	t	65
5226	14	If Vodka were to ever die, Gin would go berzerk and ... join the clergy.	Abs-	f	65
5225	14	Agasa and Gin are one and the same! You never see them both in the same place at the same time. He just switches cars.	Abs-	f	66
5227	14	The final showdown will not be Conan or Shinichi vs. the B.O. boss, but rather the Night Baroness vs. Kamen Yaiba.	Abs-	f	66
5224	14	Gin is Ayumi's father.	Abs-	f	66
5012	1	Conan sees dead people.	Abs-	f	66
5000	14	Gin, Vodka, Vermouth, Chianti, and Korn will be revealed to be future!Conan, future!Mitsuhiko, future!Haibara, future!Ayumi, and future!Genta.	Abs-	t	59
5008	70	Abs- is your mom.	Abs-	f	0
5004	14	The B.O. Boss is Gomera.	Abs-	f	65
5003	14	Ran and Yamamura will eventually merge to form Ranmamura.	Abs-	f	66
5010	14	If it weren't for the organization's dress code, Gin's entire wardrobe would be hot pink.	Abs-	f	66
5001	14	Gin, Vodka, Vermouth, Chianti, and Korn will be revealed to be future!Conan, future!Genta, future!Haibara, future!Ayumi, and future!Mitsuhiko. 	Abs-	f	66
5009	14	Haibara will win 100,000,000,000,000 yen if she actually gets Conan to shit a brick.	Abs-	f	66
5005	14	Actually Ayumi is only acting nice to Conan because she wants to get in good with the real object of her crush: Haibara.	Abs-	f	66
5007	14	Goro and Gin have had a past secret relationship.	Abs-	f	66
5006	14	Takagi's never dated before. Women, that is.	Abs-	f	66
5002	14	That guy wasn't just Chiba's roommate.	Abs-	f	67
5013	78	Before you die, there is something you should know about us, %NICK%. I am your father's brother's nephew's cousin's former roommate.	Sazaai`Aries	f	5
3546	44	HELO KEPKEP	shiranpuri	f	106
5011	78	Before you die, there is something you should know about us, %NICK. I am your father's brother's nephew's cousin's former roommate.	Sazaai`Aries	t	2
5231	14	The sixth member of the DB will be named Ginny, and will have an unhealthy obsession with killing things.	Abs-	f	66
5230	14	Mitsuhiko's sister will end up marrying Gin.	Abs-	f	66
5228	14	The antidote is a lie.	Abs-	f	66
5015	3	Jd-, Jing, Kamikazeeee, Zdm321, JD_SUCKS, and Abs- all live in the same house.	Null^	f	33
5014	14	Gin's Porsche actually breaks down as often as Agasa's Beetle does, but we don't get to see his and Vodka's wacky adventures.	Abs-	f	66
5064	52	[Track 11.] KOBUSHIWONIGIRE [Album: ELEVEN]	B`z	f	2
5016	52	test	Null^	t	1
5017	52	[Track 12.] Go★Fight★Win [Album: GREEN]	B`z	t	1
5018	52	[Track 01.] spirit loose [Album: LOOSE]	B`z	f	1
5019	52	[Track 02.] THE LOOSE [Album: LOOSE]	B`z	f	1
5020	52	[Track 03.] Negai ("BUZZ!!" STYLE) [Album: LOOSE]	B`z	f	1
5021	52	[Track 04.] Yumemigaoka [Album: LOOSE]	B`z	f	1
5022	52	[Track 05.] BAD COMMUNICATION(000-18) [Album: LOOSE]	B`z	f	1
5023	52	[Track 06.] Kienai Niji [Album: LOOSE]	B`z	f	1
5024	52	[Track 07.] love me, I love you (with G Bass) [Album: LOOSE]	B`z	f	1
5025	52	[Track 08.] LOVE PHANTOM [Album: LOOSE]	B`z	f	1
5026	52	[Track 09.] Teaki ga Inakerya [Album: LOOSE]	B`z	f	1
4765	67	tarpon	simcop2387-lap	f	1
5028	52	[Track 11.] Kirei na Ai jya Nakutemo [Album: LOOSE]	B`z	f	1
5029	52	[Track 12.] Big [Album: LOOSE]	B`z	f	1
5030	52	[Track 13.] drive to MY WORLD [Album: LOOSE]	B`z	f	1
5031	52	[Track 01.] Deep Kiss [Album: SURVIVE]	B`z	f	1
5032	52	[Track 02.] Suima-Yo!! [Album: SURVIVE]	B`z	f	1
5033	52	[Track 03.] Survive [Album: SURVIVE]	B`z	f	1
5034	52	[Track 04.] Liar Liar [Album: SURVIVE]	B`z	f	1
5037	52	[Track 07.] Do Me [Album: SURVIVE]	B`z	f	1
5038	52	[Track 08.] Naite Naite Nakiyandara [Album: SURVIVE]	B`z	f	1
5039	52	[Track 09.] Cat [Album: SURVIVE]	B`z	f	1
5040	52	[Track 10.] Dattara Agechaeyo [Album: SURVIVE]	B`z	f	1
5042	52	[Track 12.] Calling [Album: SURVIVE]	B`z	f	1
5043	52	[Track 01.] F・E・A・R [Album: Brotherhood]	B`z	f	1
5044	52	[Track 02.] GIRIGIRI Chop(Version 51) [Album: Brotherhood]	B`z	f	1
5045	52	[Track 03.] Brotherhood [Album: Brotherhood]	B`z	f	1
5046	52	[Track 04.] Nagai Ai [Album: Brotherhood]	B`z	f	1
5047	52	[Track 05.] Yume no Youna Hibi [Album: Brotherhood]	B`z	f	1
5049	52	[Track 07.] Sono Te de Furete Goran [Album: Brotherhood]	B`z	f	1
5050	52	[Track 08.] Nagara Yuku Hibi [Album: Brotherhood]	B`z	f	1
5051	52	[Track 09.] SKIN [Album: Brotherhood]	B`z	f	1
5052	52	[Track 10.] Ikasete Okure! [Album: Brotherhood]	B`z	f	1
5054	52	[Track 01.] I [Album: ELEVEN]	B`z	f	1
5055	52	[Track 02.] Seventh Heaven [Album: ELEVEN]	B`z	f	1
5056	52	[Track 03.] Shinjiru Kurai li Darou [Album: ELEVEN]	B`z	f	1
5057	52	[Track 04.] RING [Album: ELEVEN]	B`z	f	1
5058	52	[Track 05.] Ai no prisoner [Album: ELEVEN]	B`z	f	1
5060	52	[Track 07.] May [Album: ELEVEN]	B`z	f	1
5061	52	[Track 08.] juice(PM mix) [Album: ELEVEN]	B`z	f	1
5062	52	[Track 09.] Raging River [Album: ELEVEN]	B`z	f	1
5063	52	[Track 10.] TOKYO DEVIL [Album: ELEVEN]	B`z	f	1
5059	52	[Track 06.] Kirameku Hito [Album: ELEVEN]	B`z	f	2
5067	52	[Track 14.] Konya Tsuki no Mieru Oka ni (Alternative Guitar Solo ver.） [Album: ELEVEN]	B`z	f	1
5069	52	[Track 02.] Atsuki Kodō no Hate [Album: GREEN]	B`z	f	1
5071	52	[Track 04.] Signal [Album: GREEN]	B`z	f	1
5072	52	[Track 05.] Surfin' 3000GTR [Album: GREEN]	B`z	f	1
5074	52	[Track 07.] Ultra Soul [Album: GREEN]	B`z	f	1
5075	52	[Track 08.] Utsukushiki Sekai [Album: GREEN]	B`z	f	1
5076	52	[Track 09.] Everlasting [Album: GREEN]	B`z	f	1
5077	52	[Track 10.] Forever Mine [Album: GREEN]	B`z	f	1
5048	52	[Track 06.] Gin no Tsubasa de Tobe [Album: Brotherhood]	B`z	f	2
5080	52	[Track 01.] Arakure [Album: BIG MACHINE]	B`z	f	1
5081	52	[Track 02.] Yasei no ENERGY [Album: BIG MACHINE]	B`z	f	1
5083	52	[Track 04.] Hakanai Diamond [Album: BIG MACHINE]	B`z	f	1
5084	52	[Track 05.] I'm In Love [Album: BIG MACHINE]	B`z	f	1
5085	52	[Track 06.] It's Showtime [Album: BIG MACHINE]	B`z	f	1
5086	52	[Track 07.] Ai to Nikushimi no Hajimari [Album: BIG MACHINE]	B`z	f	1
5088	52	[Track 09.] Nightbird [Album: BIG MACHINE]	B`z	f	1
5089	52	[Track 10.] Bluesy na Asa [Album: BIG MACHINE]	B`z	f	1
5090	52	[Track 11.] Mabushii Sign [Album: BIG MACHINE]	B`z	f	1
5091	52	[Track 12.] Change The Future [Album: BIG MACHINE]	B`z	f	1
5092	52	[Track 13.] Roots [Album: BIG MACHINE]	B`z	f	1
5093	52	[Track 01.] The Circle [Album: THE CIRCLE]	B`z	f	1
5094	52	[Track 02.] x [Album: THE CIRCLE]	B`z	f	1
5096	52	[Track 04.] Ai no Bakudan [Album: THE CIRCLE]	B`z	f	1
5099	52	[Track 07.] Suiren [Album: THE CIRCLE]	B`z	f	1
5100	52	[Track 08.] Sanctuary [Album: THE CIRCLE]	B`z	f	1
5097	52	[Track 05.] Fly The Flag [Album: THE CIRCLE]	B`z	f	2
5073	52	[Track 06.] Blue Sunshine [Album: GREEN]	B`z	f	2
5068	52	[Track 01.] STAY GREEN ~Mijyuku na Tabi wa Tomaranai~ [Album: GREEN]	B`z	f	2
5065	52	[Track 12.] Thinking of You [Album: ELEVEN]	B`z	f	2
5098	52	[Track 06.] AQUA BLUE [Album: THE CIRCLE]	B`z	f	2
5041	52	[Track 11.] Shower [Album: SURVIVE]	B`z	f	2
5087	52	[Track 08.] Big Machine [Album: BIG MACHINE]	B`z	f	2
5035	52	[Track 05.] HAPPINESS [Album: SURVIVE]	B`z	f	2
5078	52	[Track 11.] The Spiral [Album: GREEN]	B`z	f	2
5082	52	[Track 03.] Wake up, right now [Album: BIG MACHINE]	B`z	f	2
5053	52	[Track 11.] SHINE [Album: Brotherhood]	B`z	f	2
5095	52	[Track 03.] PULSE [Album: THE CIRCLE]	B`z	f	2
5070	52	[Track 03.] Warp [Album: GREEN]	B`z	f	2
5175	52	[Track 06.] Native Dance [Album: RUN]	B`z	f	2
5101	52	[Track 09.] Fever [Album: THE CIRCLE]	B`z	f	1
5102	52	[Track 10.] Shiroi Hibana [Album: THE CIRCLE]	B`z	f	1
5103	52	[Track 11.] ICARUS [Album: THE CIRCLE]	B`z	f	1
5104	52	[Track 12.] Black And White [Album: THE CIRCLE]	B`z	f	1
5105	52	[Track 13.] Brighter Day [Album: THE CIRCLE]	B`z	f	1
5106	52	[Track 01.] ALL-OUT ATTACK [Album: MONSTER]	B`z	f	1
5107	52	[Track 02.] Splash [Album: MONSTER]	B`z	f	1
5108	52	[Track 03.] Yuruginai Mono Hitotsu [Album: MONSTER]	B`z	f	1
5109	52	[Track 04.] Kog no SUMMER SESSION [Album: MONSTER]	B`z	f	1
5110	52	[Track 05.] KEMURI no Sekai [Album: MONSTER]	B`z	f	1
5111	52	[Track 06.] Shoudou ~MONSTER MiX~ [Album: MONSTER]	B`z	f	1
5112	52	[Track 07.] Mugon no Promise [Album: MONSTER]	B`z	f	1
5113	52	[Track 08.] MONSTER [Album: MONSTER]	B`z	f	1
5114	52	[Track 09.] NETEMOSAMETEMO [Album: MONSTER]	B`z	f	1
5115	52	[Track 10.] Happy Birthday [Album: MONSTER]	B`z	f	1
5116	52	[Track 11.] PIERROT [Album: MONSTER]	B`z	f	1
5117	52	[Track 12.] Amadare Blues [Album: MONSTER]	B`z	f	1
5118	52	[Track 13.] Ashita Mata Hi ga Noboru Nara [Album: MONSTER]	B`z	f	1
5119	52	[Track 14.] OCEAN~2006 MiX~ [Album: MONSTER]	B`z	f	1
5120	52	[Track 01.] Junjou Action [Album: ACTION]	B`z	f	1
5121	52	[Track 02.] Kuroi Seishun [Album: ACTION]	B`z	f	1
5122	52	[Track 03.] SUPER　LOVE　SONG [Album: ACTION]	B`z	f	1
5123	52	[Track 04.] Mangetsu yo Terase [Album: ACTION]	B`z	f	1
5124	52	[Track 05.] PERFECT LIFE [Album: ACTION]	B`z	f	1
5125	52	[Track 06.] Isshinfuran [Album: ACTION]	B`z	f	1
5126	52	[Track 07.] FRICTION -LAP2 [Album: ACTION]	B`z	f	1
5128	52	[Track 09.] Boku ni wa Kimi ga iru [Album: ACTION]	B`z	f	2
5127	52	[Track 08.] ONE　ON　ONE [Album: ACTION]	B`z	f	1
5129	52	[Track 10.] Nanto iu Shiawase [Album: ACTION]	B`z	f	1
5130	52	[Track 11.] Warui Yume [Album: ACTION]	B`z	f	1
5132	52	[Track 13.] Koubou [Album: ACTION]	B`z	f	1
5133	52	[Track 14.] Travelin' Men no Theme [Album: ACTION]	B`z	f	1
5135	52	[Track 16.] Eien no Tsubasa [Album: ACTION]	B`z	f	1
5136	52	[Track 17.] Buddy [Album: ACTION]	B`z	f	1
5137	52	[Track 01.] Introduction [Album: MAGIC]	B`z	f	1
5138	52	[Track 02.] Dive [Album: MAGIC]	B`z	f	1
5139	52	[Track 03.] Time Flies [Album: MAGIC]	B`z	f	1
5140	52	[Track 04.] MY LONELY TOWN [Album: MAGIC]	B`z	f	1
5141	52	[Track 05.] Long Time No See [Album: MAGIC]	B`z	f	1
5142	52	[Track 06.] Ichibu no Zenbu [Album: MAGIC]	B`z	f	1
5143	52	[Track 07.] Pray [Album: MAGIC]	B`z	f	1
5144	52	[Track 08.] Magic [Album: MAGIC]	B`z	f	1
5146	52	[Track 10.] Tiny Drops [Album: MAGIC]	B`z	f	1
5148	52	[Track 12.] Yume no Naka de Aimashō [Album: MAGIC]	B`z	f	1
5066	52	[Track 13.] Tobira [Album: ELEVEN]	B`z	f	2
5147	52	[Track 11.] Dare ni mo Ienē [Album: MAGIC]	B`z	f	2
5027	52	[Track 10.] Suna no Hanabira [Album: LOOSE]	B`z	f	2
5150	52	[Track 01.] Risky [Album: RISKY]	B`z	f	1
5151	52	[Track 02.] Gimme Your Love -Fukutsu no Love Driver- [Album: RISKY]	B`z	f	1
5152	52	[Track 03.] Hot Fashion -Ryukoukata- [Album: RISKY]	B`z	f	1
5154	52	[Track 05.] Itoshi Hitoyo Good Night... [Album: RISKY]	B`z	f	2
5156	52	[Track 07.] Vampire Woman [Album: RISKY]	B`z	f	1
5157	52	[Track 08.] Tashikana Mono wa Yami no Naka [Album: RISKY]	B`z	f	1
5158	52	[Track 09.] Friday Midnight Blue [Album: RISKY]	B`z	f	1
5159	52	[Track 10.] It's Raining [Album: RISKY]	B`z	f	1
5160	52	[Track 01.] Wonderful Opportunity [Album: In the Life]	B`z	f	1
5161	52	[Track 02.] Tonight (Is the Night) [Album: In the Life]	B`z	f	1
5162	52	[Track 03.] Kairaku no Heya [Album: In the Life]	B`z	f	1
5163	52	[Track 04.] Urei no GYPSY [Album: In the Life]	B`z	f	1
5164	52	[Track 05.] Crazy Rendezvous [Album: In the Life]	B`z	f	1
5165	52	[Track 06.] Mou Ichidou Kiss Shitakatta [Album: In the Life]	B`z	f	1
5166	52	[Track 07.] Wild Life [Album: In the Life]	B`z	f	1
5167	52	[Track 08.] Soredemo Kiminiwa Modorenai [Album: In the Life]	B`z	f	1
5168	52	[Track 09.] Aikawarazu na Bokura [Album: In the Life]	B`z	f	1
5169	52	[Track 10.] Alone [Album: In the Life]	B`z	f	1
5170	52	[Track 01.] The Gambler [Album: RUN]	B`z	f	1
5172	52	[Track 03.] Akai Kagerou [Album: RUN]	B`z	f	1
5173	52	[Track 04.] Run [Album: RUN]	B`z	f	1
5174	52	[Track 05.] Out of Control [Album: RUN]	B`z	f	1
5176	52	[Track 07.] Mr. Rolling Thunder [Album: RUN]	B`z	f	1
5178	52	[Track 09.] Gekkou [Album: RUN]	B`z	f	1
5179	52	[Track 10.] Baby, You're My Home [Album: RUN]	B`z	f	1
5180	52	[Track 1.01.] LOVE IS DEAD [Album: The 7th Blues]	B`z	f	1
5181	52	[Track 1.02.] Odekake Shinmashou [Album: The 7th Blues]	B`z	f	1
5182	52	[Track 1.03.] Miseinen [Album: The 7th Blues]	B`z	f	1
5155	52	[Track 06.] Holy Night ni Kuchizuke wo [Album: RISKY]	B`z	f	2
5134	52	[Track 15.] Ore to Omae no Atarashii Kisetsu [Album: ACTION]	B`z	f	2
5145	52	[Track 09.] Mayday [Album: MAGIC]	B`z	f	2
5149	52	[Track 13.] Freedom Train [Album: MAGIC]	B`z	f	2
5177	52	[Track 08.] Sayonara Nanka wa Iwasenai [Album: RUN]	B`z	f	2
5131	52	[Track 12.] HOMETOWN　BOYS’　MARCH　 [Album: ACTION]	B`z	f	2
5171	52	[Track 02.] Zero [Album: RUN]	B`z	f	2
5183	52	[Track 1.04.] Yami no Ame [Album: The 7th Blues]	B`z	f	1
5184	52	[Track 1.05.] MY SAD LOVE [Album: The 7th Blues]	B`z	f	1
5185	52	[Track 1.06.] Queen of Madrid [Album: The 7th Blues]	B`z	f	1
5186	52	[Track 1.07.] Himitsu na Futari [Album: The 7th Blues]	B`z	f	1
5187	52	[Track 1.08.] Strings of My Soul [Album: The 7th Blues]	B`z	f	1
5188	52	[Track 1.09.] Akai Kawa [Album: The 7th Blues]	B`z	f	1
5189	52	[Track 1.10.] WILD ROAD [Album: The 7th Blues]	B`z	f	1
5193	52	[Track 2.04.] JAP THE RIPPER [Album: The 7th Blues]	B`z	f	1
5194	52	[Track 2.05.] SLAVE TO THE NIGHT [Album: The 7th Blues]	B`z	f	1
5195	52	[Track 2.06.] Haru [Album: The 7th Blues]	B`z	f	1
5196	52	[Track 2.07.] Yaburenu Yume wo Hikizutte [Album: The 7th Blues]	B`z	f	1
5197	52	[Track 2.08.] LADY NAVIGATION [Album: The 7th Blues]	B`z	f	1
5199	52	[Track 2.10.] farewell song [Album: The 7th Blues]	B`z	f	1
5153	52	[Track 04.] Easy Come, Easy Go [Album: RISKY]	B`z	f	2
5192	52	[Track 2.03.] THE BORDER [Album: The 7th Blues]	B`z	f	2
4925	11	<Kaitoukidsama> maybe god is a woman  <Kaitoukidsama> and the old testament is just one holy menstrual period	Jing	f	34
5200	72	I'm still waiting, Zdm321... :(	zhan	f	31
5190	52	[Track 2.01.] Don't Leave Me [Album: The 7th Blues]	B`z	f	2
5191	52	[Track 2.02.] Sweet Lil' Devil [Album: The 7th Blues]	B`z	f	2
5198	52	[Track 2.09.] Mou Kari Makka [Album: The 7th Blues]	B`z	f	2
5215	55	Random shiranpuri Fact: 0106.25 <%shiranpuri> I should sub conan! it could be a whole new level of bad :D	JD_SUCKS	f	14
5204	14	Kazuha's mother.  Kaitou's mother.  Secret lovers.  Tragic deaths.	Abs-	t	60
5211	17	The latest releases can be found at http://ninjajosh.com/dctp/ , for older episodes downloads use the XDCC bots, or !MU for megaupload links, or ! plus the episode number to get a link directly (e.g !1, !2, !530, etc.)	Null^	t	296
5214	55	0106.25 <%shiranpuri> I should sub conan! it could be a whole new level of bad :D	JD_SUCKS	t	8
5203	14	Vodka is actually related to Shinichi, hence his overreaction to hearing "Kudo" during his first appearance.	Abs-	f	65
5207	14	Teitan Elementary is built on top of a hellmouth.	Abs-	f	65
5206	14	Korn's last name is Aoyama.	Abs-	f	66
5205	14	Kazuha's mother.  Kaito's mother.  Secret lovers.  Tragic deaths.	Abs-	f	66
5202	14	Tsuburaya Mitsuhiko is just an alias.  His real name is Jd-.	Abs-	f	66
5209	14	The next Conan movie: Conan tries to solve the mystery of the serial killer in America who stabs his victims with scissors.	Abs-	f	66
5210	14	Yuusaku started the B.O. in order to lure out the organization that killed Toichi, Yuusaku's secret lover.	Abs-	f	66
5208	14	If B.O. members don't drink alcohol all the time, they die. Guess what Haibara does at night.	Abs-	f	66
5213	17	The latest releases can be found at http://ninjajosh.com/dctp/ , for older episodes downloads use the XDCC bots, or !MU for megaupload links, or ! plus the episode number to get a link directly (e.g !1, !2, !560, etc.)	Null^	t	327
3278	37	The Meaningful Music Box (Episodes 194-195)	shiranpuri	f	1
3288	37	The Water Palace of Five Colors (Episodes 210-211)	shiranpuri	f	1
4947	73	Jd thinks Zdm321 is the best!	Null^	f	16
4950	3	Jd is the owner of a red panda farm.	Null^	f	34
4942	3	Jd- doesn't trust KaizouSeizo, that's why he didn't give him full access to #DCTP.	Null^	f	34
193	666	FCLNBW.DLL	simcop2387	f	1
4944	73	There was a time when Zdm didn't fail.	Null^	f	24
4943	72	Forgetting to un-comment one line in 232, leading to a v2.	Null^	f	24
4945	72	Zdm doesn't last.	Null^	f	28
54	7	Starting from their first to their last release, DCTP has released something Conan related about every 9 day, close to 1/3 of the time needed for Kaizoufansubs (about 22 days) or Nazonokami (about 27 days) to release some Conan content.	conankudo	f	43
4949	56	Everything is shiranpuri's fault.	Null^	f	12
4946	11	NEGI SPRINGFIELD IS JUST A REINCARNATION OF CHARLES MANSON	simcop2387-lap	f	34
4926	11	<+kyuuketsuki> i asked a religion teacher "Was Jesus really a savior? He seems more like a really good magician, he even did an escape trick for his finale before disappearing into infamy" <+kyuuketsuki> i never did get an answer btw	MrWiz88	f	34
4928	11	<3~jmaeshawn> now i'm picturing the girl with the brother complex going "Jmae-oniisan!!!" <7&MrWiz88> and youd enjoy every minute of it <7&MrWiz88> DONT LIE! <3~jmaeshawn> >.> <3~jmaeshawn> <.<  * ~jmaeshawn hides <7&MrWiz88> 13girl: matte! jmae-oniisan!!!  *trips* *looks up with puppy dog look*	MrWiz88	f	33
4960	14	Satou and Yumi used to be an item.	Abs-	f	66
4970	74	o[v-v]o	Null^	f	8
4969	14	Both Gin and Bourbon hate Akai. Gin and Bourbon aren't on speaking terms. Akai must have been playing both of them while he was dating Akemi.	Abs-	f	66
4955	14	Shiratori has a loli fetish. He tries to compare every woman he meets to a 7-year old girl.	Abs-	f	66
4978	74	o[ӧ_ӧ]o	o[^_^]o	f	7
1447	667	Marinated Shrimp Kabobs	Bubba	f	3
4984	74	o[@_@]o	o[^_^]o	f	5
4948	68	"Octopuses prefer HDTV"	jmaeshawn	f	7
4989	74	o[^-^]o	SPIDER	f	7
4986	75	Gao	shiranpuri	f	59
4976	74	o[x_^]o	Null^	f	7
4967	70	<Zdm321> those people are just go too overboard	Jing	f	0
4974	74	~[^_^]~	Null^	t	2
4954	14	Vodka actually can't stand Vodka, and prefers to drink Gin instead.	Abs-	f	66
4991	74	o[T_T]o	SPIDER	f	5
4951	3	Jd claims to be a female who has a preference for other females.	Abs-	f	34
5216	14	Jodie was more upset about not getting invited into the threesome than she was about getting dumped. 	Abs-	f	66
4979	74	O[._.]O	Abs-	f	7
4977	74	o[   ]o	Abs-	f	6
4985	74	o[oₓo]o	o[^_^]o	f	6
4959	14	Agasa is a lolicon and everyone knows it. That's why no one questions why he took custody of Haibara, but not Conan.	Abs-	f	66
4971	74	o[X_X]o	Abs-	f	5
4990	74	o[T-T]o	SPIDER	f	7
4972	74	o[^_^]o	o[v-v]o	f	5
4980	74	o[o_o]o	o[^_^]o	f	3
4964	14	Gin turned Vermouth down because it crushed his ego too much when they were last together and Vermouth called out Sherry's name instead.	Abs-	t	60
4962	43	I'm on a strict Jd-only diet.	Abs-	f	27
4982	74	0[#_#]0	Abs-	f	5
4953	14	Ayumi is actually Vermouth's father.	Abs-	f	66
4956	14	If Shinichi ever gets married, Yuusaku and Yukiko will use that occasion to announce that Yukiko is actually his father and Yuusaku his mother.	Abs-	f	66
4957	14	Sonoko is actually a sadist that likes it when people get hurt trying to protect her.	Abs-	f	66
4965	14	Asked by his raving fans what his favorite beauty product was, Kaitou Kid replied, "Lotion."	Abs-	f	66
4975	74	o[J_d]o	Abs-	f	6
4973	74	o[~_~]o	Abs-	f	7
4966	14	Genta is Agasa's favorite DB.	Abs-	f	66
4961	14	The type of alcohol the night guard at Teitan Elementary drinks while on duty is bourbon.	Abs-	f	66
4963	14	Eri actually got jealous of Kogorou's infatuation with Yōko and stormed into Yōko's office once to give her a piece of her mind. The two women are now secret lovers.	Abs-	f	66
4958	14	Shinichi gets excited by dead bodies. Really, really excited. So it would be less awkward for everyone if he would just remain Conan.	Abs-	f	66
4983	74	o[ˣ_ˣ]o	Null^	f	7
4968	14	The reason why Shinichi never confesses to Ran when he reverts back to being Shinichi is because the temporary antidote doesn't restore a crucial part of his anatomy to the original size. This is also why Haibara is working so feverishly for a permanent antidote.	Abs-	f	66
4988	74	o[.-.]o	SPIDER	f	6
4999	78	shiranpuri.	shiranpuri	f	3
4998	14	Gin turned Vermouth down because it crushed his ego too much when Vermouth called out Sherry's name the last time they were together.	Abs-	f	66
4995	78	Make me a step-by-step guide on how to follow instructions.	sitdownplease	f	6
4992	78	Where's my armored bike?	sitdownplease	f	6
4993	78	Quit postponing your procrastination and get to it already!	sitdownplease	f	8
4996	78	The radar sir! It appears to be... jammed! Jammed... *tastes it* Raspberry!	Sazaai`Aries	f	3
4997	78	Before you die, there is something you should know about us, Lone Star. What? I am your father's brother's nephew's cousin's former roommate.  What's that make us? Absolutely nothing!	Sazaai`Aries	t	4
4994	78	Would someone be so kind as to go buy me the book "How to Read For Dummies"?	sitdownplease	f	5
-9000	-9000	10,01Episode 15,01[11900015,01] Conan will not be going over 9000 any time soon	\N	f	31
-666	-666	10,01Episode 15,01[1166615,01] Conan's Halloween Special (real title TBA)04,01 || 07Aired 2012/Oct/3107,01	\N	f	58
3690	40	For a list of all episodes see, http://www.dctp.ws/DC_Subbed_List.pdf , see also !ddl	simcop2387	f	3234
4930	11	<~Kaitoukidsama> AND I HAD SEX WITH MY HAND <~Kaitoukidsama> jmaeshawn <~Kaitoukidsama> could you help me out a bit? <~jmaeshawn> oh yeah kks	MrWiz88	f	34
24	4	It's ten years too early for you to be interested in a lady's bedroom!	simcop2387	f	55
4929	11	<Kaitoukidsama> I  <Kaitoukidsama> WANT  <Kaitoukidsama> SEX  <Kaitoukidsama> JM  <Kaitoukidsama> GET OVER HAR	Zdm321	f	34
-10001	-10001	15,01[11MOVIE115,01] Skyscraper on a Timer04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01700mb Codec: [XviD] CRC: [231C23E1]04,01 || 07Aired 1997/Apr/1907,01	\N	f	10
-10002	-10002	15,01[11MOVIE215,01] The Fourteenth Target04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01700mb Codec: [XviD] CRC: [4CF3FD87]04,01 || 07Aired 1998/Apr/1807,01	\N	f	8
-10003	-10003	15,01[11MOVIE315,01] The Last Wizard of the Century04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01700mb Codec: [XviD] CRC: [A9501AF8]04,01 || 07Aired 1999/Apr/1707,01	\N	f	2
-10004	-10004	15,01[11MOVIE415,01] Captured In Her Eyes04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01660mb | 654mb Codec: [XviD] CRC: [40561A83 | EE49972B]04,01 || 07Aired 2000/Apr/2207,01	\N	f	4
-10005	-10005	15,01[11MOVIE515,01] Countdown to Heaven04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01696mb | 675mb Codec: [XviD] CRC: [24196F62 | 4F6780C7]04,01 || 07Aired 2001/Apr/2107,01	\N	f	5
-10006	-10006	15,01[11MOVIE615,01] The Phantom of Baker Street04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01659mb | 660mb Codec: [XviD] CRC: [706FEDE6 | 36124CA0]04,01 || 07Aired 2002/Apr/2007,01	\N	f	3
-10007	-10007	15,01[11MOVIE715,01] Crossroad in the Ancient Capital04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01702mb | 699mb Codec: [XviD] CRC: [A9EAE906 | E4AD16FE]04,01 || 07Aired 2003/Apr/1907,01	\N	f	4
-10008	-10008	15,01[11MOVIE815,01] Magician of the Silver Sky04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01700mb | 700mb Codec: [XviD] CRC: [4CD25276 | 7A0E0189]04,01 || 07Aired 2004/Apr/1707,01	\N	f	7
1965	29	Wallpaper :: http://forums.dctp.ws/index.php?topic=108.0	conanquotes	f	2
-10009	-10009	15,01[11MOVIE915,01] Strategy Above the Depths04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01700mb | 698mb Codec: [XviD] CRC: [359C82E9 | 422D609A]04,01 || 07Aired 2005/Apr/1907,01	\N	f	1
-10010	-10010	15,01[11MOVIE1015,01] The Private Eyes' Requiem04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01700mb | 700mb Codec: [XviD] CRC: [AD7851FC | 7AA97A53]04,01 || 07Aired 2006/Apr/1507,01	\N	f	4
1625	9	That must be them because there's no good reason for a bad to be flying around in the daytime without a good reason [naruto shippuden #98]	simcop2387	f	34
-10011	-10011	15,01[11MOVIE1115,01] Jolly Roger of the Deep Azure04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01699mb Codec: [h264] CRC: [DDBEA582]04,01 || 07Aired 2007/Apr/2707,01	\N	f	2
-10012	-10012	15,01[11MOVIE1215,01] Full Score of Fear04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,011.4gb | 700mb | 1.4gb Codec: [XviD] CRC: [AF83B2E0 | 52DC8F36 | BA79F197]04,01 || 07Aired 2008/Apr/1907,01	\N	f	5
5036	52	[Track 06.] Fireball [Album: SURVIVE]	B`z	f	2
5361	99	google	shiranpuri	f	2
5362	59	Random Chainsaw: http://puu.sh/HnC	Vylen	t	13
5363	59	http://puu.sh/HnC	Vylen	f	13
5234	9	[Lost Universe - ep2] System Marfunction	simcop2387	f	34
5222	51	You have misunderstood, please try !213 for an example; for a list of all episodes see http://en.wikipedia.org/wiki/List_of_Case_Closed_episodes	simcop2387-lap	f	243
-20001	-20001	15,01[11OVA115,01] Conan vs Kid vs YAIBA04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01233mb Codec: [XviD] CRC: [FE02E18F]	\N	f	3
-20002	-20002	15,01[11OVA215,01]  16 Suspects04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01233mb Codec: [XviD] CRC: [922F42C9]	\N	f	2
5217	9	"Today we gonne watch a premiere but we still gotta need my detective skills."	Null^	f	34
453	666	QuickTimeStreamingLocalized.dll	simcop2387	f	1
375	666	Mono.Data.TdsClient.dll	simcop2387	f	1
5334	82	BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBZ	simcop2387	f	14
5220	59	http://tinyurl.com/27m8z8v	shiranpuri	t	11
5219	11	Why is this trigger not funny?	Ishnatal	f	34
5218	38	http://tinyurl.com/y5rcgzg http://tinyurl.com/y48lj37	Null^	f	6
-20003	-20003	15,01[11OVA315,01] Conan and Heiji and the Vanished Boy04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01234mb Codec: [XviD] CRC: [10BCA14F]	\N	f	1
5221	14	The chipmunk was an expression of Shiratori's love.	Abs-	f	66
-20004	-20004	15,01[11OVA415,01] Conan and Kid and the Crystal Mother04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01228mb Codec: [XviD] CRC: [5DCAF827]	\N	f	1
-20005	-20005	15,01[11OVA515,01] The Target is Kogoro! The Detective Boys' Secret Investigation04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01233mb Codec: [XviD] CRC: [77091E38]	\N	f	0
-20006	-20006	15,01[11OVA615,01] Follow the Vanished Diamond! Conan and Heiji vs Kid!04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01171mb Codec: [XviD] CRC: [09673073]	\N	f	0
-20007	-20007	15,01[11OVA715,01] A Challenge from Agasa04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01233mb Codec: [XviD] CRC: [82978EC7]	\N	f	0
5269	50	%NICK% isn't liked by anyone	Vylen	f	33
5271	50	%NICK% can't think of any generic insults	Vylen	f	33
5272	50	%NICK% is a masticating cow	Vylen	f	33
5270	50	%NICK% likes to molest fish	Vylen	f	34
5273	50	%NICK% just won't die.	Huty	f	34
-20008	-20008	15,01[11OVA815,01] The Casebook of Female High-School Detective Suzuki Sonoko04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01224mb Codec: [XviD] CRC: [18A002E4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U6AC55WM04,01 || 07,01Original filename says CRC: [927B7E46] but that is wrong!	\N	f	1
-20009	-20009	15,01[11OVA915,01] The Stranger in 10 Years...04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01233mb | 233mb Codec: [XviD | h264] CRC: [BED9C41A | FA9E88C7]	\N	f	1
-20010	-20010	15,01[11OVA1015,01] Kid in Trap Island04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01231mb | 232mb Codec: [XviD | H264] CRC: [7A4B799E | 33564CE5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VM06KOBH | H264: http://www.megaupload.com/?d=37ZOMFSA	\N	f	20
-30001	-30001	15,01[11SPECIAL115,01] Magic File 1 (Movie 8)04,01 || 07,01Nothing new from this, just episodes 132-134 and 196 on dvd	\N	f	4
5274	1	When Detective conan ends it'll be in a special issue that has the entire magazine made with an e-ink display so the ending can be animated	simcop2387-lap	f	66
-10014	-10014	15,01[11MOVIE1415,01] The Lost Ship in the Sky04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,011.4gb Codec: [XviD | H264] CRC: [44005329 | DA6EB844]04,01 || 07MU Link: 15,01Part1: http://www.fileserve.com/file/scE3n2d Part2: http://www.fileserve.com/file/4dYkaph | h264: part1: http://www.fileserve.com/file/DbduqMB part2: http://www.fileserve.com/file/yhDNXMk04,01 || 07Aired 2010/Apr/1707,0104,01 || 07,01http://www.dctp.ws/Detective%20Conan%20-%20Movie%2014%20-%20The%20Lost%20Ship%20in%20the%20Sky%20[DCTP][XviD][44005329].torrent | http://www.dctp.ws/Detective%20Conan%20-%20Movie%2014%20-%20The%20Lost%20Ship%20in%20the%20Sky%20[DCTP][H264][DA6EB844].torrent	\N	f	69
-30002	-30002	15,01[11SPECIAL215,01] Magic File 2 (Movie 12 Side-Story OVA)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01252mb CRC: [BB07F5EC]	\N	f	3
5279	17	The latest releases can be found at http://ninjajosh.com/dctp/ , for older episodes downloads use the XDCC bots, or !MU for megaupload links, or !MF for mediafire links, or ! plus the episode number to get a link directly (e.g !1, !2, !560, etc.)	simcop2387-lap	f	376
4952	14	Ayumi is a shemale and the DBs all know it. That's why no one stopped her from going into the men's toilet when they were in the bank.	Abs-	f	66
5276	14	The Black Organization will change its name to "Mafia" and kill Akonyl on Night 1.	Abs-	f	66
5277	14	The antidote to APTX4869 actually involves the usage of Shinichi's seminal fluids, but Conan is too young to provide any useful samples	Vylen	f	66
5229	14	Ran once put someone's eye out with her horn. They tried to sue, but when they entered the courtroom and saw who her lawyer was, they ran out screaming... Never to be heard from again.	Abs-	f	66
5278	85	http://forums.dctp.ws/index.php?topic=3878.0	simcop2387-lap	f	499
-30003	-30003	15,01[11SPECIAL315,01] Magic File 3 (Movie 13 Side-Story OVA)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01214mb CRC: [642DAAF9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=J2ODW8DK	\N	f	2
5275	14	The boss of the Black Organization is named Shinjitsu Hitotsu.	Abs-	f	66
-30004	-30004	15,01[11SPECIAL415,01] Magic File 4 (Movie 14 Side-Story OVA)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01230mb | 253mb Codec: [XviD | H264] CRC: [E61E3B41 | 6C982512]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VMJVLHYO | Alt: http://www.megaupload.com/?d=0KKCXUM604,01 || 15,0107,01 AO04,01 || 07,01please yell at simcop2387 if any info is wrong	\N	f	8
-30006	-30006	15,01[11SPECIAL615,01]	\N	f	0
-30007	-30007	15,01[11SPECIAL715,01]	\N	f	1
-30008	-30008	15,01[11SPECIAL815,01]	\N	f	0
-30009	-30009	15,01[11SPECIAL915,01]	\N	f	0
-30010	-30010	15,01[11SPECIAL1015,01] Lupin III vs. Detective Conan TV Special04,01 || 15,01Subbed by [10 | Frostii-DCTP15,01] 07@ 15,01 | 1.4gb CRC: [ | 07CEE0B0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HBCFJ01X (soft-subbed MKV - 720p)04,01 || 07,01In parts http://www.megaupload.com/?d=6HM922ZV http://www.megaupload.com/?d=DY4RB5IE	\N	f	0
4468	67	pilot fish	simcop2387-lap	f	1
1476	667	Grilled Shrimp and Pancetta with Garbanzo Salsa	Bubba	f	3
-30011	-30011	15,01[11SPECIAL1115,01] Kid the Phantom Thief  (Magic Kaito)「マジク カイト」04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01231mb | 484mb CRC: [75950F73 | 700395F0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=N9EPK86O | 720: http://www.megaupload.com/?d=3V8ZXWFL04,01 || 07Aired 2010/Apr/1707,01	\N	f	10
-30012	-30012	15,01[11SPECIAL1215,01]	\N	f	0
-30013	-30013	15,01[11SPECIAL1315,01]	\N	f	0
-30014	-30014	15,01[11SPECIAL1415,01]	\N	f	0
-30015	-30015	15,01[11SPECIAL1515,01]	\N	f	0
-30016	-30016	15,01[11SPECIAL1615,01]	\N	f	0
-30017	-30017	15,01[11SPECIAL1715,01]	\N	f	0
-30018	-30018	15,01[11SPECIAL1815,01]	\N	f	0
-30019	-30019	15,01[11SPECIAL1915,01]	\N	f	0
-30020	-30020	15,01[11SPECIAL2015,01] Live Action 0104,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01698mb CRC: [6C34F857]	\N	f	0
-30021	-30021	15,01[11SPECIAL2115,01] Live Action 0204,01 || 15,01Subbed by [10TimeLes15,01] 07@ 15,01782mb	\N	f	1
-30022	-30022	15,01[11SPECIAL2215,01]	\N	f	0
-30023	-30023	15,01[11SPECIAL2315,01]	\N	f	0
-30024	-30024	15,01[11SPECIAL2415,01]	\N	f	0
-30025	-30025	15,01[11SPECIAL2515,01]	\N	f	0
-30026	-30026	15,01[11SPECIAL2615,01]	\N	f	0
-126	-126	10,01Episode 15,01[1112615,01] The Travelling Drama Troupe Murder Case (Part 1)04,01 || 15,01Subbed by [10Puto15,01] 07@ 15,01225mb Codec: [H264-AC3] CRC: [FBDCE79B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=II5YWNXI04,01 || 07Aired 1998/Nov/3007,01 with an audience share of 20.1%04,01 || 15,0107,01 AO	\N	f	25
-30027	-30027	15,01[11SPECIAL2715,01]	\N	f	0
-127	-127	10,01Episode 15,01[1112715,01] The Travelling Drama Troupe Murder Case (Part 2)04,01 || 15,01Subbed by [10Puto15,01] 07@ 15,01225mb Codec: [H264-AC3] CRC: [3F6914E2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SLP4GODE04,01 || 07Aired 1998/Dec/0707,01 with an audience share of 21.3%04,01 || 15,0107,01 AO	\N	f	18
-30028	-30028	15,01[11SPECIAL2815,01]	\N	f	0
-30029	-30029	15,01[11SPECIAL2915,01]	\N	f	0
-30030	-30030	15,01[11SPECIAL3015,01]	\N	f	0
-30031	-30031	15,01[11SPECIAL3115,01]	\N	f	0
-30032	-30032	15,01[11SPECIAL3215,01]	\N	f	0
-30033	-30033	15,01[11SPECIAL3315,01]	\N	f	0
-30034	-30034	15,01[11SPECIAL3415,01]	\N	f	0
-30035	-30035	15,01[11SPECIAL3515,01]	\N	f	0
-30036	-30036	15,01[11SPECIAL3615,01]	\N	f	0
-30037	-30037	15,01[11SPECIAL3715,01]	\N	f	0
-30038	-30038	15,01[11SPECIAL3815,01]	\N	f	0
-30039	-30039	15,01[11SPECIAL3915,01]	\N	f	0
-30040	-30040	15,01[11SPECIAL4015,01]	\N	f	0
-30041	-30041	15,01[11SPECIAL4115,01]	\N	f	0
-30042	-30042	15,01[11SPECIAL4215,01]	\N	f	0
-30043	-30043	15,01[11SPECIAL4315,01]	\N	f	0
-30044	-30044	15,01[11SPECIAL4415,01]	\N	f	0
-30045	-30045	15,01[11SPECIAL4515,01]	\N	f	0
-30046	-30046	15,01[11SPECIAL4615,01]	\N	f	0
-30047	-30047	15,01[11SPECIAL4715,01]	\N	f	0
-30048	-30048	15,01[11SPECIAL4815,01]	\N	f	0
5242	82	Huty eats your vuvuzela	Vylen	f	10
5237	82	Bzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz	Sir_Gallantmon	f	15
-30049	-30049	15,01[11SPECIAL4915,01]	\N	f	0
5235	1	There are actually two Ran's. One that is friends with Sonoko and another with Kazuha. That's why they're never seen together.	Vylen	t	61
5236	14	There are actually two Ran's. One that is friends with Sonoko and another with Kazuha. That's why they're never seen together.	Vylen	f	66
5246	82	%NICK% goes bzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz	Huty	f	14
5238	82	bzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz	Vylen	f	13
5247	82	BZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ	Conan_Kudo	f	15
5240	82	bzzzzzzzzzzZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ	Vylen	f	14
5239	84	SHUT THE FUCK UP	Vylen	f	15
5244	82	BZZZZZZZZZZZZZzzzzzzzzzz................ *gasp*	Vylen	f	12
5241	82	Vylen goes bzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz	Huty	f	13
5245	82	BZZZZZZZZZZZZZZZZZZZzzzz BzzzzzzzzzzZZZZZZZZZ	Vylen	f	10
5248	82	zzzzzzzzzzzzzzzZZZZZZZZZZZZZZB	Vylen	f	17
5243	82	BZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ	Sir_Gallantmon	f	12
5283	86	Sum Ting Wong?	Sazaai`Aries	f	5
-30050	-30050	15,01[11SPECIAL5015,01]	\N	f	1
5281	1	In a recent interview with Gosho, he stated that he knows how he would like to end Conan, but doesn't know when he'll get the chance to draw it.	Vylen	f	66
5280	14	Conan actually has the hots for Mitsuhiko's sister.	Vylen	f	66
5312	88	Random Boob-Grab: http://i35.tinypic.com/es3s52.jpg	Vylen	f	15
5311	88	Random Boob-Grab: http://i38.tinypic.com/25zrtsn.jpg	Vylen	f	14
5304	88	Random Boob-Grab: http://i37.tinypic.com/207thq8.jpg	Vylen	f	14
5302	88	Random Boob-Grab: http://i38.tinypic.com/260g5cy.gif	Vylen	f	14
5297	87	http://tinyurl.com/36gtkpn Better safe than sorry.	Sazaai`Aries	f	1
3932	67	bristlemouth	simcop2387-lap	f	1
5296	87	http://tinyurl.com/2wpdm5n It's a good thing there are caution signs - wouldn't know some things otherwise.	Sazaai`Aries	f	1
5310	88	Random Boob-Grab: http://i33.tinypic.com/mt11lx.jpg	Vylen	f	15
5307	88	Random Boob-Grab: http://i37.tinypic.com/25z1jxe.jpg	Vylen	f	15
5298	87	http://tinyurl.com/2wpdm5n Better safe than sorry.	Sazaai`Aries	f	1
5286	11	<&Parkur> .gtfo Abs- <&Parkur> :( <&Abs-> :D * Quits: &Abs- (~Abs-@Jd-.is.the.loliest) (Ping timeout: 240 seconds) <&Parkur> :)	Parkur	f	34
5285	59	http://i34.tinypic.com/1z15q2h.jpg	Sazaai`Aries	f	19
5303	88	Random Boob-Grab: http://i36.tinypic.com/2nc1185.png	Vylen	f	15
5288	59	http://i38.tinypic.com/1zv8o3o.jpg	Sazaai`Aries	f	13
5289	87	Buy and read the book http://i34.tinypic.com/14ak0a8.jpg	Sazaai`Aries	f	0
5290	87	Buy and read the book http://tinyurl.com/2etae9c	Sazaai`Aries	f	0
5291	87	Buy and read the book http://tinyurl.com/2v8ys6o	Sazaai`Aries	f	0
5292	87	http://tinyurl.com/287454s - you're reading it wrong.	Sazaai`Aries	f	0
5293	87	http://tinyurl.com/287454s - you're doing it wrong.	Sazaai`Aries	f	0
5294	87	Let the doctor put you under before he pulls out http://tinyurl.com/2c9xr8x .  Less stress = better chances of success :P	Sazaai`Aries	f	0
5295	87	http://tinyurl.com/2wpdm5n beware of caution signs.	Sazaai`Aries	f	0
5299	87	http://tinyurl.com/28svv9d bored?	Sazaai`Aries	f	0
5300	87	Next time you're in your car, waiting for people to finish crossing the road - http://tinyurl.com/3xpt5za	Sazaai`Aries	f	0
5301	88	http://i38.tinypic.com/260g5cy.gif	Vylen	t	2
5309	88	Random Boob-Grab: http://i33.tinypic.com/2dkh7km.jpg	Vylen	f	15
5287	59	http://i38.tinypic.com/250qvj5.jpg	Sazaai`Aries	f	13
5308	88	Random Boob-Grab: http://i36.tinypic.com/eg3fbp.jpg	Vylen	f	9
5305	88	Random Boob-Grab: http://i37.tinypic.com/dcu555.jpg	Vylen	f	13
5306	88	Random Boob-Grab: http://i37.tinypic.com/k54wag.jpg	Vylen	f	13
5284	59	http://i35.tinypic.com/2z5jckn.jpg	Sazaai`Aries	f	11
5314	88	Random Boob-Grab: http://i33.tinypic.com/1zxwmy1.jpg	Vylen	f	15
5316	88	Random Boob-Grab: http://i33.tinypic.com/mt7ac9.png	Vylen	f	15
5319	88	Random Boob-Grab: http://i37.tinypic.com/2gya6hk.png	Vylen	f	14
5313	88	Random Boob-Grab: http://i35.tinypic.com/17co61.png	Vylen	f	14
5317	88	Random Boob-Grab: http://i37.tinypic.com/28svq5c.jpg	Vylen	f	15
5315	88	Random Boob-Grab: http://i38.tinypic.com/20rxb4.jpg	Vylen	f	14
5318	88	Random Boob-Grab: http://i33.tinypic.com/2h2i0du.png	Vylen	f	14
5322	11	I THINK THE JAPANESE ARE OBSESSED WITH THE THREE STOOGES. WHY ELSE WOULD THEY TALK ABOUT GIRLS BEING MOE?	simcop2387-lap	f	34
5323	90	!?	shiranpuri	f	10
5321	86	Chinese Semen Cannabis Drink	Vylen	f	5
5324	90	?!	shiranpuri	f	5
5320	89	Scooby Dooby Doo, where are you??	shiranpuri	f	9
-80	-80	10,01Episode 15,01[118015,01] The Wandering Artist Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [05ED0052]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BZN32YQY04,01 || 07Aired 1997/Nov/1007,01 with an audience share of 19.2%04,01 || 15,0107,01 AO	\N	f	4
-348	-348	10,01Episode 15,01[1134815,01] Love, Ghosts, and World Heritage (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350mb Codec: [XviD] CRC: [E6388A1A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZXWEZ40H04,01 || 07Aired 2004/Jan/2607,01 with an audience share of 13.5%04,01 || 15,0107,01 AO04,01 || 07,01v2 available. Part 2 is concatenated.	\N	f	15
-349	-349	10,01Episode 15,01[1134915,01] Love, Ghosts, and World Heritage (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350mb Codec: [XviD] CRC: [E6388A1A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZXWEZ40H04,01 || 07Aired 2004/Feb/0207,01 with an audience share of 13.4%04,01 || 15,0107,01 AO04,01 || 07,01v2 available. Part 1 is concatenated.	\N	f	8
5338	88	Random Boob-Grab: http://oi52.tinypic.com/10s6nmb.jpg	____	f	14
5339	88	Random Boob-Grab: http://oi52.tinypic.com/2e5mbya.jpg	____	f	15
5335	92	Ran gets pregnant	simcop2387	f	13
5336	88	http://oi55.tinypic.com/jkvz3c.jpg	____	t	12
5337	88	Random Boob-Grab: http://oi55.tinypic.com/jkvz3c.jpg	____	f	14
5343	88	Random Boob-Grab: http://oi51.tinypic.com/sxm8ep.jpg	____	f	15
5346	88	Random Boob-Grab: http://i52.tinypic.com/2hdctg5.png	Vylen	f	14
5344	88	Random Boob-Grab: http://i56.tinypic.com/2lj6kol.png	Vylen	f	15
5345	88	Random Boob-Grab: http://i55.tinypic.com/2jbjd75.png	Vylen	f	14
5351	88	Random Boob-Grab: http://i53.tinypic.com/2cz662w.png	Vylen	f	14
5350	88	Random Boob-Grab: http://i56.tinypic.com/2q2fyoo.png	Vylen	f	15
5342	88	Random Boob-Grab: http://oi52.tinypic.com/10gi9sx.jpg	____	f	14
-10013	-10013	15,01[11MOVIE1315,01] The Raven Chaser04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01700mb | 700mb | 1.4gb Codec: [XviD] CRC: [65983713 | 8D95E707 | EFC07D98]04,01 || 07MU Link: 15,01XviD Part 1: http://www.megaupload.com/?d=Y1DH1LPI :: XviD Part 2: http://www.megaupload.com/?d=TZRWH16L04,01 || 07Aired 2009/Apr/1807,0104,01 || 07,01H264-AC3: [multipart rar] http://www.megaupload.com/?d=KU19K9MJ http://www.megaupload.com/?d=NUBUQY12 http://www.megaupload.com/?d=WNXB4VTZ	\N	f	13
5349	88	Random Boob-Grab: http://i51.tinypic.com/vgu55k.jpg	____	f	14
5352	93	banned command used	simcop2387	f	1
5340	88	Random Boob-Grab: http://oi54.tinypic.com/1j96rl.jpg	____	f	13
5347	88	Random Boob-Grab: http://i53.tinypic.com/abjepc.png	Vylen	f	13
5341	88	Random Boob-Grab: http://oi56.tinypic.com/2hs0w7s.jpg	____	f	13
5348	88	Random Boob-Grab: http://i51.tinypic.com/r0c8px.png	Vylen	f	14
5357	57	Cup puts a %nick% on SPIDER.	Huty	f	186
5355	97	Current bots: [XCHKS]Xevious-xdcc, KaitouKid, Jodie-sensei	Vylen	f	11
5356	57	Cup puts a %nick on SPIDER.	Huty	t	169
5364	9	DAQ ch52 (KA Workz Scans): "Yeah... The Internet has the power to amplificate one's hideous parts."	simcop2387	t	34
5365	9	DAQ ch52 (KA Workz Scans): "Yeah... The Internet has the power to amplificate one's hideous parts...!"	simcop2387	f	34
5366	9	Good Ending 77 (Imanga) - This Hydrogen reacts and gets together with that, and it becomes H4.	simcop2387	f	34
-71	-71	10,01Episode 15,01[117115,01] A Stalker's Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [B2E178E7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NT5WXP6G04,01 || 07Aired 1997/Aug/1107,01 with an audience share of 13.2%04,01 || 15,0107,01 AO	\N	f	3
-206	-206	10,01Episode 15,01[1120615,01] The Metropolitan Police Detective Love Story 3 (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [A80FD81B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=EXGSMYFO04,01 || 07Aired 2000/Sep/0407,01 with an audience share of 18.6%04,01 || 07,01 Based on Vol 27 File 4-6	\N	f	2
-443	-443	10,01Episode 15,01[1144315,01] Gathering Shellfish With a Sigh (Part 1)04,01 || 15,0107@ 15,01gb CRC: [66689482]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=37HIVZBF04,01 || 07Aired 2006/Jun/2607,01 with an audience share of 9.5%04,01 || 07,01 Based on Vol 51 File 4-6	\N	f	1
-559	-559	10,01Episode 15,01[1155915,01] The Mansion of Death and the Red Wall: Item in Hand04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 318mb Codec: [XviD] CRC: [F52DCAA8 | E739F95E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=E3IP1GEE | 720: http://www.megaupload.com/?d=Z47OGM4X04,01 || 07Aired 2009/Dec/1207,0104,01 || 07,01 Based on Vol 65 File 8 - Vol 66 File 1	\N	f	3
-118	-118	10,01Episode 15,01[1111815,01] The Naniwa Serial Murder Case (1 Hour Special)04,01 || 15,0107@ 15,01gb CRC: [6FC6004C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GBVQYDYO04,01 || 07Aired 1998/Sep/2807,01 with an audience share of 21.2%04,01 || 07,01 Based on Vol 19 File 5-8	\N	f	4
-32	-32	10,01Episode 15,01[113215,01] Coffee Shop Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01169mb CRC: [FDA732D5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZX2WNC5G04,01 || 07Aired 1996/Sep/0907,01 with an audience share of 14.3%04,01 || 07,01 Based on Vol 11 File 5-7	\N	f	0
-194	-194	10,01Episode 15,01[1119415,01] The Significant Music Box (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [9B96FE12]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QDC18R9M04,01 || 07Aired 2000/Jun/1207,01 with an audience share of 19.3%04,01 || 07,01 Based on Vol 26 File 8-10	\N	f	0
-31	-31	10,01Episode 15,01[113115,01] The TV Station Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01170mb CRC: [B8B3F6E3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CG4OL4CL04,01 || 07Aired 1996/Sep/0207,01 with an audience share of 13.2%04,01 || 07,01 Based on Vol 11 File 2-4	\N	f	0
-29	-29	10,01Episode 15,01[112915,01] Computer Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01169mb CRC: [33EBDD5A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5AC765RM04,01 || 07Aired 1996/Aug/1907,01 with an audience share of 12.9%04,01 || 15,0107,01 AO	\N	f	0
-325	-325	10,01Episode 15,01[1132515,01] The Red Horse within the Flames (Part 1: The Case)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [C179102B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UYHL1FR304,01 || 07Aired 2003/Jun/2307,01 with an audience share of 16.1%04,01 || 07,01 Based on Vol 39 File 1-515,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 14	\N	f	3
-378	-378	10,01Episode 15,01[1137815,01] Momotarou Mystery Solving Tour (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb CRC: [0DE1791E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4C386X4Z04,01 || 07Aired 2004/Nov/1507,01 with an audience share of 15%04,01 || 15,0107,01 AO	\N	f	1
-350	-350	10,01Episode 15,01[1135015,01] The Forgetten Cellphone (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 MB Codec: [XviD] CRC: [DA59B622]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HHSW0S7304,01 || 07Aired 2004/Feb/0907,01 with an audience share of 14.1%04,01 || 07,01 Based on Vol 43 File 3-5	\N	f	1
-517	-517	10,01Episode 15,01[1151715,01] Fuurinkazan, Shadow and Lightning's Conclusion (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [769074C4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LESN3QQB04,01 || 07Aired 2008/Nov/1007,01 with an audience share of 8%04,01 || 07,01 Based on Vol 59 File 5-10	\N	f	2
-331	-331	10,01Episode 15,01[1133115,01] Suspicious Curry (Part 1)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01147mb Codec: [XviD] CRC: [31A1F9E5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=42YBWUR704,01 || 07Aired 2003/Aug/1107,01 with an audience share of 13.9%04,01 || 07,01 Based on Vol 40 File 4-615,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 28	\N	f	3
-220	-220	10,01Episode 15,01[1122015,01] The Client Full of Lies (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb CRC: [0AC8F430]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=17C0PMYZ04,01 || 07Aired 2001/Jan/1507,01 with an audience share of 17.5%04,01 || 07,01 Based on Vol 28 File 3-5	\N	f	2
-451	-451	10,01Episode 15,01[1145115,01] Trick VS Magic (Part 2)04,01 || 15,0107@ 15,01gb CRC: [0A2DC789]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SZ6XPBDO04,01 || 07Aired 2006/Sep/0407,01 with an audience share of 8.8%04,01 || 15,0107,01 AO	\N	f	1
-437	-437	10,01Episode 15,01[1143715,01] Aya Ueto & Shinichi: The Promise from 4 Years Ago04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01170mb Codec: [XviD] CRC: [18E9D4EF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ECXHY8A504,01 || 07Aired 2006/May/0807,01 with an audience share of 12%04,01 || 15,0107,01 AO	\N	f	1
-580	-580	10,01Episode 15,01[1158015,01] The Black Time Limit Drawing Near04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 319mb Codec: [XviD | H264] CRC: [06DB6ACE | D8A23606]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=493PIDK9 | 720: http://www.megaupload.com/?d=MCW61FZW04,01 || 07Aired 2010/Jul/1007,0104,01 || 07,01 Based on Vol 67 Files 4-8	\N	f	3
-560	-560	10,01Episode 15,01[1156015,01] The Mansion of Death and the Red Wall: The Late Koumei04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD] CRC: [E7770DA4 | 102B9F54]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0AU49AB9 | 720: http://www.megaupload.com/?d=R60UT0H104,01 || 07Aired 2009/Dec/1907,0104,01 || 07,01 Based on Vol 65 File 8 - Vol 66 File 1	\N	f	4
-454	-454	10,01Episode 15,01[1145415,01] Upsetting Outcome (Part 1)04,01 || 15,01Subbed by [10anfs15,01] Codec: [XviD] CRC: [D1F3746D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7ZFFTK3Q04,01 || 07Aired 2006/Oct/3007,01 with an audience share of 8.2%04,01 || 07,01 Based on Vol 52 File 6-8	\N	f	2
-84	-84	10,01Episode 15,01[118415,01] Ski Lodge Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [53BAF26F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=TM4N0T6A04,01 || 07Aired 1997/Dec/0807,01 with an audience share of 19.7%04,01 || 07,01 Based on Vol 14 File 9 - Vol 15 File 3	\N	f	2
-458	-458	10,01Episode 15,01[1145815,01] Sonoko's Red Handkerchief (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [9B4BC73A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UZT5PLW704,01 || 07Aired 2006/Nov/2707,01 with an audience share of 8.2%04,01 || 07,01 Based on Vol 52 File 9-11	\N	f	1
-358	-358	10,01Episode 15,01[1135815,01] The Metropolitan Police Detective Love Story 5 (Part 1)04,01 || 15,01Subbed by [10Kaizou-Neg15,01] 07@ 15,01172mb CRC: [3B4E158E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L3XU4EXP04,01 || 07Aired 2004/May/0307,01 with an audience share of 8.8%04,01 || 07,01 Based on Vol 40 File 1-3	\N	f	2
-512	-512	10,01Episode 15,01[1151215,01] The Broken Horoscope04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [23D57BA7 | 1699D57C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MSE2AAMR04,01 || 07Aired 2008/Aug/1107,01 with an audience share of 7.7%04,01 || 15,0107,01 AO	\N	f	2
-463	-463	10,01Episode 15,01[1146315,01] Shadow of Black Organization: Strange Illumination04,01 || 15,01Subbed by [10Kaizou-Neg15,01] 07@ 15,01172mb | 233mb CRC: [2F8BAA4D | 6143EE64]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=70PV4E4V | 720: http://www.megaupload.com/?d=M68UDJCA04,01 || 07Aired 2007/Feb/0507,01 with an audience share of 10.3%04,01 || 07,01 Based on Vol 53 File 7 - Vol 54 File 2	\N	f	3
-58	-58	10,01Episode 15,01[115815,01] Holmes Freak Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [818D5D25]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QIUOTS9K04,01 || 07Aired 1997/May/1207,01 with an audience share of 16.5%04,01 || 07,01 Based on Vol 12 File 7 - Vol 13 File 1	\N	f	0
-101	-101	10,01Episode 15,01[1110115,01] The Memories of First Love Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [341DDDE3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=M8AQGN0U04,01 || 07Aired 1998/May/1807,01 with an audience share of 19.5%04,01 || 07,01 Based on Vol 18 File 3-5	\N	f	0
-139	-139	10,01Episode 15,01[1113915,01] The Final Screening Murder Case (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [38D8DC20]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V168JTM104,01 || 07Aired 1999/Apr/0507,01 with an audience share of 21.3%04,01 || 07,01 Based on Vol 23 File 1-3	\N	f	0
-153	-153	10,01Episode 15,01[1115315,01] Sonoko's Dangerous Summer Story (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01174mb CRC: [20F59BCF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0MQIQ6JM04,01 || 07Aired 1999/Jul/1907,01 with an audience share of 18.3%04,01 || 07,01 Based on Vol 22 File 8-10	\N	f	3
-211	-211	10,01Episode 15,01[1121115,01] The Water Palace of Five Colors (Part 2)04,01 || 15,0107@ 15,01gb CRC: [EE5A4E84]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4ACS5KZX04,01 || 07Aired 2000/Oct/3007,01 with an audience share of 20.4%04,01 || 15,0107,01 AO	\N	f	1
-337	-337	10,01Episode 15,01[1133715,01] Hidden Circumstances of the Falling Incident04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [32B174AE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z3Q3MPM804,01 || 07Aired 2003/Oct/1307,01 with an audience share of 14.7%04,01 || 15,0107,01 AO	\N	f	2
-311	-311	10,01Episode 15,01[1131115,01] Contact with the Black Organization (Part 3: The Desperation)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01212mb CRC: [04177903]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=88XPXWAC04,01 || 07Aired 2003/Feb/2407,01 with an audience share of 16.4%04,01 || 07,01 Based on Vol 37 File 8 - Vol 38 File 1	\N	f	1
-527	-527	10,01Episode 15,01[1152715,01] The Malice Hidden Behind the Masque04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [6D24B0D7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PCSUBQPR04,01 || 07Aired 2009/Mar/0207,01 with an audience share of 8.4%04,01 || 15,0107,01 AO	\N	f	2
-340	-340	10,01Episode 15,01[1134015,01] Hidden Bathroom Secret (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01174mb CRC: [C65D4487]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4DNQRACJ04,01 || 07Aired 2003/Nov/0307,01 with an audience share of 12.1%04,01 || 07,01 Based on Vol 41 File 10 - Vol 42 File 1	\N	f	1
-389	-389	10,01Episode 15,01[1138915,01] Kogoro Gets Drunk in Satsuma (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [86CE4F04]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Q9IH48IG04,01 || 07Aired 2005/Feb/2107,01 with an audience share of 12.9%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 18	\N	f	4
-76	-76	10,01Episode 15,01[117615,01] Conan VS Kaitou Kid (1 Hour Special)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01344mb CRC: [1D551AE9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JSBD0J0M04,01 || 07Aired 1997/Sep/2207,01 with an audience share of 21.7%04,01 || 07,01 Based on Vol 16 File 6-9	\N	f	2
-15	-15	10,01Episode 15,01[111515,01] Missing Corpse Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01209mb CRC: [0CCDA374]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PZ931C0904,01 || 07Aired 1996/May/1307,01 with an audience share of 13.9%04,01 || 07,01 Based on Vol 6 File 6-8	\N	f	3
-431	-431	10,01Episode 15,01[1143115,01] The Metropolitan Police Detective Love Story 7 (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01174mb CRC: [7518AB85]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4YC25BCA04,01 || 07Aired 2006/Feb/2007,01 with an audience share of 10.4%04,01 || 07,01 Based on Vol 50 File 1-4	\N	f	4
-226	-226	10,01Episode 15,01[1122615,01] The Battle Game Trap (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [D75485F6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CHOWE04N04,01 || 07Aired 2001/Feb/2607,01 with an audience share of 20.9%04,01 || 07,01 Based on Vol 27 File 7-9	\N	f	1
-62	-62	10,01Episode 15,01[116215,01] A Ghost Ship Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [217D9718]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z04GYJNC04,01 || 07Aired 1997/Jun/0907,01 with an audience share of 20.8%04,01 || 15,0107,01 AO	\N	f	2
-129	-129	10,01Episode 15,01[1112915,01] A University Professor and the Girl Who Came from the Black Organization (2 Hour Special)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01700mb CRC: [53E92AEB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XS6N9HCZ04,01 || 07Aired 1998/Dec/2107,01 with an audience share of 20.9%04,01 || 07,01 Based on Vol 18 File 6 - Vol 19 File 1	\N	f	2
-382	-382	10,01Episode 15,01[1138215,01] Which One's Reasoning Show? (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [C6E445F4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=D011VQZZ04,01 || 07Aired 2004/Dec/1307,01 with an audience share of 12.8%04,01 || 07,01 Based on Vol 43 File 6-9	\N	f	2
-540	-540	10,01Episode 15,01[1154015,01] The Day Mouri Kogorou Ceased Being A Detective (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 527mb Codec: [XviD] CRC: [97902320 | DD063EDF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=T8RXZMTF04,01 || 07Aired 2009/July/1107,01 with an audience share of 5.9%04,01 || 15,0107,01 AO	\N	f	1
-572	-572	10,01Episode 15,01[1157215,01] Battle of the Haunted Warehouse's Treasure (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 321mb Codec: [XviD | H264] CRC: [916F59D1 | 6E5E1655]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WWYS9QQ0 | 720: http://www.megaupload.com/?d=E1B29BNU04,01 || 07Aired 2010/May/0807,0104,01 || 07,01 Based on Vol 66 File 5-7	\N	f	4
-168	-168	10,01Episode 15,01[1116815,01] The Monster of the Tottori Spider Mansion (Part 3: The Solution)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [BADEED18]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DL4HBAJ004,01 || 07Aired 1999/Nov/1507,01 with an audience share of 17.6%04,01 || 07,01 Based on Vol 25 File 4-8	\N	f	0
-178	-178	10,01Episode 15,01[1117815,01] Meeting with the Black Organization Again (Part 3)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [E09CEDF2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=27U6YNG804,01 || 07Aired 2000/Jan/3107,01 with an audience share of 19.5%04,01 || 07,01 Based on Vol 24 File 7-11	\N	f	0
-96	-96	10,01Episode 15,01[119615,01] Caught Up with the Great Detective! Two Murder Cases (2 Hour Special)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01688mb CRC: [9E6BA9A8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HI4ROCC204,01 || 07Aired 1998/Mar/2307,01 with an audience share of 22.4%04,01 || 07,01 Based on Vol 14 File 1-8	\N	f	0
-145	-145	10,01Episode 15,01[1114515,01] The North Star #3 Express Leaving Ueno (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [8C7A529C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NE9OR00M04,01 || 07Aired 1999/May/2407,01 with an audience share of 19.4%04,01 || 07,01 Based on Vol 22 File 4-7	\N	f	5
-124	-124	10,01Episode 15,01[1112415,01] A Mysterious Sniper Murder Case (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 MB Codec: [XVID-MP3] CRC: [B6DE84CB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=W4QXGOPQ04,01 || 07Aired 1998/Nov/1607,01 with an audience share of 19.9%04,01 || 15,0107,01 AO	\N	f	4
-23	-23	10,01Episode 15,01[112315,01] Luxury Liner Serial Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01173mb CRC: [3CF3292F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=16AZB48004,01 || 07Aired 1996/Jul/0807,01 with an audience share of 13.4%04,01 || 07,01 Based on Vol 3 File 1-6	\N	f	2
-364	-364	10,01Episode 15,01[1136415,01] Synchronicity Case (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01176mb CRC: [4988B7A3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=146OQKKO04,01 || 07Aired 2004/Jun/1407,01 with an audience share of 13.1%04,01 || 15,0107,01 AO	\N	f	3
-14	-14	10,01Episode 15,01[111415,01] The Mysterious Shooting Message Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01234mb CRC: [E0774045]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=25H529VM04,01 || 07Aired 1996/Apr/2907,01 with an audience share of 12.5%04,01 || 15,0107,01 AO	\N	f	2
-49	-49	10,01Episode 15,01[114915,01] The Diplomat Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [BBC1768D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=T3D2WD8004,01 || 07Aired 1997/Feb/2407,01 with an audience share of 18.2%04,01 || 07,01 Based on Vol 10 File 2-6	\N	f	1
-135	-135	10,01Episode 15,01[1113515,01] The Search for the Disappearing Weapon Case04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [26D6AA65]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=F5L55OUZ04,01 || 07Aired 1999/Feb/1507,01 with an audience share of 22.6%04,01 || 15,0107,01 AO	\N	f	1
-548	-548	10,01Episode 15,01[1154815,01] Two Days With the Culprit (Second Day)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 300mb Codec: [XViD] CRC: [B5D16277 | F538C882]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9K98RZBD04,01 || 07Aired 2009/Sept/2607,01 with an audience share of 9.8%04,01 || 15,0107,01 AO	\N	f	3
-234	-234	10,01Episode 15,01[1123415,01] The Evidence That Didn't Disappear (Part 2)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01174mb Codec: [XviD] CRC: [0BFB42BC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K5GCRA2C04,01 || 07Aired 2001/May/2107,01 with an audience share of 19.5%04,01 || 07,01 Based on Vol 29 File 6-815,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 10	\N	f	2
-418	-418	10,01Episode 15,01[1141815,01] Home of Beika's Grenier04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb CRC: [8DE508E2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=M79LYD6D04,01 || 07Aired 2005/Oct/3107,01 with an audience share of 11%04,01 || 15,0107,01 AO	\N	f	1
-486	-486	10,01Episode 15,01[1148615,01] Beckoning Cat from Right to Left04,01 || 15,01Subbed by [10DCTP-Kienai15,01] 07@ 15,01170mb | 320mb Codec: [XviD | H264] CRC: [848045F5 | A67266F2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=91UZUA9Q | 720: http://www.megaupload.com/?d=OU2V41QO04,01 || 07Aired 2007/Sep/0307,01 with an audience share of 7%04,01 || 15,0107,01 AO	\N	f	3
-594	-594	10,01Episode 15,01[1159415,01] The Seven Wonders of the Hiroshima Miyajima Tour (Part 1: Miyajima)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [E83101ED | 1E82699E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K4L6BO13 | 720: http://www.megaupload.com/?d=B997VF5C04,01 || 07Aired 2010/Nov/0607,0104,01 || 15,0107,01 AO	\N	f	2
-495	-495	10,01Episode 15,01[1149515,01] Clash of Red and Black: Coma04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [3BFE1F11 | F9314E79]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=S67HB5V6 | 720: http://www.megaupload.com/?d=LZVD8JYA04,01 || 07Aired 2008/Feb/1107,01 with an audience share of 9.2%04,01 || 07,01 Based on Vol 57 File 9 - Vol 58 File 2	\N	f	1
-509	-509	10,01Episode 15,01[1150915,01] Red, White, Yellow and the Detective Boys04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [AC1B7554 | 882B5470]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YFHKDX8H04,01 || 07Aired 2008/Jul/1407,01 with an audience share of 8.5%04,01 || 07,01 Based on Vol 60 File 3-5	\N	f	2
-218	-218	10,01Episode 15,01[1121815,01] Megure's Sealed Secret (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb Codec: [XviD] CRC: [5A6B4196]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NCWQBFTJ04,01 || 07Aired 2000/Dec/1807,01 with an audience share of 18.1%04,01 || 07,01 Based on Vol 28 File 11 - Vol 29 File 2	\N	f	2
-43	-43	10,01Episode 15,01[114315,01] Edogawa Conan Kidnapping Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01160mb CRC: [A43EE10B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=C9NSB7A304,01 || 07Aired 1997/Jan/1307,01 with an audience share of 18.8%04,01 || 07,01 Based on Vol 5 File 10 - Vol 6 File 1	\N	f	0
-193	-193	10,01Episode 15,01[1119315,01] The Desperate Revival: The Promised Place04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [EC557BF7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K017X4DX04,01 || 07Aired 2000/Jun/0507,01 with an audience share of 20.1%04,01 || 07,01 Based on Vol 25 File 9 - Vol 26 File 7	\N	f	0
-180	-180	10,01Episode 15,01[1118015,01] The Nocturne of the Red Murderous Intent (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [D6757907]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7BW3HXN504,01 || 07Aired 2000/Feb/1407,01 with an audience share of 20.1%04,01 || 15,0107,01 AO	\N	f	0
-244	-244	10,01Episode 15,01[1124415,01] Mori Kogoro's Imposter (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 Codec: [XviD] CRC: [EAA4972B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NW97NYLK04,01 || 07Aired 2001/Jul/3007,01 with an audience share of 18.4%04,01 || 07,01 Based on Vol 31 File 2-4	\N	f	0
-367	-367	10,01Episode 15,01[1136715,01] The Tradegy of the Pier in Plain Sight (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01176mb CRC: [7A27E55A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZK7RQE7L04,01 || 07Aired 2004/Jul/1207,01 with an audience share of 13%04,01 || 07,01 Based on Vol 45 File 3-5	\N	f	2
-288	-288	10,01Episode 15,01[1128815,01] Kudo Shinichi's NY Case (Part 3)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01226mb CRC: [C2A293F7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=380DRGD204,01 || 07Aired 2002/Jul/2907,01 with an audience share of 15.8%04,01 || 07,01 Based on Vol 34 File 11 - Vol 35 File 4	\N	f	2
-460	-460	10,01Episode 15,01[1146015,01] The Big Battle of 1-B04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01145mb CRC: [BFDC3111]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6F40D30804,01 || 07Aired 2007/Jan/1507,01 with an audience share of 9%04,01 || 07,01 Based on Vol 53 File 5-6	\N	f	1
-160	-160	10,01Episode 15,01[1116015,01] The Legend of the Mysterious Pagoda (Part 2)04,01 || 15,0107@ 15,01gb CRC: [B79A77D2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AX31KV1004,01 || 07Aired 1999/Sep/2007,01 with an audience share of 17.7%04,01 || 15,0107,01 AO	\N	f	4
-324	-324	10,01Episode 15,01[1132415,01] Hattori Heiji's Desperate Situation! (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [1FEAACB0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V0AHJVVG04,01 || 07Aired 2003/Jun/1607,01 with an audience share of 15.6%04,01 || 07,01 Based on Vol 38 File 8-10	\N	f	1
-98	-98	10,01Episode 15,01[119815,01] The Famous Potter Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [A314FCC4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YVKRHG6Q04,01 || 07Aired 1998/Apr/2007,01 with an audience share of 19.5%04,01 || 07,01 Based on Vol 16 File 10 - Vol 17 File 2	\N	f	1
-119	-119	10,01Episode 15,01[1111915,01] The Kamen Yaiba Murder Case04,01 || 15,01Subbed by [10Conan-A15,01] 07@ 15,01172mb Codec: [H264] CRC: [82B6F710]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=EDN0F52B04,01 || 07Aired 1998/Oct/1207,01 with an audience share of 18%04,01 || 15,0107,01 AO	\N	f	3
-351	-351	10,01Episode 15,01[1135115,01] The Forgetten Cellphone (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 MB Codec: [XviD] CRC: [DA59B622]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HHSW0S7304,01 || 07Aired 2004/Feb/1607,01 with an audience share of 12.2%04,01 || 07,01 Based on Vol 43 File 3-5	\N	f	1
-8	-8	10,01Episode 15,01[11815,01] Art Gallery Owner Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01209mb CRC: [B70E0F42]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SMUDW0JK04,01 || 07Aired 1996/Feb/2607,01 with an audience share of 13.1%04,01 || 07,01 Based on Vol 4 File 1-3	\N	f	5
-485	-485	10,01Episode 15,01[1148515,01] The Location of the Black Photograph (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 234mb CRC: [281B2896 | CE4AD850]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7Z5OL2QW | 720: http://www.megaupload.com/?d=C5H0C66O04,01 || 07Aired 2007/Aug/2707,01 with an audience share of 7.7%04,01 || 07,01 Based on Vol 56 File 7-9	\N	f	2
-21	-21	10,01Episode 15,01[112115,01] On Location, TV Drama Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01178mb CRC: [C89D6A73]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4HV5VUUA04,01 || 07Aired 1996/Jun/2407,01 with an audience share of 13.4%04,01 || 15,0107,01 AO	\N	f	4
-586	-586	10,01Episode 15,01[1158615,01] The Kirin's Horn that Vanished into the Dark04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [B925D8BC | A3813FD2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NS23VQNP | 720: http://www.megaupload.com/?d=Z79WM5ZG04,01 || 07Aired 2010/Sept/0407,0104,01 || 07,01 Based on Vol 68 File 5-8	\N	f	2
-569	-569	10,01Episode 15,01[1156915,01] Inspector Shiratori, Memories of the Cherry Blossom (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [F2A4A477 | CC638498]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VX0YSLRG | 720: http://www.megaupload.com/?d=2Y40UCIY04,01 || 07Aired 2010/Mar/2007,0104,01 || 07,01 Based on Vol 66 File 2-4	\N	f	3
-523	-523	10,01Episode 15,01[1152315,01] What She Truly Wants to Ask04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [BE5261BC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BGSQID1N04,01 || 07Aired 2009/Feb/0207,01 with an audience share of 7.9%04,01 || 07,01 Based on Vol 63 File 1-2	\N	f	3
-453	-453	10,01Episode 15,01[1145315,01] Preview Meeting of Friendship and Fate04,01 || 15,0107@ 15,01gb CRC: [77E1DA00]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XNGZWCIA04,01 || 07Aired 2006/Oct/2307,01 with an audience share of 9%04,01 || 07,01 Based on Vol 52 File 1-2   (First HD)	\N	f	4
-189	-189	10,01Episode 15,01[1118915,01] The Desperate Revival: A Wounded Great Detective04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01176mb CRC: [BCDE6234]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4C9W6ANW04,01 || 07Aired 2000/May/0807,01 with an audience share of 20.6%04,01 || 07,01 Based on Vol 25 File 9 - Vol 26 File 7	\N	f	0
-61	-61	10,01Episode 15,01[116115,01] A Ghost Ship Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [26D24C4C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5I5D0NP104,01 || 07Aired 1997/Jun/0207,01 with an audience share of 19.1%04,01 || 15,0107,01 AO	\N	f	1
-341	-341	10,01Episode 15,01[1134115,01] Hidden Bathroom Secret (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [D5A62685]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z842F6BG04,01 || 07Aired 2003/Nov/1007,01 with an audience share of 13.1%04,01 || 07,01 Based on Vol 41 File 10 - Vol 42 File 1	\N	f	1
-430	-430	10,01Episode 15,01[1143015,01] Two People Who Can't Return Yet (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01176mb CRC: [C84B3664]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4GUWAEIM04,01 || 07Aired 2006/Feb/1307,01 with an audience share of 9.7%04,01 || 07,01 Based on Vol 49 File 8-10	\N	f	2
-438	-438	10,01Episode 15,01[1143815,01] The Pursuit of the Fish Mail04,01 || 15,0107@ 15,01gb CRC: [3CF87E6C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=786SL1DE04,01 || 07Aired 2006/May/1507,01 with an audience share of 11%04,01 || 07,01 Based on Vol 51 File 2-3	\N	f	2
-113	-113	10,01Episode 15,01[1111315,01] The White Sand Beach Murder Case04,01 || 15,01Subbed by [10Atavus15,01] 07@ 15,01171mb Codec: [XviD] CRC: [83E384F7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=055QDKT704,01 || 07Aired 1998/Aug/1707,01 with an audience share of 18.4%04,01 || 07,01 Based on Special Vol 1 File 6	\N	f	2
-379	-379	10,01Episode 15,01[1137915,01] Case of the Long Sleeved Kimono at the Hidden Hot Spring on a Snowy Night (Part 1)04,01 || 15,0107@ 15,01gb CRC: [B468E254]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=OSVKVVT904,01 || 07Aired 2004/Nov/2207,01 with an audience share of 13.5%04,01 || 15,0107,01 AO	\N	f	6
-152	-152	10,01Episode 15,01[1115215,01] The Mysterious Old Man Disappearance Case04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [1D95568B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=J4H708M404,01 || 07Aired 1999/Jul/1207,01 with an audience share of 19.6%04,01 || 15,0107,01 AO	\N	f	1
-415	-415	10,01Episode 15,01[1141515,01] The Evil Spirit Appeared on an Unlucky Day (Part 1: The Case)04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01180mb CRC: [D5C6D422]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V9K6Y60W04,01 || 07Aired 2005/Oct/1007,01 with an audience share of 11.3%04,01 || 07,01 Based on Vol 48 File 4-8	\N	f	1
-410	-410	10,01Episode 15,01[1141015,01] Simultaneous Stages of Advancing and Kidnapping (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [67F54DB3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KJSSONKL04,01 || 07Aired 2005/Aug/1507,01 with an audience share of 10.3%04,01 || 15,0107,01 AO04,01 || 07,01http://www.anirena.com/viewtracker.php?action=download&id=38163	\N	f	5
-246	-246	10,01Episode 15,01[1124615,01] The Mystery in the Net (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01347mb Codec: [XviD] CRC: [5F7ECB81]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BZJEMGS904,01 || 07Aired 2001/Aug/1307,01 with an audience share of 13.6%04,01 || 07,01 Based on Vol 31 File 5-704,01 || 07,01Part 2 is concatenated.	\N	f	1
-595	-595	10,01Episode 15,01[1159515,01] The Seven Wonders of the Hiroshima Miyajima Tour (Part 2: Hiroshima)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [2E47E0E2 | 68E4988D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GVT6DKER | 720: http://www.megaupload.com/?d=UH7MJ7E004,01 || 07Aired 2010/Nov/1307,0104,01 || 15,0107,01 AO	\N	f	2
-480	-480	10,01Episode 15,01[1148015,01] Yellow Alibi04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [3050B6D1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WMPCE3C004,01 || 07Aired 2007/Jul/2307,01 with an audience share of 9.4%04,01 || 15,0107,01 AO	\N	f	4
-536	-536	10,01Episode 15,01[1153615,01] The Secret of the Vanished Masterpiece04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [35BEA4EF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ESMJ4MKJ04,01 || 07Aired 2009/May/3007,01 with an audience share of 6.7%04,01 || 15,0107,01 AO	\N	f	1
-579	-579	10,01Episode 15,01[1157915,01] The Suggestion of Black Thirteen04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [BDCCE830 | FCD1AE48]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YZJJ4YK0 | 720: http://www.megaupload.com/?d=E81XCNCS04,01 || 07Aired 2010/Jul/0307,0104,01 || 07,01 Based on Vol 67 Files 4-8	\N	f	3
-447	-447	10,01Episode 15,01[1144715,01] The Sealed Western-style Window (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01176mb CRC: [FFEEF6B8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CFTF18XG04,01 || 07Aired 2006/Jul/3107,01 with an audience share of 8.4%04,01 || 07,01 Based on Vol 51 File 9-11	\N	f	1
-488	-488	10,01Episode 15,01[1148815,01] The Devil of the TV Station (1 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01351mb Codec: [XviD] CRC: [E1AF0CC1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZXUV0EV104,01 || 07Aired 2007/Oct/2207,01 with an audience share of 9.4%04,01 || 07,01 Based on Vol 57 File 5-7	\N	f	2
-342	-342	10,01Episode 15,01[1134215,01] The Bride of Huis Ten Bosch (1 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01351mb Codec: [XviD] CRC: [AEA537AB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Y1C0XSE504,01 || 07Aired 2003/Nov/1707,01 with an audience share of 12.9%04,01 || 15,0107,01 AO	\N	f	2
-452	-452	10,01Episode 15,01[1145215,01] The Phantom of the Theatre (1 Hour Special)04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01350mb CRC: [B3A9066D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L9PNYQ3004,01 || 07Aired 2006/Sep/1107,01 with an audience share of 9.1%04,01 || 15,0107,01 AO	\N	f	2
-87	-87	10,01Episode 15,01[118715,01] The Crane's Return of a Favor Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [D5C714FA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GP64P1KI04,01 || 07Aired 1998/Jan/1907,01 with an audience share of 20.8%04,01 || 15,0107,01 AO	\N	f	0
-39	-39	10,01Episode 15,01[113915,01] The Wealthy Daughter Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01163mb CRC: [A20A5263]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4HLHV0TF04,01 || 07Aired 1996/Nov/2507,01 with an audience share of 17.4%04,01 || 07,01 Based on Vol 9 File 7 - Vol 10 File 1	\N	f	0
-45	-45	10,01Episode 15,01[114515,01] Facial Pack Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01170mb CRC: [76BC4D37]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=IMZ61GS004,01 || 07Aired 1997/Jan/2707,01 with an audience share of 17%04,01 || 15,0107,01 AO	\N	f	0
-444	-444	10,01Episode 15,01[1144415,01] Gathering Shellfish With a Sigh (Part 2)04,01 || 15,0107@ 15,01gb CRC: [42F90667]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZRNPUFHF04,01 || 07Aired 2006/Jul/0307,01 with an audience share of 8.3%04,01 || 07,01 Based on Vol 51 File 4-6	\N	f	1
-535	-535	10,01Episode 15,01[1153515,01] An Old Scar & The Detective's Spirit04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01169mb Codec: [XviD] CRC: [BC055C0C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UYTUK7KM04,01 || 07Aired 2009/May/2307,01 with an audience share of 7.2%04,01 || 07,01 Based on Vol 64 File 7-10	\N	f	1
-64	-64	10,01Episode 15,01[116415,01] The Third Fingerprint Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [49C7C8E2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SXCPB85X04,01 || 07Aired 1997/Jun/2307,01 with an audience share of 18%04,01 || 15,0107,01 AO	\N	f	2
-370	-370	10,01Episode 15,01[1137015,01] Running Away in a Game04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01176mb CRC: [9E9786BD]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NL3US52R04,01 || 07Aired 2004/Aug/0907,01 with an audience share of 11.5%04,01 || 15,0107,01 AO	\N	f	2
-417	-417	10,01Episode 15,01[1141715,01] The Evil Spirit Appeared on an Unlucky Day (Part 3: The Solution)04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01157mb CRC: [0F57C657]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VSSSWNJ804,01 || 07Aired 2005/Oct/2407,01 with an audience share of 11.1%04,01 || 07,01 Based on Vol 48 File 4-8	\N	f	1
-221	-221	10,01Episode 15,01[1122115,01] The Client Full of Lies (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb CRC: [0AC8F430]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=17C0PMYZ04,01 || 07Aired 2001/Jan/2207,01 with an audience share of 20.3%04,01 || 07,01 Based on Vol 28 File 3-5	\N	f	1
-12	-12	10,01Episode 15,01[111215,01] Ayumi-chan's Kidnapping Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01231mb CRC: [1CAD6E80]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YZJW2EM004,01 || 07Aired 1996/Apr/1507,01 with an audience share of 13.6%04,01 || 07,01 Based on Vol 9 File 1-3	\N	f	4
-77	-77	10,01Episode 15,01[117715,01] A Distinguished Family's Consecutive Accidental Death Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [22ED7172]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0DTY3XL004,01 || 07Aired 1997/Oct/2007,01 with an audience share of 19.1%04,01 || 07,01 Based on Vol 15 File 10 - Vol 16 File 3	\N	f	1
-107	-107	10,01Episode 15,01[1110715,01] Mogura Star Man Murder Case (Part 1)04,01 || 15,01Subbed by [10Atavus15,01] 07@ 15,01172mb Codec: [XviD] CRC: [D1FA39FC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7S7PJ3BP04,01 || 07Aired 1998/Jun/2907,01 with an audience share of 18.5%04,01 || 15,0107,01 AO	\N	f	1
-507	-507	10,01Episode 15,01[1150715,01] Blind Spot in the Karaoke Box (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [E5B22EB5 | 3E5A0B8D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QZLCKN3R04,01 || 07Aired 2008/Jun/3007,01 with an audience share of 6.3%04,01 || 07,01 Based on Vol 59 File 11 - Vol 60 File 2	\N	f	4
-312	-312	10,01Episode 15,01[1131215,01] Festival Dolls Dyed in the Setting Sun (Part 1)04,01 || 15,0107@ 15,01gb Codec: [XviD] CRC: [E1DD441F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Y8F6QOPV04,01 || 07Aired 2003/Mar/0307,01 with an audience share of 16.1%04,01 || 07,01 Based on Vol 38 File 2-4	\N	f	3
-541	-541	10,01Episode 15,01[1154115,01] The Day Mouri Kogorou Ceased Being A Detective (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb Codec: [XviD] CRC: [8850C4A3 | D270660B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2PCI9Q8Y04,01 || 07Aired 2009/July/1807,01 with an audience share of 5%04,01 || 15,0107,01 AO	\N	f	1
-405	-405	10,01Episode 15,01[1140515,01] The Man Who Went to Call For an Ambulance04,01 || 15,0107@ 15,01gb CRC: [546F4B5D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Y0OD3QLD04,01 || 07Aired 2005/Jul/0407,01 with an audience share of 13%04,01 || 15,0107,01 AO	\N	f	3
-573	-573	10,01Episode 15,01[1157315,01] The Whereabouts of the Embarrasing Charm (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 321mb Codec: [XviD | H264] CRC: [1C686E18 | CD729068]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YMCYJ9G8 | 720: http://www.megaupload.com/?d=9Y5F693A04,01 || 07Aired 2010/May/1507,0104,01 || 07,01 Based on Vol 66 File 8-10	\N	f	5
-558	-558	10,01Episode 15,01[1155815,01] The Mansion of Death and the Red Wall: Three Visits04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD] CRC: [FF51923A | 2D18F3C4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6TMOYQC1 | 720: http://www.megaupload.com/?d=QLV5EP7B04,01 || 07Aired 2009/Dec/507,0104,01 || 07,01 Based on Vol 65 File 8 - Vol 66 File 1	\N	f	2
-508	-508	10,01Episode 15,01[1150815,01] Blind Spot in the Karaoke Box (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [1547A040 | 48087D4B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PX6LGO6304,01 || 07Aired 2008/Jul/0707,01 with an audience share of 7.9%04,01 || 07,01 Based on Vol 59 File 11 - Vol 60 File 2	\N	f	3
-100	-100	10,01Episode 15,01[1110015,01] The Memories of First Love Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [D0B6ADD7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MO68NX6704,01 || 07Aired 1998/May/1107,01 with an audience share of 20.3%04,01 || 07,01 Based on Vol 18 File 3-5	\N	f	0
-381	-381	10,01Episode 15,01[1138115,01] Which One's Reasoning Show? (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [8D3B781E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RKJBYG9004,01 || 07Aired 2004/Dec/0607,01 with an audience share of 13.5%04,01 || 07,01 Based on Vol 43 File 6-9	\N	f	2
-286	-286	10,01Episode 15,01[1128615,01] Kudo Shinichi's NY Case (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01225mb CRC: [28AA5843]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=TRM8US8K04,01 || 07Aired 2002/Jul/1507,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 34 File 11 - Vol 35 File 4	\N	f	1
-309	-309	10,01Episode 15,01[1130915,01] Contact with the Black Organization (Part 1: The Negotiation)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01226mb CRC: [89764AC0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=C4Z8X2JO04,01 || 07Aired 2003/Feb/1007,01 with an audience share of 13.4%04,01 || 07,01 Based on Vol 37 File 8 - Vol 38 File 1	\N	f	2
-1	-1	10,01Episode 15,01[11115,01] Roller Coaster Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01200mb Codec: [XviD] CRC: [B58E4700]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HF35FVPU04,01 || 07Aired 1996/Jan/0807,01 with an audience share of 8.5%04,01 || 07,01 Based on Vol 1 File 1	\N	f	21
-380	-380	10,01Episode 15,01[1138015,01] Case of the Long Sleeved Kimono at the Hidden Hot Spring on a Snowy Night (Part 2)04,01 || 15,0107@ 15,01gb CRC: [43773BC2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QZUADL4504,01 || 07Aired 2004/Nov/2907,01 with an audience share of 11.8%04,01 || 15,0107,01 AO	\N	f	5
-120	-120	10,01Episode 15,01[1112015,01] The Honey Cocktail Murder Case04,01 || 15,01Subbed by [10Puto-F15,01] 07@ 15,01250MB Codec: [H264] CRC: [0d6811d4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AJ2A8G9Y04,01 || 07Aired 1998/Oct/1907,01 with an audience share of 18.2%04,01 || 15,0107,01 AO	\N	f	2
-416	-416	10,01Episode 15,01[1141615,01] The Evil Spirit Appeared on an Unlucky Day (Part 2: The Suspicion)04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01159mb CRC: [DEE9936A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ASXBJ48504,01 || 07Aired 2005/Oct/1707,01 with an audience share of 10.6%04,01 || 07,01 Based on Vol 48 File 4-8	\N	f	1
-581	-581	10,01Episode 15,01[1158115,01] The Red Shaking Target04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [01139EB3 | 4414B661]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3V6W0JPB | 720: http://www.megaupload.com/?d=Q8NZCMRQ04,01 || 07Aired 2010/Jul/1707,0104,01 || 07,01 Based on Vol 67 Files 4-8	\N	f	4
-177	-177	10,01Episode 15,01[1117715,01] Meeting with the Black Organization Again (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [CAC93F36]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2SGURN3104,01 || 07Aired 2000/Jan/2407,01 with an audience share of 18.8%04,01 || 07,01 Based on Vol 24 File 7-11	\N	f	1
-308	-308	10,01Episode 15,01[1130815,01] The Dark Footprint (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01226mb CRC: [D9C056A4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LQO8YCI604,01 || 07Aired 2003/Feb/0307,01 with an audience share of 16.2%04,01 || 07,01 Based on Vol 37 File 5-7	\N	f	2
-496	-496	10,01Episode 15,01[1149615,01] Clash of Red and Black: Invasion04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [832C75DF | DED5E6AE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZMKUQWPK | 720: http://www.megaupload.com/?d=6P6QH25Q04,01 || 07Aired 2008/Feb/1807,01 with an audience share of 8.2%04,01 || 07,01 Based on Vol 57 File 9 - Vol 58 File 2	\N	f	1
-136	-136	10,01Episode 15,01[1113615,01] The Old Castle Investigation Case (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [4DE982A2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LYHQL67Y04,01 || 07Aired 1999/Feb/2207,01 with an audience share of 22.8%04,01 || 07,01 Based on Vol 20 File 10 - Vol 21 File 3	\N	f	1
-392	-392	10,01Episode 15,01[1139215,01] The Mysterious Height Difference of 20cm04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [893B9CDB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=8VL65Z0P04,01 || 07Aired 2005/Mar/1407,01 with an audience share of 12.5%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 19	\N	f	7
-116	-116	10,01Episode 15,01[1111615,01] The Mystery Writer Disappearance Case (Part 1)04,01 || 15,01Subbed by [10Puto-F15,01] 07@ 15,01404MB Codec: [H264-AAC] CRC: [de80222a]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YPZQBEBS04,01 || 07Aired 1998/Sep/1407,01 with an audience share of 19.7%04,01 || 07,01 Based on Vol 19 File 2-4	\N	f	10
-144	-144	10,01Episode 15,01[1114415,01] The North Star #3 Express Leaving Ueno (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [86A82794]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NUVF8BY104,01 || 07Aired 1999/May/1007,01 with an audience share of 18.9%04,01 || 07,01 Based on Vol 22 File 4-7	\N	f	3
-567	-567	10,01Episode 15,01[1156715,01] Murderous Intent Raining on an Outdoor Spa04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [9072ADFF | E3C34FD7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1NEA2BRJ | 720: http://www.megaupload.com/?d=1Y7ZCZIP04,01 || 07Aired 2010/Mar/0607,0104,01 || 15,0107,01 AO	\N	f	3
-506	-506	10,01Episode 15,01[1150615,01] Lawyer Kisaki Eri's Testimony (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [D60A3ACF | 1CB1C2F7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RPOO0CNJ04,01 || 07Aired 2008/Jun/2307,01 with an audience share of 8%04,01 || 07,01 Based on Vol 59 File 2-4	\N	f	3
-436	-436	10,01Episode 15,01[1143615,01] Information Gathered on the Detective Boys (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01163mb CRC: [A4A855A6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AVI9LQT304,01 || 07Aired 2006/Apr/2407,01 with an audience share of 11.6%04,01 || 07,01 Based on Vol 50 File 5-7	\N	f	0
-94	-94	10,01Episode 15,01[119415,01] Snow Woman Legend Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01173mb CRC: [FA49D107]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5UM977CB04,01 || 07Aired 1998/Mar/0907,01 with an audience share of 21.2%04,01 || 15,0107,01 AO	\N	f	0
-171	-171	10,01Episode 15,01[1117115,01] The Unseen Angle of the Darkness (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01162mb CRC: [455414B1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BFQCBNL604,01 || 07Aired 1999/Dec/0607,01 with an audience share of 17.9%04,01 || 07,01 Based on Vol 24 File 3-6	\N	f	0
-265	-265	10,01Episode 15,01[1126515,01] The Courtroom Battle: Kisaki VS Kogoro (Part 2)04,01 || 15,0107@ 15,01gb CRC: [D5A317D1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XM6IVT6D04,01 || 07Aired 2002/Jan/2107,01 with an audience share of 21.3%04,01 || 15,0107,01 AO	\N	f	7
-213	-213	10,01Episode 15,01[1121315,01] Mushrooms, Bears, and the Detective Boys (Part 2)04,01 || 15,0107@ 15,01gb CRC: [CA243F6D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L9CS3D3U04,01 || 07Aired 2000/Nov/1307,01 with an audience share of 19.7%04,01 || 07,01 Based on Vol 27 File 10 - Vol 28 File 2	\N	f	9
-386	-386	10,01Episode 15,01[1138615,01] The Dissonance of the Stradivarius (Part 2: Interlude)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [D8475642]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=8TFBEXT304,01 || 07Aired 2005/Jan/3107,01 with an audience share of 12.4%04,01 || 07,01 Based on Vol 46 File 2-6	\N	f	2
-205	-205	10,01Episode 15,01[1120515,01] The Metropolitan Police Detective Love Story 3 (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [EBBF9669]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0WP3RKB704,01 || 07Aired 2000/Aug/2807,01 with an audience share of 19.2%04,01 || 07,01 Based on Vol 27 File 4-6	\N	f	2
-22	-22	10,01Episode 15,01[112215,01] Luxury Liner Serial Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01178mb CRC: [700BAC97]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=O2EGFKW004,01 || 07Aired 1996/Jul/0107,01 with an audience share of 12.8%04,01 || 07,01 Based on Vol 3 File 1-6	\N	f	1
-235	-235	10,01Episode 15,01[1123515,01] The Locked Wine Cellar04,01 || 15,01Subbed by [10AZFS-BG15,01] 07@ 15,01171mb Codec: [XviD] CRC: [B1F1C7CC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NQPD5QFW04,01 || 07Aired 2001/May/2807,01 with an audience share of 17.8%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 11	\N	f	1
-301	-301	10,01Episode 15,01[1130115,01] Parade of Malice and Saint (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01343mb CRC: [3722E962]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K8TYY6KD04,01 || 07Aired 2002/Nov/2507,01 with an audience share of 15.4%04,01 || 07,01 Based on Vol 36 File 5-7	\N	f	4
-456	-456	10,01Episode 15,01[1145615,01] The Mystery I Loved04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01162mb CRC: [777634B0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=O99HJ8YW04,01 || 07Aired 2006/Nov/1307,01 with an audience share of 9.7%04,01 || 15,0107,01 AO	\N	f	1
-399	-399	10,01Episode 15,01[1139915,01] Weird Family's Request (Part 2)04,01 || 15,0107@ 15,01gb CRC: [E8DDD693]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ITOEAXZX04,01 || 07Aired 2005/May/2307,01 with an audience share of 12.2%04,01 || 07,01 Based on Vol 46 File 11 - Vol 47 File 3	\N	f	3
-51	-51	10,01Episode 15,01[115115,01] The Golf Driving Range Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [445A16D5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JZCJYD7X04,01 || 07Aired 1997/Mar/1007,01 with an audience share of 15.3%04,01 || 15,0107,01 AO	\N	f	1
-605	-605	10,01Episode 15,01[1160515,01] The Seance's Double Locked Room Mystery Case (Opened Locked Room)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [2D33D524/8B13241E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=INYQ8XX5 | 720: http://www.megaupload.com/?d=2R74JYIY04,01 || 07Aired 2/12/201107,01	\N	f	2
-493	-493	10,01Episode 15,01[1149315,01] Clash of Red and Black: Exclamation04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [FD72DF16 | 5F855570]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=A4T5PQ37 | 720: http://www.megaupload.com/?d=WTQDTTIG04,01 || 07Aired 2008/Jan/2807,01 with an audience share of 9.5%04,01 || 07,01 Based on Vol 56 File 10 - Vol 57 File 4	\N	f	1
-373	-373	10,01Episode 15,01[1137315,01] Poisonous Spider Trap04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01171mb CRC: [FA170B2B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=10MR33YL04,01 || 07Aired 2004/Sep/0607,01 with an audience share of 12.6%04,01 || 15,0107,01 AO	\N	f	2
-42	-42	10,01Episode 15,01[114215,01] A Karaoke Box Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01160mb CRC: [9B7C6A45]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=115A76M304,01 || 07Aired 1996/Dec/1607,01 with an audience share of 17.5%04,01 || 07,01 Based on Vol 5 File 6-9	\N	f	1
-568	-568	10,01Episode 15,01[1156815,01] Inspector Shiratori, Memories of the Cherry Blossom (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 320mb Codec: [XviD | H264] CRC: [350C0F45 | CE9865AA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6MJD7FZ4 | 720: http://www.megaupload.com/?d=ZCYNHJ0W04,01 || 07Aired 2010/Mar/1307,0104,01 || 07,01 Based on Vol 66 File 2-4	\N	f	3
-445	-445	10,01Episode 15,01[1144515,01] Secret of the Russian Blue04,01 || 15,0107@ 15,01gb CRC: [1A1F18BE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DND0OV1V04,01 || 07Aired 2006/Jul/1007,01 with an audience share of 9.3%04,01 || 07,01 Based on Vol 51 File 7-8	\N	f	2
-27	-27	10,01Episode 15,01[112715,01] Kogoro's Class Reunion Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01168mb CRC: [29D2940D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=H5F29J7304,01 || 07Aired 1996/Aug/0507,01 with an audience share of 13.4%04,01 || 07,01 Based on Vol 9 File 4-6	\N	f	0
-366	-366	10,01Episode 15,01[1136615,01] The Tragedy of the Pier in Plain Sight (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01176mb CRC: [F8AD935E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=OTHM9CPE04,01 || 07Aired 2004/Jul/0507,01 with an audience share of 13.2%04,01 || 07,01 Based on Vol 45 File 3-5	\N	f	2
-329	-329	10,01Episode 15,01[1132915,01] A Friendship That Can't Be Bought (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [FDFCDF93]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=B7ZZKGW904,01 || 07Aired 2003/Jul/2807,01 with an audience share of 16%04,01 || 07,01 Based on Vol 39 File 6-8	\N	f	1
-525	-525	10,01Episode 15,01[1152515,01] Blue Spark of Hate (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [33E5E754]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2O5T201904,01 || 07Aired 2009/Feb/1607,01 with an audience share of 9.4%04,01 || 07,01 Based on Vol 61 File 5-8	\N	f	2
-441	-441	10,01Episode 15,01[1144115,01] The Final "Ahh"04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [EEF44907]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=P3F5Q0CP04,01 || 07Aired 2006/Jun/0507,01 with an audience share of 8.7%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 28	\N	f	1
-582	-582	10,01Episode 15,01[1158215,01] The Night the Zombie Died04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD | H264] CRC: [8F0914FD | BE21C825]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MELJGJVH | 720: http://www.megaupload.com/?d=P4FFR9UC04,01 || 07Aired 2010/Jul/2407,0104,01 || 15,0107,01 AO	\N	f	2
-287	-287	10,01Episode 15,01[1128715,01] Kudo Shinichi's NY Case (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01225mb CRC: [3813C54A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=E1510SHN04,01 || 07Aired 2002/Jul/2207,01 with an audience share of 16.5%04,01 || 07,01 Based on Vol 34 File 11 - Vol 35 File 4	\N	f	3
-200	-200	10,01Episode 15,01[1120015,01] Kogoro Mouri, Suspect (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01172mb CRC: [3115FE41]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=O8ZKXTYL04,01 || 07Aired 2000/Jul/2407,01 with an audience share of 19.2%04,01 || 07,01 Based on Vol 27 File 1-3	\N	f	5
-272	-272	10,01Episode 15,01[1127215,01] Secret Rushed Omission (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [1F93FA50]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UFOCUL7J04,01 || 07Aired 2002/Mar/1807,01 with an audience share of 20.9%04,01 || 07,01 Based on Vol 33 File 10 - Vol 34 File 1	\N	f	2
-468	-468	10,01Episode 15,01[1146815,01] Mysterious Case Near the Pond04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [5D4E105E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=E6B4589104,01 || 07Aired 2007/Mar/1207,01 with an audience share of 10%04,01 || 15,0107,01 AO	\N	f	2
-161	-161	10,01Episode 15,01[1116115,01] The Murder Floating in the Stream Restaurant04,01 || 15,01Subbed by [10Neg-Kaizou15,01] 07@ 15,01175mb CRC: [0517C3A8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SXNWE5OI04,01 || 07Aired 1999/Sep/2707,01 with an audience share of 20.1%04,01 || 15,0107,01 AO	\N	f	1
-20	-20	10,01Episode 15,01[112015,01] A Haunted Mansion Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01168mb CRC: [2D43F421]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ANOEG72A04,01 || 07Aired 1996/Jun/1707,01 with an audience share of 14.3%04,01 || 07,01 Based on Vol 2 File 7-10	\N	f	3
-400	-400	10,01Episode 15,01[1140015,01] Ran's Suspicions04,01 || 15,01Subbed by [10Shinsen-Subs15,01] 07@ 15,01174mb CRC: [0363196E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DI82P1CX04,01 || 07Aired 2005/May/3007,01 with an audience share of 12.3%04,01 || 07,01 Based on Vol 47 File 4	\N	f	4
-475	-475	10,01Episode 15,01[1147515,01] Bad Luck Grand Prix04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [E1FACDD4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FDHEBADC04,01 || 07Aired 2007/Jun/1807,01 with an audience share of 10%04,01 || 15,0107,01 AO	\N	f	1
-151	-151	10,01Episode 15,01[1115115,01] The Truth of the Exploding Car Case (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [CCDAFDE6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6SVZC9KW04,01 || 07Aired 1999/Jul/0507,01 with an audience share of 19.5%04,01 || 15,0107,01 AO	\N	f	2
-534	-534	10,01Episode 15,01[1153415,01] A New Scar & The Whistling Man04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [0950AC6A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZBT7Q7BY04,01 || 07Aired 2009/May/1607,01 with an audience share of 8.7%04,01 || 07,01 Based on Vol 64 File 7-10	\N	f	1
-109	-109	10,01Episode 15,01[1110915,01] Detective Club Pursuit Case04,01 || 15,01Subbed by [10Atavus15,01] 07@ 15,01172mb Codec: [XviD] CRC: [C64716F5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7M2FEO1T04,01 || 07Aired 1998/Jul/1307,01 with an audience share of 16.7%04,01 || 15,0107,01 AO	\N	f	2
-557	-557	10,01Episode 15,01[1155715,01] A Dangerous Party of Two04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01174mb | 320mb Codec: [XviD] CRC: [F3FF61BB | E8B25134]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9730P0AV | 720: http://www.megaupload.com/?d=0ZRVVVKE04,01 || 07Aired 2009/Nov/2807,0104,01 || 07,01 Based on Vol 65 File 6-7	\N	f	3
-413	-413	10,01Episode 15,01[1141315,01] The Half Completed Crime Mystery04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [AC7BE100]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JWRJC78B04,01 || 07Aired 2005/Sep/0507,01 with an audience share of 10.1%04,01 || 15,0107,01 AO04,01 || 07,01v2 fixed a styling error.	\N	f	2
-197	-197	10,01Episode 15,01[1119715,01] A Super Car's Trap (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01171mb CRC: [98765DA0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0LFF6WRK04,01 || 07Aired 2000/Jul/0307,01 with an audience share of 16.3%04,01 || 15,0107,01 AO	\N	f	0
-82	-82	10,01Episode 15,01[118215,01] The Kidnapping of a Popular Artist Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [A271CDFA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=461ZIWFW04,01 || 07Aired 1997/Nov/2407,01 with an audience share of 20.6%04,01 || 07,01 Based on Vol 15 File 4-6	\N	f	0
-137	-137	10,01Episode 15,01[1113715,01] The Old Castle Investigation Case (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [B202E1CF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6UPX0HZC04,01 || 07Aired 1999/Mar/0807,01 with an audience share of 23.4%04,01 || 07,01 Based on Vol 20 File 10 - Vol 21 File 3	\N	f	1
-346	-346	10,01Episode 15,01[1134615,01] Find the Buttocks' Mark (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [415766DC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XNN09RHW04,01 || 07Aired 2004/Jan/1207,01 with an audience share of 12.5%04,01 || 07,01 Based on Vol 42 File 11 - Vol 43 File 2	\N	f	5
-294	-294	10,01Episode 15,01[1129415,01] The Smash of Determination and Love (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01173mb Codec: [H264] CRC: [B6666771]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4YL5TP7T04,01 || 07Aired 2002/Sep/0907,01 with an audience share of 16.7%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 11	\N	f	2
-333	-333	10,01Episode 15,01[1133315,01] Alike Princesses (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [B66C1706]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CDPKFPLA04,01 || 07Aired 2003/Aug/2507,01 with an audience share of 12.9%04,01 || 07,01 Based on Vol 40 File 10 - Vol 41 File 3	\N	f	1
-427	-427	10,01Episode 15,01[1142715,01] Super Secret of the Road to School (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [54364B63]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KRVIXATU04,01 || 07Aired 2006/Jan/2307,01 with an audience share of 10.6%04,01 || 07,01 Based on Vol 49 File 5-715,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 25	\N	f	5
-110	-110	10,01Episode 15,01[1111015,01] Cooking Classroom Murder Case (Part 1)04,01 || 15,01Subbed by [10Atavus15,01] 07@ 15,01171mb Codec: [XviD] CRC: [1EB1F20A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DB2CJK6204,01 || 07Aired 1998/Jul/2707,01 with an audience share of 19.1%04,01 || 15,0107,01 AO	\N	f	1
-17	-17	10,01Episode 15,01[111715,01] Department Store Highjacking Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01181mb CRC: [C3D57183]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BWCI86J804,01 || 07Aired 1996/May/2707,01 with an audience share of 15.7%04,01 || 15,0107,01 AO	\N	f	2
-69	-69	10,01Episode 15,01[116915,01] The Night Baron Murder Case (Part 2: The Suspicion)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [9206D552]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3FMBPHNH04,01 || 07Aired 1997/Jul/2807,01 with an audience share of 18.2%04,01 || 07,01 Based on Vol 8 File 2-7	\N	f	1
-60	-60	10,01Episode 15,01[116015,01] An Illustrator Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [9A37D2D8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Y2IR7H0R04,01 || 07Aired 1997/May/2607,01 with an audience share of 19.5%04,01 || 07,01 Based on Vol 13 File 5-7	\N	f	2
-112	-112	10,01Episode 15,01[1111215,01] The Seven Mysteries of Teitan Elementary04,01 || 15,01Subbed by [10Atavus15,01] 07@ 15,01171mb Codec: [XviD] CRC: [B237EAD7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WYKB2C4U04,01 || 07Aired 1998/Aug/1007,01 with an audience share of 18.9%04,01 || 07,01 Based on Vol 16 File 4-5	\N	f	1
-138	-138	10,01Episode 15,01[1113815,01] The Final Screening Murder Case (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [33292D88]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZJU8523404,01 || 07Aired 1999/Mar/2207,01 with an audience share of 21.1%04,01 || 07,01 Based on Vol 23 File 1-3	\N	f	1
-515	-515	10,01Episode 15,01[1151515,01] Kaitou Kid's Teleportation Magic (1 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb CRC: [672A3AC6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XLS27GJI04,01 || 07Aired 2008/Oct/2007,01 with an audience share of 9.6%04,01 || 07,01 Based on Vol 61 File 1-4	\N	f	3
-519	-519	10,01Episode 15,01[1151915,01] The Meiji Restoration Mystery Tour (Part 2: Deciphering Chapter)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [67CA3A32]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0X8A5WRE04,01 || 07Aired 2008/Dec/0807,01 with an audience share of 7.7%04,01 || 15,0107,01 AO	\N	f	1
-590	-590	10,01Episode 15,01[1159015,01] The Worst Birthday (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [B857DD38 | DEDC48BA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=8IS1SVZA | 720: http://www.megaupload.com/?d=V4O677AQ04,01 || 07Aired 2010/Oct/0207,0104,01 || 07,01 Based on Vol 68 File 2-4	\N	f	2
-432	-432	10,01Episode 15,01[1143215,01] The Metropolitan Police Detective Love Story 7 (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01174mb CRC: [CD58B73F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=X6D1REYM04,01 || 07Aired 2006/Feb/2707,01 with an audience share of 9.1%04,01 || 07,01 Based on Vol 50 File 1-4	\N	f	1
-323	-323	10,01Episode 15,01[1132315,01] Hattori Heiji's Desperate Situation! (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [C13D5F40]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=R8YDTAZQ04,01 || 07Aired 2003/Jun/0907,01 with an audience share of 16.8%04,01 || 07,01 Based on Vol 38 File 8-10	\N	f	3
-187	-187	10,01Episode 15,01[1118715,01] The Mysterious Gun Shot in the Dark04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [17C11F91]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U9ASOUE104,01 || 07Aired 2000/Apr/2407,01 with an audience share of 21.6%04,01 || 15,0107,01 AO	\N	f	0
-212	-212	10,01Episode 15,01[1121215,01] Mushrooms, Bears, and the Detective Boys (Part 1)04,01 || 15,0107@ 15,01gb CRC: [CA243F6D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L9CS3D3U04,01 || 07Aired 2000/Nov/0607,01 with an audience share of 18.2%04,01 || 07,01 Based on Vol 27 File 10 - Vol 28 File 2	\N	f	1
-426	-426	10,01Episode 15,01[1142615,01] Love Letter to Ran04,01 || 15,0107@ 15,01gb CRC: [EF80A53A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=H4RPWNTR04,01 || 07Aired 2006/Jan/1607,01 with an audience share of 9.8%04,01 || 15,0107,01 AO	\N	f	3
-176	-176	10,01Episode 15,01[1117615,01] Meeting with the Black Organization Again (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [BABEBFDA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GWO4NW4904,01 || 07Aired 2000/Jan/1707,01 with an audience share of 22%04,01 || 07,01 Based on Vol 24 File 7-11	\N	f	1
-352	-352	10,01Episode 15,01[1135215,01] The Fishing Tournament Tragedy (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01347mb Codec: [XviD] CRC: [34715510]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7K75JP9Y04,01 || 07Aired 2004/Feb/2307,01 with an audience share of 13.6%04,01 || 15,0107,01 AO04,01 || 07,01Part 2 is concatenated.	\N	f	1
-372	-372	10,01Episode 15,01[1137215,01] A Course Without Protest (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01171mb CRC: [708F0077]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JWJZ3VW104,01 || 07Aired 2004/Aug/3007,01 with an audience share of 13.1%04,01 || 07,01 Based on Vol 45 File 6-8	\N	f	2
-483	-483	10,01Episode 15,01[1148315,01] The Vanished Policeman04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [A66A1F55]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=H4Y5IU9J04,01 || 07Aired 2007/Aug/1307,01 with an audience share of 6.7%04,01 || 15,0107,01 AO	\N	f	2
-191	-191	10,01Episode 15,01[1119115,01] The Desperate Revival: The Black Knight04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [C17911F7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7Z33C4KB04,01 || 07Aired 2000/May/2207,01 with an audience share of 20.2%04,01 || 07,01 Based on Vol 25 File 9 - Vol 26 File 7	\N	f	1
-224	-224	10,01Episode 15,01[1122415,01] And Then There Were No Mermaids (Part 3: The Solution)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb CRC: [1388584C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VVHLEWU504,01 || 07Aired 2001/Feb/1207,01 with an audience share of 19.9%04,01 || 07,01 Based on Vol 28 File 6-10	\N	f	2
-79	-79	10,01Episode 15,01[117915,01] The Bank Heist Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [B266E4D0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MB0XX7HS04,01 || 07Aired 1997/Nov/0307,01 with an audience share of 18.9%04,01 || 15,0107,01 AO	\N	f	1
-262	-262	10,01Episode 15,01[1126215,01] The Fearful Legend of the Snowy Night (Part 2)04,01 || 15,01Subbed by [10Kienai-AZFS15,01] 07@ 15,01172mb Codec: [h264] CRC: [47F6A339]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5EGFXYOB04,01 || 07Aired 2001/Dec/1707,01 with an audience share of 18.8%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 23	\N	f	2
-256	-256	10,01Episode 15,01[1125615,01] The 14th Round of the Matsue Tamatsukuri Linked Verse Contest (Part 2)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [5221C707]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6F5YCKRN04,01 || 07Aired 2001/Nov/0507,01 with an audience share of 18.6%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 19	\N	f	3
-588	-588	10,01Episode 15,01[1158815,01] The Trap of the Rooftop Farm04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [8A1D3BE4 | D505AE2D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1N30R2SP | 720: http://www.megaupload.com/?d=UZUK35U604,01 || 07Aired 2010/Sept/1807,0104,01 || 15,0107,01 AO	\N	f	2
-369	-369	10,01Episode 15,01[1136915,01] The Lucky Man's Suspense04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01176mb CRC: [9788B705]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=55G7218D04,01 || 07Aired 2004/Aug/0207,01 with an audience share of 13.7%04,01 || 15,0107,01 AO	\N	f	2
-498	-498	10,01Episode 15,01[1149815,01] Clash of Red and Black: Disturbance04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [D01E9532 | 6AD78584]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7P786YUI | 720: http://www.megaupload.com/?d=YQ9OGARV04,01 || 07Aired 2008/Mar/0307,01 with an audience share of 10%04,01 || 07,01 Based on Vol 58 File 3 - Vol 58 File 7	\N	f	1
-279	-279	10,01Episode 15,01[1127915,01] The Hooligan's Labyrinth (Part 1)04,01 || 15,01Subbed by [10MCT15,01] 07@ 15,01gb Codec: [h264] CRC: [677B2F46]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WGIAEJSY04,01 || 07Aired 2002/May/2707,01 with an audience share of 16.7%04,01 || 07,01 Based on Vol 34 File 5-7	\N	f	4
-545	-545	10,01Episode 15,01[1154515,01] The Witch Enshrouded in Fog (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XViD] CRC: [3B02B0B4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=433FLRLH04,01 || 07Aired 2009/Sept/507,01 with an audience share of 8.4%04,01 || 07,01 Based on Vol 63 File 9-11	\N	f	6
-446	-446	10,01Episode 15,01[1144615,01] The Sealed Western-style Window (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01176mb CRC: [1821E31B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QMAX0Z8N04,01 || 07Aired 2006/Jul/2407,01 with an audience share of 8.4%04,01 || 07,01 Based on Vol 51 File 9-11	\N	f	1
-542	-542	10,01Episode 15,01[1154215,01] Ikkaku Rock's Disappearing Fish (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 349mb Codec: [XviD] CRC: [36AE9399 | 60A2570C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QV4XJC02 | 720: http://www.megaupload.com/?d=KB7SKHV204,01 || 07Aired 2009/July/2507,01 with an audience share of 6.5%04,01 || 07,01 Based on Vol 64 File 1-3	\N	f	4
-33	-33	10,01Episode 15,01[113315,01] Detective Boys' Survival Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01184mb CRC: [A1859733]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=750QCK2004,01 || 07Aired 1996/Oct/1407,01 with an audience share of 14.7%04,01 || 15,0107,01 AO	\N	f	0
-26	-26	10,01Episode 15,01[112615,01] Pet Dog Jon Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01175mb CRC: [4B2D3093]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K37JGJSX04,01 || 07Aired 1996/Jul/2907,01 with an audience share of 13.6%04,01 || 15,0107,01 AO	\N	f	0
-99	-99	10,01Episode 15,01[119915,01] The Famous Potter Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [F6904CAF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MCD7H1P104,01 || 07Aired 1998/Apr/2707,01 with an audience share of 18.6%04,01 || 07,01 Based on Vol 16 File 10 - Vol 17 File 2	\N	f	0
-526	-526	10,01Episode 15,01[1152615,01] A Present from the True Culprit04,01 || 15,0107@ 15,01gb CRC: [97258452]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RHN2DWI004,01 || 07Aired 2009/Feb/2307,01 with an audience share of 8.9%04,01 || 07,01 Based on Vol 61 File 11 - Vol 62 File 1	\N	f	1
-72	-72	10,01Episode 15,01[117215,01] The Triplet's Country Home Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [40AF4685]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1O21EXND04,01 || 07Aired 1997/Aug/1807,01 with an audience share of 18.3%04,01 || 07,01 Based on Vol 13 File 2-4	\N	f	2
-500	-500	10,01Episode 15,01[1150015,01] Clash of Red and Black: Testament04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 233mb CRC: [2E12755B | 2EAF56C8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UTDDIYKT | 720: http://www.megaupload.com/?d=GMDQXZZX04,01 || 07Aired 2008/Mar/1707,01 with an audience share of 9.5%04,01 || 07,01 Based on Vol 58 File 3 - Vol 58 File 7	\N	f	4
-566	-566	10,01Episode 15,01[1156615,01] The Partner is Santa-san04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD] CRC: [C0D8FE4B | 08642737]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CPIKMKKH | 720: http://www.megaupload.com/?d=RVR4L1DC04,01 || 07Aired 2010/Feb/2707,01	\N	f	2
-7	-7	10,01Episode 15,01[11715,01] Once-A-Month Present Threat Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01218mb CRC: [863DA758]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1S90X5QK04,01 || 07Aired 1996/Feb/1907,01 with an audience share of 12.7%04,01 || 07,01 Based on Vol 3 File 7-10	\N	f	3
-264	-264	10,01Episode 15,01[1126415,01] The Courtroom Battle: Kisaki VS Kogoro (Part 1)04,01 || 15,0107@ 15,01gb CRC: [7BC84F90]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0XU8I92W04,01 || 07Aired 2002/Jan/1407,01 with an audience share of 17.8%04,01 || 15,0107,01 AO	\N	f	6
-170	-170	10,01Episode 15,01[1117015,01] The Unseen Angle of the Darkness (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [91509B90]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ACBEDL2X04,01 || 07Aired 1999/Nov/2907,01 with an audience share of 17.5%04,01 || 07,01 Based on Vol 24 File 3-6	\N	f	1
-574	-574	10,01Episode 15,01[1157415,01] The Whereabouts of the Embarrasing Charm (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 320mb Codec: [XviD | H264] CRC: [E8E0AB2D | 34B845E5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FKR29HAI | 720: http://www.megaupload.com/?d=QWT4VKO504,01 || 07Aired 2010/May/2207,0104,01 || 07,01 Based on Vol 66 File 8-10	\N	f	4
-108	-108	10,01Episode 15,01[1110815,01] Mogura Star Man Murder Case (Part 2)04,01 || 15,01Subbed by [10Atavus15,01] 07@ 15,01172mb Codec: [XviD] CRC: [8977AF04]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YNF7KOR504,01 || 07Aired 1998/Jul/0607,01 with an audience share of 19.5%04,01 || 15,0107,01 AO	\N	f	1
-359	-359	10,01Episode 15,01[1135915,01] The Metropolitan Police Detective Love Story 5 (Part 2)04,01 || 15,01Subbed by [10Kaizou-Neg15,01] 07@ 15,01172mb CRC: [6DC042DC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6ATJ3R5F04,01 || 07Aired 2004/May/1007,01 with an audience share of 12.8%04,01 || 07,01 Based on Vol 40 File 1-3	\N	f	2
-604	-604	10,01Episode 15,01[1160415,01] The Seance's Double Locked Room Mystery Case (Second Locked Room)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [D4759F0B/5B8E0EC0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=999B4QMN | 720: http://www.megaupload.com/?d=85N683CJ04,01 || 07Aired 2/5/201107,01	\N	f	3
-227	-227	10,01Episode 15,01[1122715,01] The Battle Game Trap (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [51177822]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XMU045AD04,01 || 07Aired 2001/Mar/0507,01 with an audience share of 20.4%04,01 || 07,01 Based on Vol 27 File 7-9	\N	f	1
-182	-182	10,01Episode 15,01[1118215,01] The Big Investigation of the Nine Doors04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [C51366F9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FT8W7O5104,01 || 07Aired 2000/Feb/2807,01 with an audience share of 21%04,01 || 15,0107,01 AO	\N	f	1
-533	-533	10,01Episode 15,01[1153315,01] The Scar That Invokes the Past04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [C4ED94A4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L9HRE7AH04,01 || 07Aired 2009/May/0907,01 with an audience share of 6.5%04,01 || 07,01 Based on Vol 64 File 4-6	\N	f	1
-556	-556	10,01Episode 15,01[1155615,01] Intersection of Fear04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb Codec: [XviD] CRC: [07496DE8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U4PVA9B004,01 || 07Aired 2009/Nov/2107,0104,01 || 15,0107,01 AO	\N	f	2
-428	-428	10,01Episode 15,01[1142815,01] Super Secret of the Road to School (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [D797179F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U3899RNR04,01 || 07Aired 2006/Jan/3007,01 with an audience share of 10.9%04,01 || 07,01 Based on Vol 49 File 5-715,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 26	\N	f	4
-167	-167	10,01Episode 15,01[1116715,01] The Monster of the Tottori Spider Mansion (Part 2: The Suspicion)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [2C304964]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MBN4VLF004,01 || 07Aired 1999/Nov/0807,01 with an audience share of 17.7%04,01 || 07,01 Based on Vol 25 File 4-8	\N	f	0
-50	-50	10,01Episode 15,01[115015,01] Library Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01168mb CRC: [44495EDE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=91YHWFJ804,01 || 07Aired 1997/Mar/0307,01 with an audience share of 16.8%04,01 || 07,01 Based on Vol 10 File 7-8	\N	f	1
-142	-142	10,01Episode 15,01[1114215,01] The Night Before the Wedding Locked Room Murder Case (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [F5EA9297]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U3PZP30I04,01 || 07Aired 1999/Apr/2607,01 with an audience share of 18.7%04,01 || 07,01 Based on Vol 21 File 11 - Vol 22 File 1-3	\N	f	1
-48	-48	10,01Episode 15,01[114815,01] The Diplomat Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01167mb CRC: [C4DEA49E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LZS1W55J04,01 || 07Aired 1997/Feb/1707,01 with an audience share of 19%04,01 || 07,01 Based on Vol 10 File 2-6	\N	f	1
-462	-462	10,01Episode 15,01[1146215,01] Shadow of Black Organization: A Very Young Witness04,01 || 15,01Subbed by [10Kaizou-Neg15,01] 07@ 15,01178mb | 233mb CRC: [912A3CBD | 27B0E5C1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=N1KY8F8P | 720: http://www.megaupload.com/?d=QGLVSP6304,01 || 07Aired 2007/Jan/2907,01 with an audience share of 9.7%04,01 || 07,01 Based on Vol 53 File 7 - Vol 54 File 2	\N	f	3
-393	-393	10,01Episode 15,01[1139315,01] A Kidnapping... So It Would Seem04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [5C2C0BB0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=24VDFUOY04,01 || 07Aired 2005/Mar/2107,01 with an audience share of 10.5%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 20	\N	f	5
-514	-514	10,01Episode 15,01[1151415,01] Coffee Aroma's Murderous Intent (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [45B629CF | C7F40E2D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ID5JF11K04,01 || 07Aired 2008/Sep/0807,01 with an audience share of 7.9%04,01 || 07,01 Based on Vol 60 File 9-11	\N	f	2
-513	-513	10,01Episode 15,01[1151315,01] Coffee Aroma's Murderous Intent (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [3E0355DB | 59B5C9E2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=68G4294304,01 || 07Aired 2008/Sep/0107,01 with an audience share of 9.3%04,01 || 07,01 Based on Vol 60 File 9-11	\N	f	1
-477	-477	10,01Episode 15,01[1147715,01] Genta's Killer Shoot (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [5D3584FF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KD2NL5KE04,01 || 07Aired 2007/Jul/0207,01 with an audience share of 8.7%04,01 || 07,01 Based on Vol 55 File 3-5	\N	f	1
-232	-232	10,01Episode 15,01[1123215,01] The Falling Off the Condo Case04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01174mb Codec: [XviD] CRC: [EEB51E7F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=O6X15C9O04,01 || 07Aired 2001/May/0707,01 with an audience share of 15.8%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 8	\N	f	2
-207	-207	10,01Episode 15,01[1120715,01] The Deduction That Was Too Good04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [AC7A59A3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=TN5B3HUC04,01 || 07Aired 2000/Sep/1107,01 with an audience share of 21.4%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 2	\N	f	4
-360	-360	10,01Episode 15,01[1136015,01] A Mysterious Spring Beetle04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [5039E9FC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NCE57PRK04,01 || 07Aired 2004/May/1707,01 with an audience share of 13.7%04,01 || 15,0107,01 AO	\N	f	2
-330	-330	10,01Episode 15,01[1133015,01] A Friendship That Can't Be Bought (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [2042CB43]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NCLT26QN04,01 || 07Aired 2003/Aug/0407,01 with an audience share of 14.5%04,01 || 07,01 Based on Vol 39 File 6-8	\N	f	2
-476	-476	10,01Episode 15,01[1147615,01] Genta's Killer Shoot (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [92F68EFA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=58HD0OLH04,01 || 07Aired 2007/Jun/2507,01 with an audience share of 9.3%04,01 || 07,01 Based on Vol 55 File 3-5	\N	f	2
-510	-510	10,01Episode 15,01[1151015,01] Conan VS W-Code Mystery04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [0F65CA68 | 358CC177]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=S60I5L7I04,01 || 07Aired 2008/Jul/2807,01 with an audience share of 8.5%04,01 || 07,01 Based on Vol 61 File 8-10	\N	f	4
-394	-394	10,01Episode 15,01[1139415,01] Big Adventure at the Unconventional Mansion (Sealing Part)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [C0256C47]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PWKCZ7S904,01 || 07Aired 2005/Apr/1807,01 with an audience share of 11.4%04,01 || 07,01 Based on Vol 46 File 7-1004,01 || 07,01http://www.nyaa.eu/?page=download&tid=19581315,0104,01 || \n07,01Bot Listing: 15,01/MSG [xdcc]Ecchilicious XDCC SEND 375	\N	f	5
-225	-225	10,01Episode 15,01[1122515,01] The Secret of the High Sales04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [8D3D6EEA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AGIPFG6V04,01 || 07Aired 2001/Feb/1907,01 with an audience share of 21.6%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 5	\N	f	1
-414	-414	10,01Episode 15,01[1141415,01] The Detective Boys' Bluebird Chase04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [0E4DE4C1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QVJFUKOI04,01 || 07Aired 2005/Sep/1207,01 with an audience share of 12.6%04,01 || 15,0107,01 AO	\N	f	2
-353	-353	10,01Episode 15,01[1135315,01] The Fishing Tournament Tragedy (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01347mb Codec: [XviD] CRC: [34715510]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7K75JP9Y04,01 || 07Aired 2004/Mar/0107,01 with an audience share of 13.8%04,01 || 15,0107,01 AO04,01 || 07,01Part 1 is concatenated.	\N	f	0
-40	-40	10,01Episode 15,01[114015,01] The Wealthy Daughter Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01141mb CRC: [FE7BD853]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=769ISQEQ04,01 || 07Aired 1996/Dec/0207,01 with an audience share of 17.1%04,01 || 07,01 Based on Vol 9 File 7 - Vol 10 File 1	\N	f	0
-192	-192	10,01Episode 15,01[1119215,01] The Desperate Revival: Shinichi's Return04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [9E091060]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=OG2G00TT04,01 || 07Aired 2000/May/2907,01 with an audience share of 20.3%04,01 || 07,01 Based on Vol 25 File 9 - Vol 26 File 7	\N	f	0
-128	-128	10,01Episode 15,01[1112815,01] The Black Corps One Billion Yen Robbery Case04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb Codec: [XviD] CRC: [7D1B5A8B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=E1FV31L204,01 || 07Aired 1998/Dec/1407,01 with an audience share of 20.9%04,01 || 15,0107,01 AO	\N	f	3
-274	-274	10,01Episode 15,01[1127415,01] Truth About the Haunted House (Part 1)04,01 || 15,01Subbed by [10DCTP-AZFS15,01] 07@ 15,01350mb Codec: [XviD] CRC: [2E75803F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2GHYKWF504,01 || 07Aired 2002/Apr/1507,01 with an audience share of 17.8%04,01 || 07,01 Based on Vol 35 File 5-704,01 || 07,01Two parter, contains 275 also15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 25	\N	f	1
-303	-303	10,01Episode 15,01[1130315,01] The Victim who Came Back04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [5C0AA1F1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HEL3EPLS04,01 || 07Aired 2002/Dec/0907,01 with an audience share of 16.4%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 13	\N	f	1
-310	-310	10,01Episode 15,01[1131015,01] Contact with the Black Organization (Part 2: The Pursuit)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01215mb CRC: [E28D0AC6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2LYY4H8N04,01 || 07Aired 2003/Feb/1707,01 with an audience share of 15.8%04,01 || 07,01 Based on Vol 37 File 8 - Vol 38 File 1	\N	f	3
-449	-449	10,01Episode 15,01[1144915,01] The Metropolitan Police Detective Love Story, The Fake Wedding (1 Hour Special)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01382mb CRC: [DE318ABE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7WD8XVZ504,01 || 07Aired 2006/Aug/2107,01 with an audience share of 11.8%04,01 || 07,01 Based on Vol 52 File 3-5	\N	f	1
-575	-575	10,01Episode 15,01[1157515,01] The Alibi of the Black Dress (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 320mb Codec: [XviD | H264] CRC: [5E3C28F1 | 95D00067]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LK6B7XP2 | 720: http://www.megaupload.com/?d=ZU71OL2N04,01 || 07Aired 2010/May/2907,0104,01 || 07,01 Based on Vol 66 File 11 - Vol 67 File 2	\N	f	5
-375	-375	10,01Episode 15,01[1137515,01] A Code for Star and Tabacco (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01173mb CRC: [3DA3D7BF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L5L4EO0T04,01 || 07Aired 2004/Oct/2507,01 with an audience share of 14.4%04,01 || 07,01 Based on Vol 45 File 9 - Vol 46 File 1	\N	f	3
-215	-215	10,01Episode 15,01[1121515,01] The Bay of Revenge (Part 1)04,01 || 15,0107@ 15,01gb Codec: [XviD] CRC: [49F6C29F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0EEJ6DUR04,01 || 07Aired 2000/Nov/2707,01 with an audience share of 17.9%04,01 || 15,0107,01 AO	\N	f	1
-589	-589	10,01Episode 15,01[1158915,01] The Worst Birthday (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [866BC59A | 11BAF210]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=S4GTCZ1S | 720: http://www.megaupload.com/?d=88GCCNWB04,01 || 07Aired 2010/Sept/2507,0104,01 || 07,01 Based on Vol 68 File 2-4	\N	f	4
-469	-469	10,01Episode 15,01[1146915,01] Kaitou Kid and the 4 Masterpieces (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [99866371]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=M6HCZ1QT04,01 || 07Aired 2007/Apr/1607,01 with an audience share of 11.1%04,01 || 07,01 Based on Vol 53 File 1-4	\N	f	2
-28	-28	10,01Episode 15,01[112815,01] Kogoro's Class Reunion Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01175mb CRC: [A80F1708]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QLJWWO4L04,01 || 07Aired 1996/Aug/1207,01 with an audience share of 12.6%04,01 || 07,01 Based on Vol 9 File 4-6	\N	f	1
-501	-501	10,01Episode 15,01[1150115,01] Clash of Red and Black: Suspicion04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [AB9C91B7 | B4208A26]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=67G9L6MU | 720: http://www.megaupload.com/?d=MALMYO4B04,01 || 07Aired 2008/Apr/1407,01 with an audience share of 8.1%04,01 || 07,01 Based on Vol 58 File 8 - Vol 59 File 1	\N	f	1
-549	-549	10,01Episode 15,01[1154915,01] The Revolving Sushi Mystery (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 300mb Codec: [XViD] CRC: [550C3267 | 9DF41DBA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=J1QD82ER | 720: http://www.megaupload.com/?d=V4PPEY5T04,01 || 07Aired 2009/Oct/307,0104,01 || 07,01 Based on Vol 63 File 3-5	\N	f	2
-322	-322	10,01Episode 15,01[1132215,01] The Vanished Kidnapper's Getaway Vehicle (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172 MB Codec: [H264] CRC: [327361F5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YY7CIEF204,01 || 07Aired 2003/Jun/0207,01 with an audience share of 14.1%04,01 || 15,0107,01 AO	\N	f	5
-293	-293	10,01Episode 15,01[1129315,01] Solitary Island of the Princess and the Dragon King's Palace (Part 3)04,01 || 15,0107@ 15,01gb CRC: [3B4D9CF1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=G7D8DI0F04,01 || 07Aired 2002/Sep/0207,01 with an audience share of 18.4%04,01 || 07,01 Based on Vol 35 File 11 - Vol 36 File 4	\N	f	2
-85	-85	10,01Episode 15,01[118515,01] Ski Lodge Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [E15CFC51]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HOP7GQ7W04,01 || 07Aired 1997/Dec/1507,01 with an audience share of 21.2%04,01 || 07,01 Based on Vol 14 File 9 - Vol 15 File 3	\N	f	0
-591	-591	10,01Episode 15,01[1159115,01] The House with the Aquarium04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [B8FA4A05 | 18332712]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KOJUCWTX | 720: http://www.megaupload.com/?d=TXWIT1YM04,01 || 07Aired 2010/Oct/1607,0104,01 || 15,0107,01 AO	\N	f	2
-387	-387	10,01Episode 15,01[1138715,01] The Dissonance of the Stradivarius (Part 3: Postlude)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [ABAAD442]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z63IIPD304,01 || 07Aired 2005/Feb/0607,01 with an audience share of 12.9%04,01 || 07,01 Based on Vol 46 File 2-6	\N	f	2
-53	-53	10,01Episode 15,01[115315,01] The Mystery Weapon Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01173mb CRC: [8678BAEF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=D078YNUE04,01 || 07Aired 1997/Apr/0707,01 with an audience share of 17.3%04,01 || 15,0107,01 AO	\N	f	1
-543	-543	10,01Episode 15,01[1154315,01] Ikkaku Rock's Disappearing Fish (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 341mb Codec: [XviD] CRC: [5E49558B | 569A7A36]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7JVO4TJF | 720: http://www.megaupload.com/?d=0J2B7PGM04,01 || 07Aired 2009/August/107,01 with an audience share of 5.9%04,01 || 07,01 Based on Vol 64 File 1-3	\N	f	1
-425	-425	10,01Episode 15,01[1142515,01] Black Impact! The Instant That the Black Organization Reaches (2.5 Hour Special)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01700mb CRC: [B455E63E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RVMFTTMC04,01 || 07Aired 2006/Jan/0907,01 with an audience share of 13.1%04,01 || 07,01 Based on Vol 48 File 9 - Vol 49 File 4	\N	f	3
-9	-9	10,01Episode 15,01[11915,01] Tenkaichi Night Festival Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01209mb CRC: [20E49285]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AQ9PG4SM04,01 || 07Aired 1996/Mar/0407,01 with an audience share of 13%04,01 || 07,01 Based on Vol 6 File 9 - Vol 7 File 1	\N	f	3
-267	-267	10,01Episode 15,01[1126715,01] The Truth Behind Valentine's (Part 2: The Deduction)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [467EE1AE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=G7AODAE004,01 || 07Aired 2002/Feb/0407,01 with an audience share of 20%04,01 || 07,01 Based on Vol 33 File 3-615,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 5	\N	f	3
-461	-461	10,01Episode 15,01[1146115,01] The Missing Page04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [0C6ABCFB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PG5BO49F04,01 || 07Aired 2007/Jan/2207,01 with an audience share of 9.8%04,01 || 15,0107,01 AO	\N	f	1
-332	-332	10,01Episode 15,01[1133215,01] Suspicious Curry (Part 2)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01139mb Codec: [XviD] CRC: [82027A13]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XWST5AIF04,01 || 07Aired 2003/Aug/1807,01 with an audience share of 15%04,01 || 07,01 Based on Vol 40 File 4-615,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 29	\N	f	1
-354	-354	10,01Episode 15,01[1135415,01] A Small Client (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [66FF93DB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZOCGC4G004,01 || 07Aired 2004/Mar/0807,01 with an audience share of 14.1%04,01 || 07,01 Based on Vol 39 File 9-11	\N	f	2
-278	-278	10,01Episode 15,01[1127815,01] English Teacher VS Great Western Detective (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [E7C23E5A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z5W1L2BQ04,01 || 07Aired 2002/May/2007,01 with an audience share of 16.9%04,01 || 07,01 Based on Vol 34 File 2-4	\N	f	2
-257	-257	10,01Episode 15,01[1125715,01] The Mysterious Punishment from Heaven04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01170mb CRC: [E746202E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WU2H9IDS04,01 || 07Aired 2001/Nov/1207,01 with an audience share of 19.6%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 20	\N	f	2
-130	-130	10,01Episode 15,01[1113015,01] The Stadium Indiscriminate Threatening Case (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01346mb CRC: [E66D5124]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4I3MB34I04,01 || 07Aired 1999/Jan/1107,01 with an audience share of 21.3%04,01 || 07,01 Based on Vol 19 File 9 - Vol 20 File 1	\N	f	3
-490	-490	10,01Episode 15,01[1149015,01] Hattori Heiji VS Kudo Shinichi, Deduction Showdown at Ski Slope (1 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01347mb | 466mb CRC: [6F27DBA4 | FC50E0A2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=R1AM2OAW | 720: http://www.megaupload.com/?d=73P8D50W04,01 || 07Aired 2007/Dec/0307,01 with an audience share of 8.8%04,01 || 07,01 Based on Vol 50 File 8 - Vol 51 File 1	\N	f	4
-583	-583	10,01Episode 15,01[1158315,01] Kobayashi-sensei's Love04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [C4D3DE4A | 711C3A8F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SC2LZ2T3 | 720: http://www.megaupload.com/?d=OMNVFO0204,01 || 07Aired 2010/Aug/1407,0104,01 || 07,01 Based on Vol 67 File 9 - Vol 68 File 1	\N	f	3
-155	-155	10,01Episode 15,01[1115515,01] The Underwater Key in the Locked Room Case04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [A75E88A1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3QHMOHKI04,01 || 07Aired 1999/Aug/0207,01 with an audience share of 16.4%04,01 || 15,0107,01 AO	\N	f	1
-150	-150	10,01Episode 15,01[1115015,01] The Truth of the Exploding Car Case (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [119A37EC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9WCCM6A304,01 || 07Aired 1999/Jun/2807,01 with an audience share of 17.2%04,01 || 15,0107,01 AO	\N	f	3
-90	-90	10,01Episode 15,01[119015,01] The Flower's Smell Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [05587938]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6Q7IGB0W04,01 || 07Aired 1998/Feb/0907,01 with an audience share of 22.8%04,01 || 15,0107,01 AO	\N	f	0
-89	-89	10,01Episode 15,01[118915,01] Dracula Villa Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [C27DBA8F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=31PACSK004,01 || 07Aired 1998/Feb/0207,01 with an audience share of 20%04,01 || 15,0107,01 AO	\N	f	0
-233	-233	10,01Episode 15,01[1123315,01] The Evidence That Didn't Disappear (Part 1)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01174mb Codec: [XviD] CRC: [5E808474]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=C2KX9FT904,01 || 07Aired 2001/May/1407,01 with an audience share of 18.5%04,01 || 07,01 Based on Vol 29 File 6-815,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 9	\N	f	1
-603	-603	10,01Episode 15,01[1160315,01] The Seance's Double Locked Room Mystery Case (First Locked Room)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [3B82DC94/281B97D6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZY4CTHFM | 720: http://www.megaupload.com/?d=UJL4X0EE04,01 || 07Aired 1/29/1107,01	\N	f	3
-316	-316	10,01Episode 15,01[1131615,01] The Sullied Masked Hero (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01348mb Codec: [XviD] CRC: [2411959B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZW5IJM2304,01 || 07Aired 2003/Apr/2107,01 with an audience share of 15.4%04,01 || 07,01 Based on Vol 38 File 5-704,01 || 07,01Part 2 is concatenated.	\N	f	1
-55	-55	10,01Episode 15,01[115515,01] The Train Trick Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [8BE6D54F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XZJZJP4904,01 || 07Aired 1997/Apr/2107,01 with an audience share of 18.4%04,01 || 15,0107,01 AO	\N	f	1
-102	-102	10,01Episode 15,01[1110215,01] Historical Play Performer Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [9E98E939]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ASYJT0ZG04,01 || 07Aired 1998/May/2507,01 with an audience share of 17.7%04,01 || 07,01 Based on Vol 17 File 10 - Vol 18 File 2	\N	f	1
-276	-276	10,01Episode 15,01[1127615,01] Case of the Missing Policeman's Notebook04,01 || 15,0107@ 15,01gb CRC: [AF9F5452]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=E23MI6ML04,01 || 07Aired 2002/May/0607,01 with an audience share of 19.4%04,01 || 15,0107,01 AO	\N	f	2
-259	-259	10,01Episode 15,01[1125915,01] The Man from Chicago (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [C1C1D598]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MWWS4N9L04,01 || 07Aired 2001/Nov/2607,01 with an audience share of 18.2%04,01 || 07,01 Based on Vol 32 File 8-10	\N	f	2
-16	-16	10,01Episode 15,01[111615,01] The Antique Collector Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01186mb CRC: [29EC415E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DEER010004,01 || 07Aired 1996/May/2007,01 with an audience share of 12.3%04,01 || 07,01 Based on Vol 6 File 2-5	\N	f	2
-163	-163	10,01Episode 15,01[1116315,01] The Secret of the Moon, the Star, and the Sun (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [E5CF4EC7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=I516EREH04,01 || 07Aired 1999/Oct/1107,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 12 File 1-3	\N	f	2
-520	-520	10,01Episode 15,01[1152015,01] Indictment by Red Wine04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [AFBFD601]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=R3PB2FWA04,01 || 07Aired 2008/Dec/1507,01 with an audience share of 9.6%04,01 || 15,0107,01 AO	\N	f	1
-368	-368	10,01Episode 15,01[1136815,01] The Candy House the Witch Lives In04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [8AAEED40]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=E4E0TFYF04,01 || 07Aired 2004/Jul/2607,01 with an audience share of 12.7%04,01 || 15,0107,01 AO	\N	f	2
-34	-34	10,01Episode 15,01[113415,01] The Mountain Villa Bandaged Man Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01170mb CRC: [92B7A16B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FUXU34R604,01 || 07Aired 1996/Oct/2107,01 with an audience share of 17.8%04,01 || 07,01 Based on Vol 5 File 1-5	\N	f	3
-57	-57	10,01Episode 15,01[115715,01] Holmes Freak Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [EAAEB05D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SM1CKVFD04,01 || 07Aired 1997/May/0507,01 with an audience share of 16%04,01 || 07,01 Based on Vol 12 File 7 - Vol 13 File 1	\N	f	1
-532	-532	10,01Episode 15,01[1153215,01] The Scar of First Love04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [B79B926D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CCOM4YPE04,01 || 07Aired 2009/May/0207,01 with an audience share of 5.9%04,01 || 07,01 Based on Vol 64 File 4-6	\N	f	1
-539	-539	10,01Episode 15,01[1153915,01] A Fool's Inheritance04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD-MP3] CRC: [A25593AD]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HNJOEULJ04,01 || 07Aired 2009/July/407,01 with an audience share of 6.7%04,01 || 15,0107,01 AO	\N	f	1
-565	-565	10,01Episode 15,01[1156515,01] The Eyewitness who Did Not See04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD] CRC: [3A47A234 | 3F7C5868]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HRGH5FJ6 | 720: http://www.megaupload.com/?d=XJO8XQ9J04,01 || 07Aired 2010/Feb/0607,01	\N	f	2
-424	-424	10,01Episode 15,01[1142415,01] The Photograph Mail from a Clown04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01152mb CRC: [89BCC6D2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=8F7W36C904,01 || 07Aired 2005/Dec/1907,01 with an audience share of 9.4%04,01 || 15,0107,01 AO	\N	f	2
-175	-175	10,01Episode 15,01[1117515,01] The Man Who Was Killed Four Times04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [2661B1FC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JTKCG6FW04,01 || 07Aired 2000/Jan/1007,01 with an audience share of 18%04,01 || 15,0107,01 AO	\N	f	0
-35	-35	10,01Episode 15,01[113515,01] The Mountain Villa Bandaged Man Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01152mb CRC: [0BE30950]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=71FIWAHD04,01 || 07Aired 1996/Oct/2807,01 with an audience share of 18.7%04,01 || 07,01 Based on Vol 5 File 1-5	\N	f	0
-93	-93	10,01Episode 15,01[119315,01] The Fearful Mountain Trek Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [93754999]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=O5LCECH704,01 || 07Aired 1998/Mar/0207,01 with an audience share of 19.9%04,01 || 15,0107,01 AO	\N	f	0
-199	-199	10,01Episode 15,01[1119915,01] Kogoro Mouri, Suspect (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01172mb CRC: [FB11F027]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4UAUCXN604,01 || 07Aired 2000/Jul/1707,01 with an audience share of 18.7%04,01 || 07,01 Based on Vol 27 File 1-3	\N	f	0
-291	-291	10,01Episode 15,01[1129115,01] Solitary Island of the Princess and the Dragon King's Palace (Part 1)04,01 || 15,0107@ 15,01gb CRC: [BB65FE79]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FEG0XW1N04,01 || 07Aired 2002/Aug/1907,01 with an audience share of 17%04,01 || 07,01 Based on Vol 35 File 11 - Vol 36 File 4	\N	f	3
-11	-11	10,01Episode 15,01[111115,01] Moonlight Sonata Murder Case (1 Hour Special)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01402mb CRC: [FE40A4FC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U8BANZYF04,01 || 07Aired 1996/Apr/0807,01 with an audience share of 12.9%04,01 || 07,01 Based on Vol 7 File 2-7	\N	f	3
-492	-492	10,01Episode 15,01[1149215,01] Clash of Red and Black: Blood Relative04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [E97E8995 | 5F139F72]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RJYIPIBG | 720: http://www.megaupload.com/?d=30M6WMUP04,01 || 07Aired 2008/Jan/2107,01 with an audience share of 9%04,01 || 07,01 Based on Vol 56 File 10 - Vol 57 File 4	\N	f	1
-409	-409	10,01Episode 15,01[1140915,01] The Simultaneous Stage Advance and Kidnapping (Part 1)04,01 || 15,01Codec: [04F0F911]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=8S33WPVE04,01 || 07Aired 2005/Aug/0807,01 with an audience share of 8.7%04,01 || 15,0107,01 AO04,01 || 07,01http://www.anirena.com/viewtracker.php?action=download&id=38096	\N	f	5
-347	-347	10,01Episode 15,01[1134715,01] Find the Buttock's Mark (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [A72565FA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5V8M3BE104,01 || 07Aired 2004/Jan/1907,01 with an audience share of 14.7%04,01 || 07,01 Based on Vol 42 File 11 - Vol 43 File 2	\N	f	2
-67	-67	10,01Episode 15,01[116715,01] Stage Actress Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [F9104ADF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Q3LGGY0Q04,01 || 07Aired 1997/Jul/1407,01 with an audience share of 19.3%04,01 || 15,0107,01 AO	\N	f	1
-73	-73	10,01Episode 15,01[117315,01] The Detective Boys' Disaster Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [147B553B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GZ9H0KCI04,01 || 07Aired 1997/Aug/2507,01 with an audience share of 18.8%04,01 || 15,0107,01 AO	\N	f	1
-114	-114	10,01Episode 15,01[1111415,01] The Scuba Diving Murder Case (Part 1)04,01 || 15,01Subbed by [10Atavus-MCT15,01] 07@ 15,01172mb Codec: [XviD] CRC: [55C159DC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QCCU74BT04,01 || 07Aired 1998/Aug/2407,01 with an audience share of 20.1%04,01 || 07,01 Based on Vol 17 File 3-5	\N	f	1
-442	-442	10,01Episode 15,01[1144215,01] The Man Obstructed by the Steel Frame04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [565245AF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BF5ODP4104,01 || 07Aired 2006/Jun/1207,01 with an audience share of 10.5%04,01 || 15,0107,01 AO04,01 || 07,01Details will be filled out on sunday :P	\N	f	1
-584	-584	10,01Episode 15,01[1158415,01] Inspector Shiratori's Lost Love04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD | H264] CRC: [B9BF3C0B | 51C5C7A8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FDQL2AQ1 | 720: http://www.megaupload.com/?d=J2XQ80GJ04,01 || 07Aired 2010/Aug/2107,0104,01 || 07,01 Based on Vol 67 File 9 - Vol 68 File 1	\N	f	3
-374	-374	10,01Episode 15,01[1137415,01] A Code for Star and Tabacco (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01171mb CRC: [BA352DAD]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LN70XOW604,01 || 07Aired 2004/Oct/1807,01 with an audience share of 12.6%04,01 || 07,01 Based on Vol 45 File 9 - Vol 46 File 1	\N	f	2
-292	-292	10,01Episode 15,01[1129215,01] Solitary Island of the Princess and the Dragon King's Palace (Part 2)04,01 || 15,0107@ 15,01gb CRC: [904A9038]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1Z93N0E704,01 || 07Aired 2002/Aug/2607,01 with an audience share of 18%04,01 || 07,01 Based on Vol 35 File 11 - Vol 36 File 4	\N	f	2
-511	-511	10,01Episode 15,01[1151115,01] Deduction Showdown! Shinichi VS Okiya Subaru04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb Codec: [XviD] CRC: [686C8273 | E527E9D6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=TC3VY7ID04,01 || 07Aired 2008/Aug/0407,01 with an audience share of 7.8%04,01 || 07,01 Based on Vol 61 File 8-10	\N	f	3
-429	-429	10,01Episode 15,01[1142915,01] Two People Who Can't Return Yet (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01176mb CRC: [1BB66556]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QMHOCN5K04,01 || 07Aired 2006/Feb/0607,01 with an audience share of 11.9%04,01 || 07,01 Based on Vol 49 File 8-10	\N	f	2
-434	-434	10,01Episode 15,01[1143415,01] The Great Dog Coeur's Triumph04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [307F88CC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=OB5GTJAX04,01 || 07Aired 2006/Apr/1007,01 with an audience share of 7.7%04,01 || 15,0107,01 AO	\N	f	5
-198	-198	10,01Episode 15,01[1119815,01] A Super Car's Trap (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01172mb CRC: [A8D9DA01]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V5UVI2BT04,01 || 07Aired 2000/Jul/1007,01 with an audience share of 16.8%04,01 || 15,0107,01 AO	\N	f	0
-202	-202	10,01Episode 15,01[1120215,01] The Tenth Passenger (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01172mb CRC: [A55E8074]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UTRHB4Z204,01 || 07Aired 2000/Aug/0707,01 with an audience share of 16.7%04,01 || 15,0107,01 AO	\N	f	0
-184	-184	10,01Episode 15,01[1118415,01] A Cursed Mask Coldly Laughs (1 Hour Special)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01350mb CRC: [5FD4A836]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RAOFVDGT04,01 || 07Aired 2000/Mar/1307,01 with an audience share of 20.1%04,01 || 15,0107,01 AO	\N	f	0
-140	-140	10,01Episode 15,01[1114015,01] SOS! A Messages from Ayumi04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01177mb CRC: [BCD9D3C8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z9JZ6TO504,01 || 07Aired 1999/Apr/1207,01 with an audience share of 18.4%04,01 || 15,0107,01 AO	\N	f	0
-181	-181	10,01Episode 15,01[1118115,01] The Nocturne of the Red Murderous Intent (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [BEAADFE6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1AKWPDXL04,01 || 07Aired 2000/Feb/2107,01 with an audience share of 20.1%04,01 || 15,0107,01 AO	\N	f	0
-273	-273	10,01Episode 15,01[1127315,01] Quiz-Lady's Dart Clue Case04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [DD2E01DF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GF2J4YDO04,01 || 07Aired 2002/Apr/0807,01 with an audience share of 17.9%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 24	\N	f	3
-131	-131	10,01Episode 15,01[1113115,01] The Stadium Indiscriminate Threatening Case (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01346mb CRC: [E66D5124]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4I3MB34I04,01 || 07Aired 1999/Jan/1807,01 with an audience share of 23.2%04,01 || 07,01 Based on Vol 19 File 9 - Vol 20 File 1	\N	f	2
-320	-320	10,01Episode 15,01[1132015,01] The Art of Ninja Alibi Craft04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb Codec: [XviD] CRC: [6D2ABC34]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WIHR2CEN04,01 || 07Aired 2003/May/1907,01 with an audience share of 13.4%04,01 || 15,0107,01 AO	\N	f	1
-471	-471	10,01Episode 15,01[1147115,01] Rented Car Out of Control!04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [260F6773]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ECH8CEZ004,01 || 07Aired 2007/May/0707,01 with an audience share of 9.9%04,01 || 15,0107,01 AO	\N	f	2
-361	-361	10,01Episode 15,01[1136115,01] Teitan High School's Ghost Story (Part 1)04,01 || 15,01Subbed by [10Shi-Fa15,01] 07@ 15,01108mb CRC: [BF4F4D29]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=T02ZTT6704,01 || 07Aired 2004/May/2407,01 with an audience share of 14%04,01 || 07,01 Based on Vol 44 File 11 - Vol 45 File 2	\N	f	2
-241	-241	10,01Episode 15,01[1124115,01] The Bullet Train Transport Case (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [7EA18808]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2QLIG2X704,01 || 07Aired 2001/Jul/0907,01 with an audience share of 16.7%04,01 || 07,01 Based on Vol 30 File 1-3	\N	f	2
-412	-412	10,01Episode 15,01[1141215,01] The Shinto Shrine Torii's Surprising Code (Part 2)04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01171mb CRC: [9EC1C8C9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=87O2EQIP04,01 || 07Aired 2005/Aug/2907,01 with an audience share of 8.4%04,01 || 07,01 Based on Vol 48 File 1-3	\N	f	1
-104	-104	10,01Episode 15,01[1110415,01] The Mysterious Robbers and Mansion Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [9FA26CAC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=T5JFICSF04,01 || 07Aired 1998/Jun/0807,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 17 File 7-9	\N	f	1
-24	-24	10,01Episode 15,01[112415,01] The Mysterious Woman With Amnesia Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01168mb CRC: [03796898]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=A2CTQRAN04,01 || 07Aired 1996/Jul/1507,01 with an audience share of 14.4%04,01 || 15,0107,01 AO	\N	f	1
-280	-280	10,01Episode 15,01[1128015,01] The Hooligan's Labyrinth (Part 2)04,01 || 15,01Subbed by [10MCT15,01] 07@ 15,01gb Codec: [h264] CRC: [E8BB2836]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6D5LUI8F04,01 || 07Aired 2002/Jun/0307,01 with an audience share of 17.2%04,01 || 07,01 Based on Vol 34 File 5-7	\N	f	4
-465	-465	10,01Episode 15,01[1146515,01] Shadow of Black Organization: A Pearl Shooting Star04,01 || 15,01Subbed by [10Kaizou-Neg15,01] 07@ 15,01172mb | 233mb CRC: [43FDD0EC | EB2F7986]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NEWJ052N | 720: http://www.megaupload.com/?d=7NDWUQQQ04,01 || 07Aired 2007/Feb/1907,01 with an audience share of 9.5%04,01 || 07,01 Based on Vol 53 File 7 - Vol 54 File 204,01 || 07,01Alternate CRC for some copies 7A11DBF6 still plays fine	\N	f	4
-585	-585	10,01Episode 15,01[1158515,01] Timeless Sakura's Love04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [665D7CB3 | B2886C56]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JT2NSD09 | 720: http://www.megaupload.com/?d=8QZC201104,01 || 07Aired 2010/Aug/2807,0104,01 || 07,01 Based on Vol 67 File 9 - Vol 68 File 1	\N	f	3
-489	-489	10,01Episode 15,01[1148915,01] Courtroom Confrontation III: Prosecutor as Eyewitness (1 Hour Special)04,01 || 15,01Subbed by [10DCTP-AZFS15,01] 07@ 15,01347mb Codec: [XviD] CRC: [6C29281A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KYNUDIXS04,01 || 07Aired 2007/Nov/2607,01 with an audience share of 10.1%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 30	\N	f	4
-47	-47	10,01Episode 15,01[114715,01] Sports Club Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01168mb CRC: [313AFDAA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KVG2R8TH04,01 || 07Aired 1997/Feb/1007,01 with an audience share of 16.6%04,01 || 15,0107,01 AO	\N	f	0
-37	-37	10,01Episode 15,01[113715,01] Cactus Flower Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01160mb CRC: [AA1A75EE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4PQUUW9S04,01 || 07Aired 1996/Nov/1107,01 with an audience share of 18.5%04,01 || 15,0107,01 AO	\N	f	0
-195	-195	10,01Episode 15,01[1119515,01] The Significant Music Box (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [8515BF23]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4J3V996404,01 || 07Aired 2000/Jun/1907,01 with an audience share of 18.9%04,01 || 07,01 Based on Vol 26 File 8-10	\N	f	0
-5	-5	10,01Episode 15,01[11515,01] The Shinkansen's Bomb Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01266mb CRC: [4CD51933]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=8F9LA0OV04,01 || 07Aired 1996/Feb/0507,01 with an audience share of 13.4%04,01 || 07,01 Based on Vol 4 File 4-6	\N	f	3
-555	-555	10,01Episode 15,01[1155515,01] The Stork Mystery Tour (Haruna's Pursuit Chapter)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb Codec: [XviD] CRC: [D6A054E7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MBOLSCC304,01 || 07Aired 2009/Nov/1407,0104,01 || 15,0107,01 AO	\N	f	3
-159	-159	10,01Episode 15,01[1115915,01] The Legend of the Mysterious Pagoda (Part 1)04,01 || 15,0107@ 15,01gb CRC: [584A2EBF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PN2M7WCJ04,01 || 07Aired 1999/Sep/1307,01 with an audience share of 19.3%04,01 || 15,0107,01 AO	\N	f	5
-326	-326	10,01Episode 15,01[1132615,01] The Red Horse within the Flames (Part 2: The Investigation)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [0C548183]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1KMGD5JE04,01 || 07Aired 2003/Jun/2907,01 with an audience share of 14.8%04,01 || 07,01 Based on Vol 39 File 1-515,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 15	\N	f	1
-335	-335	10,01Episode 15,01[1133515,01] Secret of the Touto Film Developer Place (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb CRC: [D55F9653]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6VQ7AFZV04,01 || 07Aired 2003/Sep/0807,01 with an audience share of 12.9%04,01 || 07,01 Based on Vol 41 File 4-6	\N	f	1
-502	-502	10,01Episode 15,01[1150215,01] Clash of Red and Black: Innocence04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [15981E4B | 6428FF22]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2T3F110W | 720: http://www.megaupload.com/?d=H0YR7AQC04,01 || 07Aired 2008/Apr/2807,01 with an audience share of 9.6%04,01 || 07,01 Based on Vol 58 File 8 - Vol 59 File 1	\N	f	2
-355	-355	10,01Episode 15,01[1135515,01] A Small Client (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [ABD88C9C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=R9XTOOPI04,01 || 07Aired 2004/Mar/1507,01 with an audience share of 13%04,01 || 07,01 Based on Vol 39 File 9-11	\N	f	2
-307	-307	10,01Episode 15,01[1130715,01] The Dark Footprint (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01226mb CRC: [E2D65F07]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KMY5AK5604,01 || 07Aired 2003/Jan/2707,01 with an audience share of 16.5%04,01 || 07,01 Based on Vol 37 File 5-7	\N	f	5
-385	-385	10,01Episode 15,01[1138515,01] The Dissonance of the Stradivarius (Part 1: Prelude)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [AAE5D213]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GWCN70QA04,01 || 07Aired 2005/Jan/2407,01 with an audience share of 12.3%04,01 || 07,01 Based on Vol 46 File 2-6	\N	f	1
-154	-154	10,01Episode 15,01[1115415,01] Sonoko's Dangerous Summer Story (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01174mb CRC: [F85274CD]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VG38BOGG04,01 || 07Aired 1999/Jul/2607,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 22 File 8-10	\N	f	1
-479	-479	10,01Episode 15,01[1147915,01] Three Days with Hattori Heiji (2 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01700mb CRC: [E4400147]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6M9PQWGD04,01 || 07Aired 2007/Jul/1607,01 with an audience share of 10.5%04,01 || 07,01 Based on Vol 54 File 4 - Vol 55 File 2	\N	f	2
-538	-538	10,01Episode 15,01[1153815,01]  Kaitou Kid vs. The Strongest Safe (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01170mb | 483mb | 231mb Codec: [XviD] CRC: [968A2C89 | 745F8158 | 9C6A24E7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4IZ36V3X | 720: http://www.megaupload.com/?d=2CKHGW8604,01 || 07Aired 2009/Jun/2007,01 with an audience share of 6.8%04,01 || 07,01 Based on Vol 64 File 11 - Vol 65 File 2	\N	f	1
-553	-553	10,01Episode 15,01[1155315,01] The Interrogation Room04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [A37149EB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YKCBLII804,01 || 07Aired 2009/Oct/3107,0104,01 || 15,0107,01 AO	\N	f	1
-554	-554	10,01Episode 15,01[1155415,01] The Stork Mystery Tour (Ran's Search Chapter)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [3399CA00]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V2HF5DLR04,01 || 07Aired 2009/Nov/0707,0104,01 || 15,0107,01 AO	\N	f	1
-524	-524	10,01Episode 15,01[1152415,01] Blue Spark of Hate (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [80D7888F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=QHW919ZQ04,01 || 07Aired 2009/Feb/0907,01 with an audience share of 8.2%04,01 || 07,01 Based on Vol 61 File 5-8	\N	f	2
-314	-314	10,01Episode 15,01[1131415,01] Broken Fence of the Observatory04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01169mb CRC: [2944FE1B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=W770J2DW04,01 || 07Aired 2003/Mar/1707,01 with an audience share of 14.6%04,01 || 15,0107,01 AO	\N	f	3
-230	-230	10,01Episode 15,01[1123015,01] The Mysterious Passenger (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [5F262FF3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=I34X2C0R04,01 || 07Aired 2001/Apr/1607,01 with an audience share of 18.4%04,01 || 07,01 Based on Vol 29 File 3-5	\N	f	0
-115	-115	10,01Episode 15,01[1111515,01] The Scuba Diving Murder Case (Part 2)04,01 || 15,01Subbed by [10Atavus-MCT15,01] 07@ 15,01171mb Codec: [XviD] CRC: [18F93A3E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0MSRDOT804,01 || 07Aired 1998/Aug/3107,01 with an audience share of 19.8%04,01 || 07,01 Based on Vol 17 File 3-5	\N	f	0
-91	-91	10,01Episode 15,01[119115,01] Hospitalized Burgulary Suspect Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [A6F17D48]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=183YECZY04,01 || 07Aired 1998/Feb/1607,01 with an audience share of 21.5%04,01 || 07,01 Based on Vol 17 File 6	\N	f	0
-467	-467	10,01Episode 15,01[1146715,01] Unsmashable Snowman (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [0ED1C3E2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FWC23FMU04,01 || 07Aired 2007/Mar/0507,01 with an audience share of 8.3%04,01 || 07,01 Based on Vol 54 File 3-5	\N	f	6
-334	-334	10,01Episode 15,01[1133415,01] Alike Princesses (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [681D5915]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U16ZUQLK04,01 || 07Aired 2003/Sep/0107,01 with an audience share of 15%04,01 || 07,01 Based on Vol 40 File 10 - Vol 41 File 3	\N	f	2
-377	-377	10,01Episode 15,01[1137715,01] Momotarou Mystery Solving Tour (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01172mb CRC: [5B3CABFD]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=OD93TF2U04,01 || 07Aired 2004/Nov/0807,01 with an audience share of 14.9%04,01 || 15,0107,01 AO	\N	f	2
-162	-162	10,01Episode 15,01[1116215,01] The Case of the Flying Locked Room, Shinichi Kudo's First Case (1 Hour Special)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01350mb CRC: [6BC13D6D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2F6TRCT404,01 || 07Aired 1999/Oct/0407,01 with an audience share of 19.8%04,01 || 07,01 Based on Vol 21 File 4-7	\N	f	2
-74	-74	10,01Episode 15,01[117415,01] The Death God Jinnai Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [449F2616]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2QQX4EFX04,01 || 07Aired 1997/Sep/0107,01 with an audience share of 19.3%04,01 || 15,0107,01 AO	\N	f	1
-440	-440	10,01Episode 15,01[1144015,01] The Car Stunt's Utmost Limit04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [E0DC3FB7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LUVG7CPR04,01 || 07Aired 2006/May/2907,01 with an audience share of 11.6%04,01 || 15,0107,01 AO04,01 || 07,01v2 available. it fixes a name.	\N	f	2
-201	-201	10,01Episode 15,01[1120115,01] The Tenth Passenger (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01171mb CRC: [CBD8A729]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=TIWAA3M004,01 || 07Aired 2000/Jul/3107,01 with an audience share of 19.2%04,01 || 15,0107,01 AO	\N	f	2
-433	-433	10,01Episode 15,01[1143315,01] Conan: A Strange Child04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01173mb CRC: [8B30ECF0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AUOSFF7Y04,01 || 07Aired 2006/Mar/0607,01 with an audience share of 10%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 27	\N	f	2
-103	-103	10,01Episode 15,01[1110315,01] Historical Play Performer Murder Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [96F07295]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9DYBHZUI04,01 || 07Aired 1998/Jun/0107,01 with an audience share of 17.6%04,01 || 07,01 Based on Vol 17 File 10 - Vol 18 File 2	\N	f	1
-521	-521	10,01Episode 15,01[1152115,01] Murderer, Kudou Shinichi (1 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb Codec: [XviD] CRC: [0D16476F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VG15DQJB04,01 || 07Aired 2009/Jan/1907,01 with an audience share of 9.6%04,01 || 07,01 Based on Vol 62 File 5-11	\N	f	2
-240	-240	10,01Episode 15,01[1124015,01] The Bullet Train Transport Case (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb CRC: [2B3050BE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=EMNJ2BXX04,01 || 07Aired 2001/Jul/0207,01 with an audience share of 17%04,01 || 07,01 Based on Vol 30 File 1-3	\N	f	2
-598	-598	10,01Episode 15,01[1159815,01] The Scenario of the Steaming Locked Room (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [EE6E0333 | BB0BEFBE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0CMN50M8 | 720: http://www.megaupload.com/?d=YPHSH72V04,01 || 07Aired 2010/Dec/0407,0104,01 || 07,01 Based on Vol 69 File 4-6	\N	f	2
-423	-423	10,01Episode 15,01[1142315,01] The Detective Boys and the Four Cabbage Worm Brothers04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [87598584]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZOMWA9WS04,01 || 07Aired 2005/Dec/0507,01 with an audience share of 10.8%04,01 || 15,0107,01 AO04,01 || 07,01Original filename says CRC: [87598584] but that is wrong!15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 24	\N	f	3
-552	-552	10,01Episode 15,01[1155215,01] The Culprit is Genta's Dad (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01170mb Codec: [XViD] CRC: [0ABA0722]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9BQZAP5304,01 || 07Aired 2009/Oct/2407,0104,01 || 07,01 Based on Vol 63 File 6-8	\N	f	2
-602	-602	10,01Episode 15,01[1160215,01] The Devil in the Tennis Court04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [30357DE4 | 22EBB3BB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RQO5YTCJ | 720: http://www.megaupload.com/?d=MTKRYT5M15,0104,01 || \n07,01Bot Listing: 15,01http://www.fileserve.com/file/tu64vAR | 720: http://www.fileserve.com/file/5Gb3WWM	\N	f	4
-97	-97	10,01Episode 15,01[119715,01] The Farewell Wine Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [13C8C197]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=47MRKP0K04,01 || 07Aired 1998/Apr/1307,01 with an audience share of 20.1%04,01 || 15,0107,01 AO	\N	f	0
-41	-41	10,01Episode 15,01[114115,01] The Victory Flag Vandalism Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01173mb CRC: [35B2122D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LUBMWY2X04,01 || 07Aired 1996/Dec/0907,01 with an audience share of 17.6%04,01 || 15,0107,01 AO	\N	f	0
-52	-52	10,01Episode 15,01[115215,01] The Mist Goblin Legend Murder Case (1 Hour Special)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01344mb CRC: [D473694A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0KVFK0PH04,01 || 07Aired 1997/Mar/1707,01 with an audience share of 17.9%04,01 || 07,01 Based on Vol 11 File 8 - 10	\N	f	0
-229	-229	10,01Episode 15,01[1122915,01] The Murderous Pottery Class (Part 2)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01173mb Codec: [XviD] CRC: [74838EC0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LZFUEP0N04,01 || 07Aired 2001/Mar/1907,01 with an audience share of 19.8%04,01 || 07,01 Based on Vol 30 File 10 - Vol 31 File 115,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 7	\N	f	0
-188	-188	10,01Episode 15,01[1118815,01] The Desperate Revival: The Cavern of the Detective Boys04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [34FF1167]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0N2FRSUU04,01 || 07Aired 2000/May/0107,01 with an audience share of 19.8%04,01 || 07,01 Based on Vol 25 File 9 - Vol 26 File 7	\N	f	0
-68	-68	10,01Episode 15,01[116815,01] The Night Baron Murder Case (Part 1: The Case)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [4588DDEC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VFU0TD4B04,01 || 07Aired 1997/Jul/2107,01 with an audience share of 17.5%04,01 || 07,01 Based on Vol 8 File 2-7	\N	f	1
-422	-422	10,01Episode 15,01[1142215,01] Gingko-Colored First Love (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [0A6B5B43]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NTYKV5K604,01 || 07Aired 2005/Nov/2807,01 with an audience share of 9%04,01 || 07,01 Based on Vol 40 File 7-915,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 23	\N	f	3
-315	-315	10,01Episode 15,01[1131515,01] Place Exposed to the Sun04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [0073C61A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7M0P60VG04,01 || 07Aired 2003/Apr/1407,01 with an audience share of 14.9%04,01 || 15,0107,01 AO	\N	f	1
-338	-338	10,01Episode 15,01[1133815,01] Four Porsche's (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [3EBB47BB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3AVB424T04,01 || 07Aired 2003/Oct/2007,01 with an audience share of 13.9%04,01 || 07,01 Based on Vol 41 File 7-9	\N	f	2
-302	-302	10,01Episode 15,01[1130215,01] Parade of Malice and Saint (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01343mb CRC: [3722E962]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K8TYY6KD04,01 || 07Aired 2002/Dec/0207,01 with an audience share of 16.3%04,01 || 07,01 Based on Vol 36 File 5-7	\N	f	2
-474	-474	10,01Episode 15,01[1147415,01] The Love of Lawyer Kisaki Eri04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [967EDEF0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KM1FLOI704,01 || 07Aired 2007/Jun/0407,01 with an audience share of 8.6%04,01 || 07,01 Based on Vol 55 File 10-11	\N	f	2
-174	-174	10,01Episode 15,01[1117415,01] The 20 Year old Murder Case, The Symphony Serial Murders (2 Hour Special)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01700mb CRC: [5EBFABF8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FSSETVIT04,01 || 07Aired 2000/Jan/0307,01 with an audience share of 14.6%04,01 || 07,01 Based on Vol 23 File 4-9	\N	f	1
-563	-563	10,01Episode 15,01[1156315,01] Detective Boys VS Robber Group (Part 1: Turmoil)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD] CRC: [C16CB080 | 2359EB05]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K1899GWQ | 720: http://www.megaupload.com/?d=DBX69M3T04,01 || 07Aired 2010/Jan/2307,01	\N	f	4
-481	-481	10,01Episode 15,01[1148115,01] The Mountain Witch's Cutlery (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01348mb Codec: [XviD] CRC: [AA275948]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KXOH1SER04,01 || 07Aired 2007/Jul/3007,01 with an audience share of 8.8%04,01 || 07,01 Based on Vol 56 File 4-604,01 || 07,01Part 2 is concatenated.	\N	f	3
-503	-503	10,01Episode 15,01[1150315,01] Clash of Red and Black: Ready to Die04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [073FCD49 | 4BE776AF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HHSBRKIR | 720: http://www.megaupload.com/?d=8SP77YCR04,01 || 07Aired 2008/May/1207,01 with an audience share of 9.1%04,01 || 07,01 Based on Vol 58 File 8 - Vol 59 File 1	\N	f	1
-402	-402	10,01Episode 15,01[1140215,01] The Caught Red-handed Jewel Robber (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01167mb CRC: [9169FA30]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K7UD67R904,01 || 07Aired 2005/Jun/1307,01 with an audience share of 11.4%04,01 || 07,01 Based on Vol 47 File 5-7	\N	f	3
-564	-564	10,01Episode 15,01[1156415,01] Detective Boys VS Robber Group (Part 2: Silence)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD] CRC: [36493561 | 2003310D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=1TLAT5A6 | 720: http://www.megaupload.com/?d=PC1TDCA904,01 || 07Aired 2010/Jan/3007,01	\N	f	3
-214	-214	10,01Episode 15,01[1121415,01] The Mysterious Retro Room Case04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01179mb Codec: [XviD] CRC: [9AFE4475]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=T3PYTNE804,01 || 07Aired 2000/Nov/2007,01 with an audience share of 19.4%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 4	\N	f	3
-25	-25	10,01Episode 15,01[112515,01] The False Kidnapping and Hostage Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01175mb CRC: [2A9B7781]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=M2CP6PN704,01 || 07Aired 1996/Jul/2207,01 with an audience share of 16.2%04,01 || 15,0107,01 AO	\N	f	0
-83	-83	10,01Episode 15,01[118315,01] General Hospital Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [3D9F6B92]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=45Y5SZ9G04,01 || 07Aired 1997/Dec/0107,01 with an audience share of 23.1%04,01 || 15,0107,01 AO	\N	f	0
-217	-217	10,01Episode 15,01[1121715,01] Megure's Sealed Secret (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb Codec: [XviD] CRC: [51872EDC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HDWEXHW104,01 || 07Aired 2000/Dec/1107,01 with an audience share of 19.2%04,01 || 07,01 Based on Vol 28 File 11 - Vol 29 File 2	\N	f	0
-158	-158	10,01Episode 15,01[1115815,01] The Silent Kanjo Line04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [3576093A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GT32JV2004,01 || 07Aired 1999/Sep/0607,01 with an audience share of 17.8%04,01 || 15,0107,01 AO	\N	f	0
-46	-46	10,01Episode 15,01[114615,01] The Snowy Mountain Villa Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01168mb CRC: [163C4C11]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3R3K2L1X04,01 || 07Aired 1997/Feb/0307,01 with an audience share of 18%04,01 || 07,01 Based on Vol 10 File 9 - Vol 11 File 1	\N	f	0
-328	-328	10,01Episode 15,01[1132815,01] Mystery of the Birthday Wine04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [690B685D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=A6RHA27K04,01 || 07Aired 2003/Jul/1407,01 with an audience share of 14.2%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 27	\N	f	1
-281	-281	10,01Episode 15,01[1128115,01] The Small Eye Witnesses04,01 || 15,01Subbed by [10DCTP-AZFS15,01] 07@ 15,01172MB04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HHOIRKH504,01 || 07Aired 2002/Jun/1007,01 with an audience share of 13.2%04,01 || 15,0107,01 AO	\N	f	1
-143	-143	10,01Episode 15,01[1114315,01] The Suspicious Astronomical Observation04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [07A2A410]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=A1GCZN7Y04,01 || 07Aired 1999/May/0307,01 with an audience share of 15.6%04,01 || 15,0107,01 AO	\N	f	3
-222	-222	10,01Episode 15,01[1122215,01] And Then There Were No Mermaids (Part 1: The Murder)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb CRC: [1388584C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VVHLEWU504,01 || 07Aired 2001/Jan/2907,01 with an audience share of 19.2%04,01 || 07,01 Based on Vol 28 File 6-10	\N	f	1
-576	-576	10,01Episode 15,01[1157615,01] The Alibi of the Black Dress (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD | H264] CRC: [CFFEBCAE | 16A5EA2A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V1UYPM27 | 720: http://www.megaupload.com/?d=BDOX810C04,01 || 07Aired 2010/Jun/507,0104,01 || 07,01 Based on Vol 66 File 11 - Vol 67 File 2	\N	f	5
-484	-484	10,01Episode 15,01[1148415,01] The Location of the Black Photograph (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [65F4D5BF | 7D8F5A3F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=07ZRZBTZ | 720: http://www.megaupload.com/?d=4IKFNWEW04,01 || 07Aired 2007/Aug/2007,01 with an audience share of 8.2%04,01 || 07,01 Based on Vol 56 File 7-9	\N	f	2
-6	-6	10,01Episode 15,01[11615,01] Valentine Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01210mb CRC: [5A16D9BA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JGROZCZ204,01 || 07Aired 1996/Feb/1207,01 with an audience share of 12%04,01 || 15,0107,01 AO	\N	f	2
-122	-122	10,01Episode 15,01[1112215,01] The Sealed Bathroom Murder Case (Part 2)04,01 || 15,01Subbed by [10Puto-F15,01] 07@ 15,01250MB Codec: [H264] CRC: [bef658fe]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BZV6U43204,01 || 07Aired 1998/Nov/0207,01 with an audience share of 20%04,01 || 07,01 Based on Vol 20 File 7-9	\N	f	2
-363	-363	10,01Episode 15,01[1136315,01] The City's Crows04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [40161E8E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2HCRHPJH04,01 || 07Aired 2004/Jun/0707,01 with an audience share of 11.5%04,01 || 15,0107,01 AO	\N	f	2
-464	-464	10,01Episode 15,01[1146415,01] Shadow of Black Organization: The Mysterious Big Reward04,01 || 15,01Subbed by [10Kaizou-Neg15,01] 07@ 15,01172mb | 172mb CRC: [17DCD753 | 8BB2B378]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K0S8CEA3 | 720: http://www.megaupload.com/?d=JVQFM43H04,01 || 07Aired 2007/Feb/1207,01 with an audience share of 9.8%04,01 || 07,01 Based on Vol 53 File 7 - Vol 54 File 2	\N	f	5
-239	-239	10,01Episode 15,01[1123915,01] The Osaka 3 "K" Case (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb CRC: [16CCB3DC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=O7XDYHOR04,01 || 07Aired 2001/Jun/2507,01 with an audience share of 16.3%04,01 || 07,01 Based on Vol 29 File 9-11	\N	f	2
-305	-305	10,01Episode 15,01[1130515,01] The Unseen Suspect (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01348mb Codec: [XviD] CRC: [221A7174]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=22ADVSIO04,01 || 07Aired 2003/Jan/1307,01 with an audience share of 15.4%04,01 || 07,01 Based on Vol 37 File 2-404,01 || 07,01Part 2 is concatenated.	\N	f	3
-562	-562	10,01Episode 15,01[1156215,01] Rainbow Color Kidnapping04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD] CRC: [8E336675 | C74F5412]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZUJJF346 | 720: http://www.megaupload.com/?d=RVUEYBQ204,01 || 07Aired 2010/Jan/1607,01	\N	f	4
-544	-544	10,01Episode 15,01[1154415,01] The Hand That Plays in Dissonance04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [643472A1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SX0CHM1804,01 || 07Aired 2009/August/807,01 with an audience share of 6.5%04,01 || 15,0107,01 AO	\N	f	2
-36	-36	10,01Episode 15,01[113615,01] Monday Night 7:30 PM Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01165mb CRC: [A22525D9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3ANT5DBF04,01 || 07Aired 1996/Nov/0407,01 with an audience share of 17%04,01 || 15,0107,01 AO	\N	f	0
-183	-183	10,01Episode 15,01[1118315,01] A Dangerous Receipe04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [776F6CDC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6WD0BY0X04,01 || 07Aired 2000/Mar/0607,01 with an audience share of 19.5%04,01 || 15,0107,01 AO	\N	f	0
-317	-317	10,01Episode 15,01[1131715,01] The Sullied Masked Hero (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01348mb Codec: [XviD] CRC: [2411959B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZW5IJM2304,01 || 07Aired 2003/Apr/2807,01 with an audience share of 15%04,01 || 07,01 Based on Vol 38 File 5-704,01 || 07,01Part 1 is concatenated.	\N	f	0
-169	-169	10,01Episode 15,01[1116915,01] Venus' Kiss04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [5DC05E70]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7YQV7CQL04,01 || 07Aired 1999/Nov/2207,01 with an audience share of 18.1%04,01 || 15,0107,01 AO	\N	f	0
-362	-362	10,01Episode 15,01[1136215,01] Teitan High School's Ghost Story (Part 2)04,01 || 15,01Subbed by [10Shi-Fa&KNKF15,01] 07@ 15,01178mb CRC: [5ABF0899]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=K109XSF804,01 || 07Aired 2004/May/3107,01 with an audience share of 13.1%04,01 || 07,01 Based on Vol 44 File 11 - Vol 45 File 2	\N	f	1
-472	-472	10,01Episode 15,01[1147215,01] Adventure of Young Kudo Shinichi (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [A2CC7396]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5NXIUP3104,01 || 07Aired 2007/May/1407,01 with an audience share of 10.6%04,01 || 07,01 Based on Vol 55 File 6-9	\N	f	1
-125	-125	10,01Episode 15,01[1112515,01] A Mysterious Sniper Murder Case (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 MB Codec: [XVID-MP3] CRC: [B6DE84CB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=W4QXGOPQ04,01 || 07Aired 1998/Nov/2307,01 with an audience share of 16.4%04,01 || 15,0107,01 AO	\N	f	1
-407	-407	10,01Episode 15,01[1140715,01] Conan and Heiji's Reasoning Magic (Part 2: The Mansion)04,01 || 15,0107@ 15,01gb CRC: [CD2083E2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=114T8EPO04,01 || 07Aired 2005/Jul/1807,01 with an audience share of 10.8%04,01 || 07,01 Based on Vol 47 File 8-11	\N	f	2
-398	-398	10,01Episode 15,01[1139815,01] Weird Family's Request (Part 1)04,01 || 15,0107@ 15,01gb CRC: [1DAE8482]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HZT1EMH204,01 || 07Aired 2005/May/1607,01 with an audience share of 12.7%04,01 || 07,01 Based on Vol 46 File 11 - Vol 47 File 3	\N	f	6
-18	-18	10,01Episode 15,01[111815,01] A June Bride Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01176mb CRC: [448DF412]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=79X6A6WU04,01 || 07Aired 1996/Jun/0307,01 with an audience share of 14.6%04,01 || 07,01 Based on Vol 8 File 8-10	\N	f	3
-376	-376	10,01Episode 15,01[1137615,01] Time Limit is 3 o'clock04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01172mb CRC: [F032D0F7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=U06LB4Q804,01 || 07Aired 2004/Nov/0107,01 with an audience share of 12.4%04,01 || 15,0107,01 AO	\N	f	2
-478	-478	10,01Episode 15,01[1147815,01] Real 30 Minutes04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [EE23923B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7VB70K9Y04,01 || 07Aired 2007/Jul/0907,01 with an audience share of 9.1%04,01 || 15,0107,01 AO	\N	f	1
-497	-497	10,01Episode 15,01[1149715,01] Clash of Red and Black: Awakening04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [F8C62E12 | 8D7E2EBB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=D4X7GBKH | 720: http://www.megaupload.com/?d=QFV1GTNZ04,01 || 07Aired 2008/Feb/2507,01 with an audience share of 9.8%04,01 || 07,01 Based on Vol 57 File 9 - Vol 58 File 2	\N	f	1
-518	-518	10,01Episode 15,01[1151815,01] The Meiji Restoration Mystery Tour (Part 1: Investigation Chapter)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [113F4C02]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=58ZGXWIB04,01 || 07Aired 2008/Dec/0107,01 with an audience share of 8.8%04,01 || 15,0107,01 AO	\N	f	1
-522	-522	10,01Episode 15,01[1152215,01] Shinichi's True Face & Ran's Tears (1 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb Codec: [XviD] CRC: [37764E7C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=7PLOMJRF04,01 || 07Aired 2009/Jan/2607,01 with an audience share of 11.3%04,01 || 07,01 Based on Vol 62 File 5-11	\N	f	2
-439	-439	10,01Episode 15,01[1143915,01] And It'd Be Nice If Everybody Disappeared04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [779D8E3D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6713L5A504,01 || 07Aired 2006/May/2207,01 with an audience share of 10.5%04,01 || 15,0107,01 AO	\N	f	3
-546	-546	10,01Episode 15,01[1154615,01] The Witch Enshrouded in Fog (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XViD] CRC: [2E4E7047]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=R54L2O2204,01 || 07Aired 2009/Sept/1207,01 with an audience share of 8.7%04,01 || 07,01 Based on Vol 63 File 9-11	\N	f	2
-411	-411	10,01Episode 15,01[1141115,01] The Shinto Shrine Torii's Surprising Code (Part 1)04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01188mb CRC: [61F7B59A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=B159AZP004,01 || 07Aired 2005/Aug/2207,01 with an audience share of 11.2%04,01 || 07,01 Based on Vol 48 File 1-3	\N	f	3
-384	-384	10,01Episode 15,01[1138415,01] The Target is Mouri Kogoro04,01 || 15,0107@ 15,01gb CRC: [E343A077]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=20Z852GR04,01 || 07Aired 2005/Jan/1707,01 with an audience share of 11.7%04,01 || 15,0107,01 AO	\N	f	11
-44	-44	10,01Episode 15,01[114415,01] The Three Hotta Brothers Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01167mb CRC: [10CB5E24]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JIP7DOJN04,01 || 07Aired 1997/Jan/2007,01 with an audience share of 17.6%04,01 || 15,0107,01 AO	\N	f	0
-190	-190	10,01Episode 15,01[1119015,01] The Desperate Revival: The Third Choice04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [901B0ABA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=607XFTXH04,01 || 07Aired 2000/May/1507,01 with an audience share of 19.8%04,01 || 07,01 Based on Vol 25 File 9 - Vol 26 File 7	\N	f	0
-95	-95	10,01Episode 15,01[119515,01] Kogoro's Date Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [F7838CEF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=APUAYQ9104,01 || 07Aired 1998/Mar/1607,01 with an audience share of 16.8%04,01 || 15,0107,01 AO	\N	f	0
-157	-157	10,01Episode 15,01[1115715,01] The Metropolitan Police Detective Love Story 2 (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [6F8AD6E7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DRWKVWHY04,01 || 07Aired 1999/Aug/1607,01 with an audience share of 13.6%04,01 || 07,01 Based on Vol 23 File 10 - Vol 24 File 2	\N	f	0
-599	-599	10,01Episode 15,01[1159915,01] A Friend of Justice04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [299C5A84 | 2C2E878A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BT72B3U3 | 720: http://www.megaupload.com/?d=JRC6N03Q04,01 || 07Aired 2010/Dec/1107,0104,01 || 15,0107,01 AO	\N	f	5
-601	-601	10,01Episode 15,01[1160115,01] The Dream the Kappa Saw (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [1224DF03 | 9F375B3F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SNG2QF5P | 720: http://www.megaupload.com/?d=9WS8N2KC04,01 || 07Aired 2010/Dec/2507,0104,01 || 07,01 Based on Vol 69 File 1-304,01 || 07,01Watch online at: http://dcod.ninjajosh.com/?code=Detective Conan - 601 [DCTP].m4v	\N	f	3
-587	-587	10,01Episode 15,01[1158715,01] Kid VS The Four Divine Detective Boys04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [14AF376D | 35049B33]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=N6DIPF8U | 720: http://www.megaupload.com/?d=TAIXXE5G04,01 || 07Aired 2010/Sept/1107,0104,01 || 07,01 Based on Vol 68 File 5-8	\N	f	6
-203	-203	10,01Episode 15,01[1120315,01] The Black Wings of Icarus (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350mb CRC: [57A97A97]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0JDG3QQU04,01 || 07Aired 2000/Aug/1407,01 with an audience share of 14.2%04,01 || 15,0107,01 AO	\N	f	2
-343	-343	10,01Episode 15,01[1134315,01] The Convenience Store Trap (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [76A7914B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=R143MN5L04,01 || 07Aired 2003/Dec/0107,01 with an audience share of 13%04,01 || 07,01 Based on Vol 42 File 2-4	\N	f	2
-148	-148	10,01Episode 15,01[1114815,01] The Sudden Streetcar Stopping Case04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01174mb CRC: [F6A8EBAB]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HMAOYTT104,01 || 07Aired 1999/Jun/1407,01 with an audience share of 19.7%04,01 || 15,0107,01 AO	\N	f	3
-231	-231	10,01Episode 15,01[1123115,01] The Mysterious Passenger (Part 2)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [7CFBC791]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UFU939AS04,01 || 07Aired 2001/Apr/2307,01 with an audience share of 19.3%04,01 || 07,01 Based on Vol 29 File 3-5	\N	f	1
-65	-65	10,01Episode 15,01[116515,01] A Crab and Whale Kidnapping Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [74840D91]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9R1RWHFA04,01 || 07Aired 1997/Jun/3007,01 with an audience share of 17.3%04,01 || 15,0107,01 AO	\N	f	3
-504	-504	10,01Episode 15,01[1150415,01] Clash of Red and Black: Killed in the Line of Duty04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [4BCB1005 | B9A1B707]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SDCPOACC04,01 || 07Aired 2008/May/1907,01 with an audience share of 8.1%04,01 || 07,01 Based on Vol 58 File 8 - Vol 59 File 1	\N	f	3
-106	-106	10,01Episode 15,01[1110615,01] Scoop Picture Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [766C8E09]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2RE1RNZH04,01 || 07Aired 1998/Jun/2207,01 with an audience share of 19.8%04,01 || 15,0107,01 AO	\N	f	1
-551	-551	10,01Episode 15,01[1155115,01] The Culprit is Genta's Dad (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [98C947A7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=024YEW5404,01 || 07Aired 2009/Oct/1707,0104,01 || 07,01 Based on Vol 63 File 6-8	\N	f	4
-243	-243	10,01Episode 15,01[1124315,01] Mori Kogoro's Imposter (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 Codec: [XviD] CRC: [EAA4972B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NW97NYLK04,01 || 07Aired 2001/Jul/2307,01 with an audience share of 19.7%04,01 || 07,01 Based on Vol 31 File 2-4	\N	f	1
-313	-313	10,01Episode 15,01[1131315,01] Festival Dolls Dyed in the Setting Sun (Part 2)04,01 || 15,0107@ 15,01gb Codec: [XviD] CRC: [0CC54970]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2VVQLRI004,01 || 07Aired 2003/Mar/1007,01 with an audience share of 14.3%04,01 || 07,01 Based on Vol 38 File 2-4	\N	f	2
-298	-298	10,01Episode 15,01[1129815,01] Courtroom Confrontation 2: Kisaki VS Kujo Reiko (Part 2)04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9PK4NA7204,01 || 07Aired 2002/Oct/2807,01 with an audience share of 16.8%04,01 || 15,0107,01 AO	\N	f	3
-529	-529	10,01Episode 15,01[1152915,01] Might Over Mystery (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [0D7F6740]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=S90Z0G6Z04,01 || 07Aired 2009/Mar/1607,01 with an audience share of 8.2%04,01 || 07,01 Based on Vol 62 File 2-4	\N	f	2
-81	-81	10,01Episode 15,01[118115,01] The Kidnapping of a Popular Artist Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [7089777F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RAF7CJ8E04,01 || 07Aired 1997/Nov/1707,01 with an audience share of 20.6%04,01 || 07,01 Based on Vol 15 File 4-6	\N	f	0
-247	-247	10,01Episode 15,01[1124715,01] The Mystery in the Net (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01347mb Codec: [XviD] CRC: [5F7ECB81]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=BZJEMGS904,01 || 07Aired 2001/Aug/2007,01 with an audience share of 19.6%04,01 || 07,01 Based on Vol 31 File 5-704,01 || 07,01Part 1 is concatenated.	\N	f	0
-186	-186	10,01Episode 15,01[1118615,01] The Murdered Famous Detective (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [B5FAC159]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L5GI9ARX04,01 || 07Aired 2000/Apr/1707,01 with an audience share of 18.9%04,01 || 15,0107,01 AO	\N	f	0
-284	-284	10,01Episode 15,01[1128415,01] Chinatown Deja Vu in the Rain (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01247mb CRC: [A9A544F9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=38IC7KC904,01 || 07Aired 2002/Jul/0107,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 34 File 8-10	\N	f	1
-459	-459	10,01Episode 15,01[1145915,01] A Mysterious Man, Overly Strict with Regulations04,01 || 15,01Subbed by [10HnI15,01] 07@ 15,01160mb CRC: [0C0519C4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ATT7UH4X04,01 || 07Aired 2006/Dec/0407,01 with an audience share of 10.9%04,01 || 15,0107,01 AO	\N	f	1
-204	-204	10,01Episode 15,01[1120415,01] The Black Wings of Icarus (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350mb CRC: [57A97A97]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0JDG3QQU04,01 || 07Aired 2000/Aug/2107,01 with an audience share of 18.7%04,01 || 15,0107,01 AO	\N	f	1
-435	-435	10,01Episode 15,01[1143515,01] Information Gathered on the Detective Boys (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01169mb CRC: [87DACF98]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2E29Y4L004,01 || 07Aired 2006/Apr/1707,01 with an audience share of 9.7%04,01 || 07,01 Based on Vol 50 File 5-7	\N	f	1
-339	-339	10,01Episode 15,01[1133915,01] Four Porsche's (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb CRC: [C8667C21]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=J4L5N6UL04,01 || 07Aired 2003/Oct/2707,01 with an audience share of 11.7%04,01 || 07,01 Based on Vol 41 File 7-9	\N	f	1
-268	-268	10,01Episode 15,01[1126815,01] The Truth Behind Valentine's (Part 3: The Solution)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [87E57591]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DM8BFEBE04,01 || 07Aired 2002/Feb/1107,01 with an audience share of 21.2%04,01 || 07,01 Based on Vol 33 File 3-615,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 6	\N	f	1
-371	-371	10,01Episode 15,01[1137115,01] A Course Without Protest (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01168mb CRC: [69C27BA7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DA1MEP9604,01 || 07Aired 2004/Aug/2307,01 with an audience share of 11.8%04,01 || 07,01 Based on Vol 45 File 6-8	\N	f	2
-165	-165	10,01Episode 15,01[1116515,01] The Case of Disappearing Detective Boys04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [62E3F9FC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AJ8KCHGP04,01 || 07Aired 1999/Oct/2507,01 with an audience share of 18%04,01 || 15,0107,01 AO	\N	f	1
-289	-289	10,01Episode 15,01[1128915,01] Mitsuhiko's Mystifying Forest (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [7FA43E1F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0WZ1B5HZ04,01 || 07Aired 2002/Aug/0507,01 with an audience share of 15%04,01 || 07,01 Based on Vol 35 File 8-1015,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 9	\N	f	7
-86	-86	10,01Episode 15,01[118615,01] The Kidnapping Location Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [81F600B8]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=C9XT8LYA04,01 || 07Aired 1998/Jan/1207,01 with an audience share of 22.7%04,01 || 07,01 Based on Special Vol 1 File 2	\N	f	2
-321	-321	10,01Episode 15,01[1132115,01] The Vanished Kidnapper's Getaway Car (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01??? mb CRC: [5F8FBC7D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V5V7UA0V04,01 || 07Aired 2003/May/2607,01 with an audience share of 14.2%04,01 || 15,0107,01 AO	\N	f	3
-531	-531	10,01Episode 15,01[1153115,01] The Truth Behind the Urban Legend (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [884F3F2A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=S3BF3IZ104,01 || 07Aired 2009/Apr/2507,01 with an audience share of 9.1%04,01 || 07,01 Based on Vol 60 File 6-8	\N	f	1
-228	-228	10,01Episode 15,01[1122815,01] The Murderous Pottery Class (Part 1)04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01174mb Codec: [XviD] CRC: [BEF9F901]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VCKK5D9I04,01 || 07Aired 2001/Mar/1207,01 with an audience share of 19.4%04,01 || 07,01 Based on Vol 30 File 10 - Vol 31 File 115,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 6	\N	f	2
-2	-2	10,01Episode 15,01[11215,01] Company President's Daughter Kidnapping Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01224mb Codec: [XviD] CRC: [3E743379]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=84KW7S4004,01 || 07Aired 1996/Jan/1507,01 with an audience share of 11.9%04,01 || 07,01 Based on Vol 1 File 2-5	\N	f	8
-401	-401	10,01Episode 15,01[1140115,01] The Caught Red-handed Jewel Robber (Part 1)04,01 || 15,01Subbed by [10Ani-Jiyuu15,01] 07@ 15,01176mb CRC: [606359E2]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=N0776DLO04,01 || 07Aired 2005/Jun/0607,01 with an audience share of 10.6%04,01 || 07,01 Based on Vol 47 File 5-7	\N	f	2
-147	-147	10,01Episode 15,01[1114715,01] The Metropolitan Police Detective Love Story (Part 2)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01175mb CRC: [93EC64F9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GPB9KXZI04,01 || 07Aired 1999/Jun/0707,01 with an audience share of 20.2%04,01 || 07,01 Based on Vol 21 File 8-10	\N	f	1
-223	-223	10,01Episode 15,01[1122315,01] And Then There Were No Mermaids (Part 2: The Deduction)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb CRC: [1388584C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VVHLEWU504,01 || 07Aired 2001/Feb/0507,01 with an audience share of 19.6%04,01 || 07,01 Based on Vol 28 File 6-10	\N	f	0
-448	-448	10,01Episode 15,01[1144815,01] The Black-eyed Saury Case04,01 || 15,0107@ 15,01gb CRC: [3F19027A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SCJ6V6V004,01 || 07Aired 2006/Aug/1407,01 with an audience share of 8%04,01 || 15,0107,01 AO	\N	f	0
-596	-596	10,01Episode 15,01[1159615,01] The Alibi of the Fall04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [7B5353ED | 6EA6D9A5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=US4YSTLV | 720: http://www.megaupload.com/?d=M9CQARCW04,01 || 07Aired 2010/Nov/2007,0104,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01http://www.fileserve.com/file/EezFsby | 720: http://www.fileserve.com/file/X8sF33N	\N	f	2
-282	-282	10,01Episode 15,01[1128215,01] The Mystery of the Water-Flowing Stone Garden (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 MB Codec: [XviD] CRC: [9BED9C34]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SG6A7VR704,01 || 07Aired 2002/Jun/1707,01 with an audience share of 16.5%04,01 || 15,0107,01 AO	\N	f	1
-319	-319	10,01Episode 15,01[1131915,01] The Cigar Case of Good Fortune (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [8800AA1E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=XEBSD6IE04,01 || 07Aired 2003/May/1207,01 with an audience share of 15.1%04,01 || 15,0107,01 AO	\N	f	2
-216	-216	10,01Episode 15,01[1121615,01] The Bay of Revenge (Part 2)04,01 || 15,0107@ 15,01gb Codec: [XviD] CRC: [F638835B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L062THZZ04,01 || 07Aired 2000/Dec/0407,01 with an audience share of 18.1%04,01 || 15,0107,01 AO	\N	f	3
-121	-121	10,01Episode 15,01[1112115,01] The Sealed Bathroom Murder Case (Part 1)04,01 || 15,01Subbed by [10Puto-F15,01] 07@ 15,01250MB Codec: [H264] CRC: [ce857e31]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HNOGWOBO04,01 || 07Aired 1998/Oct/2607,01 with an audience share of 20.9%04,01 || 07,01 Based on Vol 20 File 7-9	\N	f	4
-494	-494	10,01Episode 15,01[1149415,01] Clash of Red and Black: Hades04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [1E53B779 | 277D1F90]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=NHCG9K3I | 720: http://www.megaupload.com/?d=XBGP00BU04,01 || 07Aired 2008/Feb/0407,01 with an audience share of 9.4%04,01 || 07,01 Based on Vol 56 File 10 - Vol 57 File 4	\N	f	1
-344	-344	10,01Episode 15,01[1134415,01] The Convenience Store Trap (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [7D7E0B51]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SXGIGHCO04,01 || 07Aired 2003/Dec/0807,01 with an audience share of 16%04,01 || 07,01 Based on Vol 42 File 2-4	\N	f	4
-238	-238	10,01Episode 15,01[1123815,01] The Osaka 3 "K" Case (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb CRC: [BB1D048A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=GEV36P9H04,01 || 07Aired 2001/Jun/1807,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 29 File 9-11	\N	f	3
-561	-561	10,01Episode 15,01[1156115,01] The Mansion of Death and the Red Wall: Empty Fort Strategy04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD] CRC: [268F7374 | 02ED1FA9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6BB91NZE | 720: http://www.megaupload.com/?d=OC8HMI1204,01 || 07Aired 2009/Dec/2607,0104,01 || 07,01 Based on Vol 65 File 8 - Vol 66 File 1	\N	f	2
-420	-420	10,01Episode 15,01[1142015,01] Sword of the Eight-headed Serpent (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [EED6A84C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=FJTUVUNW04,01 || 07Aired 2005/Nov/1407,01 with an audience share of 10.7%04,01 || 15,0107,01 AO	\N	f	2
-172	-172	10,01Episode 15,01[1117215,01] The Revival of the Dying Message (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [45562C92]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5FMJCDDM04,01 || 07Aired 1999/Dec/1307,01 with an audience share of 17.3%04,01 || 07,01 Based on Vol 25 File 1-3	\N	f	2
-345	-345	10,01Episode 15,01[1134515,01] Head-to-Head Match With Black Organization, Two Mysteries on the Night of a Full Moon (2.5 Hour Special)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01698mb | 698mb CRC: [48D4549B | 1BF6B09E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DNE6LPTU | http://www.megaupload.com/?d=LAOOK4BH04,01 || 07Aired 2004/Jan/0507,01 with an audience share of 16.7%04,01 || 07,01 Based on Vol 42 File 5-10	\N	f	4
-209	-209	10,01Episode 15,01[1120915,01] The Falling from Mt. Ryuujin Case04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01171mb Codec: [XviD] CRC: [E259CFCF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=0HA1RB6M04,01 || 07Aired 2000/Oct/1607,01 with an audience share of 18.9%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 3	\N	f	2
-487	-487	10,01Episode 15,01[1148715,01] The Metropolitan Police Detective Love Story 8, The Ring on the Left Hand (1 Hour Special)04,01 || 15,01Subbed by [10DCTP-Neg15,01] 07@ 15,01347mb | 466mb CRC: [31221C8B | 0E97872A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4CM6UAJ404,01 || 07Aired 2007/Sep/1507,01 with an audience share of 10.9%04,01 || 07,01 Based on Vol 56 File 1-3	\N	f	4
-516	-516	10,01Episode 15,01[1151615,01] Fuurinkazan, The Mysterious Armored Warrior (Part 1) (1 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01344mb CRC: [FF03FAE3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WP85M32604,01 || 07Aired 2008/Nov/0307,01 with an audience share of 10.8%04,01 || 07,01 Based on Vol 59 File 5-10	\N	f	2
-404	-404	10,01Episode 15,01[1140415,01] The Mysterious Angel's Mansion (Part 2)04,01 || 15,0107@ 15,01gb CRC: [F2905876]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=94ORX99H04,01 || 07Aired 2005/Jun/2707,01 with an audience share of 11%04,01 || 15,0107,01 AO	\N	f	3
-179	-179	10,01Episode 15,01[1117915,01] The Case of the Coffee Shop Truck's Wild Entrance04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [D86DF1D1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z9YAGKZ504,01 || 07Aired 2000/Feb/0707,01 with an audience share of 20.1%04,01 || 15,0107,01 AO	\N	f	0
-92	-92	10,01Episode 15,01[119215,01] The Fearful Mountain Trek Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [F38B0AFC]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5MOFKUAD04,01 || 07Aired 1998/Feb/2307,01 with an audience share of 19.7%04,01 || 15,0107,01 AO	\N	f	0
-277	-277	10,01Episode 15,01[1127715,01] English Teacher VS Great Western Detective (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01173mb CRC: [A1BE301B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=ZA2HZBLW04,01 || 07Aired 2002/May/1307,01 with an audience share of 17.7%04,01 || 07,01 Based on Vol 34 File 2-4	\N	f	1
-592	-592	10,01Episode 15,01[1159215,01] The Detective Memoir of Monkey and Rake (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [DF204E87 | 1B3A3FF3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Y5J0DSWM | 720: http://www.megaupload.com/?d=H0YVTGUK04,01 || 07Aired 2010/Oct/2307,0104,01 || 07,01 Based on Vol 68 File 9-11	\N	f	3
-550	-550	10,01Episode 15,01[1155015,01] The Revolving Sushi Mystery (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01170mb | 300mb Codec: [XViD] CRC: [19FEBA38 | 5E698F8A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DADCHFFU | 720: http://www.megaupload.com/?d=I1CW5NF104,01 || 07Aired 2009/Oct/1007,0104,01 || 07,01 Based on Vol 63 File 3-5	\N	f	4
-297	-297	10,01Episode 15,01[1129715,01] Courtroom Confrontation 2: Kisaki VS Kujo Reiko (Part 1)04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9PK4NA7204,01 || 07Aired 2002/Oct/2107,01 with an audience share of 16.9%04,01 || 15,0107,01 AO	\N	f	5
-133	-133	10,01Episode 15,01[1113315,01] The Magic Lovers' Murder Case (Suspicion Part)04,01 || 15,01Subbed by [10Puto15,01] 07@ 15,01250MB Codec: [H264-AC3] CRC: [781c7b1a]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MNCPBWJ504,01 || 07Aired 1999/Feb/0107,01 with an audience share of 21.2%04,01 || 07,01 Based on Vol 20 File 2-6	\N	f	1
-457	-457	10,01Episode 15,01[1145715,01] Sonoko's Red Handkerchief (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01175mb CRC: [94459766]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DVWZN9V504,01 || 07Aired 2006/Nov/2007,01 with an audience share of 10.2%04,01 || 07,01 Based on Vol 52 File 9-11	\N	f	1
-336	-336	10,01Episode 15,01[1133615,01] Secret of the Touto Film Developer Place (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [DA3BF7B0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=DMT7LE1H04,01 || 07Aired 2003/Sep/1507,01 with an audience share of 12.5%04,01 || 07,01 Based on Vol 41 File 4-6	\N	f	1
-357	-357	10,01Episode 15,01[1135715,01] Sweetheart is an Illusion of Spring04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb Codec: [XviD] CRC: [BBCA9E2E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RQPGK4QX04,01 || 07Aired 2004/Apr/2607,01 with an audience share of 12.4%04,01 || 15,0107,01 AO	\N	f	4
-63	-63	10,01Episode 15,01[116315,01] Big Monster Gomera Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [DC7F5E02]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=N0VQRGZX04,01 || 07Aired 1997/Jun/1607,01 with an audience share of 17.7%04,01 || 07,01 Based on Vol13 File 8-10	\N	f	1
-117	-117	10,01Episode 15,01[1111715,01] The Mystery Writer Disappearance Case (Part 2)04,01 || 15,01Subbed by [10Puto-F15,01] 07@ 15,01404MB Codec: [H264-AAC] CRC: [de80222a]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YPZQBEBS04,01 || 07Aired 1998/Sep/2107,01 with an audience share of 17.4%04,01 || 07,01 Based on Vol 19 File 2-5	\N	f	1
-149	-149	10,01Episode 15,01[1114915,01] The Amusement Park Bungee Jumping Case04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01176mb CRC: [3C7D7980]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6F440EZV04,01 || 07Aired 1999/Jun/2107,01 with an audience share of 19.3%04,01 || 15,0107,01 AO	\N	f	2
-290	-290	10,01Episode 15,01[1129015,01] Mitsuhiko's Mystifying Forest (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [9AD69D2B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PL2XAUYR04,01 || 07Aired 2002/Aug/1207,01 with an audience share of 14.7%04,01 || 07,01 Based on Vol 35 File 8-1015,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 10	\N	f	4
-455	-455	10,01Episode 15,01[1145515,01] Upsetting Outcome (Part 2)04,01 || 15,01Subbed by [10anfs15,01] 07@ 15,01184mb Codec: [XviD] CRC: [5E7FA899]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KTHGVUF404,01 || 07Aired 2006/Nov/0607,01 with an audience share of 8.7%04,01 || 07,01 Based on Vol 52 File 6-8	\N	f	2
-10	-10	10,01Episode 15,01[111015,01] Pro Soccer Player Blackmail Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01215mb CRC: [495E2C38]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AKCGW1FV04,01 || 07Aired 1996/Mar/1107,01 with an audience share of 13.7%04,01 || 07,01 Based on Vol 7 File 8 - Vol 8 File 1	\N	f	4
-383	-383	10,01Episode 15,01[1138315,01] Miracle at Koshien Ball Park! The Defiants Face the Dark Demon (2 Hour Special)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01699mb CRC: [5B322B97]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=JAF94ZUV04,01 || 07Aired 2004/Dec/2007,01 with an audience share of 12.5%04,01 || 07,01 Based on Vol 43 File 10 - Vol 44 File 3	\N	f	3
-419	-419	10,01Episode 15,01[1141915,01] Sword of the Eight-headed Serpent (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [DD6451DA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CY0BTI1004,01 || 07Aired 2005/Nov/0707,01 with an audience share of 11.2%04,01 || 15,0107,01 AO	\N	f	2
-505	-505	10,01Episode 15,01[1150515,01] Lawyer Kisaki Eri's Testimony (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 485mb CRC: [9E92A8EA | 76D08CBA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HG8ZNL2Y04,01 || 07Aired 2008/Jun/1607,01 with an audience share of 7.9%04,01 || 07,01 Based on Vol 59 File 2-4	\N	f	4
-173	-173	10,01Episode 15,01[1117315,01] The Revival of the Dying Message (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [F419783C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MD4CWP7D04,01 || 07Aired 1999/Dec/2007,01 with an audience share of 15.9%04,01 || 07,01 Based on Vol 25 File 1-3	\N	f	0
-306	-306	10,01Episode 15,01[1130615,01] The Unseen Suspect (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01348mb Codec: [XviD] CRC: [221A7174]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=22ADVSIO04,01 || 07Aired 2003/Jan/2007,01 with an audience share of 16.2%04,01 || 07,01 Based on Vol 37 File 2-404,01 || 07,01Part 1 is concatenated.	\N	f	0
-406	-406	10,01Episode 15,01[1140615,01] Conan and Heiji's Reasoning Magic (Part 1: The Trick)04,01 || 15,0107@ 15,01gb CRC: [5D43038E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=3YWZLJC004,01 || 07Aired 2005/Jul/1107,01 with an audience share of 11.4%04,01 || 07,01 Based on Vol 47 File 8-11	\N	f	3
-54	-54	10,01Episode 15,01[115415,01] A Game Company's Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [1DCAD67E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RKK2IL0Q04,01 || 07Aired 1997/Apr/1407,01 with an audience share of 19.1%04,01 || 07,01 Based on Vol 12 File 4-6	\N	f	1
-66	-66	10,01Episode 15,01[116615,01] Night Road Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [00A17344]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z5N633IP04,01 || 07Aired 1997/Jul/0707,01 with an audience share of 18%04,01 || 15,0107,01 AO	\N	f	1
-304	-304	10,01Episode 15,01[1130415,01] Trembling Metropolitan Police Headquarters, 12 Million Hostages (2 Hour Special)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01667mb CRC: [56216D8A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=CVMH6EUM04,01 || 07Aired 2003/Jan/0607,01 with an audience share of 17%04,01 || 07,01 Based on Vol 36 File 8 - Vol 37 File 1	\N	f	2
-70	-70	10,01Episode 15,01[117015,01] The Night Baron Murder Case (Part 3: The Solution)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [8A40F61C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WFV8GZDW04,01 || 07Aired 1997/Aug/0407,01 with an audience share of 17.2%04,01 || 07,01 Based on Vol 8 File 2-7	\N	f	1
-123	-123	10,01Episode 15,01[1112315,01] The Weather Lady Abduction Case04,01 || 15,01Subbed by [10Puto-F15,01] 07@ 15,01250MB Codec: [H264] CRC: [f44f8771]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=83IBPHUQ04,01 || 07Aired 1998/Nov/0907,01 with an audience share of 21.7%04,01 || 15,0107,01 AO	\N	f	2
-56	-56	10,01Episode 15,01[115615,01] The Ojamanbou Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [2A8770F5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6TGSX65B04,01 || 07Aired 1997/Apr/2807,01 with an audience share of 17.6%04,01 || 15,0107,01 AO	\N	f	1
-210	-210	10,01Episode 15,01[1121015,01] The Water Palace of Five Colors (Part 1)04,01 || 15,0107@ 15,01gb CRC: [9B826A8A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Y7G3C9DQ04,01 || 07Aired 2000/Oct/2307,01 with an audience share of 19.6%04,01 || 15,0107,01 AO	\N	f	6
-396	-396	10,01Episode 15,01[1139615,01] Big Adventure at the Unconventional Mansion (Solution Part))04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [89E40A09]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RYZOLBD304,01 || 07Aired 2005/May/0207,01 with an audience share of 10%04,01 || 07,01 Based on Vol 46 File 7-1004,01 || 07,01http://www.nyaa.eu/?page=download&tid=20210015,0104,01 || \n07,01Bot Listing: 15,01/MSG [xdcc]Ecchilicious XDCC SEND 377	\N	f	6
-530	-530	10,01Episode 15,01[1153015,01] The Truth Behind the Urban Legend (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [DivX] CRC: [3302AE4A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=LGZCUYLY04,01 || 07Aired 2009/Apr/1807,01 with an audience share of 5.8%04,01 || 07,01 Based on Vol 60 File 6-8	\N	f	2
-166	-166	10,01Episode 15,01[1116615,01] The Monster of the Tottori Spider Mansion (Part 1: The Murder)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [CFBF0F53]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HY3QK59X04,01 || 07Aired 1999/Nov/0107,01 with an audience share of 18.5%04,01 || 07,01 Based on Vol 25 File 4-8	\N	f	2
-482	-482	10,01Episode 15,01[1148215,01] The Mountain Witch's Cutlery (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01348mb Codec: [XviD] CRC: [AA275948]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KXOH1SER04,01 || 07Aired 2007/Aug/0607,01 with an audience share of 8.2%04,01 || 07,01 Based on Vol 56 File 4-604,01 || 07,01Part 1 is concatenated.	\N	f	1
-499	-499	10,01Episode 15,01[1149915,01] Clash of Red and Black: Disguise04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 233mb CRC: [D12FD54B | 5BC2C4DF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=EADHT74Q | 720: http://www.megaupload.com/?d=FTV8YI4Y04,01 || 07Aired 2008/Mar/1007,01 with an audience share of 8.5%04,01 || 07,01 Based on Vol 58 File 3 - Vol 58 File 7	\N	f	1
-365	-365	10,01Episode 15,01[1136515,01] Synchronicity Case (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [6C9C1C1F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=6F1FEFEK04,01 || 07Aired 2004/Jun/2107,01 with an audience share of 14%04,01 || 15,0107,01 AO	\N	f	4
-547	-547	10,01Episode 15,01[1154715,01] Two Days With the Culprit (First Day)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb Codec: [XviD] CRC: [FA8B445E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=TZT8SDOQ04,01 || 07Aired 2009/Sept/2007,01 with an audience share of 9%04,01 || 15,0107,01 AO	\N	f	4
-19	-19	10,01Episode 15,01[111915,01] An Elevator Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [AFE1C198]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=5781ADMM04,01 || 07Aired 1996/Jun/1007,01 with an audience share of 15.6%04,01 || 15,0107,01 AO	\N	f	3
-470	-470	10,01Episode 15,01[1147015,01] Kaitou Kid and the 4 Masterpieces (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [D1AB9D56]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YKXUQ9MY04,01 || 07Aired 2007/Apr/2307,01 with an audience share of 10.7%04,01 || 07,01 Based on Vol 53 File 1-4	\N	f	2
-30	-30	10,01Episode 15,01[113015,01] Alibi Testimony Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01170mb CRC: [92A002D6]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=H1T5H03604,01 || 07Aired 1996/Aug/2607,01 with an audience share of 13.7%04,01 || 15,0107,01 AO	\N	f	0
-283	-283	10,01Episode 15,01[1128315,01] The Mystery of the Water-Flowing Stone Garden (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01350 MB Codec: [XviD] CRC: [9BED9C34]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SG6A7VR704,01 || 07Aired 2002/Jun/2407,01 with an audience share of 16.6%04,01 || 15,0107,01 AO	\N	f	0
-156	-156	10,01Episode 15,01[1115615,01] The Metropolitan Police Detective Love Story 2 (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01172mb CRC: [81584EFD]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VU0GNROI04,01 || 07Aired 1999/Aug/0907,01 with an audience share of 16.9%04,01 || 07,01 Based on Vol 23 File 10 - Vol 24 File 2	\N	f	0
-88	-88	10,01Episode 15,01[118815,01] Dracula Villa Murder Case (Part 1)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [FF5B52A4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=KE644CDY04,01 || 07Aired 1998/Jan/2607,01 with an audience share of 20.8%04,01 || 15,0107,01 AO	\N	f	2
-141	-141	10,01Episode 15,01[1114115,01] The Night Before the Wedding Locked Room Murder Case (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01173mb CRC: [BA8BB5AE]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2MY9ZA3T04,01 || 07Aired 1999/Apr/1907,01 with an audience share of 19.4%04,01 || 07,01 Based on Vol 21 File 11 - Vol 22 File 1-3	\N	f	1
-134	-134	10,01Episode 15,01[1113415,01] The Magic Lovers' Murder Case (Case Closed)04,01 || 15,01Subbed by [10Puto15,01] 07@ 15,01255MB Codec: [H264-AC3] CRC: [38c90d6e]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=HDULTMR804,01 || 07Aired 1999/Feb/0807,01 with an audience share of 23%04,01 || 07,01 Based on Vol 20 File 2-6	\N	f	1
-219	-219	10,01Episode 15,01[1121915,01] The Gathered Detectives! Shinichi VS Kaito Kid (2 Hour Special)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01691mb CRC: [50034C1F]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=M7D8GP5U04,01 || 07Aired 2001/Jan/0807,01 with an audience share of 17.6%04,01 || 07,01 Based on Vol 30 File 4-7	\N	f	2
-570	-570	10,01Episode 15,01[1157015,01] The Crime with Zero Possibility to Be Proven04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 321mb Codec: [XviD | H264] CRC: [60676799 | 6F5879B9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UDQKHT2Z | 720: http://www.megaupload.com/?d=4O9VC2FF04,01 || 07Aired 2010/Mar/2707,0104,01 || 15,0107,01 AO04,01 || 07,01There is a typo in filename's crc. It says 0676799, first digit is missing.	\N	f	3
-59	-59	10,01Episode 15,01[115915,01] The First Errand Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [5DBA8AA7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MP146DUO04,01 || 07Aired 1997/May/1907,01 with an audience share of 19.2%04,01 || 15,0107,01 AO	\N	f	1
-13	-13	10,01Episode 15,01[111315,01] The Strange Person Hunt Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01220mb CRC: [D4CF9F2C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WCPQ78V004,01 || 07Aired 1996/Apr/2207,01 with an audience share of 12.7%04,01 || 07,01 Based on Vol 2 File 4-6	\N	f	3
-75	-75	10,01Episode 15,01[117515,01] The Loan Company President's Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [B6B2645E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=64OT6IP204,01 || 07Aired 1997/Sep/0807,01 with an audience share of 18.5%04,01 || 07,01 Based on Vol 15 File 7-9	\N	f	1
-271	-271	10,01Episode 15,01[1127115,01] Secret Rushed Omission (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [7557DBA3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SVIFZO2F04,01 || 07Aired 2002/Mar/1107,01 with an audience share of 19.6%04,01 || 07,01 Based on Vol 33 File 10 - Vol 34 File 1	\N	f	2
-105	-105	10,01Episode 15,01[1110515,01] The Mysterious Robbers and Mansion Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [B3A96D15]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=9VLYKCJT04,01 || 07Aired 1998/Jun/1507,01 with an audience share of 19.1%04,01 || 07,01 Based on Vol 17 File 7-10	\N	f	1
-450	-450	10,01Episode 15,01[1145015,01] Trick VS Magic (Part 1)04,01 || 15,0107@ 15,01gb CRC: [600984B9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=C5MSQMR104,01 || 07Aired 2006/Aug/2807,01 with an audience share of 9.4%04,01 || 15,0107,01 AO	\N	f	1
-403	-403	10,01Episode 15,01[1140315,01] The Mysterious Angel's Mansion (Part 1)04,01 || 15,0107@ 15,01gb CRC: [50313521]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RWTULF9K04,01 || 07Aired 2005/Jun/2007,01 with an audience share of 11.5%04,01 || 15,0107,01 AO	\N	f	6
-318	-318	10,01Episode 15,01[1131815,01] The Cigar Case of Good Fortune (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01171mb Codec: [H264] CRC: [F0DB8BA4]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=L8I7VEF004,01 || 07Aired 2003/May/0507,01 with an audience share of 14.6%04,01 || 15,0107,01 AO	\N	f	2
-491	-491	10,01Episode 15,01[1149115,01] Clash of Red and Black: The Beginning04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 233mb CRC: [81DC9A9C | BD36FFA0]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PKQ84P4Z04,01 || 07Aired 2008/Jan/1407,01 with an audience share of 9.6%04,01 || 07,01 Based on Vol 56 File 10 - Vol 57 File 4	\N	f	3
-466	-466	10,01Episode 15,01[1146615,01] Unsmashable Snowman (Part 1)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01172mb CRC: [98ED9167]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=Z5DQE64E04,01 || 07Aired 2007/Feb/2607,01 with an audience share of 9.9%04,01 || 07,01 Based on Vol 54 File 3-5	\N	f	3
-421	-421	10,01Episode 15,01[1142115,01] Gingko-Colored First Love (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [2EA3BA75]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UD0WSN0F04,01 || 07Aired 2005/Nov/2107,01 with an audience share of 9.6%04,01 || 07,01 Based on Vol 40 File 7-915,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 22	\N	f	3
-185	-185	10,01Episode 15,01[1118515,01] The Murdered Famous Detective (Part 1)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01175mb CRC: [BF7F4950]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=APMK65WD04,01 || 07Aired 2000/Apr/1007,01 with an audience share of 16.2%04,01 || 15,0107,01 AO	\N	f	0
-196	-196	10,01Episode 15,01[1119615,01] The Invisible Weapon, Ran's First Investigations04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01171mb CRC: [40A35E42]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=TVN26ZOH04,01 || 07Aired 2000/Jun/2607,01 with an audience share of 18.7%04,01 || 15,0107,01 AO	\N	f	0
-4	-4	10,01Episode 15,01[11415,01] The Coded Map of the City Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01210mb Codec: [XviD] CRC: [EC789C22]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=2U8LQP1404,01 || 07Aired 1996/Jan/2907,01 with an audience share of 12.2%04,01 || 07,01 Based on Vol 4 File 7-10	\N	f	2
-164	-164	10,01Episode 15,01[1116415,01] The Secret of the Moon, the Star, and the Sun (Part 2)04,01 || 15,01Subbed by [10KnKF15,01] 07@ 15,01172mb CRC: [648025B9]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=WMBNMUN804,01 || 07Aired 1999/Oct/1807,01 with an audience share of 18.3%04,01 || 07,01 Based on Vol 12 File 1-3	\N	f	1
-390	-390	10,01Episode 15,01[1139015,01] The Metropolitan Police Detective Love Story 6 (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb CRC: [859DEC4D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=D729C6E104,01 || 07Aired 2005/Feb/2807,01 with an audience share of 13.2%04,01 || 07,01 Based on Vol 44 File 4-6	\N	f	4
-528	-528	10,01Episode 15,01[1152815,01] Might Over Mystery (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [2A348B59]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=X3PTEMC104,01 || 07Aired 2009/Mar/0907,01 with an audience share of 7.4%04,01 || 07,01 Based on Vol 62 File 2-4	\N	f	2
-242	-242	10,01Episode 15,01[1124215,01] Genta's Disaster04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01172mb Codec: [XviD] CRC: [D33CD745]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=66NGF06004,01 || 07Aired 2001/Jul/1607,01 with an audience share of 19.4%04,01 || 07,01 Based on Vol 30 File 8-915,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 12	\N	f	2
-597	-597	10,01Episode 15,01[1159715,01] The Scenario of the Steaming Locked Room (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [1C171D92 | 14502DDA]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4W657AYB | 720: http://www.megaupload.com/?d=4VHHCIU104,01 || 07Aired 2010/Nov/2707,0104,01 || 07,01 Based on Vol 69 File 4-6	\N	f	3
-111	-111	10,01Episode 15,01[1111115,01] Cooking Classroom Murder Case (Part 2)04,01 || 15,01Subbed by [10Atavus15,01] 07@ 15,01172mb Codec: [XviD] CRC: [12DBC0DF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=467O0NZA04,01 || 07Aired 1998/Aug/0307,01 with an audience share of 18%04,01 || 15,0107,01 AO	\N	f	1
-38	-38	10,01Episode 15,01[113815,01] Akaoni Village Fire Festival Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01160mb CRC: [811948F7]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=4M59JZS204,01 || 07Aired 1996/Nov/1807,01 with an audience share of 17.8%04,01 || 07,01 Based on Vol 2 File 1-3	\N	f	1
-248	-248	10,01Episode 15,01[1124815,01] The Alibi of the Soothing Forest04,01 || 15,01Subbed by [10AZFS15,01] 07@ 15,01173mb CRC: [526F3797]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=E8IZIEJ804,01 || 07Aired 2001/Aug/2707,01 with an audience share of 19.3%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #AZFS and type /msg AZFS|Kudo XDCC send 13	\N	f	4
-3	-3	10,01Episode 15,01[11315,01] An Idol's Locked Room Murder Case04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01177mb Codec: [XviD] CRC: [2E9C1FF3]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=AKZFY76V04,01 || 07Aired 1996/Jan/2207,01 with an audience share of 11.4%04,01 || 07,01 Based on Vol 1 File 6-9	\N	f	2
-473	-473	10,01Episode 15,01[1147315,01] Adventure of Young Kudo Shinichi (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb CRC: [3BF6B43B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=UJYJ1GCQ04,01 || 07Aired 2007/May/2107,01 with an audience share of 11%04,01 || 07,01 Based on Vol 55 File 6-9	\N	f	2
-578	-578	10,01Episode 15,01[1157815,01] The Crisis Beckoned by the Red Omen04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD | H264] CRC: [6AE111CB | 4F29D81D]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RFS11OSR | 720: http://www.megaupload.com/?d=C1IFRHHE04,01 || 07Aired 2010/Jun/2607,0104,01 || 07,01 Based on Vol 67 File 3	\N	f	6
-132	-132	10,01Episode 15,01[1113215,01] The Magic Lovers' Murder Case (Case Part)04,01 || 15,01Subbed by [10Puto15,01] 07@ 15,01250MB Codec: [H264-AC3] CRC: [14715c2e]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SBP7ER9W04,01 || 07Aired 1999/Jan/2507,01 with an audience share of 21.9%04,01 || 07,01 Based on Vol 20 File 2-6	\N	f	4
-356	-356	10,01Episode 15,01[1135615,01] Kaitou Kid's Miracle Midair Walk (1 Hour Special)04,01 || 15,01Subbed by [10Kaizou15,01] 07@ 15,01336mb CRC: [0FE85F33]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=YOXBL51M04,01 || 07Aired 2004/Apr/1207,01 with an audience share of 13.7%04,01 || 07,01 Based on Vol 44 File 7-10	\N	f	2
-408	-408	10,01Episode 15,01[1140815,01] Conan and Heiji's Reasoning Magic (Part 3: The Solution)04,01 || 15,0107@ 15,01gb CRC: [9EB951CF]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=MU74XD0U04,01 || 07Aired 2005/Aug/0107,01 with an audience share of 8.6%04,01 || 07,01 Based on Vol 47 File 8-11	\N	f	13
-577	-577	10,01Episode 15,01[1157715,01] The Truth Lit Up by the Fireflies04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb | 320mb Codec: [XviD | H264] CRC: [17768A3D | E5FC7B2B]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VVEAXHHX | 720: http://www.megaupload.com/?d=JPH16K0A04,01 || 07Aired 2010/Jun/1907,0104,01 || 15,0107,01 AO	\N	f	6
-146	-146	10,01Episode 15,01[1114615,01] The Metropolitan Police Detective Love Story (Part 1)04,01 || 15,01Subbed by [10Neg15,01] 07@ 15,01175mb CRC: [0AF8635A]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=PKESM38F04,01 || 07Aired 1999/May/3107,01 with an audience share of 21%04,01 || 07,01 Based on Vol 21 File 8-10	\N	f	2
-391	-391	10,01Episode 15,01[1139115,01] The Metropolitan Police Detective Love Story 6 (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01173mb CRC: [BF83A6D1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=EOFBO0RD04,01 || 07Aired 2005/Mar/0707,01 with an audience share of 11.5%04,01 || 07,01 Based on Vol 44 File 4-6	\N	f	4
-299	-299	10,01Episode 15,01[1129915,01] Kanmon Strait of Friendship & Murderous Intent (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172 MB Codec: [H264] CRC: [1E965555]04,01 || 07MU Link: 15,01http://www.fileserve.com/file/bsazt2g04,01 || 07Aired 2002/Nov/0407,01 with an audience share of 16.3%04,01 || 15,0107,01 AO	\N	f	18
5367	1	Love is 0	khaled	t	62
-78	-78	10,01Episode 15,01[117815,01] A Distinguished Family's Consecutive Accidental Death Case (Part 2)04,01 || 15,01Subbed by [10AConan15,01] 07@ 15,01172mb CRC: [6D09EB18]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=EI0NSTDE04,01 || 07Aired 1997/Oct/2707,01 with an audience share of 19.8%04,01 || 07,01 Based on Vol 15 File 10 - Vol 16 File 3	\N	f	1
-593	-593	10,01Episode 15,01[1159315,01] The Detective Memoir of Monkey and Rake (Part 2)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 319mb Codec: [XviD | H264] CRC: [EC3CFEE0 | 37480AB1]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=SDK6G7LX | 720: http://www.megaupload.com/?d=D2JEVJEP04,01 || 07Aired 2010/Oct/3007,0104,01 || 07,01 Based on Vol 68 File 9-11	\N	f	2
-300	-300	10,01Episode 15,01[1130015,01] Kanmon Strait of Friendship & Murderous Intent (Part 2)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172 MB Codec: [H264] CRC: [0283716B]04,01 || 07MU Link: 15,01http://www.fileserve.com/file/v6u6zqw04,01 || 07Aired 2002/Nov/1807,01 with an audience share of 15.3%04,01 || 15,0107,01 AO	\N	f	9
-395	-395	10,01Episode 15,01[1139515,01] Big Adventure at the Unconventional Mansion (Karakuri Part)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [673C955E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=RMFZ071J04,01 || 07Aired 2005/Apr/2507,01 with an audience share of 11.4%04,01 || 07,01 Based on Vol 46 File 7-1004,01 || 07,01http://www.nyaa.eu/?page=download&tid=20209915,0104,01 || \n07,01Bot Listing: 15,01/MSG [xdcc]Ecchilicious XDCC SEND 376	\N	f	5
-616	-616	10,01Episode 15,01[1161615,01] Holmes's Revelation (Holmes's Apprentice)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/320 MB Codec: [XviD/H264] CRC: [3729831C/C1902CFC]04,01 || 07MU Link: 15,01[SD] http://www.megaupload.com/?d=TL5C2LJ0 || [HD] http://www.megaupload.com/?d=RNRQEA22	\N	f	13
-617	-617	10,01Episode 15,01[1161715,01] Holmes's Revelation (Love Is 0)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172 MB/350 MB Codec: [XviD/H264] CRC: [C7E7101A/2AEFE8E8]04,01 || 07MU Link: 15,01[SD] http://www.megaupload.com/?d=ZUGCHZHU || [HD] http://www.megaupload.com/?d=7XDQ8XMG04,01 || 07Aired 05/29/1107,01	\N	f	10
-618	-618	10,01Episode 15,01[1161815,01] Holmes's Revelation (Satan)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01171mb | 320mb Codec: [XviD | H264] CRC: [AC8F5A74 | F9F50F17]04,01 || 07MU Link: 15,01http://www.fileserve.com/file/eHskkAV | 720: http://www.fileserve.com/file/66GMpWZ04,01 || 07Aired 6/4/1107,01	\N	f	11
879	666	install.res.1033.dll	simcop2387	f	1
-388	-388	10,01Episode 15,01[1138815,01] Kogoro Gets Drunk in Satsuma (Part 1)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [h264] CRC: [2C7D528C]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=V27W6SDP04,01 || 07Aired 2005/Feb/1407,01 with an audience share of 12.3%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 17	\N	f	6
-537	-537	10,01Episode 15,01[1153715,01] Kaitou Kid vs. The Strongest Safe (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 483mb | 232mb Codec: [XviD] CRC: [2A3D3E5F | 8E9B8D83 | C3B16071]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=A0TWZUWQ | 720: http://www.megaupload.com/?d=UXRGI2SG04,01 || 07Aired 2009/Jun/1307,01 with an audience share of 7.1%04,01 || 07,01 Based on Vol 64 File 11 - Vol 65 File 2	\N	f	1
-327	-327	10,01Episode 15,01[1132715,01] The Red Horse within the Flames (Part 3: The Solution)04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb CRC: [6368787E]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=VAD4JX9O04,01 || 07Aired 2003/Jul/0707,01 with an audience share of 16%04,01 || 07,01 Based on Vol 39 File 1-515,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 16	\N	f	2
-397	-397	10,01Episode 15,01[1139715,01] Spicy, Bitter & Sweet Soup04,01 || 15,01Subbed by [10Kienai15,01] 07@ 15,01172mb Codec: [H264] CRC: [459CFEF5]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=S7KUR12M04,01 || 07Aired 2005/May/0907,01 with an audience share of 12.9%04,01 || 15,0107,01 AO15,0104,01 || \n07,01Bot Listing: 15,01Join #Kienai and type /msg Sonoko-Chan XDCC send 21	\N	f	5
-600	-600	10,01Episode 15,01[1160015,01] The Dream the Kappa Saw (Part 1)04,01 || 15,01Subbed by [10DCTP15,01] 07@ 15,01172mb | 320mb Codec: [XviD | H264] CRC: [27610801 | 46A0C679]04,01 || 07MU Link: 15,01http://www.megaupload.com/?d=EVF3NGIV | 720: http://www.megaupload.com/?d=IPZ4CGJO04,01 || 07Aired 2010/Dec/1807,0104,01 || 07,01 Based on Vol 69 File 1-3	\N	f	11
5368	9	That's so profounding	simcop2387	f	34
\.


--
-- Data for Name: sources; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY sources (sid, name, "desc") FROM stdin;
1	RCF	Random Conan Fact
-1	1	/notice %NICK%
2	RCC	Random Conan Crime
3	RJDF	Random Jd Fact
4	MOVIE12QUOTES	Movie 12 Quotes
5	SOURCE	\N
40	LIST	/notice %NICK%
-2	2	/notice %NICK%
-3	3	/notice %NICK%
-4	4	/notice %NICK%
6	ADVERT	\N
7	RDCTPF	Random DCTP Fact
-5	5	/notice %NICK%
9	BADTRANS	Bad Translation
-6	6	/notice %NICK%
-7	7	/notice %NICK%
10	MOVIE13SPOILERS	Movie 13 Spoiler
-8	8	/notice %NICK%
11	FUNNYSHIT	\N
12	RKKF	Random Kaito Kid Fact
-9	9	/notice %NICK%
13	RDDLF	Random DDL Fact
14	RCFICT	Random Conan Fiction
666	DLL	Random DLL File
667	SHRIMP	Random Shrimp Dish
-10	10	/notice %NICK%
-11	11	/notice %NICK%
-12	12	/notice %NICK%
15	SSL	\N
-13	13	/notice %NICK%
-14	14	/notice %NICK%
-15	15	/notice %NICK%
16	RRCFF	\N
-16	16	/notice %NICK%
18	OUT	\N
19	MOVIE5QUOTES	\N
20	NCH	Next Conan's Hint
17	DDL#DCTP	Direct Download Link
-17	17	/notice %NICK%
-18	18	/notice %NICK%
21	DDL#DCTV	\N
22	DCTVRULES	DCTV Rules
23	RUNOF	\N
24	MOVIE8QUOTES	Movie 8 Quotes
25	11THHOUR	\N
26	DDL#KIENAI	\N
27	MRT	Mr. T says
28	NORRIS	Chuck Norris Fact
29	RDFT	Random DCTP Forum Topic
-19	19	/notice %NICK%
31	RJDFUN	Random Jd- Fun
32	RFC	Request for Comments
33	DLD	DLD
34	MUL	Megaupload Links
35	MU	\N
36	RCA	Random Conan Arc
37	RCT	Random Conan Title
38	RCI	Random Conan Image
39	PDF	DC Subbed Episodes List
-20	20	/notice %NICK%
-21	21	/notice %NICK%
-22	22	/notice %NICK%
-23	23	/notice %NICK%
-24	24	/notice %NICK%
-25	25	/notice %NICK%
-26	26	/notice %NICK%
-27	27	/notice %NICK%
-28	28	/notice %NICK%
-29	29	/notice %NICK%
-30	30	/notice %NICK%
-31	31	/notice %NICK%
-32	32	/notice %NICK%
-33	33	/notice %NICK%
-34	34	/notice %NICK%
-35	35	/notice %NICK%
-36	36	/notice %NICK%
-37	37	/notice %NICK%
-38	38	/notice %NICK%
-39	39	/notice %NICK%
-40	40	/notice %NICK%
-41	41	/notice %NICK%
-42	42	/notice %NICK%
-43	43	/notice %NICK%
-44	44	/notice %NICK%
-45	45	/notice %NICK%
-46	46	/notice %NICK%
-47	47	/notice %NICK%
-48	48	/notice %NICK%
-49	49	/notice %NICK%
-50	50	/notice %NICK%
-51	51	/notice %NICK%
-52	52	/notice %NICK%
-53	53	/notice %NICK%
-54	54	/notice %NICK%
-55	55	/notice %NICK%
-56	56	/notice %NICK%
-57	57	/notice %NICK%
-58	58	/notice %NICK%
-59	59	/notice %NICK%
-60	60	/notice %NICK%
-61	61	/notice %NICK%
-62	62	/notice %NICK%
-63	63	/notice %NICK%
-64	64	/notice %NICK%
-65	65	/notice %NICK%
-66	66	/notice %NICK%
-67	67	/notice %NICK%
-68	68	/notice %NICK%
-69	69	/notice %NICK%
-70	70	/notice %NICK%
-71	71	/notice %NICK%
-72	72	/notice %NICK%
-73	73	/notice %NICK%
-74	74	/notice %NICK%
-75	75	/notice %NICK%
-76	76	/notice %NICK%
-77	77	/notice %NICK%
-78	78	/notice %NICK%
-79	79	/notice %NICK%
-80	80	/notice %NICK%
-81	81	/notice %NICK%
-82	82	/notice %NICK%
-83	83	/notice %NICK%
-84	84	/notice %NICK%
-85	85	/notice %NICK%
-86	86	/notice %NICK%
-87	87	/notice %NICK%
-88	88	/notice %NICK%
-89	89	/notice %NICK%
-90	90	/notice %NICK%
-91	91	/notice %NICK%
-92	92	/notice %NICK%
-93	93	/notice %NICK%
-94	94	/notice %NICK%
-95	95	/notice %NICK%
-96	96	/notice %NICK%
-97	97	/notice %NICK%
-98	98	/notice %NICK%
-99	99	/notice %NICK%
-100	100	/notice %NICK%
-101	101	/notice %NICK%
-102	102	/notice %NICK%
-103	103	/notice %NICK%
-104	104	/notice %NICK%
-105	105	/notice %NICK%
-106	106	/notice %NICK%
-107	107	/notice %NICK%
-108	108	/notice %NICK%
-109	109	/notice %NICK%
-110	110	/notice %NICK%
-111	111	/notice %NICK%
-112	112	/notice %NICK%
-113	113	/notice %NICK%
-114	114	/notice %NICK%
-115	115	/notice %NICK%
-116	116	/notice %NICK%
-117	117	/notice %NICK%
-118	118	/notice %NICK%
-119	119	/notice %NICK%
-120	120	/notice %NICK%
-121	121	/notice %NICK%
-122	122	/notice %NICK%
-123	123	/notice %NICK%
-124	124	/notice %NICK%
-125	125	/notice %NICK%
-126	126	/notice %NICK%
-127	127	/notice %NICK%
-128	128	/notice %NICK%
-129	129	/notice %NICK%
-130	130	/notice %NICK%
-131	131	/notice %NICK%
-132	132	/notice %NICK%
-133	133	/notice %NICK%
-134	134	/notice %NICK%
-135	135	/notice %NICK%
-136	136	/notice %NICK%
-137	137	/notice %NICK%
-138	138	/notice %NICK%
-139	139	/notice %NICK%
-140	140	/notice %NICK%
-141	141	/notice %NICK%
-142	142	/notice %NICK%
-143	143	/notice %NICK%
-144	144	/notice %NICK%
-145	145	/notice %NICK%
-146	146	/notice %NICK%
-147	147	/notice %NICK%
-148	148	/notice %NICK%
-149	149	/notice %NICK%
-150	150	/notice %NICK%
-151	151	/notice %NICK%
-152	152	/notice %NICK%
-153	153	/notice %NICK%
-154	154	/notice %NICK%
-155	155	/notice %NICK%
-156	156	/notice %NICK%
-157	157	/notice %NICK%
-158	158	/notice %NICK%
-159	159	/notice %NICK%
-666	666	/notice %NICK%
-160	160	/notice %NICK%
-161	161	/notice %NICK%
-162	162	/notice %NICK%
-163	163	/notice %NICK%
-164	164	/notice %NICK%
-165	165	/notice %NICK%
-1000	1000	/notice %NICK%
-166	166	/notice %NICK%
-167	167	/notice %NICK%
-168	168	/notice %NICK%
-169	169	/notice %NICK%
-9000	9000	/notice %NICK%
-170	170	/notice %NICK%
-171	171	/notice %NICK%
-172	172	/notice %NICK%
-173	173	/notice %NICK%
-174	174	/notice %NICK%
-175	175	/notice %NICK%
-176	176	/notice %NICK%
-177	177	/notice %NICK%
-178	178	/notice %NICK%
-179	179	/notice %NICK%
-180	180	/notice %NICK%
-181	181	/notice %NICK%
-182	182	/notice %NICK%
-183	183	/notice %NICK%
-184	184	/notice %NICK%
-185	185	/notice %NICK%
-186	186	/notice %NICK%
-187	187	/notice %NICK%
-188	188	/notice %NICK%
-189	189	/notice %NICK%
-190	190	/notice %NICK%
-191	191	/notice %NICK%
-192	192	/notice %NICK%
-193	193	/notice %NICK%
-194	194	/notice %NICK%
-195	195	/notice %NICK%
-196	196	/notice %NICK%
-197	197	/notice %NICK%
-198	198	/notice %NICK%
-199	199	/notice %NICK%
-200	200	/notice %NICK%
-201	201	/notice %NICK%
-202	202	/notice %NICK%
-203	203	/notice %NICK%
-204	204	/notice %NICK%
-205	205	/notice %NICK%
-206	206	/notice %NICK%
-207	207	/notice %NICK%
-208	208	/notice %NICK%
-209	209	/notice %NICK%
-210	210	/notice %NICK%
-211	211	/notice %NICK%
-212	212	/notice %NICK%
-213	213	/notice %NICK%
-214	214	/notice %NICK%
-215	215	/notice %NICK%
-216	216	/notice %NICK%
-217	217	/notice %NICK%
-218	218	/notice %NICK%
-219	219	/notice %NICK%
-220	220	/notice %NICK%
-221	221	/notice %NICK%
-222	222	/notice %NICK%
-223	223	/notice %NICK%
-224	224	/notice %NICK%
-225	225	/notice %NICK%
-226	226	/notice %NICK%
-227	227	/notice %NICK%
-228	228	/notice %NICK%
-229	229	/notice %NICK%
-230	230	/notice %NICK%
-231	231	/notice %NICK%
-232	232	/notice %NICK%
-233	233	/notice %NICK%
-234	234	/notice %NICK%
-235	235	/notice %NICK%
-236	236	/notice %NICK%
-237	237	/notice %NICK%
-238	238	/notice %NICK%
-239	239	/notice %NICK%
-240	240	/notice %NICK%
-241	241	/notice %NICK%
-242	242	/notice %NICK%
-243	243	/notice %NICK%
-244	244	/notice %NICK%
-245	245	/notice %NICK%
-246	246	/notice %NICK%
-247	247	/notice %NICK%
-248	248	/notice %NICK%
-249	249	/notice %NICK%
-250	250	/notice %NICK%
-251	251	/notice %NICK%
-252	252	/notice %NICK%
-253	253	/notice %NICK%
-254	254	/notice %NICK%
-255	255	/notice %NICK%
-256	256	/notice %NICK%
-257	257	/notice %NICK%
-258	258	/notice %NICK%
-259	259	/notice %NICK%
-260	260	/notice %NICK%
-261	261	/notice %NICK%
-262	262	/notice %NICK%
-263	263	/notice %NICK%
-264	264	/notice %NICK%
-265	265	/notice %NICK%
-266	266	/notice %NICK%
-267	267	/notice %NICK%
-268	268	/notice %NICK%
-269	269	/notice %NICK%
-270	270	/notice %NICK%
-271	271	/notice %NICK%
-272	272	/notice %NICK%
-273	273	/notice %NICK%
-274	274	/notice %NICK%
-275	275	/notice %NICK%
-276	276	/notice %NICK%
-277	277	/notice %NICK%
-278	278	/notice %NICK%
-279	279	/notice %NICK%
-280	280	/notice %NICK%
-281	281	/notice %NICK%
-282	282	/notice %NICK%
-283	283	/notice %NICK%
-284	284	/notice %NICK%
-285	285	/notice %NICK%
-286	286	/notice %NICK%
-287	287	/notice %NICK%
-288	288	/notice %NICK%
-289	289	/notice %NICK%
-290	290	/notice %NICK%
-291	291	/notice %NICK%
-292	292	/notice %NICK%
-293	293	/notice %NICK%
-294	294	/notice %NICK%
-295	295	/notice %NICK%
-296	296	/notice %NICK%
-297	297	/notice %NICK%
-298	298	/notice %NICK%
-299	299	/notice %NICK%
-300	300	/notice %NICK%
-301	301	/notice %NICK%
-302	302	/notice %NICK%
-303	303	/notice %NICK%
-304	304	/notice %NICK%
-305	305	/notice %NICK%
-306	306	/notice %NICK%
-307	307	/notice %NICK%
-308	308	/notice %NICK%
-309	309	/notice %NICK%
-310	310	/notice %NICK%
-311	311	/notice %NICK%
-312	312	/notice %NICK%
-313	313	/notice %NICK%
-314	314	/notice %NICK%
-315	315	/notice %NICK%
-316	316	/notice %NICK%
-317	317	/notice %NICK%
-318	318	/notice %NICK%
-319	319	/notice %NICK%
-320	320	/notice %NICK%
-321	321	/notice %NICK%
-322	322	/notice %NICK%
-323	323	/notice %NICK%
-324	324	/notice %NICK%
-325	325	/notice %NICK%
-326	326	/notice %NICK%
-327	327	/notice %NICK%
-328	328	/notice %NICK%
-329	329	/notice %NICK%
-330	330	/notice %NICK%
-331	331	/notice %NICK%
-332	332	/notice %NICK%
-333	333	/notice %NICK%
-334	334	/notice %NICK%
-335	335	/notice %NICK%
-336	336	/notice %NICK%
-337	337	/notice %NICK%
-338	338	/notice %NICK%
-339	339	/notice %NICK%
-340	340	/notice %NICK%
-341	341	/notice %NICK%
-342	342	/notice %NICK%
-343	343	/notice %NICK%
-344	344	/notice %NICK%
-345	345	/notice %NICK%
-346	346	/notice %NICK%
-347	347	/notice %NICK%
-348	348	/notice %NICK%
-349	349	/notice %NICK%
-350	350	/notice %NICK%
-351	351	/notice %NICK%
-352	352	/notice %NICK%
-353	353	/notice %NICK%
-354	354	/notice %NICK%
-355	355	/notice %NICK%
-356	356	/notice %NICK%
-357	357	/notice %NICK%
-358	358	/notice %NICK%
-359	359	/notice %NICK%
-360	360	/notice %NICK%
-361	361	/notice %NICK%
-362	362	/notice %NICK%
-363	363	/notice %NICK%
-364	364	/notice %NICK%
-365	365	/notice %NICK%
-366	366	/notice %NICK%
-367	367	/notice %NICK%
-368	368	/notice %NICK%
-369	369	/notice %NICK%
-370	370	/notice %NICK%
-371	371	/notice %NICK%
-372	372	/notice %NICK%
-373	373	/notice %NICK%
-374	374	/notice %NICK%
-375	375	/notice %NICK%
-376	376	/notice %NICK%
-377	377	/notice %NICK%
-378	378	/notice %NICK%
-379	379	/notice %NICK%
-380	380	/notice %NICK%
-381	381	/notice %NICK%
-382	382	/notice %NICK%
-383	383	/notice %NICK%
-384	384	/notice %NICK%
-385	385	/notice %NICK%
-386	386	/notice %NICK%
-387	387	/notice %NICK%
-388	388	/notice %NICK%
-389	389	/notice %NICK%
-390	390	/notice %NICK%
-391	391	/notice %NICK%
-392	392	/notice %NICK%
-393	393	/notice %NICK%
-394	394	/notice %NICK%
-395	395	/notice %NICK%
-396	396	/notice %NICK%
-397	397	/notice %NICK%
-398	398	/notice %NICK%
-399	399	/notice %NICK%
-400	400	/notice %NICK%
-401	401	/notice %NICK%
-402	402	/notice %NICK%
-403	403	/notice %NICK%
-404	404	/notice %NICK%
-405	405	/notice %NICK%
-406	406	/notice %NICK%
-407	407	/notice %NICK%
-408	408	/notice %NICK%
-409	409	/notice %NICK%
-410	410	/notice %NICK%
-411	411	/notice %NICK%
-412	412	/notice %NICK%
-413	413	/notice %NICK%
-414	414	/notice %NICK%
-415	415	/notice %NICK%
-416	416	/notice %NICK%
-417	417	/notice %NICK%
-418	418	/notice %NICK%
-419	419	/notice %NICK%
-420	420	/notice %NICK%
-421	421	/notice %NICK%
-422	422	/notice %NICK%
-423	423	/notice %NICK%
-424	424	/notice %NICK%
-425	425	/notice %NICK%
-426	426	/notice %NICK%
-20026	OVA26	/notice %NICK%
-427	427	/notice %NICK%
-428	428	/notice %NICK%
-429	429	/notice %NICK%
-430	430	/notice %NICK%
-431	431	/notice %NICK%
-432	432	/notice %NICK%
-433	433	/notice %NICK%
-434	434	/notice %NICK%
-435	435	/notice %NICK%
-436	436	/notice %NICK%
-437	437	/notice %NICK%
-438	438	/notice %NICK%
-439	439	/notice %NICK%
-440	440	/notice %NICK%
-441	441	/notice %NICK%
-442	442	/notice %NICK%
-443	443	/notice %NICK%
-444	444	/notice %NICK%
-445	445	/notice %NICK%
-446	446	/notice %NICK%
-447	447	/notice %NICK%
-448	448	/notice %NICK%
-449	449	/notice %NICK%
-450	450	/notice %NICK%
-451	451	/notice %NICK%
-452	452	/notice %NICK%
-453	453	/notice %NICK%
-454	454	/notice %NICK%
-455	455	/notice %NICK%
-456	456	/notice %NICK%
-457	457	/notice %NICK%
-458	458	/notice %NICK%
-459	459	/notice %NICK%
-460	460	/notice %NICK%
-461	461	/notice %NICK%
-462	462	/notice %NICK%
-463	463	/notice %NICK%
-464	464	/notice %NICK%
-465	465	/notice %NICK%
-466	466	/notice %NICK%
-467	467	/notice %NICK%
-468	468	/notice %NICK%
-469	469	/notice %NICK%
-470	470	/notice %NICK%
-471	471	/notice %NICK%
-472	472	/notice %NICK%
-473	473	/notice %NICK%
-474	474	/notice %NICK%
-475	475	/notice %NICK%
-476	476	/notice %NICK%
-477	477	/notice %NICK%
-478	478	/notice %NICK%
-479	479	/notice %NICK%
-480	480	/notice %NICK%
-481	481	/notice %NICK%
-482	482	/notice %NICK%
-483	483	/notice %NICK%
-484	484	/notice %NICK%
-485	485	/notice %NICK%
-486	486	/notice %NICK%
-487	487	/notice %NICK%
-488	488	/notice %NICK%
-489	489	/notice %NICK%
-490	490	/notice %NICK%
-491	491	/notice %NICK%
-492	492	/notice %NICK%
-493	493	/notice %NICK%
-494	494	/notice %NICK%
-495	495	/notice %NICK%
-496	496	/notice %NICK%
-497	497	/notice %NICK%
-498	498	/notice %NICK%
-499	499	/notice %NICK%
-500	500	/notice %NICK%
-501	501	/notice %NICK%
-502	502	/notice %NICK%
-503	503	/notice %NICK%
-504	504	/notice %NICK%
-505	505	/notice %NICK%
-506	506	/notice %NICK%
-507	507	/notice %NICK%
-508	508	/notice %NICK%
-509	509	/notice %NICK%
-510	510	/notice %NICK%
-511	511	/notice %NICK%
-512	512	/notice %NICK%
-513	513	/notice %NICK%
-514	514	/notice %NICK%
-515	515	/notice %NICK%
-516	516	/notice %NICK%
-517	517	/notice %NICK%
-518	518	/notice %NICK%
-519	519	/notice %NICK%
-520	520	/notice %NICK%
-521	521	/notice %NICK%
-522	522	/notice %NICK%
-523	523	/notice %NICK%
-524	524	/notice %NICK%
-525	525	/notice %NICK%
-526	526	/notice %NICK%
-527	527	/notice %NICK%
-528	528	/notice %NICK%
-529	529	/notice %NICK%
-530	530	/notice %NICK%
-531	531	/notice %NICK%
-532	532	/notice %NICK%
-533	533	/notice %NICK%
-534	534	/notice %NICK%
-535	535	/notice %NICK%
-536	536	/notice %NICK%
-20027	OVA27	/notice %NICK%
42	QUOTES	See all my quotes at
-20028	OVA28	/notice %NICK%
43	BOTSNACK	\N
44	KEPKEP	\N
-20029	OVA29	/notice %NICK%
-20030	OVA30	/notice %NICK%
46	SIMON	Simon says
47	XDCCHELP	\N
-20031	OVA31	/notice %NICK%
49	RANDOMTRIGGER	\N
50	INSULTS	\N
51	NOTEPISODE	/notice %NICK%
-20032	OVA32	/notice %NICK%
-537	537	/notice %NICK%
52	B'Z	Random B'z Song
53	!B'Z	Random B'z Song
-538	538	/notice %NICK%
54	MAYS	Billy Mays Fact
-539	539	/notice %NICK%
-540	540	/notice %NICK%
55	SHIRANPURI	\N
56	SHIRANPURIFACTS	Random shiranpuri Fact
57	CUP	\N
-541	541	/notice %NICK%
-542	542	/notice %NICK%
58	DDL#YAMI	\N
59	CHAINSAWS	Random Chainsaw
-543	543	/notice %NICK%
60	DR.HOUSE	Greg is cool and his sarcasm, too
-544	544	/notice %NICK%
-20033	OVA33	/notice %NICK%
-20034	OVA34	/notice %NICK%
-20035	OVA35	/notice %NICK%
-20036	OVA36	/notice %NICK%
-20037	OVA37	/notice %NICK%
-20038	OVA38	/notice %NICK%
-20039	OVA39	/notice %NICK%
-20040	OVA40	/notice %NICK%
-20041	OVA41	/notice %NICK%
-20042	OVA42	/notice %NICK%
-20043	OVA43	/notice %NICK%
-545	545	/notice %NICK%
-546	546	/notice %NICK%
61	PINGS	\N
-547	547	/notice %NICK%
-548	548	/notice %NICK%
-549	549	/notice %NICK%
62	RMP	Random Motivational Poster:
-550	550	/notice %NICK%
63	M13RELEASE	/notice %NICK%
-10025	MOVIE25	/notice %NICK%
-6013	M13RELEASE2	/notice %NICK%
64	M14SPOILERS	Movie 14 Spoilers
-551	551	/notice %NICK%
-552	552	/notice %NICK%
-553	553	/notice %NICK%
-554	554	/notice %NICK%
-555	555	/notice %NICK%
-556	556	/notice %NICK%
-557	557	/notice %NICK%
-558	558	/notice %NICK%
-559	559	/notice %NICK%
-560	560	/notice %NICK%
-561	561	/notice %NICK%
-20001	ova1	/notice %NICK%
-20002	OVA2	/notice %NICK%
-20003	OVA3	/notice %NICK%
-20004	OVA4	/notice %NICK%
-20005	OVA5	/notice %NICK%
-20006	OVA6	/notice %NICK%
-20007	OVA7	/notice %NICK%
-20008	OVA8	/notice %NICK%
-20009	OVA9	/notice %NICK%
-30002	SPECIAL2	/notice %NICK%
-30003	SPECIAL3	/notice %NICK%
-30001	SPECIAL1	/notice %NICK%
-30010	SPECIAL10	/notice %NICK%
-30020	SPECIAL20	/notice %NICK%
-30021	SPECIAL21	/notice %NICK%
-10001	MOVIE1	/notice %NICK%
-10002	MOVIE2	/notice %NICK%
-10003	MOVIE3	/notice %NICK%
-10004	MOVIE4	/notice %NICK%
-10005	MOVIE5	/notice %NICK%
-10006	MOVIE6	/notice %NICK%
-10007	MOVIE7	/notice %NICK%
-10008	MOVIE8	/notice %NICK%
-10009	MOVIE9	/notice %NICK%
-10010	MOVIE10	/notice %NICK%
-10011	MOVIE11	/notice %NICK%
-10012	MOVIE12	/notice %NICK%
-10013	MOVIE13	/notice %NICK%
-10014	MOVIE14	/notice %NICK%
-562	562	/notice %NICK%
-563	563	/notice %NICK%
-564	564	/notice %NICK%
-565	565	/notice %NICK%
-566	566	/notice %NICK%
-567	567	/notice %NICK%
-568	568	/notice %NICK%
-569	569	/notice %NICK%
-570	570	/notice %NICK%
-10000	MOVIE0	/notice %NICK%
-10015	MOVIE15	/notice %NICK%
-20000	OVA0	/notice %NICK%
-20010	OVA10	/notice %NICK%
-30000	SPECIAL0	/notice %NICK%
-30004	SPECIAL4	/notice %NICK%
-30005	SPECIAL5	/notice %NICK%
-30006	SPECIAL6	/notice %NICK%
-30007	SPECIAL7	/notice %NICK%
-30008	SPECIAL8	/notice %NICK%
-30009	SPECIAL9	/notice %NICK%
-30011	SPECIAL11	/notice %NICK%
-30012	SPECIAL12	/notice %NICK%
-30013	SPECIAL13	/notice %NICK%
-30014	SPECIAL14	/notice %NICK%
-30015	SPECIAL15	/notice %NICK%
-30016	SPECIAL16	/notice %NICK%
-30017	SPECIAL17	/notice %NICK%
-30018	SPECIAL18	/notice %NICK%
-30019	SPECIAL19	/notice %NICK%
-30022	SPECIAL22	/notice %NICK%
-30023	SPECIAL23	/notice %NICK%
-30024	SPECIAL24	/notice %NICK%
-30025	SPECIAL25	/notice %NICK%
-30026	SPECIAL26	/notice %NICK%
-30027	SPECIAL27	/notice %NICK%
-30028	SPECIAL28	/notice %NICK%
-30029	SPECIAL29	/notice %NICK%
-30030	SPECIAL30	/notice %NICK%
-10016	MOVIE16	/notice %NICK%
-10017	MOVIE17	/notice %NICK%
-10018	MOVIE18	/notice %NICK%
-10019	MOVIE19	/notice %NICK%
-10020	MOVIE20	/notice %NICK%
-10021	MOVIE21	/notice %NICK%
-10022	MOVIE22	/notice %NICK%
-10023	MOVIE23	/notice %NICK%
-10024	MOVIE24	/notice %NICK%
-10026	MOVIE26	/notice %NICK%
-10027	MOVIE27	/notice %NICK%
-10028	MOVIE28	/notice %NICK%
-10029	MOVIE29	/notice %NICK%
-10030	MOVIE30	/notice %NICK%
-10031	MOVIE31	/notice %NICK%
-10032	MOVIE32	/notice %NICK%
-10033	MOVIE33	/notice %NICK%
-10034	MOVIE34	/notice %NICK%
-10035	MOVIE35	/notice %NICK%
-10036	MOVIE36	/notice %NICK%
-10037	MOVIE37	/notice %NICK%
-10038	MOVIE38	/notice %NICK%
-10039	MOVIE39	/notice %NICK%
-10040	MOVIE40	/notice %NICK%
-10041	MOVIE41	/notice %NICK%
-10042	MOVIE42	/notice %NICK%
-10043	MOVIE43	/notice %NICK%
-10044	MOVIE44	/notice %NICK%
-10045	MOVIE45	/notice %NICK%
-10046	MOVIE46	/notice %NICK%
-10047	MOVIE47	/notice %NICK%
-10048	MOVIE48	/notice %NICK%
-10049	MOVIE49	/notice %NICK%
-10050	MOVIE50	/notice %NICK%
-20011	OVA11	/notice %NICK%
-20012	OVA12	/notice %NICK%
-20013	OVA13	/notice %NICK%
-20014	OVA14	/notice %NICK%
-20015	OVA15	/notice %NICK%
-20016	OVA16	/notice %NICK%
-20017	OVA17	/notice %NICK%
-20018	OVA18	/notice %NICK%
-20019	OVA19	/notice %NICK%
-20020	OVA20	/notice %NICK%
-20021	OVA21	/notice %NICK%
-20022	OVA22	/notice %NICK%
-20023	OVA23	/notice %NICK%
-20024	OVA24	/notice %NICK%
-20025	OVA25	/notice %NICK%
93	BANNED	/notice #dctv
94	TETRIS	stereogram tetris
-597	597	/notice %NICK%
-598	598	/notice %NICK%
95	BOTS	\N
97	BOTLIST	\N
98	SIMCOP	Random Simcop Facts
-599	599	/notice %NICK%
-601	601	/notice %NICK%
99	BING	\N
-602	602	/notice %NICK%
92	MOVIE15SPOILERS	Movie 15 Spoilers
-603	603	/notice %NICK%
-604	604	/notice %NICK%
-605	605	/notice %NICK%
-20044	OVA44	/notice %NICK%
-20045	OVA45	/notice %NICK%
-20046	OVA46	/notice %NICK%
-20047	OVA47	/notice %NICK%
-20048	OVA48	/notice %NICK%
-20049	OVA49	/notice %NICK%
-20050	OVA50	/notice %NICK%
-30031	SPECIAL31	/notice %NICK%
-30032	SPECIAL32	/notice %NICK%
-30033	SPECIAL33	/notice %NICK%
-30034	SPECIAL34	/notice %NICK%
-30035	SPECIAL35	/notice %NICK%
-30036	SPECIAL36	/notice %NICK%
-30037	SPECIAL37	/notice %NICK%
-30038	SPECIAL38	/notice %NICK%
-30039	SPECIAL39	/notice %NICK%
-30040	SPECIAL40	/notice %NICK%
-30041	SPECIAL41	/notice %NICK%
-30042	SPECIAL42	/notice %NICK%
-30043	SPECIAL43	/notice %NICK%
-30044	SPECIAL44	/notice %NICK%
-30045	SPECIAL45	/notice %NICK%
-30046	SPECIAL46	/notice %NICK%
-30047	SPECIAL47	/notice %NICK%
-30048	SPECIAL48	/notice %NICK%
-30049	SPECIAL49	/notice %NICK%
-30050	SPECIAL50	/notice %NICK%
65	BOOT	\N
66	SHOE	/me 
67	FISH	Random Fish
68	RANDOMNEWS	Random News
-607	607	/notice %NICK%
-600	600	/notice %NICK%
69	JAPUNO	\N
70	RZDMF	\N
-606	606	/notice %NICK%
72	RZDMFAIL	Random Zdm Fail
73	RZDMFICT	Random Zdm Fiction
74	FACES	\N
75	DINOS	\N
76	NONSENSE	\N
-609	609	/notice %NICK%
78	ABSURD	Random Nonsense
79	RZDMFACT	Random Zdm Fact
80	SHIRANSEZ	\N
-610	610	/notice %NICK%
-571	571	/notice %NICK%
-572	572	/notice %NICK%
-573	573	/notice %NICK%
-574	574	/notice %NICK%
-575	575	/notice %NICK%
-576	576	/notice %NICK%
-577	577	/notice %NICK%
-578	578	/notice %NICK%
-579	579	/notice %NICK%
-580	580	/notice %NICK%
-581	581	/notice %NICK%
-582	582	/notice %NICK%
82	VUVUZELA	\N
-611	611	/notice %NICK%
84	STFU	\N
85	MF	\N
-583	583	/notice %NICK%
-584	584	/notice %NICK%
-585	585	/notice %NICK%
86	ENGRISH	Random Engrish
87	RANDOMHELP	Random Help
88	BOOBGRAB	\N
89	SCOOBY	\N
-586	586	/notice %NICK%
90	EXCLAMATION	\N
-587	587	/notice %NICK%
91	LIST#CONAN-FANS	/notice %NICK%
-608	608	/notice %NICK%
-612	612	/notice %NICK%
-588	588	/notice %NICK%
-589	589	/notice %NICK%
-590	590	/notice %NICK%
-591	591	/notice %NICK%
-592	592	/notice %NICK%
-593	593	/notice %NICK%
-594	594	/notice %NICK%
-595	595	/notice %NICK%
-596	596	/notice %NICK%
-616	616	/notice %NICK%
-617	617	/notice %NICK%
-618	618	/notice %NICK%
\.


--
-- Data for Name: tidsources; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY tidsources (sid, tid) FROM stdin;
-1	-1
-2	-2
-3	-3
-4	-4
-5	-5
-6	-6
-7	-7
-8	-8
-9	-9
-10	-10
-11	-11
-12	-12
-13	-13
-14	-14
-15	-15
-16	-16
-17	-17
-18	-18
-19	-19
-20	-20
-21	-21
-22	-22
-23	-23
-24	-24
-25	-25
-26	-26
-27	-27
-28	-28
-29	-29
-30	-30
-31	-31
-32	-32
-33	-33
-34	-34
-35	-35
-36	-36
-37	-37
-38	-38
-39	-39
-40	-40
-41	-41
-42	-42
-43	-43
-44	-44
-45	-45
-46	-46
-47	-47
-48	-48
-49	-49
-50	-50
-51	-51
-52	-52
-53	-53
-54	-54
-55	-55
-56	-56
-57	-57
-58	-58
-59	-59
-60	-60
-61	-61
-62	-62
-63	-63
-64	-64
-65	-65
-66	-66
-67	-67
-68	-68
-69	-69
-70	-70
-71	-71
-72	-72
-73	-73
-74	-74
-75	-75
-76	-76
-77	-77
-78	-78
-79	-79
-80	-80
-81	-81
-82	-82
-83	-83
-84	-84
-85	-85
-86	-86
-87	-87
-88	-88
-89	-89
-90	-90
-91	-91
-92	-92
-93	-93
-94	-94
-95	-95
-96	-96
-97	-97
-98	-98
-99	-99
-100	-100
-101	-101
-102	-102
-103	-103
-104	-104
-105	-105
-106	-106
-107	-107
-108	-108
-109	-109
-110	-110
1	1
2	2
3	3
21	29
2	4
2	5
4	6
5	7
6	9
7	10
9	11
-584	-584
11	13
12	14
13	15
14	1
14	16
1	17
14	18
11	19
666	20
667	21
15	22
3	23
16	24
17	25
19	27
20	28
22	30
23	31
23	32
24	33
25	34
26	35
27	37
28	38
3	39
29	40
31	42
31	23
32	43
33	44
34	45
35	45
37	47
38	48
39	50
40	51
-111	-111
-112	-112
-113	-113
-114	-114
-115	-115
-116	-116
-117	-117
-118	-118
-119	-119
-120	-120
-121	-121
-122	-122
-123	-123
-124	-124
-125	-125
-126	-126
-127	-127
-128	-128
-129	-129
-130	-130
-131	-131
-132	-132
-133	-133
-134	-134
-135	-135
-136	-136
-137	-137
-138	-138
-139	-139
-140	-140
-141	-141
-142	-142
-143	-143
-144	-144
-145	-145
-146	-146
-147	-147
-148	-148
-149	-149
-150	-150
-151	-151
-152	-152
-153	-153
-154	-154
-155	-155
-156	-156
-157	-157
-158	-158
-159	-159
-160	-160
-161	-161
-162	-162
-163	-163
-164	-164
-165	-165
-166	-166
-167	-167
-168	-168
-169	-169
-170	-170
-171	-171
-172	-172
-173	-173
-174	-174
-175	-175
-176	-176
-177	-177
-178	-178
-179	-179
-180	-180
-181	-181
-182	-182
-183	-183
-184	-184
-185	-185
-186	-186
-187	-187
-188	-188
-189	-189
-190	-190
-191	-191
-192	-192
-193	-193
-194	-194
-195	-195
-196	-196
-197	-197
-198	-198
-199	-199
-200	-200
-201	-201
-202	-202
-203	-203
-204	-204
-205	-205
-206	-206
-207	-207
-208	-208
-209	-209
-210	-210
-211	-211
-212	-212
-213	-213
-214	-214
-215	-215
-216	-216
-217	-217
-218	-218
-219	-219
-220	-220
-221	-221
-222	-222
-223	-223
-224	-224
-225	-225
-226	-226
-227	-227
-228	-228
-229	-229
-230	-230
-231	-231
-232	-232
-233	-233
-234	-234
-235	-235
-236	-236
-237	-237
-238	-238
-239	-239
-240	-240
-241	-241
-242	-242
-243	-243
-244	-244
-245	-245
-246	-246
-247	-247
-248	-248
-249	-249
-250	-250
-251	-251
-252	-252
-253	-253
-254	-254
-255	-255
-256	-256
-257	-257
-258	-258
-259	-259
-260	-260
-261	-261
-262	-262
-263	-263
-264	-264
-265	-265
-266	-266
-267	-267
-268	-268
-269	-269
-270	-270
-271	-271
-272	-272
-273	-273
-274	-274
-275	-275
-276	-276
-277	-277
-278	-278
-279	-279
-280	-280
-281	-281
-282	-282
-283	-283
-284	-284
-285	-285
-286	-286
-287	-287
-288	-288
-289	-289
-290	-290
-291	-291
-292	-292
-293	-293
-294	-294
-295	-295
-296	-296
-297	-297
-298	-298
-299	-299
-300	-300
-301	-301
-302	-302
-303	-303
-304	-304
-305	-305
-306	-306
-307	-307
-308	-308
-309	-309
-310	-310
-311	-311
-312	-312
-313	-313
-314	-314
-315	-315
-316	-316
-317	-317
-318	-318
-319	-319
-320	-320
-321	-321
-322	-322
-323	-323
-324	-324
-325	-325
-326	-326
-327	-327
-328	-328
-329	-329
-330	-330
-331	-331
-332	-332
-333	-333
-334	-334
-335	-335
-336	-336
-337	-337
-338	-338
-339	-339
-340	-340
-341	-341
-342	-342
-343	-343
-344	-344
-345	-345
-346	-346
-347	-347
-348	-348
-349	-349
-350	-350
-351	-351
-352	-352
-353	-353
-354	-354
-355	-355
-356	-356
-357	-357
-358	-358
-359	-359
-360	-360
-361	-361
-362	-362
-363	-363
-364	-364
-365	-365
-366	-366
-367	-367
-368	-368
-369	-369
-370	-370
-371	-371
-372	-372
-373	-373
-374	-374
-375	-375
-376	-376
-377	-377
-378	-378
-379	-379
-380	-380
-381	-381
-382	-382
-383	-383
-384	-384
-385	-385
-386	-386
-387	-387
-388	-388
-389	-389
-390	-390
-391	-391
-392	-392
-393	-393
-394	-394
-395	-395
-396	-396
-397	-397
-398	-398
-399	-399
-400	-400
-401	-401
-402	-402
-403	-403
-404	-404
-405	-405
-406	-406
-407	-407
-408	-408
-409	-409
-410	-410
-411	-411
-412	-412
-413	-413
-414	-414
-415	-415
-416	-416
-417	-417
-418	-418
-419	-419
-420	-420
-421	-421
-422	-422
-423	-423
-424	-424
-425	-425
-426	-426
-427	-427
-428	-428
-429	-429
-430	-430
-431	-431
-432	-432
-433	-433
-434	-434
-435	-435
-436	-436
-437	-437
-438	-438
-439	-439
-440	-440
-441	-441
-442	-442
-443	-443
-444	-444
-445	-445
-446	-446
-447	-447
-448	-448
-449	-449
-450	-450
-451	-451
-452	-452
-453	-453
-454	-454
-455	-455
-456	-456
-457	-457
-458	-458
-459	-459
-460	-460
-461	-461
-462	-462
-463	-463
-464	-464
-465	-465
-466	-466
-467	-467
-468	-468
-469	-469
-470	-470
-471	-471
-472	-472
-473	-473
-474	-474
-475	-475
-476	-476
-477	-477
-478	-478
-479	-479
-480	-480
-481	-481
-482	-482
-483	-483
-484	-484
-485	-485
-486	-486
-487	-487
-488	-488
-489	-489
-490	-490
-491	-491
-492	-492
-493	-493
-494	-494
-495	-495
-496	-496
-497	-497
-498	-498
-499	-499
-500	-500
-501	-501
-502	-502
-503	-503
-504	-504
-505	-505
-506	-506
-507	-507
-508	-508
-509	-509
-510	-510
-511	-511
-512	-512
-513	-513
-514	-514
-515	-515
-516	-516
-517	-517
-518	-518
-519	-519
-520	-520
-521	-521
-522	-522
-523	-523
-524	-524
-525	-525
-526	-526
-527	-527
-528	-528
-529	-529
-530	-530
-531	-531
-532	-532
-533	-533
-534	-534
-535	-535
-536	-536
42	52
43	58
46	59
47	60
-1000	-1000
-9000	-9000
-666	-666
-585	-585
42	9
44	53
44	54
44	55
44	56
44	63
86	125
50	65
51	66
-537	-537
51	67
52	68
-538	-538
54	70
667	71
-539	-539
-540	-540
35	72
55	73
56	73
57	74
-541	-541
-542	-542
58	75
58	76
59	77
59	78
59	79
-543	-543
60	81
-544	-544
87	126
87	127
87	128
87	129
88	130
88	131
89	132
89	133
-586	-586
90	134
-587	-587
91	135
91	136
-588	-588
-589	-589
-590	-590
57	137
-591	-591
-552	-552
-553	-553
-554	-554
-555	-555
-556	-556
-557	-557
-558	-558
-559	-559
-560	-560
-561	-561
-20001	-20001
-20002	-20002
-20003	-20003
-20004	-20004
-20005	-20005
-20006	-20006
-20007	-20007
-20008	-20008
-20009	-20009
-30002	-30002
-30003	-30003
-30001	-30001
-545	-545
-546	-546
61	82
-547	-547
-548	-548
-549	-549
62	83
-550	-550
10	84
10	85
64	86
64	87
-551	-551
-10014	88
64	138
-592	-592
-593	-593
92	139
-594	-594
-595	-595
-596	-596
-10013	12
94	142
-597	-597
-598	-598
97	144
98	145
-599	-599
-601	-601
99	146
-602	-602
97	147
-603	-603
-604	-604
-605	-605
-30010	-30010
-30020	-30020
-30021	-30021
-30001	89
-30002	91
-30003	92
-30010	93
-30020	94
-30020	97
-30021	96
-30021	95
-10001	-10001
-10002	-10002
-10003	-10003
-10004	-10004
-10005	-10005
-10006	-10006
-10007	-10007
-10008	-10008
-10009	-10009
-10010	-10010
-10011	-10011
-10012	-10012
-10013	-10013
-10014	-10014
-562	-562
-563	-563
-564	-564
-565	-565
-566	-566
-567	-567
-568	-568
-569	-569
-570	-570
-10000	-10000
-10015	-10015
-20000	-20000
-20010	-20010
-30000	-30000
-30004	-30004
-30005	-30005
-30006	-30006
-30007	-30007
-30008	-30008
-30009	-30009
-30011	-30011
-30012	-30012
-30013	-30013
-30014	-30014
-30015	-30015
-30016	-30016
-30017	-30017
-30018	-30018
-30019	-30019
-30022	-30022
-30023	-30023
-30024	-30024
-30025	-30025
-30026	-30026
-30027	-30027
-30028	-30028
-30029	-30029
-30030	-30030
-10016	-10016
-10017	-10017
-10018	-10018
-10019	-10019
-10020	-10020
-10021	-10021
-10022	-10022
-10023	-10023
-10024	-10024
-10025	-10025
-10026	-10026
-10027	-10027
-10028	-10028
-10029	-10029
-10030	-10030
-10031	-10031
-10032	-10032
-10033	-10033
-10034	-10034
-10035	-10035
-10036	-10036
-10037	-10037
-10038	-10038
-10039	-10039
-10040	-10040
-10041	-10041
-10042	-10042
-10043	-10043
-10044	-10044
-10045	-10045
-10046	-10046
-10047	-10047
-10048	-10048
-10049	-10049
-10050	-10050
-20011	-20011
-20012	-20012
-20013	-20013
-20014	-20014
-20015	-20015
-20016	-20016
-20017	-20017
-20018	-20018
-20019	-20019
-20020	-20020
-20021	-20021
-20022	-20022
-20023	-20023
-20024	-20024
-20025	-20025
-20026	-20026
-20027	-20027
-20028	-20028
-20029	-20029
-20030	-20030
-20031	-20031
-20032	-20032
-20033	-20033
-20034	-20034
-20035	-20035
-20036	-20036
-20037	-20037
-20038	-20038
-20039	-20039
-20040	-20040
-20041	-20041
-20042	-20042
-20043	-20043
-20044	-20044
-20045	-20045
-20046	-20046
-20047	-20047
-20048	-20048
-20049	-20049
-20050	-20050
-30031	-30031
-30032	-30032
-30033	-30033
-30034	-30034
-30035	-30035
-30036	-30036
-30037	-30037
-30038	-30038
-30039	-30039
-30040	-30040
-30041	-30041
-30042	-30042
-30043	-30043
-30044	-30044
-30045	-30045
-30046	-30046
-30047	-30047
-30048	-30048
-30049	-30049
-30050	-30050
65	98
66	98
67	99
68	100
-600	-600
68	102
68	101
51	103
69	104
69	105
69	106
72	107
73	107
73	108
72	108
74	109
75	110
75	111
78	112
78	113
78	114
79	107
80	73
-571	-571
-572	-572
-30011	115
-30011	116
-30011	117
-30011	119
-573	-573
-574	-574
-575	-575
-576	-576
-30004	120
-577	-577
-578	-578
-579	-579
-580	-580
-581	-581
-582	-582
82	121
84	122
84	121
85	123
85	124
-583	-583
-607	-607
-606	-606
-609	-609
-610	-610
-611	-611
-608	-608
-612	-612
-30005	148
-616	-616
-617	-617
-618	-618
\.


--
-- Data for Name: triggers; Type: TABLE DATA; Schema: public; Owner: conanquotes
--

COPY triggers (tid, trigger, counter) FROM stdin;
-142	!142	17
-94	!94	6
-118	!118	68
-115	!115	88
-25	!25	8
-100	!100	22
-244	!244	155
5	!RANDOMCONANCRIME	36
48	!RCI	123
-223	!223	26
-10	!10	26
-67	!67	7
-301	!301	35
-325	!325	50
-211	!211	59
-277	!277	22
-52	!52	10
-154	!154	19
-280	!280	82
24	!RRCFF	20
21	!SHRIMP	363
-28	!28	11
20	!DLL	445
-256	!256	122
-205	!205	24
-313	!313	60
4	!RANDOMCRIME	6
30	!RULES#DCTV	10
29	!DDL#DCTV	10
37	!MRT	126
18	!RCFICT	228
-217	!217	104
-19	!19	12
-61	!61	8
-322	!322	97
-283	!283	49
-130	!130	36
-124	!124	61
-49	!49	10
41	!RCFUN	9
-13	!13	26
-199	!199	16
-37	!37	9
26	MCT	6
35	!DDL#KIENAI	32
-169	!169	16
-7	!7	26
-82	!82	6
-136	!136	25
-328	!328	33
52	!QUOTES	55
11	!BADTRANS	488
-289	!289	83
42	!RJDFUN	15
-208	!208	23
-262	!262	77
-172	!172	12
-265	!265	58
-34	!34	20
-304	!304	53
-31	!31	16
27	!M5	39
39	!JDF	21
-97	!97	14
-190	!190	15
-145	!145	17
-184	!184	17
111	!DINOS	6
-127	!127	50
43	!RFC	31
33	!M8	18
-148	!148	29
-220	!220	22
28	!NCH	51
-295	!295	99
47	!RCT	22
-133	!133	35
-316	!316	91
-241	!241	23
38	!NORRIS	309
-160	!160	49
-226	!226	20
-163	!163	31
-286	!286	24
-4	!4	26
-181	!181	14
108	!RABSF	9
44	!DLD	15
107	!RZDMF	88
-64	!64	9
34	!11TH	19
-271	!271	24
-178	!178	8
-139	!139	19
-235	!235	39
-334	!334	28
-46	!46	6
-202	!202	13
3	!RJDF	647
40	!RDFT	19
114	!ABSURD	3
112	!NONSENSE	18
-292	!292	36
-91	!91	9
110	!DINO	53
-1	!1	312
-196	!196	10
-175	!175	11
-214	!214	40
16	!RCFIC	38
-112	!112	80
-343	!343	79
-259	!259	22
-16	!16	22
14	!RKKF	128
-247	!247	62
-340	!340	29
-298	!298	106
-103	!103	11
-229	!229	39
-157	!157	21
17	!RCFACT	477
-319	!319	58
109	!FACES	80
-43	!43	14
-55	!55	16
-268	!268	31
-73	!73	16
-307	!307	41
-58	!58	9
-310	!310	33
10	!RDCTPF	152
-151	!151	17
32	!RUF	5
31	!UNO#KIENAI	1414
-274	!274	82
-109	!109	27
-187	!187	7
-166	!166	21
-238	!238	50
2	!RCC	596
-253	!253	31
113	!SILLY	10
-287	!287	22
-352	!352	86
-329	!329	31
-398	!398	61
-71	!71	9
-182	!182	12
-308	!308	31
-281	!281	262
-245	!245	191
-302	!302	20
-146	!146	23
-41	!41	10
-188	!188	11
-77	!77	8
-161	!161	15
-305	!305	109
-155	!155	20
-317	!317	37
-269	!269	76
-420	!420	88
-83	!83	9
-149	!149	14
-59	!59	9
-362	!362	13
-68	!68	9
-224	!224	24
-164	!164	20
-296	!296	129
-254	!254	17
-173	!173	8
-185	!185	8
-125	!125	46
-370	!370	15
-44	!44	4
-131	!131	24
-47	!47	6
-239	!239	24
-350	!350	90
-326	!326	39
-191	!191	14
-170	!170	13
-406	!406	49
-101	!101	13
-65	!65	8
-372	!372	12
-410	!410	122
-80	!80	12
-314	!314	36
-143	!143	20
-293	!293	36
-203	!203	21
-242	!242	51
-382	!382	20
-11	!11	22
-212	!212	67
-346	!346	82
-320	!320	73
-364	!364	16
-427	!427	43
-251	!251	46
-356	!356	32
-8	!8	27
-257	!257	134
-284	!284	33
-167	!167	13
-134	!134	33
-74	!74	7
-272	!272	24
-113	!113	66
-402	!402	22
-386	!386	17
-26	!26	7
-107	!107	27
-374	!374	16
-323	!323	35
-197	!197	12
-248	!248	146
-384	!384	60
-32	!32	9
-206	!206	18
-20	!20	22
-194	!194	8
-56	!56	7
-95	!95	7
-422	!422	25
-428	!428	33
-400	!400	53
-390	!390	25
-290	!290	60
-2	!2	81
-348	!348	93
-299	!299	124
-152	!152	13
-62	!62	9
-311	!311	30
-236	!236	278
-221	!221	17
-416	!416	15
-158	!158	27
-230	!230	27
-17	!17	16
-394	!394	45
-128	!128	42
-368	!368	15
-263	!263	35
-408	!408	53
-140	!140	20
-376	!376	20
-354	!354	27
-5	!5	31
-200	!200	27
-404	!404	41
-14	!14	29
-338	!338	26
-98	!98	13
-275	!275	54
-29	!29	9
13	!FUNNYSHIT	635
-396	!396	31
-260	!260	46
-380	!380	35
-341	!341	19
-35	!35	11
-119	!119	73
-116	!116	84
-278	!278	19
-424	!424	23
-378	!378	21
-137	!137	13
-429	!429	24
-23	!23	12
-366	!366	18
-266	!266	56
-430	!430	19
-108	!108	20
-282	!282	81
-21	!21	29
-240	!240	23
-321	!321	79
-258	!258	27
-353	!353	31
-48	!48	13
-231	!231	29
-385	!385	24
-318	!318	75
-367	!367	12
-228	!228	49
-168	!168	11
-381	!381	24
-171	!171	14
-33	!33	11
-105	!105	14
-207	!207	42
-393	!393	71
-78	!78	8
-162	!162	22
-144	!144	19
-63	!63	8
-120	!120	50
-330	!330	39
-288	!288	20
-315	!315	86
-255	!255	152
-336	!336	24
-401	!401	26
-395	!395	35
-409	!409	146
-377	!377	22
-141	!141	18
-342	!342	54
-114	!114	71
-75	!75	10
-403	!403	41
-419	!419	110
-294	!294	106
-225	!225	53
-147	!147	13
-261	!261	97
-123	!123	47
-132	!132	43
55	HELO FRIENDS	2
-51	!51	9
6	!M12	143
-117	!117	51
-57	!57	7
-297	!297	141
-186	!186	9
-387	!387	15
-399	!399	43
-309	!309	39
-417	!417	18
-24	!24	14
-60	!60	13
-3	!3	41
-153	!153	22
-213	!213	60
-72	!72	9
-413	!413	104
-291	!291	48
-183	!183	16
-180	!180	13
-195	!195	5
-345	!345	90
-383	!383	41
-126	!126	58
23	!RJF	235
-18	!18	16
-246	!246	134
-369	!369	13
-267	!267	36
-177	!177	17
-306	!306	36
-375	!375	19
-222	!222	45
-411	!411	36
-300	!300	83
-355	!355	22
-12	!12	28
-9	!9	21
-405	!405	34
-45	!45	7
-339	!339	19
-15	!15	17
-30	!30	19
-373	!373	18
-276	!276	46
-333	!333	34
-93	!93	4
-204	!204	10
-138	!138	19
-36	!36	7
-357	!357	92
-351	!351	66
-198	!198	10
54	HELO FRIEND	1
-361	!361	27
-156	!156	22
-27	!27	8
-252	!252	160
-264	!264	72
-192	!192	14
-54	!54	9
-174	!174	19
-189	!189	9
-81	!81	7
-421	!421	31
-102	!102	11
-234	!234	63
-99	!99	18
-423	!423	96
-135	!135	21
-407	!407	40
-42	!42	17
-129	!129	56
-359	!359	24
-279	!279	106
-249	!249	54
-219	!219	62
-285	!285	27
-270	!270	56
-69	!69	15
-237	!237	175
-6	!6	23
-324	!324	30
-371	!371	20
-243	!243	213
-96	!96	25
-365	!365	23
-494	!494	26
-466	!466	28
-511	!511	38
-512	!512	33
-442	!442	87
-450	!450	38
-476	!476	16
-491	!491	52
56	HELO FRENDS	91
-347	!347	59
-538	!538	65
-501	!501	37
-510	!510	55
-441	!441	20
-437	!437	30
-536	!536	45
-209	!209	31
63	HELO FREINDS	6
-165	!165	20
1	!RCF	2073
-517	!517	34
-532	!532	30
59	!SIMON	33
-492	!492	28
53	HELO FREND	28
-471	!471	16
-436	!436	20
-666	!666	58
-460	!460	15
-463	!463	27
-474	!474	14
65	!INSULTME	94
-432	!432	18
-507	!507	39
67	!EPISODES	19
-519	!519	21
-497	!497	26
-445	!445	33
-412	!412	21
-435	!435	25
-485	!485	30
-462	!462	27
-431	!431	20
-533	!533	31
-508	!508	29
-486	!486	92
-488	!488	104
-452	!452	27
-455	!455	27
-451	!451	30
-389	!389	49
70	!MAYS	83
-518	!518	25
-509	!509	34
-438	!438	38
-215	!215	109
-527	!527	54
-490	!490	42
-493	!493	27
68	!B'Z	104
-516	!516	40
-443	!443	33
66	!EPISODE	14
-327	!327	40
-53	!53	9
-453	!453	39
-480	!480	99
-473	!473	18
-535	!535	39
-433	!433	49
-498	!498	29
-504	!504	35
-444	!444	35
-273	!273	81
-529	!529	33
-489	!489	94
-470	!470	17
-446	!446	13
60	!HELP	101
-496	!496	31
-499	!499	26
-232	!232	80
-477	!477	12
-526	!526	70
9	CONANQUOTES: ADVERTISE	167
-482	!482	46
-503	!503	29
-456	!456	14
-459	!459	16
-461	!461	99
-479	!479	41
-454	!454	29
-122	!122	47
58	CONANQUOTES: BOTSNACK	40
-528	!528	35
-467	!467	25
-505	!505	41
-9000	!9000	31
-514	!514	40
-434	!434	125
-523	!523	32
22	!SSL	34
19	!FUNNY	182
-478	!478	16
-481	!481	94
71	SHRIMP!	28
-464	!464	24
-484	!484	36
-40	!40	9
15	!RDDLF	57
-530	!530	34
-465	!465	40
-515	!515	55
-520	!520	32
-475	!475	85
-521	!521	40
-502	!502	22
-534	!534	28
-524	!524	36
-426	!426	47
-537	!537	85
-500	!500	58
-447	!447	12
-440	!440	124
-506	!506	30
-487	!487	45
-513	!513	31
-425	!425	44
-531	!531	31
-469	!469	24
-495	!495	26
-448	!448	24
49	FAIL	118
-449	!449	18
-468	!468	76
-472	!472	21
-525	!525	31
-483	!483	99
-522	!522	47
-10015	!MOVIE15	4
-539	!539	54
25	!DDL#DCTP	9734
-150	!150	21
-555	!555	173
-20009	!OVA9	17
92	!MF3	6
-79	!79	14
-565	!565	126
93	!LVC	24
-20004	!OVA4	3
-556	!556	177
81	!GAY	34
-88	!88	9
-540	!540	69
-569	!569	89
-89	!89	7
-110	!110	54
-30006	!SPECIAL6	1
-30005	!SPECIAL5	4
-20002	!OVA2	7
-10006	!MOVIE6	14
-176	!176	17
83	!RMP	11
-92	!92	10
-30012	!SPECIAL12	1
-20001	!OVA1	19
-90	!90	12
-30002	!SPECIAL2	5
-543	!543	71
94	!LA1	0
-121	!121	47
-250	!250	45
-39	!39	6
-560	!560	117
-20003	!OVA3	7
-337	!337	79
-566	!566	104
64	!RANDOM	12
87	!M14SPOILERS	10
95	!LA2	0
-111	!111	49
-545	!545	83
82	!PING	150
-76	!76	19
77	!CHAINSAW	11
78	!CHAINSAWS	0
-549	!549	69
-30003	!SPECIAL3	4
-568	!568	88
-30009	!SPECIAL9	1
-227	!227	15
-10014	!MOVIE14	30
-233	!233	65
-546	!546	67
-557	!557	190
73	!SHIRANPURI	121
-10005	!MOVIE5	16
-550	!550	95
-66	!66	10
-30011	!SPECIAL11	8
-20007	!OVA7	4
96	!JD2	6
-541	!541	68
-86	!86	12
-210	!210	81
86	!M14SPOILER	1
-30004	!SPECIAL4	6
7	CONANQUOTES: SOURCE	23
-10001	!MOVIE1	31
-84	!84	10
-22	!22	13
-547	!547	81
-30010	!SPECIAL10	4
-10011	!MOVIE11	5
80	!HP#UC-FANSUBS	2
-331	!331	39
-20005	!OVA5	3
-30001	!SPECIAL1	10
-10012	!MOVIE12	26
-20010	!OVA10	56
-10003	!MOVIE3	8
84	!M13SPOILER	4
-552	!552	148
-10004	!MOVIE4	11
-544	!544	48
76	!LIST#YAMI	4
-10009	!MOVIE9	3
-216	!216	85
-10013	!MOVIE13	21
-548	!548	59
85	!M13SPOILERS	0
-85	!85	9
-551	!551	130
-20006	!OVA6	3
-10008	!MOVIE8	12
-10010	!MOVIE10	7
-50	!50	19
-20008	!OVA8	19
-553	!553	120
72	!MOO	52
-10007	!MOVIE7	19
75	!DDL#YAMI	6
-554	!554	122
-558	!558	183
-30007	!SPECIAL7	1
-559	!559	152
-70	!70	12
-358	!358	26
91	!MF2	11
-87	!87	7
-414	!414	119
79	!CW	61
-218	!218	92
-30020	!SPECIAL20	0
-30021	!SPECIAL21	0
-30000	!SPECIAL0	0
-10000	!MOVIE0	0
-20000	!OVA0	0
-30008	!SPECIAL8	0
-30013	!SPECIAL13	0
-30014	!SPECIAL14	0
-30015	!SPECIAL15	0
-30016	!SPECIAL16	0
-30017	!SPECIAL17	0
-30018	!SPECIAL18	0
-30019	!SPECIAL19	0
-30022	!SPECIAL22	0
-30023	!SPECIAL23	0
-30024	!SPECIAL24	0
-30025	!SPECIAL25	0
-30026	!SPECIAL26	0
-30027	!SPECIAL27	0
-30028	!SPECIAL28	0
-30029	!SPECIAL29	0
-30030	!SPECIAL30	0
-415	!415	28
88	!M14	409
-418	!418	27
-30050	!SPECIAL50	1
102	!RANDOMNEWS	5
-542	!542	70
-391	!391	17
104	！一	4
-360	!360	96
51	!LIST	3653
-349	!349	46
-397	!397	117
-561	!561	115
-10016	!MOVIE16	0
-10017	!MOVIE17	0
-10018	!MOVIE18	0
-10019	!MOVIE19	0
-10020	!MOVIE20	0
-10021	!MOVIE21	0
-10022	!MOVIE22	0
-10023	!MOVIE23	0
-10024	!MOVIE24	0
-10025	!MOVIE25	0
-10026	!MOVIE26	0
-10027	!MOVIE27	0
-10028	!MOVIE28	0
-10029	!MOVIE29	0
-10030	!MOVIE30	0
-10031	!MOVIE31	0
-10032	!MOVIE32	0
-10033	!MOVIE33	0
-10034	!MOVIE34	0
-10035	!MOVIE35	0
-10036	!MOVIE36	0
-10037	!MOVIE37	0
-10038	!MOVIE38	0
-10039	!MOVIE39	0
-10040	!MOVIE40	0
-10041	!MOVIE41	0
-10042	!MOVIE42	0
-10043	!MOVIE43	0
-10044	!MOVIE44	0
-10045	!MOVIE45	0
-10046	!MOVIE46	0
-10047	!MOVIE47	0
-10048	!MOVIE48	0
-10049	!MOVIE49	0
-20015	!OVA15	0
-20016	!OVA16	0
-20017	!OVA17	0
-20018	!OVA18	0
-20019	!OVA19	0
-20020	!OVA20	0
-20021	!OVA21	0
-20022	!OVA22	0
-20023	!OVA23	0
-20024	!OVA24	0
-20025	!OVA25	0
-20026	!OVA26	0
-20027	!OVA27	0
-20028	!OVA28	0
-20029	!OVA29	0
-20030	!OVA30	0
-20031	!OVA31	0
-20032	!OVA32	0
-20033	!OVA33	0
-20034	!OVA34	0
-20035	!OVA35	0
-20036	!OVA36	0
-20037	!OVA37	0
-20038	!OVA38	0
-20039	!OVA39	0
-20040	!OVA40	0
-20041	!OVA41	0
-20042	!OVA42	0
-20043	!OVA43	0
-20044	!OVA44	0
-20045	!OVA45	0
-20046	!OVA46	0
-20047	!OVA47	0
-20048	!OVA48	0
-20049	!OVA49	0
-20050	!OVA50	0
-30031	!SPECIAL31	0
-30032	!SPECIAL32	0
-30033	!SPECIAL33	0
-30034	!SPECIAL34	0
-30035	!SPECIAL35	0
-30036	!SPECIAL36	0
-30037	!SPECIAL37	0
-30038	!SPECIAL38	0
-30039	!SPECIAL39	0
-30040	!SPECIAL40	0
-30041	!SPECIAL41	0
-30042	!SPECIAL42	0
-30043	!SPECIAL43	0
-30044	!SPECIAL44	0
-30045	!SPECIAL45	0
-30046	!SPECIAL46	0
-30047	!SPECIAL47	0
-30048	!SPECIAL48	0
-30049	!SPECIAL49	0
-303	!303	65
-193	!193	14
12	!M13	662
-363	!363	22
-10002	!MOVIE2	21
-1000	!1000	34
89	!MF1	22
-20013	!OVA13	1
-564	!564	134
-392	!392	50
-104	!104	13
106	!ICHI	29
-563	!563	118
-312	!312	84
-344	!344	73
-379	!379	44
-159	!159	72
105	!一	8
-388	!388	76
98	!BOOT	36
-38	!38	10
-179	!179	14
-20011	!OVA11	17
-570	!570	121
101	!RN	40
99	!FISH	109
97	!JD1	4
74	!CUP	545
-562	!562	117
-201	!201	25
-332	!332	32
-20012	!OVA12	2
-335	!335	30
100	!RNP	23
-567	!567	95
-106	!106	16
0	!	91
-600	!600	82
-10050	!MOVIE50	3
-20014	!OVA14	1
45	!MU	1012
120	!MF4	37
-598	!598	44
-599	!599	40
133	!SCOOBY	9
122	!STFU	7
-603	!603	48
-604	!604	34
119	!MK,	1
-605	!605	37
-606	!606	7
-607	!607	26
138	!MOVIE14SPOILERS	3
137	!SPIDERCUP	6
146	!BING	3
116	!MK	80
132	!SDD	0
147	!BOTS	3
125	!ENGRISH	5
36	LOL	7637
103	!EPNUMBER	210
-612	!612	20
50	!PDF	442
-608	!608	4
131	!RBG	222
115	!KTPT	2
126	!HLEP	1
127	!HEPL	1
123	!MF	498
128	!EHLP	1
134	!!	15
-616	!616	13
129	!READ	0
121	!VUVUZELA	157
130	RBG	1
-610	!610	4
-571	!571	99
-572	!572	71
142	!TETRIS	7
-573	!573	73
124	!MEDIAFIRE	1
-574	!574	68
-575	!575	61
-576	!576	69
-577	!577	82
-457	!457	18
-458	!458	12
-578	!578	86
-579	!579	86
117	!EMKAY	1
-580	!580	97
139	!M15	13
-581	!581	90
-582	!582	75
-583	!583	106
-584	!584	67
-585	!585	76
-586	!586	67
136	!DDL#CONAN-FANS	2
-587	!587	57
143	BOTLIST	0
-588	!588	56
135	!LIST#CONAN-FANS	15
-589	!589	58
-590	!590	77
-602	!602	37
-609	!609	25
145	!SIMCOP	14
-591	!591	44
-592	!592	60
-601	!601	52
-593	!593	50
148	!MF5	3
-617	!617	10
-594	!594	43
-595	!595	37
-618	!618	11
-611	!611	3
-439	!439	83
144	!BOTLIST	9
-596	!596	40
-597	!597	43
\.


--
-- Name: bansource_pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY bansource
    ADD CONSTRAINT bansource_pkey PRIMARY KEY (cid, sid);


--
-- Name: channelsetup_channel_key; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY channelsetup
    ADD CONSTRAINT channelsetup_channel_key UNIQUE (channel);


--
-- Name: channelsetup_pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY channelsetup
    ADD CONSTRAINT channelsetup_pkey PRIMARY KEY (cid);


--
-- Name: episodeinfo_pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY episodeinfo
    ADD CONSTRAINT episodeinfo_pkey PRIMARY KEY (eid);


--
-- Name: groupaliases_alias_key; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY groupaliases
    ADD CONSTRAINT groupaliases_alias_key UNIQUE (alias);


--
-- Name: groupinfo_pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY groupinfo
    ADD CONSTRAINT groupinfo_pkey PRIMARY KEY (gid);


--
-- Name: ignores_pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY ignores
    ADD CONSTRAINT ignores_pkey PRIMARY KEY (nick);


--
-- Name: pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY sources
    ADD CONSTRAINT pkey PRIMARY KEY (sid);


--
-- Name: primary key; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY quotes
    ADD CONSTRAINT "primary key" PRIMARY KEY (qid);


--
-- Name: quotes_quote_key; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY quotes
    ADD CONSTRAINT quotes_quote_key UNIQUE (quote, sid);


--
-- Name: sources_name_key; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY sources
    ADD CONSTRAINT sources_name_key UNIQUE (name);


--
-- Name: tidsources_pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY tidsources
    ADD CONSTRAINT tidsources_pkey PRIMARY KEY (sid, tid);


--
-- Name: triggers_pkey; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY triggers
    ADD CONSTRAINT triggers_pkey PRIMARY KEY (tid);


--
-- Name: triggers_trigger_key; Type: CONSTRAINT; Schema: public; Owner: conanquotes; Tablespace: 
--

ALTER TABLE ONLY triggers
    ADD CONSTRAINT triggers_trigger_key UNIQUE (trigger);


--
-- Name: bansource_cid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: conanquotes
--

ALTER TABLE ONLY bansource
    ADD CONSTRAINT bansource_cid_fkey FOREIGN KEY (cid) REFERENCES channelsetup(cid);


--
-- Name: bansource_sid_fkey; Type: FK CONSTRAINT; Schema: public; Owner: conanquotes
--

ALTER TABLE ONLY bansource
    ADD CONSTRAINT bansource_sid_fkey FOREIGN KEY (sid) REFERENCES sources(sid);


--
-- Name: sid; Type: FK CONSTRAINT; Schema: public; Owner: conanquotes
--

ALTER TABLE ONLY quotes
    ADD CONSTRAINT sid FOREIGN KEY (sid) REFERENCES sources(sid);


--
-- Name: sources; Type: FK CONSTRAINT; Schema: public; Owner: conanquotes
--

ALTER TABLE ONLY tidsources
    ADD CONSTRAINT sources FOREIGN KEY (sid) REFERENCES sources(sid);


--
-- Name: triggers; Type: FK CONSTRAINT; Schema: public; Owner: conanquotes
--

ALTER TABLE ONLY tidsources
    ADD CONSTRAINT triggers FOREIGN KEY (tid) REFERENCES triggers(tid);


--
-- Name: public; Type: ACL; Schema: -; Owner: conanquotes
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM conanquotes;
GRANT ALL ON SCHEMA public TO conanquotes;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- Name: adminlogs; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON TABLE adminlogs FROM PUBLIC;
REVOKE ALL ON TABLE adminlogs FROM conanquotes;
GRANT ALL ON TABLE adminlogs TO conanquotes;
GRANT ALL ON TABLE adminlogs TO PUBLIC;


--
-- Name: channelsetup; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON TABLE channelsetup FROM PUBLIC;
REVOKE ALL ON TABLE channelsetup FROM conanquotes;
GRANT ALL ON TABLE channelsetup TO conanquotes;
GRANT ALL ON TABLE channelsetup TO PUBLIC;


--
-- Name: episodeinfo; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON TABLE episodeinfo FROM PUBLIC;
REVOKE ALL ON TABLE episodeinfo FROM conanquotes;
GRANT ALL ON TABLE episodeinfo TO conanquotes;
GRANT ALL ON TABLE episodeinfo TO PUBLIC;


--
-- Name: qidseq; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON SEQUENCE qidseq FROM PUBLIC;
REVOKE ALL ON SEQUENCE qidseq FROM conanquotes;
GRANT ALL ON SEQUENCE qidseq TO conanquotes;
GRANT ALL ON SEQUENCE qidseq TO PUBLIC;


--
-- Name: quotes; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON TABLE quotes FROM PUBLIC;
REVOKE ALL ON TABLE quotes FROM conanquotes;
GRANT ALL ON TABLE quotes TO conanquotes;
GRANT ALL ON TABLE quotes TO PUBLIC;


--
-- Name: sidseq; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON SEQUENCE sidseq FROM PUBLIC;
REVOKE ALL ON SEQUENCE sidseq FROM conanquotes;
GRANT ALL ON SEQUENCE sidseq TO conanquotes;
GRANT ALL ON SEQUENCE sidseq TO PUBLIC;


--
-- Name: sources; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON TABLE sources FROM PUBLIC;
REVOKE ALL ON TABLE sources FROM conanquotes;
GRANT ALL ON TABLE sources TO conanquotes;
GRANT ALL ON TABLE sources TO PUBLIC;


--
-- Name: tidseq; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON SEQUENCE tidseq FROM PUBLIC;
REVOKE ALL ON SEQUENCE tidseq FROM conanquotes;
GRANT ALL ON SEQUENCE tidseq TO conanquotes;
GRANT ALL ON SEQUENCE tidseq TO PUBLIC;


--
-- Name: tidsources; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON TABLE tidsources FROM PUBLIC;
REVOKE ALL ON TABLE tidsources FROM conanquotes;
GRANT ALL ON TABLE tidsources TO conanquotes;
GRANT ALL ON TABLE tidsources TO PUBLIC;


--
-- Name: triggers; Type: ACL; Schema: public; Owner: conanquotes
--

REVOKE ALL ON TABLE triggers FROM PUBLIC;
REVOKE ALL ON TABLE triggers FROM conanquotes;
GRANT ALL ON TABLE triggers TO conanquotes;
GRANT ALL ON TABLE triggers TO PUBLIC;


--
-- PostgreSQL database dump complete
--

